# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: base.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='base.proto',
  package='artifacts.proto',
  syntax='proto3',
  serialized_pb=_b('\n\nbase.proto\x12\x0f\x61rtifacts.proto\"a\n\nBaseRecord\x12\x0c\n\x04step\x18\x01 \x01(\x05\x12\x11\n\ttimestamp\x18\x02 \x01(\x03\x12\x10\n\x08\x61rtifact\x18\x03 \x01(\x0c\x12\r\n\x05\x65poch\x18\x04 \x01(\x05\x12\x11\n\thas_epoch\x18\x05 \x01(\x08\x62\x06proto3')
)




_BASERECORD = _descriptor.Descriptor(
  name='BaseRecord',
  full_name='artifacts.proto.BaseRecord',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='step', full_name='artifacts.proto.BaseRecord.step', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='timestamp', full_name='artifacts.proto.BaseRecord.timestamp', index=1,
      number=2, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='artifact', full_name='artifacts.proto.BaseRecord.artifact', index=2,
      number=3, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='epoch', full_name='artifacts.proto.BaseRecord.epoch', index=3,
      number=4, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='has_epoch', full_name='artifacts.proto.BaseRecord.has_epoch', index=4,
      number=5, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=31,
  serialized_end=128,
)

DESCRIPTOR.message_types_by_name['BaseRecord'] = _BASERECORD
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

BaseRecord = _reflection.GeneratedProtocolMessageType('BaseRecord', (_message.Message,), dict(
  DESCRIPTOR = _BASERECORD,
  __module__ = 'base_pb2'
  # @@protoc_insertion_point(class_scope:artifacts.proto.BaseRecord)
  ))
_sym_db.RegisterMessage(BaseRecord)


# @@protoc_insertion_point(module_scope)
