# Aim
AIM_OBJECTS_DIR_NAME = 'objects'
AIM_METRICS_DIR_NAME = 'metrics'
AIM_METRIC_GR_DIR_NAME = 'metric_groups'
AIM_ANNOT_DIR_NAME = 'annotations'
AIM_SEG_DIR_NAME = 'segmentation'
AIM_MEDIA_DIR_NAME = 'media'
AIM_IMAGES_DIR_NAME = 'images'
AIM_MODELS_DIR_NAME = 'models'
AIM_CORR_DIR_NAME = 'correlations'
AIM_CORR_DIRS_NAME = 'dirs'
AIM_CONFIG_FILE_NAME = 'config.json'
AIM_DIFF_DIR_NAME = 'diff'
AIM_DIFF_FILE_NAME = 'diff.txt'
AIM_PARAMS_DIR_NAME = 'hyperparameters'
AIM_MAP_DIR_NAME = 'map'
AIM_STATS_DIR_NAME = 'stats'
AIM_TEXT_DIR_NAME = 'document_texts'
AIM_MAP_METRICS_KEYWORD = '__METRICS__'

# Commits
AIM_COMMIT_INDEX_DIR_NAME = 'index'
AIM_COMMIT_CONFIG_FILE_NAME = 'config.json'
AIM_COMMIT_META_FILE_NAME = 'meta.json'
