# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: metric.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='metric.proto',
  package='artifacts.proto',
  syntax='proto3',
  serialized_pb=_b('\n\x0cmetric.proto\x12\x0f\x61rtifacts.proto\"\x1d\n\x0cMetricRecord\x12\r\n\x05value\x18\x01 \x01(\x01\x62\x06proto3')
)




_METRICRECORD = _descriptor.Descriptor(
  name='MetricRecord',
  full_name='artifacts.proto.MetricRecord',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='value', full_name='artifacts.proto.MetricRecord.value', index=0,
      number=1, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=33,
  serialized_end=62,
)

DESCRIPTOR.message_types_by_name['MetricRecord'] = _METRICRECORD
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

MetricRecord = _reflection.GeneratedProtocolMessageType('MetricRecord', (_message.Message,), dict(
  DESCRIPTOR = _METRICRECORD,
  __module__ = 'metric_pb2'
  # @@protoc_insertion_point(class_scope:artifacts.proto.MetricRecord)
  ))
_sym_db.RegisterMessage(MetricRecord)


# @@protoc_insertion_point(module_scope)
