/* Generated by Cython 3.0.0a9 */

/* BEGIN: Cython Metadata
{
    "distutils": {
        "depends": [
            "/private/var/folders/2p/y3wbjwp94f99dc0glkdrp32h0000gp/T/pip-build-env-a4ehdxmy/overlay/lib/python3.9/site-packages/aimrocks/include/rdb_include/comparator_wrapper.hpp",
            "/private/var/folders/2p/y3wbjwp94f99dc0glkdrp32h0000gp/T/pip-build-env-a4ehdxmy/overlay/lib/python3.9/site-packages/aimrocks/include/rdb_include/filter_policy_wrapper.hpp",
            "/private/var/folders/2p/y3wbjwp94f99dc0glkdrp32h0000gp/T/pip-build-env-a4ehdxmy/overlay/lib/python3.9/site-packages/aimrocks/include/rdb_include/memtable_factories.hpp",
            "/private/var/folders/2p/y3wbjwp94f99dc0glkdrp32h0000gp/T/pip-build-env-a4ehdxmy/overlay/lib/python3.9/site-packages/aimrocks/include/rdb_include/merge_operator_wrapper.hpp",
            "/private/var/folders/2p/y3wbjwp94f99dc0glkdrp32h0000gp/T/pip-build-env-a4ehdxmy/overlay/lib/python3.9/site-packages/aimrocks/include/rdb_include/slice_transform_wrapper.hpp",
            "/private/var/folders/2p/y3wbjwp94f99dc0glkdrp32h0000gp/T/pip-build-env-a4ehdxmy/overlay/lib/python3.9/site-packages/aimrocks/include/rdb_include/utils.hpp",
            "/private/var/folders/2p/y3wbjwp94f99dc0glkdrp32h0000gp/T/pip-build-env-a4ehdxmy/overlay/lib/python3.9/site-packages/aimrocks/include/rdb_include/write_batch_iter_helper.hpp",
            "/private/var/folders/2p/y3wbjwp94f99dc0glkdrp32h0000gp/T/pip-build-env-a4ehdxmy/overlay/lib/python3.9/site-packages/aimrocks/include/rocksdb/cache.h",
            "/private/var/folders/2p/y3wbjwp94f99dc0glkdrp32h0000gp/T/pip-build-env-a4ehdxmy/overlay/lib/python3.9/site-packages/aimrocks/include/rocksdb/comparator.h",
            "/private/var/folders/2p/y3wbjwp94f99dc0glkdrp32h0000gp/T/pip-build-env-a4ehdxmy/overlay/lib/python3.9/site-packages/aimrocks/include/rocksdb/db.h",
            "/private/var/folders/2p/y3wbjwp94f99dc0glkdrp32h0000gp/T/pip-build-env-a4ehdxmy/overlay/lib/python3.9/site-packages/aimrocks/include/rocksdb/env.h",
            "/private/var/folders/2p/y3wbjwp94f99dc0glkdrp32h0000gp/T/pip-build-env-a4ehdxmy/overlay/lib/python3.9/site-packages/aimrocks/include/rocksdb/filter_policy.h",
            "/private/var/folders/2p/y3wbjwp94f99dc0glkdrp32h0000gp/T/pip-build-env-a4ehdxmy/overlay/lib/python3.9/site-packages/aimrocks/include/rocksdb/iterator.h",
            "/private/var/folders/2p/y3wbjwp94f99dc0glkdrp32h0000gp/T/pip-build-env-a4ehdxmy/overlay/lib/python3.9/site-packages/aimrocks/include/rocksdb/memtablerep.h",
            "/private/var/folders/2p/y3wbjwp94f99dc0glkdrp32h0000gp/T/pip-build-env-a4ehdxmy/overlay/lib/python3.9/site-packages/aimrocks/include/rocksdb/merge_operator.h",
            "/private/var/folders/2p/y3wbjwp94f99dc0glkdrp32h0000gp/T/pip-build-env-a4ehdxmy/overlay/lib/python3.9/site-packages/aimrocks/include/rocksdb/options.h",
            "/private/var/folders/2p/y3wbjwp94f99dc0glkdrp32h0000gp/T/pip-build-env-a4ehdxmy/overlay/lib/python3.9/site-packages/aimrocks/include/rocksdb/slice.h",
            "/private/var/folders/2p/y3wbjwp94f99dc0glkdrp32h0000gp/T/pip-build-env-a4ehdxmy/overlay/lib/python3.9/site-packages/aimrocks/include/rocksdb/slice_transform.h",
            "/private/var/folders/2p/y3wbjwp94f99dc0glkdrp32h0000gp/T/pip-build-env-a4ehdxmy/overlay/lib/python3.9/site-packages/aimrocks/include/rocksdb/status.h",
            "/private/var/folders/2p/y3wbjwp94f99dc0glkdrp32h0000gp/T/pip-build-env-a4ehdxmy/overlay/lib/python3.9/site-packages/aimrocks/include/rocksdb/table.h",
            "/private/var/folders/2p/y3wbjwp94f99dc0glkdrp32h0000gp/T/pip-build-env-a4ehdxmy/overlay/lib/python3.9/site-packages/aimrocks/include/rocksdb/universal_compaction.h",
            "/private/var/folders/2p/y3wbjwp94f99dc0glkdrp32h0000gp/T/pip-build-env-a4ehdxmy/overlay/lib/python3.9/site-packages/aimrocks/include/rocksdb/utilities/backup_engine.h",
            "/private/var/folders/2p/y3wbjwp94f99dc0glkdrp32h0000gp/T/pip-build-env-a4ehdxmy/overlay/lib/python3.9/site-packages/aimrocks/include/rocksdb/write_batch.h"
        ],
        "extra_compile_args": [
            "-std=c++11",
            "-O3",
            "-Wall",
            "-Wextra",
            "-Wconversion",
            "-fno-strict-aliasing",
            "-fno-rtti",
            "-fPIC"
        ],
        "include_dirs": [
            "/private/var/folders/2p/y3wbjwp94f99dc0glkdrp32h0000gp/T/pip-build-env-a4ehdxmy/overlay/lib/python3.9/site-packages/aimrocks/include"
        ],
        "language": "c++",
        "libraries": [
            "rocksdb"
        ],
        "library_dirs": [
            "/private/var/folders/2p/y3wbjwp94f99dc0glkdrp32h0000gp/T/pip-build-env-a4ehdxmy/overlay/lib/python3.9/site-packages/aimrocks"
        ],
        "name": "aim.storage.prefixview",
        "sources": [
            "aim/storage/prefixview.py"
        ]
    },
    "module_name": "aim.storage.prefixview"
}
END: Cython Metadata */

#ifndef PY_SSIZE_T_CLEAN
#define PY_SSIZE_T_CLEAN
#endif /* PY_SSIZE_T_CLEAN */
#if defined(CYTHON_LIMITED_API) && 0
  #ifndef Py_LIMITED_API
    #if CYTHON_LIMITED_API+0 > 0x03030000
      #define Py_LIMITED_API CYTHON_LIMITED_API
    #else
      #define Py_LIMITED_API 0x03030000
    #endif
  #endif
#endif

#include "Python.h"

    #if PY_MAJOR_VERSION >= 3
      #define __Pyx_PyFloat_FromString(obj)  PyFloat_FromString(obj)
    #else
      #define __Pyx_PyFloat_FromString(obj)  PyFloat_FromString(obj, NULL)
    #endif
    

    #if PY_MAJOR_VERSION <= 2
    #define PyDict_GetItemWithError _PyDict_GetItemWithError
    #endif
    

    #if PY_VERSION_HEX < 0x030700b1 && !defined(PyContextVar_Get)
    #define PyContextVar_Get(var, d, v)         ((d) ?             ((void)(var), Py_INCREF(d), (v)[0] = (d), 0) :             ((v)[0] = NULL, 0)         )
    #endif
    
#ifndef Py_PYTHON_H
    #error Python headers needed to compile C extensions, please install development version of Python.
#elif PY_VERSION_HEX < 0x02070000 || (0x03000000 <= PY_VERSION_HEX && PY_VERSION_HEX < 0x03030000)
    #error Cython requires Python 2.7+ or Python 3.3+.
#else
#define CYTHON_ABI "3_0_0a9"
#define __PYX_ABI_MODULE_NAME "_cython_" CYTHON_ABI
#define __PYX_TYPE_MODULE_PREFIX __PYX_ABI_MODULE_NAME "."
#define CYTHON_HEX_VERSION 0x030000A9
#define CYTHON_FUTURE_DIVISION 1
#include <stddef.h>
#ifndef offsetof
  #define offsetof(type, member) ( (size_t) & ((type*)0) -> member )
#endif
#if !defined(_WIN32) && !defined(WIN32) && !defined(MS_WINDOWS)
  #ifndef __stdcall
    #define __stdcall
  #endif
  #ifndef __cdecl
    #define __cdecl
  #endif
  #ifndef __fastcall
    #define __fastcall
  #endif
#endif
#ifndef DL_IMPORT
  #define DL_IMPORT(t) t
#endif
#ifndef DL_EXPORT
  #define DL_EXPORT(t) t
#endif
#define __PYX_COMMA ,
#ifndef HAVE_LONG_LONG
  #define HAVE_LONG_LONG
#endif
#ifndef PY_LONG_LONG
  #define PY_LONG_LONG LONG_LONG
#endif
#ifndef Py_HUGE_VAL
  #define Py_HUGE_VAL HUGE_VAL
#endif
#ifdef PYPY_VERSION
  #define CYTHON_COMPILING_IN_PYPY 1
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #define CYTHON_COMPILING_IN_LIMITED_API 0
  #undef CYTHON_USE_TYPE_SLOTS
  #define CYTHON_USE_TYPE_SLOTS 0
  #undef CYTHON_USE_TYPE_SPECS
  #define CYTHON_USE_TYPE_SPECS 0
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #if PY_VERSION_HEX < 0x03050000
    #undef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 0
  #elif !defined(CYTHON_USE_ASYNC_SLOTS)
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #undef CYTHON_USE_UNICODE_INTERNALS
  #define CYTHON_USE_UNICODE_INTERNALS 0
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #undef CYTHON_AVOID_BORROWED_REFS
  #define CYTHON_AVOID_BORROWED_REFS 1
  #undef CYTHON_ASSUME_SAFE_MACROS
  #define CYTHON_ASSUME_SAFE_MACROS 0
  #undef CYTHON_UNPACK_METHODS
  #define CYTHON_UNPACK_METHODS 0
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_GIL
  #define CYTHON_FAST_GIL 0
  #undef CYTHON_METH_FASTCALL
  #define CYTHON_METH_FASTCALL 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #ifndef CYTHON_PEP487_INIT_SUBCLASS
    #define CYTHON_PEP487_INIT_SUBCLASS (PY_MAJOR_VERSION >= 3)
  #endif
  #undef CYTHON_PEP489_MULTI_PHASE_INIT
  #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #undef CYTHON_USE_MODULE_STATE
  #define CYTHON_USE_MODULE_STATE 0
  #undef CYTHON_USE_TP_FINALIZE
  #define CYTHON_USE_TP_FINALIZE 0
  #undef CYTHON_USE_DICT_VERSIONS
  #define CYTHON_USE_DICT_VERSIONS 0
  #undef CYTHON_USE_EXC_INFO_STACK
  #define CYTHON_USE_EXC_INFO_STACK 0
#elif defined(CYTHON_LIMITED_API)
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #define CYTHON_COMPILING_IN_LIMITED_API 1
  #undef CYTHON_USE_TYPE_SLOTS
  #define CYTHON_USE_TYPE_SLOTS 0
  #undef CYTHON_USE_TYPE_SPECS
  #define CYTHON_USE_TYPE_SPECS 1
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #undef CYTHON_USE_ASYNC_SLOTS
  #define CYTHON_USE_ASYNC_SLOTS 0
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #undef CYTHON_USE_UNICODE_INTERNALS
  #define CYTHON_USE_UNICODE_INTERNALS 0
  #ifndef CYTHON_USE_UNICODE_WRITER
    #define CYTHON_USE_UNICODE_WRITER 1
  #endif
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #undef CYTHON_ASSUME_SAFE_MACROS
  #define CYTHON_ASSUME_SAFE_MACROS 0
  #undef CYTHON_UNPACK_METHODS
  #define CYTHON_UNPACK_METHODS 0
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_GIL
  #define CYTHON_FAST_GIL 0
  #undef CYTHON_METH_FASTCALL
  #define CYTHON_METH_FASTCALL 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #ifndef CYTHON_PEP487_INIT_SUBCLASS
    #define CYTHON_PEP487_INIT_SUBCLASS 1
  #endif
  #undef CYTHON_PEP489_MULTI_PHASE_INIT
  #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #undef CYTHON_USE_MODULE_STATE
  #define CYTHON_USE_MODULE_STATE 1
  #ifndef CYTHON_USE_TP_FINALIZE
    #define CYTHON_USE_TP_FINALIZE 1
  #endif
  #undef CYTHON_USE_DICT_VERSIONS
  #define CYTHON_USE_DICT_VERSIONS 0
  #undef CYTHON_USE_EXC_INFO_STACK
  #define CYTHON_USE_EXC_INFO_STACK 0
#else
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_CPYTHON 1
  #define CYTHON_COMPILING_IN_LIMITED_API 0
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #ifndef CYTHON_USE_TYPE_SPECS
    #define CYTHON_USE_TYPE_SPECS 0
  #endif
  #ifndef CYTHON_USE_PYTYPE_LOOKUP
    #define CYTHON_USE_PYTYPE_LOOKUP 1
  #endif
  #if PY_MAJOR_VERSION < 3
    #undef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 0
  #elif !defined(CYTHON_USE_ASYNC_SLOTS)
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #ifndef CYTHON_USE_PYLONG_INTERNALS
    #define CYTHON_USE_PYLONG_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_PYLIST_INTERNALS
    #define CYTHON_USE_PYLIST_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #if PY_VERSION_HEX < 0x030300F0
    #undef CYTHON_USE_UNICODE_WRITER
    #define CYTHON_USE_UNICODE_WRITER 0
  #elif !defined(CYTHON_USE_UNICODE_WRITER)
    #define CYTHON_USE_UNICODE_WRITER 1
  #endif
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #ifndef CYTHON_FAST_THREAD_STATE
    #define CYTHON_FAST_THREAD_STATE 1
  #endif
  #ifndef CYTHON_FAST_GIL
    #define CYTHON_FAST_GIL (PY_MAJOR_VERSION < 3 || PY_VERSION_HEX >= 0x03060000)
  #endif
  #ifndef CYTHON_METH_FASTCALL
    #define CYTHON_METH_FASTCALL (PY_VERSION_HEX >= 0x030700A1)
  #endif
  #ifndef CYTHON_FAST_PYCALL
    #define CYTHON_FAST_PYCALL 1
  #endif
  #ifndef CYTHON_PEP487_INIT_SUBCLASS
    #define CYTHON_PEP487_INIT_SUBCLASS 1
  #endif
  #if PY_VERSION_HEX < 0x03050000
    #undef CYTHON_PEP489_MULTI_PHASE_INIT
    #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #elif !defined(CYTHON_PEP489_MULTI_PHASE_INIT)
    #define CYTHON_PEP489_MULTI_PHASE_INIT 1
  #endif
  #ifndef CYTHON_USE_MODULE_STATE
    #define CYTHON_USE_MODULE_STATE 0
  #endif
  #if PY_VERSION_HEX < 0x030400a1
    #undef CYTHON_USE_TP_FINALIZE
    #define CYTHON_USE_TP_FINALIZE 0
  #elif !defined(CYTHON_USE_TP_FINALIZE)
    #define CYTHON_USE_TP_FINALIZE 1
  #endif
  #if PY_VERSION_HEX < 0x030600B1
    #undef CYTHON_USE_DICT_VERSIONS
    #define CYTHON_USE_DICT_VERSIONS 0
  #elif !defined(CYTHON_USE_DICT_VERSIONS)
    #define CYTHON_USE_DICT_VERSIONS 1
  #endif
  #if PY_VERSION_HEX < 0x030700A3
    #undef CYTHON_USE_EXC_INFO_STACK
    #define CYTHON_USE_EXC_INFO_STACK 0
  #elif !defined(CYTHON_USE_EXC_INFO_STACK)
    #define CYTHON_USE_EXC_INFO_STACK 1
  #endif
#endif
#if !defined(CYTHON_FAST_PYCCALL)
#define CYTHON_FAST_PYCCALL  (CYTHON_FAST_PYCALL && PY_VERSION_HEX >= 0x030600B1)
#endif
#if !defined(CYTHON_VECTORCALL)
#define CYTHON_VECTORCALL  (CYTHON_FAST_PYCCALL && PY_VERSION_HEX >= 0x030800B1)
#endif
#define CYTHON_BACKPORT_VECTORCALL (CYTHON_METH_FASTCALL && PY_VERSION_HEX < 0x030800B1)
#if CYTHON_USE_PYLONG_INTERNALS
  #include "longintrepr.h"
  #undef SHIFT
  #undef BASE
  #undef MASK
  #ifdef SIZEOF_VOID_P
    enum { __pyx_check_sizeof_voidp = 1 / (int)(SIZEOF_VOID_P == sizeof(void*)) };
  #endif
#endif
#ifndef __has_attribute
  #define __has_attribute(x) 0
#endif
#ifndef __has_cpp_attribute
  #define __has_cpp_attribute(x) 0
#endif
#ifndef CYTHON_RESTRICT
  #if defined(__GNUC__)
    #define CYTHON_RESTRICT __restrict__
  #elif defined(_MSC_VER) && _MSC_VER >= 1400
    #define CYTHON_RESTRICT __restrict
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_RESTRICT restrict
  #else
    #define CYTHON_RESTRICT
  #endif
#endif
#ifndef CYTHON_UNUSED
# if defined(__GNUC__)
#   if !(defined(__cplusplus)) || (__GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ >= 4))
#     define CYTHON_UNUSED __attribute__ ((__unused__))
#   else
#     define CYTHON_UNUSED
#   endif
# elif defined(__ICC) || (defined(__INTEL_COMPILER) && !defined(_MSC_VER))
#   define CYTHON_UNUSED __attribute__ ((__unused__))
# else
#   define CYTHON_UNUSED
# endif
#endif
#ifndef CYTHON_UNUSED_VAR
#  if defined(__cplusplus)
     template<class T> void CYTHON_UNUSED_VAR( const T& ) { }
#  else
#    define CYTHON_UNUSED_VAR(x) (void)(x)
#  endif
#endif
#ifndef CYTHON_MAYBE_UNUSED_VAR
  #define CYTHON_MAYBE_UNUSED_VAR(x) CYTHON_UNUSED_VAR(x)
#endif
#ifndef CYTHON_NCP_UNUSED
# if CYTHON_COMPILING_IN_CPYTHON
#  define CYTHON_NCP_UNUSED
# else
#  define CYTHON_NCP_UNUSED CYTHON_UNUSED
# endif
#endif
#define __Pyx_void_to_None(void_result) ((void)(void_result), Py_INCREF(Py_None), Py_None)
#ifdef _MSC_VER
    #ifndef _MSC_STDINT_H_
        #if _MSC_VER < 1300
            typedef unsigned char     uint8_t;
            typedef unsigned short    uint16_t;
            typedef unsigned int      uint32_t;
        #else
            typedef unsigned __int8   uint8_t;
            typedef unsigned __int16  uint16_t;
            typedef unsigned __int32  uint32_t;
        #endif
    #endif
    #if _MSC_VER < 1300
        #ifdef _WIN64
            typedef unsigned long long  __pyx_uintptr_t;
        #else
            typedef unsigned int        __pyx_uintptr_t;
        #endif
    #else
        #ifdef _WIN64
            typedef unsigned __int64    __pyx_uintptr_t;
        #else
            typedef unsigned __int32    __pyx_uintptr_t;
        #endif
    #endif
#else
    #include <stdint.h>
    typedef uintptr_t  __pyx_uintptr_t;
#endif
#ifndef CYTHON_FALLTHROUGH
  #if defined(__cplusplus) && __cplusplus >= 201103L
    #if __has_cpp_attribute(fallthrough)
      #define CYTHON_FALLTHROUGH [[fallthrough]]
    #elif __has_cpp_attribute(clang::fallthrough)
      #define CYTHON_FALLTHROUGH [[clang::fallthrough]]
    #elif __has_cpp_attribute(gnu::fallthrough)
      #define CYTHON_FALLTHROUGH [[gnu::fallthrough]]
    #endif
  #endif
  #ifndef CYTHON_FALLTHROUGH
    #if __has_attribute(fallthrough)
      #define CYTHON_FALLTHROUGH __attribute__((fallthrough))
    #else
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
  #if defined(__clang__ ) && defined(__apple_build_version__)
    #if __apple_build_version__ < 7000000
      #undef  CYTHON_FALLTHROUGH
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
#endif

#ifndef __cplusplus
  #error "Cython files generated with the C++ option must be compiled with a C++ compiler."
#endif
#ifndef CYTHON_INLINE
  #if defined(__clang__)
    #define CYTHON_INLINE __inline__ __attribute__ ((__unused__))
  #else
    #define CYTHON_INLINE inline
  #endif
#endif
template<typename T>
void __Pyx_call_destructor(T& x) {
    x.~T();
}
template<typename T>
class __Pyx_FakeReference {
  public:
    __Pyx_FakeReference() : ptr(NULL) { }
    __Pyx_FakeReference(const T& ref) : ptr(const_cast<T*>(&ref)) { }
    T *operator->() { return ptr; }
    T *operator&() { return ptr; }
    operator T&() { return *ptr; }
    template<typename U> bool operator ==(U other) { return *ptr == other; }
    template<typename U> bool operator !=(U other) { return *ptr != other; }
  private:
    T *ptr;
};

#if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x02070600 && !defined(Py_OptimizeFlag)
  #define Py_OptimizeFlag 0
#endif
#define __PYX_BUILD_PY_SSIZE_T "n"
#define CYTHON_FORMAT_SSIZE_T "z"
#if PY_MAJOR_VERSION < 3
  #define __Pyx_BUILTIN_MODULE_NAME "__builtin__"
  #define __Pyx_DefaultClassType PyClass_Type
  #define __Pyx_PyCode_New(a, p, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a+k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
#else
  #define __Pyx_BUILTIN_MODULE_NAME "builtins"
  #define __Pyx_DefaultClassType PyType_Type
#if PY_VERSION_HEX >= 0x030800B2
  #define __Pyx_PyCode_New(a, p, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_NewWithPosOnlyArgs(a, p, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
#elif PY_VERSION_HEX >= 0x030800A4
  #define __Pyx_PyCode_New(a, p, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a, p, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
#else
  #define __Pyx_PyCode_New(a, p, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
#endif
#endif
#if PY_VERSION_HEX >= 0x030900A4 || defined(Py_IS_TYPE)
  #define __Pyx_IS_TYPE(ob, type) Py_IS_TYPE(ob, type)
#else
  #define __Pyx_IS_TYPE(ob, type) (((const PyObject*)ob)->ob_type == (type))
#endif
#ifndef Py_TPFLAGS_CHECKTYPES
  #define Py_TPFLAGS_CHECKTYPES 0
#endif
#ifndef Py_TPFLAGS_HAVE_INDEX
  #define Py_TPFLAGS_HAVE_INDEX 0
#endif
#ifndef Py_TPFLAGS_HAVE_NEWBUFFER
  #define Py_TPFLAGS_HAVE_NEWBUFFER 0
#endif
#ifndef Py_TPFLAGS_HAVE_FINALIZE
  #define Py_TPFLAGS_HAVE_FINALIZE 0
#endif
#ifndef METH_STACKLESS
  #define METH_STACKLESS 0
#endif
#if PY_VERSION_HEX <= 0x030700A3 || !defined(METH_FASTCALL)
  #ifndef METH_FASTCALL
     #define METH_FASTCALL 0x80
  #endif
  typedef PyObject *(*__Pyx_PyCFunctionFast) (PyObject *self, PyObject *const *args, Py_ssize_t nargs);
  typedef PyObject *(*__Pyx_PyCFunctionFastWithKeywords) (PyObject *self, PyObject *const *args,
                                                          Py_ssize_t nargs, PyObject *kwnames);
#else
  #define __Pyx_PyCFunctionFast _PyCFunctionFast
  #define __Pyx_PyCFunctionFastWithKeywords _PyCFunctionFastWithKeywords
#endif
#if CYTHON_METH_FASTCALL
  #define __Pyx_METH_FASTCALL METH_FASTCALL
  #define __Pyx_PyCFunction_FastCall __Pyx_PyCFunctionFast
  #define __Pyx_PyCFunction_FastCallWithKeywords __Pyx_PyCFunctionFastWithKeywords
#else
  #define __Pyx_METH_FASTCALL METH_VARARGS
  #define __Pyx_PyCFunction_FastCall PyCFunction
  #define __Pyx_PyCFunction_FastCallWithKeywords PyCFunctionWithKeywords
#endif
#if CYTHON_VECTORCALL
  #define __pyx_vectorcallfunc vectorcallfunc
  #define __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET  PY_VECTORCALL_ARGUMENTS_OFFSET
  #define __Pyx_PyVectorcall_NARGS(n)  PyVectorcall_NARGS((size_t)(n))
#elif CYTHON_BACKPORT_VECTORCALL
  typedef PyObject *(*__pyx_vectorcallfunc)(PyObject *callable, PyObject *const *args,
                                            size_t nargsf, PyObject *kwnames);
  #define __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET  ((size_t)1 << (8 * sizeof(size_t) - 1))
  #define __Pyx_PyVectorcall_NARGS(n)  ((Py_ssize_t)(((size_t)(n)) & ~__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET))
#else
  #define __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET  0
  #define __Pyx_PyVectorcall_NARGS(n)  ((Py_ssize_t)(n))
#endif
#if PY_VERSION_HEX < 0x030900B1
  #define __Pyx_PyType_FromModuleAndSpec(m, s, b)  ((void)m, PyType_FromSpecWithBases(s, b))
  typedef PyObject *(*__Pyx_PyCMethod)(PyObject *, PyTypeObject *, PyObject *const *, size_t, PyObject *);
#else
  #define __Pyx_PyType_FromModuleAndSpec(m, s, b)  PyType_FromModuleAndSpec(m, s, b)
  #define __Pyx_PyCMethod  PyCMethod
#endif
#ifndef METH_METHOD
  #define METH_METHOD 0x200
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyObject_Malloc)
  #define PyObject_Malloc(s)   PyMem_Malloc(s)
  #define PyObject_Free(p)     PyMem_Free(p)
  #define PyObject_Realloc(p)  PyMem_Realloc(p)
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030400A1
  #define PyMem_RawMalloc(n)           PyMem_Malloc(n)
  #define PyMem_RawRealloc(p, n)       PyMem_Realloc(p, n)
  #define PyMem_RawFree(p)             PyMem_Free(p)
#endif
#if CYTHON_COMPILING_IN_LIMITED_API
  #define __Pyx_PyCode_HasFreeVars(co)  (PyCode_GetNumFree(co) > 0)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno)
#else
  #define __Pyx_PyCode_HasFreeVars(co)  (PyCode_GetNumFree(co) > 0)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno)  (frame)->f_lineno = (lineno)
#endif
#if CYTHON_COMPILING_IN_LIMITED_API
  #define __Pyx_PyThreadState_Current PyThreadState_Get()
#elif !CYTHON_FAST_THREAD_STATE
  #define __Pyx_PyThreadState_Current PyThreadState_GET()
#elif PY_VERSION_HEX >= 0x03060000
  #define __Pyx_PyThreadState_Current _PyThreadState_UncheckedGet()
#elif PY_VERSION_HEX >= 0x03000000
  #define __Pyx_PyThreadState_Current PyThreadState_GET()
#else
  #define __Pyx_PyThreadState_Current _PyThreadState_Current
#endif
#if CYTHON_COMPILING_IN_LIMITED_API
static CYTHON_INLINE void *__Pyx_PyModule_GetState(PyObject *op)
{
    void *result;
    result = PyModule_GetState(op);
    if (!result)
        Py_FatalError("Couldn't find the module state");
    return result;
}
#endif
#if PY_VERSION_HEX < 0x030700A2 && !defined(PyThread_tss_create) && !defined(Py_tss_NEEDS_INIT)
#include "pythread.h"
#define Py_tss_NEEDS_INIT 0
typedef int Py_tss_t;
static CYTHON_INLINE int PyThread_tss_create(Py_tss_t *key) {
  *key = PyThread_create_key();
  return 0;
}
static CYTHON_INLINE Py_tss_t * PyThread_tss_alloc(void) {
  Py_tss_t *key = (Py_tss_t *)PyObject_Malloc(sizeof(Py_tss_t));
  *key = Py_tss_NEEDS_INIT;
  return key;
}
static CYTHON_INLINE void PyThread_tss_free(Py_tss_t *key) {
  PyObject_Free(key);
}
static CYTHON_INLINE int PyThread_tss_is_created(Py_tss_t *key) {
  return *key != Py_tss_NEEDS_INIT;
}
static CYTHON_INLINE void PyThread_tss_delete(Py_tss_t *key) {
  PyThread_delete_key(*key);
  *key = Py_tss_NEEDS_INIT;
}
static CYTHON_INLINE int PyThread_tss_set(Py_tss_t *key, void *value) {
  return PyThread_set_key_value(*key, value);
}
static CYTHON_INLINE void * PyThread_tss_get(Py_tss_t *key) {
  return PyThread_get_key_value(*key);
}
#endif
#if CYTHON_COMPILING_IN_CPYTHON || defined(_PyDict_NewPresized)
#define __Pyx_PyDict_NewPresized(n)  ((n <= 8) ? PyDict_New() : _PyDict_NewPresized(n))
#else
#define __Pyx_PyDict_NewPresized(n)  PyDict_New()
#endif
#if PY_MAJOR_VERSION >= 3 || CYTHON_FUTURE_DIVISION
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_TrueDivide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceTrueDivide(x,y)
#else
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_Divide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceDivide(x,y)
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX > 0x030600B4 && CYTHON_USE_UNICODE_INTERNALS
#define __Pyx_PyDict_GetItemStrWithError(dict, name)  _PyDict_GetItem_KnownHash(dict, name, ((PyASCIIObject *) name)->hash)
static CYTHON_INLINE PyObject * __Pyx_PyDict_GetItemStr(PyObject *dict, PyObject *name) {
    PyObject *res = __Pyx_PyDict_GetItemStrWithError(dict, name);
    if (res == NULL) PyErr_Clear();
    return res;
}
#elif PY_MAJOR_VERSION >= 3 && (!CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07020000)
#define __Pyx_PyDict_GetItemStrWithError  PyDict_GetItemWithError
#define __Pyx_PyDict_GetItemStr           PyDict_GetItem
#else
static CYTHON_INLINE PyObject * __Pyx_PyDict_GetItemStrWithError(PyObject *dict, PyObject *name) {
#if CYTHON_COMPILING_IN_PYPY
    return PyDict_GetItem(dict, name);
#else
    PyDictEntry *ep;
    PyDictObject *mp = (PyDictObject*) dict;
    long hash = ((PyStringObject *) name)->ob_shash;
    assert(hash != -1);
    ep = (mp->ma_lookup)(mp, name, hash);
    if (ep == NULL) {
        return NULL;
    }
    return ep->me_value;
#endif
}
#define __Pyx_PyDict_GetItemStr           PyDict_GetItem
#endif
#if CYTHON_USE_TYPE_SLOTS
  #define __Pyx_PyType_GetFlags(tp)   (((PyTypeObject *)tp)->tp_flags)
  #define __Pyx_PyType_HasFeature(type, feature)  ((__Pyx_PyType_GetFlags(type) & (feature)) != 0)
  #define __Pyx_PyObject_GetIterNextFunc(obj)  (Py_TYPE(obj)->tp_iternext)
#else
  #define __Pyx_PyType_GetFlags(tp)   (PyType_GetFlags((PyTypeObject *)tp))
  #define __Pyx_PyType_HasFeature(type, feature)  PyType_HasFeature(type, feature)
  #define __Pyx_PyObject_GetIterNextFunc(obj)  PyIter_Next
#endif
#if CYTHON_USE_TYPE_SPECS && PY_VERSION_HEX >= 0x03080000
#define __Pyx_PyHeapTypeObject_GC_Del(obj)  {\
    PyTypeObject *type = Py_TYPE(obj);\
    assert(__Pyx_PyType_HasFeature(type, Py_TPFLAGS_HEAPTYPE));\
    PyObject_GC_Del(obj);\
    Py_DECREF(type);\
}
#else
#define __Pyx_PyHeapTypeObject_GC_Del(obj)  PyObject_GC_Del(obj)
#endif
#if CYTHON_COMPILING_IN_LIMITED_API
  #define CYTHON_PEP393_ENABLED 1
  #define __Pyx_PyUnicode_READY(op)       (0)
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GetLength(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) PyUnicode_ReadChar(u, i)
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   ((void)u, 1114111)
  #define __Pyx_PyUnicode_KIND(u)         ((void)u, (0))
  #define __Pyx_PyUnicode_DATA(u)         ((void*)u)
  #define __Pyx_PyUnicode_READ(k, d, i)   ((void)k, PyUnicode_ReadChar((PyObject*)(d), i))
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != PyUnicode_GetLength(u))
#elif PY_VERSION_HEX > 0x03030000 && defined(PyUnicode_KIND)
  #define CYTHON_PEP393_ENABLED 1
  #if defined(PyUnicode_IS_READY)
  #define __Pyx_PyUnicode_READY(op)       (likely(PyUnicode_IS_READY(op)) ?\
                                              0 : _PyUnicode_Ready((PyObject *)(op)))
  #else
  #define __Pyx_PyUnicode_READY(op)       (0)
  #endif
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_LENGTH(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) PyUnicode_READ_CHAR(u, i)
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   PyUnicode_MAX_CHAR_VALUE(u)
  #define __Pyx_PyUnicode_KIND(u)         ((int)PyUnicode_KIND(u))
  #define __Pyx_PyUnicode_DATA(u)         PyUnicode_DATA(u)
  #define __Pyx_PyUnicode_READ(k, d, i)   PyUnicode_READ(k, d, i)
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  PyUnicode_WRITE(k, d, i, ch)
  #if defined(PyUnicode_IS_READY) && defined(PyUnicode_GET_SIZE)
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x03090000
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != (likely(PyUnicode_IS_READY(u)) ? PyUnicode_GET_LENGTH(u) : ((PyCompactUnicodeObject *)(u))->wstr_length))
  #else
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != (likely(PyUnicode_IS_READY(u)) ? PyUnicode_GET_LENGTH(u) : PyUnicode_GET_SIZE(u)))
  #endif
  #else
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != PyUnicode_GET_LENGTH(u))
  #endif
#else
  #define CYTHON_PEP393_ENABLED 0
  #define PyUnicode_1BYTE_KIND  1
  #define PyUnicode_2BYTE_KIND  2
  #define PyUnicode_4BYTE_KIND  4
  #define __Pyx_PyUnicode_READY(op)       (0)
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_SIZE(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) ((Py_UCS4)(PyUnicode_AS_UNICODE(u)[i]))
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   ((sizeof(Py_UNICODE) == 2) ? 65535 : 1114111)
  #define __Pyx_PyUnicode_KIND(u)         ((int)sizeof(Py_UNICODE))
  #define __Pyx_PyUnicode_DATA(u)         ((void*)PyUnicode_AS_UNICODE(u))
  #define __Pyx_PyUnicode_READ(k, d, i)   ((void)(k), (Py_UCS4)(((Py_UNICODE*)d)[i]))
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  (((void)(k)), ((Py_UNICODE*)d)[i] = ch)
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != PyUnicode_GET_SIZE(u))
#endif
#if CYTHON_COMPILING_IN_PYPY
  #define __Pyx_PyUnicode_Concat(a, b)      PyNumber_Add(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  PyNumber_Add(a, b)
#else
  #define __Pyx_PyUnicode_Concat(a, b)      PyUnicode_Concat(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  ((unlikely((a) == Py_None) || unlikely((b) == Py_None)) ?\
      PyNumber_Add(a, b) : __Pyx_PyUnicode_Concat(a, b))
#endif
#if CYTHON_COMPILING_IN_PYPY
  #if !defined(PyUnicode_DecodeUnicodeEscape)
    #define PyUnicode_DecodeUnicodeEscape(s, size, errors)  PyUnicode_Decode(s, size, "unicode_escape", errors)
  #endif
  #if !defined(PyUnicode_Contains) || (PY_MAJOR_VERSION == 2 && PYPY_VERSION_NUM < 0x07030500)
    #undef PyUnicode_Contains
    #define PyUnicode_Contains(u, s)  PySequence_Contains(u, s)
  #endif
  #if !defined(PyByteArray_Check)
    #define PyByteArray_Check(obj)  PyObject_TypeCheck(obj, &PyByteArray_Type)
  #endif
  #if !defined(PyObject_Format)
    #define PyObject_Format(obj, fmt)  PyObject_CallMethod(obj, "__format__", "O", fmt)
  #endif
#endif
#define __Pyx_PyString_FormatSafe(a, b)   ((unlikely((a) == Py_None || (PyString_Check(b) && !PyString_CheckExact(b)))) ? PyNumber_Remainder(a, b) : __Pyx_PyString_Format(a, b))
#define __Pyx_PyUnicode_FormatSafe(a, b)  ((unlikely((a) == Py_None || (PyUnicode_Check(b) && !PyUnicode_CheckExact(b)))) ? PyNumber_Remainder(a, b) : PyUnicode_Format(a, b))
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyString_Format(a, b)  PyUnicode_Format(a, b)
#else
  #define __Pyx_PyString_Format(a, b)  PyString_Format(a, b)
#endif
#if PY_MAJOR_VERSION < 3 && !defined(PyObject_ASCII)
  #define PyObject_ASCII(o)            PyObject_Repr(o)
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBaseString_Type            PyUnicode_Type
  #define PyStringObject               PyUnicodeObject
  #define PyString_Type                PyUnicode_Type
  #define PyString_Check               PyUnicode_Check
  #define PyString_CheckExact          PyUnicode_CheckExact
#ifndef PyObject_Unicode
  #define PyObject_Unicode             PyObject_Str
#endif
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyBaseString_Check(obj) PyUnicode_Check(obj)
  #define __Pyx_PyBaseString_CheckExact(obj) PyUnicode_CheckExact(obj)
#else
  #define __Pyx_PyBaseString_Check(obj) (PyString_Check(obj) || PyUnicode_Check(obj))
  #define __Pyx_PyBaseString_CheckExact(obj) (PyString_CheckExact(obj) || PyUnicode_CheckExact(obj))
#endif
#if CYTHON_COMPILING_IN_CPYTHON
  #define __Pyx_PySequence_ListKeepNew(obj)\
    (likely(PyList_CheckExact(obj) && Py_REFCNT(obj) == 1) ? __Pyx_NewRef(obj) : PySequence_List(obj))
#else
  #define __Pyx_PySequence_ListKeepNew(obj)  PySequence_List(obj)
#endif
#ifndef PySet_CheckExact
  #define PySet_CheckExact(obj)        __Pyx_IS_TYPE(obj, &PySet_Type)
#endif
#if PY_VERSION_HEX >= 0x030900A4
  #define __Pyx_SET_REFCNT(obj, refcnt) Py_SET_REFCNT(obj, refcnt)
  #define __Pyx_SET_SIZE(obj, size) Py_SET_SIZE(obj, size)
#else
  #define __Pyx_SET_REFCNT(obj, refcnt) Py_REFCNT(obj) = (refcnt)
  #define __Pyx_SET_SIZE(obj, size) Py_SIZE(obj) = (size)
#endif
#if CYTHON_ASSUME_SAFE_MACROS
  #define __Pyx_PySequence_SIZE(seq)  Py_SIZE(seq)
#else
  #define __Pyx_PySequence_SIZE(seq)  PySequence_Size(seq)
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyIntObject                  PyLongObject
  #define PyInt_Type                   PyLong_Type
  #define PyInt_Check(op)              PyLong_Check(op)
  #define PyInt_CheckExact(op)         PyLong_CheckExact(op)
  #define PyInt_FromString             PyLong_FromString
  #define PyInt_FromUnicode            PyLong_FromUnicode
  #define PyInt_FromLong               PyLong_FromLong
  #define PyInt_FromSize_t             PyLong_FromSize_t
  #define PyInt_FromSsize_t            PyLong_FromSsize_t
  #define PyInt_AsLong                 PyLong_AsLong
  #define PyInt_AS_LONG                PyLong_AS_LONG
  #define PyInt_AsSsize_t              PyLong_AsSsize_t
  #define PyInt_AsUnsignedLongMask     PyLong_AsUnsignedLongMask
  #define PyInt_AsUnsignedLongLongMask PyLong_AsUnsignedLongLongMask
  #define PyNumber_Int                 PyNumber_Long
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBoolObject                 PyLongObject
#endif
#if PY_MAJOR_VERSION >= 3 && CYTHON_COMPILING_IN_PYPY
  #ifndef PyUnicode_InternFromString
    #define PyUnicode_InternFromString(s) PyUnicode_FromString(s)
  #endif
#endif
#if PY_VERSION_HEX < 0x030200A4
  typedef long Py_hash_t;
  #define __Pyx_PyInt_FromHash_t PyInt_FromLong
  #define __Pyx_PyInt_AsHash_t   __Pyx_PyIndex_AsHash_t
#else
  #define __Pyx_PyInt_FromHash_t PyInt_FromSsize_t
  #define __Pyx_PyInt_AsHash_t   __Pyx_PyIndex_AsSsize_t
#endif
#if CYTHON_USE_ASYNC_SLOTS
  #if PY_VERSION_HEX >= 0x030500B1
    #define __Pyx_PyAsyncMethodsStruct PyAsyncMethods
    #define __Pyx_PyType_AsAsync(obj) (Py_TYPE(obj)->tp_as_async)
  #else
    #define __Pyx_PyType_AsAsync(obj) ((__Pyx_PyAsyncMethodsStruct*) (Py_TYPE(obj)->tp_reserved))
  #endif
#else
  #define __Pyx_PyType_AsAsync(obj) NULL
#endif
#ifndef __Pyx_PyAsyncMethodsStruct
    typedef struct {
        unaryfunc am_await;
        unaryfunc am_aiter;
        unaryfunc am_anext;
    } __Pyx_PyAsyncMethodsStruct;
#endif

#if defined(_WIN32) || defined(WIN32) || defined(MS_WINDOWS)
  #define _USE_MATH_DEFINES
#endif
#include <math.h>
#ifdef NAN
#define __PYX_NAN() ((float) NAN)
#else
static CYTHON_INLINE float __PYX_NAN() {
  float value;
  memset(&value, 0xFF, sizeof(value));
  return value;
}
#endif
#if defined(__CYGWIN__) && defined(_LDBL_EQ_DBL)
#define __Pyx_truncl trunc
#else
#define __Pyx_truncl truncl
#endif

#define __PYX_MARK_ERR_POS(f_index, lineno) \
    { __pyx_filename = __pyx_f[f_index]; (void)__pyx_filename; __pyx_lineno = lineno; (void)__pyx_lineno; __pyx_clineno = __LINE__; (void)__pyx_clineno; }
#define __PYX_ERR(f_index, lineno, Ln_error) \
    { __PYX_MARK_ERR_POS(f_index, lineno) goto Ln_error; }

#ifndef __PYX_EXTERN_C
  #ifdef __cplusplus
    #define __PYX_EXTERN_C extern "C"
  #else
    #define __PYX_EXTERN_C extern
  #endif
#endif

#define __PYX_HAVE__aim__storage__prefixview
#define __PYX_HAVE_API__aim__storage__prefixview
/* Early includes */
#include <string.h>
#include <string>
#include "ios"
#include "new"
#include "stdexcept"
#include "typeinfo"
#include <deque>
#include <vector>
#include <stdio.h>
#include "pythread.h"
#include <stdint.h>
#include <memory>
#include "rocksdb/slice.h"
#include "rocksdb/env.h"
#include "rocksdb/comparator.h"
#include "rdb_include/comparator_wrapper.hpp"
#include "rocksdb/merge_operator.h"
#include "rdb_include/merge_operator_wrapper.hpp"
#include "rocksdb/db.h"
#include "rocksdb/slice_transform.h"
#include "rdb_include/slice_transform_wrapper.hpp"
#include "rocksdb/cache.h"
#include "rocksdb/filter_policy.h"
#include "rdb_include/filter_policy_wrapper.hpp"
#include "rocksdb/table.h"
#include "rocksdb/memtablerep.h"
#include "rdb_include/memtable_factories.hpp"
#include "rocksdb/universal_compaction.h"
#include "rocksdb/options.h"
#include "rocksdb/status.h"
#include "rocksdb/iterator.h"
#include "rocksdb/write_batch.h"
#include "rdb_include/write_batch_iter_helper.hpp"
#include "rocksdb/utilities/backup_engine.h"
#include "rdb_include/utils.hpp"
#include <utility>

    #if __cplusplus >= 201103L || (defined(_MSC_VER) && _MSC_VER >= 1600)
    // move should be defined for these versions of MSVC, but __cplusplus isn't set usefully
    #include <type_traits>

    namespace cython_std {
    template <typename T> typename std::remove_reference<T>::type&& move(T& t) noexcept { return std::move(t); }
    template <typename T> typename std::remove_reference<T>::type&& move(T&& t) noexcept { return std::move(t); }
    }

    #endif
    
#ifdef _OPENMP
#include <omp.h>
#endif /* _OPENMP */

#if defined(PYREX_WITHOUT_ASSERTIONS) && !defined(CYTHON_WITHOUT_ASSERTIONS)
#define CYTHON_WITHOUT_ASSERTIONS
#endif

typedef struct {PyObject **p; const char *s; const Py_ssize_t n; const char* encoding;
                const char is_unicode; const char is_str; const char intern; } __Pyx_StringTabEntry;

#define __PYX_DEFAULT_STRING_ENCODING_IS_ASCII 0
#define __PYX_DEFAULT_STRING_ENCODING_IS_UTF8 0
#define __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT (PY_MAJOR_VERSION >= 3 && __PYX_DEFAULT_STRING_ENCODING_IS_UTF8)
#define __PYX_DEFAULT_STRING_ENCODING ""
#define __Pyx_PyObject_FromString __Pyx_PyBytes_FromString
#define __Pyx_PyObject_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#define __Pyx_uchar_cast(c) ((unsigned char)c)
#define __Pyx_long_cast(x) ((long)x)
#define __Pyx_fits_Py_ssize_t(v, type, is_signed)  (\
    (sizeof(type) < sizeof(Py_ssize_t))  ||\
    (sizeof(type) > sizeof(Py_ssize_t) &&\
          likely(v < (type)PY_SSIZE_T_MAX ||\
                 v == (type)PY_SSIZE_T_MAX)  &&\
          (!is_signed || likely(v > (type)PY_SSIZE_T_MIN ||\
                                v == (type)PY_SSIZE_T_MIN)))  ||\
    (sizeof(type) == sizeof(Py_ssize_t) &&\
          (is_signed || likely(v < (type)PY_SSIZE_T_MAX ||\
                               v == (type)PY_SSIZE_T_MAX)))  )
static CYTHON_INLINE int __Pyx_is_valid_index(Py_ssize_t i, Py_ssize_t limit) {
    return (size_t) i < (size_t) limit;
}
#if defined (__cplusplus) && __cplusplus >= 201103L
    #include <cstdlib>
    #define __Pyx_sst_abs(value) std::abs(value)
#elif SIZEOF_INT >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) abs(value)
#elif SIZEOF_LONG >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) labs(value)
#elif defined (_MSC_VER)
    #define __Pyx_sst_abs(value) ((Py_ssize_t)_abs64(value))
#elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define __Pyx_sst_abs(value) llabs(value)
#elif defined (__GNUC__)
    #define __Pyx_sst_abs(value) __builtin_llabs(value)
#else
    #define __Pyx_sst_abs(value) ((value<0) ? -value : value)
#endif
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject*);
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject*, Py_ssize_t* length);
#define __Pyx_PyByteArray_FromString(s) PyByteArray_FromStringAndSize((const char*)s, strlen((const char*)s))
#define __Pyx_PyByteArray_FromStringAndSize(s, l) PyByteArray_FromStringAndSize((const char*)s, l)
#define __Pyx_PyBytes_FromString        PyBytes_FromString
#define __Pyx_PyBytes_FromStringAndSize PyBytes_FromStringAndSize
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char*);
#if PY_MAJOR_VERSION < 3
    #define __Pyx_PyStr_FromString        __Pyx_PyBytes_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#else
    #define __Pyx_PyStr_FromString        __Pyx_PyUnicode_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyUnicode_FromStringAndSize
#endif
#define __Pyx_PyBytes_AsWritableString(s)     ((char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsWritableSString(s)    ((signed char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsWritableUString(s)    ((unsigned char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsString(s)     ((const char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsSString(s)    ((const signed char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsUString(s)    ((const unsigned char*) PyBytes_AS_STRING(s))
#define __Pyx_PyObject_AsWritableString(s)    ((char*)(__pyx_uintptr_t) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableSString(s)    ((signed char*)(__pyx_uintptr_t) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableUString(s)    ((unsigned char*)(__pyx_uintptr_t) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsSString(s)    ((const signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsUString(s)    ((const unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_FromCString(s)  __Pyx_PyObject_FromString((const char*)s)
#define __Pyx_PyBytes_FromCString(s)   __Pyx_PyBytes_FromString((const char*)s)
#define __Pyx_PyByteArray_FromCString(s)   __Pyx_PyByteArray_FromString((const char*)s)
#define __Pyx_PyStr_FromCString(s)     __Pyx_PyStr_FromString((const char*)s)
#define __Pyx_PyUnicode_FromCString(s) __Pyx_PyUnicode_FromString((const char*)s)
#if CYTHON_COMPILING_IN_LIMITED_API
static CYTHON_INLINE size_t __Pyx_Py_UNICODE_strlen(const wchar_t *u)
{
    const wchar_t *u_end = u;
    while (*u_end++) ;
    return (size_t)(u_end - u - 1);
}
#else
static CYTHON_INLINE size_t __Pyx_Py_UNICODE_strlen(const Py_UNICODE *u)
{
    const Py_UNICODE *u_end = u;
    while (*u_end++) ;
    return (size_t)(u_end - u - 1);
}
#endif
#define __Pyx_PyUnicode_FromUnicode(u)       PyUnicode_FromUnicode(u, __Pyx_Py_UNICODE_strlen(u))
#define __Pyx_PyUnicode_FromUnicodeAndLength PyUnicode_FromUnicode
#define __Pyx_PyUnicode_AsUnicode            PyUnicode_AsUnicode
#define __Pyx_NewRef(obj) (Py_INCREF(obj), obj)
#define __Pyx_Owned_Py_None(b) __Pyx_NewRef(Py_None)
static CYTHON_INLINE PyObject * __Pyx_PyBool_FromLong(long b);
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject*);
static CYTHON_INLINE int __Pyx_PyObject_IsTrueAndDecref(PyObject*);
static CYTHON_INLINE PyObject* __Pyx_PyNumber_IntOrLong(PyObject* x);
#define __Pyx_PySequence_Tuple(obj)\
    (likely(PyTuple_CheckExact(obj)) ? __Pyx_NewRef(obj) : PySequence_Tuple(obj))
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject*);
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t);
static CYTHON_INLINE Py_hash_t __Pyx_PyIndex_AsHash_t(PyObject*);
#if CYTHON_ASSUME_SAFE_MACROS
#define __pyx_PyFloat_AsDouble(x) (PyFloat_CheckExact(x) ? PyFloat_AS_DOUBLE(x) : PyFloat_AsDouble(x))
#else
#define __pyx_PyFloat_AsDouble(x) PyFloat_AsDouble(x)
#endif
#define __pyx_PyFloat_AsFloat(x) ((float) __pyx_PyFloat_AsDouble(x))
#if PY_MAJOR_VERSION >= 3
#define __Pyx_PyNumber_Int(x) (PyLong_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Long(x))
#else
#define __Pyx_PyNumber_Int(x) (PyInt_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Int(x))
#endif
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
static int __Pyx_sys_getdefaultencoding_not_ascii;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    PyObject* ascii_chars_u = NULL;
    PyObject* ascii_chars_b = NULL;
    const char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    if (strcmp(default_encoding_c, "ascii") == 0) {
        __Pyx_sys_getdefaultencoding_not_ascii = 0;
    } else {
        char ascii_chars[128];
        int c;
        for (c = 0; c < 128; c++) {
            ascii_chars[c] = c;
        }
        __Pyx_sys_getdefaultencoding_not_ascii = 1;
        ascii_chars_u = PyUnicode_DecodeASCII(ascii_chars, 128, NULL);
        if (!ascii_chars_u) goto bad;
        ascii_chars_b = PyUnicode_AsEncodedString(ascii_chars_u, default_encoding_c, NULL);
        if (!ascii_chars_b || !PyBytes_Check(ascii_chars_b) || memcmp(ascii_chars, PyBytes_AS_STRING(ascii_chars_b), 128) != 0) {
            PyErr_Format(
                PyExc_ValueError,
                "This module compiled with c_string_encoding=ascii, but default encoding '%.200s' is not a superset of ascii.",
                default_encoding_c);
            goto bad;
        }
        Py_DECREF(ascii_chars_u);
        Py_DECREF(ascii_chars_b);
    }
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    Py_XDECREF(ascii_chars_u);
    Py_XDECREF(ascii_chars_b);
    return -1;
}
#endif
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT && PY_MAJOR_VERSION >= 3
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_DecodeUTF8(c_str, size, NULL)
#else
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_Decode(c_str, size, __PYX_DEFAULT_STRING_ENCODING, NULL)
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
static char* __PYX_DEFAULT_STRING_ENCODING;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) (const char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    __PYX_DEFAULT_STRING_ENCODING = (char*) malloc(strlen(default_encoding_c) + 1);
    if (!__PYX_DEFAULT_STRING_ENCODING) goto bad;
    strcpy(__PYX_DEFAULT_STRING_ENCODING, default_encoding_c);
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    return -1;
}
#endif
#endif


/* Test for GCC > 2.95 */
#if defined(__GNUC__)     && (__GNUC__ > 2 || (__GNUC__ == 2 && (__GNUC_MINOR__ > 95)))
  #define likely(x)   __builtin_expect(!!(x), 1)
  #define unlikely(x) __builtin_expect(!!(x), 0)
#else /* !__GNUC__ or GCC < 2.95 */
  #define likely(x)   (x)
  #define unlikely(x) (x)
#endif /* __GNUC__ */
static CYTHON_INLINE void __Pyx_pretend_to_initialize(void* ptr) { (void)ptr; }

#if !CYTHON_USE_MODULE_STATE
static PyObject *__pyx_m = NULL;
static PyObject *__pyx_d;
static PyObject *__pyx_b;
static PyObject *__pyx_cython_runtime = NULL;
static PyObject *__pyx_empty_tuple;
static PyObject *__pyx_empty_bytes;
static PyObject *__pyx_empty_unicode;
#endif
static int __pyx_lineno;
static int __pyx_clineno = 0;
static const char * __pyx_cfilenm = __FILE__;
static const char *__pyx_filename;

/* #### Code section: filename_table ### */

static const char *__pyx_f[] = {
  "aim/storage/prefixview.py",
  "aim/storage/prefixview.pxd",
  "stringsource",
  "contextvars.pxd",
  "type.pxd",
  "bool.pxd",
  "complex.pxd",
  "lib_rocksdb.pxd",
  "aim/storage/utils.pxd",
  "aim/storage/treeview.pxd",
  "aim/storage/containertreeview.pxd",
};
/* #### Code section: utility_code_proto_before_types ### */
/* #### Code section: numeric_typedefs ### */

/* "aim/storage/encoding/encoding_native.pxd":15
 *     PATH_SENTINEL_CODE = 0xfe
 * 
 * ctypedef long long int64             # <<<<<<<<<<<<<<
 * ctypedef unsigned long long uint64
 * 
 */
typedef PY_LONG_LONG __pyx_t_3aim_7storage_8encoding_15encoding_native_int64;

/* "aim/storage/encoding/encoding_native.pxd":16
 * 
 * ctypedef long long int64
 * ctypedef unsigned long long uint64             # <<<<<<<<<<<<<<
 * 
 * cpdef bytes encode_int64_big_endian(int64 value)
 */
typedef unsigned PY_LONG_LONG __pyx_t_3aim_7storage_8encoding_15encoding_native_uint64;
/* #### Code section: complex_type_declarations ### */
/* #### Code section: type_declarations ### */

/*--- Type declarations ---*/
struct __pyx_obj_8aimrocks_11lib_rocksdb_PyComparator;
struct __pyx_obj_8aimrocks_11lib_rocksdb_PyGenericComparator;
struct __pyx_obj_8aimrocks_11lib_rocksdb_PyBytewiseComparator;
struct __pyx_obj_8aimrocks_11lib_rocksdb_PyFilterPolicy;
struct __pyx_obj_8aimrocks_11lib_rocksdb_PyGenericFilterPolicy;
struct __pyx_obj_8aimrocks_11lib_rocksdb_PyBloomFilterPolicy;
struct __pyx_obj_8aimrocks_11lib_rocksdb_PyMergeOperator;
struct __pyx_obj_8aimrocks_11lib_rocksdb_PyCache;
struct __pyx_obj_8aimrocks_11lib_rocksdb_PyLRUCache;
struct __pyx_obj_8aimrocks_11lib_rocksdb_PySliceTransform;
struct __pyx_obj_8aimrocks_11lib_rocksdb_PyTableFactory;
struct __pyx_obj_8aimrocks_11lib_rocksdb_BlockBasedTableFactory;
struct __pyx_obj_8aimrocks_11lib_rocksdb_PlainTableFactory;
struct __pyx_obj_8aimrocks_11lib_rocksdb_PyMemtableFactory;
struct __pyx_obj_8aimrocks_11lib_rocksdb_SkipListMemtableFactory;
struct __pyx_obj_8aimrocks_11lib_rocksdb_VectorMemtableFactory;
struct __pyx_obj_8aimrocks_11lib_rocksdb_HashSkipListMemtableFactory;
struct __pyx_obj_8aimrocks_11lib_rocksdb_HashLinkListMemtableFactory;
struct __pyx_obj_8aimrocks_11lib_rocksdb_CompressionType;
struct __pyx_obj_8aimrocks_11lib_rocksdb_CompactionPri;
struct __pyx_obj_8aimrocks_11lib_rocksdb__ColumnFamilyHandle;
struct __pyx_obj_8aimrocks_11lib_rocksdb_ColumnFamilyHandle;
struct __pyx_obj_8aimrocks_11lib_rocksdb_ColumnFamilyOptions;
struct __pyx_obj_8aimrocks_11lib_rocksdb_Options;
struct __pyx_obj_8aimrocks_11lib_rocksdb_WriteBatch;
struct __pyx_obj_8aimrocks_11lib_rocksdb_WriteBatchIterator;
struct __pyx_obj_8aimrocks_11lib_rocksdb_IDB;
struct __pyx_obj_8aimrocks_11lib_rocksdb_DB;
struct __pyx_obj_8aimrocks_11lib_rocksdb_Snapshot;
struct __pyx_obj_8aimrocks_11lib_rocksdb_Iterator;
struct __pyx_obj_8aimrocks_11lib_rocksdb_BaseIterator;
struct __pyx_obj_8aimrocks_11lib_rocksdb_KeysIterator;
struct __pyx_obj_8aimrocks_11lib_rocksdb_ValuesIterator;
struct __pyx_obj_8aimrocks_11lib_rocksdb_ItemsIterator;
struct __pyx_obj_8aimrocks_11lib_rocksdb_ReversedIterator;
struct __pyx_obj_8aimrocks_11lib_rocksdb_BackupEngine;
struct __pyx_obj_3aim_7storage_5utils_ArrayFlagType;
struct __pyx_obj_3aim_7storage_5utils_ObjectFlagType;
struct __pyx_obj_3aim_7storage_5utils_CustomObjectFlagType;
struct __pyx_obj_3aim_7storage_5utils_BLOB;
struct __pyx_obj_3aim_7storage_9container_ContainerItemsIterator;
struct __pyx_obj_3aim_7storage_9container_Container;
struct __pyx_obj_3aim_7storage_8treeview_TreeView;
struct __pyx_obj_3aim_7storage_17containertreeview_ContainerTreeView;
struct __pyx_obj_3aim_7storage_10prefixview_PrefixView;
struct __pyx_obj_3aim_7storage_10prefixview_PrefixViewItemsIterator;
struct __pyx_obj_3aim_7storage_10prefixview___pyx_scope_struct__walk;
struct __pyx_opt_args_7cpython_11contextvars_get_value;
struct __pyx_opt_args_7cpython_11contextvars_get_value_no_default;

/* "cpython/contextvars.pxd":112
 * 
 * 
 * cdef inline object get_value(var, default_value=None):             # <<<<<<<<<<<<<<
 *     """Return a new reference to the value of the context variable,
 *     or the default value of the context variable,
 */
struct __pyx_opt_args_7cpython_11contextvars_get_value {
  int __pyx_n;
  PyObject *default_value;
};

/* "cpython/contextvars.pxd":129
 * 
 * 
 * cdef inline object get_value_no_default(var, default_value=None):             # <<<<<<<<<<<<<<
 *     """Return a new reference to the value of the context variable,
 *     or the provided default value if no such value was found.
 */
struct __pyx_opt_args_7cpython_11contextvars_get_value_no_default {
  int __pyx_n;
  PyObject *default_value;
};

/* "aimrocks/comparator.pxd":13
 *     cdef extern const Comparator* BytewiseComparator() nogil except +
 * 
 * ctypedef int (*compare_func)(             # <<<<<<<<<<<<<<
 *     void*,
 *     Logger*,
 */
typedef int (*__pyx_t_8aimrocks_10comparator_compare_func)(void *, rocksdb::Logger *, std::string &, rocksdb::Slice const &, rocksdb::Slice const &);

/* "aimrocks/merge_operator.pxd":12
 *         pass
 * 
 * ctypedef cpp_bool (*merge_func)(             # <<<<<<<<<<<<<<
 *     void*,
 *     const Slice&,
 */
typedef bool (*__pyx_t_8aimrocks_14merge_operator_merge_func)(void *, rocksdb::Slice const &, rocksdb::Slice const *, rocksdb::Slice const &, std::string *, rocksdb::Logger *);

/* "aimrocks/merge_operator.pxd":20
 *     Logger*)
 * 
 * ctypedef cpp_bool (*full_merge_func)(             # <<<<<<<<<<<<<<
 *     void* ctx,
 *     const Slice& key,
 */
typedef bool (*__pyx_t_8aimrocks_14merge_operator_full_merge_func)(void *, rocksdb::Slice const &, rocksdb::Slice const *, std::deque<std::string>  const &, std::string *, rocksdb::Logger *);

/* "aimrocks/merge_operator.pxd":28
 *     Logger* logger)
 * 
 * ctypedef cpp_bool (*partial_merge_func)(             # <<<<<<<<<<<<<<
 *     void* ctx,
 *     const Slice& key,
 */
typedef bool (*__pyx_t_8aimrocks_14merge_operator_partial_merge_func)(void *, rocksdb::Slice const &, rocksdb::Slice const &, rocksdb::Slice const &, std::string *, rocksdb::Logger *);

/* "aimrocks/slice_transform.pxd":11
 *         pass
 * 
 * ctypedef Slice (*transform_func)(             # <<<<<<<<<<<<<<
 *     void*,
 *     Logger*,
 */
typedef rocksdb::Slice (*__pyx_t_8aimrocks_15slice_transform_transform_func)(void *, rocksdb::Logger *, std::string &, rocksdb::Slice const &);

/* "aimrocks/slice_transform.pxd":17
 *     const Slice&)
 * 
 * ctypedef cpp_bool (*in_domain_func)(             # <<<<<<<<<<<<<<
 *     void*,
 *     Logger*,
 */
typedef bool (*__pyx_t_8aimrocks_15slice_transform_in_domain_func)(void *, rocksdb::Logger *, std::string &, rocksdb::Slice const &);

/* "aimrocks/slice_transform.pxd":23
 *     const Slice&)
 * 
 * ctypedef cpp_bool (*in_range_func)(             # <<<<<<<<<<<<<<
 *     void*,
 *     Logger*,
 */
typedef bool (*__pyx_t_8aimrocks_15slice_transform_in_range_func)(void *, rocksdb::Logger *, std::string &, rocksdb::Slice const &);

/* "aimrocks/filter_policy.pxd":16
 *     cdef extern const FilterPolicy* NewBloomFilterPolicy(int) nogil except+
 * 
 * ctypedef void (*create_filter_func)(             # <<<<<<<<<<<<<<
 *     void*,
 *     Logger*,
 */
typedef void (*__pyx_t_8aimrocks_13filter_policy_create_filter_func)(void *, rocksdb::Logger *, std::string &, rocksdb::Slice const *, int, std::string *);

/* "aimrocks/filter_policy.pxd":24
 *     string*)
 * 
 * ctypedef cpp_bool (*key_may_match_func)(             # <<<<<<<<<<<<<<
 *     void*,
 *     Logger*,
 */
typedef bool (*__pyx_t_8aimrocks_13filter_policy_key_may_match_func)(void *, rocksdb::Logger *, std::string &, rocksdb::Slice const &, rocksdb::Slice const &);
struct __pyx_opt_args_8aimrocks_11lib_rocksdb_10WriteBatch_put;
struct __pyx_opt_args_8aimrocks_11lib_rocksdb_10WriteBatch_merge;
struct __pyx_opt_args_8aimrocks_11lib_rocksdb_10WriteBatch_delete_single;
struct __pyx_opt_args_8aimrocks_11lib_rocksdb_10WriteBatch_delete_range;
struct __pyx_opt_args_8aimrocks_11lib_rocksdb_3IDB_put;
struct __pyx_opt_args_8aimrocks_11lib_rocksdb_3IDB_delete_single;
struct __pyx_opt_args_8aimrocks_11lib_rocksdb_3IDB_delete_range;
struct __pyx_opt_args_8aimrocks_11lib_rocksdb_3IDB_flush_wal;
struct __pyx_opt_args_8aimrocks_11lib_rocksdb_3IDB_merge;
struct __pyx_opt_args_8aimrocks_11lib_rocksdb_3IDB_write;
struct __pyx_opt_args_8aimrocks_11lib_rocksdb_3IDB_get;
struct __pyx_opt_args_8aimrocks_11lib_rocksdb_3IDB_key_may_exist;
struct __pyx_opt_args_8aimrocks_11lib_rocksdb_3IDB_iterkeys;
struct __pyx_opt_args_8aimrocks_11lib_rocksdb_3IDB_itervalues;
struct __pyx_opt_args_8aimrocks_11lib_rocksdb_3IDB_iteritems;
struct __pyx_opt_args_8aimrocks_11lib_rocksdb_2DB_get_property;
struct __pyx_opt_args_8aimrocks_11lib_rocksdb_2DB_compact_range;
struct __pyx_opt_args_8aimrocks_11lib_rocksdb_12BackupEngine_create_backup;

/* "aimrocks/lib_rocksdb.pxd":48
 * from aimrocks.status cimport Status
 * 
 * ctypedef const filter_policy.FilterPolicy ConstFilterPolicy             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "rdb_include/utils.hpp" namespace "py_rocks":
 */
typedef rocksdb::FilterPolicy const __pyx_t_8aimrocks_11lib_rocksdb_ConstFilterPolicy;

/* "aimrocks/lib_rocksdb.pxd":577
 *     # def __cinit__(self, data = *)
 *     # def __dealloc__(self)
 *     cpdef void put(self, bytes key, bytes value, ColumnFamilyHandle column_family = *)             # <<<<<<<<<<<<<<
 *     cpdef void merge(self, bytes key, bytes value, ColumnFamilyHandle column_family = *)
 *     cpdef void delete_single(self, bytes key, ColumnFamilyHandle column_family = *)
 */
struct __pyx_opt_args_8aimrocks_11lib_rocksdb_10WriteBatch_put {
  int __pyx_n;
  struct __pyx_obj_8aimrocks_11lib_rocksdb_ColumnFamilyHandle *column_family;
};

/* "aimrocks/lib_rocksdb.pxd":578
 *     # def __dealloc__(self)
 *     cpdef void put(self, bytes key, bytes value, ColumnFamilyHandle column_family = *)
 *     cpdef void merge(self, bytes key, bytes value, ColumnFamilyHandle column_family = *)             # <<<<<<<<<<<<<<
 *     cpdef void delete_single(self, bytes key, ColumnFamilyHandle column_family = *)
 *     cpdef void delete_range(self, bytes begin_key, bytes end_key, ColumnFamilyHandle column_family = *)
 */
struct __pyx_opt_args_8aimrocks_11lib_rocksdb_10WriteBatch_merge {
  int __pyx_n;
  struct __pyx_obj_8aimrocks_11lib_rocksdb_ColumnFamilyHandle *column_family;
};

/* "aimrocks/lib_rocksdb.pxd":579
 *     cpdef void put(self, bytes key, bytes value, ColumnFamilyHandle column_family = *)
 *     cpdef void merge(self, bytes key, bytes value, ColumnFamilyHandle column_family = *)
 *     cpdef void delete_single(self, bytes key, ColumnFamilyHandle column_family = *)             # <<<<<<<<<<<<<<
 *     cpdef void delete_range(self, bytes begin_key, bytes end_key, ColumnFamilyHandle column_family = *)
 *     cpdef void clear(self)
 */
struct __pyx_opt_args_8aimrocks_11lib_rocksdb_10WriteBatch_delete_single {
  int __pyx_n;
  struct __pyx_obj_8aimrocks_11lib_rocksdb_ColumnFamilyHandle *column_family;
};

/* "aimrocks/lib_rocksdb.pxd":580
 *     cpdef void merge(self, bytes key, bytes value, ColumnFamilyHandle column_family = *)
 *     cpdef void delete_single(self, bytes key, ColumnFamilyHandle column_family = *)
 *     cpdef void delete_range(self, bytes begin_key, bytes end_key, ColumnFamilyHandle column_family = *)             # <<<<<<<<<<<<<<
 *     cpdef void clear(self)
 *     cpdef data(self)
 */
struct __pyx_opt_args_8aimrocks_11lib_rocksdb_10WriteBatch_delete_range {
  int __pyx_n;
  struct __pyx_obj_8aimrocks_11lib_rocksdb_ColumnFamilyHandle *column_family;
};

/* "aimrocks/lib_rocksdb.pxd":597
 * 
 * cdef class IDB(object):
 *     cpdef void put(self, bytes key, bytes value, cpp_bool sync = *, cpp_bool disable_wal = *, ColumnFamilyHandle column_family = *)             # <<<<<<<<<<<<<<
 *     cpdef void delete_single(self, bytes key, cpp_bool sync = *, cpp_bool disable_wal = *, ColumnFamilyHandle column_family = *)
 *     cpdef void delete_range(self, bytes begin_key, bytes end_key, cpp_bool sync = *, cpp_bool disable_wal = *, ColumnFamilyHandle column_family = *)
 */
struct __pyx_opt_args_8aimrocks_11lib_rocksdb_3IDB_put {
  int __pyx_n;
  bool sync;
  bool disable_wal;
  struct __pyx_obj_8aimrocks_11lib_rocksdb_ColumnFamilyHandle *column_family;
};

/* "aimrocks/lib_rocksdb.pxd":598
 * cdef class IDB(object):
 *     cpdef void put(self, bytes key, bytes value, cpp_bool sync = *, cpp_bool disable_wal = *, ColumnFamilyHandle column_family = *)
 *     cpdef void delete_single(self, bytes key, cpp_bool sync = *, cpp_bool disable_wal = *, ColumnFamilyHandle column_family = *)             # <<<<<<<<<<<<<<
 *     cpdef void delete_range(self, bytes begin_key, bytes end_key, cpp_bool sync = *, cpp_bool disable_wal = *, ColumnFamilyHandle column_family = *)
 *     cpdef void flush(self)
 */
struct __pyx_opt_args_8aimrocks_11lib_rocksdb_3IDB_delete_single {
  int __pyx_n;
  bool sync;
  bool disable_wal;
  struct __pyx_obj_8aimrocks_11lib_rocksdb_ColumnFamilyHandle *column_family;
};

/* "aimrocks/lib_rocksdb.pxd":599
 *     cpdef void put(self, bytes key, bytes value, cpp_bool sync = *, cpp_bool disable_wal = *, ColumnFamilyHandle column_family = *)
 *     cpdef void delete_single(self, bytes key, cpp_bool sync = *, cpp_bool disable_wal = *, ColumnFamilyHandle column_family = *)
 *     cpdef void delete_range(self, bytes begin_key, bytes end_key, cpp_bool sync = *, cpp_bool disable_wal = *, ColumnFamilyHandle column_family = *)             # <<<<<<<<<<<<<<
 *     cpdef void flush(self)
 *     cpdef void flush_wal(self, cpp_bool sync = *)
 */
struct __pyx_opt_args_8aimrocks_11lib_rocksdb_3IDB_delete_range {
  int __pyx_n;
  bool sync;
  bool disable_wal;
  struct __pyx_obj_8aimrocks_11lib_rocksdb_ColumnFamilyHandle *column_family;
};

/* "aimrocks/lib_rocksdb.pxd":601
 *     cpdef void delete_range(self, bytes begin_key, bytes end_key, cpp_bool sync = *, cpp_bool disable_wal = *, ColumnFamilyHandle column_family = *)
 *     cpdef void flush(self)
 *     cpdef void flush_wal(self, cpp_bool sync = *)             # <<<<<<<<<<<<<<
 *     cpdef void merge(self, bytes key, bytes value, cpp_bool sync = *, cpp_bool disable_wal = *, ColumnFamilyHandle column_family = *)
 *     cpdef void write(self, WriteBatch batch, cpp_bool sync = *, cpp_bool disable_wal = *)
 */
struct __pyx_opt_args_8aimrocks_11lib_rocksdb_3IDB_flush_wal {
  int __pyx_n;
  bool sync;
};

/* "aimrocks/lib_rocksdb.pxd":602
 *     cpdef void flush(self)
 *     cpdef void flush_wal(self, cpp_bool sync = *)
 *     cpdef void merge(self, bytes key, bytes value, cpp_bool sync = *, cpp_bool disable_wal = *, ColumnFamilyHandle column_family = *)             # <<<<<<<<<<<<<<
 *     cpdef void write(self, WriteBatch batch, cpp_bool sync = *, cpp_bool disable_wal = *)
 *     cpdef get(self, bytes key, ColumnFamilyHandle column_family = *)
 */
struct __pyx_opt_args_8aimrocks_11lib_rocksdb_3IDB_merge {
  int __pyx_n;
  bool sync;
  bool disable_wal;
  struct __pyx_obj_8aimrocks_11lib_rocksdb_ColumnFamilyHandle *column_family;
};

/* "aimrocks/lib_rocksdb.pxd":603
 *     cpdef void flush_wal(self, cpp_bool sync = *)
 *     cpdef void merge(self, bytes key, bytes value, cpp_bool sync = *, cpp_bool disable_wal = *, ColumnFamilyHandle column_family = *)
 *     cpdef void write(self, WriteBatch batch, cpp_bool sync = *, cpp_bool disable_wal = *)             # <<<<<<<<<<<<<<
 *     cpdef get(self, bytes key, ColumnFamilyHandle column_family = *)
 *     cpdef multi_get(self, keys)
 */
struct __pyx_opt_args_8aimrocks_11lib_rocksdb_3IDB_write {
  int __pyx_n;
  bool sync;
  bool disable_wal;
};

/* "aimrocks/lib_rocksdb.pxd":604
 *     cpdef void merge(self, bytes key, bytes value, cpp_bool sync = *, cpp_bool disable_wal = *, ColumnFamilyHandle column_family = *)
 *     cpdef void write(self, WriteBatch batch, cpp_bool sync = *, cpp_bool disable_wal = *)
 *     cpdef get(self, bytes key, ColumnFamilyHandle column_family = *)             # <<<<<<<<<<<<<<
 *     cpdef multi_get(self, keys)
 *     cpdef key_may_exist(self, bytes key, cpp_bool fetch = *, ColumnFamilyHandle column_family = *)
 */
struct __pyx_opt_args_8aimrocks_11lib_rocksdb_3IDB_get {
  int __pyx_n;
  struct __pyx_obj_8aimrocks_11lib_rocksdb_ColumnFamilyHandle *column_family;
};

/* "aimrocks/lib_rocksdb.pxd":606
 *     cpdef get(self, bytes key, ColumnFamilyHandle column_family = *)
 *     cpdef multi_get(self, keys)
 *     cpdef key_may_exist(self, bytes key, cpp_bool fetch = *, ColumnFamilyHandle column_family = *)             # <<<<<<<<<<<<<<
 *     cpdef Iterator iterkeys(self, ColumnFamilyHandle column_family = *)
 *     cpdef Iterator itervalues(self, ColumnFamilyHandle column_family = *)
 */
struct __pyx_opt_args_8aimrocks_11lib_rocksdb_3IDB_key_may_exist {
  int __pyx_n;
  bool fetch;
  struct __pyx_obj_8aimrocks_11lib_rocksdb_ColumnFamilyHandle *column_family;
};

/* "aimrocks/lib_rocksdb.pxd":607
 *     cpdef multi_get(self, keys)
 *     cpdef key_may_exist(self, bytes key, cpp_bool fetch = *, ColumnFamilyHandle column_family = *)
 *     cpdef Iterator iterkeys(self, ColumnFamilyHandle column_family = *)             # <<<<<<<<<<<<<<
 *     cpdef Iterator itervalues(self, ColumnFamilyHandle column_family = *)
 *     cpdef Iterator iteritems(self, ColumnFamilyHandle column_family = *)
 */
struct __pyx_opt_args_8aimrocks_11lib_rocksdb_3IDB_iterkeys {
  int __pyx_n;
  struct __pyx_obj_8aimrocks_11lib_rocksdb_ColumnFamilyHandle *column_family;
};

/* "aimrocks/lib_rocksdb.pxd":608
 *     cpdef key_may_exist(self, bytes key, cpp_bool fetch = *, ColumnFamilyHandle column_family = *)
 *     cpdef Iterator iterkeys(self, ColumnFamilyHandle column_family = *)
 *     cpdef Iterator itervalues(self, ColumnFamilyHandle column_family = *)             # <<<<<<<<<<<<<<
 *     cpdef Iterator iteritems(self, ColumnFamilyHandle column_family = *)
 * 
 */
struct __pyx_opt_args_8aimrocks_11lib_rocksdb_3IDB_itervalues {
  int __pyx_n;
  struct __pyx_obj_8aimrocks_11lib_rocksdb_ColumnFamilyHandle *column_family;
};

/* "aimrocks/lib_rocksdb.pxd":609
 *     cpdef Iterator iterkeys(self, ColumnFamilyHandle column_family = *)
 *     cpdef Iterator itervalues(self, ColumnFamilyHandle column_family = *)
 *     cpdef Iterator iteritems(self, ColumnFamilyHandle column_family = *)             # <<<<<<<<<<<<<<
 * 
 * @cython.no_gc_clear
 */
struct __pyx_opt_args_8aimrocks_11lib_rocksdb_3IDB_iteritems {
  int __pyx_n;
  struct __pyx_obj_8aimrocks_11lib_rocksdb_ColumnFamilyHandle *column_family;
};

/* "aimrocks/lib_rocksdb.pxd":627
 *     cpdef itersitems(self, column_families)
 *     cpdef snapshot(self)
 *     cpdef get_property(self, prop, ColumnFamilyHandle column_family = *)             # <<<<<<<<<<<<<<
 *     cpdef get_live_files_metadata(self)
 *     cpdef void compact_range(
 */
struct __pyx_opt_args_8aimrocks_11lib_rocksdb_2DB_get_property {
  int __pyx_n;
  struct __pyx_obj_8aimrocks_11lib_rocksdb_ColumnFamilyHandle *column_family;
};

/* "aimrocks/lib_rocksdb.pxd":629
 *     cpdef get_property(self, prop, ColumnFamilyHandle column_family = *)
 *     cpdef get_live_files_metadata(self)
 *     cpdef void compact_range(             # <<<<<<<<<<<<<<
 *         self,
 *         bytes begin = *,
 */
struct __pyx_opt_args_8aimrocks_11lib_rocksdb_2DB_compact_range {
  int __pyx_n;
  PyObject *begin;
  PyObject *end;
  bool change_level;
  int target_level;
  PyObject *bottommost_level_compaction;
  struct __pyx_obj_8aimrocks_11lib_rocksdb_ColumnFamilyHandle *column_family;
};

/* "aimrocks/lib_rocksdb.pxd":713
 *     # def  __cinit__(self, backup_dir)s
 *     # def __dealloc__(self)
 *     cpdef create_backup(self, DB db, flush_before_backup = *)             # <<<<<<<<<<<<<<
 *     cpdef restore_backup(self, backup_id, db_dir, wal_dir)
 *     cpdef restore_latest_backup(self, db_dir, wal_dir)
 */
struct __pyx_opt_args_8aimrocks_11lib_rocksdb_12BackupEngine_create_backup {
  int __pyx_n;
  PyObject *flush_before_backup;
};
struct __pyx_opt_args_3aim_7storage_9container_9Container_get;
struct __pyx_opt_args_3aim_7storage_9container_9Container_set;
struct __pyx_opt_args_3aim_7storage_9container_9Container_delete_range;
struct __pyx_opt_args_3aim_7storage_9container_9Container_items;
struct __pyx_opt_args_3aim_7storage_9container_9Container_next_key;
struct __pyx_opt_args_3aim_7storage_9container_9Container_next_value;
struct __pyx_opt_args_3aim_7storage_9container_9Container_next_key_value;
struct __pyx_opt_args_3aim_7storage_9container_9Container_next_item;
struct __pyx_opt_args_3aim_7storage_9container_9Container_prev_key;
struct __pyx_opt_args_3aim_7storage_9container_9Container_prev_value;
struct __pyx_opt_args_3aim_7storage_9container_9Container_prev_key_value;
struct __pyx_opt_args_3aim_7storage_9container_9Container_prev_item;

/* "aim/storage/container.pxd":15
 *     cpdef void close(self)
 *     cpdef void preload(self)
 *     cpdef object get(self, bytes key, object default = *)             # <<<<<<<<<<<<<<
 *     cpdef void set(self, bytes key, object value, store_batch = *)
 *     cpdef void delete_range(self, bytes begin, bytes end, store_batch = *)
 */
struct __pyx_opt_args_3aim_7storage_9container_9Container_get {
  int __pyx_n;
  PyObject *__pyx_default;
};

/* "aim/storage/container.pxd":16
 *     cpdef void preload(self)
 *     cpdef object get(self, bytes key, object default = *)
 *     cpdef void set(self, bytes key, object value, store_batch = *)             # <<<<<<<<<<<<<<
 *     cpdef void delete_range(self, bytes begin, bytes end, store_batch = *)
 * 
 */
struct __pyx_opt_args_3aim_7storage_9container_9Container_set {
  int __pyx_n;
  PyObject *store_batch;
};

/* "aim/storage/container.pxd":17
 *     cpdef object get(self, bytes key, object default = *)
 *     cpdef void set(self, bytes key, object value, store_batch = *)
 *     cpdef void delete_range(self, bytes begin, bytes end, store_batch = *)             # <<<<<<<<<<<<<<
 * 
 *     cpdef ContainerItemsIterator items(self, bytes prefix = *)
 */
struct __pyx_opt_args_3aim_7storage_9container_9Container_delete_range {
  int __pyx_n;
  PyObject *store_batch;
};

/* "aim/storage/container.pxd":19
 *     cpdef void delete_range(self, bytes begin, bytes end, store_batch = *)
 * 
 *     cpdef ContainerItemsIterator items(self, bytes prefix = *)             # <<<<<<<<<<<<<<
 * 
 *     cpdef bytes next_key(self, bytes key = *)
 */
struct __pyx_opt_args_3aim_7storage_9container_9Container_items {
  int __pyx_n;
  PyObject *prefix;
};

/* "aim/storage/container.pxd":21
 *     cpdef ContainerItemsIterator items(self, bytes prefix = *)
 * 
 *     cpdef bytes next_key(self, bytes key = *)             # <<<<<<<<<<<<<<
 *     cpdef object next_value(self, bytes key = *)
 *     cpdef tuple next_key_value(self, bytes key = *)
 */
struct __pyx_opt_args_3aim_7storage_9container_9Container_next_key {
  int __pyx_n;
  PyObject *key;
};

/* "aim/storage/container.pxd":22
 * 
 *     cpdef bytes next_key(self, bytes key = *)
 *     cpdef object next_value(self, bytes key = *)             # <<<<<<<<<<<<<<
 *     cpdef tuple next_key_value(self, bytes key = *)
 *     cpdef tuple next_item(self, bytes key = *)
 */
struct __pyx_opt_args_3aim_7storage_9container_9Container_next_value {
  int __pyx_n;
  PyObject *key;
};

/* "aim/storage/container.pxd":23
 *     cpdef bytes next_key(self, bytes key = *)
 *     cpdef object next_value(self, bytes key = *)
 *     cpdef tuple next_key_value(self, bytes key = *)             # <<<<<<<<<<<<<<
 *     cpdef tuple next_item(self, bytes key = *)
 *     cpdef bytes prev_key(self, bytes key = *)
 */
struct __pyx_opt_args_3aim_7storage_9container_9Container_next_key_value {
  int __pyx_n;
  PyObject *key;
};

/* "aim/storage/container.pxd":24
 *     cpdef object next_value(self, bytes key = *)
 *     cpdef tuple next_key_value(self, bytes key = *)
 *     cpdef tuple next_item(self, bytes key = *)             # <<<<<<<<<<<<<<
 *     cpdef bytes prev_key(self, bytes key = *)
 *     cpdef object prev_value(self, bytes key = *)
 */
struct __pyx_opt_args_3aim_7storage_9container_9Container_next_item {
  int __pyx_n;
  PyObject *key;
};

/* "aim/storage/container.pxd":25
 *     cpdef tuple next_key_value(self, bytes key = *)
 *     cpdef tuple next_item(self, bytes key = *)
 *     cpdef bytes prev_key(self, bytes key = *)             # <<<<<<<<<<<<<<
 *     cpdef object prev_value(self, bytes key = *)
 *     cpdef tuple prev_key_value(self, bytes key = *)
 */
struct __pyx_opt_args_3aim_7storage_9container_9Container_prev_key {
  int __pyx_n;
  PyObject *key;
};

/* "aim/storage/container.pxd":26
 *     cpdef tuple next_item(self, bytes key = *)
 *     cpdef bytes prev_key(self, bytes key = *)
 *     cpdef object prev_value(self, bytes key = *)             # <<<<<<<<<<<<<<
 *     cpdef tuple prev_key_value(self, bytes key = *)
 *     cpdef tuple prev_item(self, bytes key = *)
 */
struct __pyx_opt_args_3aim_7storage_9container_9Container_prev_value {
  int __pyx_n;
  PyObject *key;
};

/* "aim/storage/container.pxd":27
 *     cpdef bytes prev_key(self, bytes key = *)
 *     cpdef object prev_value(self, bytes key = *)
 *     cpdef tuple prev_key_value(self, bytes key = *)             # <<<<<<<<<<<<<<
 *     cpdef tuple prev_item(self, bytes key = *)
 */
struct __pyx_opt_args_3aim_7storage_9container_9Container_prev_key_value {
  int __pyx_n;
  PyObject *key;
};

/* "aim/storage/container.pxd":28
 *     cpdef object prev_value(self, bytes key = *)
 *     cpdef tuple prev_key_value(self, bytes key = *)
 *     cpdef tuple prev_item(self, bytes key = *)             # <<<<<<<<<<<<<<
 */
struct __pyx_opt_args_3aim_7storage_9container_9Container_prev_item {
  int __pyx_n;
  PyObject *key;
};
struct __pyx_opt_args_3aim_7storage_8treeview_8TreeView_view;
struct __pyx_opt_args_3aim_7storage_8treeview_8TreeView_make_array;
struct __pyx_opt_args_3aim_7storage_8treeview_8TreeView_collect;

/* "aim/storage/treeview.pxd":8
 *         object __weakref__
 * 
 *     cpdef view(self, object path, bint resolve = *)             # <<<<<<<<<<<<<<
 *     cpdef object make_array(self, object path = *)
 *     cpdef object collect(self, object path = *, bint strict = *, bint resolve_objects = *)
 */
struct __pyx_opt_args_3aim_7storage_8treeview_8TreeView_view {
  int __pyx_n;
  int resolve;
};

/* "aim/storage/treeview.pxd":9
 * 
 *     cpdef view(self, object path, bint resolve = *)
 *     cpdef object make_array(self, object path = *)             # <<<<<<<<<<<<<<
 *     cpdef object collect(self, object path = *, bint strict = *, bint resolve_objects = *)
 */
struct __pyx_opt_args_3aim_7storage_8treeview_8TreeView_make_array {
  int __pyx_n;
  PyObject *path;
};

/* "aim/storage/treeview.pxd":10
 *     cpdef view(self, object path, bint resolve = *)
 *     cpdef object make_array(self, object path = *)
 *     cpdef object collect(self, object path = *, bint strict = *, bint resolve_objects = *)             # <<<<<<<<<<<<<<
 */
struct __pyx_opt_args_3aim_7storage_8treeview_8TreeView_collect {
  int __pyx_n;
  PyObject *path;
  int strict;
  int resolve_objects;
};
struct __pyx_opt_args_3aim_7storage_8encoding_15encoding_native_decode_int64_big_endian;
struct __pyx_opt_args_3aim_7storage_8encoding_15encoding_native_decode_int64;
struct __pyx_opt_args_3aim_7storage_8encoding_15encoding_native_decode_double;

/* "aim/storage/encoding/encoding_native.pxd":12
 * 
 * 
 * cpdef enum:             # <<<<<<<<<<<<<<
 *     PATH_SENTINEL_CODE = 0xfe
 * 
 */
enum  {
  __pyx_e_3aim_7storage_8encoding_15encoding_native_PATH_SENTINEL_CODE = 0xfe
};

/* "aim/storage/encoding/encoding_native.pxd":22
 * cpdef bytes encode_int64(int64 value)
 * 
 * cpdef int64 decode_int64_big_endian(const unsigned char* b, int offset = *) nogil             # <<<<<<<<<<<<<<
 * 
 * cpdef int64 decode_int64(const unsigned char* buffer, int offset = *) nogil
 */
struct __pyx_opt_args_3aim_7storage_8encoding_15encoding_native_decode_int64_big_endian {
  int __pyx_n;
  int offset;
};

/* "aim/storage/encoding/encoding_native.pxd":24
 * cpdef int64 decode_int64_big_endian(const unsigned char* b, int offset = *) nogil
 * 
 * cpdef int64 decode_int64(const unsigned char* buffer, int offset = *) nogil             # <<<<<<<<<<<<<<
 * 
 * cpdef vector[pair[int64, int64]] split_path(
 */
struct __pyx_opt_args_3aim_7storage_8encoding_15encoding_native_decode_int64 {
  int __pyx_n;
  int offset;
};

/* "aim/storage/encoding/encoding_native.pxd":35
 * cpdef bytes encode_double(double value)
 * 
 * cpdef double decode_double(const unsigned char* bytes, int offset = *) nogil             # <<<<<<<<<<<<<<
 * 
 * cpdef bytes encode_utf_8_str(str value)
 */
struct __pyx_opt_args_3aim_7storage_8encoding_15encoding_native_decode_double {
  int __pyx_n;
  int offset;
};

/* "aim/storage/encoding/encoding.pxd":6
 * # We define a mapping for primitive types into a single bit
 * # The types are going to be stored in a single byte, so 7 more bits are reserved
 * cdef enum FLAGS:             # <<<<<<<<<<<<<<
 *     _NONE = 0
 *     _BOOL = 1
 */
enum __pyx_t_3aim_7storage_8encoding_8encoding_FLAGS {

  /* "aim/storage/encoding/encoding.pxd":15
 *     _ARRAY = 6
 *     _OBJECT = 7
 *     _CUSTOM_OBJECT = 8 | 7             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_e_3aim_7storage_8encoding_8encoding__NONE = 0,
  __pyx_e_3aim_7storage_8encoding_8encoding__BOOL = 1,
  __pyx_e_3aim_7storage_8encoding_8encoding__INT = 2,
  __pyx_e_3aim_7storage_8encoding_8encoding__FLOAT = 3,
  __pyx_e_3aim_7storage_8encoding_8encoding__STRING = 4,
  __pyx_e_3aim_7storage_8encoding_8encoding__BYTES = 5,
  __pyx_e_3aim_7storage_8encoding_8encoding__ARRAY = 6,
  __pyx_e_3aim_7storage_8encoding_8encoding__OBJECT = 7,
  __pyx_e_3aim_7storage_8encoding_8encoding__CUSTOM_OBJECT = (8 | 7)
};
struct __pyx_opt_args_3aim_7storage_10prefixview_10PrefixView_absolute_path;
struct __pyx_opt_args_3aim_7storage_10prefixview_10PrefixView_get;
struct __pyx_opt_args_3aim_7storage_10prefixview_10PrefixView_set;
struct __pyx_opt_args_3aim_7storage_10prefixview_10PrefixView_delete_range;
struct __pyx_opt_args_3aim_7storage_10prefixview_10PrefixView_next_item;
struct __pyx_opt_args_3aim_7storage_10prefixview_10PrefixView_prev_item;
struct __pyx_opt_args_3aim_7storage_10prefixview_10PrefixView_items;

/* "aim/storage/prefixview.pxd":15
 *     cdef public bint read_only
 * 
 *     cpdef bytes absolute_path(self, bytes path = *)             # <<<<<<<<<<<<<<
 *     cpdef ContainerTreeView tree(self)
 * 
 */
struct __pyx_opt_args_3aim_7storage_10prefixview_10PrefixView_absolute_path {
  int __pyx_n;
  PyObject *path;
};

/* "aim/storage/prefixview.py":83
 *         return self.prefix + path
 * 
 *     def get(             # <<<<<<<<<<<<<<
 *         self,
 *         key: ContainerKey,
 */
struct __pyx_opt_args_3aim_7storage_10prefixview_10PrefixView_get {
  int __pyx_n;
  PyObject *__pyx_default;
};

/* "aim/storage/prefixview.py":103
 *         return self.parent[path]
 * 
 *     def set(             # <<<<<<<<<<<<<<
 *         self,
 *         key: ContainerKey,
 */
struct __pyx_opt_args_3aim_7storage_10prefixview_10PrefixView_set {
  int __pyx_n;
  PyObject *store_batch;
};

/* "aim/storage/prefixview.py":154
 *         del self.parent[path]
 * 
 *     def delete_range(             # <<<<<<<<<<<<<<
 *         self,
 *         begin: ContainerKey,
 */
struct __pyx_opt_args_3aim_7storage_10prefixview_10PrefixView_delete_range {
  int __pyx_n;
  PyObject *store_batch;
};

/* "aim/storage/prefixview.py":166
 *                                  store_batch=store_batch)
 * 
 *     def next_item(             # <<<<<<<<<<<<<<
 *         self,
 *         key: ContainerKey = b''
 */
struct __pyx_opt_args_3aim_7storage_10prefixview_10PrefixView_next_item {
  int __pyx_n;
  PyObject *key;
};

/* "aim/storage/prefixview.py":181
 *         return keys, value
 * 
 *     def prev_item(             # <<<<<<<<<<<<<<
 *         self,
 *         key: ContainerKey = b''
 */
struct __pyx_opt_args_3aim_7storage_10prefixview_10PrefixView_prev_item {
  int __pyx_n;
  PyObject *key;
};

/* "aim/storage/prefixview.py":232
 *             p = self.absolute_path(key)
 * 
 *     def items(             # <<<<<<<<<<<<<<
 *         self,
 *         key: ContainerKey = b''
 */
struct __pyx_opt_args_3aim_7storage_10prefixview_10PrefixView_items {
  int __pyx_n;
  PyObject *key;
};

/* "aimrocks/lib_rocksdb.pxd":70
 * cdef object string_to_path(string path)
 * 
 * cdef class PyComparator(object):             # <<<<<<<<<<<<<<
 *     cdef object get_ob(self)
 * 
 */
struct __pyx_obj_8aimrocks_11lib_rocksdb_PyComparator {
  PyObject_HEAD
  struct __pyx_vtabstruct_8aimrocks_11lib_rocksdb_PyComparator *__pyx_vtab;
};


/* "aimrocks/lib_rocksdb.pxd":77
 *     cdef set_info_log(self, shared_ptr[logger.Logger] info_log)
 * 
 * cdef class PyGenericComparator(PyComparator):             # <<<<<<<<<<<<<<
 *     cdef comparator.ComparatorWrapper* comparator_ptr
 *     cdef object ob
 */
struct __pyx_obj_8aimrocks_11lib_rocksdb_PyGenericComparator {
  struct __pyx_obj_8aimrocks_11lib_rocksdb_PyComparator __pyx_base;
  py_rocks::ComparatorWrapper *comparator_ptr;
  PyObject *ob;
};


/* "aimrocks/lib_rocksdb.pxd":86
 *     cdef set_info_log(self, shared_ptr[logger.Logger] info_log)
 * 
 * cdef class PyBytewiseComparator(PyComparator):             # <<<<<<<<<<<<<<
 *     cdef const comparator.Comparator* comparator_ptr
 *     # def __cinit__(self)
 */
struct __pyx_obj_8aimrocks_11lib_rocksdb_PyBytewiseComparator {
  struct __pyx_obj_8aimrocks_11lib_rocksdb_PyComparator __pyx_base;
  rocksdb::Comparator const *comparator_ptr;
};


/* "aimrocks/lib_rocksdb.pxd":108
 * 
 * ## Here comes the stuff for the filter policy
 * cdef class PyFilterPolicy(object):             # <<<<<<<<<<<<<<
 *     cdef object get_ob(self)
 *     cdef shared_ptr[ConstFilterPolicy] get_policy(self)
 */
struct __pyx_obj_8aimrocks_11lib_rocksdb_PyFilterPolicy {
  PyObject_HEAD
  struct __pyx_vtabstruct_8aimrocks_11lib_rocksdb_PyFilterPolicy *__pyx_vtab;
};


/* "aimrocks/lib_rocksdb.pxd":113
 *     cdef set_info_log(self, shared_ptr[logger.Logger] info_log)
 * 
 * cdef class PyGenericFilterPolicy(PyFilterPolicy):             # <<<<<<<<<<<<<<
 *     cdef shared_ptr[filter_policy.FilterPolicyWrapper] policy
 *     cdef object ob
 */
struct __pyx_obj_8aimrocks_11lib_rocksdb_PyGenericFilterPolicy {
  struct __pyx_obj_8aimrocks_11lib_rocksdb_PyFilterPolicy __pyx_base;
  std::shared_ptr<py_rocks::FilterPolicyWrapper>  policy;
  PyObject *ob;
};


/* "aimrocks/lib_rocksdb.pxd":137
 *     const Slice& filt) with gil
 * 
 * cdef class PyBloomFilterPolicy(PyFilterPolicy):             # <<<<<<<<<<<<<<
 *     cdef shared_ptr[ConstFilterPolicy] policy
 *     # def __cinit__(self, int bits_per_key)
 */
struct __pyx_obj_8aimrocks_11lib_rocksdb_PyBloomFilterPolicy {
  struct __pyx_obj_8aimrocks_11lib_rocksdb_PyFilterPolicy __pyx_base;
  std::shared_ptr<__pyx_t_8aimrocks_11lib_rocksdb_ConstFilterPolicy>  policy;
};


/* "aimrocks/lib_rocksdb.pxd":152
 * 
 * ## Here comes the stuff for the merge operator
 * cdef class PyMergeOperator(object):             # <<<<<<<<<<<<<<
 *     cdef shared_ptr[merge_operator.MergeOperator] merge_op
 *     cdef object ob
 */
struct __pyx_obj_8aimrocks_11lib_rocksdb_PyMergeOperator {
  PyObject_HEAD
  struct __pyx_vtabstruct_8aimrocks_11lib_rocksdb_PyMergeOperator *__pyx_vtab;
  std::shared_ptr<rocksdb::MergeOperator>  merge_op;
  PyObject *ob;
};


/* "aimrocks/lib_rocksdb.pxd":183
 *     logger.Logger* log) with gil
 * 
 * cdef class PyCache(object):             # <<<<<<<<<<<<<<
 *     cdef shared_ptr[cache.Cache] get_cache(self)
 * 
 */
struct __pyx_obj_8aimrocks_11lib_rocksdb_PyCache {
  PyObject_HEAD
  struct __pyx_vtabstruct_8aimrocks_11lib_rocksdb_PyCache *__pyx_vtab;
};


/* "aimrocks/lib_rocksdb.pxd":186
 *     cdef shared_ptr[cache.Cache] get_cache(self)
 * 
 * cdef class PyLRUCache(PyCache):             # <<<<<<<<<<<<<<
 *     cdef shared_ptr[cache.Cache] cache_ob
 *     # def __cinit__(self, capacity, shard_bits = *)
 */
struct __pyx_obj_8aimrocks_11lib_rocksdb_PyLRUCache {
  struct __pyx_obj_8aimrocks_11lib_rocksdb_PyCache __pyx_base;
  std::shared_ptr<rocksdb::Cache>  cache_ob;
};


/* "aimrocks/lib_rocksdb.pxd":195
 * 
 * ### Here comes the stuff for SliceTransform
 * cdef class PySliceTransform(object):             # <<<<<<<<<<<<<<
 *     cdef shared_ptr[slice_transform.SliceTransform] transfomer
 *     cdef object ob
 */
struct __pyx_obj_8aimrocks_11lib_rocksdb_PySliceTransform {
  PyObject_HEAD
  struct __pyx_vtabstruct_8aimrocks_11lib_rocksdb_PySliceTransform *__pyx_vtab;
  std::shared_ptr<rocksdb::SliceTransform>  transfomer;
  PyObject *ob;
};


/* "aimrocks/lib_rocksdb.pxd":223
 * 
 * ## Here are the TableFactories
 * cdef class PyTableFactory(object):             # <<<<<<<<<<<<<<
 *     cdef shared_ptr[table_factory.TableFactory] factory
 *     cdef shared_ptr[table_factory.TableFactory] get_table_factory(self)
 */
struct __pyx_obj_8aimrocks_11lib_rocksdb_PyTableFactory {
  PyObject_HEAD
  struct __pyx_vtabstruct_8aimrocks_11lib_rocksdb_PyTableFactory *__pyx_vtab;
  std::shared_ptr<rocksdb::TableFactory>  factory;
};


/* "aimrocks/lib_rocksdb.pxd":228
 *     cdef set_info_log(self, shared_ptr[logger.Logger] info_log)
 * 
 * cdef class BlockBasedTableFactory(PyTableFactory):             # <<<<<<<<<<<<<<
 *     cdef PyFilterPolicy py_filter_policy
 *     # def __init__(self,
 */
struct __pyx_obj_8aimrocks_11lib_rocksdb_BlockBasedTableFactory {
  struct __pyx_obj_8aimrocks_11lib_rocksdb_PyTableFactory __pyx_base;
  struct __pyx_obj_8aimrocks_11lib_rocksdb_PyFilterPolicy *py_filter_policy;
};


/* "aimrocks/lib_rocksdb.pxd":244
 *     cdef set_info_log(self, shared_ptr[logger.Logger] info_log)
 * 
 * cdef class PlainTableFactory(PyTableFactory):             # <<<<<<<<<<<<<<
 *     pass
 *     # def __init__(
 */
struct __pyx_obj_8aimrocks_11lib_rocksdb_PlainTableFactory {
  struct __pyx_obj_8aimrocks_11lib_rocksdb_PyTableFactory __pyx_base;
};


/* "aimrocks/lib_rocksdb.pxd":258
 * 
 * ### Here are the MemtableFactories
 * cdef class PyMemtableFactory(object):             # <<<<<<<<<<<<<<
 *     cdef shared_ptr[memtablerep.MemTableRepFactory] factory
 *     cdef shared_ptr[memtablerep.MemTableRepFactory] get_memtable_factory(self)
 */
struct __pyx_obj_8aimrocks_11lib_rocksdb_PyMemtableFactory {
  PyObject_HEAD
  struct __pyx_vtabstruct_8aimrocks_11lib_rocksdb_PyMemtableFactory *__pyx_vtab;
  std::shared_ptr<rocksdb::MemTableRepFactory>  factory;
};


/* "aimrocks/lib_rocksdb.pxd":262
 *     cdef shared_ptr[memtablerep.MemTableRepFactory] get_memtable_factory(self)
 * 
 * cdef class SkipListMemtableFactory(PyMemtableFactory):             # <<<<<<<<<<<<<<
 *     pass
 *     # def __init__(self)
 */
struct __pyx_obj_8aimrocks_11lib_rocksdb_SkipListMemtableFactory {
  struct __pyx_obj_8aimrocks_11lib_rocksdb_PyMemtableFactory __pyx_base;
};


/* "aimrocks/lib_rocksdb.pxd":266
 *     # def __init__(self)
 * 
 * cdef class VectorMemtableFactory(PyMemtableFactory):             # <<<<<<<<<<<<<<
 *     pass
 *     # def __init__(self, count=0)
 */
struct __pyx_obj_8aimrocks_11lib_rocksdb_VectorMemtableFactory {
  struct __pyx_obj_8aimrocks_11lib_rocksdb_PyMemtableFactory __pyx_base;
};


/* "aimrocks/lib_rocksdb.pxd":270
 *     # def __init__(self, count=0)
 * 
 * cdef class HashSkipListMemtableFactory(PyMemtableFactory):             # <<<<<<<<<<<<<<
 *     pass
 *     # def __init__(
 */
struct __pyx_obj_8aimrocks_11lib_rocksdb_HashSkipListMemtableFactory {
  struct __pyx_obj_8aimrocks_11lib_rocksdb_PyMemtableFactory __pyx_base;
};


/* "aimrocks/lib_rocksdb.pxd":278
 *     #         skiplist_branching_factor=4)
 * 
 * cdef class HashLinkListMemtableFactory(PyMemtableFactory):             # <<<<<<<<<<<<<<
 *     pass
 *     # def __init__(self, bucket_count=50000)
 */
struct __pyx_obj_8aimrocks_11lib_rocksdb_HashLinkListMemtableFactory {
  struct __pyx_obj_8aimrocks_11lib_rocksdb_PyMemtableFactory __pyx_base;
};


/* "aimrocks/lib_rocksdb.pxd":284
 * 
 * 
 * cdef class CompressionType(object):             # <<<<<<<<<<<<<<
 *     pass
 *     # no_compression = u'no_compression'
 */
struct __pyx_obj_8aimrocks_11lib_rocksdb_CompressionType {
  PyObject_HEAD
};


/* "aimrocks/lib_rocksdb.pxd":297
 *     # disable_compression = u'disable_compression'
 * 
 * cdef class CompactionPri(object):             # <<<<<<<<<<<<<<
 *     pass
 *     # by_compensated_size = u'by_compensated_size'
 */
struct __pyx_obj_8aimrocks_11lib_rocksdb_CompactionPri {
  PyObject_HEAD
};


/* "aimrocks/lib_rocksdb.pxd":304
 *     # min_overlapping_ratio = u'min_overlapping_ratio'
 * 
 * cdef class _ColumnFamilyHandle:             # <<<<<<<<<<<<<<
 *     """ This is an internal class that we will weakref for safety """
 *     cdef db.ColumnFamilyHandle* handle
 */
struct __pyx_obj_8aimrocks_11lib_rocksdb__ColumnFamilyHandle {
  PyObject_HEAD
  struct __pyx_vtabstruct_8aimrocks_11lib_rocksdb__ColumnFamilyHandle *__pyx_vtab;
  rocksdb::ColumnFamilyHandle *handle;
  PyObject *__weakref__;
  PyObject *weak_handle;
};


/* "aimrocks/lib_rocksdb.pxd":320
 *     # def weakref(self)
 * 
 * cdef class ColumnFamilyHandle:             # <<<<<<<<<<<<<<
 *     """ This represents a ColumnFamilyHandle """
 *     cdef object _ref
 */
struct __pyx_obj_8aimrocks_11lib_rocksdb_ColumnFamilyHandle {
  PyObject_HEAD
  struct __pyx_vtabstruct_8aimrocks_11lib_rocksdb_ColumnFamilyHandle *__pyx_vtab;
  PyObject *_ref;
  PyObject *name;
  int id;
};


/* "aimrocks/lib_rocksdb.pxd":342
 *     # def __hash__(self)
 * 
 * cdef class ColumnFamilyOptions(object):             # <<<<<<<<<<<<<<
 *     cdef options.ColumnFamilyOptions* copts
 *     cdef PyComparator py_comparator
 */
struct __pyx_obj_8aimrocks_11lib_rocksdb_ColumnFamilyOptions {
  PyObject_HEAD
  rocksdb::ColumnFamilyOptions *copts;
  struct __pyx_obj_8aimrocks_11lib_rocksdb_PyComparator *py_comparator;
  struct __pyx_obj_8aimrocks_11lib_rocksdb_PyMergeOperator *py_merge_operator;
  struct __pyx_obj_8aimrocks_11lib_rocksdb_PySliceTransform *py_prefix_extractor;
  struct __pyx_obj_8aimrocks_11lib_rocksdb_PyTableFactory *py_table_factory;
  struct __pyx_obj_8aimrocks_11lib_rocksdb_PyMemtableFactory *py_memtable_factory;
  bool in_use;
};


/* "aimrocks/lib_rocksdb.pxd":455
 *     #     def __set__(self, value)
 * 
 * cdef class Options(ColumnFamilyOptions):             # <<<<<<<<<<<<<<
 *     cdef options.Options* opts
 *     cdef PyCache py_row_cache
 */
struct __pyx_obj_8aimrocks_11lib_rocksdb_Options {
  struct __pyx_obj_8aimrocks_11lib_rocksdb_ColumnFamilyOptions __pyx_base;
  rocksdb::Options *opts;
  struct __pyx_obj_8aimrocks_11lib_rocksdb_PyCache *py_row_cache;
};


/* "aimrocks/lib_rocksdb.pxd":572
 * # cdef class WriteBatchIterator
 * 
 * cdef class WriteBatch(object):             # <<<<<<<<<<<<<<
 *     cdef db.WriteBatch* batch
 * 
 */
struct __pyx_obj_8aimrocks_11lib_rocksdb_WriteBatch {
  PyObject_HEAD
  struct __pyx_vtabstruct_8aimrocks_11lib_rocksdb_WriteBatch *__pyx_vtab;
  rocksdb::WriteBatch *batch;
};


/* "aimrocks/lib_rocksdb.pxd":586
 *     # def __iter__(self)
 * 
 * cdef class WriteBatchIterator(object):             # <<<<<<<<<<<<<<
 *     # Need a reference to the WriteBatch.
 *     # The BatchItems are only pointers to the memory in WriteBatch.
 */
struct __pyx_obj_8aimrocks_11lib_rocksdb_WriteBatchIterator {
  PyObject_HEAD
  struct __pyx_obj_8aimrocks_11lib_rocksdb_WriteBatch *batch;
  std::vector<py_rocks::RecordItemsHandler::BatchItem>  items;
  size_t pos;
};


/* "aimrocks/lib_rocksdb.pxd":596
 *     # def __next__(self)
 * 
 * cdef class IDB(object):             # <<<<<<<<<<<<<<
 *     cpdef void put(self, bytes key, bytes value, cpp_bool sync = *, cpp_bool disable_wal = *, ColumnFamilyHandle column_family = *)
 *     cpdef void delete_single(self, bytes key, cpp_bool sync = *, cpp_bool disable_wal = *, ColumnFamilyHandle column_family = *)
 */
struct __pyx_obj_8aimrocks_11lib_rocksdb_IDB {
  PyObject_HEAD
  struct __pyx_vtabstruct_8aimrocks_11lib_rocksdb_IDB *__pyx_vtab;
};


/* "aimrocks/lib_rocksdb.pxd":612
 * 
 * @cython.no_gc_clear
 * cdef class DB(IDB):             # <<<<<<<<<<<<<<
 *     cdef Options opts
 *     cdef db.DB* db
 */
struct __pyx_obj_8aimrocks_11lib_rocksdb_DB {
  struct __pyx_obj_8aimrocks_11lib_rocksdb_IDB __pyx_base;
  struct __pyx_obj_8aimrocks_11lib_rocksdb_Options *opts;
  rocksdb::DB *db;
  PyObject *cf_handles;
  PyObject *cf_options;
};


/* "aimrocks/lib_rocksdb.pxd":655
 * 
 * @cython.no_gc_clear
 * cdef class Snapshot(object):             # <<<<<<<<<<<<<<
 *     cdef const snapshot.Snapshot* ptr
 *     cdef DB db
 */
struct __pyx_obj_8aimrocks_11lib_rocksdb_Snapshot {
  PyObject_HEAD
  rocksdb::Snapshot const *ptr;
  struct __pyx_obj_8aimrocks_11lib_rocksdb_DB *db;
};


/* "aimrocks/lib_rocksdb.pxd":663
 * 
 * 
 * cdef class Iterator:             # <<<<<<<<<<<<<<
 *     cdef object _current_value
 * 
 */
struct __pyx_obj_8aimrocks_11lib_rocksdb_Iterator {
  PyObject_HEAD
  struct __pyx_vtabstruct_8aimrocks_11lib_rocksdb_Iterator *__pyx_vtab;
  PyObject *_current_value;
};


/* "aimrocks/lib_rocksdb.pxd":671
 * 
 * 
 * cdef class BaseIterator(Iterator):             # <<<<<<<<<<<<<<
 *     cdef iterator.Iterator* ptr
 *     cdef DB db
 */
struct __pyx_obj_8aimrocks_11lib_rocksdb_BaseIterator {
  struct __pyx_obj_8aimrocks_11lib_rocksdb_Iterator __pyx_base;
  rocksdb::Iterator *ptr;
  struct __pyx_obj_8aimrocks_11lib_rocksdb_DB *db;
  struct __pyx_obj_8aimrocks_11lib_rocksdb_ColumnFamilyHandle *handle;
};


/* "aimrocks/lib_rocksdb.pxd":687
 *     cdef object get_ob(self)
 * 
 * cdef class KeysIterator(BaseIterator):             # <<<<<<<<<<<<<<
 *     cdef object get_ob(self)
 * 
 */
struct __pyx_obj_8aimrocks_11lib_rocksdb_KeysIterator {
  struct __pyx_obj_8aimrocks_11lib_rocksdb_BaseIterator __pyx_base;
};


/* "aimrocks/lib_rocksdb.pxd":690
 *     cdef object get_ob(self)
 * 
 * cdef class ValuesIterator(BaseIterator):             # <<<<<<<<<<<<<<
 *     cdef object get_ob(self)
 * 
 */
struct __pyx_obj_8aimrocks_11lib_rocksdb_ValuesIterator {
  struct __pyx_obj_8aimrocks_11lib_rocksdb_BaseIterator __pyx_base;
};


/* "aimrocks/lib_rocksdb.pxd":693
 *     cdef object get_ob(self)
 * 
 * cdef class ItemsIterator(BaseIterator):             # <<<<<<<<<<<<<<
 *     cdef object get_ob(self)
 * 
 */
struct __pyx_obj_8aimrocks_11lib_rocksdb_ItemsIterator {
  struct __pyx_obj_8aimrocks_11lib_rocksdb_BaseIterator __pyx_base;
};


/* "aimrocks/lib_rocksdb.pxd":696
 *     cdef object get_ob(self)
 * 
 * cdef class ReversedIterator(Iterator):             # <<<<<<<<<<<<<<
 *     cdef BaseIterator it
 * 
 */
struct __pyx_obj_8aimrocks_11lib_rocksdb_ReversedIterator {
  struct __pyx_obj_8aimrocks_11lib_rocksdb_Iterator __pyx_base;
  struct __pyx_obj_8aimrocks_11lib_rocksdb_BaseIterator *it;
};


/* "aimrocks/lib_rocksdb.pxd":708
 *     # def __next__(self)
 * 
 * cdef class BackupEngine(object):             # <<<<<<<<<<<<<<
 *     cdef backup.BackupEngine* engine
 * 
 */
struct __pyx_obj_8aimrocks_11lib_rocksdb_BackupEngine {
  PyObject_HEAD
  struct __pyx_vtabstruct_8aimrocks_11lib_rocksdb_BackupEngine *__pyx_vtab;
  rocksdb::BackupEngine *engine;
};


/* "utils.pxd":6
 * cimport aimrocks.lib_rocksdb as interfaces
 * 
 * cdef class ArrayFlagType:             # <<<<<<<<<<<<<<
 *     pass
 * 
 */
struct __pyx_obj_3aim_7storage_5utils_ArrayFlagType {
  PyObject_HEAD
};


/* "utils.pxd":9
 *     pass
 * 
 * cdef class ObjectFlagType:             # <<<<<<<<<<<<<<
 *     pass
 * 
 */
struct __pyx_obj_3aim_7storage_5utils_ObjectFlagType {
  PyObject_HEAD
};


/* "utils.pxd":12
 *     pass
 * 
 * cdef class CustomObjectFlagType:             # <<<<<<<<<<<<<<
 *     cdef:
 *         public str aim_name
 */
struct __pyx_obj_3aim_7storage_5utils_CustomObjectFlagType {
  PyObject_HEAD
  PyObject *aim_name;
};


/* "utils.pxd":16
 *         public str aim_name
 * 
 * cdef class BLOB:             # <<<<<<<<<<<<<<
 *     cdef:
 *         object data
 */
struct __pyx_obj_3aim_7storage_5utils_BLOB {
  PyObject_HEAD
  struct __pyx_vtabstruct_3aim_7storage_5utils_BLOB *__pyx_vtab;
  PyObject *data;
  PyObject *loader_fn;
};


/* "aim/storage/container.pxd":7
 * from aim.storage.utils cimport interfaces
 * 
 * cdef class ContainerItemsIterator(interfaces.Iterator):             # <<<<<<<<<<<<<<
 *     pass
 * 
 */
struct __pyx_obj_3aim_7storage_9container_ContainerItemsIterator {
  struct __pyx_obj_8aimrocks_11lib_rocksdb_Iterator __pyx_base;
};


/* "aim/storage/container.pxd":10
 *     pass
 * 
 * cdef class Container:             # <<<<<<<<<<<<<<
 *     cdef __weakref__
 * 
 */
struct __pyx_obj_3aim_7storage_9container_Container {
  PyObject_HEAD
  struct __pyx_vtabstruct_3aim_7storage_9container_Container *__pyx_vtab;
  PyObject *__weakref__;
};


/* "aim/storage/treeview.pxd":4
 * # cython: language_level = 3
 * 
 * cdef class TreeView:             # <<<<<<<<<<<<<<
 *     cdef:
 *         object __weakref__
 */
struct __pyx_obj_3aim_7storage_8treeview_TreeView {
  PyObject_HEAD
  struct __pyx_vtabstruct_3aim_7storage_8treeview_TreeView *__pyx_vtab;
  PyObject *__weakref__;
};


/* "aim/storage/containertreeview.pxd":10
 * from aim.storage.container cimport Container
 * 
 * cdef class ContainerTreeView(TreeView):             # <<<<<<<<<<<<<<
 *     cdef public Container container
 */
struct __pyx_obj_3aim_7storage_17containertreeview_ContainerTreeView {
  struct __pyx_obj_3aim_7storage_8treeview_TreeView __pyx_base;
  struct __pyx_obj_3aim_7storage_9container_Container *container;
};


/* "aim/storage/prefixview.pxd":9
 * from aim.storage.containertreeview cimport ContainerTreeView
 * 
 * cdef class PrefixView(Container):             # <<<<<<<<<<<<<<
 *     cdef public bytes prefix
 *     # TODO cdef public Container parent after all container bindings are ready
 */
struct __pyx_obj_3aim_7storage_10prefixview_PrefixView {
  struct __pyx_obj_3aim_7storage_9container_Container __pyx_base;
  PyObject *prefix;
  PyObject *parent;
  int read_only;
};


/* "aim/storage/prefixview.pxd":18
 *     cpdef ContainerTreeView tree(self)
 * 
 * cdef class PrefixViewItemsIterator(ContainerItemsIterator):             # <<<<<<<<<<<<<<
 *     cdef PrefixView prefix_view
 *     cdef bytes path
 */
struct __pyx_obj_3aim_7storage_10prefixview_PrefixViewItemsIterator {
  struct __pyx_obj_3aim_7storage_9container_ContainerItemsIterator __pyx_base;
  struct __pyx_obj_3aim_7storage_10prefixview_PrefixView *prefix_view;
  PyObject *path;
  int prefix_len;
  struct __pyx_obj_3aim_7storage_9container_ContainerItemsIterator *it;
};


/* "aim/storage/prefixview.py":196
 *         return keys, value
 * 
 *     def walk(             # <<<<<<<<<<<<<<
 *         self,
 *         key: ContainerKey = b''
 */
struct __pyx_obj_3aim_7storage_10prefixview___pyx_scope_struct__walk {
  PyObject_HEAD
  PyObject *__pyx_v__path;
  PyObject *__pyx_v__prefix;
  PyObject *__pyx_v_key;
  PyObject *__pyx_v_next_key;
  PyObject *__pyx_v_p;
  PyObject *__pyx_v_path;
  struct __pyx_obj_3aim_7storage_10prefixview_PrefixView *__pyx_v_self;
  PyObject *__pyx_v_walker;
};



/* "aimrocks/lib_rocksdb.pxd":70
 * cdef object string_to_path(string path)
 * 
 * cdef class PyComparator(object):             # <<<<<<<<<<<<<<
 *     cdef object get_ob(self)
 * 
 */

struct __pyx_vtabstruct_8aimrocks_11lib_rocksdb_PyComparator {
  PyObject *(*get_ob)(struct __pyx_obj_8aimrocks_11lib_rocksdb_PyComparator *);
  rocksdb::Comparator const *(*get_comparator)(struct __pyx_obj_8aimrocks_11lib_rocksdb_PyComparator *);
  PyObject *(*set_info_log)(struct __pyx_obj_8aimrocks_11lib_rocksdb_PyComparator *, std::shared_ptr<rocksdb::Logger> );
};
static struct __pyx_vtabstruct_8aimrocks_11lib_rocksdb_PyComparator *__pyx_vtabptr_8aimrocks_11lib_rocksdb_PyComparator;


/* "aimrocks/lib_rocksdb.pxd":77
 *     cdef set_info_log(self, shared_ptr[logger.Logger] info_log)
 * 
 * cdef class PyGenericComparator(PyComparator):             # <<<<<<<<<<<<<<
 *     cdef comparator.ComparatorWrapper* comparator_ptr
 *     cdef object ob
 */

struct __pyx_vtabstruct_8aimrocks_11lib_rocksdb_PyGenericComparator {
  struct __pyx_vtabstruct_8aimrocks_11lib_rocksdb_PyComparator __pyx_base;
};
static struct __pyx_vtabstruct_8aimrocks_11lib_rocksdb_PyGenericComparator *__pyx_vtabptr_8aimrocks_11lib_rocksdb_PyGenericComparator;


/* "aimrocks/lib_rocksdb.pxd":86
 *     cdef set_info_log(self, shared_ptr[logger.Logger] info_log)
 * 
 * cdef class PyBytewiseComparator(PyComparator):             # <<<<<<<<<<<<<<
 *     cdef const comparator.Comparator* comparator_ptr
 *     # def __cinit__(self)
 */

struct __pyx_vtabstruct_8aimrocks_11lib_rocksdb_PyBytewiseComparator {
  struct __pyx_vtabstruct_8aimrocks_11lib_rocksdb_PyComparator __pyx_base;
  PyObject *(*name)(struct __pyx_obj_8aimrocks_11lib_rocksdb_PyBytewiseComparator *, int __pyx_skip_dispatch);
  PyObject *(*compare)(struct __pyx_obj_8aimrocks_11lib_rocksdb_PyBytewiseComparator *, PyObject *, PyObject *, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_8aimrocks_11lib_rocksdb_PyBytewiseComparator *__pyx_vtabptr_8aimrocks_11lib_rocksdb_PyBytewiseComparator;


/* "aimrocks/lib_rocksdb.pxd":108
 * 
 * ## Here comes the stuff for the filter policy
 * cdef class PyFilterPolicy(object):             # <<<<<<<<<<<<<<
 *     cdef object get_ob(self)
 *     cdef shared_ptr[ConstFilterPolicy] get_policy(self)
 */

struct __pyx_vtabstruct_8aimrocks_11lib_rocksdb_PyFilterPolicy {
  PyObject *(*get_ob)(struct __pyx_obj_8aimrocks_11lib_rocksdb_PyFilterPolicy *);
  std::shared_ptr<__pyx_t_8aimrocks_11lib_rocksdb_ConstFilterPolicy>  (*get_policy)(struct __pyx_obj_8aimrocks_11lib_rocksdb_PyFilterPolicy *);
  PyObject *(*set_info_log)(struct __pyx_obj_8aimrocks_11lib_rocksdb_PyFilterPolicy *, std::shared_ptr<rocksdb::Logger> );
};
static struct __pyx_vtabstruct_8aimrocks_11lib_rocksdb_PyFilterPolicy *__pyx_vtabptr_8aimrocks_11lib_rocksdb_PyFilterPolicy;


/* "aimrocks/lib_rocksdb.pxd":113
 *     cdef set_info_log(self, shared_ptr[logger.Logger] info_log)
 * 
 * cdef class PyGenericFilterPolicy(PyFilterPolicy):             # <<<<<<<<<<<<<<
 *     cdef shared_ptr[filter_policy.FilterPolicyWrapper] policy
 *     cdef object ob
 */

struct __pyx_vtabstruct_8aimrocks_11lib_rocksdb_PyGenericFilterPolicy {
  struct __pyx_vtabstruct_8aimrocks_11lib_rocksdb_PyFilterPolicy __pyx_base;
};
static struct __pyx_vtabstruct_8aimrocks_11lib_rocksdb_PyGenericFilterPolicy *__pyx_vtabptr_8aimrocks_11lib_rocksdb_PyGenericFilterPolicy;


/* "aimrocks/lib_rocksdb.pxd":137
 *     const Slice& filt) with gil
 * 
 * cdef class PyBloomFilterPolicy(PyFilterPolicy):             # <<<<<<<<<<<<<<
 *     cdef shared_ptr[ConstFilterPolicy] policy
 *     # def __cinit__(self, int bits_per_key)
 */

struct __pyx_vtabstruct_8aimrocks_11lib_rocksdb_PyBloomFilterPolicy {
  struct __pyx_vtabstruct_8aimrocks_11lib_rocksdb_PyFilterPolicy __pyx_base;
  PyObject *(*name)(struct __pyx_obj_8aimrocks_11lib_rocksdb_PyBloomFilterPolicy *, int __pyx_skip_dispatch);
  PyObject *(*create_filter)(struct __pyx_obj_8aimrocks_11lib_rocksdb_PyBloomFilterPolicy *, PyObject *, int __pyx_skip_dispatch);
  PyObject *(*key_may_match)(struct __pyx_obj_8aimrocks_11lib_rocksdb_PyBloomFilterPolicy *, PyObject *, PyObject *, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_8aimrocks_11lib_rocksdb_PyBloomFilterPolicy *__pyx_vtabptr_8aimrocks_11lib_rocksdb_PyBloomFilterPolicy;


/* "aimrocks/lib_rocksdb.pxd":152
 * 
 * ## Here comes the stuff for the merge operator
 * cdef class PyMergeOperator(object):             # <<<<<<<<<<<<<<
 *     cdef shared_ptr[merge_operator.MergeOperator] merge_op
 *     cdef object ob
 */

struct __pyx_vtabstruct_8aimrocks_11lib_rocksdb_PyMergeOperator {
  PyObject *(*get_ob)(struct __pyx_obj_8aimrocks_11lib_rocksdb_PyMergeOperator *);
  std::shared_ptr<rocksdb::MergeOperator>  (*get_operator)(struct __pyx_obj_8aimrocks_11lib_rocksdb_PyMergeOperator *);
};
static struct __pyx_vtabstruct_8aimrocks_11lib_rocksdb_PyMergeOperator *__pyx_vtabptr_8aimrocks_11lib_rocksdb_PyMergeOperator;


/* "aimrocks/lib_rocksdb.pxd":183
 *     logger.Logger* log) with gil
 * 
 * cdef class PyCache(object):             # <<<<<<<<<<<<<<
 *     cdef shared_ptr[cache.Cache] get_cache(self)
 * 
 */

struct __pyx_vtabstruct_8aimrocks_11lib_rocksdb_PyCache {
  std::shared_ptr<rocksdb::Cache>  (*get_cache)(struct __pyx_obj_8aimrocks_11lib_rocksdb_PyCache *);
};
static struct __pyx_vtabstruct_8aimrocks_11lib_rocksdb_PyCache *__pyx_vtabptr_8aimrocks_11lib_rocksdb_PyCache;


/* "aimrocks/lib_rocksdb.pxd":186
 *     cdef shared_ptr[cache.Cache] get_cache(self)
 * 
 * cdef class PyLRUCache(PyCache):             # <<<<<<<<<<<<<<
 *     cdef shared_ptr[cache.Cache] cache_ob
 *     # def __cinit__(self, capacity, shard_bits = *)
 */

struct __pyx_vtabstruct_8aimrocks_11lib_rocksdb_PyLRUCache {
  struct __pyx_vtabstruct_8aimrocks_11lib_rocksdb_PyCache __pyx_base;
};
static struct __pyx_vtabstruct_8aimrocks_11lib_rocksdb_PyLRUCache *__pyx_vtabptr_8aimrocks_11lib_rocksdb_PyLRUCache;


/* "aimrocks/lib_rocksdb.pxd":195
 * 
 * ### Here comes the stuff for SliceTransform
 * cdef class PySliceTransform(object):             # <<<<<<<<<<<<<<
 *     cdef shared_ptr[slice_transform.SliceTransform] transfomer
 *     cdef object ob
 */

struct __pyx_vtabstruct_8aimrocks_11lib_rocksdb_PySliceTransform {
  PyObject *(*get_ob)(struct __pyx_obj_8aimrocks_11lib_rocksdb_PySliceTransform *);
  std::shared_ptr<rocksdb::SliceTransform>  (*get_transformer)(struct __pyx_obj_8aimrocks_11lib_rocksdb_PySliceTransform *);
  PyObject *(*set_info_log)(struct __pyx_obj_8aimrocks_11lib_rocksdb_PySliceTransform *, std::shared_ptr<rocksdb::Logger> );
};
static struct __pyx_vtabstruct_8aimrocks_11lib_rocksdb_PySliceTransform *__pyx_vtabptr_8aimrocks_11lib_rocksdb_PySliceTransform;


/* "aimrocks/lib_rocksdb.pxd":223
 * 
 * ## Here are the TableFactories
 * cdef class PyTableFactory(object):             # <<<<<<<<<<<<<<
 *     cdef shared_ptr[table_factory.TableFactory] factory
 *     cdef shared_ptr[table_factory.TableFactory] get_table_factory(self)
 */

struct __pyx_vtabstruct_8aimrocks_11lib_rocksdb_PyTableFactory {
  std::shared_ptr<rocksdb::TableFactory>  (*get_table_factory)(struct __pyx_obj_8aimrocks_11lib_rocksdb_PyTableFactory *);
  PyObject *(*set_info_log)(struct __pyx_obj_8aimrocks_11lib_rocksdb_PyTableFactory *, std::shared_ptr<rocksdb::Logger> );
};
static struct __pyx_vtabstruct_8aimrocks_11lib_rocksdb_PyTableFactory *__pyx_vtabptr_8aimrocks_11lib_rocksdb_PyTableFactory;


/* "aimrocks/lib_rocksdb.pxd":228
 *     cdef set_info_log(self, shared_ptr[logger.Logger] info_log)
 * 
 * cdef class BlockBasedTableFactory(PyTableFactory):             # <<<<<<<<<<<<<<
 *     cdef PyFilterPolicy py_filter_policy
 *     # def __init__(self,
 */

struct __pyx_vtabstruct_8aimrocks_11lib_rocksdb_BlockBasedTableFactory {
  struct __pyx_vtabstruct_8aimrocks_11lib_rocksdb_PyTableFactory __pyx_base;
};
static struct __pyx_vtabstruct_8aimrocks_11lib_rocksdb_BlockBasedTableFactory *__pyx_vtabptr_8aimrocks_11lib_rocksdb_BlockBasedTableFactory;


/* "aimrocks/lib_rocksdb.pxd":244
 *     cdef set_info_log(self, shared_ptr[logger.Logger] info_log)
 * 
 * cdef class PlainTableFactory(PyTableFactory):             # <<<<<<<<<<<<<<
 *     pass
 *     # def __init__(
 */

struct __pyx_vtabstruct_8aimrocks_11lib_rocksdb_PlainTableFactory {
  struct __pyx_vtabstruct_8aimrocks_11lib_rocksdb_PyTableFactory __pyx_base;
};
static struct __pyx_vtabstruct_8aimrocks_11lib_rocksdb_PlainTableFactory *__pyx_vtabptr_8aimrocks_11lib_rocksdb_PlainTableFactory;


/* "aimrocks/lib_rocksdb.pxd":258
 * 
 * ### Here are the MemtableFactories
 * cdef class PyMemtableFactory(object):             # <<<<<<<<<<<<<<
 *     cdef shared_ptr[memtablerep.MemTableRepFactory] factory
 *     cdef shared_ptr[memtablerep.MemTableRepFactory] get_memtable_factory(self)
 */

struct __pyx_vtabstruct_8aimrocks_11lib_rocksdb_PyMemtableFactory {
  std::shared_ptr<rocksdb::MemTableRepFactory>  (*get_memtable_factory)(struct __pyx_obj_8aimrocks_11lib_rocksdb_PyMemtableFactory *);
};
static struct __pyx_vtabstruct_8aimrocks_11lib_rocksdb_PyMemtableFactory *__pyx_vtabptr_8aimrocks_11lib_rocksdb_PyMemtableFactory;


/* "aimrocks/lib_rocksdb.pxd":262
 *     cdef shared_ptr[memtablerep.MemTableRepFactory] get_memtable_factory(self)
 * 
 * cdef class SkipListMemtableFactory(PyMemtableFactory):             # <<<<<<<<<<<<<<
 *     pass
 *     # def __init__(self)
 */

struct __pyx_vtabstruct_8aimrocks_11lib_rocksdb_SkipListMemtableFactory {
  struct __pyx_vtabstruct_8aimrocks_11lib_rocksdb_PyMemtableFactory __pyx_base;
};
static struct __pyx_vtabstruct_8aimrocks_11lib_rocksdb_SkipListMemtableFactory *__pyx_vtabptr_8aimrocks_11lib_rocksdb_SkipListMemtableFactory;


/* "aimrocks/lib_rocksdb.pxd":266
 *     # def __init__(self)
 * 
 * cdef class VectorMemtableFactory(PyMemtableFactory):             # <<<<<<<<<<<<<<
 *     pass
 *     # def __init__(self, count=0)
 */

struct __pyx_vtabstruct_8aimrocks_11lib_rocksdb_VectorMemtableFactory {
  struct __pyx_vtabstruct_8aimrocks_11lib_rocksdb_PyMemtableFactory __pyx_base;
};
static struct __pyx_vtabstruct_8aimrocks_11lib_rocksdb_VectorMemtableFactory *__pyx_vtabptr_8aimrocks_11lib_rocksdb_VectorMemtableFactory;


/* "aimrocks/lib_rocksdb.pxd":270
 *     # def __init__(self, count=0)
 * 
 * cdef class HashSkipListMemtableFactory(PyMemtableFactory):             # <<<<<<<<<<<<<<
 *     pass
 *     # def __init__(
 */

struct __pyx_vtabstruct_8aimrocks_11lib_rocksdb_HashSkipListMemtableFactory {
  struct __pyx_vtabstruct_8aimrocks_11lib_rocksdb_PyMemtableFactory __pyx_base;
};
static struct __pyx_vtabstruct_8aimrocks_11lib_rocksdb_HashSkipListMemtableFactory *__pyx_vtabptr_8aimrocks_11lib_rocksdb_HashSkipListMemtableFactory;


/* "aimrocks/lib_rocksdb.pxd":278
 *     #         skiplist_branching_factor=4)
 * 
 * cdef class HashLinkListMemtableFactory(PyMemtableFactory):             # <<<<<<<<<<<<<<
 *     pass
 *     # def __init__(self, bucket_count=50000)
 */

struct __pyx_vtabstruct_8aimrocks_11lib_rocksdb_HashLinkListMemtableFactory {
  struct __pyx_vtabstruct_8aimrocks_11lib_rocksdb_PyMemtableFactory __pyx_base;
};
static struct __pyx_vtabstruct_8aimrocks_11lib_rocksdb_HashLinkListMemtableFactory *__pyx_vtabptr_8aimrocks_11lib_rocksdb_HashLinkListMemtableFactory;


/* "aimrocks/lib_rocksdb.pxd":304
 *     # min_overlapping_ratio = u'min_overlapping_ratio'
 * 
 * cdef class _ColumnFamilyHandle:             # <<<<<<<<<<<<<<
 *     """ This is an internal class that we will weakref for safety """
 *     cdef db.ColumnFamilyHandle* handle
 */

struct __pyx_vtabstruct_8aimrocks_11lib_rocksdb__ColumnFamilyHandle {
  PyObject *(*from_handle_ptr)(rocksdb::ColumnFamilyHandle *);
};
static struct __pyx_vtabstruct_8aimrocks_11lib_rocksdb__ColumnFamilyHandle *__pyx_vtabptr_8aimrocks_11lib_rocksdb__ColumnFamilyHandle;


/* "aimrocks/lib_rocksdb.pxd":320
 *     # def weakref(self)
 * 
 * cdef class ColumnFamilyHandle:             # <<<<<<<<<<<<<<
 *     """ This represents a ColumnFamilyHandle """
 *     cdef object _ref
 */

struct __pyx_vtabstruct_8aimrocks_11lib_rocksdb_ColumnFamilyHandle {
  PyObject *(*from_wrapper)(struct __pyx_obj_8aimrocks_11lib_rocksdb__ColumnFamilyHandle *);
  rocksdb::ColumnFamilyHandle *(*get_handle)(struct __pyx_obj_8aimrocks_11lib_rocksdb_ColumnFamilyHandle *);
};
static struct __pyx_vtabstruct_8aimrocks_11lib_rocksdb_ColumnFamilyHandle *__pyx_vtabptr_8aimrocks_11lib_rocksdb_ColumnFamilyHandle;


/* "aimrocks/lib_rocksdb.pxd":572
 * # cdef class WriteBatchIterator
 * 
 * cdef class WriteBatch(object):             # <<<<<<<<<<<<<<
 *     cdef db.WriteBatch* batch
 * 
 */

struct __pyx_vtabstruct_8aimrocks_11lib_rocksdb_WriteBatch {
  void (*put)(struct __pyx_obj_8aimrocks_11lib_rocksdb_WriteBatch *, PyObject *, PyObject *, int __pyx_skip_dispatch, struct __pyx_opt_args_8aimrocks_11lib_rocksdb_10WriteBatch_put *__pyx_optional_args);
  void (*merge)(struct __pyx_obj_8aimrocks_11lib_rocksdb_WriteBatch *, PyObject *, PyObject *, int __pyx_skip_dispatch, struct __pyx_opt_args_8aimrocks_11lib_rocksdb_10WriteBatch_merge *__pyx_optional_args);
  void (*delete_single)(struct __pyx_obj_8aimrocks_11lib_rocksdb_WriteBatch *, PyObject *, int __pyx_skip_dispatch, struct __pyx_opt_args_8aimrocks_11lib_rocksdb_10WriteBatch_delete_single *__pyx_optional_args);
  void (*delete_range)(struct __pyx_obj_8aimrocks_11lib_rocksdb_WriteBatch *, PyObject *, PyObject *, int __pyx_skip_dispatch, struct __pyx_opt_args_8aimrocks_11lib_rocksdb_10WriteBatch_delete_range *__pyx_optional_args);
  void (*clear)(struct __pyx_obj_8aimrocks_11lib_rocksdb_WriteBatch *, int __pyx_skip_dispatch);
  PyObject *(*data)(struct __pyx_obj_8aimrocks_11lib_rocksdb_WriteBatch *, int __pyx_skip_dispatch);
  int (*count)(struct __pyx_obj_8aimrocks_11lib_rocksdb_WriteBatch *, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_8aimrocks_11lib_rocksdb_WriteBatch *__pyx_vtabptr_8aimrocks_11lib_rocksdb_WriteBatch;


/* "aimrocks/lib_rocksdb.pxd":596
 *     # def __next__(self)
 * 
 * cdef class IDB(object):             # <<<<<<<<<<<<<<
 *     cpdef void put(self, bytes key, bytes value, cpp_bool sync = *, cpp_bool disable_wal = *, ColumnFamilyHandle column_family = *)
 *     cpdef void delete_single(self, bytes key, cpp_bool sync = *, cpp_bool disable_wal = *, ColumnFamilyHandle column_family = *)
 */

struct __pyx_vtabstruct_8aimrocks_11lib_rocksdb_IDB {
  void (*put)(struct __pyx_obj_8aimrocks_11lib_rocksdb_IDB *, PyObject *, PyObject *, int __pyx_skip_dispatch, struct __pyx_opt_args_8aimrocks_11lib_rocksdb_3IDB_put *__pyx_optional_args);
  void (*delete_single)(struct __pyx_obj_8aimrocks_11lib_rocksdb_IDB *, PyObject *, int __pyx_skip_dispatch, struct __pyx_opt_args_8aimrocks_11lib_rocksdb_3IDB_delete_single *__pyx_optional_args);
  void (*delete_range)(struct __pyx_obj_8aimrocks_11lib_rocksdb_IDB *, PyObject *, PyObject *, int __pyx_skip_dispatch, struct __pyx_opt_args_8aimrocks_11lib_rocksdb_3IDB_delete_range *__pyx_optional_args);
  void (*flush)(struct __pyx_obj_8aimrocks_11lib_rocksdb_IDB *, int __pyx_skip_dispatch);
  void (*flush_wal)(struct __pyx_obj_8aimrocks_11lib_rocksdb_IDB *, int __pyx_skip_dispatch, struct __pyx_opt_args_8aimrocks_11lib_rocksdb_3IDB_flush_wal *__pyx_optional_args);
  void (*merge)(struct __pyx_obj_8aimrocks_11lib_rocksdb_IDB *, PyObject *, PyObject *, int __pyx_skip_dispatch, struct __pyx_opt_args_8aimrocks_11lib_rocksdb_3IDB_merge *__pyx_optional_args);
  void (*write)(struct __pyx_obj_8aimrocks_11lib_rocksdb_IDB *, struct __pyx_obj_8aimrocks_11lib_rocksdb_WriteBatch *, int __pyx_skip_dispatch, struct __pyx_opt_args_8aimrocks_11lib_rocksdb_3IDB_write *__pyx_optional_args);
  PyObject *(*get)(struct __pyx_obj_8aimrocks_11lib_rocksdb_IDB *, PyObject *, int __pyx_skip_dispatch, struct __pyx_opt_args_8aimrocks_11lib_rocksdb_3IDB_get *__pyx_optional_args);
  PyObject *(*multi_get)(struct __pyx_obj_8aimrocks_11lib_rocksdb_IDB *, PyObject *, int __pyx_skip_dispatch);
  PyObject *(*key_may_exist)(struct __pyx_obj_8aimrocks_11lib_rocksdb_IDB *, PyObject *, int __pyx_skip_dispatch, struct __pyx_opt_args_8aimrocks_11lib_rocksdb_3IDB_key_may_exist *__pyx_optional_args);
  struct __pyx_obj_8aimrocks_11lib_rocksdb_Iterator *(*iterkeys)(struct __pyx_obj_8aimrocks_11lib_rocksdb_IDB *, int __pyx_skip_dispatch, struct __pyx_opt_args_8aimrocks_11lib_rocksdb_3IDB_iterkeys *__pyx_optional_args);
  struct __pyx_obj_8aimrocks_11lib_rocksdb_Iterator *(*itervalues)(struct __pyx_obj_8aimrocks_11lib_rocksdb_IDB *, int __pyx_skip_dispatch, struct __pyx_opt_args_8aimrocks_11lib_rocksdb_3IDB_itervalues *__pyx_optional_args);
  struct __pyx_obj_8aimrocks_11lib_rocksdb_Iterator *(*iteritems)(struct __pyx_obj_8aimrocks_11lib_rocksdb_IDB *, int __pyx_skip_dispatch, struct __pyx_opt_args_8aimrocks_11lib_rocksdb_3IDB_iteritems *__pyx_optional_args);
};
static struct __pyx_vtabstruct_8aimrocks_11lib_rocksdb_IDB *__pyx_vtabptr_8aimrocks_11lib_rocksdb_IDB;


/* "aimrocks/lib_rocksdb.pxd":612
 * 
 * @cython.no_gc_clear
 * cdef class DB(IDB):             # <<<<<<<<<<<<<<
 *     cdef Options opts
 *     cdef db.DB* db
 */

struct __pyx_vtabstruct_8aimrocks_11lib_rocksdb_DB {
  struct __pyx_vtabstruct_8aimrocks_11lib_rocksdb_IDB __pyx_base;
  void (*close)(struct __pyx_obj_8aimrocks_11lib_rocksdb_DB *, int __pyx_skip_dispatch);
  PyObject *(*get_column_family)(struct __pyx_obj_8aimrocks_11lib_rocksdb_DB *, PyObject *, int __pyx_skip_dispatch);
  PyObject *(*iterskeys)(struct __pyx_obj_8aimrocks_11lib_rocksdb_DB *, PyObject *, int __pyx_skip_dispatch);
  PyObject *(*itersvalues)(struct __pyx_obj_8aimrocks_11lib_rocksdb_DB *, PyObject *, int __pyx_skip_dispatch);
  PyObject *(*itersitems)(struct __pyx_obj_8aimrocks_11lib_rocksdb_DB *, PyObject *, int __pyx_skip_dispatch);
  PyObject *(*snapshot)(struct __pyx_obj_8aimrocks_11lib_rocksdb_DB *, int __pyx_skip_dispatch);
  PyObject *(*get_property)(struct __pyx_obj_8aimrocks_11lib_rocksdb_DB *, PyObject *, int __pyx_skip_dispatch, struct __pyx_opt_args_8aimrocks_11lib_rocksdb_2DB_get_property *__pyx_optional_args);
  PyObject *(*get_live_files_metadata)(struct __pyx_obj_8aimrocks_11lib_rocksdb_DB *, int __pyx_skip_dispatch);
  void (*compact_range)(struct __pyx_obj_8aimrocks_11lib_rocksdb_DB *, int __pyx_skip_dispatch, struct __pyx_opt_args_8aimrocks_11lib_rocksdb_2DB_compact_range *__pyx_optional_args);
  rocksdb::ReadOptions (*build_read_opts)(struct __pyx_obj_8aimrocks_11lib_rocksdb_DB *, PyObject *);
  PyObject *(*create_column_family)(struct __pyx_obj_8aimrocks_11lib_rocksdb_DB *, PyObject *, struct __pyx_obj_8aimrocks_11lib_rocksdb_ColumnFamilyOptions *, int __pyx_skip_dispatch);
  void (*drop_column_family)(struct __pyx_obj_8aimrocks_11lib_rocksdb_DB *, struct __pyx_obj_8aimrocks_11lib_rocksdb_ColumnFamilyHandle *, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_8aimrocks_11lib_rocksdb_DB *__pyx_vtabptr_8aimrocks_11lib_rocksdb_DB;


/* "aimrocks/lib_rocksdb.pxd":663
 * 
 * 
 * cdef class Iterator:             # <<<<<<<<<<<<<<
 *     cdef object _current_value
 * 
 */

struct __pyx_vtabstruct_8aimrocks_11lib_rocksdb_Iterator {
  PyObject *(*next)(struct __pyx_obj_8aimrocks_11lib_rocksdb_Iterator *, int __pyx_skip_dispatch);
  PyObject *(*get)(struct __pyx_obj_8aimrocks_11lib_rocksdb_Iterator *, int __pyx_skip_dispatch);
  void (*skip)(struct __pyx_obj_8aimrocks_11lib_rocksdb_Iterator *, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_8aimrocks_11lib_rocksdb_Iterator *__pyx_vtabptr_8aimrocks_11lib_rocksdb_Iterator;


/* "aimrocks/lib_rocksdb.pxd":671
 * 
 * 
 * cdef class BaseIterator(Iterator):             # <<<<<<<<<<<<<<
 *     cdef iterator.Iterator* ptr
 *     cdef DB db
 */

struct __pyx_vtabstruct_8aimrocks_11lib_rocksdb_BaseIterator {
  struct __pyx_vtabstruct_8aimrocks_11lib_rocksdb_Iterator __pyx_base;
  void (*seek_to_first)(struct __pyx_obj_8aimrocks_11lib_rocksdb_BaseIterator *, int __pyx_skip_dispatch);
  void (*seek_to_last)(struct __pyx_obj_8aimrocks_11lib_rocksdb_BaseIterator *, int __pyx_skip_dispatch);
  void (*seek)(struct __pyx_obj_8aimrocks_11lib_rocksdb_BaseIterator *, PyObject *, int __pyx_skip_dispatch);
  void (*seek_for_prev)(struct __pyx_obj_8aimrocks_11lib_rocksdb_BaseIterator *, PyObject *, int __pyx_skip_dispatch);
  PyObject *(*get_ob)(struct __pyx_obj_8aimrocks_11lib_rocksdb_BaseIterator *);
};
static struct __pyx_vtabstruct_8aimrocks_11lib_rocksdb_BaseIterator *__pyx_vtabptr_8aimrocks_11lib_rocksdb_BaseIterator;


/* "aimrocks/lib_rocksdb.pxd":687
 *     cdef object get_ob(self)
 * 
 * cdef class KeysIterator(BaseIterator):             # <<<<<<<<<<<<<<
 *     cdef object get_ob(self)
 * 
 */

struct __pyx_vtabstruct_8aimrocks_11lib_rocksdb_KeysIterator {
  struct __pyx_vtabstruct_8aimrocks_11lib_rocksdb_BaseIterator __pyx_base;
};
static struct __pyx_vtabstruct_8aimrocks_11lib_rocksdb_KeysIterator *__pyx_vtabptr_8aimrocks_11lib_rocksdb_KeysIterator;


/* "aimrocks/lib_rocksdb.pxd":690
 *     cdef object get_ob(self)
 * 
 * cdef class ValuesIterator(BaseIterator):             # <<<<<<<<<<<<<<
 *     cdef object get_ob(self)
 * 
 */

struct __pyx_vtabstruct_8aimrocks_11lib_rocksdb_ValuesIterator {
  struct __pyx_vtabstruct_8aimrocks_11lib_rocksdb_BaseIterator __pyx_base;
};
static struct __pyx_vtabstruct_8aimrocks_11lib_rocksdb_ValuesIterator *__pyx_vtabptr_8aimrocks_11lib_rocksdb_ValuesIterator;


/* "aimrocks/lib_rocksdb.pxd":693
 *     cdef object get_ob(self)
 * 
 * cdef class ItemsIterator(BaseIterator):             # <<<<<<<<<<<<<<
 *     cdef object get_ob(self)
 * 
 */

struct __pyx_vtabstruct_8aimrocks_11lib_rocksdb_ItemsIterator {
  struct __pyx_vtabstruct_8aimrocks_11lib_rocksdb_BaseIterator __pyx_base;
};
static struct __pyx_vtabstruct_8aimrocks_11lib_rocksdb_ItemsIterator *__pyx_vtabptr_8aimrocks_11lib_rocksdb_ItemsIterator;


/* "aimrocks/lib_rocksdb.pxd":696
 *     cdef object get_ob(self)
 * 
 * cdef class ReversedIterator(Iterator):             # <<<<<<<<<<<<<<
 *     cdef BaseIterator it
 * 
 */

struct __pyx_vtabstruct_8aimrocks_11lib_rocksdb_ReversedIterator {
  struct __pyx_vtabstruct_8aimrocks_11lib_rocksdb_Iterator __pyx_base;
  void (*seek_to_first)(struct __pyx_obj_8aimrocks_11lib_rocksdb_ReversedIterator *, int __pyx_skip_dispatch);
  void (*seek_to_last)(struct __pyx_obj_8aimrocks_11lib_rocksdb_ReversedIterator *, int __pyx_skip_dispatch);
  void (*seek)(struct __pyx_obj_8aimrocks_11lib_rocksdb_ReversedIterator *, PyObject *, int __pyx_skip_dispatch);
  void (*seek_for_prev)(struct __pyx_obj_8aimrocks_11lib_rocksdb_ReversedIterator *, PyObject *, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_8aimrocks_11lib_rocksdb_ReversedIterator *__pyx_vtabptr_8aimrocks_11lib_rocksdb_ReversedIterator;


/* "aimrocks/lib_rocksdb.pxd":708
 *     # def __next__(self)
 * 
 * cdef class BackupEngine(object):             # <<<<<<<<<<<<<<
 *     cdef backup.BackupEngine* engine
 * 
 */

struct __pyx_vtabstruct_8aimrocks_11lib_rocksdb_BackupEngine {
  PyObject *(*create_backup)(struct __pyx_obj_8aimrocks_11lib_rocksdb_BackupEngine *, struct __pyx_obj_8aimrocks_11lib_rocksdb_DB *, int __pyx_skip_dispatch, struct __pyx_opt_args_8aimrocks_11lib_rocksdb_12BackupEngine_create_backup *__pyx_optional_args);
  PyObject *(*restore_backup)(struct __pyx_obj_8aimrocks_11lib_rocksdb_BackupEngine *, PyObject *, PyObject *, PyObject *, int __pyx_skip_dispatch);
  PyObject *(*restore_latest_backup)(struct __pyx_obj_8aimrocks_11lib_rocksdb_BackupEngine *, PyObject *, PyObject *, int __pyx_skip_dispatch);
  PyObject *(*stop_backup)(struct __pyx_obj_8aimrocks_11lib_rocksdb_BackupEngine *, int __pyx_skip_dispatch);
  PyObject *(*purge_old_backups)(struct __pyx_obj_8aimrocks_11lib_rocksdb_BackupEngine *, PyObject *, int __pyx_skip_dispatch);
  PyObject *(*delete_backup)(struct __pyx_obj_8aimrocks_11lib_rocksdb_BackupEngine *, PyObject *, int __pyx_skip_dispatch);
  PyObject *(*get_backup_info)(struct __pyx_obj_8aimrocks_11lib_rocksdb_BackupEngine *, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_8aimrocks_11lib_rocksdb_BackupEngine *__pyx_vtabptr_8aimrocks_11lib_rocksdb_BackupEngine;


/* "utils.pxd":16
 *         public str aim_name
 * 
 * cdef class BLOB:             # <<<<<<<<<<<<<<
 *     cdef:
 *         object data
 */

struct __pyx_vtabstruct_3aim_7storage_5utils_BLOB {
  PyObject *(*load)(struct __pyx_obj_3aim_7storage_5utils_BLOB *, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_3aim_7storage_5utils_BLOB *__pyx_vtabptr_3aim_7storage_5utils_BLOB;


/* "aim/storage/container.pxd":7
 * from aim.storage.utils cimport interfaces
 * 
 * cdef class ContainerItemsIterator(interfaces.Iterator):             # <<<<<<<<<<<<<<
 *     pass
 * 
 */

struct __pyx_vtabstruct_3aim_7storage_9container_ContainerItemsIterator {
  struct __pyx_vtabstruct_8aimrocks_11lib_rocksdb_Iterator __pyx_base;
};
static struct __pyx_vtabstruct_3aim_7storage_9container_ContainerItemsIterator *__pyx_vtabptr_3aim_7storage_9container_ContainerItemsIterator;


/* "aim/storage/container.pxd":10
 *     pass
 * 
 * cdef class Container:             # <<<<<<<<<<<<<<
 *     cdef __weakref__
 * 
 */

struct __pyx_vtabstruct_3aim_7storage_9container_Container {
  void (*close)(struct __pyx_obj_3aim_7storage_9container_Container *, int __pyx_skip_dispatch);
  void (*preload)(struct __pyx_obj_3aim_7storage_9container_Container *, int __pyx_skip_dispatch);
  PyObject *(*get)(struct __pyx_obj_3aim_7storage_9container_Container *, PyObject *, int __pyx_skip_dispatch, struct __pyx_opt_args_3aim_7storage_9container_9Container_get *__pyx_optional_args);
  void (*set)(struct __pyx_obj_3aim_7storage_9container_Container *, PyObject *, PyObject *, int __pyx_skip_dispatch, struct __pyx_opt_args_3aim_7storage_9container_9Container_set *__pyx_optional_args);
  void (*delete_range)(struct __pyx_obj_3aim_7storage_9container_Container *, PyObject *, PyObject *, int __pyx_skip_dispatch, struct __pyx_opt_args_3aim_7storage_9container_9Container_delete_range *__pyx_optional_args);
  struct __pyx_obj_3aim_7storage_9container_ContainerItemsIterator *(*items)(struct __pyx_obj_3aim_7storage_9container_Container *, int __pyx_skip_dispatch, struct __pyx_opt_args_3aim_7storage_9container_9Container_items *__pyx_optional_args);
  PyObject *(*next_key)(struct __pyx_obj_3aim_7storage_9container_Container *, int __pyx_skip_dispatch, struct __pyx_opt_args_3aim_7storage_9container_9Container_next_key *__pyx_optional_args);
  PyObject *(*next_value)(struct __pyx_obj_3aim_7storage_9container_Container *, int __pyx_skip_dispatch, struct __pyx_opt_args_3aim_7storage_9container_9Container_next_value *__pyx_optional_args);
  PyObject *(*next_key_value)(struct __pyx_obj_3aim_7storage_9container_Container *, int __pyx_skip_dispatch, struct __pyx_opt_args_3aim_7storage_9container_9Container_next_key_value *__pyx_optional_args);
  PyObject *(*next_item)(struct __pyx_obj_3aim_7storage_9container_Container *, int __pyx_skip_dispatch, struct __pyx_opt_args_3aim_7storage_9container_9Container_next_item *__pyx_optional_args);
  PyObject *(*prev_key)(struct __pyx_obj_3aim_7storage_9container_Container *, int __pyx_skip_dispatch, struct __pyx_opt_args_3aim_7storage_9container_9Container_prev_key *__pyx_optional_args);
  PyObject *(*prev_value)(struct __pyx_obj_3aim_7storage_9container_Container *, int __pyx_skip_dispatch, struct __pyx_opt_args_3aim_7storage_9container_9Container_prev_value *__pyx_optional_args);
  PyObject *(*prev_key_value)(struct __pyx_obj_3aim_7storage_9container_Container *, int __pyx_skip_dispatch, struct __pyx_opt_args_3aim_7storage_9container_9Container_prev_key_value *__pyx_optional_args);
  PyObject *(*prev_item)(struct __pyx_obj_3aim_7storage_9container_Container *, int __pyx_skip_dispatch, struct __pyx_opt_args_3aim_7storage_9container_9Container_prev_item *__pyx_optional_args);
};
static struct __pyx_vtabstruct_3aim_7storage_9container_Container *__pyx_vtabptr_3aim_7storage_9container_Container;


/* "aim/storage/treeview.pxd":4
 * # cython: language_level = 3
 * 
 * cdef class TreeView:             # <<<<<<<<<<<<<<
 *     cdef:
 *         object __weakref__
 */

struct __pyx_vtabstruct_3aim_7storage_8treeview_TreeView {
  PyObject *(*view)(struct __pyx_obj_3aim_7storage_8treeview_TreeView *, PyObject *, int __pyx_skip_dispatch, struct __pyx_opt_args_3aim_7storage_8treeview_8TreeView_view *__pyx_optional_args);
  PyObject *(*make_array)(struct __pyx_obj_3aim_7storage_8treeview_TreeView *, int __pyx_skip_dispatch, struct __pyx_opt_args_3aim_7storage_8treeview_8TreeView_make_array *__pyx_optional_args);
  PyObject *(*collect)(struct __pyx_obj_3aim_7storage_8treeview_TreeView *, int __pyx_skip_dispatch, struct __pyx_opt_args_3aim_7storage_8treeview_8TreeView_collect *__pyx_optional_args);
};
static struct __pyx_vtabstruct_3aim_7storage_8treeview_TreeView *__pyx_vtabptr_3aim_7storage_8treeview_TreeView;


/* "aim/storage/containertreeview.pxd":10
 * from aim.storage.container cimport Container
 * 
 * cdef class ContainerTreeView(TreeView):             # <<<<<<<<<<<<<<
 *     cdef public Container container
 */

struct __pyx_vtabstruct_3aim_7storage_17containertreeview_ContainerTreeView {
  struct __pyx_vtabstruct_3aim_7storage_8treeview_TreeView __pyx_base;
};
static struct __pyx_vtabstruct_3aim_7storage_17containertreeview_ContainerTreeView *__pyx_vtabptr_3aim_7storage_17containertreeview_ContainerTreeView;


/* "aim/storage/prefixview.py":7
 * 
 * 
 * class PrefixView(Container):             # <<<<<<<<<<<<<<
 *     """
 *     A mutable view to a :obj:`Container` given by a key prefix.
 */

struct __pyx_vtabstruct_3aim_7storage_10prefixview_PrefixView {
  struct __pyx_vtabstruct_3aim_7storage_9container_Container __pyx_base;
  PyObject *(*absolute_path)(struct __pyx_obj_3aim_7storage_10prefixview_PrefixView *, int __pyx_skip_dispatch, struct __pyx_opt_args_3aim_7storage_10prefixview_10PrefixView_absolute_path *__pyx_optional_args);
  struct __pyx_obj_3aim_7storage_17containertreeview_ContainerTreeView *(*tree)(struct __pyx_obj_3aim_7storage_10prefixview_PrefixView *, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_3aim_7storage_10prefixview_PrefixView *__pyx_vtabptr_3aim_7storage_10prefixview_PrefixView;


/* "aim/storage/prefixview.py":328
 * 
 * 
 * class PrefixViewItemsIterator(ContainerItemsIterator):             # <<<<<<<<<<<<<<
 * 
 *     def __init__(self, prefix_view, key):
 */

struct __pyx_vtabstruct_3aim_7storage_10prefixview_PrefixViewItemsIterator {
  struct __pyx_vtabstruct_3aim_7storage_9container_ContainerItemsIterator __pyx_base;
};
static struct __pyx_vtabstruct_3aim_7storage_10prefixview_PrefixViewItemsIterator *__pyx_vtabptr_3aim_7storage_10prefixview_PrefixViewItemsIterator;
/* #### Code section: utility_code_proto ### */

/* --- Runtime support code (head) --- */
/* Refnanny.proto */
#ifndef CYTHON_REFNANNY
  #define CYTHON_REFNANNY 0
#endif
#if CYTHON_REFNANNY
  typedef struct {
    void (*INCREF)(void*, PyObject*, Py_ssize_t);
    void (*DECREF)(void*, PyObject*, Py_ssize_t);
    void (*GOTREF)(void*, PyObject*, Py_ssize_t);
    void (*GIVEREF)(void*, PyObject*, Py_ssize_t);
    void* (*SetupContext)(const char*, Py_ssize_t, const char*);
    void (*FinishContext)(void**);
  } __Pyx_RefNannyAPIStruct;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNanny = NULL;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname);
  #define __Pyx_RefNannyDeclarations void *__pyx_refnanny = NULL;
#ifdef WITH_THREAD
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          if (acquire_gil) {\
              PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), (__LINE__), (__FILE__));\
              PyGILState_Release(__pyx_gilstate_save);\
          } else {\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), (__LINE__), (__FILE__));\
          }
  #define __Pyx_RefNannyFinishContextNogil() {\
              PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();\
              __Pyx_RefNannyFinishContext();\
              PyGILState_Release(__pyx_gilstate_save);\
          }
#else
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), (__LINE__), (__FILE__))
  #define __Pyx_RefNannyFinishContextNogil() __Pyx_RefNannyFinishContext()
#endif
  #define __Pyx_RefNannyFinishContext()\
          __Pyx_RefNanny->FinishContext(&__pyx_refnanny)
  #define __Pyx_INCREF(r)  __Pyx_RefNanny->INCREF(__pyx_refnanny, (PyObject *)(r), (__LINE__))
  #define __Pyx_DECREF(r)  __Pyx_RefNanny->DECREF(__pyx_refnanny, (PyObject *)(r), (__LINE__))
  #define __Pyx_GOTREF(r)  __Pyx_RefNanny->GOTREF(__pyx_refnanny, (PyObject *)(r), (__LINE__))
  #define __Pyx_GIVEREF(r) __Pyx_RefNanny->GIVEREF(__pyx_refnanny, (PyObject *)(r), (__LINE__))
  #define __Pyx_XINCREF(r)  do { if((r) == NULL); else {__Pyx_INCREF(r); }} while(0)
  #define __Pyx_XDECREF(r)  do { if((r) == NULL); else {__Pyx_DECREF(r); }} while(0)
  #define __Pyx_XGOTREF(r)  do { if((r) == NULL); else {__Pyx_GOTREF(r); }} while(0)
  #define __Pyx_XGIVEREF(r) do { if((r) == NULL); else {__Pyx_GIVEREF(r);}} while(0)
#else
  #define __Pyx_RefNannyDeclarations
  #define __Pyx_RefNannySetupContext(name, acquire_gil)
  #define __Pyx_RefNannyFinishContextNogil()
  #define __Pyx_RefNannyFinishContext()
  #define __Pyx_INCREF(r) Py_INCREF(r)
  #define __Pyx_DECREF(r) Py_DECREF(r)
  #define __Pyx_GOTREF(r)
  #define __Pyx_GIVEREF(r)
  #define __Pyx_XINCREF(r) Py_XINCREF(r)
  #define __Pyx_XDECREF(r) Py_XDECREF(r)
  #define __Pyx_XGOTREF(r)
  #define __Pyx_XGIVEREF(r)
#endif
#define __Pyx_Py_XDECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; Py_XDECREF(tmp);\
    } while (0)
#define __Pyx_XDECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_XDECREF(tmp);\
    } while (0)
#define __Pyx_DECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_DECREF(tmp);\
    } while (0)
#define __Pyx_CLEAR(r)    do { PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);} while(0)
#define __Pyx_XCLEAR(r)   do { if((r) != NULL) {PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);}} while(0)

/* PyErrExceptionMatches.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyErr_ExceptionMatches(err) __Pyx_PyErr_ExceptionMatchesInState(__pyx_tstate, err)
static CYTHON_INLINE int __Pyx_PyErr_ExceptionMatchesInState(PyThreadState* tstate, PyObject* err);
#else
#define __Pyx_PyErr_ExceptionMatches(err)  PyErr_ExceptionMatches(err)
#endif

/* PyThreadStateGet.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyThreadState_declare  PyThreadState *__pyx_tstate;
#define __Pyx_PyThreadState_assign  __pyx_tstate = __Pyx_PyThreadState_Current;
#define __Pyx_PyErr_Occurred()  __pyx_tstate->curexc_type
#else
#define __Pyx_PyThreadState_declare
#define __Pyx_PyThreadState_assign
#define __Pyx_PyErr_Occurred()  PyErr_Occurred()
#endif

/* PyErrFetchRestore.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyErr_Clear() __Pyx_ErrRestore(NULL, NULL, NULL)
#define __Pyx_ErrRestoreWithState(type, value, tb)  __Pyx_ErrRestoreInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)    __Pyx_ErrFetchInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  __Pyx_ErrRestoreInState(__pyx_tstate, type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)    __Pyx_ErrFetchInState(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb);
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_PyErr_SetNone(exc) (Py_INCREF(exc), __Pyx_ErrRestore((exc), NULL, NULL))
#else
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#endif
#else
#define __Pyx_PyErr_Clear() PyErr_Clear()
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#define __Pyx_ErrRestoreWithState(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestoreInState(tstate, type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchInState(tstate, type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)  PyErr_Fetch(type, value, tb)
#endif

/* PyObjectGetAttrStr.proto */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name);
#else
#define __Pyx_PyObject_GetAttrStr(o,n) PyObject_GetAttr(o,n)
#endif

/* PyObjectGetAttrStrNoError.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStrNoError(PyObject* obj, PyObject* attr_name);

/* GetBuiltinName.proto */
static PyObject *__Pyx_GetBuiltinName(PyObject *name);

/* TupleAndListFromArray.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyList_FromArray(PyObject *const *src, Py_ssize_t n);
static CYTHON_INLINE PyObject* __Pyx_PyTuple_FromArray(PyObject *const *src, Py_ssize_t n);
#endif

/* IncludeStringH.proto */
#include <string.h>

/* BytesEquals.proto */
static CYTHON_INLINE int __Pyx_PyBytes_Equals(PyObject* s1, PyObject* s2, int equals);

/* UnicodeEquals.proto */
static CYTHON_INLINE int __Pyx_PyUnicode_Equals(PyObject* s1, PyObject* s2, int equals);

/* fastcall.proto */
#define __Pyx_Arg_VARARGS(args, i) PyTuple_GET_ITEM(args, i)
#define __Pyx_NumKwargs_VARARGS(kwds) PyDict_Size(kwds)
#define __Pyx_KwValues_VARARGS(args, nargs) NULL
#define __Pyx_GetKwValue_VARARGS(kw, kwvalues, s) __Pyx_PyDict_GetItemStrWithError(kw, s)
#define __Pyx_KwargsAsDict_VARARGS(kw, kwvalues) PyDict_Copy(kw)
#if CYTHON_METH_FASTCALL
    #define __Pyx_Arg_FASTCALL(args, i) args[i]
    #define __Pyx_NumKwargs_FASTCALL(kwds) PyTuple_GET_SIZE(kwds)
    #define __Pyx_KwValues_FASTCALL(args, nargs) (&args[nargs])
    static CYTHON_INLINE PyObject * __Pyx_GetKwValue_FASTCALL(PyObject *kwnames, PyObject *const *kwvalues, PyObject *s);
    #define __Pyx_KwargsAsDict_FASTCALL(kw, kwvalues) _PyStack_AsDict(kwvalues, kw)
#else
    #define __Pyx_Arg_FASTCALL __Pyx_Arg_VARARGS
    #define __Pyx_NumKwargs_FASTCALL __Pyx_NumKwargs_VARARGS
    #define __Pyx_KwValues_FASTCALL __Pyx_KwValues_VARARGS
    #define __Pyx_GetKwValue_FASTCALL __Pyx_GetKwValue_VARARGS
    #define __Pyx_KwargsAsDict_FASTCALL __Pyx_KwargsAsDict_VARARGS
#endif
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_ArgsSlice_VARARGS(args, start, stop) __Pyx_PyTuple_FromArray(&__Pyx_Arg_VARARGS(args, start), stop - start)
#define __Pyx_ArgsSlice_FASTCALL(args, start, stop) __Pyx_PyTuple_FromArray(&__Pyx_Arg_FASTCALL(args, start), stop - start)
#else
#define __Pyx_ArgsSlice_VARARGS(args, start, stop) PyTuple_GetSlice(args, start, stop)
#define __Pyx_ArgsSlice_FASTCALL(args, start, stop) PyTuple_GetSlice(args, start, stop)
#endif

/* RaiseKeywordRequired.proto */
static void __Pyx_RaiseKeywordRequired(const char* func_name, PyObject* kw_name);

/* RaiseDoubleKeywords.proto */
static void __Pyx_RaiseDoubleKeywordsError(const char* func_name, PyObject* kw_name);

/* ParseKeywords.proto */
static int __Pyx_ParseOptionalKeywords(PyObject *kwds, PyObject *const *kwvalues,
    PyObject **argnames[],
    PyObject *kwds2, PyObject *values[], Py_ssize_t num_pos_args,
    const char* function_name);

/* RaiseArgTupleInvalid.proto */
static void __Pyx_RaiseArgtupleInvalid(const char* func_name, int exact,
    Py_ssize_t num_min, Py_ssize_t num_max, Py_ssize_t num_found);

/* ArgTypeTest.proto */
#define __Pyx_ArgTypeTest(obj, type, none_allowed, name, exact)\
    ((likely(__Pyx_IS_TYPE(obj, type) | (none_allowed && (obj == Py_None)))) ? 1 :\
        __Pyx__ArgTypeTest(obj, type, name, exact))
static int __Pyx__ArgTypeTest(PyObject *obj, PyTypeObject *type, const char *name, int exact);

/* PyDictVersioning.proto */
#if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_TYPE_SLOTS
#define __PYX_DICT_VERSION_INIT  ((PY_UINT64_T) -1)
#define __PYX_GET_DICT_VERSION(dict)  (((PyDictObject*)(dict))->ma_version_tag)
#define __PYX_UPDATE_DICT_CACHE(dict, value, cache_var, version_var)\
    (version_var) = __PYX_GET_DICT_VERSION(dict);\
    (cache_var) = (value);
#define __PYX_PY_DICT_LOOKUP_IF_MODIFIED(VAR, DICT, LOOKUP) {\
    static PY_UINT64_T __pyx_dict_version = 0;\
    static PyObject *__pyx_dict_cached_value = NULL;\
    if (likely(__PYX_GET_DICT_VERSION(DICT) == __pyx_dict_version)) {\
        (VAR) = __pyx_dict_cached_value;\
    } else {\
        (VAR) = __pyx_dict_cached_value = (LOOKUP);\
        __pyx_dict_version = __PYX_GET_DICT_VERSION(DICT);\
    }\
}
static CYTHON_INLINE PY_UINT64_T __Pyx_get_tp_dict_version(PyObject *obj);
static CYTHON_INLINE PY_UINT64_T __Pyx_get_object_dict_version(PyObject *obj);
static CYTHON_INLINE int __Pyx_object_dict_version_matches(PyObject* obj, PY_UINT64_T tp_dict_version, PY_UINT64_T obj_dict_version);
#else
#define __PYX_GET_DICT_VERSION(dict)  (0)
#define __PYX_UPDATE_DICT_CACHE(dict, value, cache_var, version_var)
#define __PYX_PY_DICT_LOOKUP_IF_MODIFIED(VAR, DICT, LOOKUP)  (VAR) = (LOOKUP);
#endif

/* PyFunctionFastCall.proto */
#if CYTHON_FAST_PYCALL
#if !CYTHON_VECTORCALL
#define __Pyx_PyFunction_FastCall(func, args, nargs)\
    __Pyx_PyFunction_FastCallDict((func), (args), (nargs), NULL)
static PyObject *__Pyx_PyFunction_FastCallDict(PyObject *func, PyObject **args, Py_ssize_t nargs, PyObject *kwargs);
#endif
#define __Pyx_BUILD_ASSERT_EXPR(cond)\
    (sizeof(char [1 - 2*!(cond)]) - 1)
#ifndef Py_MEMBER_SIZE
#define Py_MEMBER_SIZE(type, member) sizeof(((type *)0)->member)
#endif
  static size_t __pyx_pyframe_localsplus_offset = 0;
  #include "frameobject.h"
  #define __Pxy_PyFrame_Initialize_Offsets()\
    ((void)__Pyx_BUILD_ASSERT_EXPR(sizeof(PyFrameObject) == offsetof(PyFrameObject, f_localsplus) + Py_MEMBER_SIZE(PyFrameObject, f_localsplus)),\
     (void)(__pyx_pyframe_localsplus_offset = ((size_t)PyFrame_Type.tp_basicsize) - Py_MEMBER_SIZE(PyFrameObject, f_localsplus)))
  #define __Pyx_PyFrame_GetLocalsplus(frame)\
    (assert(__pyx_pyframe_localsplus_offset), (PyObject **)(((char *)(frame)) + __pyx_pyframe_localsplus_offset))
#endif

/* PyObjectCall.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw);
#else
#define __Pyx_PyObject_Call(func, arg, kw) PyObject_Call(func, arg, kw)
#endif

/* PyObjectCallMethO.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg);
#endif

/* PyObjectFastCall.proto */
#define __Pyx_PyObject_FastCall(func, args, nargs)  __Pyx_PyObject_FastCallDict(func, args, (size_t)(nargs), NULL)
static CYTHON_INLINE PyObject* __Pyx_PyObject_FastCallDict(PyObject *func, PyObject **args, size_t nargs, PyObject *kwargs);

/* WriteUnraisableException.proto */
static void __Pyx_WriteUnraisable(const char *name, int clineno,
                                  int lineno, const char *filename,
                                  int full_traceback, int nogil);

/* KeywordStringCheck.proto */
static int __Pyx_CheckKeywordStrings(PyObject *kw, const char* function_name, int kw_allowed);

/* RaiseUnexpectedTypeError.proto */
static int __Pyx_RaiseUnexpectedTypeError(const char *expected, PyObject *obj);

/* GetItemInt.proto */
#define __Pyx_GetItemInt(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Fast(o, (Py_ssize_t)i, is_list, wraparound, boundscheck) :\
    (is_list ? (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL) :\
               __Pyx_GetItemInt_Generic(o, to_py_func(i))))
#define __Pyx_GetItemInt_List(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_List_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) :\
    (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck);
#define __Pyx_GetItemInt_Tuple(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Tuple_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) :\
    (PyErr_SetString(PyExc_IndexError, "tuple index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck);
static PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j);
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i,
                                                     int is_list, int wraparound, int boundscheck);

/* PyObjectCallOneArg.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg);

/* ObjectGetItem.proto */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject *__Pyx_PyObject_GetItem(PyObject *obj, PyObject *key);
#else
#define __Pyx_PyObject_GetItem(obj, key)  PyObject_GetItem(obj, key)
#endif

/* RaiseTooManyValuesToUnpack.proto */
static CYTHON_INLINE void __Pyx_RaiseTooManyValuesError(Py_ssize_t expected);

/* RaiseNeedMoreValuesToUnpack.proto */
static CYTHON_INLINE void __Pyx_RaiseNeedMoreValuesError(Py_ssize_t index);

/* IterFinish.proto */
static CYTHON_INLINE int __Pyx_IterFinish(void);

/* UnpackItemEndCheck.proto */
static int __Pyx_IternextUnpackEndCheck(PyObject *retval, Py_ssize_t expected);

/* RaiseException.proto */
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause);

/* IterNext.proto */
#define __Pyx_PyIter_Next(obj) __Pyx_PyIter_Next2(obj, NULL)
static CYTHON_INLINE PyObject *__Pyx_PyIter_Next2(PyObject *, PyObject *);

/* GetException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_GetException(type, value, tb)  __Pyx__GetException(__pyx_tstate, type, value, tb)
static int __Pyx__GetException(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#else
static int __Pyx_GetException(PyObject **type, PyObject **value, PyObject **tb);
#endif

/* pep479.proto */
static void __Pyx_Generator_Replace_StopIteration(int in_async_gen);

/* ExtTypeTest.proto */
static CYTHON_INLINE int __Pyx_TypeTest(PyObject *obj, PyTypeObject *type);

/* GetAttr3.proto */
static CYTHON_INLINE PyObject *__Pyx_GetAttr3(PyObject *, PyObject *, PyObject *);

/* GetModuleGlobalName.proto */
#if CYTHON_USE_DICT_VERSIONS
#define __Pyx_GetModuleGlobalName(var, name)  {\
    static PY_UINT64_T __pyx_dict_version = 0;\
    static PyObject *__pyx_dict_cached_value = NULL;\
    (var) = (likely(__pyx_dict_version == __PYX_GET_DICT_VERSION(__pyx_d))) ?\
        (likely(__pyx_dict_cached_value) ? __Pyx_NewRef(__pyx_dict_cached_value) : __Pyx_GetBuiltinName(name)) :\
        __Pyx__GetModuleGlobalName(name, &__pyx_dict_version, &__pyx_dict_cached_value);\
}
#define __Pyx_GetModuleGlobalNameUncached(var, name)  {\
    PY_UINT64_T __pyx_dict_version;\
    PyObject *__pyx_dict_cached_value;\
    (var) = __Pyx__GetModuleGlobalName(name, &__pyx_dict_version, &__pyx_dict_cached_value);\
}
static PyObject *__Pyx__GetModuleGlobalName(PyObject *name, PY_UINT64_T *dict_version, PyObject **dict_cached_value);
#else
#define __Pyx_GetModuleGlobalName(var, name)  (var) = __Pyx__GetModuleGlobalName(name)
#define __Pyx_GetModuleGlobalNameUncached(var, name)  (var) = __Pyx__GetModuleGlobalName(name)
static CYTHON_INLINE PyObject *__Pyx__GetModuleGlobalName(PyObject *name);
#endif

/* Import.proto */
static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level);

/* ImportFrom.proto */
static PyObject* __Pyx_ImportFrom(PyObject* module, PyObject* name);

/* GetAttr.proto */
static CYTHON_INLINE PyObject *__Pyx_GetAttr(PyObject *, PyObject *);

/* HasAttr.proto */
static CYTHON_INLINE int __Pyx_HasAttr(PyObject *, PyObject *);

/* CallNextTpDealloc.proto */
static void __Pyx_call_next_tp_dealloc(PyObject* obj, destructor current_tp_dealloc);

/* CallNextTpTraverse.proto */
static int __Pyx_call_next_tp_traverse(PyObject* obj, visitproc v, void *a, traverseproc current_tp_traverse);

/* CallNextTpClear.proto */
static void __Pyx_call_next_tp_clear(PyObject* obj, inquiry current_tp_dealloc);

/* TypeImport.proto */
#ifndef __PYX_HAVE_RT_ImportType_proto
#define __PYX_HAVE_RT_ImportType_proto
enum __Pyx_ImportType_CheckSize {
   __Pyx_ImportType_CheckSize_Error = 0,
   __Pyx_ImportType_CheckSize_Warn = 1,
   __Pyx_ImportType_CheckSize_Ignore = 2
};
static PyTypeObject *__Pyx_ImportType(PyObject* module, const char *module_name, const char *class_name, size_t size, enum __Pyx_ImportType_CheckSize check_size);
#endif

/* GetVTable.proto */
static void* __Pyx_GetVtable(PyTypeObject *type);

/* IncludeStructmemberH.proto */
#include <structmember.h>

/* FixUpExtensionType.proto */
#if CYTHON_USE_TYPE_SPECS
static int __Pyx_fix_up_extension_type_from_spec(PyType_Spec *spec, PyTypeObject *type);
#endif

/* PyObjectCallNoArg.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func);

/* PyObjectGetMethod.proto */
static int __Pyx_PyObject_GetMethod(PyObject *obj, PyObject *name, PyObject **method);

/* PyObjectCallMethod0.proto */
static PyObject* __Pyx_PyObject_CallMethod0(PyObject* obj, PyObject* method_name);

/* ValidateBasesTuple.proto */
#if CYTHON_COMPILING_IN_CPYTHON || CYTHON_COMPILING_IN_LIMITED_API || CYTHON_USE_TYPE_SPECS
static int __Pyx_validate_bases_tuple(const char *type_name, Py_ssize_t dictoffset, PyObject *bases);
#endif

/* PyType_Ready.proto */
static int __Pyx_PyType_Ready(PyTypeObject *t);

/* PyObject_GenericGetAttrNoDict.proto */
#if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static CYTHON_INLINE PyObject* __Pyx_PyObject_GenericGetAttrNoDict(PyObject* obj, PyObject* attr_name);
#else
#define __Pyx_PyObject_GenericGetAttrNoDict PyObject_GenericGetAttr
#endif

/* PyObject_GenericGetAttr.proto */
#if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static PyObject* __Pyx_PyObject_GenericGetAttr(PyObject* obj, PyObject* attr_name);
#else
#define __Pyx_PyObject_GenericGetAttr PyObject_GenericGetAttr
#endif

/* SetVTable.proto */
static int __Pyx_SetVtable(PyTypeObject* typeptr , void* vtable);

/* MergeVTables.proto */
#if !CYTHON_COMPILING_IN_LIMITED_API
static int __Pyx_MergeVtables(PyTypeObject *type);
#endif

/* SetupReduce.proto */
#if !CYTHON_COMPILING_IN_LIMITED_API
static int __Pyx_setup_reduce(PyObject* type_obj);
#endif

/* FetchCommonType.proto */
#if !CYTHON_USE_TYPE_SPECS
static PyTypeObject* __Pyx_FetchCommonType(PyTypeObject* type);
#else
static PyTypeObject* __Pyx_FetchCommonTypeFromSpec(PyObject *module, PyType_Spec *spec, PyObject *bases);
#endif

/* PyMethodNew.proto */
#if PY_MAJOR_VERSION >= 3
static PyObject *__Pyx_PyMethod_New(PyObject *func, PyObject *self, PyObject *typ) {
    CYTHON_UNUSED_VAR(typ);
    if (!self)
        return __Pyx_NewRef(func);
    return PyMethod_New(func, self);
}
#else
    #define __Pyx_PyMethod_New PyMethod_New
#endif

/* PyVectorcallFastCallDict.proto */
#if CYTHON_METH_FASTCALL
static CYTHON_INLINE PyObject *__Pyx_PyVectorcall_FastCallDict(PyObject *func, __pyx_vectorcallfunc vc, PyObject *const *args, size_t nargs, PyObject *kw);
#endif

/* CythonFunctionShared.proto */
#define __Pyx_CyFunction_USED
#define __Pyx_CYFUNCTION_STATICMETHOD  0x01
#define __Pyx_CYFUNCTION_CLASSMETHOD   0x02
#define __Pyx_CYFUNCTION_CCLASS        0x04
#define __Pyx_CYFUNCTION_COROUTINE     0x08
#define __Pyx_CyFunction_GetClosure(f)\
    (((__pyx_CyFunctionObject *) (f))->func_closure)
#if PY_VERSION_HEX < 0x030900B1
  #define __Pyx_CyFunction_GetClassObj(f)\
      (((__pyx_CyFunctionObject *) (f))->func_classobj)
#else
  #define __Pyx_CyFunction_GetClassObj(f)\
      ((PyObject*) ((PyCMethodObject *) (f))->mm_class)
#endif
#define __Pyx_CyFunction_SetClassObj(f, classobj)\
    __Pyx__CyFunction_SetClassObj((__pyx_CyFunctionObject *) (f), (classobj))
#define __Pyx_CyFunction_Defaults(type, f)\
    ((type *)(((__pyx_CyFunctionObject *) (f))->defaults))
#define __Pyx_CyFunction_SetDefaultsGetter(f, g)\
    ((__pyx_CyFunctionObject *) (f))->defaults_getter = (g)
typedef struct {
#if PY_VERSION_HEX < 0x030900B1
    PyCFunctionObject func;
#else
    PyCMethodObject func;
#endif
#if CYTHON_BACKPORT_VECTORCALL
    __pyx_vectorcallfunc func_vectorcall;
#endif
#if PY_VERSION_HEX < 0x030500A0
    PyObject *func_weakreflist;
#endif
    PyObject *func_dict;
    PyObject *func_name;
    PyObject *func_qualname;
    PyObject *func_doc;
    PyObject *func_globals;
    PyObject *func_code;
    PyObject *func_closure;
#if PY_VERSION_HEX < 0x030900B1
    PyObject *func_classobj;
#endif
    void *defaults;
    int defaults_pyobjects;
    size_t defaults_size;  // used by FusedFunction for copying defaults
    int flags;
    PyObject *defaults_tuple;
    PyObject *defaults_kwdict;
    PyObject *(*defaults_getter)(PyObject *);
    PyObject *func_annotations;
    PyObject *func_is_coroutine;
} __pyx_CyFunctionObject;
#if !CYTHON_USE_MODULE_STATE
static PyTypeObject *__pyx_CyFunctionType = 0;
#endif
#define __Pyx_CyFunction_Check(obj)  __Pyx_TypeCheck(obj, __pyx_CyFunctionType)
#define __Pyx_IsCyOrPyCFunction(obj)  __Pyx_TypeCheck2(obj, __pyx_CyFunctionType, &PyCFunction_Type)
#define __Pyx_CyFunction_CheckExact(obj)  __Pyx_IS_TYPE(obj, __pyx_CyFunctionType)
static PyObject *__Pyx_CyFunction_Init(__pyx_CyFunctionObject* op, PyMethodDef *ml,
                                      int flags, PyObject* qualname,
                                      PyObject *closure,
                                      PyObject *module, PyObject *globals,
                                      PyObject* code);
static CYTHON_INLINE void __Pyx__CyFunction_SetClassObj(__pyx_CyFunctionObject* f, PyObject* classobj);
static CYTHON_INLINE void *__Pyx_CyFunction_InitDefaults(PyObject *m,
                                                         size_t size,
                                                         int pyobjects);
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsTuple(PyObject *m,
                                                            PyObject *tuple);
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsKwDict(PyObject *m,
                                                             PyObject *dict);
static CYTHON_INLINE void __Pyx_CyFunction_SetAnnotationsDict(PyObject *m,
                                                              PyObject *dict);
static int __pyx_CyFunction_init(PyObject *module);
#if CYTHON_METH_FASTCALL
static PyObject * __Pyx_CyFunction_Vectorcall_NOARGS(PyObject *func, PyObject *const *args, size_t nargsf, PyObject *kwnames);
static PyObject * __Pyx_CyFunction_Vectorcall_O(PyObject *func, PyObject *const *args, size_t nargsf, PyObject *kwnames);
static PyObject * __Pyx_CyFunction_Vectorcall_FASTCALL_KEYWORDS(PyObject *func, PyObject *const *args, size_t nargsf, PyObject *kwnames);
static PyObject * __Pyx_CyFunction_Vectorcall_FASTCALL_KEYWORDS_METHOD(PyObject *func, PyObject *const *args, size_t nargsf, PyObject *kwnames);
#if CYTHON_BACKPORT_VECTORCALL
#define __Pyx_CyFunction_func_vectorcall(f) (((__pyx_CyFunctionObject*)f)->func_vectorcall)
#else
#define __Pyx_CyFunction_func_vectorcall(f) (((PyCFunctionObject*)f)->vectorcall)
#endif
#endif

/* CythonFunction.proto */
static PyObject *__Pyx_CyFunction_New(PyMethodDef *ml,
                                      int flags, PyObject* qualname,
                                      PyObject *closure,
                                      PyObject *module, PyObject *globals,
                                      PyObject* code);

/* CLineInTraceback.proto */
#ifdef CYTHON_CLINE_IN_TRACEBACK
#define __Pyx_CLineForTraceback(tstate, c_line)  (((CYTHON_CLINE_IN_TRACEBACK)) ? c_line : 0)
#else
static int __Pyx_CLineForTraceback(PyThreadState *tstate, int c_line);
#endif

/* CodeObjectCache.proto */
#if !CYTHON_COMPILING_IN_LIMITED_API
typedef struct {
    PyCodeObject* code_object;
    int code_line;
} __Pyx_CodeObjectCacheEntry;
struct __Pyx_CodeObjectCache {
    int count;
    int max_count;
    __Pyx_CodeObjectCacheEntry* entries;
};
static struct __Pyx_CodeObjectCache __pyx_code_cache = {0,0,NULL};
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line);
static PyCodeObject *__pyx_find_code_object(int code_line);
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object);
#endif

/* AddTraceback.proto */
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename);

/* GCCDiagnostics.proto */
#if defined(__GNUC__) && (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 6))
#define __Pyx_HAS_GCC_DIAGNOSTIC
#endif

/* None.proto */
#include <new>

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int(int value);

/* CIntFromPy.proto */
static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value);

/* FormatTypeName.proto */
#if CYTHON_COMPILING_IN_LIMITED_API
typedef PyObject *__Pyx_TypeName;
#define __Pyx_FMT_TYPENAME "%U"
static __Pyx_TypeName __Pyx_PyType_GetName(PyTypeObject* tp);
#define __Pyx_DECREF_TypeName(obj) Py_XDECREF(obj)
#else
typedef const char *__Pyx_TypeName;
#define __Pyx_FMT_TYPENAME "%.200s"
#define __Pyx_PyType_GetName(tp) ((tp)->tp_name)
#define __Pyx_DECREF_TypeName(obj)
#endif

/* FastTypeChecks.proto */
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_TypeCheck(obj, type) __Pyx_IsSubtype(Py_TYPE(obj), (PyTypeObject *)type)
#define __Pyx_TypeCheck2(obj, type1, type2) __Pyx_IsAnySubtype2(Py_TYPE(obj), (PyTypeObject *)type1, (PyTypeObject *)type2)
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b);
static CYTHON_INLINE int __Pyx_IsAnySubtype2(PyTypeObject *cls, PyTypeObject *a, PyTypeObject *b);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject *type);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *type1, PyObject *type2);
#else
#define __Pyx_TypeCheck(obj, type) PyObject_TypeCheck(obj, (PyTypeObject *)type)
#define __Pyx_TypeCheck2(obj, type1, type2) (PyObject_TypeCheck(obj, (PyTypeObject *)type1) || PyObject_TypeCheck(obj, (PyTypeObject *)type2))
#define __Pyx_PyErr_GivenExceptionMatches(err, type) PyErr_GivenExceptionMatches(err, type)
#define __Pyx_PyErr_GivenExceptionMatches2(err, type1, type2) (PyErr_GivenExceptionMatches(err, type1) || PyErr_GivenExceptionMatches(err, type2))
#endif
#define __Pyx_PyErr_ExceptionMatches2(err1, err2)  __Pyx_PyErr_GivenExceptionMatches2(__Pyx_PyErr_Occurred(), err1, err2)
#define __Pyx_PyException_Check(obj) __Pyx_TypeCheck(obj, PyExc_Exception)

/* GetTopmostException.proto */
#if CYTHON_USE_EXC_INFO_STACK
static _PyErr_StackItem * __Pyx_PyErr_GetTopmostException(PyThreadState *tstate);
#endif

/* SaveResetException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_ExceptionSave(type, value, tb)  __Pyx__ExceptionSave(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionSave(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#define __Pyx_ExceptionReset(type, value, tb)  __Pyx__ExceptionReset(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionReset(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb);
#else
#define __Pyx_ExceptionSave(type, value, tb)   PyErr_GetExcInfo(type, value, tb)
#define __Pyx_ExceptionReset(type, value, tb)  PyErr_SetExcInfo(type, value, tb)
#endif

/* SwapException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_ExceptionSwap(type, value, tb)  __Pyx__ExceptionSwap(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionSwap(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#else
static CYTHON_INLINE void __Pyx_ExceptionSwap(PyObject **type, PyObject **value, PyObject **tb);
#endif

/* PyObjectCall2Args.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call2Args(PyObject* function, PyObject* arg1, PyObject* arg2);

/* PyObjectCallMethod1.proto */
static PyObject* __Pyx_PyObject_CallMethod1(PyObject* obj, PyObject* method_name, PyObject* arg);

/* CoroutineBase.proto */
struct __pyx_CoroutineObject;
typedef PyObject *(*__pyx_coroutine_body_t)(struct __pyx_CoroutineObject *, PyThreadState *, PyObject *);
#if CYTHON_USE_EXC_INFO_STACK
#define __Pyx_ExcInfoStruct  _PyErr_StackItem
#else
typedef struct {
    PyObject *exc_type;
    PyObject *exc_value;
    PyObject *exc_traceback;
} __Pyx_ExcInfoStruct;
#endif
typedef struct __pyx_CoroutineObject {
    PyObject_HEAD
    __pyx_coroutine_body_t body;
    PyObject *closure;
    __Pyx_ExcInfoStruct gi_exc_state;
    PyObject *gi_weakreflist;
    PyObject *classobj;
    PyObject *yieldfrom;
    PyObject *gi_name;
    PyObject *gi_qualname;
    PyObject *gi_modulename;
    PyObject *gi_code;
    PyObject *gi_frame;
    int resume_label;
    char is_running;
} __pyx_CoroutineObject;
static __pyx_CoroutineObject *__Pyx__Coroutine_New(
    PyTypeObject *type, __pyx_coroutine_body_t body, PyObject *code, PyObject *closure,
    PyObject *name, PyObject *qualname, PyObject *module_name);
static __pyx_CoroutineObject *__Pyx__Coroutine_NewInit(
            __pyx_CoroutineObject *gen, __pyx_coroutine_body_t body, PyObject *code, PyObject *closure,
            PyObject *name, PyObject *qualname, PyObject *module_name);
static CYTHON_INLINE void __Pyx_Coroutine_ExceptionClear(__Pyx_ExcInfoStruct *self);
static int __Pyx_Coroutine_clear(PyObject *self);
static PyObject *__Pyx_Coroutine_Send(PyObject *self, PyObject *value);
static PyObject *__Pyx_Coroutine_Close(PyObject *self);
static PyObject *__Pyx_Coroutine_Throw(PyObject *gen, PyObject *args);
#if CYTHON_USE_EXC_INFO_STACK
#define __Pyx_Coroutine_SwapException(self)
#define __Pyx_Coroutine_ResetAndClearException(self)  __Pyx_Coroutine_ExceptionClear(&(self)->gi_exc_state)
#else
#define __Pyx_Coroutine_SwapException(self) {\
    __Pyx_ExceptionSwap(&(self)->gi_exc_state.exc_type, &(self)->gi_exc_state.exc_value, &(self)->gi_exc_state.exc_traceback);\
    __Pyx_Coroutine_ResetFrameBackpointer(&(self)->gi_exc_state);\
    }
#define __Pyx_Coroutine_ResetAndClearException(self) {\
    __Pyx_ExceptionReset((self)->gi_exc_state.exc_type, (self)->gi_exc_state.exc_value, (self)->gi_exc_state.exc_traceback);\
    (self)->gi_exc_state.exc_type = (self)->gi_exc_state.exc_value = (self)->gi_exc_state.exc_traceback = NULL;\
    }
#endif
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyGen_FetchStopIterationValue(pvalue)\
    __Pyx_PyGen__FetchStopIterationValue(__pyx_tstate, pvalue)
#else
#define __Pyx_PyGen_FetchStopIterationValue(pvalue)\
    __Pyx_PyGen__FetchStopIterationValue(__Pyx_PyThreadState_Current, pvalue)
#endif
static int __Pyx_PyGen__FetchStopIterationValue(PyThreadState *tstate, PyObject **pvalue);
static CYTHON_INLINE void __Pyx_Coroutine_ResetFrameBackpointer(__Pyx_ExcInfoStruct *exc_state);

/* PatchModuleWithCoroutine.proto */
static PyObject* __Pyx_Coroutine_patch_module(PyObject* module, const char* py_code);

/* PatchGeneratorABC.proto */
static int __Pyx_patch_abc(void);

/* Generator.proto */
#define __Pyx_Generator_USED
static PyTypeObject *__pyx_GeneratorType = 0;
#define __Pyx_Generator_CheckExact(obj) __Pyx_IS_TYPE(obj, __pyx_GeneratorType)
#define __Pyx_Generator_New(body, code, closure, name, qualname, module_name)\
    __Pyx__Coroutine_New(__pyx_GeneratorType, body, code, closure, name, qualname, module_name)
static PyObject *__Pyx_Generator_Next(PyObject *self);
static int __pyx_Generator_init(PyObject *module);

/* CheckBinaryVersion.proto */
static int __Pyx_check_binary_version(void);

/* FunctionImport.proto */
static int __Pyx_ImportFunction(PyObject *module, const char *funcname, void (**f)(void), const char *sig);

/* InitStrings.proto */
#if CYTHON_COMPILING_IN_LIMITED_API
static int __Pyx_InitString(__Pyx_StringTabEntry t, PyObject **str);
#else
static int __Pyx_InitStrings(__Pyx_StringTabEntry *t);
#endif

/* #### Code section: module_declarations ### */
static void __pyx_f_3aim_7storage_10prefixview_10PrefixView_close(CYTHON_UNUSED struct __pyx_obj_3aim_7storage_10prefixview_PrefixView *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static void __pyx_f_3aim_7storage_10prefixview_10PrefixView_preload(struct __pyx_obj_3aim_7storage_10prefixview_PrefixView *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_3aim_7storage_10prefixview_10PrefixView_absolute_path(struct __pyx_obj_3aim_7storage_10prefixview_PrefixView *__pyx_v_self, int __pyx_skip_dispatch, struct __pyx_opt_args_3aim_7storage_10prefixview_10PrefixView_absolute_path *__pyx_optional_args); /* proto*/
static PyObject *__pyx_f_3aim_7storage_10prefixview_10PrefixView_get(struct __pyx_obj_3aim_7storage_10prefixview_PrefixView *__pyx_v_self, PyObject *__pyx_v_key, int __pyx_skip_dispatch, struct __pyx_opt_args_3aim_7storage_10prefixview_10PrefixView_get *__pyx_optional_args); /* proto*/
static void __pyx_f_3aim_7storage_10prefixview_10PrefixView_set(struct __pyx_obj_3aim_7storage_10prefixview_PrefixView *__pyx_v_self, PyObject *__pyx_v_key, PyObject *__pyx_v_value, int __pyx_skip_dispatch, struct __pyx_opt_args_3aim_7storage_10prefixview_10PrefixView_set *__pyx_optional_args); /* proto*/
static void __pyx_f_3aim_7storage_10prefixview_10PrefixView_delete_range(struct __pyx_obj_3aim_7storage_10prefixview_PrefixView *__pyx_v_self, PyObject *__pyx_v_begin, PyObject *__pyx_v_end, int __pyx_skip_dispatch, struct __pyx_opt_args_3aim_7storage_10prefixview_10PrefixView_delete_range *__pyx_optional_args); /* proto*/
static PyObject *__pyx_f_3aim_7storage_10prefixview_10PrefixView_next_item(struct __pyx_obj_3aim_7storage_10prefixview_PrefixView *__pyx_v_self, int __pyx_skip_dispatch, struct __pyx_opt_args_3aim_7storage_10prefixview_10PrefixView_next_item *__pyx_optional_args); /* proto*/
static PyObject *__pyx_f_3aim_7storage_10prefixview_10PrefixView_prev_item(struct __pyx_obj_3aim_7storage_10prefixview_PrefixView *__pyx_v_self, int __pyx_skip_dispatch, struct __pyx_opt_args_3aim_7storage_10prefixview_10PrefixView_prev_item *__pyx_optional_args); /* proto*/
static struct __pyx_obj_3aim_7storage_9container_ContainerItemsIterator *__pyx_f_3aim_7storage_10prefixview_10PrefixView_items(struct __pyx_obj_3aim_7storage_10prefixview_PrefixView *__pyx_v_self, int __pyx_skip_dispatch, struct __pyx_opt_args_3aim_7storage_10prefixview_10PrefixView_items *__pyx_optional_args); /* proto*/
static struct __pyx_obj_3aim_7storage_17containertreeview_ContainerTreeView *__pyx_f_3aim_7storage_10prefixview_10PrefixView_tree(struct __pyx_obj_3aim_7storage_10prefixview_PrefixView *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_3aim_7storage_10prefixview_23PrefixViewItemsIterator_next(struct __pyx_obj_3aim_7storage_10prefixview_PrefixViewItemsIterator *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static CYTHON_INLINE double __pyx_f_7cpython_7complex_7complex_4real_real(PyComplexObject *__pyx_v_self); /* proto*/
static CYTHON_INLINE double __pyx_f_7cpython_7complex_7complex_4imag_imag(PyComplexObject *__pyx_v_self); /* proto*/

/* Module declarations from "cython" */
#if !CYTHON_USE_MODULE_STATE
#endif

/* Module declarations from "aim.storage" */
#if !CYTHON_USE_MODULE_STATE
#endif

/* Module declarations from "libc.string" */
#if !CYTHON_USE_MODULE_STATE
#endif

/* Module declarations from "libcpp.string" */
#if !CYTHON_USE_MODULE_STATE
#endif

/* Module declarations from "libcpp.deque" */
#if !CYTHON_USE_MODULE_STATE
#endif

/* Module declarations from "libcpp.vector" */
#if !CYTHON_USE_MODULE_STATE
#endif

/* Module declarations from "cpython.version" */
#if !CYTHON_USE_MODULE_STATE
#endif

/* Module declarations from "__builtin__" */
#if !CYTHON_USE_MODULE_STATE
#endif

/* Module declarations from "cpython.type" */
#if !CYTHON_USE_MODULE_STATE
static PyTypeObject *__pyx_ptype_7cpython_4type_type = 0;
#endif

/* Module declarations from "libc.stdio" */
#if !CYTHON_USE_MODULE_STATE
#endif

/* Module declarations from "cpython.object" */
#if !CYTHON_USE_MODULE_STATE
#endif

/* Module declarations from "cpython.ref" */
#if !CYTHON_USE_MODULE_STATE
#endif

/* Module declarations from "cpython.exc" */
#if !CYTHON_USE_MODULE_STATE
#endif

/* Module declarations from "cpython.module" */
#if !CYTHON_USE_MODULE_STATE
#endif

/* Module declarations from "cpython.mem" */
#if !CYTHON_USE_MODULE_STATE
#endif

/* Module declarations from "cpython.tuple" */
#if !CYTHON_USE_MODULE_STATE
#endif

/* Module declarations from "cpython.list" */
#if !CYTHON_USE_MODULE_STATE
#endif

/* Module declarations from "cpython.sequence" */
#if !CYTHON_USE_MODULE_STATE
#endif

/* Module declarations from "cpython.mapping" */
#if !CYTHON_USE_MODULE_STATE
#endif

/* Module declarations from "cpython.iterator" */
#if !CYTHON_USE_MODULE_STATE
#endif

/* Module declarations from "cpython.number" */
#if !CYTHON_USE_MODULE_STATE
#endif

/* Module declarations from "cpython.int" */
#if !CYTHON_USE_MODULE_STATE
#endif

/* Module declarations from "__builtin__" */
#if !CYTHON_USE_MODULE_STATE
#endif

/* Module declarations from "cpython.bool" */
#if !CYTHON_USE_MODULE_STATE
static PyTypeObject *__pyx_ptype_7cpython_4bool_bool = 0;
#endif

/* Module declarations from "cpython.long" */
#if !CYTHON_USE_MODULE_STATE
#endif

/* Module declarations from "cpython.float" */
#if !CYTHON_USE_MODULE_STATE
#endif

/* Module declarations from "__builtin__" */
#if !CYTHON_USE_MODULE_STATE
#endif

/* Module declarations from "cpython.complex" */
#if !CYTHON_USE_MODULE_STATE
static PyTypeObject *__pyx_ptype_7cpython_7complex_complex = 0;
#endif

/* Module declarations from "cpython.string" */
#if !CYTHON_USE_MODULE_STATE
#endif

/* Module declarations from "cpython.unicode" */
#if !CYTHON_USE_MODULE_STATE
#endif

/* Module declarations from "cpython.pyport" */
#if !CYTHON_USE_MODULE_STATE
#endif

/* Module declarations from "cpython.dict" */
#if !CYTHON_USE_MODULE_STATE
#endif

/* Module declarations from "cpython.instance" */
#if !CYTHON_USE_MODULE_STATE
#endif

/* Module declarations from "cpython.function" */
#if !CYTHON_USE_MODULE_STATE
#endif

/* Module declarations from "cpython.method" */
#if !CYTHON_USE_MODULE_STATE
#endif

/* Module declarations from "cpython.weakref" */
#if !CYTHON_USE_MODULE_STATE
#endif

/* Module declarations from "cpython.getargs" */
#if !CYTHON_USE_MODULE_STATE
#endif

/* Module declarations from "cpython.pythread" */
#if !CYTHON_USE_MODULE_STATE
#endif

/* Module declarations from "cpython.pystate" */
#if !CYTHON_USE_MODULE_STATE
#endif

/* Module declarations from "cpython.cobject" */
#if !CYTHON_USE_MODULE_STATE
#endif

/* Module declarations from "cpython.oldbuffer" */
#if !CYTHON_USE_MODULE_STATE
#endif

/* Module declarations from "cpython.set" */
#if !CYTHON_USE_MODULE_STATE
#endif

/* Module declarations from "cpython.buffer" */
#if !CYTHON_USE_MODULE_STATE
#endif

/* Module declarations from "cpython.bytes" */
#if !CYTHON_USE_MODULE_STATE
#endif

/* Module declarations from "cpython.pycapsule" */
#if !CYTHON_USE_MODULE_STATE
#endif

/* Module declarations from "cpython.contextvars" */
#if !CYTHON_USE_MODULE_STATE
#endif

/* Module declarations from "cpython" */
#if !CYTHON_USE_MODULE_STATE
#endif

/* Module declarations from "libcpp" */
#if !CYTHON_USE_MODULE_STATE
#endif

/* Module declarations from "libc.stdint" */
#if !CYTHON_USE_MODULE_STATE
#endif

/* Module declarations from "aimrocks.std_memory" */
#if !CYTHON_USE_MODULE_STATE
#endif

/* Module declarations from "aimrocks.slice_" */
#if !CYTHON_USE_MODULE_STATE
#endif

/* Module declarations from "aimrocks.logger" */
#if !CYTHON_USE_MODULE_STATE
#endif

/* Module declarations from "aimrocks.comparator" */
#if !CYTHON_USE_MODULE_STATE
#endif

/* Module declarations from "aimrocks.merge_operator" */
#if !CYTHON_USE_MODULE_STATE
#endif

/* Module declarations from "aimrocks.snapshot" */
#if !CYTHON_USE_MODULE_STATE
#endif

/* Module declarations from "aimrocks.slice_transform" */
#if !CYTHON_USE_MODULE_STATE
#endif

/* Module declarations from "aimrocks.cache" */
#if !CYTHON_USE_MODULE_STATE
#endif

/* Module declarations from "aimrocks.filter_policy" */
#if !CYTHON_USE_MODULE_STATE
#endif

/* Module declarations from "aimrocks.table_factory" */
#if !CYTHON_USE_MODULE_STATE
#endif

/* Module declarations from "aimrocks.memtablerep" */
#if !CYTHON_USE_MODULE_STATE
#endif

/* Module declarations from "aimrocks.universal_compaction" */
#if !CYTHON_USE_MODULE_STATE
#endif

/* Module declarations from "aimrocks.options" */
#if !CYTHON_USE_MODULE_STATE
#endif

/* Module declarations from "aimrocks.status" */
#if !CYTHON_USE_MODULE_STATE
#endif

/* Module declarations from "aimrocks.iterator" */
#if !CYTHON_USE_MODULE_STATE
#endif

/* Module declarations from "aimrocks.db" */
#if !CYTHON_USE_MODULE_STATE
#endif

/* Module declarations from "aimrocks.env" */
#if !CYTHON_USE_MODULE_STATE
#endif

/* Module declarations from "aimrocks.backup" */
#if !CYTHON_USE_MODULE_STATE
#endif

/* Module declarations from "aimrocks.lib_rocksdb" */
#if !CYTHON_USE_MODULE_STATE
static PyTypeObject *__pyx_ptype_8aimrocks_11lib_rocksdb_PyComparator = 0;
static PyTypeObject *__pyx_ptype_8aimrocks_11lib_rocksdb_PyGenericComparator = 0;
static PyTypeObject *__pyx_ptype_8aimrocks_11lib_rocksdb_PyBytewiseComparator = 0;
static PyTypeObject *__pyx_ptype_8aimrocks_11lib_rocksdb_PyFilterPolicy = 0;
static PyTypeObject *__pyx_ptype_8aimrocks_11lib_rocksdb_PyGenericFilterPolicy = 0;
static PyTypeObject *__pyx_ptype_8aimrocks_11lib_rocksdb_PyBloomFilterPolicy = 0;
static PyTypeObject *__pyx_ptype_8aimrocks_11lib_rocksdb_PyMergeOperator = 0;
static PyTypeObject *__pyx_ptype_8aimrocks_11lib_rocksdb_PyCache = 0;
static PyTypeObject *__pyx_ptype_8aimrocks_11lib_rocksdb_PyLRUCache = 0;
static PyTypeObject *__pyx_ptype_8aimrocks_11lib_rocksdb_PySliceTransform = 0;
static PyTypeObject *__pyx_ptype_8aimrocks_11lib_rocksdb_PyTableFactory = 0;
static PyTypeObject *__pyx_ptype_8aimrocks_11lib_rocksdb_BlockBasedTableFactory = 0;
static PyTypeObject *__pyx_ptype_8aimrocks_11lib_rocksdb_PlainTableFactory = 0;
static PyTypeObject *__pyx_ptype_8aimrocks_11lib_rocksdb_PyMemtableFactory = 0;
static PyTypeObject *__pyx_ptype_8aimrocks_11lib_rocksdb_SkipListMemtableFactory = 0;
static PyTypeObject *__pyx_ptype_8aimrocks_11lib_rocksdb_VectorMemtableFactory = 0;
static PyTypeObject *__pyx_ptype_8aimrocks_11lib_rocksdb_HashSkipListMemtableFactory = 0;
static PyTypeObject *__pyx_ptype_8aimrocks_11lib_rocksdb_HashLinkListMemtableFactory = 0;
static PyTypeObject *__pyx_ptype_8aimrocks_11lib_rocksdb_CompressionType = 0;
static PyTypeObject *__pyx_ptype_8aimrocks_11lib_rocksdb_CompactionPri = 0;
static PyTypeObject *__pyx_ptype_8aimrocks_11lib_rocksdb__ColumnFamilyHandle = 0;
static PyTypeObject *__pyx_ptype_8aimrocks_11lib_rocksdb_ColumnFamilyHandle = 0;
static PyTypeObject *__pyx_ptype_8aimrocks_11lib_rocksdb_ColumnFamilyOptions = 0;
static PyTypeObject *__pyx_ptype_8aimrocks_11lib_rocksdb_Options = 0;
static PyTypeObject *__pyx_ptype_8aimrocks_11lib_rocksdb_WriteBatch = 0;
static PyTypeObject *__pyx_ptype_8aimrocks_11lib_rocksdb_WriteBatchIterator = 0;
static PyTypeObject *__pyx_ptype_8aimrocks_11lib_rocksdb_IDB = 0;
static PyTypeObject *__pyx_ptype_8aimrocks_11lib_rocksdb_DB = 0;
static PyTypeObject *__pyx_ptype_8aimrocks_11lib_rocksdb_Snapshot = 0;
static PyTypeObject *__pyx_ptype_8aimrocks_11lib_rocksdb_Iterator = 0;
static PyTypeObject *__pyx_ptype_8aimrocks_11lib_rocksdb_BaseIterator = 0;
static PyTypeObject *__pyx_ptype_8aimrocks_11lib_rocksdb_KeysIterator = 0;
static PyTypeObject *__pyx_ptype_8aimrocks_11lib_rocksdb_ValuesIterator = 0;
static PyTypeObject *__pyx_ptype_8aimrocks_11lib_rocksdb_ItemsIterator = 0;
static PyTypeObject *__pyx_ptype_8aimrocks_11lib_rocksdb_ReversedIterator = 0;
static PyTypeObject *__pyx_ptype_8aimrocks_11lib_rocksdb_BackupEngine = 0;
#endif

/* Module declarations from "aim.storage.utils" */
#if !CYTHON_USE_MODULE_STATE
static PyTypeObject *__pyx_ptype_3aim_7storage_5utils_ArrayFlagType = 0;
static PyTypeObject *__pyx_ptype_3aim_7storage_5utils_ObjectFlagType = 0;
static PyTypeObject *__pyx_ptype_3aim_7storage_5utils_CustomObjectFlagType = 0;
static PyTypeObject *__pyx_ptype_3aim_7storage_5utils_BLOB = 0;
#endif

/* Module declarations from "aim.storage.container" */
#if !CYTHON_USE_MODULE_STATE
static PyTypeObject *__pyx_ptype_3aim_7storage_9container_ContainerItemsIterator = 0;
static PyTypeObject *__pyx_ptype_3aim_7storage_9container_Container = 0;
#endif

/* Module declarations from "aim.storage.treeview" */
#if !CYTHON_USE_MODULE_STATE
static PyTypeObject *__pyx_ptype_3aim_7storage_8treeview_TreeView = 0;
#endif

/* Module declarations from "libcpp.utility" */
#if !CYTHON_USE_MODULE_STATE
#endif

/* Module declarations from "libcpp.pair" */
#if !CYTHON_USE_MODULE_STATE
#endif

/* Module declarations from "aim.storage.encoding.encoding_native" */
#if !CYTHON_USE_MODULE_STATE
#endif

/* Module declarations from "aim.storage.encoding.encoding" */
#if !CYTHON_USE_MODULE_STATE
#endif
static PyObject *(*__pyx_f_3aim_7storage_8encoding_8encoding_decode)(PyObject *, int __pyx_skip_dispatch); /*proto*/

/* Module declarations from "aim.storage.encoding" */
#if !CYTHON_USE_MODULE_STATE
#endif

/* Module declarations from "aim.storage.containertreeview" */
#if !CYTHON_USE_MODULE_STATE
static PyTypeObject *__pyx_ptype_3aim_7storage_17containertreeview_ContainerTreeView = 0;
#endif

/* Module declarations from "aim.storage.prefixview" */
#if !CYTHON_USE_MODULE_STATE
static PyTypeObject *__pyx_ptype_3aim_7storage_10prefixview_PrefixView = 0;
static PyTypeObject *__pyx_ptype_3aim_7storage_10prefixview_PrefixViewItemsIterator = 0;
static PyTypeObject *__pyx_ptype_3aim_7storage_10prefixview___pyx_scope_struct__walk = 0;
#endif
static PyObject *__pyx_f_3aim_7storage_10prefixview___pyx_unpickle_PrefixView__set_state(struct __pyx_obj_3aim_7storage_10prefixview_PrefixView *, PyObject *); /*proto*/
static PyObject *__pyx_f_3aim_7storage_10prefixview___pyx_unpickle_PrefixViewItemsIterator__set_state(struct __pyx_obj_3aim_7storage_10prefixview_PrefixViewItemsIterator *, PyObject *); /*proto*/
/* #### Code section: typeinfo ### */
/* #### Code section: before_global_var ### */
#define __Pyx_MODULE_NAME "aim.storage.prefixview"
extern int __pyx_module_is_main_aim__storage__prefixview;
int __pyx_module_is_main_aim__storage__prefixview = 0;

/* Implementation of "aim.storage.prefixview" */
/* #### Code section: global_var ### */
static PyObject *__pyx_builtin_KeyError;
/* #### Code section: string_decls ### */
static const char __pyx_k_[] = "";
static const char __pyx_k_p[] = "p";
static const char __pyx_k__2[] = "\377";
static const char __pyx_k_gc[] = "gc";
static const char __pyx_k__54[] = "?";
static const char __pyx_k_end[] = "end";
static const char __pyx_k_get[] = "get";
static const char __pyx_k_key[] = "key";
static const char __pyx_k_new[] = "__new__";
static const char __pyx_k_set[] = "set";
static const char __pyx_k_args[] = "args";
static const char __pyx_k_dict[] = "__dict__";
static const char __pyx_k_main[] = "__main__";
static const char __pyx_k_name[] = "__name__";
static const char __pyx_k_next[] = "next";
static const char __pyx_k_path[] = "path";
static const char __pyx_k_self[] = "self";
static const char __pyx_k_send[] = "send";
static const char __pyx_k_test[] = "__test__";
static const char __pyx_k_tree[] = "tree";
static const char __pyx_k_view[] = "view";
static const char __pyx_k_walk[] = "walk";
static const char __pyx_k_Tuple[] = "Tuple";
static const char __pyx_k_batch[] = "batch";
static const char __pyx_k_begin[] = "begin";
static const char __pyx_k_bytes[] = "bytes";
static const char __pyx_k_close[] = "close";
static const char __pyx_k_index[] = "index";
static const char __pyx_k_items[] = "items";
static const char __pyx_k_state[] = "state";
static const char __pyx_k_throw[] = "throw";
static const char __pyx_k_value[] = "value";
static const char __pyx_k_commit[] = "commit";
static const char __pyx_k_delete[] = "delete";
static const char __pyx_k_dict_2[] = "_dict";
static const char __pyx_k_enable[] = "enable";
static const char __pyx_k_import[] = "__import__";
static const char __pyx_k_parent[] = "parent";
static const char __pyx_k_path_2[] = "_path";
static const char __pyx_k_pickle[] = "pickle";
static const char __pyx_k_prefix[] = "prefix";
static const char __pyx_k_reduce[] = "__reduce__";
static const char __pyx_k_return[] = "return";
static const char __pyx_k_typing[] = "typing";
static const char __pyx_k_update[] = "update";
static const char __pyx_k_walker[] = "walker";
static const char __pyx_k_default[] = "default";
static const char __pyx_k_disable[] = "disable";
static const char __pyx_k_preload[] = "preload";
static const char __pyx_k_Iterator[] = "Iterator";
static const char __pyx_k_KeyError[] = "KeyError";
static const char __pyx_k_finalize[] = "finalize";
static const char __pyx_k_getstate[] = "__getstate__";
static const char __pyx_k_next_key[] = "next_key";
static const char __pyx_k_prefix_2[] = "_prefix";
static const char __pyx_k_pyx_type[] = "__pyx_type";
static const char __pyx_k_setstate[] = "__setstate__";
static const char __pyx_k_Container[] = "Container";
static const char __pyx_k_container[] = "container";
static const char __pyx_k_isenabled[] = "isenabled";
static const char __pyx_k_next_item[] = "next_item";
static const char __pyx_k_partition[] = "partition";
static const char __pyx_k_prev_item[] = "prev_item";
static const char __pyx_k_pyx_state[] = "__pyx_state";
static const char __pyx_k_read_only[] = "read_only";
static const char __pyx_k_reduce_ex[] = "__reduce_ex__";
static const char __pyx_k_PrefixView[] = "PrefixView";
static const char __pyx_k_pyx_result[] = "__pyx_result";
static const char __pyx_k_pyx_vtable[] = "__pyx_vtable__";
static const char __pyx_k_PickleError[] = "PickleError";
static const char __pyx_k_prefix_view[] = "prefix_view";
static const char __pyx_k_store_batch[] = "store_batch";
static const char __pyx_k_ContainerKey[] = "ContainerKey";
static const char __pyx_k_delete_range[] = "delete_range";
static const char __pyx_k_is_coroutine[] = "_is_coroutine";
static const char __pyx_k_pyx_checksum[] = "__pyx_checksum";
static const char __pyx_k_stringsource[] = "stringsource";
static const char __pyx_k_use_setstate[] = "use_setstate";
static const char __pyx_k_absolute_path[] = "absolute_path";
static const char __pyx_k_class_getitem[] = "__class_getitem__";
static const char __pyx_k_reduce_cython[] = "__reduce_cython__";
static const char __pyx_k_ContainerValue[] = "ContainerValue";
static const char __pyx_k_PrefixView_get[] = "PrefixView.get";
static const char __pyx_k_PrefixView_set[] = "PrefixView.set";
static const char __pyx_k_PrefixView_tree[] = "PrefixView.tree";
static const char __pyx_k_PrefixView_view[] = "PrefixView.view";
static const char __pyx_k_PrefixView_walk[] = "PrefixView.walk";
static const char __pyx_k_pyx_PickleError[] = "__pyx_PickleError";
static const char __pyx_k_setstate_cython[] = "__setstate_cython__";
static const char __pyx_k_PrefixView_batch[] = "PrefixView.batch";
static const char __pyx_k_PrefixView_close[] = "PrefixView.close";
static const char __pyx_k_PrefixView_items[] = "PrefixView.items";
static const char __pyx_k_ContainerTreeView[] = "ContainerTreeView";
static const char __pyx_k_PrefixView_commit[] = "PrefixView.commit";
static const char __pyx_k_PrefixView_delete[] = "PrefixView.delete";
static const char __pyx_k_PrefixView_preload[] = "PrefixView.preload";
static const char __pyx_k_asyncio_coroutines[] = "asyncio.coroutines";
static const char __pyx_k_cline_in_traceback[] = "cline_in_traceback";
static const char __pyx_k_PrefixView_finalize[] = "PrefixView.finalize";
static const char __pyx_k_PrefixView_next_item[] = "PrefixView.next_item";
static const char __pyx_k_PrefixView_prev_item[] = "PrefixView.prev_item";
static const char __pyx_k_aim_storage_container[] = "aim.storage.container";
static const char __pyx_k_ContainerItemsIterator[] = "ContainerItemsIterator";
static const char __pyx_k_aim_storage_prefixview[] = "aim.storage.prefixview";
static const char __pyx_k_PrefixViewItemsIterator[] = "PrefixViewItemsIterator";
static const char __pyx_k_PrefixView_delete_range[] = "PrefixView.delete_range";
static const char __pyx_k_pyx_unpickle_PrefixView[] = "__pyx_unpickle_PrefixView";
static const char __pyx_k_PrefixView_absolute_path[] = "PrefixView.absolute_path";
static const char __pyx_k_aim_storage_prefixview_py[] = "aim/storage/prefixview.py";
static const char __pyx_k_PrefixView___reduce_cython[] = "PrefixView.__reduce_cython__";
static const char __pyx_k_PrefixViewItemsIterator_next[] = "PrefixViewItemsIterator.next";
static const char __pyx_k_PrefixView___setstate_cython[] = "PrefixView.__setstate_cython__";
static const char __pyx_k_aim_storage_containertreeview[] = "aim.storage.containertreeview";
static const char __pyx_k_pyx_unpickle_PrefixViewItemsIt[] = "__pyx_unpickle_PrefixViewItemsIterator";
static const char __pyx_k_Incompatible_checksums_s_vs_0x0d[] = "Incompatible checksums (%s vs 0x0d8b696 = (parent, prefix, read_only))";
static const char __pyx_k_Incompatible_checksums_s_vs_0xda[] = "Incompatible checksums (%s vs 0xda33529 = (_current_value, it, path, prefix_len, prefix_view))";
static const char __pyx_k_PrefixViewItemsIterator___reduce[] = "PrefixViewItemsIterator.__reduce_cython__";
static const char __pyx_k_PrefixViewItemsIterator___setsta[] = "PrefixViewItemsIterator.__setstate_cython__";
#if !CYTHON_USE_MODULE_STATE
static PyObject *__pyx_kp_b_;
static PyObject *__pyx_n_s_Container;
static PyObject *__pyx_n_s_ContainerItemsIterator;
static PyObject *__pyx_n_s_ContainerKey;
static PyObject *__pyx_n_s_ContainerTreeView;
static PyObject *__pyx_n_s_ContainerValue;
static PyObject *__pyx_kp_s_Incompatible_checksums_s_vs_0x0d;
static PyObject *__pyx_kp_s_Incompatible_checksums_s_vs_0xda;
static PyObject *__pyx_n_s_Iterator;
static PyObject *__pyx_n_s_KeyError;
static PyObject *__pyx_n_s_PickleError;
static PyObject *__pyx_n_s_PrefixView;
static PyObject *__pyx_n_s_PrefixViewItemsIterator;
static PyObject *__pyx_n_s_PrefixViewItemsIterator___reduce;
static PyObject *__pyx_n_s_PrefixViewItemsIterator___setsta;
static PyObject *__pyx_n_s_PrefixViewItemsIterator_next;
static PyObject *__pyx_n_s_PrefixView___reduce_cython;
static PyObject *__pyx_n_s_PrefixView___setstate_cython;
static PyObject *__pyx_n_s_PrefixView_absolute_path;
static PyObject *__pyx_n_s_PrefixView_batch;
static PyObject *__pyx_n_s_PrefixView_close;
static PyObject *__pyx_n_s_PrefixView_commit;
static PyObject *__pyx_n_s_PrefixView_delete;
static PyObject *__pyx_n_s_PrefixView_delete_range;
static PyObject *__pyx_n_s_PrefixView_finalize;
static PyObject *__pyx_n_s_PrefixView_get;
static PyObject *__pyx_n_s_PrefixView_items;
static PyObject *__pyx_n_s_PrefixView_next_item;
static PyObject *__pyx_n_s_PrefixView_preload;
static PyObject *__pyx_n_s_PrefixView_prev_item;
static PyObject *__pyx_n_s_PrefixView_set;
static PyObject *__pyx_n_s_PrefixView_tree;
static PyObject *__pyx_n_s_PrefixView_view;
static PyObject *__pyx_n_s_PrefixView_walk;
static PyObject *__pyx_n_s_Tuple;
static PyObject *__pyx_kp_b__2;
static PyObject *__pyx_n_s__54;
static PyObject *__pyx_n_s_absolute_path;
static PyObject *__pyx_n_s_aim_storage_container;
static PyObject *__pyx_n_s_aim_storage_containertreeview;
static PyObject *__pyx_n_s_aim_storage_prefixview;
static PyObject *__pyx_kp_s_aim_storage_prefixview_py;
static PyObject *__pyx_n_s_args;
static PyObject *__pyx_n_s_asyncio_coroutines;
static PyObject *__pyx_n_s_batch;
static PyObject *__pyx_n_s_begin;
static PyObject *__pyx_n_s_bytes;
static PyObject *__pyx_n_s_class_getitem;
static PyObject *__pyx_n_s_cline_in_traceback;
static PyObject *__pyx_n_s_close;
static PyObject *__pyx_n_s_commit;
static PyObject *__pyx_n_s_container;
static PyObject *__pyx_n_s_default;
static PyObject *__pyx_n_s_delete;
static PyObject *__pyx_n_s_delete_range;
static PyObject *__pyx_n_s_dict;
static PyObject *__pyx_n_s_dict_2;
static PyObject *__pyx_kp_u_disable;
static PyObject *__pyx_kp_u_enable;
static PyObject *__pyx_n_s_end;
static PyObject *__pyx_n_s_finalize;
static PyObject *__pyx_kp_u_gc;
static PyObject *__pyx_n_s_get;
static PyObject *__pyx_n_s_getstate;
static PyObject *__pyx_n_s_import;
static PyObject *__pyx_n_s_index;
static PyObject *__pyx_n_s_is_coroutine;
static PyObject *__pyx_kp_u_isenabled;
static PyObject *__pyx_n_s_items;
static PyObject *__pyx_n_s_key;
static PyObject *__pyx_n_s_main;
static PyObject *__pyx_n_s_name;
static PyObject *__pyx_n_s_new;
static PyObject *__pyx_n_s_next;
static PyObject *__pyx_n_s_next_item;
static PyObject *__pyx_n_s_next_key;
static PyObject *__pyx_n_s_p;
static PyObject *__pyx_n_s_parent;
static PyObject *__pyx_n_s_partition;
static PyObject *__pyx_n_s_path;
static PyObject *__pyx_n_s_path_2;
static PyObject *__pyx_n_s_pickle;
static PyObject *__pyx_n_s_prefix;
static PyObject *__pyx_n_s_prefix_2;
static PyObject *__pyx_n_s_prefix_view;
static PyObject *__pyx_n_s_preload;
static PyObject *__pyx_n_s_prev_item;
static PyObject *__pyx_n_s_pyx_PickleError;
static PyObject *__pyx_n_s_pyx_checksum;
static PyObject *__pyx_n_s_pyx_result;
static PyObject *__pyx_n_s_pyx_state;
static PyObject *__pyx_n_s_pyx_type;
static PyObject *__pyx_n_s_pyx_unpickle_PrefixView;
static PyObject *__pyx_n_s_pyx_unpickle_PrefixViewItemsIt;
static PyObject *__pyx_n_s_pyx_vtable;
static PyObject *__pyx_n_s_read_only;
static PyObject *__pyx_n_s_reduce;
static PyObject *__pyx_n_s_reduce_cython;
static PyObject *__pyx_n_s_reduce_ex;
static PyObject *__pyx_n_s_return;
static PyObject *__pyx_n_s_self;
static PyObject *__pyx_n_s_send;
static PyObject *__pyx_n_s_set;
static PyObject *__pyx_n_s_setstate;
static PyObject *__pyx_n_s_setstate_cython;
static PyObject *__pyx_n_s_state;
static PyObject *__pyx_n_s_store_batch;
static PyObject *__pyx_kp_s_stringsource;
static PyObject *__pyx_n_s_test;
static PyObject *__pyx_n_s_throw;
static PyObject *__pyx_n_s_tree;
static PyObject *__pyx_n_s_typing;
static PyObject *__pyx_n_s_update;
static PyObject *__pyx_n_s_use_setstate;
static PyObject *__pyx_n_s_value;
static PyObject *__pyx_n_s_view;
static PyObject *__pyx_n_s_walk;
static PyObject *__pyx_n_s_walker;
#endif
/* #### Code section: decls ### */
static int __pyx_pf_3aim_7storage_10prefixview_10PrefixView___init__(struct __pyx_obj_3aim_7storage_10prefixview_PrefixView *__pyx_v_self, PyObject *__pyx_v_prefix, struct __pyx_obj_3aim_7storage_9container_Container *__pyx_v_container, PyObject *__pyx_v_read_only); /* proto */
static PyObject *__pyx_pf_3aim_7storage_10prefixview_10PrefixView_2close(struct __pyx_obj_3aim_7storage_10prefixview_PrefixView *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_3aim_7storage_10prefixview_10PrefixView_4preload(struct __pyx_obj_3aim_7storage_10prefixview_PrefixView *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_3aim_7storage_10prefixview_10PrefixView_6finalize(struct __pyx_obj_3aim_7storage_10prefixview_PrefixView *__pyx_v_self, struct __pyx_obj_3aim_7storage_9container_Container *__pyx_v_index); /* proto */
static PyObject *__pyx_pf_3aim_7storage_10prefixview_10PrefixView_8absolute_path(struct __pyx_obj_3aim_7storage_10prefixview_PrefixView *__pyx_v_self, PyObject *__pyx_v_path); /* proto */
static PyObject *__pyx_pf_3aim_7storage_10prefixview_10PrefixView_10get(struct __pyx_obj_3aim_7storage_10prefixview_PrefixView *__pyx_v_self, PyObject *__pyx_v_key, PyObject *__pyx_v_default); /* proto */
static PyObject *__pyx_pf_3aim_7storage_10prefixview_10PrefixView_12__getitem__(struct __pyx_obj_3aim_7storage_10prefixview_PrefixView *__pyx_v_self, PyObject *__pyx_v_key); /* proto */
static PyObject *__pyx_pf_3aim_7storage_10prefixview_10PrefixView_14set(struct __pyx_obj_3aim_7storage_10prefixview_PrefixView *__pyx_v_self, PyObject *__pyx_v_key, PyObject *__pyx_v_value, PyObject *__pyx_v_store_batch); /* proto */
static int __pyx_pf_3aim_7storage_10prefixview_10PrefixView_16__setitem__(struct __pyx_obj_3aim_7storage_10prefixview_PrefixView *__pyx_v_self, PyObject *__pyx_v_key, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_3aim_7storage_10prefixview_10PrefixView_18delete(struct __pyx_obj_3aim_7storage_10prefixview_PrefixView *__pyx_v_self, PyObject *__pyx_v_key, PyObject *__pyx_v_store_batch); /* proto */
static int __pyx_pf_3aim_7storage_10prefixview_10PrefixView_20__delitem__(struct __pyx_obj_3aim_7storage_10prefixview_PrefixView *__pyx_v_self, PyObject *__pyx_v_key); /* proto */
static PyObject *__pyx_pf_3aim_7storage_10prefixview_10PrefixView_22delete_range(struct __pyx_obj_3aim_7storage_10prefixview_PrefixView *__pyx_v_self, PyObject *__pyx_v_begin, PyObject *__pyx_v_end, PyObject *__pyx_v_store_batch); /* proto */
static PyObject *__pyx_pf_3aim_7storage_10prefixview_10PrefixView_24next_item(struct __pyx_obj_3aim_7storage_10prefixview_PrefixView *__pyx_v_self, PyObject *__pyx_v_key); /* proto */
static PyObject *__pyx_pf_3aim_7storage_10prefixview_10PrefixView_26prev_item(struct __pyx_obj_3aim_7storage_10prefixview_PrefixView *__pyx_v_self, PyObject *__pyx_v_key); /* proto */
static PyObject *__pyx_pf_3aim_7storage_10prefixview_10PrefixView_28walk(struct __pyx_obj_3aim_7storage_10prefixview_PrefixView *__pyx_v_self, PyObject *__pyx_v_key); /* proto */
static PyObject *__pyx_pf_3aim_7storage_10prefixview_10PrefixView_31items(struct __pyx_obj_3aim_7storage_10prefixview_PrefixView *__pyx_v_self, PyObject *__pyx_v_key); /* proto */
static struct __pyx_obj_3aim_7storage_9container_Container *__pyx_pf_3aim_7storage_10prefixview_10PrefixView_33view(struct __pyx_obj_3aim_7storage_10prefixview_PrefixView *__pyx_v_self, PyObject *__pyx_v_prefix); /* proto */
static PyObject *__pyx_pf_3aim_7storage_10prefixview_10PrefixView_35tree(struct __pyx_obj_3aim_7storage_10prefixview_PrefixView *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_3aim_7storage_10prefixview_10PrefixView_37batch(struct __pyx_obj_3aim_7storage_10prefixview_PrefixView *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_3aim_7storage_10prefixview_10PrefixView_39commit(struct __pyx_obj_3aim_7storage_10prefixview_PrefixView *__pyx_v_self, PyObject *__pyx_v_batch); /* proto */
static PyObject *__pyx_pf_3aim_7storage_10prefixview_10PrefixView_6prefix___get__(struct __pyx_obj_3aim_7storage_10prefixview_PrefixView *__pyx_v_self); /* proto */
static int __pyx_pf_3aim_7storage_10prefixview_10PrefixView_6prefix_2__set__(struct __pyx_obj_3aim_7storage_10prefixview_PrefixView *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_3aim_7storage_10prefixview_10PrefixView_6prefix_4__del__(struct __pyx_obj_3aim_7storage_10prefixview_PrefixView *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_3aim_7storage_10prefixview_10PrefixView_6parent___get__(struct __pyx_obj_3aim_7storage_10prefixview_PrefixView *__pyx_v_self); /* proto */
static int __pyx_pf_3aim_7storage_10prefixview_10PrefixView_6parent_2__set__(struct __pyx_obj_3aim_7storage_10prefixview_PrefixView *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_3aim_7storage_10prefixview_10PrefixView_6parent_4__del__(struct __pyx_obj_3aim_7storage_10prefixview_PrefixView *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_3aim_7storage_10prefixview_10PrefixView_9read_only___get__(struct __pyx_obj_3aim_7storage_10prefixview_PrefixView *__pyx_v_self); /* proto */
static int __pyx_pf_3aim_7storage_10prefixview_10PrefixView_9read_only_2__set__(struct __pyx_obj_3aim_7storage_10prefixview_PrefixView *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_3aim_7storage_10prefixview_10PrefixView_41__reduce_cython__(struct __pyx_obj_3aim_7storage_10prefixview_PrefixView *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_3aim_7storage_10prefixview_10PrefixView_43__setstate_cython__(struct __pyx_obj_3aim_7storage_10prefixview_PrefixView *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_3aim_7storage_10prefixview_23PrefixViewItemsIterator___init__(struct __pyx_obj_3aim_7storage_10prefixview_PrefixViewItemsIterator *__pyx_v_self, PyObject *__pyx_v_prefix_view, PyObject *__pyx_v_key); /* proto */
static PyObject *__pyx_pf_3aim_7storage_10prefixview_23PrefixViewItemsIterator_2next(struct __pyx_obj_3aim_7storage_10prefixview_PrefixViewItemsIterator *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_3aim_7storage_10prefixview_23PrefixViewItemsIterator_4__reduce_cython__(struct __pyx_obj_3aim_7storage_10prefixview_PrefixViewItemsIterator *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_3aim_7storage_10prefixview_23PrefixViewItemsIterator_6__setstate_cython__(struct __pyx_obj_3aim_7storage_10prefixview_PrefixViewItemsIterator *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_3aim_7storage_10prefixview___pyx_unpickle_PrefixView(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_3aim_7storage_10prefixview_2__pyx_unpickle_PrefixViewItemsIterator(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_tp_new_3aim_7storage_10prefixview_PrefixView(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_3aim_7storage_10prefixview_PrefixViewItemsIterator(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_3aim_7storage_10prefixview___pyx_scope_struct__walk(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
#if !CYTHON_USE_MODULE_STATE
static PyObject *__pyx_int_14202518;
static PyObject *__pyx_int_228799785;
#endif
#if !CYTHON_USE_MODULE_STATE
static PyObject *__pyx_tuple__4;
static PyObject *__pyx_tuple__6;
static PyObject *__pyx_tuple__8;
static PyObject *__pyx_tuple__10;
static PyObject *__pyx_tuple__12;
static PyObject *__pyx_tuple__13;
static PyObject *__pyx_tuple__15;
static PyObject *__pyx_tuple__17;
static PyObject *__pyx_tuple__19;
static PyObject *__pyx_tuple__21;
static PyObject *__pyx_tuple__23;
static PyObject *__pyx_tuple__24;
static PyObject *__pyx_tuple__26;
static PyObject *__pyx_tuple__27;
static PyObject *__pyx_tuple__28;
static PyObject *__pyx_tuple__29;
static PyObject *__pyx_tuple__31;
static PyObject *__pyx_tuple__32;
static PyObject *__pyx_tuple__34;
static PyObject *__pyx_tuple__36;
static PyObject *__pyx_tuple__38;
static PyObject *__pyx_tuple__40;
static PyObject *__pyx_tuple__42;
static PyObject *__pyx_tuple__44;
static PyObject *__pyx_tuple__46;
static PyObject *__pyx_tuple__48;
static PyObject *__pyx_tuple__50;
static PyObject *__pyx_tuple__52;
static PyObject *__pyx_codeobj__3;
static PyObject *__pyx_codeobj__5;
static PyObject *__pyx_codeobj__7;
static PyObject *__pyx_codeobj__9;
static PyObject *__pyx_codeobj__11;
static PyObject *__pyx_codeobj__14;
static PyObject *__pyx_codeobj__16;
static PyObject *__pyx_codeobj__18;
static PyObject *__pyx_codeobj__20;
static PyObject *__pyx_codeobj__22;
static PyObject *__pyx_codeobj__25;
static PyObject *__pyx_codeobj__30;
static PyObject *__pyx_codeobj__33;
static PyObject *__pyx_codeobj__35;
static PyObject *__pyx_codeobj__37;
static PyObject *__pyx_codeobj__39;
static PyObject *__pyx_codeobj__41;
static PyObject *__pyx_codeobj__43;
static PyObject *__pyx_codeobj__45;
static PyObject *__pyx_codeobj__47;
static PyObject *__pyx_codeobj__49;
static PyObject *__pyx_codeobj__51;
static PyObject *__pyx_codeobj__53;
#endif
/* #### Code section: late_includes ### */
/* #### Code section: module_state ### */
#if CYTHON_USE_MODULE_STATE
typedef struct {
  PyObject *__pyx_d;
  PyObject *__pyx_b;
  PyObject *__pyx_cython_runtime;
  PyObject *__pyx_empty_tuple;
  PyObject *__pyx_empty_bytes;
  PyObject *__pyx_empty_unicode;
  #ifdef __Pyx_CyFunction_USED
  PyTypeObject *__pyx_CyFunctionType;
  #endif
  #ifdef __Pyx_FusedFunction_USED
  PyTypeObject *__pyx_FusedFunctionType;
  #endif
  PyTypeObject *__pyx_ptype_7cpython_4type_type;
  PyTypeObject *__pyx_ptype_7cpython_4bool_bool;
  PyTypeObject *__pyx_ptype_7cpython_7complex_complex;
  PyTypeObject *__pyx_ptype_8aimrocks_11lib_rocksdb_PyComparator;
  PyTypeObject *__pyx_ptype_8aimrocks_11lib_rocksdb_PyGenericComparator;
  PyTypeObject *__pyx_ptype_8aimrocks_11lib_rocksdb_PyBytewiseComparator;
  PyTypeObject *__pyx_ptype_8aimrocks_11lib_rocksdb_PyFilterPolicy;
  PyTypeObject *__pyx_ptype_8aimrocks_11lib_rocksdb_PyGenericFilterPolicy;
  PyTypeObject *__pyx_ptype_8aimrocks_11lib_rocksdb_PyBloomFilterPolicy;
  PyTypeObject *__pyx_ptype_8aimrocks_11lib_rocksdb_PyMergeOperator;
  PyTypeObject *__pyx_ptype_8aimrocks_11lib_rocksdb_PyCache;
  PyTypeObject *__pyx_ptype_8aimrocks_11lib_rocksdb_PyLRUCache;
  PyTypeObject *__pyx_ptype_8aimrocks_11lib_rocksdb_PySliceTransform;
  PyTypeObject *__pyx_ptype_8aimrocks_11lib_rocksdb_PyTableFactory;
  PyTypeObject *__pyx_ptype_8aimrocks_11lib_rocksdb_BlockBasedTableFactory;
  PyTypeObject *__pyx_ptype_8aimrocks_11lib_rocksdb_PlainTableFactory;
  PyTypeObject *__pyx_ptype_8aimrocks_11lib_rocksdb_PyMemtableFactory;
  PyTypeObject *__pyx_ptype_8aimrocks_11lib_rocksdb_SkipListMemtableFactory;
  PyTypeObject *__pyx_ptype_8aimrocks_11lib_rocksdb_VectorMemtableFactory;
  PyTypeObject *__pyx_ptype_8aimrocks_11lib_rocksdb_HashSkipListMemtableFactory;
  PyTypeObject *__pyx_ptype_8aimrocks_11lib_rocksdb_HashLinkListMemtableFactory;
  PyTypeObject *__pyx_ptype_8aimrocks_11lib_rocksdb_CompressionType;
  PyTypeObject *__pyx_ptype_8aimrocks_11lib_rocksdb_CompactionPri;
  PyTypeObject *__pyx_ptype_8aimrocks_11lib_rocksdb__ColumnFamilyHandle;
  PyTypeObject *__pyx_ptype_8aimrocks_11lib_rocksdb_ColumnFamilyHandle;
  PyTypeObject *__pyx_ptype_8aimrocks_11lib_rocksdb_ColumnFamilyOptions;
  PyTypeObject *__pyx_ptype_8aimrocks_11lib_rocksdb_Options;
  PyTypeObject *__pyx_ptype_8aimrocks_11lib_rocksdb_WriteBatch;
  PyTypeObject *__pyx_ptype_8aimrocks_11lib_rocksdb_WriteBatchIterator;
  PyTypeObject *__pyx_ptype_8aimrocks_11lib_rocksdb_IDB;
  PyTypeObject *__pyx_ptype_8aimrocks_11lib_rocksdb_DB;
  PyTypeObject *__pyx_ptype_8aimrocks_11lib_rocksdb_Snapshot;
  PyTypeObject *__pyx_ptype_8aimrocks_11lib_rocksdb_Iterator;
  PyTypeObject *__pyx_ptype_8aimrocks_11lib_rocksdb_BaseIterator;
  PyTypeObject *__pyx_ptype_8aimrocks_11lib_rocksdb_KeysIterator;
  PyTypeObject *__pyx_ptype_8aimrocks_11lib_rocksdb_ValuesIterator;
  PyTypeObject *__pyx_ptype_8aimrocks_11lib_rocksdb_ItemsIterator;
  PyTypeObject *__pyx_ptype_8aimrocks_11lib_rocksdb_ReversedIterator;
  PyTypeObject *__pyx_ptype_8aimrocks_11lib_rocksdb_BackupEngine;
  PyTypeObject *__pyx_ptype_3aim_7storage_5utils_ArrayFlagType;
  PyTypeObject *__pyx_ptype_3aim_7storage_5utils_ObjectFlagType;
  PyTypeObject *__pyx_ptype_3aim_7storage_5utils_CustomObjectFlagType;
  PyTypeObject *__pyx_ptype_3aim_7storage_5utils_BLOB;
  PyTypeObject *__pyx_ptype_3aim_7storage_9container_ContainerItemsIterator;
  PyTypeObject *__pyx_ptype_3aim_7storage_9container_Container;
  PyTypeObject *__pyx_ptype_3aim_7storage_8treeview_TreeView;
  PyTypeObject *__pyx_ptype_3aim_7storage_17containertreeview_ContainerTreeView;
  PyTypeObject *__pyx_ptype_3aim_7storage_10prefixview_PrefixView;
  PyObject *__pyx_type_3aim_7storage_10prefixview_PrefixView;
  PyTypeObject *__pyx_ptype_3aim_7storage_10prefixview_PrefixViewItemsIterator;
  PyObject *__pyx_type_3aim_7storage_10prefixview_PrefixViewItemsIterator;
  PyTypeObject *__pyx_ptype_3aim_7storage_10prefixview___pyx_scope_struct__walk;
  PyObject *__pyx_type_3aim_7storage_10prefixview___pyx_scope_struct__walk;
  PyObject *__pyx_kp_b_;
  PyObject *__pyx_n_s_Container;
  PyObject *__pyx_n_s_ContainerItemsIterator;
  PyObject *__pyx_n_s_ContainerKey;
  PyObject *__pyx_n_s_ContainerTreeView;
  PyObject *__pyx_n_s_ContainerValue;
  PyObject *__pyx_kp_s_Incompatible_checksums_s_vs_0x0d;
  PyObject *__pyx_kp_s_Incompatible_checksums_s_vs_0xda;
  PyObject *__pyx_n_s_Iterator;
  PyObject *__pyx_n_s_KeyError;
  PyObject *__pyx_n_s_PickleError;
  PyObject *__pyx_n_s_PrefixView;
  PyObject *__pyx_n_s_PrefixViewItemsIterator;
  PyObject *__pyx_n_s_PrefixViewItemsIterator___reduce;
  PyObject *__pyx_n_s_PrefixViewItemsIterator___setsta;
  PyObject *__pyx_n_s_PrefixViewItemsIterator_next;
  PyObject *__pyx_n_s_PrefixView___reduce_cython;
  PyObject *__pyx_n_s_PrefixView___setstate_cython;
  PyObject *__pyx_n_s_PrefixView_absolute_path;
  PyObject *__pyx_n_s_PrefixView_batch;
  PyObject *__pyx_n_s_PrefixView_close;
  PyObject *__pyx_n_s_PrefixView_commit;
  PyObject *__pyx_n_s_PrefixView_delete;
  PyObject *__pyx_n_s_PrefixView_delete_range;
  PyObject *__pyx_n_s_PrefixView_finalize;
  PyObject *__pyx_n_s_PrefixView_get;
  PyObject *__pyx_n_s_PrefixView_items;
  PyObject *__pyx_n_s_PrefixView_next_item;
  PyObject *__pyx_n_s_PrefixView_preload;
  PyObject *__pyx_n_s_PrefixView_prev_item;
  PyObject *__pyx_n_s_PrefixView_set;
  PyObject *__pyx_n_s_PrefixView_tree;
  PyObject *__pyx_n_s_PrefixView_view;
  PyObject *__pyx_n_s_PrefixView_walk;
  PyObject *__pyx_n_s_Tuple;
  PyObject *__pyx_kp_b__2;
  PyObject *__pyx_n_s__54;
  PyObject *__pyx_n_s_absolute_path;
  PyObject *__pyx_n_s_aim_storage_container;
  PyObject *__pyx_n_s_aim_storage_containertreeview;
  PyObject *__pyx_n_s_aim_storage_prefixview;
  PyObject *__pyx_kp_s_aim_storage_prefixview_py;
  PyObject *__pyx_n_s_args;
  PyObject *__pyx_n_s_asyncio_coroutines;
  PyObject *__pyx_n_s_batch;
  PyObject *__pyx_n_s_begin;
  PyObject *__pyx_n_s_bytes;
  PyObject *__pyx_n_s_class_getitem;
  PyObject *__pyx_n_s_cline_in_traceback;
  PyObject *__pyx_n_s_close;
  PyObject *__pyx_n_s_commit;
  PyObject *__pyx_n_s_container;
  PyObject *__pyx_n_s_default;
  PyObject *__pyx_n_s_delete;
  PyObject *__pyx_n_s_delete_range;
  PyObject *__pyx_n_s_dict;
  PyObject *__pyx_n_s_dict_2;
  PyObject *__pyx_kp_u_disable;
  PyObject *__pyx_kp_u_enable;
  PyObject *__pyx_n_s_end;
  PyObject *__pyx_n_s_finalize;
  PyObject *__pyx_kp_u_gc;
  PyObject *__pyx_n_s_get;
  PyObject *__pyx_n_s_getstate;
  PyObject *__pyx_n_s_import;
  PyObject *__pyx_n_s_index;
  PyObject *__pyx_n_s_is_coroutine;
  PyObject *__pyx_kp_u_isenabled;
  PyObject *__pyx_n_s_items;
  PyObject *__pyx_n_s_key;
  PyObject *__pyx_n_s_main;
  PyObject *__pyx_n_s_name;
  PyObject *__pyx_n_s_new;
  PyObject *__pyx_n_s_next;
  PyObject *__pyx_n_s_next_item;
  PyObject *__pyx_n_s_next_key;
  PyObject *__pyx_n_s_p;
  PyObject *__pyx_n_s_parent;
  PyObject *__pyx_n_s_partition;
  PyObject *__pyx_n_s_path;
  PyObject *__pyx_n_s_path_2;
  PyObject *__pyx_n_s_pickle;
  PyObject *__pyx_n_s_prefix;
  PyObject *__pyx_n_s_prefix_2;
  PyObject *__pyx_n_s_prefix_view;
  PyObject *__pyx_n_s_preload;
  PyObject *__pyx_n_s_prev_item;
  PyObject *__pyx_n_s_pyx_PickleError;
  PyObject *__pyx_n_s_pyx_checksum;
  PyObject *__pyx_n_s_pyx_result;
  PyObject *__pyx_n_s_pyx_state;
  PyObject *__pyx_n_s_pyx_type;
  PyObject *__pyx_n_s_pyx_unpickle_PrefixView;
  PyObject *__pyx_n_s_pyx_unpickle_PrefixViewItemsIt;
  PyObject *__pyx_n_s_pyx_vtable;
  PyObject *__pyx_n_s_read_only;
  PyObject *__pyx_n_s_reduce;
  PyObject *__pyx_n_s_reduce_cython;
  PyObject *__pyx_n_s_reduce_ex;
  PyObject *__pyx_n_s_return;
  PyObject *__pyx_n_s_self;
  PyObject *__pyx_n_s_send;
  PyObject *__pyx_n_s_set;
  PyObject *__pyx_n_s_setstate;
  PyObject *__pyx_n_s_setstate_cython;
  PyObject *__pyx_n_s_state;
  PyObject *__pyx_n_s_store_batch;
  PyObject *__pyx_kp_s_stringsource;
  PyObject *__pyx_n_s_test;
  PyObject *__pyx_n_s_throw;
  PyObject *__pyx_n_s_tree;
  PyObject *__pyx_n_s_typing;
  PyObject *__pyx_n_s_update;
  PyObject *__pyx_n_s_use_setstate;
  PyObject *__pyx_n_s_value;
  PyObject *__pyx_n_s_view;
  PyObject *__pyx_n_s_walk;
  PyObject *__pyx_n_s_walker;
  PyObject *__pyx_int_14202518;
  PyObject *__pyx_int_228799785;
  PyObject *__pyx_tuple__4;
  PyObject *__pyx_tuple__6;
  PyObject *__pyx_tuple__8;
  PyObject *__pyx_tuple__10;
  PyObject *__pyx_tuple__12;
  PyObject *__pyx_tuple__13;
  PyObject *__pyx_tuple__15;
  PyObject *__pyx_tuple__17;
  PyObject *__pyx_tuple__19;
  PyObject *__pyx_tuple__21;
  PyObject *__pyx_tuple__23;
  PyObject *__pyx_tuple__24;
  PyObject *__pyx_tuple__26;
  PyObject *__pyx_tuple__27;
  PyObject *__pyx_tuple__28;
  PyObject *__pyx_tuple__29;
  PyObject *__pyx_tuple__31;
  PyObject *__pyx_tuple__32;
  PyObject *__pyx_tuple__34;
  PyObject *__pyx_tuple__36;
  PyObject *__pyx_tuple__38;
  PyObject *__pyx_tuple__40;
  PyObject *__pyx_tuple__42;
  PyObject *__pyx_tuple__44;
  PyObject *__pyx_tuple__46;
  PyObject *__pyx_tuple__48;
  PyObject *__pyx_tuple__50;
  PyObject *__pyx_tuple__52;
  PyObject *__pyx_codeobj__3;
  PyObject *__pyx_codeobj__5;
  PyObject *__pyx_codeobj__7;
  PyObject *__pyx_codeobj__9;
  PyObject *__pyx_codeobj__11;
  PyObject *__pyx_codeobj__14;
  PyObject *__pyx_codeobj__16;
  PyObject *__pyx_codeobj__18;
  PyObject *__pyx_codeobj__20;
  PyObject *__pyx_codeobj__22;
  PyObject *__pyx_codeobj__25;
  PyObject *__pyx_codeobj__30;
  PyObject *__pyx_codeobj__33;
  PyObject *__pyx_codeobj__35;
  PyObject *__pyx_codeobj__37;
  PyObject *__pyx_codeobj__39;
  PyObject *__pyx_codeobj__41;
  PyObject *__pyx_codeobj__43;
  PyObject *__pyx_codeobj__45;
  PyObject *__pyx_codeobj__47;
  PyObject *__pyx_codeobj__49;
  PyObject *__pyx_codeobj__51;
  PyObject *__pyx_codeobj__53;
} __pyx_mstate;

#ifdef __cplusplus
namespace {
  extern struct PyModuleDef __pyx_moduledef;
} /* anonymous namespace */
#else
static struct PyModuleDef __pyx_moduledef;
#endif

#define __pyx_mstate(o) ((__pyx_mstate *)__Pyx_PyModule_GetState(o))

#define __pyx_mstate_global (__pyx_mstate(PyState_FindModule(&__pyx_moduledef)))

#define __pyx_m (PyState_FindModule(&__pyx_moduledef))
#endif
/* #### Code section: module_state_clear ### */
#if CYTHON_USE_MODULE_STATE
static int __pyx_m_clear(PyObject *m) {
  __pyx_mstate *clear_module_state = __pyx_mstate(m);
  if (!clear_module_state) return 0;
  Py_CLEAR(clear_module_state->__pyx_d);
  Py_CLEAR(clear_module_state->__pyx_b);
  Py_CLEAR(clear_module_state->__pyx_cython_runtime);
  Py_CLEAR(clear_module_state->__pyx_empty_tuple);
  Py_CLEAR(clear_module_state->__pyx_empty_bytes);
  Py_CLEAR(clear_module_state->__pyx_empty_unicode);
  #ifdef __Pyx_CyFunction_USED
  Py_CLEAR(clear_module_state->__pyx_CyFunctionType);
  #endif
  #ifdef __Pyx_FusedFunction_USED
  Py_CLEAR(clear_module_state->__pyx_FusedFunctionType);
  #endif
  Py_CLEAR(clear_module_state->__pyx_ptype_7cpython_4type_type);
  Py_CLEAR(clear_module_state->__pyx_ptype_7cpython_4bool_bool);
  Py_CLEAR(clear_module_state->__pyx_ptype_7cpython_7complex_complex);
  Py_CLEAR(clear_module_state->__pyx_ptype_8aimrocks_11lib_rocksdb_PyComparator);
  Py_CLEAR(clear_module_state->__pyx_ptype_8aimrocks_11lib_rocksdb_PyGenericComparator);
  Py_CLEAR(clear_module_state->__pyx_ptype_8aimrocks_11lib_rocksdb_PyBytewiseComparator);
  Py_CLEAR(clear_module_state->__pyx_ptype_8aimrocks_11lib_rocksdb_PyFilterPolicy);
  Py_CLEAR(clear_module_state->__pyx_ptype_8aimrocks_11lib_rocksdb_PyGenericFilterPolicy);
  Py_CLEAR(clear_module_state->__pyx_ptype_8aimrocks_11lib_rocksdb_PyBloomFilterPolicy);
  Py_CLEAR(clear_module_state->__pyx_ptype_8aimrocks_11lib_rocksdb_PyMergeOperator);
  Py_CLEAR(clear_module_state->__pyx_ptype_8aimrocks_11lib_rocksdb_PyCache);
  Py_CLEAR(clear_module_state->__pyx_ptype_8aimrocks_11lib_rocksdb_PyLRUCache);
  Py_CLEAR(clear_module_state->__pyx_ptype_8aimrocks_11lib_rocksdb_PySliceTransform);
  Py_CLEAR(clear_module_state->__pyx_ptype_8aimrocks_11lib_rocksdb_PyTableFactory);
  Py_CLEAR(clear_module_state->__pyx_ptype_8aimrocks_11lib_rocksdb_BlockBasedTableFactory);
  Py_CLEAR(clear_module_state->__pyx_ptype_8aimrocks_11lib_rocksdb_PlainTableFactory);
  Py_CLEAR(clear_module_state->__pyx_ptype_8aimrocks_11lib_rocksdb_PyMemtableFactory);
  Py_CLEAR(clear_module_state->__pyx_ptype_8aimrocks_11lib_rocksdb_SkipListMemtableFactory);
  Py_CLEAR(clear_module_state->__pyx_ptype_8aimrocks_11lib_rocksdb_VectorMemtableFactory);
  Py_CLEAR(clear_module_state->__pyx_ptype_8aimrocks_11lib_rocksdb_HashSkipListMemtableFactory);
  Py_CLEAR(clear_module_state->__pyx_ptype_8aimrocks_11lib_rocksdb_HashLinkListMemtableFactory);
  Py_CLEAR(clear_module_state->__pyx_ptype_8aimrocks_11lib_rocksdb_CompressionType);
  Py_CLEAR(clear_module_state->__pyx_ptype_8aimrocks_11lib_rocksdb_CompactionPri);
  Py_CLEAR(clear_module_state->__pyx_ptype_8aimrocks_11lib_rocksdb__ColumnFamilyHandle);
  Py_CLEAR(clear_module_state->__pyx_ptype_8aimrocks_11lib_rocksdb_ColumnFamilyHandle);
  Py_CLEAR(clear_module_state->__pyx_ptype_8aimrocks_11lib_rocksdb_ColumnFamilyOptions);
  Py_CLEAR(clear_module_state->__pyx_ptype_8aimrocks_11lib_rocksdb_Options);
  Py_CLEAR(clear_module_state->__pyx_ptype_8aimrocks_11lib_rocksdb_WriteBatch);
  Py_CLEAR(clear_module_state->__pyx_ptype_8aimrocks_11lib_rocksdb_WriteBatchIterator);
  Py_CLEAR(clear_module_state->__pyx_ptype_8aimrocks_11lib_rocksdb_IDB);
  Py_CLEAR(clear_module_state->__pyx_ptype_8aimrocks_11lib_rocksdb_DB);
  Py_CLEAR(clear_module_state->__pyx_ptype_8aimrocks_11lib_rocksdb_Snapshot);
  Py_CLEAR(clear_module_state->__pyx_ptype_8aimrocks_11lib_rocksdb_Iterator);
  Py_CLEAR(clear_module_state->__pyx_ptype_8aimrocks_11lib_rocksdb_BaseIterator);
  Py_CLEAR(clear_module_state->__pyx_ptype_8aimrocks_11lib_rocksdb_KeysIterator);
  Py_CLEAR(clear_module_state->__pyx_ptype_8aimrocks_11lib_rocksdb_ValuesIterator);
  Py_CLEAR(clear_module_state->__pyx_ptype_8aimrocks_11lib_rocksdb_ItemsIterator);
  Py_CLEAR(clear_module_state->__pyx_ptype_8aimrocks_11lib_rocksdb_ReversedIterator);
  Py_CLEAR(clear_module_state->__pyx_ptype_8aimrocks_11lib_rocksdb_BackupEngine);
  Py_CLEAR(clear_module_state->__pyx_ptype_3aim_7storage_5utils_ArrayFlagType);
  Py_CLEAR(clear_module_state->__pyx_ptype_3aim_7storage_5utils_ObjectFlagType);
  Py_CLEAR(clear_module_state->__pyx_ptype_3aim_7storage_5utils_CustomObjectFlagType);
  Py_CLEAR(clear_module_state->__pyx_ptype_3aim_7storage_5utils_BLOB);
  Py_CLEAR(clear_module_state->__pyx_ptype_3aim_7storage_9container_ContainerItemsIterator);
  Py_CLEAR(clear_module_state->__pyx_ptype_3aim_7storage_9container_Container);
  Py_CLEAR(clear_module_state->__pyx_ptype_3aim_7storage_8treeview_TreeView);
  Py_CLEAR(clear_module_state->__pyx_ptype_3aim_7storage_17containertreeview_ContainerTreeView);
  Py_CLEAR(clear_module_state->__pyx_ptype_3aim_7storage_10prefixview_PrefixView);
  Py_CLEAR(clear_module_state->__pyx_type_3aim_7storage_10prefixview_PrefixView);
  Py_CLEAR(clear_module_state->__pyx_ptype_3aim_7storage_10prefixview_PrefixViewItemsIterator);
  Py_CLEAR(clear_module_state->__pyx_type_3aim_7storage_10prefixview_PrefixViewItemsIterator);
  Py_CLEAR(clear_module_state->__pyx_ptype_3aim_7storage_10prefixview___pyx_scope_struct__walk);
  Py_CLEAR(clear_module_state->__pyx_type_3aim_7storage_10prefixview___pyx_scope_struct__walk);
  Py_CLEAR(clear_module_state->__pyx_kp_b_);
  Py_CLEAR(clear_module_state->__pyx_n_s_Container);
  Py_CLEAR(clear_module_state->__pyx_n_s_ContainerItemsIterator);
  Py_CLEAR(clear_module_state->__pyx_n_s_ContainerKey);
  Py_CLEAR(clear_module_state->__pyx_n_s_ContainerTreeView);
  Py_CLEAR(clear_module_state->__pyx_n_s_ContainerValue);
  Py_CLEAR(clear_module_state->__pyx_kp_s_Incompatible_checksums_s_vs_0x0d);
  Py_CLEAR(clear_module_state->__pyx_kp_s_Incompatible_checksums_s_vs_0xda);
  Py_CLEAR(clear_module_state->__pyx_n_s_Iterator);
  Py_CLEAR(clear_module_state->__pyx_n_s_KeyError);
  Py_CLEAR(clear_module_state->__pyx_n_s_PickleError);
  Py_CLEAR(clear_module_state->__pyx_n_s_PrefixView);
  Py_CLEAR(clear_module_state->__pyx_n_s_PrefixViewItemsIterator);
  Py_CLEAR(clear_module_state->__pyx_n_s_PrefixViewItemsIterator___reduce);
  Py_CLEAR(clear_module_state->__pyx_n_s_PrefixViewItemsIterator___setsta);
  Py_CLEAR(clear_module_state->__pyx_n_s_PrefixViewItemsIterator_next);
  Py_CLEAR(clear_module_state->__pyx_n_s_PrefixView___reduce_cython);
  Py_CLEAR(clear_module_state->__pyx_n_s_PrefixView___setstate_cython);
  Py_CLEAR(clear_module_state->__pyx_n_s_PrefixView_absolute_path);
  Py_CLEAR(clear_module_state->__pyx_n_s_PrefixView_batch);
  Py_CLEAR(clear_module_state->__pyx_n_s_PrefixView_close);
  Py_CLEAR(clear_module_state->__pyx_n_s_PrefixView_commit);
  Py_CLEAR(clear_module_state->__pyx_n_s_PrefixView_delete);
  Py_CLEAR(clear_module_state->__pyx_n_s_PrefixView_delete_range);
  Py_CLEAR(clear_module_state->__pyx_n_s_PrefixView_finalize);
  Py_CLEAR(clear_module_state->__pyx_n_s_PrefixView_get);
  Py_CLEAR(clear_module_state->__pyx_n_s_PrefixView_items);
  Py_CLEAR(clear_module_state->__pyx_n_s_PrefixView_next_item);
  Py_CLEAR(clear_module_state->__pyx_n_s_PrefixView_preload);
  Py_CLEAR(clear_module_state->__pyx_n_s_PrefixView_prev_item);
  Py_CLEAR(clear_module_state->__pyx_n_s_PrefixView_set);
  Py_CLEAR(clear_module_state->__pyx_n_s_PrefixView_tree);
  Py_CLEAR(clear_module_state->__pyx_n_s_PrefixView_view);
  Py_CLEAR(clear_module_state->__pyx_n_s_PrefixView_walk);
  Py_CLEAR(clear_module_state->__pyx_n_s_Tuple);
  Py_CLEAR(clear_module_state->__pyx_kp_b__2);
  Py_CLEAR(clear_module_state->__pyx_n_s__54);
  Py_CLEAR(clear_module_state->__pyx_n_s_absolute_path);
  Py_CLEAR(clear_module_state->__pyx_n_s_aim_storage_container);
  Py_CLEAR(clear_module_state->__pyx_n_s_aim_storage_containertreeview);
  Py_CLEAR(clear_module_state->__pyx_n_s_aim_storage_prefixview);
  Py_CLEAR(clear_module_state->__pyx_kp_s_aim_storage_prefixview_py);
  Py_CLEAR(clear_module_state->__pyx_n_s_args);
  Py_CLEAR(clear_module_state->__pyx_n_s_asyncio_coroutines);
  Py_CLEAR(clear_module_state->__pyx_n_s_batch);
  Py_CLEAR(clear_module_state->__pyx_n_s_begin);
  Py_CLEAR(clear_module_state->__pyx_n_s_bytes);
  Py_CLEAR(clear_module_state->__pyx_n_s_class_getitem);
  Py_CLEAR(clear_module_state->__pyx_n_s_cline_in_traceback);
  Py_CLEAR(clear_module_state->__pyx_n_s_close);
  Py_CLEAR(clear_module_state->__pyx_n_s_commit);
  Py_CLEAR(clear_module_state->__pyx_n_s_container);
  Py_CLEAR(clear_module_state->__pyx_n_s_default);
  Py_CLEAR(clear_module_state->__pyx_n_s_delete);
  Py_CLEAR(clear_module_state->__pyx_n_s_delete_range);
  Py_CLEAR(clear_module_state->__pyx_n_s_dict);
  Py_CLEAR(clear_module_state->__pyx_n_s_dict_2);
  Py_CLEAR(clear_module_state->__pyx_kp_u_disable);
  Py_CLEAR(clear_module_state->__pyx_kp_u_enable);
  Py_CLEAR(clear_module_state->__pyx_n_s_end);
  Py_CLEAR(clear_module_state->__pyx_n_s_finalize);
  Py_CLEAR(clear_module_state->__pyx_kp_u_gc);
  Py_CLEAR(clear_module_state->__pyx_n_s_get);
  Py_CLEAR(clear_module_state->__pyx_n_s_getstate);
  Py_CLEAR(clear_module_state->__pyx_n_s_import);
  Py_CLEAR(clear_module_state->__pyx_n_s_index);
  Py_CLEAR(clear_module_state->__pyx_n_s_is_coroutine);
  Py_CLEAR(clear_module_state->__pyx_kp_u_isenabled);
  Py_CLEAR(clear_module_state->__pyx_n_s_items);
  Py_CLEAR(clear_module_state->__pyx_n_s_key);
  Py_CLEAR(clear_module_state->__pyx_n_s_main);
  Py_CLEAR(clear_module_state->__pyx_n_s_name);
  Py_CLEAR(clear_module_state->__pyx_n_s_new);
  Py_CLEAR(clear_module_state->__pyx_n_s_next);
  Py_CLEAR(clear_module_state->__pyx_n_s_next_item);
  Py_CLEAR(clear_module_state->__pyx_n_s_next_key);
  Py_CLEAR(clear_module_state->__pyx_n_s_p);
  Py_CLEAR(clear_module_state->__pyx_n_s_parent);
  Py_CLEAR(clear_module_state->__pyx_n_s_partition);
  Py_CLEAR(clear_module_state->__pyx_n_s_path);
  Py_CLEAR(clear_module_state->__pyx_n_s_path_2);
  Py_CLEAR(clear_module_state->__pyx_n_s_pickle);
  Py_CLEAR(clear_module_state->__pyx_n_s_prefix);
  Py_CLEAR(clear_module_state->__pyx_n_s_prefix_2);
  Py_CLEAR(clear_module_state->__pyx_n_s_prefix_view);
  Py_CLEAR(clear_module_state->__pyx_n_s_preload);
  Py_CLEAR(clear_module_state->__pyx_n_s_prev_item);
  Py_CLEAR(clear_module_state->__pyx_n_s_pyx_PickleError);
  Py_CLEAR(clear_module_state->__pyx_n_s_pyx_checksum);
  Py_CLEAR(clear_module_state->__pyx_n_s_pyx_result);
  Py_CLEAR(clear_module_state->__pyx_n_s_pyx_state);
  Py_CLEAR(clear_module_state->__pyx_n_s_pyx_type);
  Py_CLEAR(clear_module_state->__pyx_n_s_pyx_unpickle_PrefixView);
  Py_CLEAR(clear_module_state->__pyx_n_s_pyx_unpickle_PrefixViewItemsIt);
  Py_CLEAR(clear_module_state->__pyx_n_s_pyx_vtable);
  Py_CLEAR(clear_module_state->__pyx_n_s_read_only);
  Py_CLEAR(clear_module_state->__pyx_n_s_reduce);
  Py_CLEAR(clear_module_state->__pyx_n_s_reduce_cython);
  Py_CLEAR(clear_module_state->__pyx_n_s_reduce_ex);
  Py_CLEAR(clear_module_state->__pyx_n_s_return);
  Py_CLEAR(clear_module_state->__pyx_n_s_self);
  Py_CLEAR(clear_module_state->__pyx_n_s_send);
  Py_CLEAR(clear_module_state->__pyx_n_s_set);
  Py_CLEAR(clear_module_state->__pyx_n_s_setstate);
  Py_CLEAR(clear_module_state->__pyx_n_s_setstate_cython);
  Py_CLEAR(clear_module_state->__pyx_n_s_state);
  Py_CLEAR(clear_module_state->__pyx_n_s_store_batch);
  Py_CLEAR(clear_module_state->__pyx_kp_s_stringsource);
  Py_CLEAR(clear_module_state->__pyx_n_s_test);
  Py_CLEAR(clear_module_state->__pyx_n_s_throw);
  Py_CLEAR(clear_module_state->__pyx_n_s_tree);
  Py_CLEAR(clear_module_state->__pyx_n_s_typing);
  Py_CLEAR(clear_module_state->__pyx_n_s_update);
  Py_CLEAR(clear_module_state->__pyx_n_s_use_setstate);
  Py_CLEAR(clear_module_state->__pyx_n_s_value);
  Py_CLEAR(clear_module_state->__pyx_n_s_view);
  Py_CLEAR(clear_module_state->__pyx_n_s_walk);
  Py_CLEAR(clear_module_state->__pyx_n_s_walker);
  Py_CLEAR(clear_module_state->__pyx_int_14202518);
  Py_CLEAR(clear_module_state->__pyx_int_228799785);
  Py_CLEAR(clear_module_state->__pyx_tuple__4);
  Py_CLEAR(clear_module_state->__pyx_tuple__6);
  Py_CLEAR(clear_module_state->__pyx_tuple__8);
  Py_CLEAR(clear_module_state->__pyx_tuple__10);
  Py_CLEAR(clear_module_state->__pyx_tuple__12);
  Py_CLEAR(clear_module_state->__pyx_tuple__13);
  Py_CLEAR(clear_module_state->__pyx_tuple__15);
  Py_CLEAR(clear_module_state->__pyx_tuple__17);
  Py_CLEAR(clear_module_state->__pyx_tuple__19);
  Py_CLEAR(clear_module_state->__pyx_tuple__21);
  Py_CLEAR(clear_module_state->__pyx_tuple__23);
  Py_CLEAR(clear_module_state->__pyx_tuple__24);
  Py_CLEAR(clear_module_state->__pyx_tuple__26);
  Py_CLEAR(clear_module_state->__pyx_tuple__27);
  Py_CLEAR(clear_module_state->__pyx_tuple__28);
  Py_CLEAR(clear_module_state->__pyx_tuple__29);
  Py_CLEAR(clear_module_state->__pyx_tuple__31);
  Py_CLEAR(clear_module_state->__pyx_tuple__32);
  Py_CLEAR(clear_module_state->__pyx_tuple__34);
  Py_CLEAR(clear_module_state->__pyx_tuple__36);
  Py_CLEAR(clear_module_state->__pyx_tuple__38);
  Py_CLEAR(clear_module_state->__pyx_tuple__40);
  Py_CLEAR(clear_module_state->__pyx_tuple__42);
  Py_CLEAR(clear_module_state->__pyx_tuple__44);
  Py_CLEAR(clear_module_state->__pyx_tuple__46);
  Py_CLEAR(clear_module_state->__pyx_tuple__48);
  Py_CLEAR(clear_module_state->__pyx_tuple__50);
  Py_CLEAR(clear_module_state->__pyx_tuple__52);
  Py_CLEAR(clear_module_state->__pyx_codeobj__3);
  Py_CLEAR(clear_module_state->__pyx_codeobj__5);
  Py_CLEAR(clear_module_state->__pyx_codeobj__7);
  Py_CLEAR(clear_module_state->__pyx_codeobj__9);
  Py_CLEAR(clear_module_state->__pyx_codeobj__11);
  Py_CLEAR(clear_module_state->__pyx_codeobj__14);
  Py_CLEAR(clear_module_state->__pyx_codeobj__16);
  Py_CLEAR(clear_module_state->__pyx_codeobj__18);
  Py_CLEAR(clear_module_state->__pyx_codeobj__20);
  Py_CLEAR(clear_module_state->__pyx_codeobj__22);
  Py_CLEAR(clear_module_state->__pyx_codeobj__25);
  Py_CLEAR(clear_module_state->__pyx_codeobj__30);
  Py_CLEAR(clear_module_state->__pyx_codeobj__33);
  Py_CLEAR(clear_module_state->__pyx_codeobj__35);
  Py_CLEAR(clear_module_state->__pyx_codeobj__37);
  Py_CLEAR(clear_module_state->__pyx_codeobj__39);
  Py_CLEAR(clear_module_state->__pyx_codeobj__41);
  Py_CLEAR(clear_module_state->__pyx_codeobj__43);
  Py_CLEAR(clear_module_state->__pyx_codeobj__45);
  Py_CLEAR(clear_module_state->__pyx_codeobj__47);
  Py_CLEAR(clear_module_state->__pyx_codeobj__49);
  Py_CLEAR(clear_module_state->__pyx_codeobj__51);
  Py_CLEAR(clear_module_state->__pyx_codeobj__53);
  return 0;
}
#endif
/* #### Code section: module_state_traverse ### */
#if CYTHON_USE_MODULE_STATE
static int __pyx_m_traverse(PyObject *m, visitproc visit, void *arg) {
  __pyx_mstate *traverse_module_state = __pyx_mstate(m);
  if (!traverse_module_state) return 0;
  Py_VISIT(traverse_module_state->__pyx_d);
  Py_VISIT(traverse_module_state->__pyx_b);
  Py_VISIT(traverse_module_state->__pyx_cython_runtime);
  Py_VISIT(traverse_module_state->__pyx_empty_tuple);
  Py_VISIT(traverse_module_state->__pyx_empty_bytes);
  Py_VISIT(traverse_module_state->__pyx_empty_unicode);
  #ifdef __Pyx_CyFunction_USED
  Py_VISIT(traverse_module_state->__pyx_CyFunctionType);
  #endif
  #ifdef __Pyx_FusedFunction_USED
  Py_VISIT(traverse_module_state->__pyx_FusedFunctionType);
  #endif
  Py_VISIT(traverse_module_state->__pyx_ptype_7cpython_4type_type);
  Py_VISIT(traverse_module_state->__pyx_ptype_7cpython_4bool_bool);
  Py_VISIT(traverse_module_state->__pyx_ptype_7cpython_7complex_complex);
  Py_VISIT(traverse_module_state->__pyx_ptype_8aimrocks_11lib_rocksdb_PyComparator);
  Py_VISIT(traverse_module_state->__pyx_ptype_8aimrocks_11lib_rocksdb_PyGenericComparator);
  Py_VISIT(traverse_module_state->__pyx_ptype_8aimrocks_11lib_rocksdb_PyBytewiseComparator);
  Py_VISIT(traverse_module_state->__pyx_ptype_8aimrocks_11lib_rocksdb_PyFilterPolicy);
  Py_VISIT(traverse_module_state->__pyx_ptype_8aimrocks_11lib_rocksdb_PyGenericFilterPolicy);
  Py_VISIT(traverse_module_state->__pyx_ptype_8aimrocks_11lib_rocksdb_PyBloomFilterPolicy);
  Py_VISIT(traverse_module_state->__pyx_ptype_8aimrocks_11lib_rocksdb_PyMergeOperator);
  Py_VISIT(traverse_module_state->__pyx_ptype_8aimrocks_11lib_rocksdb_PyCache);
  Py_VISIT(traverse_module_state->__pyx_ptype_8aimrocks_11lib_rocksdb_PyLRUCache);
  Py_VISIT(traverse_module_state->__pyx_ptype_8aimrocks_11lib_rocksdb_PySliceTransform);
  Py_VISIT(traverse_module_state->__pyx_ptype_8aimrocks_11lib_rocksdb_PyTableFactory);
  Py_VISIT(traverse_module_state->__pyx_ptype_8aimrocks_11lib_rocksdb_BlockBasedTableFactory);
  Py_VISIT(traverse_module_state->__pyx_ptype_8aimrocks_11lib_rocksdb_PlainTableFactory);
  Py_VISIT(traverse_module_state->__pyx_ptype_8aimrocks_11lib_rocksdb_PyMemtableFactory);
  Py_VISIT(traverse_module_state->__pyx_ptype_8aimrocks_11lib_rocksdb_SkipListMemtableFactory);
  Py_VISIT(traverse_module_state->__pyx_ptype_8aimrocks_11lib_rocksdb_VectorMemtableFactory);
  Py_VISIT(traverse_module_state->__pyx_ptype_8aimrocks_11lib_rocksdb_HashSkipListMemtableFactory);
  Py_VISIT(traverse_module_state->__pyx_ptype_8aimrocks_11lib_rocksdb_HashLinkListMemtableFactory);
  Py_VISIT(traverse_module_state->__pyx_ptype_8aimrocks_11lib_rocksdb_CompressionType);
  Py_VISIT(traverse_module_state->__pyx_ptype_8aimrocks_11lib_rocksdb_CompactionPri);
  Py_VISIT(traverse_module_state->__pyx_ptype_8aimrocks_11lib_rocksdb__ColumnFamilyHandle);
  Py_VISIT(traverse_module_state->__pyx_ptype_8aimrocks_11lib_rocksdb_ColumnFamilyHandle);
  Py_VISIT(traverse_module_state->__pyx_ptype_8aimrocks_11lib_rocksdb_ColumnFamilyOptions);
  Py_VISIT(traverse_module_state->__pyx_ptype_8aimrocks_11lib_rocksdb_Options);
  Py_VISIT(traverse_module_state->__pyx_ptype_8aimrocks_11lib_rocksdb_WriteBatch);
  Py_VISIT(traverse_module_state->__pyx_ptype_8aimrocks_11lib_rocksdb_WriteBatchIterator);
  Py_VISIT(traverse_module_state->__pyx_ptype_8aimrocks_11lib_rocksdb_IDB);
  Py_VISIT(traverse_module_state->__pyx_ptype_8aimrocks_11lib_rocksdb_DB);
  Py_VISIT(traverse_module_state->__pyx_ptype_8aimrocks_11lib_rocksdb_Snapshot);
  Py_VISIT(traverse_module_state->__pyx_ptype_8aimrocks_11lib_rocksdb_Iterator);
  Py_VISIT(traverse_module_state->__pyx_ptype_8aimrocks_11lib_rocksdb_BaseIterator);
  Py_VISIT(traverse_module_state->__pyx_ptype_8aimrocks_11lib_rocksdb_KeysIterator);
  Py_VISIT(traverse_module_state->__pyx_ptype_8aimrocks_11lib_rocksdb_ValuesIterator);
  Py_VISIT(traverse_module_state->__pyx_ptype_8aimrocks_11lib_rocksdb_ItemsIterator);
  Py_VISIT(traverse_module_state->__pyx_ptype_8aimrocks_11lib_rocksdb_ReversedIterator);
  Py_VISIT(traverse_module_state->__pyx_ptype_8aimrocks_11lib_rocksdb_BackupEngine);
  Py_VISIT(traverse_module_state->__pyx_ptype_3aim_7storage_5utils_ArrayFlagType);
  Py_VISIT(traverse_module_state->__pyx_ptype_3aim_7storage_5utils_ObjectFlagType);
  Py_VISIT(traverse_module_state->__pyx_ptype_3aim_7storage_5utils_CustomObjectFlagType);
  Py_VISIT(traverse_module_state->__pyx_ptype_3aim_7storage_5utils_BLOB);
  Py_VISIT(traverse_module_state->__pyx_ptype_3aim_7storage_9container_ContainerItemsIterator);
  Py_VISIT(traverse_module_state->__pyx_ptype_3aim_7storage_9container_Container);
  Py_VISIT(traverse_module_state->__pyx_ptype_3aim_7storage_8treeview_TreeView);
  Py_VISIT(traverse_module_state->__pyx_ptype_3aim_7storage_17containertreeview_ContainerTreeView);
  Py_VISIT(traverse_module_state->__pyx_ptype_3aim_7storage_10prefixview_PrefixView);
  Py_VISIT(traverse_module_state->__pyx_type_3aim_7storage_10prefixview_PrefixView);
  Py_VISIT(traverse_module_state->__pyx_ptype_3aim_7storage_10prefixview_PrefixViewItemsIterator);
  Py_VISIT(traverse_module_state->__pyx_type_3aim_7storage_10prefixview_PrefixViewItemsIterator);
  Py_VISIT(traverse_module_state->__pyx_ptype_3aim_7storage_10prefixview___pyx_scope_struct__walk);
  Py_VISIT(traverse_module_state->__pyx_type_3aim_7storage_10prefixview___pyx_scope_struct__walk);
  Py_VISIT(traverse_module_state->__pyx_kp_b_);
  Py_VISIT(traverse_module_state->__pyx_n_s_Container);
  Py_VISIT(traverse_module_state->__pyx_n_s_ContainerItemsIterator);
  Py_VISIT(traverse_module_state->__pyx_n_s_ContainerKey);
  Py_VISIT(traverse_module_state->__pyx_n_s_ContainerTreeView);
  Py_VISIT(traverse_module_state->__pyx_n_s_ContainerValue);
  Py_VISIT(traverse_module_state->__pyx_kp_s_Incompatible_checksums_s_vs_0x0d);
  Py_VISIT(traverse_module_state->__pyx_kp_s_Incompatible_checksums_s_vs_0xda);
  Py_VISIT(traverse_module_state->__pyx_n_s_Iterator);
  Py_VISIT(traverse_module_state->__pyx_n_s_KeyError);
  Py_VISIT(traverse_module_state->__pyx_n_s_PickleError);
  Py_VISIT(traverse_module_state->__pyx_n_s_PrefixView);
  Py_VISIT(traverse_module_state->__pyx_n_s_PrefixViewItemsIterator);
  Py_VISIT(traverse_module_state->__pyx_n_s_PrefixViewItemsIterator___reduce);
  Py_VISIT(traverse_module_state->__pyx_n_s_PrefixViewItemsIterator___setsta);
  Py_VISIT(traverse_module_state->__pyx_n_s_PrefixViewItemsIterator_next);
  Py_VISIT(traverse_module_state->__pyx_n_s_PrefixView___reduce_cython);
  Py_VISIT(traverse_module_state->__pyx_n_s_PrefixView___setstate_cython);
  Py_VISIT(traverse_module_state->__pyx_n_s_PrefixView_absolute_path);
  Py_VISIT(traverse_module_state->__pyx_n_s_PrefixView_batch);
  Py_VISIT(traverse_module_state->__pyx_n_s_PrefixView_close);
  Py_VISIT(traverse_module_state->__pyx_n_s_PrefixView_commit);
  Py_VISIT(traverse_module_state->__pyx_n_s_PrefixView_delete);
  Py_VISIT(traverse_module_state->__pyx_n_s_PrefixView_delete_range);
  Py_VISIT(traverse_module_state->__pyx_n_s_PrefixView_finalize);
  Py_VISIT(traverse_module_state->__pyx_n_s_PrefixView_get);
  Py_VISIT(traverse_module_state->__pyx_n_s_PrefixView_items);
  Py_VISIT(traverse_module_state->__pyx_n_s_PrefixView_next_item);
  Py_VISIT(traverse_module_state->__pyx_n_s_PrefixView_preload);
  Py_VISIT(traverse_module_state->__pyx_n_s_PrefixView_prev_item);
  Py_VISIT(traverse_module_state->__pyx_n_s_PrefixView_set);
  Py_VISIT(traverse_module_state->__pyx_n_s_PrefixView_tree);
  Py_VISIT(traverse_module_state->__pyx_n_s_PrefixView_view);
  Py_VISIT(traverse_module_state->__pyx_n_s_PrefixView_walk);
  Py_VISIT(traverse_module_state->__pyx_n_s_Tuple);
  Py_VISIT(traverse_module_state->__pyx_kp_b__2);
  Py_VISIT(traverse_module_state->__pyx_n_s__54);
  Py_VISIT(traverse_module_state->__pyx_n_s_absolute_path);
  Py_VISIT(traverse_module_state->__pyx_n_s_aim_storage_container);
  Py_VISIT(traverse_module_state->__pyx_n_s_aim_storage_containertreeview);
  Py_VISIT(traverse_module_state->__pyx_n_s_aim_storage_prefixview);
  Py_VISIT(traverse_module_state->__pyx_kp_s_aim_storage_prefixview_py);
  Py_VISIT(traverse_module_state->__pyx_n_s_args);
  Py_VISIT(traverse_module_state->__pyx_n_s_asyncio_coroutines);
  Py_VISIT(traverse_module_state->__pyx_n_s_batch);
  Py_VISIT(traverse_module_state->__pyx_n_s_begin);
  Py_VISIT(traverse_module_state->__pyx_n_s_bytes);
  Py_VISIT(traverse_module_state->__pyx_n_s_class_getitem);
  Py_VISIT(traverse_module_state->__pyx_n_s_cline_in_traceback);
  Py_VISIT(traverse_module_state->__pyx_n_s_close);
  Py_VISIT(traverse_module_state->__pyx_n_s_commit);
  Py_VISIT(traverse_module_state->__pyx_n_s_container);
  Py_VISIT(traverse_module_state->__pyx_n_s_default);
  Py_VISIT(traverse_module_state->__pyx_n_s_delete);
  Py_VISIT(traverse_module_state->__pyx_n_s_delete_range);
  Py_VISIT(traverse_module_state->__pyx_n_s_dict);
  Py_VISIT(traverse_module_state->__pyx_n_s_dict_2);
  Py_VISIT(traverse_module_state->__pyx_kp_u_disable);
  Py_VISIT(traverse_module_state->__pyx_kp_u_enable);
  Py_VISIT(traverse_module_state->__pyx_n_s_end);
  Py_VISIT(traverse_module_state->__pyx_n_s_finalize);
  Py_VISIT(traverse_module_state->__pyx_kp_u_gc);
  Py_VISIT(traverse_module_state->__pyx_n_s_get);
  Py_VISIT(traverse_module_state->__pyx_n_s_getstate);
  Py_VISIT(traverse_module_state->__pyx_n_s_import);
  Py_VISIT(traverse_module_state->__pyx_n_s_index);
  Py_VISIT(traverse_module_state->__pyx_n_s_is_coroutine);
  Py_VISIT(traverse_module_state->__pyx_kp_u_isenabled);
  Py_VISIT(traverse_module_state->__pyx_n_s_items);
  Py_VISIT(traverse_module_state->__pyx_n_s_key);
  Py_VISIT(traverse_module_state->__pyx_n_s_main);
  Py_VISIT(traverse_module_state->__pyx_n_s_name);
  Py_VISIT(traverse_module_state->__pyx_n_s_new);
  Py_VISIT(traverse_module_state->__pyx_n_s_next);
  Py_VISIT(traverse_module_state->__pyx_n_s_next_item);
  Py_VISIT(traverse_module_state->__pyx_n_s_next_key);
  Py_VISIT(traverse_module_state->__pyx_n_s_p);
  Py_VISIT(traverse_module_state->__pyx_n_s_parent);
  Py_VISIT(traverse_module_state->__pyx_n_s_partition);
  Py_VISIT(traverse_module_state->__pyx_n_s_path);
  Py_VISIT(traverse_module_state->__pyx_n_s_path_2);
  Py_VISIT(traverse_module_state->__pyx_n_s_pickle);
  Py_VISIT(traverse_module_state->__pyx_n_s_prefix);
  Py_VISIT(traverse_module_state->__pyx_n_s_prefix_2);
  Py_VISIT(traverse_module_state->__pyx_n_s_prefix_view);
  Py_VISIT(traverse_module_state->__pyx_n_s_preload);
  Py_VISIT(traverse_module_state->__pyx_n_s_prev_item);
  Py_VISIT(traverse_module_state->__pyx_n_s_pyx_PickleError);
  Py_VISIT(traverse_module_state->__pyx_n_s_pyx_checksum);
  Py_VISIT(traverse_module_state->__pyx_n_s_pyx_result);
  Py_VISIT(traverse_module_state->__pyx_n_s_pyx_state);
  Py_VISIT(traverse_module_state->__pyx_n_s_pyx_type);
  Py_VISIT(traverse_module_state->__pyx_n_s_pyx_unpickle_PrefixView);
  Py_VISIT(traverse_module_state->__pyx_n_s_pyx_unpickle_PrefixViewItemsIt);
  Py_VISIT(traverse_module_state->__pyx_n_s_pyx_vtable);
  Py_VISIT(traverse_module_state->__pyx_n_s_read_only);
  Py_VISIT(traverse_module_state->__pyx_n_s_reduce);
  Py_VISIT(traverse_module_state->__pyx_n_s_reduce_cython);
  Py_VISIT(traverse_module_state->__pyx_n_s_reduce_ex);
  Py_VISIT(traverse_module_state->__pyx_n_s_return);
  Py_VISIT(traverse_module_state->__pyx_n_s_self);
  Py_VISIT(traverse_module_state->__pyx_n_s_send);
  Py_VISIT(traverse_module_state->__pyx_n_s_set);
  Py_VISIT(traverse_module_state->__pyx_n_s_setstate);
  Py_VISIT(traverse_module_state->__pyx_n_s_setstate_cython);
  Py_VISIT(traverse_module_state->__pyx_n_s_state);
  Py_VISIT(traverse_module_state->__pyx_n_s_store_batch);
  Py_VISIT(traverse_module_state->__pyx_kp_s_stringsource);
  Py_VISIT(traverse_module_state->__pyx_n_s_test);
  Py_VISIT(traverse_module_state->__pyx_n_s_throw);
  Py_VISIT(traverse_module_state->__pyx_n_s_tree);
  Py_VISIT(traverse_module_state->__pyx_n_s_typing);
  Py_VISIT(traverse_module_state->__pyx_n_s_update);
  Py_VISIT(traverse_module_state->__pyx_n_s_use_setstate);
  Py_VISIT(traverse_module_state->__pyx_n_s_value);
  Py_VISIT(traverse_module_state->__pyx_n_s_view);
  Py_VISIT(traverse_module_state->__pyx_n_s_walk);
  Py_VISIT(traverse_module_state->__pyx_n_s_walker);
  Py_VISIT(traverse_module_state->__pyx_int_14202518);
  Py_VISIT(traverse_module_state->__pyx_int_228799785);
  Py_VISIT(traverse_module_state->__pyx_tuple__4);
  Py_VISIT(traverse_module_state->__pyx_tuple__6);
  Py_VISIT(traverse_module_state->__pyx_tuple__8);
  Py_VISIT(traverse_module_state->__pyx_tuple__10);
  Py_VISIT(traverse_module_state->__pyx_tuple__12);
  Py_VISIT(traverse_module_state->__pyx_tuple__13);
  Py_VISIT(traverse_module_state->__pyx_tuple__15);
  Py_VISIT(traverse_module_state->__pyx_tuple__17);
  Py_VISIT(traverse_module_state->__pyx_tuple__19);
  Py_VISIT(traverse_module_state->__pyx_tuple__21);
  Py_VISIT(traverse_module_state->__pyx_tuple__23);
  Py_VISIT(traverse_module_state->__pyx_tuple__24);
  Py_VISIT(traverse_module_state->__pyx_tuple__26);
  Py_VISIT(traverse_module_state->__pyx_tuple__27);
  Py_VISIT(traverse_module_state->__pyx_tuple__28);
  Py_VISIT(traverse_module_state->__pyx_tuple__29);
  Py_VISIT(traverse_module_state->__pyx_tuple__31);
  Py_VISIT(traverse_module_state->__pyx_tuple__32);
  Py_VISIT(traverse_module_state->__pyx_tuple__34);
  Py_VISIT(traverse_module_state->__pyx_tuple__36);
  Py_VISIT(traverse_module_state->__pyx_tuple__38);
  Py_VISIT(traverse_module_state->__pyx_tuple__40);
  Py_VISIT(traverse_module_state->__pyx_tuple__42);
  Py_VISIT(traverse_module_state->__pyx_tuple__44);
  Py_VISIT(traverse_module_state->__pyx_tuple__46);
  Py_VISIT(traverse_module_state->__pyx_tuple__48);
  Py_VISIT(traverse_module_state->__pyx_tuple__50);
  Py_VISIT(traverse_module_state->__pyx_tuple__52);
  Py_VISIT(traverse_module_state->__pyx_codeobj__3);
  Py_VISIT(traverse_module_state->__pyx_codeobj__5);
  Py_VISIT(traverse_module_state->__pyx_codeobj__7);
  Py_VISIT(traverse_module_state->__pyx_codeobj__9);
  Py_VISIT(traverse_module_state->__pyx_codeobj__11);
  Py_VISIT(traverse_module_state->__pyx_codeobj__14);
  Py_VISIT(traverse_module_state->__pyx_codeobj__16);
  Py_VISIT(traverse_module_state->__pyx_codeobj__18);
  Py_VISIT(traverse_module_state->__pyx_codeobj__20);
  Py_VISIT(traverse_module_state->__pyx_codeobj__22);
  Py_VISIT(traverse_module_state->__pyx_codeobj__25);
  Py_VISIT(traverse_module_state->__pyx_codeobj__30);
  Py_VISIT(traverse_module_state->__pyx_codeobj__33);
  Py_VISIT(traverse_module_state->__pyx_codeobj__35);
  Py_VISIT(traverse_module_state->__pyx_codeobj__37);
  Py_VISIT(traverse_module_state->__pyx_codeobj__39);
  Py_VISIT(traverse_module_state->__pyx_codeobj__41);
  Py_VISIT(traverse_module_state->__pyx_codeobj__43);
  Py_VISIT(traverse_module_state->__pyx_codeobj__45);
  Py_VISIT(traverse_module_state->__pyx_codeobj__47);
  Py_VISIT(traverse_module_state->__pyx_codeobj__49);
  Py_VISIT(traverse_module_state->__pyx_codeobj__51);
  Py_VISIT(traverse_module_state->__pyx_codeobj__53);
  return 0;
}
#endif
/* #### Code section: module_state_defines ### */
#if CYTHON_USE_MODULE_STATE
#define __pyx_d __pyx_mstate_global->__pyx_d
#define __pyx_b __pyx_mstate_global->__pyx_b
#define __pyx_cython_runtime __pyx_mstate_global->__pyx_cython_runtime
#define __pyx_empty_tuple __pyx_mstate_global->__pyx_empty_tuple
#define __pyx_empty_bytes __pyx_mstate_global->__pyx_empty_bytes
#define __pyx_empty_unicode __pyx_mstate_global->__pyx_empty_unicode
#ifdef __Pyx_CyFunction_USED
#define __pyx_CyFunctionType __pyx_mstate_global->__pyx_CyFunctionType
#endif
#ifdef __Pyx_FusedFunction_USED
#define __pyx_FusedFunctionType __pyx_mstate_global->__pyx_FusedFunctionType
#endif
#define __pyx_ptype_7cpython_4type_type __pyx_mstate_global->__pyx_ptype_7cpython_4type_type
#define __pyx_ptype_7cpython_4bool_bool __pyx_mstate_global->__pyx_ptype_7cpython_4bool_bool
#define __pyx_ptype_7cpython_7complex_complex __pyx_mstate_global->__pyx_ptype_7cpython_7complex_complex
#define __pyx_ptype_8aimrocks_11lib_rocksdb_PyComparator __pyx_mstate_global->__pyx_ptype_8aimrocks_11lib_rocksdb_PyComparator
#define __pyx_ptype_8aimrocks_11lib_rocksdb_PyGenericComparator __pyx_mstate_global->__pyx_ptype_8aimrocks_11lib_rocksdb_PyGenericComparator
#define __pyx_ptype_8aimrocks_11lib_rocksdb_PyBytewiseComparator __pyx_mstate_global->__pyx_ptype_8aimrocks_11lib_rocksdb_PyBytewiseComparator
#define __pyx_ptype_8aimrocks_11lib_rocksdb_PyFilterPolicy __pyx_mstate_global->__pyx_ptype_8aimrocks_11lib_rocksdb_PyFilterPolicy
#define __pyx_ptype_8aimrocks_11lib_rocksdb_PyGenericFilterPolicy __pyx_mstate_global->__pyx_ptype_8aimrocks_11lib_rocksdb_PyGenericFilterPolicy
#define __pyx_ptype_8aimrocks_11lib_rocksdb_PyBloomFilterPolicy __pyx_mstate_global->__pyx_ptype_8aimrocks_11lib_rocksdb_PyBloomFilterPolicy
#define __pyx_ptype_8aimrocks_11lib_rocksdb_PyMergeOperator __pyx_mstate_global->__pyx_ptype_8aimrocks_11lib_rocksdb_PyMergeOperator
#define __pyx_ptype_8aimrocks_11lib_rocksdb_PyCache __pyx_mstate_global->__pyx_ptype_8aimrocks_11lib_rocksdb_PyCache
#define __pyx_ptype_8aimrocks_11lib_rocksdb_PyLRUCache __pyx_mstate_global->__pyx_ptype_8aimrocks_11lib_rocksdb_PyLRUCache
#define __pyx_ptype_8aimrocks_11lib_rocksdb_PySliceTransform __pyx_mstate_global->__pyx_ptype_8aimrocks_11lib_rocksdb_PySliceTransform
#define __pyx_ptype_8aimrocks_11lib_rocksdb_PyTableFactory __pyx_mstate_global->__pyx_ptype_8aimrocks_11lib_rocksdb_PyTableFactory
#define __pyx_ptype_8aimrocks_11lib_rocksdb_BlockBasedTableFactory __pyx_mstate_global->__pyx_ptype_8aimrocks_11lib_rocksdb_BlockBasedTableFactory
#define __pyx_ptype_8aimrocks_11lib_rocksdb_PlainTableFactory __pyx_mstate_global->__pyx_ptype_8aimrocks_11lib_rocksdb_PlainTableFactory
#define __pyx_ptype_8aimrocks_11lib_rocksdb_PyMemtableFactory __pyx_mstate_global->__pyx_ptype_8aimrocks_11lib_rocksdb_PyMemtableFactory
#define __pyx_ptype_8aimrocks_11lib_rocksdb_SkipListMemtableFactory __pyx_mstate_global->__pyx_ptype_8aimrocks_11lib_rocksdb_SkipListMemtableFactory
#define __pyx_ptype_8aimrocks_11lib_rocksdb_VectorMemtableFactory __pyx_mstate_global->__pyx_ptype_8aimrocks_11lib_rocksdb_VectorMemtableFactory
#define __pyx_ptype_8aimrocks_11lib_rocksdb_HashSkipListMemtableFactory __pyx_mstate_global->__pyx_ptype_8aimrocks_11lib_rocksdb_HashSkipListMemtableFactory
#define __pyx_ptype_8aimrocks_11lib_rocksdb_HashLinkListMemtableFactory __pyx_mstate_global->__pyx_ptype_8aimrocks_11lib_rocksdb_HashLinkListMemtableFactory
#define __pyx_ptype_8aimrocks_11lib_rocksdb_CompressionType __pyx_mstate_global->__pyx_ptype_8aimrocks_11lib_rocksdb_CompressionType
#define __pyx_ptype_8aimrocks_11lib_rocksdb_CompactionPri __pyx_mstate_global->__pyx_ptype_8aimrocks_11lib_rocksdb_CompactionPri
#define __pyx_ptype_8aimrocks_11lib_rocksdb__ColumnFamilyHandle __pyx_mstate_global->__pyx_ptype_8aimrocks_11lib_rocksdb__ColumnFamilyHandle
#define __pyx_ptype_8aimrocks_11lib_rocksdb_ColumnFamilyHandle __pyx_mstate_global->__pyx_ptype_8aimrocks_11lib_rocksdb_ColumnFamilyHandle
#define __pyx_ptype_8aimrocks_11lib_rocksdb_ColumnFamilyOptions __pyx_mstate_global->__pyx_ptype_8aimrocks_11lib_rocksdb_ColumnFamilyOptions
#define __pyx_ptype_8aimrocks_11lib_rocksdb_Options __pyx_mstate_global->__pyx_ptype_8aimrocks_11lib_rocksdb_Options
#define __pyx_ptype_8aimrocks_11lib_rocksdb_WriteBatch __pyx_mstate_global->__pyx_ptype_8aimrocks_11lib_rocksdb_WriteBatch
#define __pyx_ptype_8aimrocks_11lib_rocksdb_WriteBatchIterator __pyx_mstate_global->__pyx_ptype_8aimrocks_11lib_rocksdb_WriteBatchIterator
#define __pyx_ptype_8aimrocks_11lib_rocksdb_IDB __pyx_mstate_global->__pyx_ptype_8aimrocks_11lib_rocksdb_IDB
#define __pyx_ptype_8aimrocks_11lib_rocksdb_DB __pyx_mstate_global->__pyx_ptype_8aimrocks_11lib_rocksdb_DB
#define __pyx_ptype_8aimrocks_11lib_rocksdb_Snapshot __pyx_mstate_global->__pyx_ptype_8aimrocks_11lib_rocksdb_Snapshot
#define __pyx_ptype_8aimrocks_11lib_rocksdb_Iterator __pyx_mstate_global->__pyx_ptype_8aimrocks_11lib_rocksdb_Iterator
#define __pyx_ptype_8aimrocks_11lib_rocksdb_BaseIterator __pyx_mstate_global->__pyx_ptype_8aimrocks_11lib_rocksdb_BaseIterator
#define __pyx_ptype_8aimrocks_11lib_rocksdb_KeysIterator __pyx_mstate_global->__pyx_ptype_8aimrocks_11lib_rocksdb_KeysIterator
#define __pyx_ptype_8aimrocks_11lib_rocksdb_ValuesIterator __pyx_mstate_global->__pyx_ptype_8aimrocks_11lib_rocksdb_ValuesIterator
#define __pyx_ptype_8aimrocks_11lib_rocksdb_ItemsIterator __pyx_mstate_global->__pyx_ptype_8aimrocks_11lib_rocksdb_ItemsIterator
#define __pyx_ptype_8aimrocks_11lib_rocksdb_ReversedIterator __pyx_mstate_global->__pyx_ptype_8aimrocks_11lib_rocksdb_ReversedIterator
#define __pyx_ptype_8aimrocks_11lib_rocksdb_BackupEngine __pyx_mstate_global->__pyx_ptype_8aimrocks_11lib_rocksdb_BackupEngine
#define __pyx_ptype_3aim_7storage_5utils_ArrayFlagType __pyx_mstate_global->__pyx_ptype_3aim_7storage_5utils_ArrayFlagType
#define __pyx_ptype_3aim_7storage_5utils_ObjectFlagType __pyx_mstate_global->__pyx_ptype_3aim_7storage_5utils_ObjectFlagType
#define __pyx_ptype_3aim_7storage_5utils_CustomObjectFlagType __pyx_mstate_global->__pyx_ptype_3aim_7storage_5utils_CustomObjectFlagType
#define __pyx_ptype_3aim_7storage_5utils_BLOB __pyx_mstate_global->__pyx_ptype_3aim_7storage_5utils_BLOB
#define __pyx_ptype_3aim_7storage_9container_ContainerItemsIterator __pyx_mstate_global->__pyx_ptype_3aim_7storage_9container_ContainerItemsIterator
#define __pyx_ptype_3aim_7storage_9container_Container __pyx_mstate_global->__pyx_ptype_3aim_7storage_9container_Container
#define __pyx_ptype_3aim_7storage_8treeview_TreeView __pyx_mstate_global->__pyx_ptype_3aim_7storage_8treeview_TreeView
#define __pyx_ptype_3aim_7storage_17containertreeview_ContainerTreeView __pyx_mstate_global->__pyx_ptype_3aim_7storage_17containertreeview_ContainerTreeView
#define __pyx_ptype_3aim_7storage_10prefixview_PrefixView __pyx_mstate_global->__pyx_ptype_3aim_7storage_10prefixview_PrefixView
#define __pyx_type_3aim_7storage_10prefixview_PrefixView __pyx_mstate_global->__pyx_type_3aim_7storage_10prefixview_PrefixView
#define __pyx_ptype_3aim_7storage_10prefixview_PrefixViewItemsIterator __pyx_mstate_global->__pyx_ptype_3aim_7storage_10prefixview_PrefixViewItemsIterator
#define __pyx_type_3aim_7storage_10prefixview_PrefixViewItemsIterator __pyx_mstate_global->__pyx_type_3aim_7storage_10prefixview_PrefixViewItemsIterator
#define __pyx_ptype_3aim_7storage_10prefixview___pyx_scope_struct__walk __pyx_mstate_global->__pyx_ptype_3aim_7storage_10prefixview___pyx_scope_struct__walk
#define __pyx_type_3aim_7storage_10prefixview___pyx_scope_struct__walk __pyx_mstate_global->__pyx_type_3aim_7storage_10prefixview___pyx_scope_struct__walk
#define __pyx_kp_b_ __pyx_mstate_global->__pyx_kp_b_
#define __pyx_n_s_Container __pyx_mstate_global->__pyx_n_s_Container
#define __pyx_n_s_ContainerItemsIterator __pyx_mstate_global->__pyx_n_s_ContainerItemsIterator
#define __pyx_n_s_ContainerKey __pyx_mstate_global->__pyx_n_s_ContainerKey
#define __pyx_n_s_ContainerTreeView __pyx_mstate_global->__pyx_n_s_ContainerTreeView
#define __pyx_n_s_ContainerValue __pyx_mstate_global->__pyx_n_s_ContainerValue
#define __pyx_kp_s_Incompatible_checksums_s_vs_0x0d __pyx_mstate_global->__pyx_kp_s_Incompatible_checksums_s_vs_0x0d
#define __pyx_kp_s_Incompatible_checksums_s_vs_0xda __pyx_mstate_global->__pyx_kp_s_Incompatible_checksums_s_vs_0xda
#define __pyx_n_s_Iterator __pyx_mstate_global->__pyx_n_s_Iterator
#define __pyx_n_s_KeyError __pyx_mstate_global->__pyx_n_s_KeyError
#define __pyx_n_s_PickleError __pyx_mstate_global->__pyx_n_s_PickleError
#define __pyx_n_s_PrefixView __pyx_mstate_global->__pyx_n_s_PrefixView
#define __pyx_n_s_PrefixViewItemsIterator __pyx_mstate_global->__pyx_n_s_PrefixViewItemsIterator
#define __pyx_n_s_PrefixViewItemsIterator___reduce __pyx_mstate_global->__pyx_n_s_PrefixViewItemsIterator___reduce
#define __pyx_n_s_PrefixViewItemsIterator___setsta __pyx_mstate_global->__pyx_n_s_PrefixViewItemsIterator___setsta
#define __pyx_n_s_PrefixViewItemsIterator_next __pyx_mstate_global->__pyx_n_s_PrefixViewItemsIterator_next
#define __pyx_n_s_PrefixView___reduce_cython __pyx_mstate_global->__pyx_n_s_PrefixView___reduce_cython
#define __pyx_n_s_PrefixView___setstate_cython __pyx_mstate_global->__pyx_n_s_PrefixView___setstate_cython
#define __pyx_n_s_PrefixView_absolute_path __pyx_mstate_global->__pyx_n_s_PrefixView_absolute_path
#define __pyx_n_s_PrefixView_batch __pyx_mstate_global->__pyx_n_s_PrefixView_batch
#define __pyx_n_s_PrefixView_close __pyx_mstate_global->__pyx_n_s_PrefixView_close
#define __pyx_n_s_PrefixView_commit __pyx_mstate_global->__pyx_n_s_PrefixView_commit
#define __pyx_n_s_PrefixView_delete __pyx_mstate_global->__pyx_n_s_PrefixView_delete
#define __pyx_n_s_PrefixView_delete_range __pyx_mstate_global->__pyx_n_s_PrefixView_delete_range
#define __pyx_n_s_PrefixView_finalize __pyx_mstate_global->__pyx_n_s_PrefixView_finalize
#define __pyx_n_s_PrefixView_get __pyx_mstate_global->__pyx_n_s_PrefixView_get
#define __pyx_n_s_PrefixView_items __pyx_mstate_global->__pyx_n_s_PrefixView_items
#define __pyx_n_s_PrefixView_next_item __pyx_mstate_global->__pyx_n_s_PrefixView_next_item
#define __pyx_n_s_PrefixView_preload __pyx_mstate_global->__pyx_n_s_PrefixView_preload
#define __pyx_n_s_PrefixView_prev_item __pyx_mstate_global->__pyx_n_s_PrefixView_prev_item
#define __pyx_n_s_PrefixView_set __pyx_mstate_global->__pyx_n_s_PrefixView_set
#define __pyx_n_s_PrefixView_tree __pyx_mstate_global->__pyx_n_s_PrefixView_tree
#define __pyx_n_s_PrefixView_view __pyx_mstate_global->__pyx_n_s_PrefixView_view
#define __pyx_n_s_PrefixView_walk __pyx_mstate_global->__pyx_n_s_PrefixView_walk
#define __pyx_n_s_Tuple __pyx_mstate_global->__pyx_n_s_Tuple
#define __pyx_kp_b__2 __pyx_mstate_global->__pyx_kp_b__2
#define __pyx_n_s__54 __pyx_mstate_global->__pyx_n_s__54
#define __pyx_n_s_absolute_path __pyx_mstate_global->__pyx_n_s_absolute_path
#define __pyx_n_s_aim_storage_container __pyx_mstate_global->__pyx_n_s_aim_storage_container
#define __pyx_n_s_aim_storage_containertreeview __pyx_mstate_global->__pyx_n_s_aim_storage_containertreeview
#define __pyx_n_s_aim_storage_prefixview __pyx_mstate_global->__pyx_n_s_aim_storage_prefixview
#define __pyx_kp_s_aim_storage_prefixview_py __pyx_mstate_global->__pyx_kp_s_aim_storage_prefixview_py
#define __pyx_n_s_args __pyx_mstate_global->__pyx_n_s_args
#define __pyx_n_s_asyncio_coroutines __pyx_mstate_global->__pyx_n_s_asyncio_coroutines
#define __pyx_n_s_batch __pyx_mstate_global->__pyx_n_s_batch
#define __pyx_n_s_begin __pyx_mstate_global->__pyx_n_s_begin
#define __pyx_n_s_bytes __pyx_mstate_global->__pyx_n_s_bytes
#define __pyx_n_s_class_getitem __pyx_mstate_global->__pyx_n_s_class_getitem
#define __pyx_n_s_cline_in_traceback __pyx_mstate_global->__pyx_n_s_cline_in_traceback
#define __pyx_n_s_close __pyx_mstate_global->__pyx_n_s_close
#define __pyx_n_s_commit __pyx_mstate_global->__pyx_n_s_commit
#define __pyx_n_s_container __pyx_mstate_global->__pyx_n_s_container
#define __pyx_n_s_default __pyx_mstate_global->__pyx_n_s_default
#define __pyx_n_s_delete __pyx_mstate_global->__pyx_n_s_delete
#define __pyx_n_s_delete_range __pyx_mstate_global->__pyx_n_s_delete_range
#define __pyx_n_s_dict __pyx_mstate_global->__pyx_n_s_dict
#define __pyx_n_s_dict_2 __pyx_mstate_global->__pyx_n_s_dict_2
#define __pyx_kp_u_disable __pyx_mstate_global->__pyx_kp_u_disable
#define __pyx_kp_u_enable __pyx_mstate_global->__pyx_kp_u_enable
#define __pyx_n_s_end __pyx_mstate_global->__pyx_n_s_end
#define __pyx_n_s_finalize __pyx_mstate_global->__pyx_n_s_finalize
#define __pyx_kp_u_gc __pyx_mstate_global->__pyx_kp_u_gc
#define __pyx_n_s_get __pyx_mstate_global->__pyx_n_s_get
#define __pyx_n_s_getstate __pyx_mstate_global->__pyx_n_s_getstate
#define __pyx_n_s_import __pyx_mstate_global->__pyx_n_s_import
#define __pyx_n_s_index __pyx_mstate_global->__pyx_n_s_index
#define __pyx_n_s_is_coroutine __pyx_mstate_global->__pyx_n_s_is_coroutine
#define __pyx_kp_u_isenabled __pyx_mstate_global->__pyx_kp_u_isenabled
#define __pyx_n_s_items __pyx_mstate_global->__pyx_n_s_items
#define __pyx_n_s_key __pyx_mstate_global->__pyx_n_s_key
#define __pyx_n_s_main __pyx_mstate_global->__pyx_n_s_main
#define __pyx_n_s_name __pyx_mstate_global->__pyx_n_s_name
#define __pyx_n_s_new __pyx_mstate_global->__pyx_n_s_new
#define __pyx_n_s_next __pyx_mstate_global->__pyx_n_s_next
#define __pyx_n_s_next_item __pyx_mstate_global->__pyx_n_s_next_item
#define __pyx_n_s_next_key __pyx_mstate_global->__pyx_n_s_next_key
#define __pyx_n_s_p __pyx_mstate_global->__pyx_n_s_p
#define __pyx_n_s_parent __pyx_mstate_global->__pyx_n_s_parent
#define __pyx_n_s_partition __pyx_mstate_global->__pyx_n_s_partition
#define __pyx_n_s_path __pyx_mstate_global->__pyx_n_s_path
#define __pyx_n_s_path_2 __pyx_mstate_global->__pyx_n_s_path_2
#define __pyx_n_s_pickle __pyx_mstate_global->__pyx_n_s_pickle
#define __pyx_n_s_prefix __pyx_mstate_global->__pyx_n_s_prefix
#define __pyx_n_s_prefix_2 __pyx_mstate_global->__pyx_n_s_prefix_2
#define __pyx_n_s_prefix_view __pyx_mstate_global->__pyx_n_s_prefix_view
#define __pyx_n_s_preload __pyx_mstate_global->__pyx_n_s_preload
#define __pyx_n_s_prev_item __pyx_mstate_global->__pyx_n_s_prev_item
#define __pyx_n_s_pyx_PickleError __pyx_mstate_global->__pyx_n_s_pyx_PickleError
#define __pyx_n_s_pyx_checksum __pyx_mstate_global->__pyx_n_s_pyx_checksum
#define __pyx_n_s_pyx_result __pyx_mstate_global->__pyx_n_s_pyx_result
#define __pyx_n_s_pyx_state __pyx_mstate_global->__pyx_n_s_pyx_state
#define __pyx_n_s_pyx_type __pyx_mstate_global->__pyx_n_s_pyx_type
#define __pyx_n_s_pyx_unpickle_PrefixView __pyx_mstate_global->__pyx_n_s_pyx_unpickle_PrefixView
#define __pyx_n_s_pyx_unpickle_PrefixViewItemsIt __pyx_mstate_global->__pyx_n_s_pyx_unpickle_PrefixViewItemsIt
#define __pyx_n_s_pyx_vtable __pyx_mstate_global->__pyx_n_s_pyx_vtable
#define __pyx_n_s_read_only __pyx_mstate_global->__pyx_n_s_read_only
#define __pyx_n_s_reduce __pyx_mstate_global->__pyx_n_s_reduce
#define __pyx_n_s_reduce_cython __pyx_mstate_global->__pyx_n_s_reduce_cython
#define __pyx_n_s_reduce_ex __pyx_mstate_global->__pyx_n_s_reduce_ex
#define __pyx_n_s_return __pyx_mstate_global->__pyx_n_s_return
#define __pyx_n_s_self __pyx_mstate_global->__pyx_n_s_self
#define __pyx_n_s_send __pyx_mstate_global->__pyx_n_s_send
#define __pyx_n_s_set __pyx_mstate_global->__pyx_n_s_set
#define __pyx_n_s_setstate __pyx_mstate_global->__pyx_n_s_setstate
#define __pyx_n_s_setstate_cython __pyx_mstate_global->__pyx_n_s_setstate_cython
#define __pyx_n_s_state __pyx_mstate_global->__pyx_n_s_state
#define __pyx_n_s_store_batch __pyx_mstate_global->__pyx_n_s_store_batch
#define __pyx_kp_s_stringsource __pyx_mstate_global->__pyx_kp_s_stringsource
#define __pyx_n_s_test __pyx_mstate_global->__pyx_n_s_test
#define __pyx_n_s_throw __pyx_mstate_global->__pyx_n_s_throw
#define __pyx_n_s_tree __pyx_mstate_global->__pyx_n_s_tree
#define __pyx_n_s_typing __pyx_mstate_global->__pyx_n_s_typing
#define __pyx_n_s_update __pyx_mstate_global->__pyx_n_s_update
#define __pyx_n_s_use_setstate __pyx_mstate_global->__pyx_n_s_use_setstate
#define __pyx_n_s_value __pyx_mstate_global->__pyx_n_s_value
#define __pyx_n_s_view __pyx_mstate_global->__pyx_n_s_view
#define __pyx_n_s_walk __pyx_mstate_global->__pyx_n_s_walk
#define __pyx_n_s_walker __pyx_mstate_global->__pyx_n_s_walker
#define __pyx_int_14202518 __pyx_mstate_global->__pyx_int_14202518
#define __pyx_int_228799785 __pyx_mstate_global->__pyx_int_228799785
#define __pyx_tuple__4 __pyx_mstate_global->__pyx_tuple__4
#define __pyx_tuple__6 __pyx_mstate_global->__pyx_tuple__6
#define __pyx_tuple__8 __pyx_mstate_global->__pyx_tuple__8
#define __pyx_tuple__10 __pyx_mstate_global->__pyx_tuple__10
#define __pyx_tuple__12 __pyx_mstate_global->__pyx_tuple__12
#define __pyx_tuple__13 __pyx_mstate_global->__pyx_tuple__13
#define __pyx_tuple__15 __pyx_mstate_global->__pyx_tuple__15
#define __pyx_tuple__17 __pyx_mstate_global->__pyx_tuple__17
#define __pyx_tuple__19 __pyx_mstate_global->__pyx_tuple__19
#define __pyx_tuple__21 __pyx_mstate_global->__pyx_tuple__21
#define __pyx_tuple__23 __pyx_mstate_global->__pyx_tuple__23
#define __pyx_tuple__24 __pyx_mstate_global->__pyx_tuple__24
#define __pyx_tuple__26 __pyx_mstate_global->__pyx_tuple__26
#define __pyx_tuple__27 __pyx_mstate_global->__pyx_tuple__27
#define __pyx_tuple__28 __pyx_mstate_global->__pyx_tuple__28
#define __pyx_tuple__29 __pyx_mstate_global->__pyx_tuple__29
#define __pyx_tuple__31 __pyx_mstate_global->__pyx_tuple__31
#define __pyx_tuple__32 __pyx_mstate_global->__pyx_tuple__32
#define __pyx_tuple__34 __pyx_mstate_global->__pyx_tuple__34
#define __pyx_tuple__36 __pyx_mstate_global->__pyx_tuple__36
#define __pyx_tuple__38 __pyx_mstate_global->__pyx_tuple__38
#define __pyx_tuple__40 __pyx_mstate_global->__pyx_tuple__40
#define __pyx_tuple__42 __pyx_mstate_global->__pyx_tuple__42
#define __pyx_tuple__44 __pyx_mstate_global->__pyx_tuple__44
#define __pyx_tuple__46 __pyx_mstate_global->__pyx_tuple__46
#define __pyx_tuple__48 __pyx_mstate_global->__pyx_tuple__48
#define __pyx_tuple__50 __pyx_mstate_global->__pyx_tuple__50
#define __pyx_tuple__52 __pyx_mstate_global->__pyx_tuple__52
#define __pyx_codeobj__3 __pyx_mstate_global->__pyx_codeobj__3
#define __pyx_codeobj__5 __pyx_mstate_global->__pyx_codeobj__5
#define __pyx_codeobj__7 __pyx_mstate_global->__pyx_codeobj__7
#define __pyx_codeobj__9 __pyx_mstate_global->__pyx_codeobj__9
#define __pyx_codeobj__11 __pyx_mstate_global->__pyx_codeobj__11
#define __pyx_codeobj__14 __pyx_mstate_global->__pyx_codeobj__14
#define __pyx_codeobj__16 __pyx_mstate_global->__pyx_codeobj__16
#define __pyx_codeobj__18 __pyx_mstate_global->__pyx_codeobj__18
#define __pyx_codeobj__20 __pyx_mstate_global->__pyx_codeobj__20
#define __pyx_codeobj__22 __pyx_mstate_global->__pyx_codeobj__22
#define __pyx_codeobj__25 __pyx_mstate_global->__pyx_codeobj__25
#define __pyx_codeobj__30 __pyx_mstate_global->__pyx_codeobj__30
#define __pyx_codeobj__33 __pyx_mstate_global->__pyx_codeobj__33
#define __pyx_codeobj__35 __pyx_mstate_global->__pyx_codeobj__35
#define __pyx_codeobj__37 __pyx_mstate_global->__pyx_codeobj__37
#define __pyx_codeobj__39 __pyx_mstate_global->__pyx_codeobj__39
#define __pyx_codeobj__41 __pyx_mstate_global->__pyx_codeobj__41
#define __pyx_codeobj__43 __pyx_mstate_global->__pyx_codeobj__43
#define __pyx_codeobj__45 __pyx_mstate_global->__pyx_codeobj__45
#define __pyx_codeobj__47 __pyx_mstate_global->__pyx_codeobj__47
#define __pyx_codeobj__49 __pyx_mstate_global->__pyx_codeobj__49
#define __pyx_codeobj__51 __pyx_mstate_global->__pyx_codeobj__51
#define __pyx_codeobj__53 __pyx_mstate_global->__pyx_codeobj__53
#endif
/* #### Code section: module_code ### */

/* "aim/storage/prefixview.py":32
 *     """
 * 
 *     def __init__(             # <<<<<<<<<<<<<<
 *         self,
 *         *,
 */

/* Python wrapper */
static int __pyx_pw_3aim_7storage_10prefixview_10PrefixView_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_3aim_7storage_10prefixview_10PrefixView_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_prefix = 0;
  struct __pyx_obj_3aim_7storage_9container_Container *__pyx_v_container = 0;
  PyObject *__pyx_v_read_only = 0;
  CYTHON_UNUSED const Py_ssize_t __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    #if CYTHON_USE_MODULE_STATE
    PyObject **__pyx_pyargnames[] = {&__pyx_n_s_container,&__pyx_n_s_prefix,&__pyx_n_s_read_only,0};
    #else
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_container,&__pyx_n_s_prefix,&__pyx_n_s_read_only,0};
    #endif
    PyObject* values[3] = {0,0,0};
    values[1] = ((PyObject*)__pyx_kp_b_);

    /* "aim/storage/prefixview.py":37
 *         prefix: bytes = b'',
 *         container: Container,
 *         read_only: bool = None             # <<<<<<<<<<<<<<
 *     ) -> None:
 *         self.prefix = prefix
 */
    values[2] = ((PyObject *)Py_None);
    if (likely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      switch (__pyx_nargs) {
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = __Pyx_NumKwargs_VARARGS(__pyx_kwds);
      if (likely((values[0] = __Pyx_GetKwValue_VARARGS(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_container)) != 0)) kw_args--;
      else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 32, __pyx_L3_error)
      else {
        __Pyx_RaiseKeywordRequired("__init__", __pyx_n_s_container); __PYX_ERR(0, 32, __pyx_L3_error)
      }
      if (kw_args > 0 && likely(kw_args <= 2)) {
        Py_ssize_t index;
        for (index = 1; index < 3 && kw_args > 0; index++) {
          PyObject* value = __Pyx_GetKwValue_VARARGS(__pyx_kwds, __pyx_kwvalues, *__pyx_pyargnames[index]);
          if (value) { values[index] = value; kw_args--; }
          else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 32, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values + 0, 0, "__init__") < 0)) __PYX_ERR(0, 32, __pyx_L3_error)
      }
    } else if (unlikely(__pyx_nargs != 0)) {
      goto __pyx_L5_argtuple_error;
    } else {
      __Pyx_RaiseKeywordRequired("__init__", __pyx_n_s_container); __PYX_ERR(0, 32, __pyx_L3_error)
    }
    __pyx_v_container = ((struct __pyx_obj_3aim_7storage_9container_Container *)values[0]);
    __pyx_v_prefix = ((PyObject*)values[1]);
    __pyx_v_read_only = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 0, 0, __pyx_nargs); __PYX_ERR(0, 32, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("aim.storage.prefixview.PrefixView.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_prefix), (&PyBytes_Type), 1, "prefix", 1))) __PYX_ERR(0, 35, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_container), __pyx_ptype_3aim_7storage_9container_Container, 1, "container", 0))) __PYX_ERR(0, 36, __pyx_L1_error)
  __pyx_r = __pyx_pf_3aim_7storage_10prefixview_10PrefixView___init__(((struct __pyx_obj_3aim_7storage_10prefixview_PrefixView *)__pyx_v_self), __pyx_v_prefix, __pyx_v_container, __pyx_v_read_only);

  /* "aim/storage/prefixview.py":32
 *     """
 * 
 *     def __init__(             # <<<<<<<<<<<<<<
 *         self,
 *         *,
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_3aim_7storage_10prefixview_10PrefixView___init__(struct __pyx_obj_3aim_7storage_10prefixview_PrefixView *__pyx_v_self, PyObject *__pyx_v_prefix, struct __pyx_obj_3aim_7storage_9container_Container *__pyx_v_container, PyObject *__pyx_v_read_only) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "aim/storage/prefixview.py":39
 *         read_only: bool = None
 *     ) -> None:
 *         self.prefix = prefix             # <<<<<<<<<<<<<<
 *         self.parent = container
 * 
 */
  __Pyx_INCREF(__pyx_v_prefix);
  __Pyx_GIVEREF(__pyx_v_prefix);
  __Pyx_GOTREF(__pyx_v_self->prefix);
  __Pyx_DECREF(__pyx_v_self->prefix);
  __pyx_v_self->prefix = __pyx_v_prefix;

  /* "aim/storage/prefixview.py":40
 *     ) -> None:
 *         self.prefix = prefix
 *         self.parent = container             # <<<<<<<<<<<<<<
 * 
 *         self.read_only = read_only
 */
  __Pyx_INCREF((PyObject *)__pyx_v_container);
  __Pyx_GIVEREF((PyObject *)__pyx_v_container);
  __Pyx_GOTREF(__pyx_v_self->parent);
  __Pyx_DECREF(__pyx_v_self->parent);
  __pyx_v_self->parent = ((PyObject *)__pyx_v_container);

  /* "aim/storage/prefixview.py":42
 *         self.parent = container
 * 
 *         self.read_only = read_only             # <<<<<<<<<<<<<<
 * 
 *     def close(self):
 */
  __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_v_read_only); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 42, __pyx_L1_error)
  __pyx_v_self->read_only = __pyx_t_1;

  /* "aim/storage/prefixview.py":32
 *     """
 * 
 *     def __init__(             # <<<<<<<<<<<<<<
 *         self,
 *         *,
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("aim.storage.prefixview.PrefixView.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "aim/storage/prefixview.py":44
 *         self.read_only = read_only
 * 
 *     def close(self):             # <<<<<<<<<<<<<<
 *         """Close all the resources."""
 *         pass
 */

static PyObject *__pyx_pw_3aim_7storage_10prefixview_10PrefixView_3close(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static void __pyx_f_3aim_7storage_10prefixview_10PrefixView_close(CYTHON_UNUSED struct __pyx_obj_3aim_7storage_10prefixview_PrefixView *__pyx_v_self, int __pyx_skip_dispatch) {
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("close", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || __Pyx_PyType_HasFeature(Py_TYPE(((PyObject *)__pyx_v_self)), (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_typedict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_close); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 44, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      #ifdef __Pyx_CyFunction_USED
      if (!__Pyx_IsCyOrPyCFunction(__pyx_t_1)
      #else
      if (!PyCFunction_Check(__pyx_t_1)
      #endif
              || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_3aim_7storage_10prefixview_10PrefixView_3close)) {
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        __pyx_t_5 = 0;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
            __pyx_t_5 = 1;
          }
        }
        {
          PyObject *__pyx_callargs[1] = {__pyx_t_4, };
          __pyx_t_2 = __Pyx_PyObject_FastCall(__pyx_t_3, __pyx_callargs+1-__pyx_t_5, 0+__pyx_t_5);
          __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
          if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 44, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        }
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_typedict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "aim/storage/prefixview.py":45
 * 
 *     def close(self):
 *         """Close all the resources."""             # <<<<<<<<<<<<<<
 *         pass
 * 
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_WriteUnraisable("aim.storage.prefixview.PrefixView.close", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
}

/* "aim/storage/prefixview.py":44
 *         self.read_only = read_only
 * 
 *     def close(self):             # <<<<<<<<<<<<<<
 *         """Close all the resources."""
 *         pass
 */

/* Python wrapper */
static PyObject *__pyx_pw_3aim_7storage_10prefixview_10PrefixView_3close(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_3aim_7storage_10prefixview_10PrefixView_2close, "Close all the resources.");
static PyMethodDef __pyx_mdef_3aim_7storage_10prefixview_10PrefixView_3close = {"close", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_3aim_7storage_10prefixview_10PrefixView_3close, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_3aim_7storage_10prefixview_10PrefixView_2close};
static PyObject *__pyx_pw_3aim_7storage_10prefixview_10PrefixView_3close(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED const Py_ssize_t __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("close (wrapper)", 0);
  if (unlikely(__pyx_nargs > 0)) {
    __Pyx_RaiseArgtupleInvalid("close", 1, 0, 0, __pyx_nargs); return NULL;}
  if (unlikely(__pyx_kwds) && __Pyx_NumKwargs_FASTCALL(__pyx_kwds) && unlikely(!__Pyx_CheckKeywordStrings(__pyx_kwds, "close", 0))) return NULL;
  __pyx_r = __pyx_pf_3aim_7storage_10prefixview_10PrefixView_2close(((struct __pyx_obj_3aim_7storage_10prefixview_PrefixView *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_3aim_7storage_10prefixview_10PrefixView_2close(struct __pyx_obj_3aim_7storage_10prefixview_PrefixView *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("close", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_void_to_None(__pyx_f_3aim_7storage_10prefixview_10PrefixView_close(__pyx_v_self, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 44, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("aim.storage.prefixview.PrefixView.close", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "aim/storage/prefixview.py":48
 *         pass
 * 
 *     def preload(self):             # <<<<<<<<<<<<<<
 *         """Preload the Container.
 * 
 */

static PyObject *__pyx_pw_3aim_7storage_10prefixview_10PrefixView_5preload(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static void __pyx_f_3aim_7storage_10prefixview_10PrefixView_preload(struct __pyx_obj_3aim_7storage_10prefixview_PrefixView *__pyx_v_self, int __pyx_skip_dispatch) {
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("preload", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || __Pyx_PyType_HasFeature(Py_TYPE(((PyObject *)__pyx_v_self)), (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_typedict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_preload); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 48, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      #ifdef __Pyx_CyFunction_USED
      if (!__Pyx_IsCyOrPyCFunction(__pyx_t_1)
      #else
      if (!PyCFunction_Check(__pyx_t_1)
      #endif
              || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_3aim_7storage_10prefixview_10PrefixView_5preload)) {
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        __pyx_t_5 = 0;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
            __pyx_t_5 = 1;
          }
        }
        {
          PyObject *__pyx_callargs[1] = {__pyx_t_4, };
          __pyx_t_2 = __Pyx_PyObject_FastCall(__pyx_t_3, __pyx_callargs+1-__pyx_t_5, 0+__pyx_t_5);
          __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
          if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 48, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        }
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_typedict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "aim/storage/prefixview.py":56
 *         operation that will cause an actual read / write access.
 *         """
 *         self.parent.preload()             # <<<<<<<<<<<<<<
 * 
 *     def finalize(self, index: Container):
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->parent, __pyx_n_s_preload); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 56, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_5 = 1;
    }
  }
  {
    PyObject *__pyx_callargs[1] = {__pyx_t_3, };
    __pyx_t_1 = __Pyx_PyObject_FastCall(__pyx_t_2, __pyx_callargs+1-__pyx_t_5, 0+__pyx_t_5);
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 56, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "aim/storage/prefixview.py":48
 *         pass
 * 
 *     def preload(self):             # <<<<<<<<<<<<<<
 *         """Preload the Container.
 * 
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_WriteUnraisable("aim.storage.prefixview.PrefixView.preload", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
}

/* Python wrapper */
static PyObject *__pyx_pw_3aim_7storage_10prefixview_10PrefixView_5preload(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_3aim_7storage_10prefixview_10PrefixView_4preload, "Preload the Container.\n\n        The interface of Container is designed in such a way that (almost) all\n        the operations are supported to be done lazily.\n        Sometimes there is need to preload the storage without performing an\n        operation that will cause an actual read / write access.\n        ");
static PyMethodDef __pyx_mdef_3aim_7storage_10prefixview_10PrefixView_5preload = {"preload", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_3aim_7storage_10prefixview_10PrefixView_5preload, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_3aim_7storage_10prefixview_10PrefixView_4preload};
static PyObject *__pyx_pw_3aim_7storage_10prefixview_10PrefixView_5preload(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED const Py_ssize_t __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("preload (wrapper)", 0);
  if (unlikely(__pyx_nargs > 0)) {
    __Pyx_RaiseArgtupleInvalid("preload", 1, 0, 0, __pyx_nargs); return NULL;}
  if (unlikely(__pyx_kwds) && __Pyx_NumKwargs_FASTCALL(__pyx_kwds) && unlikely(!__Pyx_CheckKeywordStrings(__pyx_kwds, "preload", 0))) return NULL;
  __pyx_r = __pyx_pf_3aim_7storage_10prefixview_10PrefixView_4preload(((struct __pyx_obj_3aim_7storage_10prefixview_PrefixView *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_3aim_7storage_10prefixview_10PrefixView_4preload(struct __pyx_obj_3aim_7storage_10prefixview_PrefixView *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("preload", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_void_to_None(__pyx_f_3aim_7storage_10prefixview_10PrefixView_preload(__pyx_v_self, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 48, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("aim.storage.prefixview.PrefixView.preload", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "aim/storage/prefixview.py":58
 *         self.parent.preload()
 * 
 *     def finalize(self, index: Container):             # <<<<<<<<<<<<<<
 *         """Finalize the Container.
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_3aim_7storage_10prefixview_10PrefixView_7finalize(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_3aim_7storage_10prefixview_10PrefixView_6finalize, "Finalize the Container.\n\n        Perform operations of compactions, indexing, optimization, etc.\n        ");
static PyMethodDef __pyx_mdef_3aim_7storage_10prefixview_10PrefixView_7finalize = {"finalize", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_3aim_7storage_10prefixview_10PrefixView_7finalize, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_3aim_7storage_10prefixview_10PrefixView_6finalize};
static PyObject *__pyx_pw_3aim_7storage_10prefixview_10PrefixView_7finalize(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  struct __pyx_obj_3aim_7storage_9container_Container *__pyx_v_index = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED const Py_ssize_t __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("finalize (wrapper)", 0);
  {
    #if CYTHON_USE_MODULE_STATE
    PyObject **__pyx_pyargnames[] = {&__pyx_n_s_index,0};
    #else
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_index,0};
    #endif
    PyObject* values[1] = {0};
    if (__pyx_kwds) {
      Py_ssize_t kw_args;
      switch (__pyx_nargs) {
        case  1: values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = __Pyx_NumKwargs_FASTCALL(__pyx_kwds);
      switch (__pyx_nargs) {
        case  0:
        if (likely((values[0] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_index)) != 0)) kw_args--;
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 58, __pyx_L3_error)
        else goto __pyx_L5_argtuple_error;
      }
      if (unlikely(kw_args > 0)) {
        const Py_ssize_t kwd_pos_args = __pyx_nargs;
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values + 0, kwd_pos_args, "finalize") < 0)) __PYX_ERR(0, 58, __pyx_L3_error)
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
    }
    __pyx_v_index = ((struct __pyx_obj_3aim_7storage_9container_Container *)values[0]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("finalize", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 58, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("aim.storage.prefixview.PrefixView.finalize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_index), __pyx_ptype_3aim_7storage_9container_Container, 1, "index", 0))) __PYX_ERR(0, 58, __pyx_L1_error)
  __pyx_r = __pyx_pf_3aim_7storage_10prefixview_10PrefixView_6finalize(((struct __pyx_obj_3aim_7storage_10prefixview_PrefixView *)__pyx_v_self), __pyx_v_index);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_3aim_7storage_10prefixview_10PrefixView_6finalize(struct __pyx_obj_3aim_7storage_10prefixview_PrefixView *__pyx_v_self, struct __pyx_obj_3aim_7storage_9container_Container *__pyx_v_index) {
  PyObject *__pyx_v_prefix = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("finalize", 0);

  /* "aim/storage/prefixview.py":63
 *         Perform operations of compactions, indexing, optimization, etc.
 *         """
 *         prefix = self.absolute_path()             # <<<<<<<<<<<<<<
 *         # Shadowing
 *         index.delete_range(prefix, prefix + b'\xff')  # TODO check validity
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_3aim_7storage_10prefixview_PrefixView *)__pyx_v_self->__pyx_base.__pyx_vtab)->absolute_path(__pyx_v_self, 0, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 63, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_prefix = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "aim/storage/prefixview.py":65
 *         prefix = self.absolute_path()
 *         # Shadowing
 *         index.delete_range(prefix, prefix + b'\xff')  # TODO check validity             # <<<<<<<<<<<<<<
 *         self.parent.finalize(index=index)
 * 
 */
  __pyx_t_1 = PyNumber_Add(__pyx_v_prefix, __pyx_kp_b__2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 65, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  ((struct __pyx_vtabstruct_3aim_7storage_9container_Container *)__pyx_v_index->__pyx_vtab)->delete_range(__pyx_v_index, __pyx_v_prefix, ((PyObject*)__pyx_t_1), 0, NULL);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "aim/storage/prefixview.py":66
 *         # Shadowing
 *         index.delete_range(prefix, prefix + b'\xff')  # TODO check validity
 *         self.parent.finalize(index=index)             # <<<<<<<<<<<<<<
 * 
 *     def absolute_path(
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->parent, __pyx_n_s_finalize); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 66, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 66, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_index, ((PyObject *)__pyx_v_index)) < 0) __PYX_ERR(0, 66, __pyx_L1_error)
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_empty_tuple, __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 66, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "aim/storage/prefixview.py":58
 *         self.parent.preload()
 * 
 *     def finalize(self, index: Container):             # <<<<<<<<<<<<<<
 *         """Finalize the Container.
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("aim.storage.prefixview.PrefixView.finalize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_prefix);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "aim/storage/prefixview.py":68
 *         self.parent.finalize(index=index)
 * 
 *     def absolute_path(             # <<<<<<<<<<<<<<
 *         self,
 *         path: bytes = None
 */

static PyObject *__pyx_pw_3aim_7storage_10prefixview_10PrefixView_9absolute_path(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyObject *__pyx_f_3aim_7storage_10prefixview_10PrefixView_absolute_path(struct __pyx_obj_3aim_7storage_10prefixview_PrefixView *__pyx_v_self, int __pyx_skip_dispatch, struct __pyx_opt_args_3aim_7storage_10prefixview_10PrefixView_absolute_path *__pyx_optional_args) {

  /* "aim/storage/prefixview.py":70
 *     def absolute_path(
 *         self,
 *         path: bytes = None             # <<<<<<<<<<<<<<
 *     ) -> bytes:
 *         """Returns the absolute path for the given relative `path`.
 */
  PyObject *__pyx_v_path = ((PyObject*)Py_None);
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_t_6;
  int __pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("absolute_path", 0);
  if (__pyx_optional_args) {
    if (__pyx_optional_args->__pyx_n > 0) {
      __pyx_v_path = __pyx_optional_args->path;
    }
  }

  /* "aim/storage/prefixview.py":68
 *         self.parent.finalize(index=index)
 * 
 *     def absolute_path(             # <<<<<<<<<<<<<<
 *         self,
 *         path: bytes = None
 */
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || __Pyx_PyType_HasFeature(Py_TYPE(((PyObject *)__pyx_v_self)), (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_typedict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_absolute_path); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 68, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      #ifdef __Pyx_CyFunction_USED
      if (!__Pyx_IsCyOrPyCFunction(__pyx_t_1)
      #else
      if (!PyCFunction_Check(__pyx_t_1)
      #endif
              || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_3aim_7storage_10prefixview_10PrefixView_9absolute_path)) {
        __Pyx_XDECREF(__pyx_r);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        __pyx_t_5 = 0;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
            __pyx_t_5 = 1;
          }
        }
        {
          PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_v_path};
          __pyx_t_2 = __Pyx_PyObject_FastCall(__pyx_t_3, __pyx_callargs+1-__pyx_t_5, 1+__pyx_t_5);
          __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
          if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 68, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        }
        if (!(likely(PyBytes_CheckExact(__pyx_t_2))||((__pyx_t_2) == Py_None) || __Pyx_RaiseUnexpectedTypeError("bytes", __pyx_t_2))) __PYX_ERR(0, 68, __pyx_L1_error)
        __pyx_r = ((PyObject*)__pyx_t_2);
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_typedict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "aim/storage/prefixview.py":79
 *         `join('a/b/c/', 'e/f/') == 'a/b/c/' + 'e/f/' = 'a/b/c/e/f/'`
 *         """
 *         if path is None:             # <<<<<<<<<<<<<<
 *             return self.prefix
 *         return self.prefix + path
 */
  __pyx_t_6 = (__pyx_v_path == ((PyObject*)Py_None));
  __pyx_t_7 = (__pyx_t_6 != 0);
  if (__pyx_t_7) {

    /* "aim/storage/prefixview.py":80
 *         """
 *         if path is None:
 *             return self.prefix             # <<<<<<<<<<<<<<
 *         return self.prefix + path
 * 
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_v_self->prefix);
    __pyx_r = __pyx_v_self->prefix;
    goto __pyx_L0;

    /* "aim/storage/prefixview.py":79
 *         `join('a/b/c/', 'e/f/') == 'a/b/c/' + 'e/f/' = 'a/b/c/e/f/'`
 *         """
 *         if path is None:             # <<<<<<<<<<<<<<
 *             return self.prefix
 *         return self.prefix + path
 */
  }

  /* "aim/storage/prefixview.py":81
 *         if path is None:
 *             return self.prefix
 *         return self.prefix + path             # <<<<<<<<<<<<<<
 * 
 *     def get(
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyNumber_Add(__pyx_v_self->prefix, __pyx_v_path); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 81, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "aim/storage/prefixview.py":68
 *         self.parent.finalize(index=index)
 * 
 *     def absolute_path(             # <<<<<<<<<<<<<<
 *         self,
 *         path: bytes = None
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("aim.storage.prefixview.PrefixView.absolute_path", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_3aim_7storage_10prefixview_10PrefixView_9absolute_path(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_3aim_7storage_10prefixview_10PrefixView_8absolute_path, "Returns the absolute path for the given relative `path`.\n\n        Path separators / sentinels should be handled in higher level so that\n        `join(a, b) == a + b` property holds. This can be easily achieved by\n        having all the paths end with the sentinel:\n        `join('a/b/c/', 'e/f/') == 'a/b/c/' + 'e/f/' = 'a/b/c/e/f/'`\n        ");
static PyMethodDef __pyx_mdef_3aim_7storage_10prefixview_10PrefixView_9absolute_path = {"absolute_path", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_3aim_7storage_10prefixview_10PrefixView_9absolute_path, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_3aim_7storage_10prefixview_10PrefixView_8absolute_path};
static PyObject *__pyx_pw_3aim_7storage_10prefixview_10PrefixView_9absolute_path(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_path = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED const Py_ssize_t __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("absolute_path (wrapper)", 0);
  {
    #if CYTHON_USE_MODULE_STATE
    PyObject **__pyx_pyargnames[] = {&__pyx_n_s_path,0};
    #else
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_path,0};
    #endif
    PyObject* values[1] = {0};

    /* "aim/storage/prefixview.py":70
 *     def absolute_path(
 *         self,
 *         path: bytes = None             # <<<<<<<<<<<<<<
 *     ) -> bytes:
 *         """Returns the absolute path for the given relative `path`.
 */
    values[0] = ((PyObject*)Py_None);
    if (__pyx_kwds) {
      Py_ssize_t kw_args;
      switch (__pyx_nargs) {
        case  1: values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = __Pyx_NumKwargs_FASTCALL(__pyx_kwds);
      switch (__pyx_nargs) {
        case  0:
        if (kw_args > 0) {
          PyObject* value = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_path);
          if (value) { values[0] = value; kw_args--; }
          else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 68, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        const Py_ssize_t kwd_pos_args = __pyx_nargs;
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values + 0, kwd_pos_args, "absolute_path") < 0)) __PYX_ERR(0, 68, __pyx_L3_error)
      }
    } else {
      switch (__pyx_nargs) {
        case  1: values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_path = ((PyObject*)values[0]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("absolute_path", 0, 0, 1, __pyx_nargs); __PYX_ERR(0, 68, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("aim.storage.prefixview.PrefixView.absolute_path", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_path), (&PyBytes_Type), 1, "path", 1))) __PYX_ERR(0, 70, __pyx_L1_error)
  __pyx_r = __pyx_pf_3aim_7storage_10prefixview_10PrefixView_8absolute_path(((struct __pyx_obj_3aim_7storage_10prefixview_PrefixView *)__pyx_v_self), __pyx_v_path);

  /* "aim/storage/prefixview.py":68
 *         self.parent.finalize(index=index)
 * 
 *     def absolute_path(             # <<<<<<<<<<<<<<
 *         self,
 *         path: bytes = None
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_3aim_7storage_10prefixview_10PrefixView_8absolute_path(struct __pyx_obj_3aim_7storage_10prefixview_PrefixView *__pyx_v_self, PyObject *__pyx_v_path) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  struct __pyx_opt_args_3aim_7storage_10prefixview_10PrefixView_absolute_path __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("absolute_path", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2.__pyx_n = 1;
  __pyx_t_2.path = __pyx_v_path;
  __pyx_t_1 = __pyx_vtabptr_3aim_7storage_10prefixview_PrefixView->absolute_path(__pyx_v_self, 1, &__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 68, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("aim.storage.prefixview.PrefixView.absolute_path", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "aim/storage/prefixview.py":83
 *         return self.prefix + path
 * 
 *     def get(             # <<<<<<<<<<<<<<
 *         self,
 *         key: ContainerKey,
 */

static PyObject *__pyx_pw_3aim_7storage_10prefixview_10PrefixView_11get(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyObject *__pyx_f_3aim_7storage_10prefixview_10PrefixView_get(struct __pyx_obj_3aim_7storage_10prefixview_PrefixView *__pyx_v_self, PyObject *__pyx_v_key, int __pyx_skip_dispatch, struct __pyx_opt_args_3aim_7storage_10prefixview_10PrefixView_get *__pyx_optional_args) {

  /* "aim/storage/prefixview.py":86
 *         self,
 *         key: ContainerKey,
 *         default=None             # <<<<<<<<<<<<<<
 *     ) -> ContainerValue:
 *         """Returns the value by the given `key` if it exists else `default`.
 */
  PyObject *__pyx_v_default = ((PyObject *)Py_None);
  PyObject *__pyx_v_path = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  struct __pyx_opt_args_3aim_7storage_10prefixview_10PrefixView_absolute_path __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get", 0);
  if (__pyx_optional_args) {
    if (__pyx_optional_args->__pyx_n > 0) {
      __pyx_v_default = __pyx_optional_args->__pyx_default;
    }
  }

  /* "aim/storage/prefixview.py":83
 *         return self.prefix + path
 * 
 *     def get(             # <<<<<<<<<<<<<<
 *         self,
 *         key: ContainerKey,
 */
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || __Pyx_PyType_HasFeature(Py_TYPE(((PyObject *)__pyx_v_self)), (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_typedict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_get); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 83, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      #ifdef __Pyx_CyFunction_USED
      if (!__Pyx_IsCyOrPyCFunction(__pyx_t_1)
      #else
      if (!PyCFunction_Check(__pyx_t_1)
      #endif
              || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_3aim_7storage_10prefixview_10PrefixView_11get)) {
        __Pyx_XDECREF(__pyx_r);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        __pyx_t_5 = 0;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
            __pyx_t_5 = 1;
          }
        }
        {
          PyObject *__pyx_callargs[3] = {__pyx_t_4, __pyx_v_key, __pyx_v_default};
          __pyx_t_2 = __Pyx_PyObject_FastCall(__pyx_t_3, __pyx_callargs+1-__pyx_t_5, 2+__pyx_t_5);
          __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
          if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 83, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        }
        __pyx_r = __pyx_t_2;
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_typedict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "aim/storage/prefixview.py":92
 *         The `default` is :obj:`None` by default.
 *         """
 *         path = self.absolute_path(key)             # <<<<<<<<<<<<<<
 *         return self.parent.get(path, default)
 * 
 */
  __pyx_t_6.__pyx_n = 1;
  __pyx_t_6.path = __pyx_v_key;
  __pyx_t_1 = ((struct __pyx_vtabstruct_3aim_7storage_10prefixview_PrefixView *)__pyx_v_self->__pyx_base.__pyx_vtab)->absolute_path(__pyx_v_self, 0, &__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 92, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_path = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "aim/storage/prefixview.py":93
 *         """
 *         path = self.absolute_path(key)
 *         return self.parent.get(path, default)             # <<<<<<<<<<<<<<
 * 
 *     def __getitem__(
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->parent, __pyx_n_s_get); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 93, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_5 = 1;
    }
  }
  {
    PyObject *__pyx_callargs[3] = {__pyx_t_3, __pyx_v_path, __pyx_v_default};
    __pyx_t_1 = __Pyx_PyObject_FastCall(__pyx_t_2, __pyx_callargs+1-__pyx_t_5, 2+__pyx_t_5);
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 93, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  }
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "aim/storage/prefixview.py":83
 *         return self.prefix + path
 * 
 *     def get(             # <<<<<<<<<<<<<<
 *         self,
 *         key: ContainerKey,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("aim.storage.prefixview.PrefixView.get", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_path);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_3aim_7storage_10prefixview_10PrefixView_11get(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_3aim_7storage_10prefixview_10PrefixView_10get, "Returns the value by the given `key` if it exists else `default`.\n\n        The `default` is :obj:`None` by default.\n        ");
static PyMethodDef __pyx_mdef_3aim_7storage_10prefixview_10PrefixView_11get = {"get", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_3aim_7storage_10prefixview_10PrefixView_11get, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_3aim_7storage_10prefixview_10PrefixView_10get};
static PyObject *__pyx_pw_3aim_7storage_10prefixview_10PrefixView_11get(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_key = 0;
  PyObject *__pyx_v_default = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED const Py_ssize_t __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get (wrapper)", 0);
  {
    #if CYTHON_USE_MODULE_STATE
    PyObject **__pyx_pyargnames[] = {&__pyx_n_s_key,&__pyx_n_s_default,0};
    #else
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_key,&__pyx_n_s_default,0};
    #endif
    PyObject* values[2] = {0,0};

    /* "aim/storage/prefixview.py":86
 *         self,
 *         key: ContainerKey,
 *         default=None             # <<<<<<<<<<<<<<
 *     ) -> ContainerValue:
 *         """Returns the value by the given `key` if it exists else `default`.
 */
    values[1] = ((PyObject *)Py_None);
    if (__pyx_kwds) {
      Py_ssize_t kw_args;
      switch (__pyx_nargs) {
        case  2: values[1] = __Pyx_Arg_FASTCALL(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = __Pyx_NumKwargs_FASTCALL(__pyx_kwds);
      switch (__pyx_nargs) {
        case  0:
        if (likely((values[0] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_key)) != 0)) kw_args--;
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 83, __pyx_L3_error)
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (kw_args > 0) {
          PyObject* value = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_default);
          if (value) { values[1] = value; kw_args--; }
          else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 83, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        const Py_ssize_t kwd_pos_args = __pyx_nargs;
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values + 0, kwd_pos_args, "get") < 0)) __PYX_ERR(0, 83, __pyx_L3_error)
      }
    } else {
      switch (__pyx_nargs) {
        case  2: values[1] = __Pyx_Arg_FASTCALL(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_key = ((PyObject*)values[0]);
    __pyx_v_default = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("get", 0, 1, 2, __pyx_nargs); __PYX_ERR(0, 83, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("aim.storage.prefixview.PrefixView.get", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_key), (&PyBytes_Type), 1, "key", 1))) __PYX_ERR(0, 85, __pyx_L1_error)
  __pyx_r = __pyx_pf_3aim_7storage_10prefixview_10PrefixView_10get(((struct __pyx_obj_3aim_7storage_10prefixview_PrefixView *)__pyx_v_self), __pyx_v_key, __pyx_v_default);

  /* "aim/storage/prefixview.py":83
 *         return self.prefix + path
 * 
 *     def get(             # <<<<<<<<<<<<<<
 *         self,
 *         key: ContainerKey,
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_3aim_7storage_10prefixview_10PrefixView_10get(struct __pyx_obj_3aim_7storage_10prefixview_PrefixView *__pyx_v_self, PyObject *__pyx_v_key, PyObject *__pyx_v_default) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  struct __pyx_opt_args_3aim_7storage_9container_9Container_get __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2.__pyx_n = 1;
  __pyx_t_2.__pyx_default = __pyx_v_default;
  __pyx_t_1 = __pyx_vtabptr_3aim_7storage_10prefixview_PrefixView->__pyx_base.get(((struct __pyx_obj_3aim_7storage_9container_Container *)__pyx_v_self), __pyx_v_key, 1, &__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 83, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("aim.storage.prefixview.PrefixView.get", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "aim/storage/prefixview.py":95
 *         return self.parent.get(path, default)
 * 
 *     def __getitem__(             # <<<<<<<<<<<<<<
 *         self,
 *         key: ContainerKey
 */

/* Python wrapper */
static PyObject *__pyx_pw_3aim_7storage_10prefixview_10PrefixView_13__getitem__(PyObject *__pyx_v_self, PyObject *__pyx_v_key); /*proto*/
PyDoc_STRVAR(__pyx_doc_3aim_7storage_10prefixview_10PrefixView_12__getitem__, "Returns the value by the given `key`.");
#if CYTHON_COMPILING_IN_CPYTHON
struct wrapperbase __pyx_wrapperbase_3aim_7storage_10prefixview_10PrefixView_12__getitem__;
#endif
static PyObject *__pyx_pw_3aim_7storage_10prefixview_10PrefixView_13__getitem__(PyObject *__pyx_v_self, PyObject *__pyx_v_key) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__getitem__ (wrapper)", 0);
  __pyx_r = __pyx_pf_3aim_7storage_10prefixview_10PrefixView_12__getitem__(((struct __pyx_obj_3aim_7storage_10prefixview_PrefixView *)__pyx_v_self), ((PyObject *)__pyx_v_key));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_3aim_7storage_10prefixview_10PrefixView_12__getitem__(struct __pyx_obj_3aim_7storage_10prefixview_PrefixView *__pyx_v_self, PyObject *__pyx_v_key) {
  PyObject *__pyx_v_path = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  struct __pyx_opt_args_3aim_7storage_10prefixview_10PrefixView_absolute_path __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__getitem__", 0);

  /* "aim/storage/prefixview.py":100
 *     ) -> ContainerValue:
 *         """Returns the value by the given `key`."""
 *         path = self.absolute_path(key)             # <<<<<<<<<<<<<<
 *         return self.parent[path]
 * 
 */
  if (!(likely(PyBytes_CheckExact(__pyx_v_key))||((__pyx_v_key) == Py_None) || __Pyx_RaiseUnexpectedTypeError("bytes", __pyx_v_key))) __PYX_ERR(0, 100, __pyx_L1_error)
  __pyx_t_2.__pyx_n = 1;
  __pyx_t_2.path = ((PyObject*)__pyx_v_key);
  __pyx_t_1 = ((struct __pyx_vtabstruct_3aim_7storage_10prefixview_PrefixView *)__pyx_v_self->__pyx_base.__pyx_vtab)->absolute_path(__pyx_v_self, 0, &__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 100, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_path = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "aim/storage/prefixview.py":101
 *         """Returns the value by the given `key`."""
 *         path = self.absolute_path(key)
 *         return self.parent[path]             # <<<<<<<<<<<<<<
 * 
 *     def set(
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyObject_GetItem(__pyx_v_self->parent, __pyx_v_path); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 101, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "aim/storage/prefixview.py":95
 *         return self.parent.get(path, default)
 * 
 *     def __getitem__(             # <<<<<<<<<<<<<<
 *         self,
 *         key: ContainerKey
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("aim.storage.prefixview.PrefixView.__getitem__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_path);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "aim/storage/prefixview.py":103
 *         return self.parent[path]
 * 
 *     def set(             # <<<<<<<<<<<<<<
 *         self,
 *         key: ContainerKey,
 */

static PyObject *__pyx_pw_3aim_7storage_10prefixview_10PrefixView_15set(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static void __pyx_f_3aim_7storage_10prefixview_10PrefixView_set(struct __pyx_obj_3aim_7storage_10prefixview_PrefixView *__pyx_v_self, PyObject *__pyx_v_key, PyObject *__pyx_v_value, int __pyx_skip_dispatch, struct __pyx_opt_args_3aim_7storage_10prefixview_10PrefixView_set *__pyx_optional_args) {

  /* "aim/storage/prefixview.py":107
 *         key: ContainerKey,
 *         value: ContainerValue,
 *         store_batch=None             # <<<<<<<<<<<<<<
 *     ) -> None:
 *         """Set a value for given key, optionally store in a batch.
 */
  PyObject *__pyx_v_store_batch = ((PyObject *)Py_None);
  PyObject *__pyx_v_path = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  struct __pyx_opt_args_3aim_7storage_10prefixview_10PrefixView_absolute_path __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("set", 0);
  if (__pyx_optional_args) {
    if (__pyx_optional_args->__pyx_n > 0) {
      __pyx_v_store_batch = __pyx_optional_args->store_batch;
    }
  }

  /* "aim/storage/prefixview.py":103
 *         return self.parent[path]
 * 
 *     def set(             # <<<<<<<<<<<<<<
 *         self,
 *         key: ContainerKey,
 */
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || __Pyx_PyType_HasFeature(Py_TYPE(((PyObject *)__pyx_v_self)), (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_typedict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_set); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 103, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      #ifdef __Pyx_CyFunction_USED
      if (!__Pyx_IsCyOrPyCFunction(__pyx_t_1)
      #else
      if (!PyCFunction_Check(__pyx_t_1)
      #endif
              || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_3aim_7storage_10prefixview_10PrefixView_15set)) {
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        __pyx_t_5 = 0;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
            __pyx_t_5 = 1;
          }
        }
        {
          PyObject *__pyx_callargs[4] = {__pyx_t_4, __pyx_v_key, __pyx_v_value, __pyx_v_store_batch};
          __pyx_t_2 = __Pyx_PyObject_FastCall(__pyx_t_3, __pyx_callargs+1-__pyx_t_5, 3+__pyx_t_5);
          __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
          if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 103, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        }
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_typedict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "aim/storage/prefixview.py":117
 *         atomic writes, etc.
 *         """
 *         path = self.absolute_path(key)             # <<<<<<<<<<<<<<
 *         self.parent.set(path, value, store_batch=store_batch)
 * 
 */
  __pyx_t_6.__pyx_n = 1;
  __pyx_t_6.path = __pyx_v_key;
  __pyx_t_1 = ((struct __pyx_vtabstruct_3aim_7storage_10prefixview_PrefixView *)__pyx_v_self->__pyx_base.__pyx_vtab)->absolute_path(__pyx_v_self, 0, &__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 117, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_path = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "aim/storage/prefixview.py":118
 *         """
 *         path = self.absolute_path(key)
 *         self.parent.set(path, value, store_batch=store_batch)             # <<<<<<<<<<<<<<
 * 
 *     def __setitem__(
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->parent, __pyx_n_s_set); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 118, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 118, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_v_path);
  __Pyx_GIVEREF(__pyx_v_path);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_v_path);
  __Pyx_INCREF(__pyx_v_value);
  __Pyx_GIVEREF(__pyx_v_value);
  PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_v_value);
  __pyx_t_3 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 118, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_store_batch, __pyx_v_store_batch) < 0) __PYX_ERR(0, 118, __pyx_L1_error)
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_2, __pyx_t_3); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 118, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "aim/storage/prefixview.py":103
 *         return self.parent[path]
 * 
 *     def set(             # <<<<<<<<<<<<<<
 *         self,
 *         key: ContainerKey,
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_WriteUnraisable("aim.storage.prefixview.PrefixView.set", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_path);
  __Pyx_RefNannyFinishContext();
}

/* Python wrapper */
static PyObject *__pyx_pw_3aim_7storage_10prefixview_10PrefixView_15set(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_3aim_7storage_10prefixview_10PrefixView_14set, "Set a value for given key, optionally store in a batch.\n\n        If `store_batch` is provided, instead of the `(key, value)` being added\n        to the collection immediately, the operation is stored in a batch in\n        order to be executed in a whole with other write operations. Depending\n        on the :obj:`Conainer` implementation, this may feature transactions,\n        atomic writes, etc.\n        ");
static PyMethodDef __pyx_mdef_3aim_7storage_10prefixview_10PrefixView_15set = {"set", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_3aim_7storage_10prefixview_10PrefixView_15set, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_3aim_7storage_10prefixview_10PrefixView_14set};
static PyObject *__pyx_pw_3aim_7storage_10prefixview_10PrefixView_15set(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_key = 0;
  PyObject *__pyx_v_value = 0;
  PyObject *__pyx_v_store_batch = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED const Py_ssize_t __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("set (wrapper)", 0);
  {
    #if CYTHON_USE_MODULE_STATE
    PyObject **__pyx_pyargnames[] = {&__pyx_n_s_key,&__pyx_n_s_value,&__pyx_n_s_store_batch,0};
    #else
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_key,&__pyx_n_s_value,&__pyx_n_s_store_batch,0};
    #endif
    PyObject* values[3] = {0,0,0};

    /* "aim/storage/prefixview.py":107
 *         key: ContainerKey,
 *         value: ContainerValue,
 *         store_batch=None             # <<<<<<<<<<<<<<
 *     ) -> None:
 *         """Set a value for given key, optionally store in a batch.
 */
    values[2] = ((PyObject *)Py_None);
    if (__pyx_kwds) {
      Py_ssize_t kw_args;
      switch (__pyx_nargs) {
        case  3: values[2] = __Pyx_Arg_FASTCALL(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = __Pyx_Arg_FASTCALL(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = __Pyx_NumKwargs_FASTCALL(__pyx_kwds);
      switch (__pyx_nargs) {
        case  0:
        if (likely((values[0] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_key)) != 0)) kw_args--;
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 103, __pyx_L3_error)
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_value)) != 0)) kw_args--;
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 103, __pyx_L3_error)
        else {
          __Pyx_RaiseArgtupleInvalid("set", 0, 2, 3, 1); __PYX_ERR(0, 103, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (kw_args > 0) {
          PyObject* value = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_store_batch);
          if (value) { values[2] = value; kw_args--; }
          else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 103, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        const Py_ssize_t kwd_pos_args = __pyx_nargs;
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values + 0, kwd_pos_args, "set") < 0)) __PYX_ERR(0, 103, __pyx_L3_error)
      }
    } else {
      switch (__pyx_nargs) {
        case  3: values[2] = __Pyx_Arg_FASTCALL(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = __Pyx_Arg_FASTCALL(__pyx_args, 1);
        values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_key = ((PyObject*)values[0]);
    __pyx_v_value = values[1];
    __pyx_v_store_batch = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("set", 0, 2, 3, __pyx_nargs); __PYX_ERR(0, 103, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("aim.storage.prefixview.PrefixView.set", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_key), (&PyBytes_Type), 1, "key", 1))) __PYX_ERR(0, 105, __pyx_L1_error)
  __pyx_r = __pyx_pf_3aim_7storage_10prefixview_10PrefixView_14set(((struct __pyx_obj_3aim_7storage_10prefixview_PrefixView *)__pyx_v_self), __pyx_v_key, __pyx_v_value, __pyx_v_store_batch);

  /* "aim/storage/prefixview.py":103
 *         return self.parent[path]
 * 
 *     def set(             # <<<<<<<<<<<<<<
 *         self,
 *         key: ContainerKey,
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_3aim_7storage_10prefixview_10PrefixView_14set(struct __pyx_obj_3aim_7storage_10prefixview_PrefixView *__pyx_v_self, PyObject *__pyx_v_key, PyObject *__pyx_v_value, PyObject *__pyx_v_store_batch) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  struct __pyx_opt_args_3aim_7storage_9container_9Container_set __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("set", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1.__pyx_n = 1;
  __pyx_t_1.store_batch = __pyx_v_store_batch;
  __pyx_vtabptr_3aim_7storage_10prefixview_PrefixView->__pyx_base.set(((struct __pyx_obj_3aim_7storage_9container_Container *)__pyx_v_self), __pyx_v_key, __pyx_v_value, 1, &__pyx_t_1); 
  __pyx_t_2 = __Pyx_void_to_None(NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 103, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("aim.storage.prefixview.PrefixView.set", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "aim/storage/prefixview.py":120
 *         self.parent.set(path, value, store_batch=store_batch)
 * 
 *     def __setitem__(             # <<<<<<<<<<<<<<
 *         self,
 *         key: ContainerKey,
 */

/* Python wrapper */
static int __pyx_pw_3aim_7storage_10prefixview_10PrefixView_17__setitem__(PyObject *__pyx_v_self, PyObject *__pyx_v_key, PyObject *__pyx_v_value); /*proto*/
PyDoc_STRVAR(__pyx_doc_3aim_7storage_10prefixview_10PrefixView_16__setitem__, "Set a value for given key.");
#if CYTHON_COMPILING_IN_CPYTHON
struct wrapperbase __pyx_wrapperbase_3aim_7storage_10prefixview_10PrefixView_16__setitem__;
#endif
static int __pyx_pw_3aim_7storage_10prefixview_10PrefixView_17__setitem__(PyObject *__pyx_v_self, PyObject *__pyx_v_key, PyObject *__pyx_v_value) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setitem__ (wrapper)", 0);
  __pyx_r = __pyx_pf_3aim_7storage_10prefixview_10PrefixView_16__setitem__(((struct __pyx_obj_3aim_7storage_10prefixview_PrefixView *)__pyx_v_self), ((PyObject *)__pyx_v_key), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_3aim_7storage_10prefixview_10PrefixView_16__setitem__(struct __pyx_obj_3aim_7storage_10prefixview_PrefixView *__pyx_v_self, PyObject *__pyx_v_key, PyObject *__pyx_v_value) {
  PyObject *__pyx_v_path = NULL;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  struct __pyx_opt_args_3aim_7storage_10prefixview_10PrefixView_absolute_path __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setitem__", 0);

  /* "aim/storage/prefixview.py":126
 *     ) -> None:
 *         """Set a value for given key."""
 *         path = self.absolute_path(key)             # <<<<<<<<<<<<<<
 *         self.parent[path] = value
 * 
 */
  if (!(likely(PyBytes_CheckExact(__pyx_v_key))||((__pyx_v_key) == Py_None) || __Pyx_RaiseUnexpectedTypeError("bytes", __pyx_v_key))) __PYX_ERR(0, 126, __pyx_L1_error)
  __pyx_t_2.__pyx_n = 1;
  __pyx_t_2.path = ((PyObject*)__pyx_v_key);
  __pyx_t_1 = ((struct __pyx_vtabstruct_3aim_7storage_10prefixview_PrefixView *)__pyx_v_self->__pyx_base.__pyx_vtab)->absolute_path(__pyx_v_self, 0, &__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 126, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_path = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "aim/storage/prefixview.py":127
 *         """Set a value for given key."""
 *         path = self.absolute_path(key)
 *         self.parent[path] = value             # <<<<<<<<<<<<<<
 * 
 *     def delete(
 */
  if (unlikely((PyObject_SetItem(__pyx_v_self->parent, __pyx_v_path, __pyx_v_value) < 0))) __PYX_ERR(0, 127, __pyx_L1_error)

  /* "aim/storage/prefixview.py":120
 *         self.parent.set(path, value, store_batch=store_batch)
 * 
 *     def __setitem__(             # <<<<<<<<<<<<<<
 *         self,
 *         key: ContainerKey,
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("aim.storage.prefixview.PrefixView.__setitem__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_path);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "aim/storage/prefixview.py":129
 *         self.parent[path] = value
 * 
 *     def delete(             # <<<<<<<<<<<<<<
 *         self,
 *         key: ContainerKey,
 */

/* Python wrapper */
static PyObject *__pyx_pw_3aim_7storage_10prefixview_10PrefixView_19delete(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_3aim_7storage_10prefixview_10PrefixView_18delete, "Delete a key-value record by the given key,\n        optionally store in a batch.\n\n        If `store_batch` is provided, instead of the `(key, value)` being added\n        to the collection immediately, the operation is stored in a batch in\n        order to be executed in a whole with other write operations. Depending\n        on the :obj:`Conainer` implementation, this may feature transactions,\n        atomic writes, etc.\n        ");
static PyMethodDef __pyx_mdef_3aim_7storage_10prefixview_10PrefixView_19delete = {"delete", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_3aim_7storage_10prefixview_10PrefixView_19delete, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_3aim_7storage_10prefixview_10PrefixView_18delete};
static PyObject *__pyx_pw_3aim_7storage_10prefixview_10PrefixView_19delete(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_key = 0;
  PyObject *__pyx_v_store_batch = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED const Py_ssize_t __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("delete (wrapper)", 0);
  {
    #if CYTHON_USE_MODULE_STATE
    PyObject **__pyx_pyargnames[] = {&__pyx_n_s_key,&__pyx_n_s_store_batch,0};
    #else
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_key,&__pyx_n_s_store_batch,0};
    #endif
    PyObject* values[2] = {0,0};

    /* "aim/storage/prefixview.py":132
 *         self,
 *         key: ContainerKey,
 *         store_batch=None             # <<<<<<<<<<<<<<
 *     ):
 *         """Delete a key-value record by the given key,
 */
    values[1] = ((PyObject *)Py_None);
    if (__pyx_kwds) {
      Py_ssize_t kw_args;
      switch (__pyx_nargs) {
        case  2: values[1] = __Pyx_Arg_FASTCALL(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = __Pyx_NumKwargs_FASTCALL(__pyx_kwds);
      switch (__pyx_nargs) {
        case  0:
        if (likely((values[0] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_key)) != 0)) kw_args--;
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 129, __pyx_L3_error)
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (kw_args > 0) {
          PyObject* value = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_store_batch);
          if (value) { values[1] = value; kw_args--; }
          else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 129, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        const Py_ssize_t kwd_pos_args = __pyx_nargs;
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values + 0, kwd_pos_args, "delete") < 0)) __PYX_ERR(0, 129, __pyx_L3_error)
      }
    } else {
      switch (__pyx_nargs) {
        case  2: values[1] = __Pyx_Arg_FASTCALL(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_key = values[0];
    __pyx_v_store_batch = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("delete", 0, 1, 2, __pyx_nargs); __PYX_ERR(0, 129, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("aim.storage.prefixview.PrefixView.delete", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_3aim_7storage_10prefixview_10PrefixView_18delete(((struct __pyx_obj_3aim_7storage_10prefixview_PrefixView *)__pyx_v_self), __pyx_v_key, __pyx_v_store_batch);

  /* "aim/storage/prefixview.py":129
 *         self.parent[path] = value
 * 
 *     def delete(             # <<<<<<<<<<<<<<
 *         self,
 *         key: ContainerKey,
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_3aim_7storage_10prefixview_10PrefixView_18delete(struct __pyx_obj_3aim_7storage_10prefixview_PrefixView *__pyx_v_self, PyObject *__pyx_v_key, PyObject *__pyx_v_store_batch) {
  PyObject *__pyx_v_path = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  struct __pyx_opt_args_3aim_7storage_10prefixview_10PrefixView_absolute_path __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("delete", 0);

  /* "aim/storage/prefixview.py":143
 *         atomic writes, etc.
 *         """
 *         path = self.absolute_path(key)             # <<<<<<<<<<<<<<
 *         return self.parent.delete(path, store_batch=store_batch)
 * 
 */
  if (!(likely(PyBytes_CheckExact(__pyx_v_key))||((__pyx_v_key) == Py_None) || __Pyx_RaiseUnexpectedTypeError("bytes", __pyx_v_key))) __PYX_ERR(0, 143, __pyx_L1_error)
  __pyx_t_2.__pyx_n = 1;
  __pyx_t_2.path = ((PyObject*)__pyx_v_key);
  __pyx_t_1 = ((struct __pyx_vtabstruct_3aim_7storage_10prefixview_PrefixView *)__pyx_v_self->__pyx_base.__pyx_vtab)->absolute_path(__pyx_v_self, 0, &__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 143, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_path = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "aim/storage/prefixview.py":144
 *         """
 *         path = self.absolute_path(key)
 *         return self.parent.delete(path, store_batch=store_batch)             # <<<<<<<<<<<<<<
 * 
 *     def __delitem__(
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->parent, __pyx_n_s_delete); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 144, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = PyTuple_New(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 144, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_INCREF(__pyx_v_path);
  __Pyx_GIVEREF(__pyx_v_path);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_v_path);
  __pyx_t_4 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 144, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_store_batch, __pyx_v_store_batch) < 0) __PYX_ERR(0, 144, __pyx_L1_error)
  __pyx_t_5 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_3, __pyx_t_4); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 144, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_r = __pyx_t_5;
  __pyx_t_5 = 0;
  goto __pyx_L0;

  /* "aim/storage/prefixview.py":129
 *         self.parent[path] = value
 * 
 *     def delete(             # <<<<<<<<<<<<<<
 *         self,
 *         key: ContainerKey,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("aim.storage.prefixview.PrefixView.delete", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_path);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "aim/storage/prefixview.py":146
 *         return self.parent.delete(path, store_batch=store_batch)
 * 
 *     def __delitem__(             # <<<<<<<<<<<<<<
 *         self,
 *         key: ContainerKey
 */

/* Python wrapper */
static int __pyx_pw_3aim_7storage_10prefixview_10PrefixView_21__delitem__(PyObject *__pyx_v_self, PyObject *__pyx_v_key); /*proto*/
PyDoc_STRVAR(__pyx_doc_3aim_7storage_10prefixview_10PrefixView_20__delitem__, "Delete a key-value record by the given key.");
#if CYTHON_COMPILING_IN_CPYTHON
struct wrapperbase __pyx_wrapperbase_3aim_7storage_10prefixview_10PrefixView_20__delitem__;
#endif
static int __pyx_pw_3aim_7storage_10prefixview_10PrefixView_21__delitem__(PyObject *__pyx_v_self, PyObject *__pyx_v_key) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__delitem__ (wrapper)", 0);
  __pyx_r = __pyx_pf_3aim_7storage_10prefixview_10PrefixView_20__delitem__(((struct __pyx_obj_3aim_7storage_10prefixview_PrefixView *)__pyx_v_self), ((PyObject *)__pyx_v_key));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_3aim_7storage_10prefixview_10PrefixView_20__delitem__(struct __pyx_obj_3aim_7storage_10prefixview_PrefixView *__pyx_v_self, PyObject *__pyx_v_key) {
  PyObject *__pyx_v_path = NULL;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  struct __pyx_opt_args_3aim_7storage_10prefixview_10PrefixView_absolute_path __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__delitem__", 0);

  /* "aim/storage/prefixview.py":151
 *     ) -> None:
 *         """Delete a key-value record by the given key."""
 *         path = self.absolute_path(key)             # <<<<<<<<<<<<<<
 *         del self.parent[path]
 * 
 */
  if (!(likely(PyBytes_CheckExact(__pyx_v_key))||((__pyx_v_key) == Py_None) || __Pyx_RaiseUnexpectedTypeError("bytes", __pyx_v_key))) __PYX_ERR(0, 151, __pyx_L1_error)
  __pyx_t_2.__pyx_n = 1;
  __pyx_t_2.path = ((PyObject*)__pyx_v_key);
  __pyx_t_1 = ((struct __pyx_vtabstruct_3aim_7storage_10prefixview_PrefixView *)__pyx_v_self->__pyx_base.__pyx_vtab)->absolute_path(__pyx_v_self, 0, &__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 151, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_path = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "aim/storage/prefixview.py":152
 *         """Delete a key-value record by the given key."""
 *         path = self.absolute_path(key)
 *         del self.parent[path]             # <<<<<<<<<<<<<<
 * 
 *     def delete_range(
 */
  if (unlikely((PyObject_DelItem(__pyx_v_self->parent, __pyx_v_path) < 0))) __PYX_ERR(0, 152, __pyx_L1_error)

  /* "aim/storage/prefixview.py":146
 *         return self.parent.delete(path, store_batch=store_batch)
 * 
 *     def __delitem__(             # <<<<<<<<<<<<<<
 *         self,
 *         key: ContainerKey
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("aim.storage.prefixview.PrefixView.__delitem__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_path);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "aim/storage/prefixview.py":154
 *         del self.parent[path]
 * 
 *     def delete_range(             # <<<<<<<<<<<<<<
 *         self,
 *         begin: ContainerKey,
 */

static PyObject *__pyx_pw_3aim_7storage_10prefixview_10PrefixView_23delete_range(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static void __pyx_f_3aim_7storage_10prefixview_10PrefixView_delete_range(struct __pyx_obj_3aim_7storage_10prefixview_PrefixView *__pyx_v_self, PyObject *__pyx_v_begin, PyObject *__pyx_v_end, int __pyx_skip_dispatch, struct __pyx_opt_args_3aim_7storage_10prefixview_10PrefixView_delete_range *__pyx_optional_args) {

  /* "aim/storage/prefixview.py":158
 *         begin: ContainerKey,
 *         end: ContainerKey,
 *         store_batch=None             # <<<<<<<<<<<<<<
 *     ):
 *         """Delete all the records in the given `[begin, end)` key range."""
 */
  PyObject *__pyx_v_store_batch = ((PyObject *)Py_None);
  PyObject *__pyx_v_begin_path = NULL;
  PyObject *__pyx_v_end_path = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  struct __pyx_opt_args_3aim_7storage_10prefixview_10PrefixView_absolute_path __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("delete_range", 0);
  if (__pyx_optional_args) {
    if (__pyx_optional_args->__pyx_n > 0) {
      __pyx_v_store_batch = __pyx_optional_args->store_batch;
    }
  }

  /* "aim/storage/prefixview.py":154
 *         del self.parent[path]
 * 
 *     def delete_range(             # <<<<<<<<<<<<<<
 *         self,
 *         begin: ContainerKey,
 */
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || __Pyx_PyType_HasFeature(Py_TYPE(((PyObject *)__pyx_v_self)), (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_typedict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_delete_range); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 154, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      #ifdef __Pyx_CyFunction_USED
      if (!__Pyx_IsCyOrPyCFunction(__pyx_t_1)
      #else
      if (!PyCFunction_Check(__pyx_t_1)
      #endif
              || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_3aim_7storage_10prefixview_10PrefixView_23delete_range)) {
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        __pyx_t_5 = 0;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
            __pyx_t_5 = 1;
          }
        }
        {
          PyObject *__pyx_callargs[4] = {__pyx_t_4, __pyx_v_begin, __pyx_v_end, __pyx_v_store_batch};
          __pyx_t_2 = __Pyx_PyObject_FastCall(__pyx_t_3, __pyx_callargs+1-__pyx_t_5, 3+__pyx_t_5);
          __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
          if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 154, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        }
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_typedict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "aim/storage/prefixview.py":161
 *     ):
 *         """Delete all the records in the given `[begin, end)` key range."""
 *         begin_path = self.absolute_path(begin)             # <<<<<<<<<<<<<<
 *         end_path = self.absolute_path(end)
 *         self.parent.delete_range(begin_path, end_path,
 */
  __pyx_t_6.__pyx_n = 1;
  __pyx_t_6.path = __pyx_v_begin;
  __pyx_t_1 = ((struct __pyx_vtabstruct_3aim_7storage_10prefixview_PrefixView *)__pyx_v_self->__pyx_base.__pyx_vtab)->absolute_path(__pyx_v_self, 0, &__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 161, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_begin_path = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "aim/storage/prefixview.py":162
 *         """Delete all the records in the given `[begin, end)` key range."""
 *         begin_path = self.absolute_path(begin)
 *         end_path = self.absolute_path(end)             # <<<<<<<<<<<<<<
 *         self.parent.delete_range(begin_path, end_path,
 *                                  store_batch=store_batch)
 */
  __pyx_t_6.__pyx_n = 1;
  __pyx_t_6.path = __pyx_v_end;
  __pyx_t_1 = ((struct __pyx_vtabstruct_3aim_7storage_10prefixview_PrefixView *)__pyx_v_self->__pyx_base.__pyx_vtab)->absolute_path(__pyx_v_self, 0, &__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 162, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_end_path = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "aim/storage/prefixview.py":163
 *         begin_path = self.absolute_path(begin)
 *         end_path = self.absolute_path(end)
 *         self.parent.delete_range(begin_path, end_path,             # <<<<<<<<<<<<<<
 *                                  store_batch=store_batch)
 * 
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->parent, __pyx_n_s_delete_range); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 163, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 163, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_v_begin_path);
  __Pyx_GIVEREF(__pyx_v_begin_path);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_v_begin_path);
  __Pyx_INCREF(__pyx_v_end_path);
  __Pyx_GIVEREF(__pyx_v_end_path);
  PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_v_end_path);

  /* "aim/storage/prefixview.py":164
 *         end_path = self.absolute_path(end)
 *         self.parent.delete_range(begin_path, end_path,
 *                                  store_batch=store_batch)             # <<<<<<<<<<<<<<
 * 
 *     def next_item(
 */
  __pyx_t_3 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 164, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_store_batch, __pyx_v_store_batch) < 0) __PYX_ERR(0, 164, __pyx_L1_error)

  /* "aim/storage/prefixview.py":163
 *         begin_path = self.absolute_path(begin)
 *         end_path = self.absolute_path(end)
 *         self.parent.delete_range(begin_path, end_path,             # <<<<<<<<<<<<<<
 *                                  store_batch=store_batch)
 * 
 */
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_2, __pyx_t_3); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 163, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "aim/storage/prefixview.py":154
 *         del self.parent[path]
 * 
 *     def delete_range(             # <<<<<<<<<<<<<<
 *         self,
 *         begin: ContainerKey,
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_WriteUnraisable("aim.storage.prefixview.PrefixView.delete_range", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_begin_path);
  __Pyx_XDECREF(__pyx_v_end_path);
  __Pyx_RefNannyFinishContext();
}

/* Python wrapper */
static PyObject *__pyx_pw_3aim_7storage_10prefixview_10PrefixView_23delete_range(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_3aim_7storage_10prefixview_10PrefixView_22delete_range, "Delete all the records in the given `[begin, end)` key range.");
static PyMethodDef __pyx_mdef_3aim_7storage_10prefixview_10PrefixView_23delete_range = {"delete_range", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_3aim_7storage_10prefixview_10PrefixView_23delete_range, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_3aim_7storage_10prefixview_10PrefixView_22delete_range};
static PyObject *__pyx_pw_3aim_7storage_10prefixview_10PrefixView_23delete_range(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_begin = 0;
  PyObject *__pyx_v_end = 0;
  PyObject *__pyx_v_store_batch = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED const Py_ssize_t __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("delete_range (wrapper)", 0);
  {
    #if CYTHON_USE_MODULE_STATE
    PyObject **__pyx_pyargnames[] = {&__pyx_n_s_begin,&__pyx_n_s_end,&__pyx_n_s_store_batch,0};
    #else
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_begin,&__pyx_n_s_end,&__pyx_n_s_store_batch,0};
    #endif
    PyObject* values[3] = {0,0,0};

    /* "aim/storage/prefixview.py":158
 *         begin: ContainerKey,
 *         end: ContainerKey,
 *         store_batch=None             # <<<<<<<<<<<<<<
 *     ):
 *         """Delete all the records in the given `[begin, end)` key range."""
 */
    values[2] = ((PyObject *)Py_None);
    if (__pyx_kwds) {
      Py_ssize_t kw_args;
      switch (__pyx_nargs) {
        case  3: values[2] = __Pyx_Arg_FASTCALL(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = __Pyx_Arg_FASTCALL(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = __Pyx_NumKwargs_FASTCALL(__pyx_kwds);
      switch (__pyx_nargs) {
        case  0:
        if (likely((values[0] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_begin)) != 0)) kw_args--;
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 154, __pyx_L3_error)
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_end)) != 0)) kw_args--;
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 154, __pyx_L3_error)
        else {
          __Pyx_RaiseArgtupleInvalid("delete_range", 0, 2, 3, 1); __PYX_ERR(0, 154, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (kw_args > 0) {
          PyObject* value = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_store_batch);
          if (value) { values[2] = value; kw_args--; }
          else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 154, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        const Py_ssize_t kwd_pos_args = __pyx_nargs;
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values + 0, kwd_pos_args, "delete_range") < 0)) __PYX_ERR(0, 154, __pyx_L3_error)
      }
    } else {
      switch (__pyx_nargs) {
        case  3: values[2] = __Pyx_Arg_FASTCALL(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = __Pyx_Arg_FASTCALL(__pyx_args, 1);
        values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_begin = ((PyObject*)values[0]);
    __pyx_v_end = ((PyObject*)values[1]);
    __pyx_v_store_batch = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("delete_range", 0, 2, 3, __pyx_nargs); __PYX_ERR(0, 154, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("aim.storage.prefixview.PrefixView.delete_range", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_begin), (&PyBytes_Type), 1, "begin", 1))) __PYX_ERR(0, 156, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_end), (&PyBytes_Type), 1, "end", 1))) __PYX_ERR(0, 157, __pyx_L1_error)
  __pyx_r = __pyx_pf_3aim_7storage_10prefixview_10PrefixView_22delete_range(((struct __pyx_obj_3aim_7storage_10prefixview_PrefixView *)__pyx_v_self), __pyx_v_begin, __pyx_v_end, __pyx_v_store_batch);

  /* "aim/storage/prefixview.py":154
 *         del self.parent[path]
 * 
 *     def delete_range(             # <<<<<<<<<<<<<<
 *         self,
 *         begin: ContainerKey,
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_3aim_7storage_10prefixview_10PrefixView_22delete_range(struct __pyx_obj_3aim_7storage_10prefixview_PrefixView *__pyx_v_self, PyObject *__pyx_v_begin, PyObject *__pyx_v_end, PyObject *__pyx_v_store_batch) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  struct __pyx_opt_args_3aim_7storage_9container_9Container_delete_range __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("delete_range", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1.__pyx_n = 1;
  __pyx_t_1.store_batch = __pyx_v_store_batch;
  __pyx_vtabptr_3aim_7storage_10prefixview_PrefixView->__pyx_base.delete_range(((struct __pyx_obj_3aim_7storage_9container_Container *)__pyx_v_self), __pyx_v_begin, __pyx_v_end, 1, &__pyx_t_1); 
  __pyx_t_2 = __Pyx_void_to_None(NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 154, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("aim.storage.prefixview.PrefixView.delete_range", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "aim/storage/prefixview.py":166
 *                                  store_batch=store_batch)
 * 
 *     def next_item(             # <<<<<<<<<<<<<<
 *         self,
 *         key: ContainerKey = b''
 */

static PyObject *__pyx_pw_3aim_7storage_10prefixview_10PrefixView_25next_item(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyObject *__pyx_f_3aim_7storage_10prefixview_10PrefixView_next_item(struct __pyx_obj_3aim_7storage_10prefixview_PrefixView *__pyx_v_self, int __pyx_skip_dispatch, struct __pyx_opt_args_3aim_7storage_10prefixview_10PrefixView_next_item *__pyx_optional_args) {
  PyObject *__pyx_v_key = ((PyObject*)__pyx_kp_b_);
  PyObject *__pyx_v_path = NULL;
  PyObject *__pyx_v_keys = NULL;
  PyObject *__pyx_v_value = NULL;
  PyObject *__pyx_v__prefix = NULL;
  PyObject *__pyx_v__path = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  struct __pyx_opt_args_3aim_7storage_10prefixview_10PrefixView_absolute_path __pyx_t_6;
  PyObject *(*__pyx_t_7)(PyObject *);
  int __pyx_t_8;
  PyObject *__pyx_t_9 = NULL;
  int __pyx_t_10;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("next_item", 0);
  if (__pyx_optional_args) {
    if (__pyx_optional_args->__pyx_n > 0) {
      __pyx_v_key = __pyx_optional_args->key;
    }
  }
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || __Pyx_PyType_HasFeature(Py_TYPE(((PyObject *)__pyx_v_self)), (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_typedict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_next_item); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 166, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      #ifdef __Pyx_CyFunction_USED
      if (!__Pyx_IsCyOrPyCFunction(__pyx_t_1)
      #else
      if (!PyCFunction_Check(__pyx_t_1)
      #endif
              || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_3aim_7storage_10prefixview_10PrefixView_25next_item)) {
        __Pyx_XDECREF(__pyx_r);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        __pyx_t_5 = 0;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
            __pyx_t_5 = 1;
          }
        }
        {
          PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_v_key};
          __pyx_t_2 = __Pyx_PyObject_FastCall(__pyx_t_3, __pyx_callargs+1-__pyx_t_5, 1+__pyx_t_5);
          __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
          if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 166, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        }
        if (!(likely(PyTuple_CheckExact(__pyx_t_2))||((__pyx_t_2) == Py_None) || __Pyx_RaiseUnexpectedTypeError("tuple", __pyx_t_2))) __PYX_ERR(0, 166, __pyx_L1_error)
        __pyx_r = ((PyObject*)__pyx_t_2);
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_typedict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "aim/storage/prefixview.py":173
 *         right after the provided `key`.
 *         """
 *         path = self.absolute_path(key)             # <<<<<<<<<<<<<<
 *         keys, value = self.parent.next_item(path)
 *         if path:
 */
  __pyx_t_6.__pyx_n = 1;
  __pyx_t_6.path = __pyx_v_key;
  __pyx_t_1 = ((struct __pyx_vtabstruct_3aim_7storage_10prefixview_PrefixView *)__pyx_v_self->__pyx_base.__pyx_vtab)->absolute_path(__pyx_v_self, 0, &__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 173, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_path = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "aim/storage/prefixview.py":174
 *         """
 *         path = self.absolute_path(key)
 *         keys, value = self.parent.next_item(path)             # <<<<<<<<<<<<<<
 *         if path:
 *             _prefix, _path, keys = keys.partition(path)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->parent, __pyx_n_s_next_item); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 174, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_5 = 1;
    }
  }
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_v_path};
    __pyx_t_1 = __Pyx_PyObject_FastCall(__pyx_t_2, __pyx_callargs+1-__pyx_t_5, 1+__pyx_t_5);
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 174, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  }
  if ((likely(PyTuple_CheckExact(__pyx_t_1))) || (PyList_CheckExact(__pyx_t_1))) {
    PyObject* sequence = __pyx_t_1;
    Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
    if (unlikely(size != 2)) {
      if (size > 2) __Pyx_RaiseTooManyValuesError(2);
      else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
      __PYX_ERR(0, 174, __pyx_L1_error)
    }
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    if (likely(PyTuple_CheckExact(sequence))) {
      __pyx_t_2 = PyTuple_GET_ITEM(sequence, 0); 
      __pyx_t_3 = PyTuple_GET_ITEM(sequence, 1); 
    } else {
      __pyx_t_2 = PyList_GET_ITEM(sequence, 0); 
      __pyx_t_3 = PyList_GET_ITEM(sequence, 1); 
    }
    __Pyx_INCREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_3);
    #else
    __pyx_t_2 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 174, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 174, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    #endif
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  } else {
    Py_ssize_t index = -1;
    __pyx_t_4 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 174, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_7 = __Pyx_PyObject_GetIterNextFunc(__pyx_t_4);
    index = 0; __pyx_t_2 = __pyx_t_7(__pyx_t_4); if (unlikely(!__pyx_t_2)) goto __pyx_L3_unpacking_failed;
    __Pyx_GOTREF(__pyx_t_2);
    index = 1; __pyx_t_3 = __pyx_t_7(__pyx_t_4); if (unlikely(!__pyx_t_3)) goto __pyx_L3_unpacking_failed;
    __Pyx_GOTREF(__pyx_t_3);
    if (__Pyx_IternextUnpackEndCheck(__pyx_t_7(__pyx_t_4), 2) < 0) __PYX_ERR(0, 174, __pyx_L1_error)
    __pyx_t_7 = NULL;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    goto __pyx_L4_unpacking_done;
    __pyx_L3_unpacking_failed:;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_7 = NULL;
    if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
    __PYX_ERR(0, 174, __pyx_L1_error)
    __pyx_L4_unpacking_done:;
  }
  __pyx_v_keys = __pyx_t_2;
  __pyx_t_2 = 0;
  __pyx_v_value = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "aim/storage/prefixview.py":175
 *         path = self.absolute_path(key)
 *         keys, value = self.parent.next_item(path)
 *         if path:             # <<<<<<<<<<<<<<
 *             _prefix, _path, keys = keys.partition(path)
 *             if _prefix or _path != path:
 */
  __pyx_t_8 = (__pyx_v_path != Py_None)&&(PyBytes_GET_SIZE(__pyx_v_path) != 0);
  if (__pyx_t_8) {

    /* "aim/storage/prefixview.py":176
 *         keys, value = self.parent.next_item(path)
 *         if path:
 *             _prefix, _path, keys = keys.partition(path)             # <<<<<<<<<<<<<<
 *             if _prefix or _path != path:
 *                 raise KeyError
 */
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_keys, __pyx_n_s_partition); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 176, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_2 = NULL;
    __pyx_t_5 = 0;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_2)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_2);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
        __pyx_t_5 = 1;
      }
    }
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_v_path};
      __pyx_t_1 = __Pyx_PyObject_FastCall(__pyx_t_3, __pyx_callargs+1-__pyx_t_5, 1+__pyx_t_5);
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 176, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    }
    if ((likely(PyTuple_CheckExact(__pyx_t_1))) || (PyList_CheckExact(__pyx_t_1))) {
      PyObject* sequence = __pyx_t_1;
      Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
      if (unlikely(size != 3)) {
        if (size > 3) __Pyx_RaiseTooManyValuesError(3);
        else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
        __PYX_ERR(0, 176, __pyx_L1_error)
      }
      #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
      if (likely(PyTuple_CheckExact(sequence))) {
        __pyx_t_3 = PyTuple_GET_ITEM(sequence, 0); 
        __pyx_t_2 = PyTuple_GET_ITEM(sequence, 1); 
        __pyx_t_4 = PyTuple_GET_ITEM(sequence, 2); 
      } else {
        __pyx_t_3 = PyList_GET_ITEM(sequence, 0); 
        __pyx_t_2 = PyList_GET_ITEM(sequence, 1); 
        __pyx_t_4 = PyList_GET_ITEM(sequence, 2); 
      }
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      #else
      __pyx_t_3 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 176, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_2 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 176, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_4 = PySequence_ITEM(sequence, 2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 176, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    } else {
      Py_ssize_t index = -1;
      __pyx_t_9 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 176, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_9);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_7 = __Pyx_PyObject_GetIterNextFunc(__pyx_t_9);
      index = 0; __pyx_t_3 = __pyx_t_7(__pyx_t_9); if (unlikely(!__pyx_t_3)) goto __pyx_L6_unpacking_failed;
      __Pyx_GOTREF(__pyx_t_3);
      index = 1; __pyx_t_2 = __pyx_t_7(__pyx_t_9); if (unlikely(!__pyx_t_2)) goto __pyx_L6_unpacking_failed;
      __Pyx_GOTREF(__pyx_t_2);
      index = 2; __pyx_t_4 = __pyx_t_7(__pyx_t_9); if (unlikely(!__pyx_t_4)) goto __pyx_L6_unpacking_failed;
      __Pyx_GOTREF(__pyx_t_4);
      if (__Pyx_IternextUnpackEndCheck(__pyx_t_7(__pyx_t_9), 3) < 0) __PYX_ERR(0, 176, __pyx_L1_error)
      __pyx_t_7 = NULL;
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      goto __pyx_L7_unpacking_done;
      __pyx_L6_unpacking_failed:;
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      __pyx_t_7 = NULL;
      if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
      __PYX_ERR(0, 176, __pyx_L1_error)
      __pyx_L7_unpacking_done:;
    }
    __pyx_v__prefix = __pyx_t_3;
    __pyx_t_3 = 0;
    __pyx_v__path = __pyx_t_2;
    __pyx_t_2 = 0;
    __Pyx_DECREF_SET(__pyx_v_keys, __pyx_t_4);
    __pyx_t_4 = 0;

    /* "aim/storage/prefixview.py":177
 *         if path:
 *             _prefix, _path, keys = keys.partition(path)
 *             if _prefix or _path != path:             # <<<<<<<<<<<<<<
 *                 raise KeyError
 *         return keys, value
 */
    __pyx_t_10 = __Pyx_PyObject_IsTrue(__pyx_v__prefix); if (unlikely((__pyx_t_10 < 0))) __PYX_ERR(0, 177, __pyx_L1_error)
    if (!__pyx_t_10) {
    } else {
      __pyx_t_8 = __pyx_t_10;
      goto __pyx_L9_bool_binop_done;
    }
    __pyx_t_10 = (__Pyx_PyBytes_Equals(__pyx_v__path, __pyx_v_path, Py_NE)); if (unlikely((__pyx_t_10 < 0))) __PYX_ERR(0, 177, __pyx_L1_error)
    __pyx_t_8 = __pyx_t_10;
    __pyx_L9_bool_binop_done:;
    if (unlikely(__pyx_t_8)) {

      /* "aim/storage/prefixview.py":178
 *             _prefix, _path, keys = keys.partition(path)
 *             if _prefix or _path != path:
 *                 raise KeyError             # <<<<<<<<<<<<<<
 *         return keys, value
 * 
 */
      __Pyx_Raise(__pyx_builtin_KeyError, 0, 0, 0);
      __PYX_ERR(0, 178, __pyx_L1_error)

      /* "aim/storage/prefixview.py":177
 *         if path:
 *             _prefix, _path, keys = keys.partition(path)
 *             if _prefix or _path != path:             # <<<<<<<<<<<<<<
 *                 raise KeyError
 *         return keys, value
 */
    }

    /* "aim/storage/prefixview.py":175
 *         path = self.absolute_path(key)
 *         keys, value = self.parent.next_item(path)
 *         if path:             # <<<<<<<<<<<<<<
 *             _prefix, _path, keys = keys.partition(path)
 *             if _prefix or _path != path:
 */
  }

  /* "aim/storage/prefixview.py":179
 *             if _prefix or _path != path:
 *                 raise KeyError
 *         return keys, value             # <<<<<<<<<<<<<<
 * 
 *     def prev_item(
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 179, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_v_keys);
  __Pyx_GIVEREF(__pyx_v_keys);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v_keys);
  __Pyx_INCREF(__pyx_v_value);
  __Pyx_GIVEREF(__pyx_v_value);
  PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_v_value);
  __pyx_r = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "aim/storage/prefixview.py":166
 *                                  store_batch=store_batch)
 * 
 *     def next_item(             # <<<<<<<<<<<<<<
 *         self,
 *         key: ContainerKey = b''
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("aim.storage.prefixview.PrefixView.next_item", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_path);
  __Pyx_XDECREF(__pyx_v_keys);
  __Pyx_XDECREF(__pyx_v_value);
  __Pyx_XDECREF(__pyx_v__prefix);
  __Pyx_XDECREF(__pyx_v__path);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_3aim_7storage_10prefixview_10PrefixView_25next_item(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_3aim_7storage_10prefixview_10PrefixView_24next_item, "Returns `(key, value)` for the key that comes (lexicographically)\n        right after the provided `key`.\n        ");
static PyMethodDef __pyx_mdef_3aim_7storage_10prefixview_10PrefixView_25next_item = {"next_item", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_3aim_7storage_10prefixview_10PrefixView_25next_item, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_3aim_7storage_10prefixview_10PrefixView_24next_item};
static PyObject *__pyx_pw_3aim_7storage_10prefixview_10PrefixView_25next_item(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_key = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED const Py_ssize_t __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("next_item (wrapper)", 0);
  {
    #if CYTHON_USE_MODULE_STATE
    PyObject **__pyx_pyargnames[] = {&__pyx_n_s_key,0};
    #else
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_key,0};
    #endif
    PyObject* values[1] = {0};
    values[0] = ((PyObject*)__pyx_kp_b_);
    if (__pyx_kwds) {
      Py_ssize_t kw_args;
      switch (__pyx_nargs) {
        case  1: values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = __Pyx_NumKwargs_FASTCALL(__pyx_kwds);
      switch (__pyx_nargs) {
        case  0:
        if (kw_args > 0) {
          PyObject* value = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_key);
          if (value) { values[0] = value; kw_args--; }
          else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 166, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        const Py_ssize_t kwd_pos_args = __pyx_nargs;
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values + 0, kwd_pos_args, "next_item") < 0)) __PYX_ERR(0, 166, __pyx_L3_error)
      }
    } else {
      switch (__pyx_nargs) {
        case  1: values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_key = ((PyObject*)values[0]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("next_item", 0, 0, 1, __pyx_nargs); __PYX_ERR(0, 166, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("aim.storage.prefixview.PrefixView.next_item", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_key), (&PyBytes_Type), 1, "key", 1))) __PYX_ERR(0, 168, __pyx_L1_error)
  __pyx_r = __pyx_pf_3aim_7storage_10prefixview_10PrefixView_24next_item(((struct __pyx_obj_3aim_7storage_10prefixview_PrefixView *)__pyx_v_self), __pyx_v_key);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_3aim_7storage_10prefixview_10PrefixView_24next_item(struct __pyx_obj_3aim_7storage_10prefixview_PrefixView *__pyx_v_self, PyObject *__pyx_v_key) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  struct __pyx_opt_args_3aim_7storage_9container_9Container_next_item __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("next_item", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2.__pyx_n = 1;
  __pyx_t_2.key = __pyx_v_key;
  __pyx_t_1 = __pyx_vtabptr_3aim_7storage_10prefixview_PrefixView->__pyx_base.next_item(((struct __pyx_obj_3aim_7storage_9container_Container *)__pyx_v_self), 1, &__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 166, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("aim.storage.prefixview.PrefixView.next_item", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "aim/storage/prefixview.py":181
 *         return keys, value
 * 
 *     def prev_item(             # <<<<<<<<<<<<<<
 *         self,
 *         key: ContainerKey = b''
 */

static PyObject *__pyx_pw_3aim_7storage_10prefixview_10PrefixView_27prev_item(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyObject *__pyx_f_3aim_7storage_10prefixview_10PrefixView_prev_item(struct __pyx_obj_3aim_7storage_10prefixview_PrefixView *__pyx_v_self, int __pyx_skip_dispatch, struct __pyx_opt_args_3aim_7storage_10prefixview_10PrefixView_prev_item *__pyx_optional_args) {
  PyObject *__pyx_v_key = ((PyObject*)__pyx_kp_b_);
  PyObject *__pyx_v_path = NULL;
  PyObject *__pyx_v_keys = NULL;
  PyObject *__pyx_v_value = NULL;
  PyObject *__pyx_v__prefix = NULL;
  PyObject *__pyx_v__path = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  struct __pyx_opt_args_3aim_7storage_10prefixview_10PrefixView_absolute_path __pyx_t_6;
  PyObject *(*__pyx_t_7)(PyObject *);
  int __pyx_t_8;
  PyObject *__pyx_t_9 = NULL;
  int __pyx_t_10;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("prev_item", 0);
  if (__pyx_optional_args) {
    if (__pyx_optional_args->__pyx_n > 0) {
      __pyx_v_key = __pyx_optional_args->key;
    }
  }
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || __Pyx_PyType_HasFeature(Py_TYPE(((PyObject *)__pyx_v_self)), (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_typedict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_prev_item); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 181, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      #ifdef __Pyx_CyFunction_USED
      if (!__Pyx_IsCyOrPyCFunction(__pyx_t_1)
      #else
      if (!PyCFunction_Check(__pyx_t_1)
      #endif
              || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_3aim_7storage_10prefixview_10PrefixView_27prev_item)) {
        __Pyx_XDECREF(__pyx_r);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        __pyx_t_5 = 0;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
            __pyx_t_5 = 1;
          }
        }
        {
          PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_v_key};
          __pyx_t_2 = __Pyx_PyObject_FastCall(__pyx_t_3, __pyx_callargs+1-__pyx_t_5, 1+__pyx_t_5);
          __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
          if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 181, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        }
        if (!(likely(PyTuple_CheckExact(__pyx_t_2))||((__pyx_t_2) == Py_None) || __Pyx_RaiseUnexpectedTypeError("tuple", __pyx_t_2))) __PYX_ERR(0, 181, __pyx_L1_error)
        __pyx_r = ((PyObject*)__pyx_t_2);
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_typedict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "aim/storage/prefixview.py":188
 *         right before the provided `key`.
 *         """
 *         path = self.absolute_path(key)             # <<<<<<<<<<<<<<
 *         keys, value = self.parent.prev_item(path)
 *         if path:
 */
  __pyx_t_6.__pyx_n = 1;
  __pyx_t_6.path = __pyx_v_key;
  __pyx_t_1 = ((struct __pyx_vtabstruct_3aim_7storage_10prefixview_PrefixView *)__pyx_v_self->__pyx_base.__pyx_vtab)->absolute_path(__pyx_v_self, 0, &__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 188, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_path = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "aim/storage/prefixview.py":189
 *         """
 *         path = self.absolute_path(key)
 *         keys, value = self.parent.prev_item(path)             # <<<<<<<<<<<<<<
 *         if path:
 *             _prefix, _path, keys = keys.partition(path)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->parent, __pyx_n_s_prev_item); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 189, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_5 = 1;
    }
  }
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_v_path};
    __pyx_t_1 = __Pyx_PyObject_FastCall(__pyx_t_2, __pyx_callargs+1-__pyx_t_5, 1+__pyx_t_5);
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 189, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  }
  if ((likely(PyTuple_CheckExact(__pyx_t_1))) || (PyList_CheckExact(__pyx_t_1))) {
    PyObject* sequence = __pyx_t_1;
    Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
    if (unlikely(size != 2)) {
      if (size > 2) __Pyx_RaiseTooManyValuesError(2);
      else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
      __PYX_ERR(0, 189, __pyx_L1_error)
    }
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    if (likely(PyTuple_CheckExact(sequence))) {
      __pyx_t_2 = PyTuple_GET_ITEM(sequence, 0); 
      __pyx_t_3 = PyTuple_GET_ITEM(sequence, 1); 
    } else {
      __pyx_t_2 = PyList_GET_ITEM(sequence, 0); 
      __pyx_t_3 = PyList_GET_ITEM(sequence, 1); 
    }
    __Pyx_INCREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_3);
    #else
    __pyx_t_2 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 189, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 189, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    #endif
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  } else {
    Py_ssize_t index = -1;
    __pyx_t_4 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 189, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_7 = __Pyx_PyObject_GetIterNextFunc(__pyx_t_4);
    index = 0; __pyx_t_2 = __pyx_t_7(__pyx_t_4); if (unlikely(!__pyx_t_2)) goto __pyx_L3_unpacking_failed;
    __Pyx_GOTREF(__pyx_t_2);
    index = 1; __pyx_t_3 = __pyx_t_7(__pyx_t_4); if (unlikely(!__pyx_t_3)) goto __pyx_L3_unpacking_failed;
    __Pyx_GOTREF(__pyx_t_3);
    if (__Pyx_IternextUnpackEndCheck(__pyx_t_7(__pyx_t_4), 2) < 0) __PYX_ERR(0, 189, __pyx_L1_error)
    __pyx_t_7 = NULL;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    goto __pyx_L4_unpacking_done;
    __pyx_L3_unpacking_failed:;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_7 = NULL;
    if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
    __PYX_ERR(0, 189, __pyx_L1_error)
    __pyx_L4_unpacking_done:;
  }
  __pyx_v_keys = __pyx_t_2;
  __pyx_t_2 = 0;
  __pyx_v_value = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "aim/storage/prefixview.py":190
 *         path = self.absolute_path(key)
 *         keys, value = self.parent.prev_item(path)
 *         if path:             # <<<<<<<<<<<<<<
 *             _prefix, _path, keys = keys.partition(path)
 *             if _prefix or _path != path:
 */
  __pyx_t_8 = (__pyx_v_path != Py_None)&&(PyBytes_GET_SIZE(__pyx_v_path) != 0);
  if (__pyx_t_8) {

    /* "aim/storage/prefixview.py":191
 *         keys, value = self.parent.prev_item(path)
 *         if path:
 *             _prefix, _path, keys = keys.partition(path)             # <<<<<<<<<<<<<<
 *             if _prefix or _path != path:
 *                 raise KeyError
 */
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_keys, __pyx_n_s_partition); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 191, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_2 = NULL;
    __pyx_t_5 = 0;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_2)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_2);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
        __pyx_t_5 = 1;
      }
    }
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_v_path};
      __pyx_t_1 = __Pyx_PyObject_FastCall(__pyx_t_3, __pyx_callargs+1-__pyx_t_5, 1+__pyx_t_5);
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 191, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    }
    if ((likely(PyTuple_CheckExact(__pyx_t_1))) || (PyList_CheckExact(__pyx_t_1))) {
      PyObject* sequence = __pyx_t_1;
      Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
      if (unlikely(size != 3)) {
        if (size > 3) __Pyx_RaiseTooManyValuesError(3);
        else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
        __PYX_ERR(0, 191, __pyx_L1_error)
      }
      #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
      if (likely(PyTuple_CheckExact(sequence))) {
        __pyx_t_3 = PyTuple_GET_ITEM(sequence, 0); 
        __pyx_t_2 = PyTuple_GET_ITEM(sequence, 1); 
        __pyx_t_4 = PyTuple_GET_ITEM(sequence, 2); 
      } else {
        __pyx_t_3 = PyList_GET_ITEM(sequence, 0); 
        __pyx_t_2 = PyList_GET_ITEM(sequence, 1); 
        __pyx_t_4 = PyList_GET_ITEM(sequence, 2); 
      }
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      #else
      __pyx_t_3 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 191, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_2 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 191, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_4 = PySequence_ITEM(sequence, 2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 191, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    } else {
      Py_ssize_t index = -1;
      __pyx_t_9 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 191, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_9);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_7 = __Pyx_PyObject_GetIterNextFunc(__pyx_t_9);
      index = 0; __pyx_t_3 = __pyx_t_7(__pyx_t_9); if (unlikely(!__pyx_t_3)) goto __pyx_L6_unpacking_failed;
      __Pyx_GOTREF(__pyx_t_3);
      index = 1; __pyx_t_2 = __pyx_t_7(__pyx_t_9); if (unlikely(!__pyx_t_2)) goto __pyx_L6_unpacking_failed;
      __Pyx_GOTREF(__pyx_t_2);
      index = 2; __pyx_t_4 = __pyx_t_7(__pyx_t_9); if (unlikely(!__pyx_t_4)) goto __pyx_L6_unpacking_failed;
      __Pyx_GOTREF(__pyx_t_4);
      if (__Pyx_IternextUnpackEndCheck(__pyx_t_7(__pyx_t_9), 3) < 0) __PYX_ERR(0, 191, __pyx_L1_error)
      __pyx_t_7 = NULL;
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      goto __pyx_L7_unpacking_done;
      __pyx_L6_unpacking_failed:;
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      __pyx_t_7 = NULL;
      if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
      __PYX_ERR(0, 191, __pyx_L1_error)
      __pyx_L7_unpacking_done:;
    }
    __pyx_v__prefix = __pyx_t_3;
    __pyx_t_3 = 0;
    __pyx_v__path = __pyx_t_2;
    __pyx_t_2 = 0;
    __Pyx_DECREF_SET(__pyx_v_keys, __pyx_t_4);
    __pyx_t_4 = 0;

    /* "aim/storage/prefixview.py":192
 *         if path:
 *             _prefix, _path, keys = keys.partition(path)
 *             if _prefix or _path != path:             # <<<<<<<<<<<<<<
 *                 raise KeyError
 *         return keys, value
 */
    __pyx_t_10 = __Pyx_PyObject_IsTrue(__pyx_v__prefix); if (unlikely((__pyx_t_10 < 0))) __PYX_ERR(0, 192, __pyx_L1_error)
    if (!__pyx_t_10) {
    } else {
      __pyx_t_8 = __pyx_t_10;
      goto __pyx_L9_bool_binop_done;
    }
    __pyx_t_10 = (__Pyx_PyBytes_Equals(__pyx_v__path, __pyx_v_path, Py_NE)); if (unlikely((__pyx_t_10 < 0))) __PYX_ERR(0, 192, __pyx_L1_error)
    __pyx_t_8 = __pyx_t_10;
    __pyx_L9_bool_binop_done:;
    if (unlikely(__pyx_t_8)) {

      /* "aim/storage/prefixview.py":193
 *             _prefix, _path, keys = keys.partition(path)
 *             if _prefix or _path != path:
 *                 raise KeyError             # <<<<<<<<<<<<<<
 *         return keys, value
 * 
 */
      __Pyx_Raise(__pyx_builtin_KeyError, 0, 0, 0);
      __PYX_ERR(0, 193, __pyx_L1_error)

      /* "aim/storage/prefixview.py":192
 *         if path:
 *             _prefix, _path, keys = keys.partition(path)
 *             if _prefix or _path != path:             # <<<<<<<<<<<<<<
 *                 raise KeyError
 *         return keys, value
 */
    }

    /* "aim/storage/prefixview.py":190
 *         path = self.absolute_path(key)
 *         keys, value = self.parent.prev_item(path)
 *         if path:             # <<<<<<<<<<<<<<
 *             _prefix, _path, keys = keys.partition(path)
 *             if _prefix or _path != path:
 */
  }

  /* "aim/storage/prefixview.py":194
 *             if _prefix or _path != path:
 *                 raise KeyError
 *         return keys, value             # <<<<<<<<<<<<<<
 * 
 *     def walk(
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 194, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_v_keys);
  __Pyx_GIVEREF(__pyx_v_keys);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v_keys);
  __Pyx_INCREF(__pyx_v_value);
  __Pyx_GIVEREF(__pyx_v_value);
  PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_v_value);
  __pyx_r = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "aim/storage/prefixview.py":181
 *         return keys, value
 * 
 *     def prev_item(             # <<<<<<<<<<<<<<
 *         self,
 *         key: ContainerKey = b''
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("aim.storage.prefixview.PrefixView.prev_item", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_path);
  __Pyx_XDECREF(__pyx_v_keys);
  __Pyx_XDECREF(__pyx_v_value);
  __Pyx_XDECREF(__pyx_v__prefix);
  __Pyx_XDECREF(__pyx_v__path);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_3aim_7storage_10prefixview_10PrefixView_27prev_item(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_3aim_7storage_10prefixview_10PrefixView_26prev_item, "Returns `(key, value)` for the key that comes (lexicographically)\n        right before the provided `key`.\n        ");
static PyMethodDef __pyx_mdef_3aim_7storage_10prefixview_10PrefixView_27prev_item = {"prev_item", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_3aim_7storage_10prefixview_10PrefixView_27prev_item, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_3aim_7storage_10prefixview_10PrefixView_26prev_item};
static PyObject *__pyx_pw_3aim_7storage_10prefixview_10PrefixView_27prev_item(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_key = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED const Py_ssize_t __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("prev_item (wrapper)", 0);
  {
    #if CYTHON_USE_MODULE_STATE
    PyObject **__pyx_pyargnames[] = {&__pyx_n_s_key,0};
    #else
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_key,0};
    #endif
    PyObject* values[1] = {0};
    values[0] = ((PyObject*)__pyx_kp_b_);
    if (__pyx_kwds) {
      Py_ssize_t kw_args;
      switch (__pyx_nargs) {
        case  1: values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = __Pyx_NumKwargs_FASTCALL(__pyx_kwds);
      switch (__pyx_nargs) {
        case  0:
        if (kw_args > 0) {
          PyObject* value = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_key);
          if (value) { values[0] = value; kw_args--; }
          else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 181, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        const Py_ssize_t kwd_pos_args = __pyx_nargs;
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values + 0, kwd_pos_args, "prev_item") < 0)) __PYX_ERR(0, 181, __pyx_L3_error)
      }
    } else {
      switch (__pyx_nargs) {
        case  1: values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_key = ((PyObject*)values[0]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("prev_item", 0, 0, 1, __pyx_nargs); __PYX_ERR(0, 181, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("aim.storage.prefixview.PrefixView.prev_item", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_key), (&PyBytes_Type), 1, "key", 1))) __PYX_ERR(0, 183, __pyx_L1_error)
  __pyx_r = __pyx_pf_3aim_7storage_10prefixview_10PrefixView_26prev_item(((struct __pyx_obj_3aim_7storage_10prefixview_PrefixView *)__pyx_v_self), __pyx_v_key);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_3aim_7storage_10prefixview_10PrefixView_26prev_item(struct __pyx_obj_3aim_7storage_10prefixview_PrefixView *__pyx_v_self, PyObject *__pyx_v_key) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  struct __pyx_opt_args_3aim_7storage_9container_9Container_prev_item __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("prev_item", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2.__pyx_n = 1;
  __pyx_t_2.key = __pyx_v_key;
  __pyx_t_1 = __pyx_vtabptr_3aim_7storage_10prefixview_PrefixView->__pyx_base.prev_item(((struct __pyx_obj_3aim_7storage_9container_Container *)__pyx_v_self), 1, &__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 181, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("aim.storage.prefixview.PrefixView.prev_item", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}
static PyObject *__pyx_gb_3aim_7storage_10prefixview_10PrefixView_30generator(__pyx_CoroutineObject *__pyx_generator, CYTHON_UNUSED PyThreadState *__pyx_tstate, PyObject *__pyx_sent_value); /* proto */

/* "aim/storage/prefixview.py":196
 *         return keys, value
 * 
 *     def walk(             # <<<<<<<<<<<<<<
 *         self,
 *         key: ContainerKey = b''
 */

/* Python wrapper */
static PyObject *__pyx_pw_3aim_7storage_10prefixview_10PrefixView_29walk(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_3aim_7storage_10prefixview_10PrefixView_28walk, "A bi-directional generator to walk over the collection of records on\n        any arbitrary order. The `prefix` sent to the generator (lets call it\n        a `walker`) seeks for lower-bound key in the collection.\n\n        In other words, if the Container contents are:\n        `{\n            b'e.y': b'012',\n            b'meta.x': b'123',\n            b'meta.z': b'x',\n            b'zzz': b'oOo'\n        }` and `walker = container.walk()` then:\n        `walker.send(b'meta') == b'meta.x'`, `walker.send(b'e.y') == b'e.y'`\n        ");
static PyMethodDef __pyx_mdef_3aim_7storage_10prefixview_10PrefixView_29walk = {"walk", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_3aim_7storage_10prefixview_10PrefixView_29walk, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_3aim_7storage_10prefixview_10PrefixView_28walk};
static PyObject *__pyx_pw_3aim_7storage_10prefixview_10PrefixView_29walk(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_key = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED const Py_ssize_t __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("walk (wrapper)", 0);
  {
    #if CYTHON_USE_MODULE_STATE
    PyObject **__pyx_pyargnames[] = {&__pyx_n_s_key,0};
    #else
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_key,0};
    #endif
    PyObject* values[1] = {0};
    values[0] = ((PyObject *)__pyx_kp_b_);
    if (__pyx_kwds) {
      Py_ssize_t kw_args;
      switch (__pyx_nargs) {
        case  1: values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = __Pyx_NumKwargs_FASTCALL(__pyx_kwds);
      switch (__pyx_nargs) {
        case  0:
        if (kw_args > 0) {
          PyObject* value = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_key);
          if (value) { values[0] = value; kw_args--; }
          else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 196, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        const Py_ssize_t kwd_pos_args = __pyx_nargs;
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values + 0, kwd_pos_args, "walk") < 0)) __PYX_ERR(0, 196, __pyx_L3_error)
      }
    } else {
      switch (__pyx_nargs) {
        case  1: values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_key = values[0];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("walk", 0, 0, 1, __pyx_nargs); __PYX_ERR(0, 196, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("aim.storage.prefixview.PrefixView.walk", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_3aim_7storage_10prefixview_10PrefixView_28walk(((struct __pyx_obj_3aim_7storage_10prefixview_PrefixView *)__pyx_v_self), __pyx_v_key);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_3aim_7storage_10prefixview_10PrefixView_28walk(struct __pyx_obj_3aim_7storage_10prefixview_PrefixView *__pyx_v_self, PyObject *__pyx_v_key) {
  struct __pyx_obj_3aim_7storage_10prefixview___pyx_scope_struct__walk *__pyx_cur_scope;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("walk", 0);
  __pyx_cur_scope = (struct __pyx_obj_3aim_7storage_10prefixview___pyx_scope_struct__walk *)__pyx_tp_new_3aim_7storage_10prefixview___pyx_scope_struct__walk(__pyx_ptype_3aim_7storage_10prefixview___pyx_scope_struct__walk, __pyx_empty_tuple, NULL);
  if (unlikely(!__pyx_cur_scope)) {
    __pyx_cur_scope = ((struct __pyx_obj_3aim_7storage_10prefixview___pyx_scope_struct__walk *)Py_None);
    __Pyx_INCREF(Py_None);
    __PYX_ERR(0, 196, __pyx_L1_error)
  } else {
    __Pyx_GOTREF((PyObject *)__pyx_cur_scope);
  }
  __pyx_cur_scope->__pyx_v_self = __pyx_v_self;
  __Pyx_INCREF((PyObject *)__pyx_cur_scope->__pyx_v_self);
  __Pyx_GIVEREF((PyObject *)__pyx_cur_scope->__pyx_v_self);
  __pyx_cur_scope->__pyx_v_key = __pyx_v_key;
  __Pyx_INCREF(__pyx_cur_scope->__pyx_v_key);
  __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_key);
  {
    __pyx_CoroutineObject *gen = __Pyx_Generator_New((__pyx_coroutine_body_t) __pyx_gb_3aim_7storage_10prefixview_10PrefixView_30generator, __pyx_codeobj__3, (PyObject *) __pyx_cur_scope, __pyx_n_s_walk, __pyx_n_s_PrefixView_walk, __pyx_n_s_aim_storage_prefixview); if (unlikely(!gen)) __PYX_ERR(0, 196, __pyx_L1_error)
    __Pyx_DECREF(__pyx_cur_scope);
    __Pyx_RefNannyFinishContext();
    return (PyObject *) gen;
  }

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("aim.storage.prefixview.PrefixView.walk", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_DECREF((PyObject *)__pyx_cur_scope);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_gb_3aim_7storage_10prefixview_10PrefixView_30generator(__pyx_CoroutineObject *__pyx_generator, CYTHON_UNUSED PyThreadState *__pyx_tstate, PyObject *__pyx_sent_value) /* generator body */
{
  struct __pyx_obj_3aim_7storage_10prefixview___pyx_scope_struct__walk *__pyx_cur_scope = ((struct __pyx_obj_3aim_7storage_10prefixview___pyx_scope_struct__walk *)__pyx_generator->closure);
  PyObject *__pyx_r = NULL;
  PyObject *__pyx_t_1 = NULL;
  struct __pyx_opt_args_3aim_7storage_10prefixview_10PrefixView_absolute_path __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_t_6;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *(*__pyx_t_10)(PyObject *);
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("walk", 0);
  switch (__pyx_generator->resume_label) {
    case 0: goto __pyx_L3_first_run;
    case 1: goto __pyx_L14_resume_from_yield;
    default: /* CPython raises the right error here */
    __Pyx_RefNannyFinishContext();
    return NULL;
  }
  __pyx_L3_first_run:;
  if (unlikely(!__pyx_sent_value)) __PYX_ERR(0, 196, __pyx_L1_error)

  /* "aim/storage/prefixview.py":213
 *         `walker.send(b'meta') == b'meta.x'`, `walker.send(b'e.y') == b'e.y'`
 *         """
 *         path = self.absolute_path(key)             # <<<<<<<<<<<<<<
 *         walker = self.parent.walk(path)
 *         p = None
 */
  if (!(likely(PyBytes_CheckExact(__pyx_cur_scope->__pyx_v_key))||((__pyx_cur_scope->__pyx_v_key) == Py_None) || __Pyx_RaiseUnexpectedTypeError("bytes", __pyx_cur_scope->__pyx_v_key))) __PYX_ERR(0, 213, __pyx_L1_error)
  __pyx_t_2.__pyx_n = 1;
  __pyx_t_2.path = ((PyObject*)__pyx_cur_scope->__pyx_v_key);
  __pyx_t_1 = ((struct __pyx_vtabstruct_3aim_7storage_10prefixview_PrefixView *)__pyx_cur_scope->__pyx_v_self->__pyx_base.__pyx_vtab)->absolute_path(__pyx_cur_scope->__pyx_v_self, 0, &__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 213, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __pyx_cur_scope->__pyx_v_path = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "aim/storage/prefixview.py":214
 *         """
 *         path = self.absolute_path(key)
 *         walker = self.parent.walk(path)             # <<<<<<<<<<<<<<
 *         p = None
 *         while True:
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_cur_scope->__pyx_v_self->parent, __pyx_n_s_walk); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 214, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
      __pyx_t_5 = 1;
    }
  }
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_cur_scope->__pyx_v_path};
    __pyx_t_1 = __Pyx_PyObject_FastCall(__pyx_t_3, __pyx_callargs+1-__pyx_t_5, 1+__pyx_t_5);
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 214, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  }
  __Pyx_GIVEREF(__pyx_t_1);
  __pyx_cur_scope->__pyx_v_walker = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "aim/storage/prefixview.py":215
 *         path = self.absolute_path(key)
 *         walker = self.parent.walk(path)
 *         p = None             # <<<<<<<<<<<<<<
 *         while True:
 *             if p is None:
 */
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __pyx_cur_scope->__pyx_v_p = ((PyObject*)Py_None);

  /* "aim/storage/prefixview.py":216
 *         walker = self.parent.walk(path)
 *         p = None
 *         while True:             # <<<<<<<<<<<<<<
 *             if p is None:
 *                 next_key = next(walker)
 */
  while (1) {

    /* "aim/storage/prefixview.py":217
 *         p = None
 *         while True:
 *             if p is None:             # <<<<<<<<<<<<<<
 *                 next_key = next(walker)
 *             else:
 */
    __pyx_t_6 = (__pyx_cur_scope->__pyx_v_p == ((PyObject*)Py_None));
    __pyx_t_7 = (__pyx_t_6 != 0);
    if (__pyx_t_7) {

      /* "aim/storage/prefixview.py":218
 *         while True:
 *             if p is None:
 *                 next_key = next(walker)             # <<<<<<<<<<<<<<
 *             else:
 *                 next_key = walker.send(p)
 */
      __pyx_t_1 = __Pyx_PyIter_Next(__pyx_cur_scope->__pyx_v_walker); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 218, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_XGOTREF(__pyx_cur_scope->__pyx_v_next_key);
      __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_next_key, __pyx_t_1);
      __Pyx_GIVEREF(__pyx_t_1);
      __pyx_t_1 = 0;

      /* "aim/storage/prefixview.py":217
 *         p = None
 *         while True:
 *             if p is None:             # <<<<<<<<<<<<<<
 *                 next_key = next(walker)
 *             else:
 */
      goto __pyx_L6;
    }

    /* "aim/storage/prefixview.py":220
 *                 next_key = next(walker)
 *             else:
 *                 next_key = walker.send(p)             # <<<<<<<<<<<<<<
 *             if next_key is None:
 *                 return
 */
    /*else*/ {
      __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_cur_scope->__pyx_v_walker, __pyx_n_s_send); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 220, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = NULL;
      __pyx_t_5 = 0;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
        __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
        if (likely(__pyx_t_4)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
          __Pyx_INCREF(__pyx_t_4);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_3, function);
          __pyx_t_5 = 1;
        }
      }
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_cur_scope->__pyx_v_p};
        __pyx_t_1 = __Pyx_PyObject_FastCall(__pyx_t_3, __pyx_callargs+1-__pyx_t_5, 1+__pyx_t_5);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 220, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      }
      __Pyx_XGOTREF(__pyx_cur_scope->__pyx_v_next_key);
      __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_next_key, __pyx_t_1);
      __Pyx_GIVEREF(__pyx_t_1);
      __pyx_t_1 = 0;
    }
    __pyx_L6:;

    /* "aim/storage/prefixview.py":221
 *             else:
 *                 next_key = walker.send(p)
 *             if next_key is None:             # <<<<<<<<<<<<<<
 *                 return
 *             if path:
 */
    __pyx_t_7 = (__pyx_cur_scope->__pyx_v_next_key == Py_None);
    __pyx_t_6 = (__pyx_t_7 != 0);
    if (__pyx_t_6) {

      /* "aim/storage/prefixview.py":222
 *                 next_key = walker.send(p)
 *             if next_key is None:
 *                 return             # <<<<<<<<<<<<<<
 *             if path:
 *                 _prefix, _path, next_key = next_key.partition(path)
 */
      __Pyx_XDECREF(__pyx_r);
      __pyx_r = NULL;
      goto __pyx_L0;

      /* "aim/storage/prefixview.py":221
 *             else:
 *                 next_key = walker.send(p)
 *             if next_key is None:             # <<<<<<<<<<<<<<
 *                 return
 *             if path:
 */
    }

    /* "aim/storage/prefixview.py":223
 *             if next_key is None:
 *                 return
 *             if path:             # <<<<<<<<<<<<<<
 *                 _prefix, _path, next_key = next_key.partition(path)
 *                 # assert not _prefix and _path == path
 */
    __pyx_t_6 = (__pyx_cur_scope->__pyx_v_path != Py_None)&&(PyBytes_GET_SIZE(__pyx_cur_scope->__pyx_v_path) != 0);
    if (__pyx_t_6) {

      /* "aim/storage/prefixview.py":224
 *                 return
 *             if path:
 *                 _prefix, _path, next_key = next_key.partition(path)             # <<<<<<<<<<<<<<
 *                 # assert not _prefix and _path == path
 *                 if _prefix or _path != path:
 */
      __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_cur_scope->__pyx_v_next_key, __pyx_n_s_partition); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 224, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = NULL;
      __pyx_t_5 = 0;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
        __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
        if (likely(__pyx_t_4)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
          __Pyx_INCREF(__pyx_t_4);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_3, function);
          __pyx_t_5 = 1;
        }
      }
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_cur_scope->__pyx_v_path};
        __pyx_t_1 = __Pyx_PyObject_FastCall(__pyx_t_3, __pyx_callargs+1-__pyx_t_5, 1+__pyx_t_5);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 224, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      }
      if ((likely(PyTuple_CheckExact(__pyx_t_1))) || (PyList_CheckExact(__pyx_t_1))) {
        PyObject* sequence = __pyx_t_1;
        Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
        if (unlikely(size != 3)) {
          if (size > 3) __Pyx_RaiseTooManyValuesError(3);
          else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
          __PYX_ERR(0, 224, __pyx_L1_error)
        }
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        if (likely(PyTuple_CheckExact(sequence))) {
          __pyx_t_3 = PyTuple_GET_ITEM(sequence, 0); 
          __pyx_t_4 = PyTuple_GET_ITEM(sequence, 1); 
          __pyx_t_8 = PyTuple_GET_ITEM(sequence, 2); 
        } else {
          __pyx_t_3 = PyList_GET_ITEM(sequence, 0); 
          __pyx_t_4 = PyList_GET_ITEM(sequence, 1); 
          __pyx_t_8 = PyList_GET_ITEM(sequence, 2); 
        }
        __Pyx_INCREF(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_8);
        #else
        __pyx_t_3 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 224, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __pyx_t_4 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 224, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        __pyx_t_8 = PySequence_ITEM(sequence, 2); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 224, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_8);
        #endif
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      } else {
        Py_ssize_t index = -1;
        __pyx_t_9 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 224, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_9);
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        __pyx_t_10 = __Pyx_PyObject_GetIterNextFunc(__pyx_t_9);
        index = 0; __pyx_t_3 = __pyx_t_10(__pyx_t_9); if (unlikely(!__pyx_t_3)) goto __pyx_L9_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_3);
        index = 1; __pyx_t_4 = __pyx_t_10(__pyx_t_9); if (unlikely(!__pyx_t_4)) goto __pyx_L9_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_4);
        index = 2; __pyx_t_8 = __pyx_t_10(__pyx_t_9); if (unlikely(!__pyx_t_8)) goto __pyx_L9_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_8);
        if (__Pyx_IternextUnpackEndCheck(__pyx_t_10(__pyx_t_9), 3) < 0) __PYX_ERR(0, 224, __pyx_L1_error)
        __pyx_t_10 = NULL;
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
        goto __pyx_L10_unpacking_done;
        __pyx_L9_unpacking_failed:;
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
        __pyx_t_10 = NULL;
        if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
        __PYX_ERR(0, 224, __pyx_L1_error)
        __pyx_L10_unpacking_done:;
      }
      __Pyx_XGOTREF(__pyx_cur_scope->__pyx_v__prefix);
      __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v__prefix, __pyx_t_3);
      __Pyx_GIVEREF(__pyx_t_3);
      __pyx_t_3 = 0;
      __Pyx_XGOTREF(__pyx_cur_scope->__pyx_v__path);
      __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v__path, __pyx_t_4);
      __Pyx_GIVEREF(__pyx_t_4);
      __pyx_t_4 = 0;
      __Pyx_GOTREF(__pyx_cur_scope->__pyx_v_next_key);
      __Pyx_DECREF_SET(__pyx_cur_scope->__pyx_v_next_key, __pyx_t_8);
      __Pyx_GIVEREF(__pyx_t_8);
      __pyx_t_8 = 0;

      /* "aim/storage/prefixview.py":226
 *                 _prefix, _path, next_key = next_key.partition(path)
 *                 # assert not _prefix and _path == path
 *                 if _prefix or _path != path:             # <<<<<<<<<<<<<<
 *                     return
 * 
 */
      __pyx_t_7 = __Pyx_PyObject_IsTrue(__pyx_cur_scope->__pyx_v__prefix); if (unlikely((__pyx_t_7 < 0))) __PYX_ERR(0, 226, __pyx_L1_error)
      if (!__pyx_t_7) {
      } else {
        __pyx_t_6 = __pyx_t_7;
        goto __pyx_L12_bool_binop_done;
      }
      __pyx_t_7 = (__Pyx_PyBytes_Equals(__pyx_cur_scope->__pyx_v__path, __pyx_cur_scope->__pyx_v_path, Py_NE)); if (unlikely((__pyx_t_7 < 0))) __PYX_ERR(0, 226, __pyx_L1_error)
      __pyx_t_6 = __pyx_t_7;
      __pyx_L12_bool_binop_done:;
      if (__pyx_t_6) {

        /* "aim/storage/prefixview.py":227
 *                 # assert not _prefix and _path == path
 *                 if _prefix or _path != path:
 *                     return             # <<<<<<<<<<<<<<
 * 
 *             key = yield next_key
 */
        __Pyx_XDECREF(__pyx_r);
        __pyx_r = NULL;
        goto __pyx_L0;

        /* "aim/storage/prefixview.py":226
 *                 _prefix, _path, next_key = next_key.partition(path)
 *                 # assert not _prefix and _path == path
 *                 if _prefix or _path != path:             # <<<<<<<<<<<<<<
 *                     return
 * 
 */
      }

      /* "aim/storage/prefixview.py":223
 *             if next_key is None:
 *                 return
 *             if path:             # <<<<<<<<<<<<<<
 *                 _prefix, _path, next_key = next_key.partition(path)
 *                 # assert not _prefix and _path == path
 */
    }

    /* "aim/storage/prefixview.py":229
 *                     return
 * 
 *             key = yield next_key             # <<<<<<<<<<<<<<
 *             p = self.absolute_path(key)
 * 
 */
    __Pyx_INCREF(__pyx_cur_scope->__pyx_v_next_key);
    __pyx_r = __pyx_cur_scope->__pyx_v_next_key;
    __Pyx_XGIVEREF(__pyx_r);
    __Pyx_RefNannyFinishContext();
    __Pyx_Coroutine_ResetAndClearException(__pyx_generator);
    /* return from generator, yielding value */
    __pyx_generator->resume_label = 1;
    return __pyx_r;
    __pyx_L14_resume_from_yield:;
    if (unlikely(!__pyx_sent_value)) __PYX_ERR(0, 229, __pyx_L1_error)
    __pyx_t_1 = __pyx_sent_value; __Pyx_INCREF(__pyx_t_1);
    __Pyx_GOTREF(__pyx_cur_scope->__pyx_v_key);
    __Pyx_DECREF_SET(__pyx_cur_scope->__pyx_v_key, __pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_1);
    __pyx_t_1 = 0;

    /* "aim/storage/prefixview.py":230
 * 
 *             key = yield next_key
 *             p = self.absolute_path(key)             # <<<<<<<<<<<<<<
 * 
 *     def items(
 */
    if (!(likely(PyBytes_CheckExact(__pyx_cur_scope->__pyx_v_key))||((__pyx_cur_scope->__pyx_v_key) == Py_None) || __Pyx_RaiseUnexpectedTypeError("bytes", __pyx_cur_scope->__pyx_v_key))) __PYX_ERR(0, 230, __pyx_L1_error)
    __pyx_t_2.__pyx_n = 1;
    __pyx_t_2.path = ((PyObject*)__pyx_cur_scope->__pyx_v_key);
    __pyx_t_1 = ((struct __pyx_vtabstruct_3aim_7storage_10prefixview_PrefixView *)__pyx_cur_scope->__pyx_v_self->__pyx_base.__pyx_vtab)->absolute_path(__pyx_cur_scope->__pyx_v_self, 0, &__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 230, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_GOTREF(__pyx_cur_scope->__pyx_v_p);
    __Pyx_DECREF_SET(__pyx_cur_scope->__pyx_v_p, ((PyObject*)__pyx_t_1));
    __Pyx_GIVEREF(__pyx_t_1);
    __pyx_t_1 = 0;
  }
  CYTHON_MAYBE_UNUSED_VAR(__pyx_cur_scope);

  /* "aim/storage/prefixview.py":196
 *         return keys, value
 * 
 *     def walk(             # <<<<<<<<<<<<<<
 *         self,
 *         key: ContainerKey = b''
 */

  /* function exit code */
  PyErr_SetNone(PyExc_StopIteration);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_Generator_Replace_StopIteration(0);
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("walk", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_r); __pyx_r = 0;
  #if !CYTHON_USE_EXC_INFO_STACK
  __Pyx_Coroutine_ResetAndClearException(__pyx_generator);
  #endif
  __pyx_generator->resume_label = -1;
  __Pyx_Coroutine_clear((PyObject*)__pyx_generator);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "aim/storage/prefixview.py":232
 *             p = self.absolute_path(key)
 * 
 *     def items(             # <<<<<<<<<<<<<<
 *         self,
 *         key: ContainerKey = b''
 */

static PyObject *__pyx_pw_3aim_7storage_10prefixview_10PrefixView_32items(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static struct __pyx_obj_3aim_7storage_9container_ContainerItemsIterator *__pyx_f_3aim_7storage_10prefixview_10PrefixView_items(struct __pyx_obj_3aim_7storage_10prefixview_PrefixView *__pyx_v_self, int __pyx_skip_dispatch, struct __pyx_opt_args_3aim_7storage_10prefixview_10PrefixView_items *__pyx_optional_args) {
  PyObject *__pyx_v_key = ((PyObject*)__pyx_kp_b_);
  struct __pyx_obj_3aim_7storage_9container_ContainerItemsIterator *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("items", 0);
  if (__pyx_optional_args) {
    if (__pyx_optional_args->__pyx_n > 0) {
      __pyx_v_key = __pyx_optional_args->key;
    }
  }
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || __Pyx_PyType_HasFeature(Py_TYPE(((PyObject *)__pyx_v_self)), (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_typedict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_items); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 232, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      #ifdef __Pyx_CyFunction_USED
      if (!__Pyx_IsCyOrPyCFunction(__pyx_t_1)
      #else
      if (!PyCFunction_Check(__pyx_t_1)
      #endif
              || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_3aim_7storage_10prefixview_10PrefixView_32items)) {
        __Pyx_XDECREF((PyObject *)__pyx_r);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        __pyx_t_5 = 0;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
            __pyx_t_5 = 1;
          }
        }
        {
          PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_v_key};
          __pyx_t_2 = __Pyx_PyObject_FastCall(__pyx_t_3, __pyx_callargs+1-__pyx_t_5, 1+__pyx_t_5);
          __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
          if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 232, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        }
        if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_ptype_3aim_7storage_9container_ContainerItemsIterator))))) __PYX_ERR(0, 232, __pyx_L1_error)
        __pyx_r = ((struct __pyx_obj_3aim_7storage_9container_ContainerItemsIterator *)__pyx_t_2);
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_typedict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "aim/storage/prefixview.py":253
 *             prefix (:obj:`bytes`): the prefix that defines the key range
 *         """
 *         return PrefixViewItemsIterator(self, key)             # <<<<<<<<<<<<<<
 * 
 *     def view(
 */
  __Pyx_XDECREF((PyObject *)__pyx_r);
  __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 253, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF((PyObject *)__pyx_v_self);
  __Pyx_GIVEREF((PyObject *)__pyx_v_self);
  PyTuple_SET_ITEM(__pyx_t_1, 0, ((PyObject *)__pyx_v_self));
  __Pyx_INCREF(__pyx_v_key);
  __Pyx_GIVEREF(__pyx_v_key);
  PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_v_key);
  __pyx_t_2 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_3aim_7storage_10prefixview_PrefixViewItemsIterator), __pyx_t_1, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 253, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_r = ((struct __pyx_obj_3aim_7storage_9container_ContainerItemsIterator *)__pyx_t_2);
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "aim/storage/prefixview.py":232
 *             p = self.absolute_path(key)
 * 
 *     def items(             # <<<<<<<<<<<<<<
 *         self,
 *         key: ContainerKey = b''
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("aim.storage.prefixview.PrefixView.items", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_3aim_7storage_10prefixview_10PrefixView_32items(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_3aim_7storage_10prefixview_10PrefixView_31items, "Iterate over all the key-value records in the prefix key range.\n\n        The iteration is always performed in lexiographic order w.r.t keys.\n        If `prefix` is provided, iterate only over those records that have key\n        starting with the `prefix`.\n\n        For example, if `prefix == b'meta.'`, and the Container consists of:\n        `{\n            b'e.y': b'012',\n            b'meta.x': b'123',\n            b'meta.z': b'x',\n            b'zzz': b'oOo'\n        }`, the method will yield `(b'meta.x', b'123')` and `(b'meta.z', b'x')`\n\n        Args:\n            prefix (:obj:`bytes`): the prefix that defines the key range\n        ");
static PyMethodDef __pyx_mdef_3aim_7storage_10prefixview_10PrefixView_32items = {"items", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_3aim_7storage_10prefixview_10PrefixView_32items, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_3aim_7storage_10prefixview_10PrefixView_31items};
static PyObject *__pyx_pw_3aim_7storage_10prefixview_10PrefixView_32items(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_key = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED const Py_ssize_t __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("items (wrapper)", 0);
  {
    #if CYTHON_USE_MODULE_STATE
    PyObject **__pyx_pyargnames[] = {&__pyx_n_s_key,0};
    #else
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_key,0};
    #endif
    PyObject* values[1] = {0};
    values[0] = ((PyObject*)__pyx_kp_b_);
    if (__pyx_kwds) {
      Py_ssize_t kw_args;
      switch (__pyx_nargs) {
        case  1: values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = __Pyx_NumKwargs_FASTCALL(__pyx_kwds);
      switch (__pyx_nargs) {
        case  0:
        if (kw_args > 0) {
          PyObject* value = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_key);
          if (value) { values[0] = value; kw_args--; }
          else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 232, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        const Py_ssize_t kwd_pos_args = __pyx_nargs;
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values + 0, kwd_pos_args, "items") < 0)) __PYX_ERR(0, 232, __pyx_L3_error)
      }
    } else {
      switch (__pyx_nargs) {
        case  1: values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_key = ((PyObject*)values[0]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("items", 0, 0, 1, __pyx_nargs); __PYX_ERR(0, 232, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("aim.storage.prefixview.PrefixView.items", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_key), (&PyBytes_Type), 1, "key", 1))) __PYX_ERR(0, 234, __pyx_L1_error)
  __pyx_r = __pyx_pf_3aim_7storage_10prefixview_10PrefixView_31items(((struct __pyx_obj_3aim_7storage_10prefixview_PrefixView *)__pyx_v_self), __pyx_v_key);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_3aim_7storage_10prefixview_10PrefixView_31items(struct __pyx_obj_3aim_7storage_10prefixview_PrefixView *__pyx_v_self, PyObject *__pyx_v_key) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  struct __pyx_opt_args_3aim_7storage_9container_9Container_items __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("items", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2.__pyx_n = 1;
  __pyx_t_2.prefix = __pyx_v_key;
  __pyx_t_1 = ((PyObject *)__pyx_vtabptr_3aim_7storage_10prefixview_PrefixView->__pyx_base.items(((struct __pyx_obj_3aim_7storage_9container_Container *)__pyx_v_self), 1, &__pyx_t_2)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 232, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("aim.storage.prefixview.PrefixView.items", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "aim/storage/prefixview.py":255
 *         return PrefixViewItemsIterator(self, key)
 * 
 *     def view(             # <<<<<<<<<<<<<<
 *         self,
 *         prefix: bytes = b''
 */

/* Python wrapper */
static struct __pyx_obj_3aim_7storage_9container_Container *__pyx_pw_3aim_7storage_10prefixview_10PrefixView_34view(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_3aim_7storage_10prefixview_10PrefixView_33view, "Return a view (even mutable ones) that enable access to the container\n        but with modifications.\n\n        Args:\n            prefix (:obj:`bytes`): the prefix that defines the key range of the\n                view-container. The resulting container will share an access to\n                only records in the `prefix` key range, but with `prefix`-es\n                stripped from them.\n\n                For example, if the Container contents are:\n                `{\n                    b'e.y': b'012',\n                    b'meta.x': b'123',\n                    b'meta.z': b'x',\n                    b'zzz': b'oOo'\n                }`, then `container.view(prefix=b'meta.')` will behave (almost)\n                exactly as an Container:\n                `{\n                    b'x': b'123',\n                    b'z': b'x',\n                }`\n        ");
static PyMethodDef __pyx_mdef_3aim_7storage_10prefixview_10PrefixView_34view = {"view", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_3aim_7storage_10prefixview_10PrefixView_34view, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_3aim_7storage_10prefixview_10PrefixView_33view};
static struct __pyx_obj_3aim_7storage_9container_Container *__pyx_pw_3aim_7storage_10prefixview_10PrefixView_34view(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_prefix = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED const Py_ssize_t __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  struct __pyx_obj_3aim_7storage_9container_Container *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("view (wrapper)", 0);
  {
    #if CYTHON_USE_MODULE_STATE
    PyObject **__pyx_pyargnames[] = {&__pyx_n_s_prefix,0};
    #else
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_prefix,0};
    #endif
    PyObject* values[1] = {0};
    values[0] = ((PyObject*)__pyx_kp_b_);
    if (__pyx_kwds) {
      Py_ssize_t kw_args;
      switch (__pyx_nargs) {
        case  1: values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = __Pyx_NumKwargs_FASTCALL(__pyx_kwds);
      switch (__pyx_nargs) {
        case  0:
        if (kw_args > 0) {
          PyObject* value = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_prefix);
          if (value) { values[0] = value; kw_args--; }
          else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 255, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        const Py_ssize_t kwd_pos_args = __pyx_nargs;
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values + 0, kwd_pos_args, "view") < 0)) __PYX_ERR(0, 255, __pyx_L3_error)
      }
    } else {
      switch (__pyx_nargs) {
        case  1: values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_prefix = ((PyObject*)values[0]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("view", 0, 0, 1, __pyx_nargs); __PYX_ERR(0, 255, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("aim.storage.prefixview.PrefixView.view", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_prefix), (&PyBytes_Type), 1, "prefix", 1))) __PYX_ERR(0, 257, __pyx_L1_error)
  __pyx_r = __pyx_pf_3aim_7storage_10prefixview_10PrefixView_33view(((struct __pyx_obj_3aim_7storage_10prefixview_PrefixView *)__pyx_v_self), __pyx_v_prefix);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static struct __pyx_obj_3aim_7storage_9container_Container *__pyx_pf_3aim_7storage_10prefixview_10PrefixView_33view(struct __pyx_obj_3aim_7storage_10prefixview_PrefixView *__pyx_v_self, PyObject *__pyx_v_prefix) {
  struct __pyx_obj_3aim_7storage_9container_Container *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("view", 0);

  /* "aim/storage/prefixview.py":282
 *         """
 *         # TODO *args instead?
 *         return self.parent.view(self.prefix + prefix)             # <<<<<<<<<<<<<<
 * 
 *     def tree(
 */
  __Pyx_XDECREF((PyObject *)__pyx_r);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->parent, __pyx_n_s_view); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 282, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyNumber_Add(__pyx_v_self->prefix, __pyx_v_prefix); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 282, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_5 = 1;
    }
  }
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_t_3};
    __pyx_t_1 = __Pyx_PyObject_FastCall(__pyx_t_2, __pyx_callargs+1-__pyx_t_5, 1+__pyx_t_5);
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 282, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  }
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_3aim_7storage_9container_Container))))) __PYX_ERR(0, 282, __pyx_L1_error)
  __pyx_r = ((struct __pyx_obj_3aim_7storage_9container_Container *)__pyx_t_1);
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "aim/storage/prefixview.py":255
 *         return PrefixViewItemsIterator(self, key)
 * 
 *     def view(             # <<<<<<<<<<<<<<
 *         self,
 *         prefix: bytes = b''
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("aim.storage.prefixview.PrefixView.view", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "aim/storage/prefixview.py":284
 *         return self.parent.view(self.prefix + prefix)
 * 
 *     def tree(             # <<<<<<<<<<<<<<
 *         self
 *     ) -> ContainerTreeView:
 */

static PyObject *__pyx_pw_3aim_7storage_10prefixview_10PrefixView_36tree(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static struct __pyx_obj_3aim_7storage_17containertreeview_ContainerTreeView *__pyx_f_3aim_7storage_10prefixview_10PrefixView_tree(struct __pyx_obj_3aim_7storage_10prefixview_PrefixView *__pyx_v_self, int __pyx_skip_dispatch) {
  struct __pyx_obj_3aim_7storage_17containertreeview_ContainerTreeView *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("tree", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || __Pyx_PyType_HasFeature(Py_TYPE(((PyObject *)__pyx_v_self)), (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_typedict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_tree); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 284, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      #ifdef __Pyx_CyFunction_USED
      if (!__Pyx_IsCyOrPyCFunction(__pyx_t_1)
      #else
      if (!PyCFunction_Check(__pyx_t_1)
      #endif
              || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_3aim_7storage_10prefixview_10PrefixView_36tree)) {
        __Pyx_XDECREF((PyObject *)__pyx_r);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        __pyx_t_5 = 0;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
            __pyx_t_5 = 1;
          }
        }
        {
          PyObject *__pyx_callargs[1] = {__pyx_t_4, };
          __pyx_t_2 = __Pyx_PyObject_FastCall(__pyx_t_3, __pyx_callargs+1-__pyx_t_5, 0+__pyx_t_5);
          __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
          if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 284, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        }
        if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_ptype_3aim_7storage_17containertreeview_ContainerTreeView))))) __PYX_ERR(0, 284, __pyx_L1_error)
        __pyx_r = ((struct __pyx_obj_3aim_7storage_17containertreeview_ContainerTreeView *)__pyx_t_2);
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_typedict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "aim/storage/prefixview.py":303
 *             `tree[b'meta'][b'x'] == b'123'`
 *         """
 *         return ContainerTreeView(self)             # <<<<<<<<<<<<<<
 * 
 *     def batch(self):
 */
  __Pyx_XDECREF((PyObject *)__pyx_r);
  __pyx_t_1 = __Pyx_PyObject_CallOneArg(((PyObject *)__pyx_ptype_3aim_7storage_17containertreeview_ContainerTreeView), ((PyObject *)__pyx_v_self)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 303, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = ((struct __pyx_obj_3aim_7storage_17containertreeview_ContainerTreeView *)__pyx_t_1);
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "aim/storage/prefixview.py":284
 *         return self.parent.view(self.prefix + prefix)
 * 
 *     def tree(             # <<<<<<<<<<<<<<
 *         self
 *     ) -> ContainerTreeView:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("aim.storage.prefixview.PrefixView.tree", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_3aim_7storage_10prefixview_10PrefixView_36tree(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_3aim_7storage_10prefixview_10PrefixView_35tree, "Return a :obj:`ContainerTreeView` which enables hierarchical view and access\n        to the container records.\n\n        This is achieved by prefixing groups and using `PATH_SENTINEL` as a\n        separator for keys.\n\n        For example, if the Container contents are:\n            `{\n                b'e.y': b'012',\n                b'meta.x': b'123',\n                b'meta.z': b'x',\n                b'zzz': b'oOo'\n            }`, and the path sentinel is `b'.'` then `tree = container.tree()`\n            will behave as a (possibly deep) dict-like object:\n            `tree[b'meta'][b'x'] == b'123'`\n        ");
static PyMethodDef __pyx_mdef_3aim_7storage_10prefixview_10PrefixView_36tree = {"tree", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_3aim_7storage_10prefixview_10PrefixView_36tree, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_3aim_7storage_10prefixview_10PrefixView_35tree};
static PyObject *__pyx_pw_3aim_7storage_10prefixview_10PrefixView_36tree(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED const Py_ssize_t __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("tree (wrapper)", 0);
  if (unlikely(__pyx_nargs > 0)) {
    __Pyx_RaiseArgtupleInvalid("tree", 1, 0, 0, __pyx_nargs); return NULL;}
  if (unlikely(__pyx_kwds) && __Pyx_NumKwargs_FASTCALL(__pyx_kwds) && unlikely(!__Pyx_CheckKeywordStrings(__pyx_kwds, "tree", 0))) return NULL;
  __pyx_r = __pyx_pf_3aim_7storage_10prefixview_10PrefixView_35tree(((struct __pyx_obj_3aim_7storage_10prefixview_PrefixView *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_3aim_7storage_10prefixview_10PrefixView_35tree(struct __pyx_obj_3aim_7storage_10prefixview_PrefixView *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("tree", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((PyObject *)__pyx_f_3aim_7storage_10prefixview_10PrefixView_tree(__pyx_v_self, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 284, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("aim.storage.prefixview.PrefixView.tree", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "aim/storage/prefixview.py":305
 *         return ContainerTreeView(self)
 * 
 *     def batch(self):             # <<<<<<<<<<<<<<
 *         """Creates a new batch object to store operations in before executing
 *         using :obj:`Container.commit`.
 */

/* Python wrapper */
static PyObject *__pyx_pw_3aim_7storage_10prefixview_10PrefixView_38batch(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_3aim_7storage_10prefixview_10PrefixView_37batch, "Creates a new batch object to store operations in before executing\n        using :obj:`Container.commit`.\n\n        The operations :obj:`Container.set`, :obj:`Container.delete`,\n        :obj:`Container.delete_range` are supported.\n\n        See more at :obj:`Container.commit`\n        ");
static PyMethodDef __pyx_mdef_3aim_7storage_10prefixview_10PrefixView_38batch = {"batch", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_3aim_7storage_10prefixview_10PrefixView_38batch, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_3aim_7storage_10prefixview_10PrefixView_37batch};
static PyObject *__pyx_pw_3aim_7storage_10prefixview_10PrefixView_38batch(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED const Py_ssize_t __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("batch (wrapper)", 0);
  if (unlikely(__pyx_nargs > 0)) {
    __Pyx_RaiseArgtupleInvalid("batch", 1, 0, 0, __pyx_nargs); return NULL;}
  if (unlikely(__pyx_kwds) && __Pyx_NumKwargs_FASTCALL(__pyx_kwds) && unlikely(!__Pyx_CheckKeywordStrings(__pyx_kwds, "batch", 0))) return NULL;
  __pyx_r = __pyx_pf_3aim_7storage_10prefixview_10PrefixView_37batch(((struct __pyx_obj_3aim_7storage_10prefixview_PrefixView *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_3aim_7storage_10prefixview_10PrefixView_37batch(struct __pyx_obj_3aim_7storage_10prefixview_PrefixView *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("batch", 0);

  /* "aim/storage/prefixview.py":314
 *         See more at :obj:`Container.commit`
 *         """
 *         return self.parent.batch()             # <<<<<<<<<<<<<<
 * 
 *     def commit(
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->parent, __pyx_n_s_batch); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 314, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  __pyx_t_4 = 0;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_4 = 1;
    }
  }
  {
    PyObject *__pyx_callargs[1] = {__pyx_t_3, };
    __pyx_t_1 = __Pyx_PyObject_FastCall(__pyx_t_2, __pyx_callargs+1-__pyx_t_4, 0+__pyx_t_4);
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 314, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  }
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "aim/storage/prefixview.py":305
 *         return ContainerTreeView(self)
 * 
 *     def batch(self):             # <<<<<<<<<<<<<<
 *         """Creates a new batch object to store operations in before executing
 *         using :obj:`Container.commit`.
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("aim.storage.prefixview.PrefixView.batch", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "aim/storage/prefixview.py":316
 *         return self.parent.batch()
 * 
 *     def commit(             # <<<<<<<<<<<<<<
 *         self,
 *         batch
 */

/* Python wrapper */
static PyObject *__pyx_pw_3aim_7storage_10prefixview_10PrefixView_40commit(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_3aim_7storage_10prefixview_10PrefixView_39commit, "Execute the accumulated write operations in the given `batch`.\n\n        Depending on the :obj:`Container` implementation, this may feature\n        transactions, atomic writes, etc.\n        ");
static PyMethodDef __pyx_mdef_3aim_7storage_10prefixview_10PrefixView_40commit = {"commit", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_3aim_7storage_10prefixview_10PrefixView_40commit, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_3aim_7storage_10prefixview_10PrefixView_39commit};
static PyObject *__pyx_pw_3aim_7storage_10prefixview_10PrefixView_40commit(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_batch = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED const Py_ssize_t __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("commit (wrapper)", 0);
  {
    #if CYTHON_USE_MODULE_STATE
    PyObject **__pyx_pyargnames[] = {&__pyx_n_s_batch,0};
    #else
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_batch,0};
    #endif
    PyObject* values[1] = {0};
    if (__pyx_kwds) {
      Py_ssize_t kw_args;
      switch (__pyx_nargs) {
        case  1: values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = __Pyx_NumKwargs_FASTCALL(__pyx_kwds);
      switch (__pyx_nargs) {
        case  0:
        if (likely((values[0] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_batch)) != 0)) kw_args--;
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 316, __pyx_L3_error)
        else goto __pyx_L5_argtuple_error;
      }
      if (unlikely(kw_args > 0)) {
        const Py_ssize_t kwd_pos_args = __pyx_nargs;
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values + 0, kwd_pos_args, "commit") < 0)) __PYX_ERR(0, 316, __pyx_L3_error)
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
    }
    __pyx_v_batch = values[0];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("commit", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 316, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("aim.storage.prefixview.PrefixView.commit", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_3aim_7storage_10prefixview_10PrefixView_39commit(((struct __pyx_obj_3aim_7storage_10prefixview_PrefixView *)__pyx_v_self), __pyx_v_batch);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_3aim_7storage_10prefixview_10PrefixView_39commit(struct __pyx_obj_3aim_7storage_10prefixview_PrefixView *__pyx_v_self, PyObject *__pyx_v_batch) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("commit", 0);

  /* "aim/storage/prefixview.py":325
 *         transactions, atomic writes, etc.
 *         """
 *         return self.parent.commit(batch)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->parent, __pyx_n_s_commit); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 325, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  __pyx_t_4 = 0;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_4 = 1;
    }
  }
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_v_batch};
    __pyx_t_1 = __Pyx_PyObject_FastCall(__pyx_t_2, __pyx_callargs+1-__pyx_t_4, 1+__pyx_t_4);
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 325, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  }
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "aim/storage/prefixview.py":316
 *         return self.parent.batch()
 * 
 *     def commit(             # <<<<<<<<<<<<<<
 *         self,
 *         batch
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("aim.storage.prefixview.PrefixView.commit", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "aim/storage/prefixview.pxd":10
 * 
 * cdef class PrefixView(Container):
 *     cdef public bytes prefix             # <<<<<<<<<<<<<<
 *     # TODO cdef public Container parent after all container bindings are ready
 *     cdef public parent
 */

/* Python wrapper */
static PyObject *__pyx_pw_3aim_7storage_10prefixview_10PrefixView_6prefix_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_3aim_7storage_10prefixview_10PrefixView_6prefix_1__get__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_3aim_7storage_10prefixview_10PrefixView_6prefix___get__(((struct __pyx_obj_3aim_7storage_10prefixview_PrefixView *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_3aim_7storage_10prefixview_10PrefixView_6prefix___get__(struct __pyx_obj_3aim_7storage_10prefixview_PrefixView *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->prefix);
  __pyx_r = __pyx_v_self->prefix;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_3aim_7storage_10prefixview_10PrefixView_6prefix_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_3aim_7storage_10prefixview_10PrefixView_6prefix_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_3aim_7storage_10prefixview_10PrefixView_6prefix_2__set__(((struct __pyx_obj_3aim_7storage_10prefixview_PrefixView *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_3aim_7storage_10prefixview_10PrefixView_6prefix_2__set__(struct __pyx_obj_3aim_7storage_10prefixview_PrefixView *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  if (!(likely(PyBytes_CheckExact(__pyx_v_value))||((__pyx_v_value) == Py_None) || __Pyx_RaiseUnexpectedTypeError("bytes", __pyx_v_value))) __PYX_ERR(1, 10, __pyx_L1_error)
  __pyx_t_1 = __pyx_v_value;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->prefix);
  __Pyx_DECREF(__pyx_v_self->prefix);
  __pyx_v_self->prefix = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("aim.storage.prefixview.PrefixView.prefix.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_3aim_7storage_10prefixview_10PrefixView_6prefix_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_3aim_7storage_10prefixview_10PrefixView_6prefix_5__del__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_3aim_7storage_10prefixview_10PrefixView_6prefix_4__del__(((struct __pyx_obj_3aim_7storage_10prefixview_PrefixView *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_3aim_7storage_10prefixview_10PrefixView_6prefix_4__del__(struct __pyx_obj_3aim_7storage_10prefixview_PrefixView *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->prefix);
  __Pyx_DECREF(__pyx_v_self->prefix);
  __pyx_v_self->prefix = ((PyObject*)Py_None);

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "aim/storage/prefixview.pxd":12
 *     cdef public bytes prefix
 *     # TODO cdef public Container parent after all container bindings are ready
 *     cdef public parent             # <<<<<<<<<<<<<<
 *     cdef public bint read_only
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_3aim_7storage_10prefixview_10PrefixView_6parent_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_3aim_7storage_10prefixview_10PrefixView_6parent_1__get__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_3aim_7storage_10prefixview_10PrefixView_6parent___get__(((struct __pyx_obj_3aim_7storage_10prefixview_PrefixView *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_3aim_7storage_10prefixview_10PrefixView_6parent___get__(struct __pyx_obj_3aim_7storage_10prefixview_PrefixView *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->parent);
  __pyx_r = __pyx_v_self->parent;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_3aim_7storage_10prefixview_10PrefixView_6parent_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_3aim_7storage_10prefixview_10PrefixView_6parent_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_3aim_7storage_10prefixview_10PrefixView_6parent_2__set__(((struct __pyx_obj_3aim_7storage_10prefixview_PrefixView *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_3aim_7storage_10prefixview_10PrefixView_6parent_2__set__(struct __pyx_obj_3aim_7storage_10prefixview_PrefixView *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);
  __Pyx_INCREF(__pyx_v_value);
  __Pyx_GIVEREF(__pyx_v_value);
  __Pyx_GOTREF(__pyx_v_self->parent);
  __Pyx_DECREF(__pyx_v_self->parent);
  __pyx_v_self->parent = __pyx_v_value;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_3aim_7storage_10prefixview_10PrefixView_6parent_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_3aim_7storage_10prefixview_10PrefixView_6parent_5__del__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_3aim_7storage_10prefixview_10PrefixView_6parent_4__del__(((struct __pyx_obj_3aim_7storage_10prefixview_PrefixView *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_3aim_7storage_10prefixview_10PrefixView_6parent_4__del__(struct __pyx_obj_3aim_7storage_10prefixview_PrefixView *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->parent);
  __Pyx_DECREF(__pyx_v_self->parent);
  __pyx_v_self->parent = Py_None;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "aim/storage/prefixview.pxd":13
 *     # TODO cdef public Container parent after all container bindings are ready
 *     cdef public parent
 *     cdef public bint read_only             # <<<<<<<<<<<<<<
 * 
 *     cpdef bytes absolute_path(self, bytes path = *)
 */

/* Python wrapper */
static PyObject *__pyx_pw_3aim_7storage_10prefixview_10PrefixView_9read_only_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_3aim_7storage_10prefixview_10PrefixView_9read_only_1__get__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_3aim_7storage_10prefixview_10PrefixView_9read_only___get__(((struct __pyx_obj_3aim_7storage_10prefixview_PrefixView *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_3aim_7storage_10prefixview_10PrefixView_9read_only___get__(struct __pyx_obj_3aim_7storage_10prefixview_PrefixView *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyBool_FromLong(__pyx_v_self->read_only); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 13, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("aim.storage.prefixview.PrefixView.read_only.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_3aim_7storage_10prefixview_10PrefixView_9read_only_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_3aim_7storage_10prefixview_10PrefixView_9read_only_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_3aim_7storage_10prefixview_10PrefixView_9read_only_2__set__(((struct __pyx_obj_3aim_7storage_10prefixview_PrefixView *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_3aim_7storage_10prefixview_10PrefixView_9read_only_2__set__(struct __pyx_obj_3aim_7storage_10prefixview_PrefixView *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_v_value); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 13, __pyx_L1_error)
  __pyx_v_self->read_only = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("aim.storage.prefixview.PrefixView.read_only.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

/* Python wrapper */
static PyObject *__pyx_pw_3aim_7storage_10prefixview_10PrefixView_42__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyMethodDef __pyx_mdef_3aim_7storage_10prefixview_10PrefixView_42__reduce_cython__ = {"__reduce_cython__", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_3aim_7storage_10prefixview_10PrefixView_42__reduce_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_3aim_7storage_10prefixview_10PrefixView_42__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED const Py_ssize_t __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  if (unlikely(__pyx_nargs > 0)) {
    __Pyx_RaiseArgtupleInvalid("__reduce_cython__", 1, 0, 0, __pyx_nargs); return NULL;}
  if (unlikely(__pyx_kwds) && __Pyx_NumKwargs_FASTCALL(__pyx_kwds) && unlikely(!__Pyx_CheckKeywordStrings(__pyx_kwds, "__reduce_cython__", 0))) return NULL;
  __pyx_r = __pyx_pf_3aim_7storage_10prefixview_10PrefixView_41__reduce_cython__(((struct __pyx_obj_3aim_7storage_10prefixview_PrefixView *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_3aim_7storage_10prefixview_10PrefixView_41__reduce_cython__(struct __pyx_obj_3aim_7storage_10prefixview_PrefixView *__pyx_v_self) {
  PyObject *__pyx_v_state = 0;
  PyObject *__pyx_v__dict = 0;
  int __pyx_v_use_setstate;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  int __pyx_t_4;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":5
 *     cdef object _dict
 *     cdef bint use_setstate
 *     state = (self.parent, self.prefix, self.read_only)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 */
  __pyx_t_1 = __Pyx_PyBool_FromLong(__pyx_v_self->read_only); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyTuple_New(3); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_v_self->parent);
  __Pyx_GIVEREF(__pyx_v_self->parent);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_v_self->parent);
  __Pyx_INCREF(__pyx_v_self->prefix);
  __Pyx_GIVEREF(__pyx_v_self->prefix);
  PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_v_self->prefix);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_2, 2, __pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_v_state = ((PyObject*)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "(tree fragment)":6
 *     cdef bint use_setstate
 *     state = (self.parent, self.prefix, self.read_only)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None:
 *         state += (_dict,)
 */
  __pyx_t_2 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_n_s_dict, Py_None); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_v__dict = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "(tree fragment)":7
 *     state = (self.parent, self.prefix, self.read_only)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
  __pyx_t_3 = (__pyx_v__dict != Py_None);
  __pyx_t_4 = (__pyx_t_3 != 0);
  if (__pyx_t_4) {

    /* "(tree fragment)":8
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
 */
    __pyx_t_2 = PyTuple_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_v__dict);
    __pyx_t_1 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_1));
    __pyx_t_1 = 0;

    /* "(tree fragment)":9
 *     if _dict is not None:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = self.parent is not None or self.prefix is not None
 */
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":7
 *     state = (self.parent, self.prefix, self.read_only)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
    goto __pyx_L3;
  }

  /* "(tree fragment)":11
 *         use_setstate = True
 *     else:
 *         use_setstate = self.parent is not None or self.prefix is not None             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_PrefixView, (type(self), 0x0d8b696, None), state
 */
  /*else*/ {
    __pyx_t_3 = (__pyx_v_self->parent != Py_None);
    __pyx_t_5 = (__pyx_t_3 != 0);
    if (!__pyx_t_5) {
    } else {
      __pyx_t_4 = __pyx_t_5;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_5 = (__pyx_v_self->prefix != ((PyObject*)Py_None));
    __pyx_t_3 = (__pyx_t_5 != 0);
    __pyx_t_4 = __pyx_t_3;
    __pyx_L4_bool_binop_done:;
    __pyx_v_use_setstate = __pyx_t_4;
  }
  __pyx_L3:;

  /* "(tree fragment)":12
 *     else:
 *         use_setstate = self.parent is not None or self.prefix is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_PrefixView, (type(self), 0x0d8b696, None), state
 *     else:
 */
  __pyx_t_4 = (__pyx_v_use_setstate != 0);
  if (__pyx_t_4) {

    /* "(tree fragment)":13
 *         use_setstate = self.parent is not None or self.prefix is not None
 *     if use_setstate:
 *         return __pyx_unpickle_PrefixView, (type(self), 0x0d8b696, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_PrefixView, (type(self), 0x0d8b696, state)
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_pyx_unpickle_PrefixView); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = PyTuple_New(3); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_2, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_14202518);
    __Pyx_GIVEREF(__pyx_int_14202518);
    PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_int_14202518);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    PyTuple_SET_ITEM(__pyx_t_2, 2, Py_None);
    __pyx_t_6 = PyTuple_New(3); if (unlikely(!__pyx_t_6)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_t_2);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_6, 2, __pyx_v_state);
    __pyx_t_1 = 0;
    __pyx_t_2 = 0;
    __pyx_r = __pyx_t_6;
    __pyx_t_6 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":12
 *     else:
 *         use_setstate = self.parent is not None or self.prefix is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_PrefixView, (type(self), 0x0d8b696, None), state
 *     else:
 */
  }

  /* "(tree fragment)":15
 *         return __pyx_unpickle_PrefixView, (type(self), 0x0d8b696, None), state
 *     else:
 *         return __pyx_unpickle_PrefixView, (type(self), 0x0d8b696, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_PrefixView__set_state(self, __pyx_state)
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_6, __pyx_n_s_pyx_unpickle_PrefixView); if (unlikely(!__pyx_t_6)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_2 = PyTuple_New(3); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_2, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_14202518);
    __Pyx_GIVEREF(__pyx_int_14202518);
    PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_int_14202518);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_2, 2, __pyx_v_state);
    __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_6);
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_t_2);
    __pyx_t_6 = 0;
    __pyx_t_2 = 0;
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("aim.storage.prefixview.PrefixView.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_PrefixView, (type(self), 0x0d8b696, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_PrefixView__set_state(self, __pyx_state)
 */

/* Python wrapper */
static PyObject *__pyx_pw_3aim_7storage_10prefixview_10PrefixView_44__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyMethodDef __pyx_mdef_3aim_7storage_10prefixview_10PrefixView_44__setstate_cython__ = {"__setstate_cython__", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_3aim_7storage_10prefixview_10PrefixView_44__setstate_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_3aim_7storage_10prefixview_10PrefixView_44__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v___pyx_state = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED const Py_ssize_t __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  {
    #if CYTHON_USE_MODULE_STATE
    PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pyx_state,0};
    #else
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pyx_state,0};
    #endif
    PyObject* values[1] = {0};
    if (__pyx_kwds) {
      Py_ssize_t kw_args;
      switch (__pyx_nargs) {
        case  1: values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = __Pyx_NumKwargs_FASTCALL(__pyx_kwds);
      switch (__pyx_nargs) {
        case  0:
        if (likely((values[0] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_pyx_state)) != 0)) kw_args--;
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(2, 16, __pyx_L3_error)
        else goto __pyx_L5_argtuple_error;
      }
      if (unlikely(kw_args > 0)) {
        const Py_ssize_t kwd_pos_args = __pyx_nargs;
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values + 0, kwd_pos_args, "__setstate_cython__") < 0)) __PYX_ERR(2, 16, __pyx_L3_error)
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
    }
    __pyx_v___pyx_state = values[0];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__setstate_cython__", 1, 1, 1, __pyx_nargs); __PYX_ERR(2, 16, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("aim.storage.prefixview.PrefixView.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_3aim_7storage_10prefixview_10PrefixView_43__setstate_cython__(((struct __pyx_obj_3aim_7storage_10prefixview_PrefixView *)__pyx_v_self), __pyx_v___pyx_state);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_3aim_7storage_10prefixview_10PrefixView_43__setstate_cython__(struct __pyx_obj_3aim_7storage_10prefixview_PrefixView *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":17
 *         return __pyx_unpickle_PrefixView, (type(self), 0x0d8b696, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_PrefixView__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
 */
  if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None) || __Pyx_RaiseUnexpectedTypeError("tuple", __pyx_v___pyx_state))) __PYX_ERR(2, 17, __pyx_L1_error)
  __pyx_t_1 = __pyx_f_3aim_7storage_10prefixview___pyx_unpickle_PrefixView__set_state(__pyx_v_self, ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_PrefixView, (type(self), 0x0d8b696, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_PrefixView__set_state(self, __pyx_state)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("aim.storage.prefixview.PrefixView.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "aim/storage/prefixview.py":330
 * class PrefixViewItemsIterator(ContainerItemsIterator):
 * 
 *     def __init__(self, prefix_view, key):             # <<<<<<<<<<<<<<
 *         self.prefix_view = prefix_view
 *         self.path = prefix_view.absolute_path(key)
 */

/* Python wrapper */
static int __pyx_pw_3aim_7storage_10prefixview_23PrefixViewItemsIterator_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_3aim_7storage_10prefixview_23PrefixViewItemsIterator_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_prefix_view = 0;
  PyObject *__pyx_v_key = 0;
  CYTHON_UNUSED const Py_ssize_t __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    #if CYTHON_USE_MODULE_STATE
    PyObject **__pyx_pyargnames[] = {&__pyx_n_s_prefix_view,&__pyx_n_s_key,0};
    #else
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_prefix_view,&__pyx_n_s_key,0};
    #endif
    PyObject* values[2] = {0,0};
    if (__pyx_kwds) {
      Py_ssize_t kw_args;
      switch (__pyx_nargs) {
        case  2: values[1] = __Pyx_Arg_VARARGS(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = __Pyx_Arg_VARARGS(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = __Pyx_NumKwargs_VARARGS(__pyx_kwds);
      switch (__pyx_nargs) {
        case  0:
        if (likely((values[0] = __Pyx_GetKwValue_VARARGS(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_prefix_view)) != 0)) kw_args--;
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 330, __pyx_L3_error)
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_GetKwValue_VARARGS(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_key)) != 0)) kw_args--;
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 330, __pyx_L3_error)
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 1, 2, 2, 1); __PYX_ERR(0, 330, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        const Py_ssize_t kwd_pos_args = __pyx_nargs;
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values + 0, kwd_pos_args, "__init__") < 0)) __PYX_ERR(0, 330, __pyx_L3_error)
      }
    } else if (unlikely(__pyx_nargs != 2)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_Arg_VARARGS(__pyx_args, 0);
      values[1] = __Pyx_Arg_VARARGS(__pyx_args, 1);
    }
    __pyx_v_prefix_view = values[0];
    __pyx_v_key = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 2, 2, __pyx_nargs); __PYX_ERR(0, 330, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("aim.storage.prefixview.PrefixViewItemsIterator.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_3aim_7storage_10prefixview_23PrefixViewItemsIterator___init__(((struct __pyx_obj_3aim_7storage_10prefixview_PrefixViewItemsIterator *)__pyx_v_self), __pyx_v_prefix_view, __pyx_v_key);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_3aim_7storage_10prefixview_23PrefixViewItemsIterator___init__(struct __pyx_obj_3aim_7storage_10prefixview_PrefixViewItemsIterator *__pyx_v_self, PyObject *__pyx_v_prefix_view, PyObject *__pyx_v_key) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  Py_ssize_t __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "aim/storage/prefixview.py":331
 * 
 *     def __init__(self, prefix_view, key):
 *         self.prefix_view = prefix_view             # <<<<<<<<<<<<<<
 *         self.path = prefix_view.absolute_path(key)
 *         self.it = prefix_view.parent.items(self.path)
 */
  if (!(likely(((__pyx_v_prefix_view) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_prefix_view, __pyx_ptype_3aim_7storage_10prefixview_PrefixView))))) __PYX_ERR(0, 331, __pyx_L1_error)
  __pyx_t_1 = __pyx_v_prefix_view;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF((PyObject *)__pyx_v_self->prefix_view);
  __Pyx_DECREF((PyObject *)__pyx_v_self->prefix_view);
  __pyx_v_self->prefix_view = ((struct __pyx_obj_3aim_7storage_10prefixview_PrefixView *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "aim/storage/prefixview.py":332
 *     def __init__(self, prefix_view, key):
 *         self.prefix_view = prefix_view
 *         self.path = prefix_view.absolute_path(key)             # <<<<<<<<<<<<<<
 *         self.it = prefix_view.parent.items(self.path)
 *         # We store length of the path to strip it from the key faster
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_prefix_view, __pyx_n_s_absolute_path); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 332, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  __pyx_t_4 = 0;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_4 = 1;
    }
  }
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_v_key};
    __pyx_t_1 = __Pyx_PyObject_FastCall(__pyx_t_2, __pyx_callargs+1-__pyx_t_4, 1+__pyx_t_4);
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 332, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  }
  if (!(likely(PyBytes_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None) || __Pyx_RaiseUnexpectedTypeError("bytes", __pyx_t_1))) __PYX_ERR(0, 332, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->path);
  __Pyx_DECREF(__pyx_v_self->path);
  __pyx_v_self->path = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "aim/storage/prefixview.py":333
 *         self.prefix_view = prefix_view
 *         self.path = prefix_view.absolute_path(key)
 *         self.it = prefix_view.parent.items(self.path)             # <<<<<<<<<<<<<<
 *         # We store length of the path to strip it from the key faster
 *         self.prefix_len = len(self.path)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_prefix_view, __pyx_n_s_parent); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 333, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_items); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 333, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = NULL;
  __pyx_t_4 = 0;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
      __pyx_t_4 = 1;
    }
  }
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_v_self->path};
    __pyx_t_1 = __Pyx_PyObject_FastCall(__pyx_t_3, __pyx_callargs+1-__pyx_t_4, 1+__pyx_t_4);
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 333, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  }
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_3aim_7storage_9container_ContainerItemsIterator))))) __PYX_ERR(0, 333, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF((PyObject *)__pyx_v_self->it);
  __Pyx_DECREF((PyObject *)__pyx_v_self->it);
  __pyx_v_self->it = ((struct __pyx_obj_3aim_7storage_9container_ContainerItemsIterator *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "aim/storage/prefixview.py":335
 *         self.it = prefix_view.parent.items(self.path)
 *         # We store length of the path to strip it from the key faster
 *         self.prefix_len = len(self.path)             # <<<<<<<<<<<<<<
 * 
 *     def next(self):
 */
  __pyx_t_1 = __pyx_v_self->path;
  __Pyx_INCREF(__pyx_t_1);
  if (unlikely(__pyx_t_1 == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(0, 335, __pyx_L1_error)
  }
  __pyx_t_5 = PyBytes_GET_SIZE(__pyx_t_1); if (unlikely(__pyx_t_5 == ((Py_ssize_t)-1))) __PYX_ERR(0, 335, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_self->prefix_len = __pyx_t_5;

  /* "aim/storage/prefixview.py":330
 * class PrefixViewItemsIterator(ContainerItemsIterator):
 * 
 *     def __init__(self, prefix_view, key):             # <<<<<<<<<<<<<<
 *         self.prefix_view = prefix_view
 *         self.path = prefix_view.absolute_path(key)
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("aim.storage.prefixview.PrefixViewItemsIterator.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "aim/storage/prefixview.py":337
 *         self.prefix_len = len(self.path)
 * 
 *     def next(self):             # <<<<<<<<<<<<<<
 *         item = self.it.next()
 * 
 */

static PyObject *__pyx_pw_3aim_7storage_10prefixview_23PrefixViewItemsIterator_3next(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyObject *__pyx_f_3aim_7storage_10prefixview_23PrefixViewItemsIterator_next(struct __pyx_obj_3aim_7storage_10prefixview_PrefixViewItemsIterator *__pyx_v_self, int __pyx_skip_dispatch) {
  PyObject *__pyx_v_item = 0;
  PyObject *__pyx_v_keys = 0;
  PyObject *__pyx_v_value = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_t_6;
  int __pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("next", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || __Pyx_PyType_HasFeature(Py_TYPE(((PyObject *)__pyx_v_self)), (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_typedict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_next); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 337, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      #ifdef __Pyx_CyFunction_USED
      if (!__Pyx_IsCyOrPyCFunction(__pyx_t_1)
      #else
      if (!PyCFunction_Check(__pyx_t_1)
      #endif
              || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_3aim_7storage_10prefixview_23PrefixViewItemsIterator_3next)) {
        __Pyx_XDECREF(__pyx_r);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        __pyx_t_5 = 0;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
            __pyx_t_5 = 1;
          }
        }
        {
          PyObject *__pyx_callargs[1] = {__pyx_t_4, };
          __pyx_t_2 = __Pyx_PyObject_FastCall(__pyx_t_3, __pyx_callargs+1-__pyx_t_5, 0+__pyx_t_5);
          __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
          if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 337, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        }
        __pyx_r = __pyx_t_2;
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_typedict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "aim/storage/prefixview.py":338
 * 
 *     def next(self):
 *         item = self.it.next()             # <<<<<<<<<<<<<<
 * 
 *         if item is None:
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_3aim_7storage_9container_ContainerItemsIterator *)__pyx_v_self->it->__pyx_base.__pyx_vtab)->__pyx_base.next(((struct __pyx_obj_8aimrocks_11lib_rocksdb_Iterator *)__pyx_v_self->it), 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 338, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyTuple_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None) || __Pyx_RaiseUnexpectedTypeError("tuple", __pyx_t_1))) __PYX_ERR(0, 338, __pyx_L1_error)
  __pyx_v_item = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "aim/storage/prefixview.py":340
 *         item = self.it.next()
 * 
 *         if item is None:             # <<<<<<<<<<<<<<
 *             return None
 * 
 */
  __pyx_t_6 = (__pyx_v_item == ((PyObject*)Py_None));
  __pyx_t_7 = (__pyx_t_6 != 0);
  if (__pyx_t_7) {

    /* "aim/storage/prefixview.py":341
 * 
 *         if item is None:
 *             return None             # <<<<<<<<<<<<<<
 * 
 *         keys = item[0]
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_r = Py_None; __Pyx_INCREF(Py_None);
    goto __pyx_L0;

    /* "aim/storage/prefixview.py":340
 *         item = self.it.next()
 * 
 *         if item is None:             # <<<<<<<<<<<<<<
 *             return None
 * 
 */
  }

  /* "aim/storage/prefixview.py":343
 *             return None
 * 
 *         keys = item[0]             # <<<<<<<<<<<<<<
 *         value = item[1]
 * 
 */
  if (unlikely(__pyx_v_item == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(0, 343, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v_item, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 343, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyBytes_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None) || __Pyx_RaiseUnexpectedTypeError("bytes", __pyx_t_1))) __PYX_ERR(0, 343, __pyx_L1_error)
  __pyx_v_keys = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "aim/storage/prefixview.py":344
 * 
 *         keys = item[0]
 *         value = item[1]             # <<<<<<<<<<<<<<
 * 
 *         return keys[self.prefix_len:], value
 */
  if (unlikely(__pyx_v_item == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(0, 344, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v_item, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 344, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_value = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "aim/storage/prefixview.py":346
 *         value = item[1]
 * 
 *         return keys[self.prefix_len:], value             # <<<<<<<<<<<<<<
 */
  __Pyx_XDECREF(__pyx_r);
  if (unlikely(__pyx_v_keys == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(0, 346, __pyx_L1_error)
  }
  __pyx_t_1 = PySequence_GetSlice(__pyx_v_keys, __pyx_v_self->prefix_len, PY_SSIZE_T_MAX); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 346, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 346, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_1);
  __Pyx_INCREF(__pyx_v_value);
  __Pyx_GIVEREF(__pyx_v_value);
  PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_v_value);
  __pyx_t_1 = 0;
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "aim/storage/prefixview.py":337
 *         self.prefix_len = len(self.path)
 * 
 *     def next(self):             # <<<<<<<<<<<<<<
 *         item = self.it.next()
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("aim.storage.prefixview.PrefixViewItemsIterator.next", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_item);
  __Pyx_XDECREF(__pyx_v_keys);
  __Pyx_XDECREF(__pyx_v_value);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_3aim_7storage_10prefixview_23PrefixViewItemsIterator_3next(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyMethodDef __pyx_mdef_3aim_7storage_10prefixview_23PrefixViewItemsIterator_3next = {"next", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_3aim_7storage_10prefixview_23PrefixViewItemsIterator_3next, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_3aim_7storage_10prefixview_23PrefixViewItemsIterator_3next(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED const Py_ssize_t __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("next (wrapper)", 0);
  if (unlikely(__pyx_nargs > 0)) {
    __Pyx_RaiseArgtupleInvalid("next", 1, 0, 0, __pyx_nargs); return NULL;}
  if (unlikely(__pyx_kwds) && __Pyx_NumKwargs_FASTCALL(__pyx_kwds) && unlikely(!__Pyx_CheckKeywordStrings(__pyx_kwds, "next", 0))) return NULL;
  __pyx_r = __pyx_pf_3aim_7storage_10prefixview_23PrefixViewItemsIterator_2next(((struct __pyx_obj_3aim_7storage_10prefixview_PrefixViewItemsIterator *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_3aim_7storage_10prefixview_23PrefixViewItemsIterator_2next(struct __pyx_obj_3aim_7storage_10prefixview_PrefixViewItemsIterator *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("next", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_3aim_7storage_10prefixview_23PrefixViewItemsIterator_next(__pyx_v_self, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 337, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("aim.storage.prefixview.PrefixViewItemsIterator.next", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

/* Python wrapper */
static PyObject *__pyx_pw_3aim_7storage_10prefixview_23PrefixViewItemsIterator_5__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyMethodDef __pyx_mdef_3aim_7storage_10prefixview_23PrefixViewItemsIterator_5__reduce_cython__ = {"__reduce_cython__", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_3aim_7storage_10prefixview_23PrefixViewItemsIterator_5__reduce_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_3aim_7storage_10prefixview_23PrefixViewItemsIterator_5__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED const Py_ssize_t __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  if (unlikely(__pyx_nargs > 0)) {
    __Pyx_RaiseArgtupleInvalid("__reduce_cython__", 1, 0, 0, __pyx_nargs); return NULL;}
  if (unlikely(__pyx_kwds) && __Pyx_NumKwargs_FASTCALL(__pyx_kwds) && unlikely(!__Pyx_CheckKeywordStrings(__pyx_kwds, "__reduce_cython__", 0))) return NULL;
  __pyx_r = __pyx_pf_3aim_7storage_10prefixview_23PrefixViewItemsIterator_4__reduce_cython__(((struct __pyx_obj_3aim_7storage_10prefixview_PrefixViewItemsIterator *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_3aim_7storage_10prefixview_23PrefixViewItemsIterator_4__reduce_cython__(struct __pyx_obj_3aim_7storage_10prefixview_PrefixViewItemsIterator *__pyx_v_self) {
  PyObject *__pyx_v_state = 0;
  PyObject *__pyx_v__dict = 0;
  int __pyx_v_use_setstate;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  int __pyx_t_4;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":5
 *     cdef object _dict
 *     cdef bint use_setstate
 *     state = (self._current_value, self.it, self.path, self.prefix_len, self.prefix_view)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->prefix_len); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyTuple_New(5); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base._current_value);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base._current_value);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_v_self->__pyx_base.__pyx_base._current_value);
  __Pyx_INCREF((PyObject *)__pyx_v_self->it);
  __Pyx_GIVEREF((PyObject *)__pyx_v_self->it);
  PyTuple_SET_ITEM(__pyx_t_2, 1, ((PyObject *)__pyx_v_self->it));
  __Pyx_INCREF(__pyx_v_self->path);
  __Pyx_GIVEREF(__pyx_v_self->path);
  PyTuple_SET_ITEM(__pyx_t_2, 2, __pyx_v_self->path);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_2, 3, __pyx_t_1);
  __Pyx_INCREF((PyObject *)__pyx_v_self->prefix_view);
  __Pyx_GIVEREF((PyObject *)__pyx_v_self->prefix_view);
  PyTuple_SET_ITEM(__pyx_t_2, 4, ((PyObject *)__pyx_v_self->prefix_view));
  __pyx_t_1 = 0;
  __pyx_v_state = ((PyObject*)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "(tree fragment)":6
 *     cdef bint use_setstate
 *     state = (self._current_value, self.it, self.path, self.prefix_len, self.prefix_view)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None:
 *         state += (_dict,)
 */
  __pyx_t_2 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_n_s_dict, Py_None); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_v__dict = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "(tree fragment)":7
 *     state = (self._current_value, self.it, self.path, self.prefix_len, self.prefix_view)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
  __pyx_t_3 = (__pyx_v__dict != Py_None);
  __pyx_t_4 = (__pyx_t_3 != 0);
  if (__pyx_t_4) {

    /* "(tree fragment)":8
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
 */
    __pyx_t_2 = PyTuple_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_v__dict);
    __pyx_t_1 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_1));
    __pyx_t_1 = 0;

    /* "(tree fragment)":9
 *     if _dict is not None:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = self._current_value is not None or self.it is not None or self.path is not None or self.prefix_view is not None
 */
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":7
 *     state = (self._current_value, self.it, self.path, self.prefix_len, self.prefix_view)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
    goto __pyx_L3;
  }

  /* "(tree fragment)":11
 *         use_setstate = True
 *     else:
 *         use_setstate = self._current_value is not None or self.it is not None or self.path is not None or self.prefix_view is not None             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_PrefixViewItemsIterator, (type(self), 0xda33529, None), state
 */
  /*else*/ {
    __pyx_t_3 = (__pyx_v_self->__pyx_base.__pyx_base._current_value != Py_None);
    __pyx_t_5 = (__pyx_t_3 != 0);
    if (!__pyx_t_5) {
    } else {
      __pyx_t_4 = __pyx_t_5;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_5 = (((PyObject *)__pyx_v_self->it) != Py_None);
    __pyx_t_3 = (__pyx_t_5 != 0);
    if (!__pyx_t_3) {
    } else {
      __pyx_t_4 = __pyx_t_3;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_3 = (__pyx_v_self->path != ((PyObject*)Py_None));
    __pyx_t_5 = (__pyx_t_3 != 0);
    if (!__pyx_t_5) {
    } else {
      __pyx_t_4 = __pyx_t_5;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_5 = (((PyObject *)__pyx_v_self->prefix_view) != Py_None);
    __pyx_t_3 = (__pyx_t_5 != 0);
    __pyx_t_4 = __pyx_t_3;
    __pyx_L4_bool_binop_done:;
    __pyx_v_use_setstate = __pyx_t_4;
  }
  __pyx_L3:;

  /* "(tree fragment)":12
 *     else:
 *         use_setstate = self._current_value is not None or self.it is not None or self.path is not None or self.prefix_view is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_PrefixViewItemsIterator, (type(self), 0xda33529, None), state
 *     else:
 */
  __pyx_t_4 = (__pyx_v_use_setstate != 0);
  if (__pyx_t_4) {

    /* "(tree fragment)":13
 *         use_setstate = self._current_value is not None or self.it is not None or self.path is not None or self.prefix_view is not None
 *     if use_setstate:
 *         return __pyx_unpickle_PrefixViewItemsIterator, (type(self), 0xda33529, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_PrefixViewItemsIterator, (type(self), 0xda33529, state)
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_pyx_unpickle_PrefixViewItemsIt); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = PyTuple_New(3); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_2, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_228799785);
    __Pyx_GIVEREF(__pyx_int_228799785);
    PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_int_228799785);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    PyTuple_SET_ITEM(__pyx_t_2, 2, Py_None);
    __pyx_t_6 = PyTuple_New(3); if (unlikely(!__pyx_t_6)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_t_2);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_6, 2, __pyx_v_state);
    __pyx_t_1 = 0;
    __pyx_t_2 = 0;
    __pyx_r = __pyx_t_6;
    __pyx_t_6 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":12
 *     else:
 *         use_setstate = self._current_value is not None or self.it is not None or self.path is not None or self.prefix_view is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_PrefixViewItemsIterator, (type(self), 0xda33529, None), state
 *     else:
 */
  }

  /* "(tree fragment)":15
 *         return __pyx_unpickle_PrefixViewItemsIterator, (type(self), 0xda33529, None), state
 *     else:
 *         return __pyx_unpickle_PrefixViewItemsIterator, (type(self), 0xda33529, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_PrefixViewItemsIterator__set_state(self, __pyx_state)
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_6, __pyx_n_s_pyx_unpickle_PrefixViewItemsIt); if (unlikely(!__pyx_t_6)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_2 = PyTuple_New(3); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_2, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_228799785);
    __Pyx_GIVEREF(__pyx_int_228799785);
    PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_int_228799785);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_2, 2, __pyx_v_state);
    __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_6);
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_t_2);
    __pyx_t_6 = 0;
    __pyx_t_2 = 0;
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("aim.storage.prefixview.PrefixViewItemsIterator.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_PrefixViewItemsIterator, (type(self), 0xda33529, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_PrefixViewItemsIterator__set_state(self, __pyx_state)
 */

/* Python wrapper */
static PyObject *__pyx_pw_3aim_7storage_10prefixview_23PrefixViewItemsIterator_7__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyMethodDef __pyx_mdef_3aim_7storage_10prefixview_23PrefixViewItemsIterator_7__setstate_cython__ = {"__setstate_cython__", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_3aim_7storage_10prefixview_23PrefixViewItemsIterator_7__setstate_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_3aim_7storage_10prefixview_23PrefixViewItemsIterator_7__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v___pyx_state = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED const Py_ssize_t __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  {
    #if CYTHON_USE_MODULE_STATE
    PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pyx_state,0};
    #else
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pyx_state,0};
    #endif
    PyObject* values[1] = {0};
    if (__pyx_kwds) {
      Py_ssize_t kw_args;
      switch (__pyx_nargs) {
        case  1: values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = __Pyx_NumKwargs_FASTCALL(__pyx_kwds);
      switch (__pyx_nargs) {
        case  0:
        if (likely((values[0] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_pyx_state)) != 0)) kw_args--;
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(2, 16, __pyx_L3_error)
        else goto __pyx_L5_argtuple_error;
      }
      if (unlikely(kw_args > 0)) {
        const Py_ssize_t kwd_pos_args = __pyx_nargs;
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values + 0, kwd_pos_args, "__setstate_cython__") < 0)) __PYX_ERR(2, 16, __pyx_L3_error)
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
    }
    __pyx_v___pyx_state = values[0];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__setstate_cython__", 1, 1, 1, __pyx_nargs); __PYX_ERR(2, 16, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("aim.storage.prefixview.PrefixViewItemsIterator.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_3aim_7storage_10prefixview_23PrefixViewItemsIterator_6__setstate_cython__(((struct __pyx_obj_3aim_7storage_10prefixview_PrefixViewItemsIterator *)__pyx_v_self), __pyx_v___pyx_state);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_3aim_7storage_10prefixview_23PrefixViewItemsIterator_6__setstate_cython__(struct __pyx_obj_3aim_7storage_10prefixview_PrefixViewItemsIterator *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":17
 *         return __pyx_unpickle_PrefixViewItemsIterator, (type(self), 0xda33529, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_PrefixViewItemsIterator__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
 */
  if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None) || __Pyx_RaiseUnexpectedTypeError("tuple", __pyx_v___pyx_state))) __PYX_ERR(2, 17, __pyx_L1_error)
  __pyx_t_1 = __pyx_f_3aim_7storage_10prefixview___pyx_unpickle_PrefixViewItemsIterator__set_state(__pyx_v_self, ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_PrefixViewItemsIterator, (type(self), 0xda33529, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_PrefixViewItemsIterator__set_state(self, __pyx_state)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("aim.storage.prefixview.PrefixViewItemsIterator.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __pyx_unpickle_PrefixView(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

/* Python wrapper */
static PyObject *__pyx_pw_3aim_7storage_10prefixview_1__pyx_unpickle_PrefixView(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyMethodDef __pyx_mdef_3aim_7storage_10prefixview_1__pyx_unpickle_PrefixView = {"__pyx_unpickle_PrefixView", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_3aim_7storage_10prefixview_1__pyx_unpickle_PrefixView, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_3aim_7storage_10prefixview_1__pyx_unpickle_PrefixView(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED const Py_ssize_t __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle_PrefixView (wrapper)", 0);
  {
    #if CYTHON_USE_MODULE_STATE
    PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pyx_type,&__pyx_n_s_pyx_checksum,&__pyx_n_s_pyx_state,0};
    #else
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pyx_type,&__pyx_n_s_pyx_checksum,&__pyx_n_s_pyx_state,0};
    #endif
    PyObject* values[3] = {0,0,0};
    if (__pyx_kwds) {
      Py_ssize_t kw_args;
      switch (__pyx_nargs) {
        case  3: values[2] = __Pyx_Arg_FASTCALL(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = __Pyx_Arg_FASTCALL(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = __Pyx_NumKwargs_FASTCALL(__pyx_kwds);
      switch (__pyx_nargs) {
        case  0:
        if (likely((values[0] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_pyx_type)) != 0)) kw_args--;
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(2, 1, __pyx_L3_error)
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_pyx_checksum)) != 0)) kw_args--;
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(2, 1, __pyx_L3_error)
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_PrefixView", 1, 3, 3, 1); __PYX_ERR(2, 1, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_pyx_state)) != 0)) kw_args--;
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(2, 1, __pyx_L3_error)
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_PrefixView", 1, 3, 3, 2); __PYX_ERR(2, 1, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        const Py_ssize_t kwd_pos_args = __pyx_nargs;
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values + 0, kwd_pos_args, "__pyx_unpickle_PrefixView") < 0)) __PYX_ERR(2, 1, __pyx_L3_error)
      }
    } else if (unlikely(__pyx_nargs != 3)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
      values[1] = __Pyx_Arg_FASTCALL(__pyx_args, 1);
      values[2] = __Pyx_Arg_FASTCALL(__pyx_args, 2);
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyInt_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(2, 1, __pyx_L3_error)
    __pyx_v___pyx_state = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_PrefixView", 1, 3, 3, __pyx_nargs); __PYX_ERR(2, 1, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("aim.storage.prefixview.__pyx_unpickle_PrefixView", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_3aim_7storage_10prefixview___pyx_unpickle_PrefixView(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_3aim_7storage_10prefixview___pyx_unpickle_PrefixView(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_PickleError = 0;
  PyObject *__pyx_v___pyx_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_PrefixView", 0);

  /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x0d8b696:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError, "Incompatible checksums (%s vs 0x0d8b696 = (parent, prefix, read_only))" % __pyx_checksum
 */
  __pyx_t_1 = ((__pyx_v___pyx_checksum != 0x0d8b696) != 0);
  if (__pyx_t_1) {

    /* "(tree fragment)":5
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x0d8b696:
 *         from pickle import PickleError as __pyx_PickleError             # <<<<<<<<<<<<<<
 *         raise __pyx_PickleError, "Incompatible checksums (%s vs 0x0d8b696 = (parent, prefix, read_only))" % __pyx_checksum
 *     __pyx_result = PrefixView.__new__(__pyx_type)
 */
    __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_n_s_PickleError);
    __Pyx_GIVEREF(__pyx_n_s_PickleError);
    PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_PickleError);
    __pyx_t_3 = __Pyx_Import(__pyx_n_s_pickle, __pyx_t_2, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_3, __pyx_n_s_PickleError); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v___pyx_PickleError = __pyx_t_2;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":6
 *     if __pyx_checksum != 0x0d8b696:
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError, "Incompatible checksums (%s vs 0x0d8b696 = (parent, prefix, read_only))" % __pyx_checksum             # <<<<<<<<<<<<<<
 *     __pyx_result = PrefixView.__new__(__pyx_type)
 *     if __pyx_state is not None:
 */
    __pyx_t_3 = __Pyx_PyInt_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_2 = __Pyx_PyString_Format(__pyx_kp_s_Incompatible_checksums_s_vs_0x0d, __pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_Raise(__pyx_v___pyx_PickleError, __pyx_t_2, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(2, 6, __pyx_L1_error)

    /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x0d8b696:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError, "Incompatible checksums (%s vs 0x0d8b696 = (parent, prefix, read_only))" % __pyx_checksum
 */
  }

  /* "(tree fragment)":7
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError, "Incompatible checksums (%s vs 0x0d8b696 = (parent, prefix, read_only))" % __pyx_checksum
 *     __pyx_result = PrefixView.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_PrefixView__set_state(<PrefixView> __pyx_result, __pyx_state)
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_3aim_7storage_10prefixview_PrefixView), __pyx_n_s_new); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
      __pyx_t_5 = 1;
    }
  }
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_v___pyx_type};
    __pyx_t_2 = __Pyx_PyObject_FastCall(__pyx_t_3, __pyx_callargs+1-__pyx_t_5, 1+__pyx_t_5);
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 7, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  }
  __pyx_v___pyx_result = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "(tree fragment)":8
 *         raise __pyx_PickleError, "Incompatible checksums (%s vs 0x0d8b696 = (parent, prefix, read_only))" % __pyx_checksum
 *     __pyx_result = PrefixView.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_PrefixView__set_state(<PrefixView> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  __pyx_t_1 = (__pyx_v___pyx_state != Py_None);
  __pyx_t_6 = (__pyx_t_1 != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":9
 *     __pyx_result = PrefixView.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_PrefixView__set_state(<PrefixView> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_PrefixView__set_state(PrefixView __pyx_result, tuple __pyx_state):
 */
    if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None) || __Pyx_RaiseUnexpectedTypeError("tuple", __pyx_v___pyx_state))) __PYX_ERR(2, 9, __pyx_L1_error)
    __pyx_t_2 = __pyx_f_3aim_7storage_10prefixview___pyx_unpickle_PrefixView__set_state(((struct __pyx_obj_3aim_7storage_10prefixview_PrefixView *)__pyx_v___pyx_result), ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 9, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "(tree fragment)":8
 *         raise __pyx_PickleError, "Incompatible checksums (%s vs 0x0d8b696 = (parent, prefix, read_only))" % __pyx_checksum
 *     __pyx_result = PrefixView.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_PrefixView__set_state(<PrefixView> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  }

  /* "(tree fragment)":10
 *     if __pyx_state is not None:
 *         __pyx_unpickle_PrefixView__set_state(<PrefixView> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_PrefixView__set_state(PrefixView __pyx_result, tuple __pyx_state):
 *     __pyx_result.parent = __pyx_state[0]; __pyx_result.prefix = __pyx_state[1]; __pyx_result.read_only = __pyx_state[2]
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_PrefixView(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("aim.storage.prefixview.__pyx_unpickle_PrefixView", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_PickleError);
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":11
 *         __pyx_unpickle_PrefixView__set_state(<PrefixView> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_PrefixView__set_state(PrefixView __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.parent = __pyx_state[0]; __pyx_result.prefix = __pyx_state[1]; __pyx_result.read_only = __pyx_state[2]
 *     if len(__pyx_state) > 3 and hasattr(__pyx_result, '__dict__'):
 */

static PyObject *__pyx_f_3aim_7storage_10prefixview___pyx_unpickle_PrefixView__set_state(struct __pyx_obj_3aim_7storage_10prefixview_PrefixView *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  Py_ssize_t __pyx_t_3;
  int __pyx_t_4;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_t_9;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_PrefixView__set_state", 0);

  /* "(tree fragment)":12
 *     return __pyx_result
 * cdef __pyx_unpickle_PrefixView__set_state(PrefixView __pyx_result, tuple __pyx_state):
 *     __pyx_result.parent = __pyx_state[0]; __pyx_result.prefix = __pyx_state[1]; __pyx_result.read_only = __pyx_state[2]             # <<<<<<<<<<<<<<
 *     if len(__pyx_state) > 3 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[3])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->parent);
  __Pyx_DECREF(__pyx_v___pyx_result->parent);
  __pyx_v___pyx_result->parent = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyBytes_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None) || __Pyx_RaiseUnexpectedTypeError("bytes", __pyx_t_1))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->prefix);
  __Pyx_DECREF(__pyx_v___pyx_result->prefix);
  __pyx_v___pyx_result->prefix = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->read_only = __pyx_t_2;

  /* "(tree fragment)":13
 * cdef __pyx_unpickle_PrefixView__set_state(PrefixView __pyx_result, tuple __pyx_state):
 *     __pyx_result.parent = __pyx_state[0]; __pyx_result.prefix = __pyx_state[1]; __pyx_result.read_only = __pyx_state[2]
 *     if len(__pyx_state) > 3 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[3])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(2, 13, __pyx_L1_error)
  }
  __pyx_t_3 = PyTuple_GET_SIZE(__pyx_v___pyx_state); if (unlikely(__pyx_t_3 == ((Py_ssize_t)-1))) __PYX_ERR(2, 13, __pyx_L1_error)
  __pyx_t_4 = ((__pyx_t_3 > 3) != 0);
  if (__pyx_t_4) {
  } else {
    __pyx_t_2 = __pyx_t_4;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_4 = __Pyx_HasAttr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(2, 13, __pyx_L1_error)
  __pyx_t_5 = (__pyx_t_4 != 0);
  __pyx_t_2 = __pyx_t_5;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_2) {

    /* "(tree fragment)":14
 *     __pyx_result.parent = __pyx_state[0]; __pyx_result.prefix = __pyx_state[1]; __pyx_result.read_only = __pyx_state[2]
 *     if len(__pyx_state) > 3 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[3])             # <<<<<<<<<<<<<<
 */
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(!__pyx_t_6)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_update); if (unlikely(!__pyx_t_7)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(2, 14, __pyx_L1_error)
    }
    __pyx_t_6 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 3, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_6)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_8 = NULL;
    __pyx_t_9 = 0;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_7))) {
      __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_7);
      if (likely(__pyx_t_8)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
        __Pyx_INCREF(__pyx_t_8);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_7, function);
        __pyx_t_9 = 1;
      }
    }
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_8, __pyx_t_6};
      __pyx_t_1 = __Pyx_PyObject_FastCall(__pyx_t_7, __pyx_callargs+1-__pyx_t_9, 1+__pyx_t_9);
      __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 14, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "(tree fragment)":13
 * cdef __pyx_unpickle_PrefixView__set_state(PrefixView __pyx_result, tuple __pyx_state):
 *     __pyx_result.parent = __pyx_state[0]; __pyx_result.prefix = __pyx_state[1]; __pyx_result.read_only = __pyx_state[2]
 *     if len(__pyx_state) > 3 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[3])
 */
  }

  /* "(tree fragment)":11
 *         __pyx_unpickle_PrefixView__set_state(<PrefixView> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_PrefixView__set_state(PrefixView __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.parent = __pyx_state[0]; __pyx_result.prefix = __pyx_state[1]; __pyx_result.read_only = __pyx_state[2]
 *     if len(__pyx_state) > 3 and hasattr(__pyx_result, '__dict__'):
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("aim.storage.prefixview.__pyx_unpickle_PrefixView__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __pyx_unpickle_PrefixViewItemsIterator(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

/* Python wrapper */
static PyObject *__pyx_pw_3aim_7storage_10prefixview_3__pyx_unpickle_PrefixViewItemsIterator(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyMethodDef __pyx_mdef_3aim_7storage_10prefixview_3__pyx_unpickle_PrefixViewItemsIterator = {"__pyx_unpickle_PrefixViewItemsIterator", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_3aim_7storage_10prefixview_3__pyx_unpickle_PrefixViewItemsIterator, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_3aim_7storage_10prefixview_3__pyx_unpickle_PrefixViewItemsIterator(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED const Py_ssize_t __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle_PrefixViewItemsIterator (wrapper)", 0);
  {
    #if CYTHON_USE_MODULE_STATE
    PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pyx_type,&__pyx_n_s_pyx_checksum,&__pyx_n_s_pyx_state,0};
    #else
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pyx_type,&__pyx_n_s_pyx_checksum,&__pyx_n_s_pyx_state,0};
    #endif
    PyObject* values[3] = {0,0,0};
    if (__pyx_kwds) {
      Py_ssize_t kw_args;
      switch (__pyx_nargs) {
        case  3: values[2] = __Pyx_Arg_FASTCALL(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = __Pyx_Arg_FASTCALL(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = __Pyx_NumKwargs_FASTCALL(__pyx_kwds);
      switch (__pyx_nargs) {
        case  0:
        if (likely((values[0] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_pyx_type)) != 0)) kw_args--;
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(2, 1, __pyx_L3_error)
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_pyx_checksum)) != 0)) kw_args--;
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(2, 1, __pyx_L3_error)
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_PrefixViewItemsIterator", 1, 3, 3, 1); __PYX_ERR(2, 1, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_pyx_state)) != 0)) kw_args--;
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(2, 1, __pyx_L3_error)
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_PrefixViewItemsIterator", 1, 3, 3, 2); __PYX_ERR(2, 1, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        const Py_ssize_t kwd_pos_args = __pyx_nargs;
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values + 0, kwd_pos_args, "__pyx_unpickle_PrefixViewItemsIterator") < 0)) __PYX_ERR(2, 1, __pyx_L3_error)
      }
    } else if (unlikely(__pyx_nargs != 3)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
      values[1] = __Pyx_Arg_FASTCALL(__pyx_args, 1);
      values[2] = __Pyx_Arg_FASTCALL(__pyx_args, 2);
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyInt_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(2, 1, __pyx_L3_error)
    __pyx_v___pyx_state = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_PrefixViewItemsIterator", 1, 3, 3, __pyx_nargs); __PYX_ERR(2, 1, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("aim.storage.prefixview.__pyx_unpickle_PrefixViewItemsIterator", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_3aim_7storage_10prefixview_2__pyx_unpickle_PrefixViewItemsIterator(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_3aim_7storage_10prefixview_2__pyx_unpickle_PrefixViewItemsIterator(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_PickleError = 0;
  PyObject *__pyx_v___pyx_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_PrefixViewItemsIterator", 0);

  /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0xda33529:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError, "Incompatible checksums (%s vs 0xda33529 = (_current_value, it, path, prefix_len, prefix_view))" % __pyx_checksum
 */
  __pyx_t_1 = ((__pyx_v___pyx_checksum != 0xda33529) != 0);
  if (__pyx_t_1) {

    /* "(tree fragment)":5
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0xda33529:
 *         from pickle import PickleError as __pyx_PickleError             # <<<<<<<<<<<<<<
 *         raise __pyx_PickleError, "Incompatible checksums (%s vs 0xda33529 = (_current_value, it, path, prefix_len, prefix_view))" % __pyx_checksum
 *     __pyx_result = PrefixViewItemsIterator.__new__(__pyx_type)
 */
    __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_n_s_PickleError);
    __Pyx_GIVEREF(__pyx_n_s_PickleError);
    PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_PickleError);
    __pyx_t_3 = __Pyx_Import(__pyx_n_s_pickle, __pyx_t_2, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_3, __pyx_n_s_PickleError); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v___pyx_PickleError = __pyx_t_2;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":6
 *     if __pyx_checksum != 0xda33529:
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError, "Incompatible checksums (%s vs 0xda33529 = (_current_value, it, path, prefix_len, prefix_view))" % __pyx_checksum             # <<<<<<<<<<<<<<
 *     __pyx_result = PrefixViewItemsIterator.__new__(__pyx_type)
 *     if __pyx_state is not None:
 */
    __pyx_t_3 = __Pyx_PyInt_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_2 = __Pyx_PyString_Format(__pyx_kp_s_Incompatible_checksums_s_vs_0xda, __pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_Raise(__pyx_v___pyx_PickleError, __pyx_t_2, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(2, 6, __pyx_L1_error)

    /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0xda33529:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError, "Incompatible checksums (%s vs 0xda33529 = (_current_value, it, path, prefix_len, prefix_view))" % __pyx_checksum
 */
  }

  /* "(tree fragment)":7
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError, "Incompatible checksums (%s vs 0xda33529 = (_current_value, it, path, prefix_len, prefix_view))" % __pyx_checksum
 *     __pyx_result = PrefixViewItemsIterator.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_PrefixViewItemsIterator__set_state(<PrefixViewItemsIterator> __pyx_result, __pyx_state)
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_3aim_7storage_10prefixview_PrefixViewItemsIterator), __pyx_n_s_new); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
      __pyx_t_5 = 1;
    }
  }
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_v___pyx_type};
    __pyx_t_2 = __Pyx_PyObject_FastCall(__pyx_t_3, __pyx_callargs+1-__pyx_t_5, 1+__pyx_t_5);
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 7, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  }
  __pyx_v___pyx_result = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "(tree fragment)":8
 *         raise __pyx_PickleError, "Incompatible checksums (%s vs 0xda33529 = (_current_value, it, path, prefix_len, prefix_view))" % __pyx_checksum
 *     __pyx_result = PrefixViewItemsIterator.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_PrefixViewItemsIterator__set_state(<PrefixViewItemsIterator> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  __pyx_t_1 = (__pyx_v___pyx_state != Py_None);
  __pyx_t_6 = (__pyx_t_1 != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":9
 *     __pyx_result = PrefixViewItemsIterator.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_PrefixViewItemsIterator__set_state(<PrefixViewItemsIterator> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_PrefixViewItemsIterator__set_state(PrefixViewItemsIterator __pyx_result, tuple __pyx_state):
 */
    if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None) || __Pyx_RaiseUnexpectedTypeError("tuple", __pyx_v___pyx_state))) __PYX_ERR(2, 9, __pyx_L1_error)
    __pyx_t_2 = __pyx_f_3aim_7storage_10prefixview___pyx_unpickle_PrefixViewItemsIterator__set_state(((struct __pyx_obj_3aim_7storage_10prefixview_PrefixViewItemsIterator *)__pyx_v___pyx_result), ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 9, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "(tree fragment)":8
 *         raise __pyx_PickleError, "Incompatible checksums (%s vs 0xda33529 = (_current_value, it, path, prefix_len, prefix_view))" % __pyx_checksum
 *     __pyx_result = PrefixViewItemsIterator.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_PrefixViewItemsIterator__set_state(<PrefixViewItemsIterator> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  }

  /* "(tree fragment)":10
 *     if __pyx_state is not None:
 *         __pyx_unpickle_PrefixViewItemsIterator__set_state(<PrefixViewItemsIterator> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_PrefixViewItemsIterator__set_state(PrefixViewItemsIterator __pyx_result, tuple __pyx_state):
 *     __pyx_result._current_value = __pyx_state[0]; __pyx_result.it = __pyx_state[1]; __pyx_result.path = __pyx_state[2]; __pyx_result.prefix_len = __pyx_state[3]; __pyx_result.prefix_view = __pyx_state[4]
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_PrefixViewItemsIterator(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("aim.storage.prefixview.__pyx_unpickle_PrefixViewItemsIterator", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_PickleError);
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":11
 *         __pyx_unpickle_PrefixViewItemsIterator__set_state(<PrefixViewItemsIterator> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_PrefixViewItemsIterator__set_state(PrefixViewItemsIterator __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._current_value = __pyx_state[0]; __pyx_result.it = __pyx_state[1]; __pyx_result.path = __pyx_state[2]; __pyx_result.prefix_len = __pyx_state[3]; __pyx_result.prefix_view = __pyx_state[4]
 *     if len(__pyx_state) > 5 and hasattr(__pyx_result, '__dict__'):
 */

static PyObject *__pyx_f_3aim_7storage_10prefixview___pyx_unpickle_PrefixViewItemsIterator__set_state(struct __pyx_obj_3aim_7storage_10prefixview_PrefixViewItemsIterator *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  Py_ssize_t __pyx_t_4;
  int __pyx_t_5;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_PrefixViewItemsIterator__set_state", 0);

  /* "(tree fragment)":12
 *     return __pyx_result
 * cdef __pyx_unpickle_PrefixViewItemsIterator__set_state(PrefixViewItemsIterator __pyx_result, tuple __pyx_state):
 *     __pyx_result._current_value = __pyx_state[0]; __pyx_result.it = __pyx_state[1]; __pyx_result.path = __pyx_state[2]; __pyx_result.prefix_len = __pyx_state[3]; __pyx_result.prefix_view = __pyx_state[4]             # <<<<<<<<<<<<<<
 *     if len(__pyx_state) > 5 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[5])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base._current_value);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.__pyx_base._current_value);
  __pyx_v___pyx_result->__pyx_base.__pyx_base._current_value = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_3aim_7storage_9container_ContainerItemsIterator))))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF((PyObject *)__pyx_v___pyx_result->it);
  __Pyx_DECREF((PyObject *)__pyx_v___pyx_result->it);
  __pyx_v___pyx_result->it = ((struct __pyx_obj_3aim_7storage_9container_ContainerItemsIterator *)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyBytes_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None) || __Pyx_RaiseUnexpectedTypeError("bytes", __pyx_t_1))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->path);
  __Pyx_DECREF(__pyx_v___pyx_result->path);
  __pyx_v___pyx_result->path = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 3, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->prefix_len = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 4, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_3aim_7storage_10prefixview_PrefixView))))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF((PyObject *)__pyx_v___pyx_result->prefix_view);
  __Pyx_DECREF((PyObject *)__pyx_v___pyx_result->prefix_view);
  __pyx_v___pyx_result->prefix_view = ((struct __pyx_obj_3aim_7storage_10prefixview_PrefixView *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "(tree fragment)":13
 * cdef __pyx_unpickle_PrefixViewItemsIterator__set_state(PrefixViewItemsIterator __pyx_result, tuple __pyx_state):
 *     __pyx_result._current_value = __pyx_state[0]; __pyx_result.it = __pyx_state[1]; __pyx_result.path = __pyx_state[2]; __pyx_result.prefix_len = __pyx_state[3]; __pyx_result.prefix_view = __pyx_state[4]
 *     if len(__pyx_state) > 5 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[5])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(2, 13, __pyx_L1_error)
  }
  __pyx_t_4 = PyTuple_GET_SIZE(__pyx_v___pyx_state); if (unlikely(__pyx_t_4 == ((Py_ssize_t)-1))) __PYX_ERR(2, 13, __pyx_L1_error)
  __pyx_t_5 = ((__pyx_t_4 > 5) != 0);
  if (__pyx_t_5) {
  } else {
    __pyx_t_3 = __pyx_t_5;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_5 = __Pyx_HasAttr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(__pyx_t_5 == ((int)-1))) __PYX_ERR(2, 13, __pyx_L1_error)
  __pyx_t_6 = (__pyx_t_5 != 0);
  __pyx_t_3 = __pyx_t_6;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_3) {

    /* "(tree fragment)":14
 *     __pyx_result._current_value = __pyx_state[0]; __pyx_result.it = __pyx_state[1]; __pyx_result.path = __pyx_state[2]; __pyx_result.prefix_len = __pyx_state[3]; __pyx_result.prefix_view = __pyx_state[4]
 *     if len(__pyx_state) > 5 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[5])             # <<<<<<<<<<<<<<
 */
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(!__pyx_t_7)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_n_s_update); if (unlikely(!__pyx_t_8)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(2, 14, __pyx_L1_error)
    }
    __pyx_t_7 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 5, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_7)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_9 = NULL;
    __pyx_t_2 = 0;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_8))) {
      __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_8);
      if (likely(__pyx_t_9)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_8);
        __Pyx_INCREF(__pyx_t_9);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_8, function);
        __pyx_t_2 = 1;
      }
    }
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_9, __pyx_t_7};
      __pyx_t_1 = __Pyx_PyObject_FastCall(__pyx_t_8, __pyx_callargs+1-__pyx_t_2, 1+__pyx_t_2);
      __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 14, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "(tree fragment)":13
 * cdef __pyx_unpickle_PrefixViewItemsIterator__set_state(PrefixViewItemsIterator __pyx_result, tuple __pyx_state):
 *     __pyx_result._current_value = __pyx_state[0]; __pyx_result.it = __pyx_state[1]; __pyx_result.path = __pyx_state[2]; __pyx_result.prefix_len = __pyx_state[3]; __pyx_result.prefix_view = __pyx_state[4]
 *     if len(__pyx_state) > 5 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[5])
 */
  }

  /* "(tree fragment)":11
 *         __pyx_unpickle_PrefixViewItemsIterator__set_state(<PrefixViewItemsIterator> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_PrefixViewItemsIterator__set_state(PrefixViewItemsIterator __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._current_value = __pyx_state[0]; __pyx_result.it = __pyx_state[1]; __pyx_result.path = __pyx_state[2]; __pyx_result.prefix_len = __pyx_state[3]; __pyx_result.prefix_view = __pyx_state[4]
 *     if len(__pyx_state) > 5 and hasattr(__pyx_result, '__dict__'):
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("aim.storage.prefixview.__pyx_unpickle_PrefixViewItemsIterator__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cpython/complex.pxd":19
 * 
 *         @property
 *         cdef inline double real(self):             # <<<<<<<<<<<<<<
 *             return self.cval.real
 * 
 */

static CYTHON_INLINE double __pyx_f_7cpython_7complex_7complex_4real_real(PyComplexObject *__pyx_v_self) {
  double __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("real", 0);

  /* "cpython/complex.pxd":20
 *         @property
 *         cdef inline double real(self):
 *             return self.cval.real             # <<<<<<<<<<<<<<
 * 
 *         @property
 */
  __pyx_r = __pyx_v_self->cval.real;
  goto __pyx_L0;

  /* "cpython/complex.pxd":19
 * 
 *         @property
 *         cdef inline double real(self):             # <<<<<<<<<<<<<<
 *             return self.cval.real
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cpython/complex.pxd":23
 * 
 *         @property
 *         cdef inline double imag(self):             # <<<<<<<<<<<<<<
 *             return self.cval.imag
 * 
 */

static CYTHON_INLINE double __pyx_f_7cpython_7complex_7complex_4imag_imag(PyComplexObject *__pyx_v_self) {
  double __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("imag", 0);

  /* "cpython/complex.pxd":24
 *         @property
 *         cdef inline double imag(self):
 *             return self.cval.imag             # <<<<<<<<<<<<<<
 * 
 *     # PyTypeObject PyComplex_Type
 */
  __pyx_r = __pyx_v_self->cval.imag;
  goto __pyx_L0;

  /* "cpython/complex.pxd":23
 * 
 *         @property
 *         cdef inline double imag(self):             # <<<<<<<<<<<<<<
 *             return self.cval.imag
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cpython/contextvars.pxd":112
 * 
 * 
 * cdef inline object get_value(var, default_value=None):             # <<<<<<<<<<<<<<
 *     """Return a new reference to the value of the context variable,
 *     or the default value of the context variable,
 */

static CYTHON_INLINE PyObject *__pyx_f_7cpython_11contextvars_get_value(PyObject *__pyx_v_var, struct __pyx_opt_args_7cpython_11contextvars_get_value *__pyx_optional_args) {
  PyObject *__pyx_v_default_value = ((PyObject *)Py_None);
  PyObject *__pyx_v_value;
  PyObject *__pyx_v_pyvalue = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_value", 0);
  if (__pyx_optional_args) {
    if (__pyx_optional_args->__pyx_n > 0) {
      __pyx_v_default_value = __pyx_optional_args->default_value;
    }
  }

  /* "cpython/contextvars.pxd":118
 *     """
 *     cdef PyObject *value
 *     PyContextVar_Get(var, NULL, &value)             # <<<<<<<<<<<<<<
 *     if value is NULL:
 *         # context variable does not have a default
 */
  __pyx_t_1 = PyContextVar_Get(__pyx_v_var, NULL, (&__pyx_v_value)); if (unlikely(__pyx_t_1 == ((int)-1))) __PYX_ERR(3, 118, __pyx_L1_error)

  /* "cpython/contextvars.pxd":119
 *     cdef PyObject *value
 *     PyContextVar_Get(var, NULL, &value)
 *     if value is NULL:             # <<<<<<<<<<<<<<
 *         # context variable does not have a default
 *         pyvalue = default_value
 */
  __pyx_t_2 = ((__pyx_v_value == NULL) != 0);
  if (__pyx_t_2) {

    /* "cpython/contextvars.pxd":121
 *     if value is NULL:
 *         # context variable does not have a default
 *         pyvalue = default_value             # <<<<<<<<<<<<<<
 *     else:
 *         # value or default value of context variable
 */
    __Pyx_INCREF(__pyx_v_default_value);
    __pyx_v_pyvalue = __pyx_v_default_value;

    /* "cpython/contextvars.pxd":119
 *     cdef PyObject *value
 *     PyContextVar_Get(var, NULL, &value)
 *     if value is NULL:             # <<<<<<<<<<<<<<
 *         # context variable does not have a default
 *         pyvalue = default_value
 */
    goto __pyx_L3;
  }

  /* "cpython/contextvars.pxd":124
 *     else:
 *         # value or default value of context variable
 *         pyvalue = <object>value             # <<<<<<<<<<<<<<
 *         Py_XDECREF(value)  # PyContextVar_Get() returned an owned reference as 'PyObject*'
 *     return pyvalue
 */
  /*else*/ {
    __pyx_t_3 = ((PyObject *)__pyx_v_value);
    __Pyx_INCREF(__pyx_t_3);
    __pyx_v_pyvalue = __pyx_t_3;
    __pyx_t_3 = 0;

    /* "cpython/contextvars.pxd":125
 *         # value or default value of context variable
 *         pyvalue = <object>value
 *         Py_XDECREF(value)  # PyContextVar_Get() returned an owned reference as 'PyObject*'             # <<<<<<<<<<<<<<
 *     return pyvalue
 * 
 */
    Py_XDECREF(__pyx_v_value);
  }
  __pyx_L3:;

  /* "cpython/contextvars.pxd":126
 *         pyvalue = <object>value
 *         Py_XDECREF(value)  # PyContextVar_Get() returned an owned reference as 'PyObject*'
 *     return pyvalue             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_pyvalue);
  __pyx_r = __pyx_v_pyvalue;
  goto __pyx_L0;

  /* "cpython/contextvars.pxd":112
 * 
 * 
 * cdef inline object get_value(var, default_value=None):             # <<<<<<<<<<<<<<
 *     """Return a new reference to the value of the context variable,
 *     or the default value of the context variable,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("cpython.contextvars.get_value", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_pyvalue);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cpython/contextvars.pxd":129
 * 
 * 
 * cdef inline object get_value_no_default(var, default_value=None):             # <<<<<<<<<<<<<<
 *     """Return a new reference to the value of the context variable,
 *     or the provided default value if no such value was found.
 */

static CYTHON_INLINE PyObject *__pyx_f_7cpython_11contextvars_get_value_no_default(PyObject *__pyx_v_var, struct __pyx_opt_args_7cpython_11contextvars_get_value_no_default *__pyx_optional_args) {
  PyObject *__pyx_v_default_value = ((PyObject *)Py_None);
  PyObject *__pyx_v_value;
  PyObject *__pyx_v_pyvalue = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_value_no_default", 0);
  if (__pyx_optional_args) {
    if (__pyx_optional_args->__pyx_n > 0) {
      __pyx_v_default_value = __pyx_optional_args->default_value;
    }
  }

  /* "cpython/contextvars.pxd":136
 *     """
 *     cdef PyObject *value
 *     PyContextVar_Get(var, <PyObject*>default_value, &value)             # <<<<<<<<<<<<<<
 *     # value of context variable or 'default_value'
 *     pyvalue = <object>value
 */
  __pyx_t_1 = PyContextVar_Get(__pyx_v_var, ((PyObject *)__pyx_v_default_value), (&__pyx_v_value)); if (unlikely(__pyx_t_1 == ((int)-1))) __PYX_ERR(3, 136, __pyx_L1_error)

  /* "cpython/contextvars.pxd":138
 *     PyContextVar_Get(var, <PyObject*>default_value, &value)
 *     # value of context variable or 'default_value'
 *     pyvalue = <object>value             # <<<<<<<<<<<<<<
 *     Py_XDECREF(value)  # PyContextVar_Get() returned an owned reference as 'PyObject*'
 *     return pyvalue
 */
  __pyx_t_2 = ((PyObject *)__pyx_v_value);
  __Pyx_INCREF(__pyx_t_2);
  __pyx_v_pyvalue = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "cpython/contextvars.pxd":139
 *     # value of context variable or 'default_value'
 *     pyvalue = <object>value
 *     Py_XDECREF(value)  # PyContextVar_Get() returned an owned reference as 'PyObject*'             # <<<<<<<<<<<<<<
 *     return pyvalue
 */
  Py_XDECREF(__pyx_v_value);

  /* "cpython/contextvars.pxd":140
 *     pyvalue = <object>value
 *     Py_XDECREF(value)  # PyContextVar_Get() returned an owned reference as 'PyObject*'
 *     return pyvalue             # <<<<<<<<<<<<<<
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_pyvalue);
  __pyx_r = __pyx_v_pyvalue;
  goto __pyx_L0;

  /* "cpython/contextvars.pxd":129
 * 
 * 
 * cdef inline object get_value_no_default(var, default_value=None):             # <<<<<<<<<<<<<<
 *     """Return a new reference to the value of the context variable,
 *     or the provided default value if no such value was found.
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("cpython.contextvars.get_value_no_default", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_pyvalue);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}
static struct __pyx_vtabstruct_3aim_7storage_10prefixview_PrefixView __pyx_vtable_3aim_7storage_10prefixview_PrefixView;

static PyObject *__pyx_tp_new_3aim_7storage_10prefixview_PrefixView(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_3aim_7storage_10prefixview_PrefixView *p;
  #if CYTHON_COMPILING_IN_LIMITED_API
  newfunc new_func = (newfunc)PyType_GetSlot(__pyx_ptype_3aim_7storage_9container_Container, Py_tp_new);
  PyObject *o = new_func(t, a, k);
  #else
  PyObject *o = __pyx_ptype_3aim_7storage_9container_Container->tp_new(t, a, k);
  #endif
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_3aim_7storage_10prefixview_PrefixView *)o);
  p->__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_3aim_7storage_9container_Container*)__pyx_vtabptr_3aim_7storage_10prefixview_PrefixView;
  p->prefix = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->parent = Py_None; Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_3aim_7storage_10prefixview_PrefixView(PyObject *o) {
  struct __pyx_obj_3aim_7storage_10prefixview_PrefixView *p = (struct __pyx_obj_3aim_7storage_10prefixview_PrefixView *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely((PY_VERSION_HEX >= 0x03080000 || __Pyx_PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE)) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->prefix);
  Py_CLEAR(p->parent);
  #if CYTHON_USE_TYPE_SLOTS
  if (PyType_IS_GC(Py_TYPE(o)->tp_base))
  #endif
  PyObject_GC_Track(o);
  if (likely(__pyx_ptype_3aim_7storage_9container_Container)) __pyx_ptype_3aim_7storage_9container_Container->tp_dealloc(o); else __Pyx_call_next_tp_dealloc(o, __pyx_tp_dealloc_3aim_7storage_10prefixview_PrefixView);
}

static int __pyx_tp_traverse_3aim_7storage_10prefixview_PrefixView(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_3aim_7storage_10prefixview_PrefixView *p = (struct __pyx_obj_3aim_7storage_10prefixview_PrefixView *)o;
  e = ((likely(__pyx_ptype_3aim_7storage_9container_Container)) ? ((__pyx_ptype_3aim_7storage_9container_Container->tp_traverse) ? __pyx_ptype_3aim_7storage_9container_Container->tp_traverse(o, v, a) : 0) : __Pyx_call_next_tp_traverse(o, v, a, __pyx_tp_traverse_3aim_7storage_10prefixview_PrefixView)); if (e) return e;
  if (p->parent) {
    e = (*v)(p->parent, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_3aim_7storage_10prefixview_PrefixView(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_3aim_7storage_10prefixview_PrefixView *p = (struct __pyx_obj_3aim_7storage_10prefixview_PrefixView *)o;
  if (likely(__pyx_ptype_3aim_7storage_9container_Container)) { if (__pyx_ptype_3aim_7storage_9container_Container->tp_clear) __pyx_ptype_3aim_7storage_9container_Container->tp_clear(o); } else __Pyx_call_next_tp_clear(o, __pyx_tp_clear_3aim_7storage_10prefixview_PrefixView);
  tmp = ((PyObject*)p->parent);
  p->parent = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}
static PyObject *__pyx_sq_item_3aim_7storage_10prefixview_PrefixView(PyObject *o, Py_ssize_t i) {
  PyObject *r;
  PyObject *x = PyInt_FromSsize_t(i); if(!x) return 0;
  r = Py_TYPE(o)->tp_as_mapping->mp_subscript(o, x);
  Py_DECREF(x);
  return r;
}

static int __pyx_mp_ass_subscript_3aim_7storage_10prefixview_PrefixView(PyObject *o, PyObject *i, PyObject *v) {
  if (v) {
    return __pyx_pw_3aim_7storage_10prefixview_10PrefixView_17__setitem__(o, i, v);
  }
  else {
    return __pyx_pw_3aim_7storage_10prefixview_10PrefixView_21__delitem__(o, i);
  }
}

static PyObject *__pyx_getprop_3aim_7storage_10prefixview_10PrefixView_prefix(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_3aim_7storage_10prefixview_10PrefixView_6prefix_1__get__(o);
}

static int __pyx_setprop_3aim_7storage_10prefixview_10PrefixView_prefix(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_3aim_7storage_10prefixview_10PrefixView_6prefix_3__set__(o, v);
  }
  else {
    return __pyx_pw_3aim_7storage_10prefixview_10PrefixView_6prefix_5__del__(o);
  }
}

static PyObject *__pyx_getprop_3aim_7storage_10prefixview_10PrefixView_parent(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_3aim_7storage_10prefixview_10PrefixView_6parent_1__get__(o);
}

static int __pyx_setprop_3aim_7storage_10prefixview_10PrefixView_parent(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_3aim_7storage_10prefixview_10PrefixView_6parent_3__set__(o, v);
  }
  else {
    return __pyx_pw_3aim_7storage_10prefixview_10PrefixView_6parent_5__del__(o);
  }
}

static PyObject *__pyx_getprop_3aim_7storage_10prefixview_10PrefixView_read_only(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_3aim_7storage_10prefixview_10PrefixView_9read_only_1__get__(o);
}

static int __pyx_setprop_3aim_7storage_10prefixview_10PrefixView_read_only(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_3aim_7storage_10prefixview_10PrefixView_9read_only_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyMethodDef __pyx_methods_3aim_7storage_10prefixview_PrefixView[] = {
  {"finalize", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_3aim_7storage_10prefixview_10PrefixView_7finalize, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_3aim_7storage_10prefixview_10PrefixView_6finalize},
  {"delete", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_3aim_7storage_10prefixview_10PrefixView_19delete, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_3aim_7storage_10prefixview_10PrefixView_18delete},
  {"walk", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_3aim_7storage_10prefixview_10PrefixView_29walk, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_3aim_7storage_10prefixview_10PrefixView_28walk},
  {"view", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_3aim_7storage_10prefixview_10PrefixView_34view, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_3aim_7storage_10prefixview_10PrefixView_33view},
  {"batch", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_3aim_7storage_10prefixview_10PrefixView_38batch, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_3aim_7storage_10prefixview_10PrefixView_37batch},
  {"commit", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_3aim_7storage_10prefixview_10PrefixView_40commit, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_3aim_7storage_10prefixview_10PrefixView_39commit},
  {"__reduce_cython__", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_3aim_7storage_10prefixview_10PrefixView_42__reduce_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0},
  {"__setstate_cython__", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_3aim_7storage_10prefixview_10PrefixView_44__setstate_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_3aim_7storage_10prefixview_PrefixView[] = {
  {(char *)"prefix", __pyx_getprop_3aim_7storage_10prefixview_10PrefixView_prefix, __pyx_setprop_3aim_7storage_10prefixview_10PrefixView_prefix, (char *)0, 0},
  {(char *)"parent", __pyx_getprop_3aim_7storage_10prefixview_10PrefixView_parent, __pyx_setprop_3aim_7storage_10prefixview_10PrefixView_parent, (char *)0, 0},
  {(char *)"read_only", __pyx_getprop_3aim_7storage_10prefixview_10PrefixView_read_only, __pyx_setprop_3aim_7storage_10prefixview_10PrefixView_read_only, (char *)0, 0},
  {0, 0, 0, 0, 0}
};
#if CYTHON_USE_TYPE_SPECS
static PyType_Slot __pyx_type_3aim_7storage_10prefixview_PrefixView_slots[] = {
  {Py_tp_dealloc, (void *)__pyx_tp_dealloc_3aim_7storage_10prefixview_PrefixView},
  {Py_sq_item, (void *)__pyx_sq_item_3aim_7storage_10prefixview_PrefixView},
  {Py_mp_subscript, (void *)__pyx_pw_3aim_7storage_10prefixview_10PrefixView_13__getitem__},
  {Py_mp_ass_subscript, (void *)__pyx_mp_ass_subscript_3aim_7storage_10prefixview_PrefixView},
  {Py_tp_doc, (void *)PyDoc_STR("\n    A mutable view to a :obj:`Container` given by a key prefix.\n\n    Args:\n        prefix (:obj:`bytes`): the prefix that defines the key range of the\n            view-container. The resulting container will share an access to\n            only records in the `prefix` key range, but with `prefix`-es\n            stripped from them.\n\n            For example, if the Container contents are:\n            `{\n                b'e.y': b'012',\n                b'meta.x': b'123',\n                b'meta.z': b'x',\n                b'zzz': b'oOo'\n            }`, then `container.view(prefix=b'meta.')` will behave (almost)\n            exactly as an Container:\n            `{\n                b'x': b'123',\n                b'z': b'x',\n            }`\n        container (:obj:`Container`): the parent container to build the view on\n    ")},
  {Py_tp_traverse, (void *)__pyx_tp_traverse_3aim_7storage_10prefixview_PrefixView},
  {Py_tp_clear, (void *)__pyx_tp_clear_3aim_7storage_10prefixview_PrefixView},
  {Py_tp_methods, (void *)__pyx_methods_3aim_7storage_10prefixview_PrefixView},
  {Py_tp_getset, (void *)__pyx_getsets_3aim_7storage_10prefixview_PrefixView},
  {Py_tp_init, (void *)__pyx_pw_3aim_7storage_10prefixview_10PrefixView_1__init__},
  {Py_tp_new, (void *)__pyx_tp_new_3aim_7storage_10prefixview_PrefixView},
  {0, 0},
};
static PyType_Spec __pyx_type_3aim_7storage_10prefixview_PrefixView_spec = {
  "aim.storage.prefixview.PrefixView",
  sizeof(struct __pyx_obj_3aim_7storage_10prefixview_PrefixView),
  0,
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC,
  __pyx_type_3aim_7storage_10prefixview_PrefixView_slots,
};
#else

static PySequenceMethods __pyx_tp_as_sequence_PrefixView = {
  0, /*sq_length*/
  0, /*sq_concat*/
  0, /*sq_repeat*/
  __pyx_sq_item_3aim_7storage_10prefixview_PrefixView, /*sq_item*/
  0, /*sq_slice*/
  0, /*sq_ass_item*/
  0, /*sq_ass_slice*/
  0, /*sq_contains*/
  0, /*sq_inplace_concat*/
  0, /*sq_inplace_repeat*/
};

static PyMappingMethods __pyx_tp_as_mapping_PrefixView = {
  0, /*mp_length*/
  __pyx_pw_3aim_7storage_10prefixview_10PrefixView_13__getitem__, /*mp_subscript*/
  __pyx_mp_ass_subscript_3aim_7storage_10prefixview_PrefixView, /*mp_ass_subscript*/
};

static PyTypeObject __pyx_type_3aim_7storage_10prefixview_PrefixView = {
  PyVarObject_HEAD_INIT(0, 0)
  "aim.storage.prefixview.""PrefixView", /*tp_name*/
  sizeof(struct __pyx_obj_3aim_7storage_10prefixview_PrefixView), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_3aim_7storage_10prefixview_PrefixView, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  &__pyx_tp_as_sequence_PrefixView, /*tp_as_sequence*/
  &__pyx_tp_as_mapping_PrefixView, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  PyDoc_STR("\n    A mutable view to a :obj:`Container` given by a key prefix.\n\n    Args:\n        prefix (:obj:`bytes`): the prefix that defines the key range of the\n            view-container. The resulting container will share an access to\n            only records in the `prefix` key range, but with `prefix`-es\n            stripped from them.\n\n            For example, if the Container contents are:\n            `{\n                b'e.y': b'012',\n                b'meta.x': b'123',\n                b'meta.z': b'x',\n                b'zzz': b'oOo'\n            }`, then `container.view(prefix=b'meta.')` will behave (almost)\n            exactly as an Container:\n            `{\n                b'x': b'123',\n                b'z': b'x',\n            }`\n        container (:obj:`Container`): the parent container to build the view on\n    "), /*tp_doc*/
  __pyx_tp_traverse_3aim_7storage_10prefixview_PrefixView, /*tp_traverse*/
  __pyx_tp_clear_3aim_7storage_10prefixview_PrefixView, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_3aim_7storage_10prefixview_PrefixView, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_3aim_7storage_10prefixview_PrefixView, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  #if !CYTHON_USE_TYPE_SPECS
  0, /*tp_dictoffset*/
  #endif
  __pyx_pw_3aim_7storage_10prefixview_10PrefixView_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_3aim_7storage_10prefixview_PrefixView, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PYPY_VERSION_NUM+0 >= 0x06000000
  0, /*tp_pypy_flags*/
  #endif
};
#endif
static struct __pyx_vtabstruct_3aim_7storage_10prefixview_PrefixViewItemsIterator __pyx_vtable_3aim_7storage_10prefixview_PrefixViewItemsIterator;

static PyObject *__pyx_tp_new_3aim_7storage_10prefixview_PrefixViewItemsIterator(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_3aim_7storage_10prefixview_PrefixViewItemsIterator *p;
  #if CYTHON_COMPILING_IN_LIMITED_API
  newfunc new_func = (newfunc)PyType_GetSlot(__pyx_ptype_3aim_7storage_9container_ContainerItemsIterator, Py_tp_new);
  PyObject *o = new_func(t, a, k);
  #else
  PyObject *o = __pyx_ptype_3aim_7storage_9container_ContainerItemsIterator->tp_new(t, a, k);
  #endif
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_3aim_7storage_10prefixview_PrefixViewItemsIterator *)o);
  p->__pyx_base.__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_8aimrocks_11lib_rocksdb_Iterator*)__pyx_vtabptr_3aim_7storage_10prefixview_PrefixViewItemsIterator;
  p->prefix_view = ((struct __pyx_obj_3aim_7storage_10prefixview_PrefixView *)Py_None); Py_INCREF(Py_None);
  p->path = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->it = ((struct __pyx_obj_3aim_7storage_9container_ContainerItemsIterator *)Py_None); Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_3aim_7storage_10prefixview_PrefixViewItemsIterator(PyObject *o) {
  struct __pyx_obj_3aim_7storage_10prefixview_PrefixViewItemsIterator *p = (struct __pyx_obj_3aim_7storage_10prefixview_PrefixViewItemsIterator *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely((PY_VERSION_HEX >= 0x03080000 || __Pyx_PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE)) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->prefix_view);
  Py_CLEAR(p->path);
  Py_CLEAR(p->it);
  PyObject_GC_Track(o);
  if (likely(__pyx_ptype_3aim_7storage_9container_ContainerItemsIterator)) __pyx_ptype_3aim_7storage_9container_ContainerItemsIterator->tp_dealloc(o); else __Pyx_call_next_tp_dealloc(o, __pyx_tp_dealloc_3aim_7storage_10prefixview_PrefixViewItemsIterator);
}

static int __pyx_tp_traverse_3aim_7storage_10prefixview_PrefixViewItemsIterator(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_3aim_7storage_10prefixview_PrefixViewItemsIterator *p = (struct __pyx_obj_3aim_7storage_10prefixview_PrefixViewItemsIterator *)o;
  e = ((likely(__pyx_ptype_3aim_7storage_9container_ContainerItemsIterator)) ? ((__pyx_ptype_3aim_7storage_9container_ContainerItemsIterator->tp_traverse) ? __pyx_ptype_3aim_7storage_9container_ContainerItemsIterator->tp_traverse(o, v, a) : 0) : __Pyx_call_next_tp_traverse(o, v, a, __pyx_tp_traverse_3aim_7storage_10prefixview_PrefixViewItemsIterator)); if (e) return e;
  if (p->prefix_view) {
    e = (*v)(((PyObject *)p->prefix_view), a); if (e) return e;
  }
  if (p->it) {
    e = (*v)(((PyObject *)p->it), a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_3aim_7storage_10prefixview_PrefixViewItemsIterator(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_3aim_7storage_10prefixview_PrefixViewItemsIterator *p = (struct __pyx_obj_3aim_7storage_10prefixview_PrefixViewItemsIterator *)o;
  if (likely(__pyx_ptype_3aim_7storage_9container_ContainerItemsIterator)) { if (__pyx_ptype_3aim_7storage_9container_ContainerItemsIterator->tp_clear) __pyx_ptype_3aim_7storage_9container_ContainerItemsIterator->tp_clear(o); } else __Pyx_call_next_tp_clear(o, __pyx_tp_clear_3aim_7storage_10prefixview_PrefixViewItemsIterator);
  tmp = ((PyObject*)p->prefix_view);
  p->prefix_view = ((struct __pyx_obj_3aim_7storage_10prefixview_PrefixView *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->it);
  p->it = ((struct __pyx_obj_3aim_7storage_9container_ContainerItemsIterator *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyMethodDef __pyx_methods_3aim_7storage_10prefixview_PrefixViewItemsIterator[] = {
  {"__reduce_cython__", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_3aim_7storage_10prefixview_23PrefixViewItemsIterator_5__reduce_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0},
  {"__setstate_cython__", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_3aim_7storage_10prefixview_23PrefixViewItemsIterator_7__setstate_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0},
  {0, 0, 0, 0}
};
#if CYTHON_USE_TYPE_SPECS
static PyType_Slot __pyx_type_3aim_7storage_10prefixview_PrefixViewItemsIterator_slots[] = {
  {Py_tp_dealloc, (void *)__pyx_tp_dealloc_3aim_7storage_10prefixview_PrefixViewItemsIterator},
  {Py_tp_traverse, (void *)__pyx_tp_traverse_3aim_7storage_10prefixview_PrefixViewItemsIterator},
  {Py_tp_clear, (void *)__pyx_tp_clear_3aim_7storage_10prefixview_PrefixViewItemsIterator},
  {Py_tp_methods, (void *)__pyx_methods_3aim_7storage_10prefixview_PrefixViewItemsIterator},
  {Py_tp_init, (void *)__pyx_pw_3aim_7storage_10prefixview_23PrefixViewItemsIterator_1__init__},
  {Py_tp_new, (void *)__pyx_tp_new_3aim_7storage_10prefixview_PrefixViewItemsIterator},
  {0, 0},
};
static PyType_Spec __pyx_type_3aim_7storage_10prefixview_PrefixViewItemsIterator_spec = {
  "aim.storage.prefixview.PrefixViewItemsIterator",
  sizeof(struct __pyx_obj_3aim_7storage_10prefixview_PrefixViewItemsIterator),
  0,
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC,
  __pyx_type_3aim_7storage_10prefixview_PrefixViewItemsIterator_slots,
};
#else

static PyTypeObject __pyx_type_3aim_7storage_10prefixview_PrefixViewItemsIterator = {
  PyVarObject_HEAD_INIT(0, 0)
  "aim.storage.prefixview.""PrefixViewItemsIterator", /*tp_name*/
  sizeof(struct __pyx_obj_3aim_7storage_10prefixview_PrefixViewItemsIterator), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_3aim_7storage_10prefixview_PrefixViewItemsIterator, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_3aim_7storage_10prefixview_PrefixViewItemsIterator, /*tp_traverse*/
  __pyx_tp_clear_3aim_7storage_10prefixview_PrefixViewItemsIterator, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_3aim_7storage_10prefixview_PrefixViewItemsIterator, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  #if !CYTHON_USE_TYPE_SPECS
  0, /*tp_dictoffset*/
  #endif
  __pyx_pw_3aim_7storage_10prefixview_23PrefixViewItemsIterator_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_3aim_7storage_10prefixview_PrefixViewItemsIterator, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PYPY_VERSION_NUM+0 >= 0x06000000
  0, /*tp_pypy_flags*/
  #endif
};
#endif

static struct __pyx_obj_3aim_7storage_10prefixview___pyx_scope_struct__walk *__pyx_freelist_3aim_7storage_10prefixview___pyx_scope_struct__walk[8];
static int __pyx_freecount_3aim_7storage_10prefixview___pyx_scope_struct__walk = 0;

static PyObject *__pyx_tp_new_3aim_7storage_10prefixview___pyx_scope_struct__walk(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  PyObject *o;
  #if CYTHON_COMPILING_IN_LIMITED_API
  allocfunc alloc_func = (allocfunc)PyType_GetSlot(t, Py_tp_alloc);
  o = alloc_func(t, 0);
  #else
  if (CYTHON_COMPILING_IN_CPYTHON && likely((__pyx_freecount_3aim_7storage_10prefixview___pyx_scope_struct__walk > 0) & (t->tp_basicsize == sizeof(struct __pyx_obj_3aim_7storage_10prefixview___pyx_scope_struct__walk)))) {
    o = (PyObject*)__pyx_freelist_3aim_7storage_10prefixview___pyx_scope_struct__walk[--__pyx_freecount_3aim_7storage_10prefixview___pyx_scope_struct__walk];
    memset(o, 0, sizeof(struct __pyx_obj_3aim_7storage_10prefixview___pyx_scope_struct__walk));
    (void) PyObject_INIT(o, t);
    PyObject_GC_Track(o);
  } else {
    o = (*t->tp_alloc)(t, 0);
    if (unlikely(!o)) return 0;
  }
  #endif
  return o;
}

static void __pyx_tp_dealloc_3aim_7storage_10prefixview___pyx_scope_struct__walk(PyObject *o) {
  struct __pyx_obj_3aim_7storage_10prefixview___pyx_scope_struct__walk *p = (struct __pyx_obj_3aim_7storage_10prefixview___pyx_scope_struct__walk *)o;
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->__pyx_v__path);
  Py_CLEAR(p->__pyx_v__prefix);
  Py_CLEAR(p->__pyx_v_key);
  Py_CLEAR(p->__pyx_v_next_key);
  Py_CLEAR(p->__pyx_v_p);
  Py_CLEAR(p->__pyx_v_path);
  Py_CLEAR(p->__pyx_v_self);
  Py_CLEAR(p->__pyx_v_walker);
  if (CYTHON_COMPILING_IN_CPYTHON && ((__pyx_freecount_3aim_7storage_10prefixview___pyx_scope_struct__walk < 8) & (Py_TYPE(o)->tp_basicsize == sizeof(struct __pyx_obj_3aim_7storage_10prefixview___pyx_scope_struct__walk)))) {
    __pyx_freelist_3aim_7storage_10prefixview___pyx_scope_struct__walk[__pyx_freecount_3aim_7storage_10prefixview___pyx_scope_struct__walk++] = ((struct __pyx_obj_3aim_7storage_10prefixview___pyx_scope_struct__walk *)o);
  } else {
    (*Py_TYPE(o)->tp_free)(o);
  }
}

static int __pyx_tp_traverse_3aim_7storage_10prefixview___pyx_scope_struct__walk(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_3aim_7storage_10prefixview___pyx_scope_struct__walk *p = (struct __pyx_obj_3aim_7storage_10prefixview___pyx_scope_struct__walk *)o;
  if (p->__pyx_v__path) {
    e = (*v)(p->__pyx_v__path, a); if (e) return e;
  }
  if (p->__pyx_v__prefix) {
    e = (*v)(p->__pyx_v__prefix, a); if (e) return e;
  }
  if (p->__pyx_v_key) {
    e = (*v)(p->__pyx_v_key, a); if (e) return e;
  }
  if (p->__pyx_v_next_key) {
    e = (*v)(p->__pyx_v_next_key, a); if (e) return e;
  }
  if (p->__pyx_v_self) {
    e = (*v)(((PyObject *)p->__pyx_v_self), a); if (e) return e;
  }
  if (p->__pyx_v_walker) {
    e = (*v)(p->__pyx_v_walker, a); if (e) return e;
  }
  return 0;
}
#if CYTHON_USE_TYPE_SPECS
static PyType_Slot __pyx_type_3aim_7storage_10prefixview___pyx_scope_struct__walk_slots[] = {
  {Py_tp_dealloc, (void *)__pyx_tp_dealloc_3aim_7storage_10prefixview___pyx_scope_struct__walk},
  {Py_tp_traverse, (void *)__pyx_tp_traverse_3aim_7storage_10prefixview___pyx_scope_struct__walk},
  {Py_tp_new, (void *)__pyx_tp_new_3aim_7storage_10prefixview___pyx_scope_struct__walk},
  {0, 0},
};
static PyType_Spec __pyx_type_3aim_7storage_10prefixview___pyx_scope_struct__walk_spec = {
  "aim.storage.prefixview.__pyx_scope_struct__walk",
  sizeof(struct __pyx_obj_3aim_7storage_10prefixview___pyx_scope_struct__walk),
  0,
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC,
  __pyx_type_3aim_7storage_10prefixview___pyx_scope_struct__walk_slots,
};
#else

static PyTypeObject __pyx_type_3aim_7storage_10prefixview___pyx_scope_struct__walk = {
  PyVarObject_HEAD_INIT(0, 0)
  "aim.storage.prefixview.""__pyx_scope_struct__walk", /*tp_name*/
  sizeof(struct __pyx_obj_3aim_7storage_10prefixview___pyx_scope_struct__walk), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_3aim_7storage_10prefixview___pyx_scope_struct__walk, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_3aim_7storage_10prefixview___pyx_scope_struct__walk, /*tp_traverse*/
  0, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  0, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  #if !CYTHON_USE_TYPE_SPECS
  0, /*tp_dictoffset*/
  #endif
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_3aim_7storage_10prefixview___pyx_scope_struct__walk, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PYPY_VERSION_NUM+0 >= 0x06000000
  0, /*tp_pypy_flags*/
  #endif
};
#endif

static PyMethodDef __pyx_methods[] = {
  {0, 0, 0, 0}
};
#ifndef CYTHON_SMALL_CODE
#if defined(__clang__)
    #define CYTHON_SMALL_CODE
#elif defined(__GNUC__) && (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 3))
    #define CYTHON_SMALL_CODE __attribute__((cold))
#else
    #define CYTHON_SMALL_CODE
#endif
#endif
/* #### Code section: pystring_table ### */

static __Pyx_StringTabEntry __pyx_string_tab[] = {
  #if CYTHON_USE_MODULE_STATE
  {0, __pyx_k_, sizeof(__pyx_k_), 0, 0, 0, 0},
  {0, __pyx_k_Container, sizeof(__pyx_k_Container), 0, 0, 1, 1},
  {0, __pyx_k_ContainerItemsIterator, sizeof(__pyx_k_ContainerItemsIterator), 0, 0, 1, 1},
  {0, __pyx_k_ContainerKey, sizeof(__pyx_k_ContainerKey), 0, 0, 1, 1},
  {0, __pyx_k_ContainerTreeView, sizeof(__pyx_k_ContainerTreeView), 0, 0, 1, 1},
  {0, __pyx_k_ContainerValue, sizeof(__pyx_k_ContainerValue), 0, 0, 1, 1},
  {0, __pyx_k_Incompatible_checksums_s_vs_0x0d, sizeof(__pyx_k_Incompatible_checksums_s_vs_0x0d), 0, 0, 1, 0},
  {0, __pyx_k_Incompatible_checksums_s_vs_0xda, sizeof(__pyx_k_Incompatible_checksums_s_vs_0xda), 0, 0, 1, 0},
  {0, __pyx_k_Iterator, sizeof(__pyx_k_Iterator), 0, 0, 1, 1},
  {0, __pyx_k_KeyError, sizeof(__pyx_k_KeyError), 0, 0, 1, 1},
  {0, __pyx_k_PickleError, sizeof(__pyx_k_PickleError), 0, 0, 1, 1},
  {0, __pyx_k_PrefixView, sizeof(__pyx_k_PrefixView), 0, 0, 1, 1},
  {0, __pyx_k_PrefixViewItemsIterator, sizeof(__pyx_k_PrefixViewItemsIterator), 0, 0, 1, 1},
  {0, __pyx_k_PrefixViewItemsIterator___reduce, sizeof(__pyx_k_PrefixViewItemsIterator___reduce), 0, 0, 1, 1},
  {0, __pyx_k_PrefixViewItemsIterator___setsta, sizeof(__pyx_k_PrefixViewItemsIterator___setsta), 0, 0, 1, 1},
  {0, __pyx_k_PrefixViewItemsIterator_next, sizeof(__pyx_k_PrefixViewItemsIterator_next), 0, 0, 1, 1},
  {0, __pyx_k_PrefixView___reduce_cython, sizeof(__pyx_k_PrefixView___reduce_cython), 0, 0, 1, 1},
  {0, __pyx_k_PrefixView___setstate_cython, sizeof(__pyx_k_PrefixView___setstate_cython), 0, 0, 1, 1},
  {0, __pyx_k_PrefixView_absolute_path, sizeof(__pyx_k_PrefixView_absolute_path), 0, 0, 1, 1},
  {0, __pyx_k_PrefixView_batch, sizeof(__pyx_k_PrefixView_batch), 0, 0, 1, 1},
  {0, __pyx_k_PrefixView_close, sizeof(__pyx_k_PrefixView_close), 0, 0, 1, 1},
  {0, __pyx_k_PrefixView_commit, sizeof(__pyx_k_PrefixView_commit), 0, 0, 1, 1},
  {0, __pyx_k_PrefixView_delete, sizeof(__pyx_k_PrefixView_delete), 0, 0, 1, 1},
  {0, __pyx_k_PrefixView_delete_range, sizeof(__pyx_k_PrefixView_delete_range), 0, 0, 1, 1},
  {0, __pyx_k_PrefixView_finalize, sizeof(__pyx_k_PrefixView_finalize), 0, 0, 1, 1},
  {0, __pyx_k_PrefixView_get, sizeof(__pyx_k_PrefixView_get), 0, 0, 1, 1},
  {0, __pyx_k_PrefixView_items, sizeof(__pyx_k_PrefixView_items), 0, 0, 1, 1},
  {0, __pyx_k_PrefixView_next_item, sizeof(__pyx_k_PrefixView_next_item), 0, 0, 1, 1},
  {0, __pyx_k_PrefixView_preload, sizeof(__pyx_k_PrefixView_preload), 0, 0, 1, 1},
  {0, __pyx_k_PrefixView_prev_item, sizeof(__pyx_k_PrefixView_prev_item), 0, 0, 1, 1},
  {0, __pyx_k_PrefixView_set, sizeof(__pyx_k_PrefixView_set), 0, 0, 1, 1},
  {0, __pyx_k_PrefixView_tree, sizeof(__pyx_k_PrefixView_tree), 0, 0, 1, 1},
  {0, __pyx_k_PrefixView_view, sizeof(__pyx_k_PrefixView_view), 0, 0, 1, 1},
  {0, __pyx_k_PrefixView_walk, sizeof(__pyx_k_PrefixView_walk), 0, 0, 1, 1},
  {0, __pyx_k_Tuple, sizeof(__pyx_k_Tuple), 0, 0, 1, 1},
  {0, __pyx_k__2, sizeof(__pyx_k__2), 0, 0, 0, 0},
  {0, __pyx_k__54, sizeof(__pyx_k__54), 0, 0, 1, 1},
  {0, __pyx_k_absolute_path, sizeof(__pyx_k_absolute_path), 0, 0, 1, 1},
  {0, __pyx_k_aim_storage_container, sizeof(__pyx_k_aim_storage_container), 0, 0, 1, 1},
  {0, __pyx_k_aim_storage_containertreeview, sizeof(__pyx_k_aim_storage_containertreeview), 0, 0, 1, 1},
  {0, __pyx_k_aim_storage_prefixview, sizeof(__pyx_k_aim_storage_prefixview), 0, 0, 1, 1},
  {0, __pyx_k_aim_storage_prefixview_py, sizeof(__pyx_k_aim_storage_prefixview_py), 0, 0, 1, 0},
  {0, __pyx_k_args, sizeof(__pyx_k_args), 0, 0, 1, 1},
  {0, __pyx_k_asyncio_coroutines, sizeof(__pyx_k_asyncio_coroutines), 0, 0, 1, 1},
  {0, __pyx_k_batch, sizeof(__pyx_k_batch), 0, 0, 1, 1},
  {0, __pyx_k_begin, sizeof(__pyx_k_begin), 0, 0, 1, 1},
  {0, __pyx_k_bytes, sizeof(__pyx_k_bytes), 0, 0, 1, 1},
  {0, __pyx_k_class_getitem, sizeof(__pyx_k_class_getitem), 0, 0, 1, 1},
  {0, __pyx_k_cline_in_traceback, sizeof(__pyx_k_cline_in_traceback), 0, 0, 1, 1},
  {0, __pyx_k_close, sizeof(__pyx_k_close), 0, 0, 1, 1},
  {0, __pyx_k_commit, sizeof(__pyx_k_commit), 0, 0, 1, 1},
  {0, __pyx_k_container, sizeof(__pyx_k_container), 0, 0, 1, 1},
  {0, __pyx_k_default, sizeof(__pyx_k_default), 0, 0, 1, 1},
  {0, __pyx_k_delete, sizeof(__pyx_k_delete), 0, 0, 1, 1},
  {0, __pyx_k_delete_range, sizeof(__pyx_k_delete_range), 0, 0, 1, 1},
  {0, __pyx_k_dict, sizeof(__pyx_k_dict), 0, 0, 1, 1},
  {0, __pyx_k_dict_2, sizeof(__pyx_k_dict_2), 0, 0, 1, 1},
  {0, __pyx_k_disable, sizeof(__pyx_k_disable), 0, 1, 0, 0},
  {0, __pyx_k_enable, sizeof(__pyx_k_enable), 0, 1, 0, 0},
  {0, __pyx_k_end, sizeof(__pyx_k_end), 0, 0, 1, 1},
  {0, __pyx_k_finalize, sizeof(__pyx_k_finalize), 0, 0, 1, 1},
  {0, __pyx_k_gc, sizeof(__pyx_k_gc), 0, 1, 0, 0},
  {0, __pyx_k_get, sizeof(__pyx_k_get), 0, 0, 1, 1},
  {0, __pyx_k_getstate, sizeof(__pyx_k_getstate), 0, 0, 1, 1},
  {0, __pyx_k_import, sizeof(__pyx_k_import), 0, 0, 1, 1},
  {0, __pyx_k_index, sizeof(__pyx_k_index), 0, 0, 1, 1},
  {0, __pyx_k_is_coroutine, sizeof(__pyx_k_is_coroutine), 0, 0, 1, 1},
  {0, __pyx_k_isenabled, sizeof(__pyx_k_isenabled), 0, 1, 0, 0},
  {0, __pyx_k_items, sizeof(__pyx_k_items), 0, 0, 1, 1},
  {0, __pyx_k_key, sizeof(__pyx_k_key), 0, 0, 1, 1},
  {0, __pyx_k_main, sizeof(__pyx_k_main), 0, 0, 1, 1},
  {0, __pyx_k_name, sizeof(__pyx_k_name), 0, 0, 1, 1},
  {0, __pyx_k_new, sizeof(__pyx_k_new), 0, 0, 1, 1},
  {0, __pyx_k_next, sizeof(__pyx_k_next), 0, 0, 1, 1},
  {0, __pyx_k_next_item, sizeof(__pyx_k_next_item), 0, 0, 1, 1},
  {0, __pyx_k_next_key, sizeof(__pyx_k_next_key), 0, 0, 1, 1},
  {0, __pyx_k_p, sizeof(__pyx_k_p), 0, 0, 1, 1},
  {0, __pyx_k_parent, sizeof(__pyx_k_parent), 0, 0, 1, 1},
  {0, __pyx_k_partition, sizeof(__pyx_k_partition), 0, 0, 1, 1},
  {0, __pyx_k_path, sizeof(__pyx_k_path), 0, 0, 1, 1},
  {0, __pyx_k_path_2, sizeof(__pyx_k_path_2), 0, 0, 1, 1},
  {0, __pyx_k_pickle, sizeof(__pyx_k_pickle), 0, 0, 1, 1},
  {0, __pyx_k_prefix, sizeof(__pyx_k_prefix), 0, 0, 1, 1},
  {0, __pyx_k_prefix_2, sizeof(__pyx_k_prefix_2), 0, 0, 1, 1},
  {0, __pyx_k_prefix_view, sizeof(__pyx_k_prefix_view), 0, 0, 1, 1},
  {0, __pyx_k_preload, sizeof(__pyx_k_preload), 0, 0, 1, 1},
  {0, __pyx_k_prev_item, sizeof(__pyx_k_prev_item), 0, 0, 1, 1},
  {0, __pyx_k_pyx_PickleError, sizeof(__pyx_k_pyx_PickleError), 0, 0, 1, 1},
  {0, __pyx_k_pyx_checksum, sizeof(__pyx_k_pyx_checksum), 0, 0, 1, 1},
  {0, __pyx_k_pyx_result, sizeof(__pyx_k_pyx_result), 0, 0, 1, 1},
  {0, __pyx_k_pyx_state, sizeof(__pyx_k_pyx_state), 0, 0, 1, 1},
  {0, __pyx_k_pyx_type, sizeof(__pyx_k_pyx_type), 0, 0, 1, 1},
  {0, __pyx_k_pyx_unpickle_PrefixView, sizeof(__pyx_k_pyx_unpickle_PrefixView), 0, 0, 1, 1},
  {0, __pyx_k_pyx_unpickle_PrefixViewItemsIt, sizeof(__pyx_k_pyx_unpickle_PrefixViewItemsIt), 0, 0, 1, 1},
  {0, __pyx_k_pyx_vtable, sizeof(__pyx_k_pyx_vtable), 0, 0, 1, 1},
  {0, __pyx_k_read_only, sizeof(__pyx_k_read_only), 0, 0, 1, 1},
  {0, __pyx_k_reduce, sizeof(__pyx_k_reduce), 0, 0, 1, 1},
  {0, __pyx_k_reduce_cython, sizeof(__pyx_k_reduce_cython), 0, 0, 1, 1},
  {0, __pyx_k_reduce_ex, sizeof(__pyx_k_reduce_ex), 0, 0, 1, 1},
  {0, __pyx_k_return, sizeof(__pyx_k_return), 0, 0, 1, 1},
  {0, __pyx_k_self, sizeof(__pyx_k_self), 0, 0, 1, 1},
  {0, __pyx_k_send, sizeof(__pyx_k_send), 0, 0, 1, 1},
  {0, __pyx_k_set, sizeof(__pyx_k_set), 0, 0, 1, 1},
  {0, __pyx_k_setstate, sizeof(__pyx_k_setstate), 0, 0, 1, 1},
  {0, __pyx_k_setstate_cython, sizeof(__pyx_k_setstate_cython), 0, 0, 1, 1},
  {0, __pyx_k_state, sizeof(__pyx_k_state), 0, 0, 1, 1},
  {0, __pyx_k_store_batch, sizeof(__pyx_k_store_batch), 0, 0, 1, 1},
  {0, __pyx_k_stringsource, sizeof(__pyx_k_stringsource), 0, 0, 1, 0},
  {0, __pyx_k_test, sizeof(__pyx_k_test), 0, 0, 1, 1},
  {0, __pyx_k_throw, sizeof(__pyx_k_throw), 0, 0, 1, 1},
  {0, __pyx_k_tree, sizeof(__pyx_k_tree), 0, 0, 1, 1},
  {0, __pyx_k_typing, sizeof(__pyx_k_typing), 0, 0, 1, 1},
  {0, __pyx_k_update, sizeof(__pyx_k_update), 0, 0, 1, 1},
  {0, __pyx_k_use_setstate, sizeof(__pyx_k_use_setstate), 0, 0, 1, 1},
  {0, __pyx_k_value, sizeof(__pyx_k_value), 0, 0, 1, 1},
  {0, __pyx_k_view, sizeof(__pyx_k_view), 0, 0, 1, 1},
  {0, __pyx_k_walk, sizeof(__pyx_k_walk), 0, 0, 1, 1},
  {0, __pyx_k_walker, sizeof(__pyx_k_walker), 0, 0, 1, 1},
  #else
  {&__pyx_kp_b_, __pyx_k_, sizeof(__pyx_k_), 0, 0, 0, 0},
  {&__pyx_n_s_Container, __pyx_k_Container, sizeof(__pyx_k_Container), 0, 0, 1, 1},
  {&__pyx_n_s_ContainerItemsIterator, __pyx_k_ContainerItemsIterator, sizeof(__pyx_k_ContainerItemsIterator), 0, 0, 1, 1},
  {&__pyx_n_s_ContainerKey, __pyx_k_ContainerKey, sizeof(__pyx_k_ContainerKey), 0, 0, 1, 1},
  {&__pyx_n_s_ContainerTreeView, __pyx_k_ContainerTreeView, sizeof(__pyx_k_ContainerTreeView), 0, 0, 1, 1},
  {&__pyx_n_s_ContainerValue, __pyx_k_ContainerValue, sizeof(__pyx_k_ContainerValue), 0, 0, 1, 1},
  {&__pyx_kp_s_Incompatible_checksums_s_vs_0x0d, __pyx_k_Incompatible_checksums_s_vs_0x0d, sizeof(__pyx_k_Incompatible_checksums_s_vs_0x0d), 0, 0, 1, 0},
  {&__pyx_kp_s_Incompatible_checksums_s_vs_0xda, __pyx_k_Incompatible_checksums_s_vs_0xda, sizeof(__pyx_k_Incompatible_checksums_s_vs_0xda), 0, 0, 1, 0},
  {&__pyx_n_s_Iterator, __pyx_k_Iterator, sizeof(__pyx_k_Iterator), 0, 0, 1, 1},
  {&__pyx_n_s_KeyError, __pyx_k_KeyError, sizeof(__pyx_k_KeyError), 0, 0, 1, 1},
  {&__pyx_n_s_PickleError, __pyx_k_PickleError, sizeof(__pyx_k_PickleError), 0, 0, 1, 1},
  {&__pyx_n_s_PrefixView, __pyx_k_PrefixView, sizeof(__pyx_k_PrefixView), 0, 0, 1, 1},
  {&__pyx_n_s_PrefixViewItemsIterator, __pyx_k_PrefixViewItemsIterator, sizeof(__pyx_k_PrefixViewItemsIterator), 0, 0, 1, 1},
  {&__pyx_n_s_PrefixViewItemsIterator___reduce, __pyx_k_PrefixViewItemsIterator___reduce, sizeof(__pyx_k_PrefixViewItemsIterator___reduce), 0, 0, 1, 1},
  {&__pyx_n_s_PrefixViewItemsIterator___setsta, __pyx_k_PrefixViewItemsIterator___setsta, sizeof(__pyx_k_PrefixViewItemsIterator___setsta), 0, 0, 1, 1},
  {&__pyx_n_s_PrefixViewItemsIterator_next, __pyx_k_PrefixViewItemsIterator_next, sizeof(__pyx_k_PrefixViewItemsIterator_next), 0, 0, 1, 1},
  {&__pyx_n_s_PrefixView___reduce_cython, __pyx_k_PrefixView___reduce_cython, sizeof(__pyx_k_PrefixView___reduce_cython), 0, 0, 1, 1},
  {&__pyx_n_s_PrefixView___setstate_cython, __pyx_k_PrefixView___setstate_cython, sizeof(__pyx_k_PrefixView___setstate_cython), 0, 0, 1, 1},
  {&__pyx_n_s_PrefixView_absolute_path, __pyx_k_PrefixView_absolute_path, sizeof(__pyx_k_PrefixView_absolute_path), 0, 0, 1, 1},
  {&__pyx_n_s_PrefixView_batch, __pyx_k_PrefixView_batch, sizeof(__pyx_k_PrefixView_batch), 0, 0, 1, 1},
  {&__pyx_n_s_PrefixView_close, __pyx_k_PrefixView_close, sizeof(__pyx_k_PrefixView_close), 0, 0, 1, 1},
  {&__pyx_n_s_PrefixView_commit, __pyx_k_PrefixView_commit, sizeof(__pyx_k_PrefixView_commit), 0, 0, 1, 1},
  {&__pyx_n_s_PrefixView_delete, __pyx_k_PrefixView_delete, sizeof(__pyx_k_PrefixView_delete), 0, 0, 1, 1},
  {&__pyx_n_s_PrefixView_delete_range, __pyx_k_PrefixView_delete_range, sizeof(__pyx_k_PrefixView_delete_range), 0, 0, 1, 1},
  {&__pyx_n_s_PrefixView_finalize, __pyx_k_PrefixView_finalize, sizeof(__pyx_k_PrefixView_finalize), 0, 0, 1, 1},
  {&__pyx_n_s_PrefixView_get, __pyx_k_PrefixView_get, sizeof(__pyx_k_PrefixView_get), 0, 0, 1, 1},
  {&__pyx_n_s_PrefixView_items, __pyx_k_PrefixView_items, sizeof(__pyx_k_PrefixView_items), 0, 0, 1, 1},
  {&__pyx_n_s_PrefixView_next_item, __pyx_k_PrefixView_next_item, sizeof(__pyx_k_PrefixView_next_item), 0, 0, 1, 1},
  {&__pyx_n_s_PrefixView_preload, __pyx_k_PrefixView_preload, sizeof(__pyx_k_PrefixView_preload), 0, 0, 1, 1},
  {&__pyx_n_s_PrefixView_prev_item, __pyx_k_PrefixView_prev_item, sizeof(__pyx_k_PrefixView_prev_item), 0, 0, 1, 1},
  {&__pyx_n_s_PrefixView_set, __pyx_k_PrefixView_set, sizeof(__pyx_k_PrefixView_set), 0, 0, 1, 1},
  {&__pyx_n_s_PrefixView_tree, __pyx_k_PrefixView_tree, sizeof(__pyx_k_PrefixView_tree), 0, 0, 1, 1},
  {&__pyx_n_s_PrefixView_view, __pyx_k_PrefixView_view, sizeof(__pyx_k_PrefixView_view), 0, 0, 1, 1},
  {&__pyx_n_s_PrefixView_walk, __pyx_k_PrefixView_walk, sizeof(__pyx_k_PrefixView_walk), 0, 0, 1, 1},
  {&__pyx_n_s_Tuple, __pyx_k_Tuple, sizeof(__pyx_k_Tuple), 0, 0, 1, 1},
  {&__pyx_kp_b__2, __pyx_k__2, sizeof(__pyx_k__2), 0, 0, 0, 0},
  {&__pyx_n_s__54, __pyx_k__54, sizeof(__pyx_k__54), 0, 0, 1, 1},
  {&__pyx_n_s_absolute_path, __pyx_k_absolute_path, sizeof(__pyx_k_absolute_path), 0, 0, 1, 1},
  {&__pyx_n_s_aim_storage_container, __pyx_k_aim_storage_container, sizeof(__pyx_k_aim_storage_container), 0, 0, 1, 1},
  {&__pyx_n_s_aim_storage_containertreeview, __pyx_k_aim_storage_containertreeview, sizeof(__pyx_k_aim_storage_containertreeview), 0, 0, 1, 1},
  {&__pyx_n_s_aim_storage_prefixview, __pyx_k_aim_storage_prefixview, sizeof(__pyx_k_aim_storage_prefixview), 0, 0, 1, 1},
  {&__pyx_kp_s_aim_storage_prefixview_py, __pyx_k_aim_storage_prefixview_py, sizeof(__pyx_k_aim_storage_prefixview_py), 0, 0, 1, 0},
  {&__pyx_n_s_args, __pyx_k_args, sizeof(__pyx_k_args), 0, 0, 1, 1},
  {&__pyx_n_s_asyncio_coroutines, __pyx_k_asyncio_coroutines, sizeof(__pyx_k_asyncio_coroutines), 0, 0, 1, 1},
  {&__pyx_n_s_batch, __pyx_k_batch, sizeof(__pyx_k_batch), 0, 0, 1, 1},
  {&__pyx_n_s_begin, __pyx_k_begin, sizeof(__pyx_k_begin), 0, 0, 1, 1},
  {&__pyx_n_s_bytes, __pyx_k_bytes, sizeof(__pyx_k_bytes), 0, 0, 1, 1},
  {&__pyx_n_s_class_getitem, __pyx_k_class_getitem, sizeof(__pyx_k_class_getitem), 0, 0, 1, 1},
  {&__pyx_n_s_cline_in_traceback, __pyx_k_cline_in_traceback, sizeof(__pyx_k_cline_in_traceback), 0, 0, 1, 1},
  {&__pyx_n_s_close, __pyx_k_close, sizeof(__pyx_k_close), 0, 0, 1, 1},
  {&__pyx_n_s_commit, __pyx_k_commit, sizeof(__pyx_k_commit), 0, 0, 1, 1},
  {&__pyx_n_s_container, __pyx_k_container, sizeof(__pyx_k_container), 0, 0, 1, 1},
  {&__pyx_n_s_default, __pyx_k_default, sizeof(__pyx_k_default), 0, 0, 1, 1},
  {&__pyx_n_s_delete, __pyx_k_delete, sizeof(__pyx_k_delete), 0, 0, 1, 1},
  {&__pyx_n_s_delete_range, __pyx_k_delete_range, sizeof(__pyx_k_delete_range), 0, 0, 1, 1},
  {&__pyx_n_s_dict, __pyx_k_dict, sizeof(__pyx_k_dict), 0, 0, 1, 1},
  {&__pyx_n_s_dict_2, __pyx_k_dict_2, sizeof(__pyx_k_dict_2), 0, 0, 1, 1},
  {&__pyx_kp_u_disable, __pyx_k_disable, sizeof(__pyx_k_disable), 0, 1, 0, 0},
  {&__pyx_kp_u_enable, __pyx_k_enable, sizeof(__pyx_k_enable), 0, 1, 0, 0},
  {&__pyx_n_s_end, __pyx_k_end, sizeof(__pyx_k_end), 0, 0, 1, 1},
  {&__pyx_n_s_finalize, __pyx_k_finalize, sizeof(__pyx_k_finalize), 0, 0, 1, 1},
  {&__pyx_kp_u_gc, __pyx_k_gc, sizeof(__pyx_k_gc), 0, 1, 0, 0},
  {&__pyx_n_s_get, __pyx_k_get, sizeof(__pyx_k_get), 0, 0, 1, 1},
  {&__pyx_n_s_getstate, __pyx_k_getstate, sizeof(__pyx_k_getstate), 0, 0, 1, 1},
  {&__pyx_n_s_import, __pyx_k_import, sizeof(__pyx_k_import), 0, 0, 1, 1},
  {&__pyx_n_s_index, __pyx_k_index, sizeof(__pyx_k_index), 0, 0, 1, 1},
  {&__pyx_n_s_is_coroutine, __pyx_k_is_coroutine, sizeof(__pyx_k_is_coroutine), 0, 0, 1, 1},
  {&__pyx_kp_u_isenabled, __pyx_k_isenabled, sizeof(__pyx_k_isenabled), 0, 1, 0, 0},
  {&__pyx_n_s_items, __pyx_k_items, sizeof(__pyx_k_items), 0, 0, 1, 1},
  {&__pyx_n_s_key, __pyx_k_key, sizeof(__pyx_k_key), 0, 0, 1, 1},
  {&__pyx_n_s_main, __pyx_k_main, sizeof(__pyx_k_main), 0, 0, 1, 1},
  {&__pyx_n_s_name, __pyx_k_name, sizeof(__pyx_k_name), 0, 0, 1, 1},
  {&__pyx_n_s_new, __pyx_k_new, sizeof(__pyx_k_new), 0, 0, 1, 1},
  {&__pyx_n_s_next, __pyx_k_next, sizeof(__pyx_k_next), 0, 0, 1, 1},
  {&__pyx_n_s_next_item, __pyx_k_next_item, sizeof(__pyx_k_next_item), 0, 0, 1, 1},
  {&__pyx_n_s_next_key, __pyx_k_next_key, sizeof(__pyx_k_next_key), 0, 0, 1, 1},
  {&__pyx_n_s_p, __pyx_k_p, sizeof(__pyx_k_p), 0, 0, 1, 1},
  {&__pyx_n_s_parent, __pyx_k_parent, sizeof(__pyx_k_parent), 0, 0, 1, 1},
  {&__pyx_n_s_partition, __pyx_k_partition, sizeof(__pyx_k_partition), 0, 0, 1, 1},
  {&__pyx_n_s_path, __pyx_k_path, sizeof(__pyx_k_path), 0, 0, 1, 1},
  {&__pyx_n_s_path_2, __pyx_k_path_2, sizeof(__pyx_k_path_2), 0, 0, 1, 1},
  {&__pyx_n_s_pickle, __pyx_k_pickle, sizeof(__pyx_k_pickle), 0, 0, 1, 1},
  {&__pyx_n_s_prefix, __pyx_k_prefix, sizeof(__pyx_k_prefix), 0, 0, 1, 1},
  {&__pyx_n_s_prefix_2, __pyx_k_prefix_2, sizeof(__pyx_k_prefix_2), 0, 0, 1, 1},
  {&__pyx_n_s_prefix_view, __pyx_k_prefix_view, sizeof(__pyx_k_prefix_view), 0, 0, 1, 1},
  {&__pyx_n_s_preload, __pyx_k_preload, sizeof(__pyx_k_preload), 0, 0, 1, 1},
  {&__pyx_n_s_prev_item, __pyx_k_prev_item, sizeof(__pyx_k_prev_item), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_PickleError, __pyx_k_pyx_PickleError, sizeof(__pyx_k_pyx_PickleError), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_checksum, __pyx_k_pyx_checksum, sizeof(__pyx_k_pyx_checksum), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_result, __pyx_k_pyx_result, sizeof(__pyx_k_pyx_result), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_state, __pyx_k_pyx_state, sizeof(__pyx_k_pyx_state), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_type, __pyx_k_pyx_type, sizeof(__pyx_k_pyx_type), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_unpickle_PrefixView, __pyx_k_pyx_unpickle_PrefixView, sizeof(__pyx_k_pyx_unpickle_PrefixView), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_unpickle_PrefixViewItemsIt, __pyx_k_pyx_unpickle_PrefixViewItemsIt, sizeof(__pyx_k_pyx_unpickle_PrefixViewItemsIt), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_vtable, __pyx_k_pyx_vtable, sizeof(__pyx_k_pyx_vtable), 0, 0, 1, 1},
  {&__pyx_n_s_read_only, __pyx_k_read_only, sizeof(__pyx_k_read_only), 0, 0, 1, 1},
  {&__pyx_n_s_reduce, __pyx_k_reduce, sizeof(__pyx_k_reduce), 0, 0, 1, 1},
  {&__pyx_n_s_reduce_cython, __pyx_k_reduce_cython, sizeof(__pyx_k_reduce_cython), 0, 0, 1, 1},
  {&__pyx_n_s_reduce_ex, __pyx_k_reduce_ex, sizeof(__pyx_k_reduce_ex), 0, 0, 1, 1},
  {&__pyx_n_s_return, __pyx_k_return, sizeof(__pyx_k_return), 0, 0, 1, 1},
  {&__pyx_n_s_self, __pyx_k_self, sizeof(__pyx_k_self), 0, 0, 1, 1},
  {&__pyx_n_s_send, __pyx_k_send, sizeof(__pyx_k_send), 0, 0, 1, 1},
  {&__pyx_n_s_set, __pyx_k_set, sizeof(__pyx_k_set), 0, 0, 1, 1},
  {&__pyx_n_s_setstate, __pyx_k_setstate, sizeof(__pyx_k_setstate), 0, 0, 1, 1},
  {&__pyx_n_s_setstate_cython, __pyx_k_setstate_cython, sizeof(__pyx_k_setstate_cython), 0, 0, 1, 1},
  {&__pyx_n_s_state, __pyx_k_state, sizeof(__pyx_k_state), 0, 0, 1, 1},
  {&__pyx_n_s_store_batch, __pyx_k_store_batch, sizeof(__pyx_k_store_batch), 0, 0, 1, 1},
  {&__pyx_kp_s_stringsource, __pyx_k_stringsource, sizeof(__pyx_k_stringsource), 0, 0, 1, 0},
  {&__pyx_n_s_test, __pyx_k_test, sizeof(__pyx_k_test), 0, 0, 1, 1},
  {&__pyx_n_s_throw, __pyx_k_throw, sizeof(__pyx_k_throw), 0, 0, 1, 1},
  {&__pyx_n_s_tree, __pyx_k_tree, sizeof(__pyx_k_tree), 0, 0, 1, 1},
  {&__pyx_n_s_typing, __pyx_k_typing, sizeof(__pyx_k_typing), 0, 0, 1, 1},
  {&__pyx_n_s_update, __pyx_k_update, sizeof(__pyx_k_update), 0, 0, 1, 1},
  {&__pyx_n_s_use_setstate, __pyx_k_use_setstate, sizeof(__pyx_k_use_setstate), 0, 0, 1, 1},
  {&__pyx_n_s_value, __pyx_k_value, sizeof(__pyx_k_value), 0, 0, 1, 1},
  {&__pyx_n_s_view, __pyx_k_view, sizeof(__pyx_k_view), 0, 0, 1, 1},
  {&__pyx_n_s_walk, __pyx_k_walk, sizeof(__pyx_k_walk), 0, 0, 1, 1},
  {&__pyx_n_s_walker, __pyx_k_walker, sizeof(__pyx_k_walker), 0, 0, 1, 1},
  #endif
  {0, 0, 0, 0, 0, 0, 0}
};
/* #### Code section: cached_builtins ### */
static CYTHON_SMALL_CODE int __Pyx_InitCachedBuiltins(void) {
  __pyx_builtin_KeyError = __Pyx_GetBuiltinName(__pyx_n_s_KeyError); if (!__pyx_builtin_KeyError) __PYX_ERR(0, 178, __pyx_L1_error)
  return 0;
  __pyx_L1_error:;
  return -1;
}
/* #### Code section: cached_constants ### */

static CYTHON_SMALL_CODE int __Pyx_InitCachedConstants(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_InitCachedConstants", 0);

  /* "aim/storage/prefixview.py":44
 *         self.read_only = read_only
 * 
 *     def close(self):             # <<<<<<<<<<<<<<
 *         """Close all the resources."""
 *         pass
 */
  __pyx_tuple__4 = PyTuple_Pack(1, __pyx_n_s_self); if (unlikely(!__pyx_tuple__4)) __PYX_ERR(0, 44, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__4);
  __Pyx_GIVEREF(__pyx_tuple__4);
  __pyx_codeobj__5 = (PyObject*)__Pyx_PyCode_New(1, 0, 0, 1, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__4, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_aim_storage_prefixview_py, __pyx_n_s_close, 44, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__5)) __PYX_ERR(0, 44, __pyx_L1_error)

  /* "aim/storage/prefixview.py":48
 *         pass
 * 
 *     def preload(self):             # <<<<<<<<<<<<<<
 *         """Preload the Container.
 * 
 */
  __pyx_tuple__6 = PyTuple_Pack(1, __pyx_n_s_self); if (unlikely(!__pyx_tuple__6)) __PYX_ERR(0, 48, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__6);
  __Pyx_GIVEREF(__pyx_tuple__6);
  __pyx_codeobj__7 = (PyObject*)__Pyx_PyCode_New(1, 0, 0, 1, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__6, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_aim_storage_prefixview_py, __pyx_n_s_preload, 48, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__7)) __PYX_ERR(0, 48, __pyx_L1_error)

  /* "aim/storage/prefixview.py":58
 *         self.parent.preload()
 * 
 *     def finalize(self, index: Container):             # <<<<<<<<<<<<<<
 *         """Finalize the Container.
 * 
 */
  __pyx_tuple__8 = PyTuple_Pack(3, __pyx_n_s_self, __pyx_n_s_index, __pyx_n_s_prefix); if (unlikely(!__pyx_tuple__8)) __PYX_ERR(0, 58, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__8);
  __Pyx_GIVEREF(__pyx_tuple__8);
  __pyx_codeobj__9 = (PyObject*)__Pyx_PyCode_New(2, 0, 0, 3, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__8, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_aim_storage_prefixview_py, __pyx_n_s_finalize, 58, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__9)) __PYX_ERR(0, 58, __pyx_L1_error)

  /* "aim/storage/prefixview.py":68
 *         self.parent.finalize(index=index)
 * 
 *     def absolute_path(             # <<<<<<<<<<<<<<
 *         self,
 *         path: bytes = None
 */
  __pyx_tuple__10 = PyTuple_Pack(2, __pyx_n_s_self, __pyx_n_s_path); if (unlikely(!__pyx_tuple__10)) __PYX_ERR(0, 68, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__10);
  __Pyx_GIVEREF(__pyx_tuple__10);
  __pyx_codeobj__11 = (PyObject*)__Pyx_PyCode_New(2, 0, 0, 2, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__10, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_aim_storage_prefixview_py, __pyx_n_s_absolute_path, 68, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__11)) __PYX_ERR(0, 68, __pyx_L1_error)
  __pyx_tuple__12 = PyTuple_Pack(1, Py_None); if (unlikely(!__pyx_tuple__12)) __PYX_ERR(0, 68, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__12);
  __Pyx_GIVEREF(__pyx_tuple__12);

  /* "aim/storage/prefixview.py":83
 *         return self.prefix + path
 * 
 *     def get(             # <<<<<<<<<<<<<<
 *         self,
 *         key: ContainerKey,
 */
  __pyx_tuple__13 = PyTuple_Pack(3, __pyx_n_s_self, __pyx_n_s_key, __pyx_n_s_default); if (unlikely(!__pyx_tuple__13)) __PYX_ERR(0, 83, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__13);
  __Pyx_GIVEREF(__pyx_tuple__13);
  __pyx_codeobj__14 = (PyObject*)__Pyx_PyCode_New(3, 0, 0, 3, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__13, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_aim_storage_prefixview_py, __pyx_n_s_get, 83, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__14)) __PYX_ERR(0, 83, __pyx_L1_error)

  /* "aim/storage/prefixview.py":103
 *         return self.parent[path]
 * 
 *     def set(             # <<<<<<<<<<<<<<
 *         self,
 *         key: ContainerKey,
 */
  __pyx_tuple__15 = PyTuple_Pack(4, __pyx_n_s_self, __pyx_n_s_key, __pyx_n_s_value, __pyx_n_s_store_batch); if (unlikely(!__pyx_tuple__15)) __PYX_ERR(0, 103, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__15);
  __Pyx_GIVEREF(__pyx_tuple__15);
  __pyx_codeobj__16 = (PyObject*)__Pyx_PyCode_New(4, 0, 0, 4, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__15, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_aim_storage_prefixview_py, __pyx_n_s_set, 103, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__16)) __PYX_ERR(0, 103, __pyx_L1_error)

  /* "aim/storage/prefixview.py":129
 *         self.parent[path] = value
 * 
 *     def delete(             # <<<<<<<<<<<<<<
 *         self,
 *         key: ContainerKey,
 */
  __pyx_tuple__17 = PyTuple_Pack(4, __pyx_n_s_self, __pyx_n_s_key, __pyx_n_s_store_batch, __pyx_n_s_path); if (unlikely(!__pyx_tuple__17)) __PYX_ERR(0, 129, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__17);
  __Pyx_GIVEREF(__pyx_tuple__17);
  __pyx_codeobj__18 = (PyObject*)__Pyx_PyCode_New(3, 0, 0, 4, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__17, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_aim_storage_prefixview_py, __pyx_n_s_delete, 129, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__18)) __PYX_ERR(0, 129, __pyx_L1_error)

  /* "aim/storage/prefixview.py":154
 *         del self.parent[path]
 * 
 *     def delete_range(             # <<<<<<<<<<<<<<
 *         self,
 *         begin: ContainerKey,
 */
  __pyx_tuple__19 = PyTuple_Pack(4, __pyx_n_s_self, __pyx_n_s_begin, __pyx_n_s_end, __pyx_n_s_store_batch); if (unlikely(!__pyx_tuple__19)) __PYX_ERR(0, 154, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__19);
  __Pyx_GIVEREF(__pyx_tuple__19);
  __pyx_codeobj__20 = (PyObject*)__Pyx_PyCode_New(4, 0, 0, 4, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__19, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_aim_storage_prefixview_py, __pyx_n_s_delete_range, 154, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__20)) __PYX_ERR(0, 154, __pyx_L1_error)

  /* "aim/storage/prefixview.py":166
 *                                  store_batch=store_batch)
 * 
 *     def next_item(             # <<<<<<<<<<<<<<
 *         self,
 *         key: ContainerKey = b''
 */
  __pyx_tuple__21 = PyTuple_Pack(2, __pyx_n_s_self, __pyx_n_s_key); if (unlikely(!__pyx_tuple__21)) __PYX_ERR(0, 166, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__21);
  __Pyx_GIVEREF(__pyx_tuple__21);
  __pyx_codeobj__22 = (PyObject*)__Pyx_PyCode_New(2, 0, 0, 2, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__21, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_aim_storage_prefixview_py, __pyx_n_s_next_item, 166, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__22)) __PYX_ERR(0, 166, __pyx_L1_error)
  __pyx_tuple__23 = PyTuple_Pack(1, __pyx_kp_b_); if (unlikely(!__pyx_tuple__23)) __PYX_ERR(0, 166, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__23);
  __Pyx_GIVEREF(__pyx_tuple__23);

  /* "aim/storage/prefixview.py":181
 *         return keys, value
 * 
 *     def prev_item(             # <<<<<<<<<<<<<<
 *         self,
 *         key: ContainerKey = b''
 */
  __pyx_tuple__24 = PyTuple_Pack(2, __pyx_n_s_self, __pyx_n_s_key); if (unlikely(!__pyx_tuple__24)) __PYX_ERR(0, 181, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__24);
  __Pyx_GIVEREF(__pyx_tuple__24);
  __pyx_codeobj__25 = (PyObject*)__Pyx_PyCode_New(2, 0, 0, 2, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__24, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_aim_storage_prefixview_py, __pyx_n_s_prev_item, 181, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__25)) __PYX_ERR(0, 181, __pyx_L1_error)
  __pyx_tuple__26 = PyTuple_Pack(1, __pyx_kp_b_); if (unlikely(!__pyx_tuple__26)) __PYX_ERR(0, 181, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__26);
  __Pyx_GIVEREF(__pyx_tuple__26);

  /* "aim/storage/prefixview.py":196
 *         return keys, value
 * 
 *     def walk(             # <<<<<<<<<<<<<<
 *         self,
 *         key: ContainerKey = b''
 */
  __pyx_tuple__27 = PyTuple_Pack(8, __pyx_n_s_self, __pyx_n_s_key, __pyx_n_s_path, __pyx_n_s_walker, __pyx_n_s_p, __pyx_n_s_next_key, __pyx_n_s_prefix_2, __pyx_n_s_path_2); if (unlikely(!__pyx_tuple__27)) __PYX_ERR(0, 196, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__27);
  __Pyx_GIVEREF(__pyx_tuple__27);
  __pyx_codeobj__3 = (PyObject*)__Pyx_PyCode_New(2, 0, 0, 8, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__27, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_aim_storage_prefixview_py, __pyx_n_s_walk, 196, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__3)) __PYX_ERR(0, 196, __pyx_L1_error)
  __pyx_tuple__28 = PyTuple_Pack(1, __pyx_kp_b_); if (unlikely(!__pyx_tuple__28)) __PYX_ERR(0, 196, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__28);
  __Pyx_GIVEREF(__pyx_tuple__28);

  /* "aim/storage/prefixview.py":232
 *             p = self.absolute_path(key)
 * 
 *     def items(             # <<<<<<<<<<<<<<
 *         self,
 *         key: ContainerKey = b''
 */
  __pyx_tuple__29 = PyTuple_Pack(2, __pyx_n_s_self, __pyx_n_s_key); if (unlikely(!__pyx_tuple__29)) __PYX_ERR(0, 232, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__29);
  __Pyx_GIVEREF(__pyx_tuple__29);
  __pyx_codeobj__30 = (PyObject*)__Pyx_PyCode_New(2, 0, 0, 2, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__29, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_aim_storage_prefixview_py, __pyx_n_s_items, 232, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__30)) __PYX_ERR(0, 232, __pyx_L1_error)
  __pyx_tuple__31 = PyTuple_Pack(1, __pyx_kp_b_); if (unlikely(!__pyx_tuple__31)) __PYX_ERR(0, 232, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__31);
  __Pyx_GIVEREF(__pyx_tuple__31);

  /* "aim/storage/prefixview.py":255
 *         return PrefixViewItemsIterator(self, key)
 * 
 *     def view(             # <<<<<<<<<<<<<<
 *         self,
 *         prefix: bytes = b''
 */
  __pyx_tuple__32 = PyTuple_Pack(2, __pyx_n_s_self, __pyx_n_s_prefix); if (unlikely(!__pyx_tuple__32)) __PYX_ERR(0, 255, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__32);
  __Pyx_GIVEREF(__pyx_tuple__32);
  __pyx_codeobj__33 = (PyObject*)__Pyx_PyCode_New(2, 0, 0, 2, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__32, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_aim_storage_prefixview_py, __pyx_n_s_view, 255, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__33)) __PYX_ERR(0, 255, __pyx_L1_error)

  /* "aim/storage/prefixview.py":284
 *         return self.parent.view(self.prefix + prefix)
 * 
 *     def tree(             # <<<<<<<<<<<<<<
 *         self
 *     ) -> ContainerTreeView:
 */
  __pyx_tuple__34 = PyTuple_Pack(1, __pyx_n_s_self); if (unlikely(!__pyx_tuple__34)) __PYX_ERR(0, 284, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__34);
  __Pyx_GIVEREF(__pyx_tuple__34);
  __pyx_codeobj__35 = (PyObject*)__Pyx_PyCode_New(1, 0, 0, 1, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__34, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_aim_storage_prefixview_py, __pyx_n_s_tree, 284, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__35)) __PYX_ERR(0, 284, __pyx_L1_error)

  /* "aim/storage/prefixview.py":305
 *         return ContainerTreeView(self)
 * 
 *     def batch(self):             # <<<<<<<<<<<<<<
 *         """Creates a new batch object to store operations in before executing
 *         using :obj:`Container.commit`.
 */
  __pyx_tuple__36 = PyTuple_Pack(1, __pyx_n_s_self); if (unlikely(!__pyx_tuple__36)) __PYX_ERR(0, 305, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__36);
  __Pyx_GIVEREF(__pyx_tuple__36);
  __pyx_codeobj__37 = (PyObject*)__Pyx_PyCode_New(1, 0, 0, 1, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__36, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_aim_storage_prefixview_py, __pyx_n_s_batch, 305, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__37)) __PYX_ERR(0, 305, __pyx_L1_error)

  /* "aim/storage/prefixview.py":316
 *         return self.parent.batch()
 * 
 *     def commit(             # <<<<<<<<<<<<<<
 *         self,
 *         batch
 */
  __pyx_tuple__38 = PyTuple_Pack(2, __pyx_n_s_self, __pyx_n_s_batch); if (unlikely(!__pyx_tuple__38)) __PYX_ERR(0, 316, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__38);
  __Pyx_GIVEREF(__pyx_tuple__38);
  __pyx_codeobj__39 = (PyObject*)__Pyx_PyCode_New(2, 0, 0, 2, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__38, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_aim_storage_prefixview_py, __pyx_n_s_commit, 316, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__39)) __PYX_ERR(0, 316, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */
  __pyx_tuple__40 = PyTuple_Pack(4, __pyx_n_s_self, __pyx_n_s_state, __pyx_n_s_dict_2, __pyx_n_s_use_setstate); if (unlikely(!__pyx_tuple__40)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__40);
  __Pyx_GIVEREF(__pyx_tuple__40);
  __pyx_codeobj__41 = (PyObject*)__Pyx_PyCode_New(1, 0, 0, 4, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__40, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_reduce_cython, 1, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__41)) __PYX_ERR(2, 1, __pyx_L1_error)

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_PrefixView, (type(self), 0x0d8b696, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_PrefixView__set_state(self, __pyx_state)
 */
  __pyx_tuple__42 = PyTuple_Pack(2, __pyx_n_s_self, __pyx_n_s_pyx_state); if (unlikely(!__pyx_tuple__42)) __PYX_ERR(2, 16, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__42);
  __Pyx_GIVEREF(__pyx_tuple__42);
  __pyx_codeobj__43 = (PyObject*)__Pyx_PyCode_New(2, 0, 0, 2, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__42, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_setstate_cython, 16, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__43)) __PYX_ERR(2, 16, __pyx_L1_error)

  /* "aim/storage/prefixview.py":337
 *         self.prefix_len = len(self.path)
 * 
 *     def next(self):             # <<<<<<<<<<<<<<
 *         item = self.it.next()
 * 
 */
  __pyx_tuple__44 = PyTuple_Pack(1, __pyx_n_s_self); if (unlikely(!__pyx_tuple__44)) __PYX_ERR(0, 337, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__44);
  __Pyx_GIVEREF(__pyx_tuple__44);
  __pyx_codeobj__45 = (PyObject*)__Pyx_PyCode_New(1, 0, 0, 1, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__44, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_aim_storage_prefixview_py, __pyx_n_s_next, 337, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__45)) __PYX_ERR(0, 337, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */
  __pyx_tuple__46 = PyTuple_Pack(4, __pyx_n_s_self, __pyx_n_s_state, __pyx_n_s_dict_2, __pyx_n_s_use_setstate); if (unlikely(!__pyx_tuple__46)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__46);
  __Pyx_GIVEREF(__pyx_tuple__46);
  __pyx_codeobj__47 = (PyObject*)__Pyx_PyCode_New(1, 0, 0, 4, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__46, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_reduce_cython, 1, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__47)) __PYX_ERR(2, 1, __pyx_L1_error)

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_PrefixViewItemsIterator, (type(self), 0xda33529, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_PrefixViewItemsIterator__set_state(self, __pyx_state)
 */
  __pyx_tuple__48 = PyTuple_Pack(2, __pyx_n_s_self, __pyx_n_s_pyx_state); if (unlikely(!__pyx_tuple__48)) __PYX_ERR(2, 16, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__48);
  __Pyx_GIVEREF(__pyx_tuple__48);
  __pyx_codeobj__49 = (PyObject*)__Pyx_PyCode_New(2, 0, 0, 2, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__48, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_setstate_cython, 16, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__49)) __PYX_ERR(2, 16, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __pyx_unpickle_PrefixView(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */
  __pyx_tuple__50 = PyTuple_Pack(5, __pyx_n_s_pyx_type, __pyx_n_s_pyx_checksum, __pyx_n_s_pyx_state, __pyx_n_s_pyx_PickleError, __pyx_n_s_pyx_result); if (unlikely(!__pyx_tuple__50)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__50);
  __Pyx_GIVEREF(__pyx_tuple__50);
  __pyx_codeobj__51 = (PyObject*)__Pyx_PyCode_New(3, 0, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__50, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_pyx_unpickle_PrefixView, 1, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__51)) __PYX_ERR(2, 1, __pyx_L1_error)
  __pyx_tuple__52 = PyTuple_Pack(5, __pyx_n_s_pyx_type, __pyx_n_s_pyx_checksum, __pyx_n_s_pyx_state, __pyx_n_s_pyx_PickleError, __pyx_n_s_pyx_result); if (unlikely(!__pyx_tuple__52)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__52);
  __Pyx_GIVEREF(__pyx_tuple__52);
  __pyx_codeobj__53 = (PyObject*)__Pyx_PyCode_New(3, 0, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__52, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_pyx_unpickle_PrefixViewItemsIt, 1, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__53)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_RefNannyFinishContext();
  return -1;
}
/* #### Code section: init_globals ### */

static CYTHON_SMALL_CODE int __Pyx_InitGlobals(void) {
  #if CYTHON_USE_MODULE_STATE
  if (__Pyx_InitString(__pyx_string_tab[0], &__pyx_kp_b_) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  if (__Pyx_InitString(__pyx_string_tab[1], &__pyx_n_s_Container) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  if (__Pyx_InitString(__pyx_string_tab[2], &__pyx_n_s_ContainerItemsIterator) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  if (__Pyx_InitString(__pyx_string_tab[3], &__pyx_n_s_ContainerKey) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  if (__Pyx_InitString(__pyx_string_tab[4], &__pyx_n_s_ContainerTreeView) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  if (__Pyx_InitString(__pyx_string_tab[5], &__pyx_n_s_ContainerValue) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  if (__Pyx_InitString(__pyx_string_tab[6], &__pyx_kp_s_Incompatible_checksums_s_vs_0x0d) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  if (__Pyx_InitString(__pyx_string_tab[7], &__pyx_kp_s_Incompatible_checksums_s_vs_0xda) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  if (__Pyx_InitString(__pyx_string_tab[8], &__pyx_n_s_Iterator) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  if (__Pyx_InitString(__pyx_string_tab[9], &__pyx_n_s_KeyError) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  if (__Pyx_InitString(__pyx_string_tab[10], &__pyx_n_s_PickleError) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  if (__Pyx_InitString(__pyx_string_tab[11], &__pyx_n_s_PrefixView) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  if (__Pyx_InitString(__pyx_string_tab[12], &__pyx_n_s_PrefixViewItemsIterator) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  if (__Pyx_InitString(__pyx_string_tab[13], &__pyx_n_s_PrefixViewItemsIterator___reduce) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  if (__Pyx_InitString(__pyx_string_tab[14], &__pyx_n_s_PrefixViewItemsIterator___setsta) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  if (__Pyx_InitString(__pyx_string_tab[15], &__pyx_n_s_PrefixViewItemsIterator_next) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  if (__Pyx_InitString(__pyx_string_tab[16], &__pyx_n_s_PrefixView___reduce_cython) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  if (__Pyx_InitString(__pyx_string_tab[17], &__pyx_n_s_PrefixView___setstate_cython) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  if (__Pyx_InitString(__pyx_string_tab[18], &__pyx_n_s_PrefixView_absolute_path) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  if (__Pyx_InitString(__pyx_string_tab[19], &__pyx_n_s_PrefixView_batch) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  if (__Pyx_InitString(__pyx_string_tab[20], &__pyx_n_s_PrefixView_close) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  if (__Pyx_InitString(__pyx_string_tab[21], &__pyx_n_s_PrefixView_commit) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  if (__Pyx_InitString(__pyx_string_tab[22], &__pyx_n_s_PrefixView_delete) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  if (__Pyx_InitString(__pyx_string_tab[23], &__pyx_n_s_PrefixView_delete_range) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  if (__Pyx_InitString(__pyx_string_tab[24], &__pyx_n_s_PrefixView_finalize) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  if (__Pyx_InitString(__pyx_string_tab[25], &__pyx_n_s_PrefixView_get) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  if (__Pyx_InitString(__pyx_string_tab[26], &__pyx_n_s_PrefixView_items) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  if (__Pyx_InitString(__pyx_string_tab[27], &__pyx_n_s_PrefixView_next_item) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  if (__Pyx_InitString(__pyx_string_tab[28], &__pyx_n_s_PrefixView_preload) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  if (__Pyx_InitString(__pyx_string_tab[29], &__pyx_n_s_PrefixView_prev_item) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  if (__Pyx_InitString(__pyx_string_tab[30], &__pyx_n_s_PrefixView_set) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  if (__Pyx_InitString(__pyx_string_tab[31], &__pyx_n_s_PrefixView_tree) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  if (__Pyx_InitString(__pyx_string_tab[32], &__pyx_n_s_PrefixView_view) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  if (__Pyx_InitString(__pyx_string_tab[33], &__pyx_n_s_PrefixView_walk) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  if (__Pyx_InitString(__pyx_string_tab[34], &__pyx_n_s_Tuple) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  if (__Pyx_InitString(__pyx_string_tab[35], &__pyx_kp_b__2) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  if (__Pyx_InitString(__pyx_string_tab[36], &__pyx_n_s__54) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  if (__Pyx_InitString(__pyx_string_tab[37], &__pyx_n_s_absolute_path) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  if (__Pyx_InitString(__pyx_string_tab[38], &__pyx_n_s_aim_storage_container) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  if (__Pyx_InitString(__pyx_string_tab[39], &__pyx_n_s_aim_storage_containertreeview) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  if (__Pyx_InitString(__pyx_string_tab[40], &__pyx_n_s_aim_storage_prefixview) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  if (__Pyx_InitString(__pyx_string_tab[41], &__pyx_kp_s_aim_storage_prefixview_py) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  if (__Pyx_InitString(__pyx_string_tab[42], &__pyx_n_s_args) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  if (__Pyx_InitString(__pyx_string_tab[43], &__pyx_n_s_asyncio_coroutines) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  if (__Pyx_InitString(__pyx_string_tab[44], &__pyx_n_s_batch) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  if (__Pyx_InitString(__pyx_string_tab[45], &__pyx_n_s_begin) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  if (__Pyx_InitString(__pyx_string_tab[46], &__pyx_n_s_bytes) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  if (__Pyx_InitString(__pyx_string_tab[47], &__pyx_n_s_class_getitem) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  if (__Pyx_InitString(__pyx_string_tab[48], &__pyx_n_s_cline_in_traceback) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  if (__Pyx_InitString(__pyx_string_tab[49], &__pyx_n_s_close) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  if (__Pyx_InitString(__pyx_string_tab[50], &__pyx_n_s_commit) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  if (__Pyx_InitString(__pyx_string_tab[51], &__pyx_n_s_container) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  if (__Pyx_InitString(__pyx_string_tab[52], &__pyx_n_s_default) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  if (__Pyx_InitString(__pyx_string_tab[53], &__pyx_n_s_delete) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  if (__Pyx_InitString(__pyx_string_tab[54], &__pyx_n_s_delete_range) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  if (__Pyx_InitString(__pyx_string_tab[55], &__pyx_n_s_dict) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  if (__Pyx_InitString(__pyx_string_tab[56], &__pyx_n_s_dict_2) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  if (__Pyx_InitString(__pyx_string_tab[57], &__pyx_kp_u_disable) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  if (__Pyx_InitString(__pyx_string_tab[58], &__pyx_kp_u_enable) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  if (__Pyx_InitString(__pyx_string_tab[59], &__pyx_n_s_end) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  if (__Pyx_InitString(__pyx_string_tab[60], &__pyx_n_s_finalize) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  if (__Pyx_InitString(__pyx_string_tab[61], &__pyx_kp_u_gc) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  if (__Pyx_InitString(__pyx_string_tab[62], &__pyx_n_s_get) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  if (__Pyx_InitString(__pyx_string_tab[63], &__pyx_n_s_getstate) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  if (__Pyx_InitString(__pyx_string_tab[64], &__pyx_n_s_import) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  if (__Pyx_InitString(__pyx_string_tab[65], &__pyx_n_s_index) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  if (__Pyx_InitString(__pyx_string_tab[66], &__pyx_n_s_is_coroutine) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  if (__Pyx_InitString(__pyx_string_tab[67], &__pyx_kp_u_isenabled) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  if (__Pyx_InitString(__pyx_string_tab[68], &__pyx_n_s_items) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  if (__Pyx_InitString(__pyx_string_tab[69], &__pyx_n_s_key) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  if (__Pyx_InitString(__pyx_string_tab[70], &__pyx_n_s_main) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  if (__Pyx_InitString(__pyx_string_tab[71], &__pyx_n_s_name) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  if (__Pyx_InitString(__pyx_string_tab[72], &__pyx_n_s_new) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  if (__Pyx_InitString(__pyx_string_tab[73], &__pyx_n_s_next) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  if (__Pyx_InitString(__pyx_string_tab[74], &__pyx_n_s_next_item) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  if (__Pyx_InitString(__pyx_string_tab[75], &__pyx_n_s_next_key) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  if (__Pyx_InitString(__pyx_string_tab[76], &__pyx_n_s_p) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  if (__Pyx_InitString(__pyx_string_tab[77], &__pyx_n_s_parent) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  if (__Pyx_InitString(__pyx_string_tab[78], &__pyx_n_s_partition) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  if (__Pyx_InitString(__pyx_string_tab[79], &__pyx_n_s_path) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  if (__Pyx_InitString(__pyx_string_tab[80], &__pyx_n_s_path_2) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  if (__Pyx_InitString(__pyx_string_tab[81], &__pyx_n_s_pickle) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  if (__Pyx_InitString(__pyx_string_tab[82], &__pyx_n_s_prefix) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  if (__Pyx_InitString(__pyx_string_tab[83], &__pyx_n_s_prefix_2) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  if (__Pyx_InitString(__pyx_string_tab[84], &__pyx_n_s_prefix_view) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  if (__Pyx_InitString(__pyx_string_tab[85], &__pyx_n_s_preload) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  if (__Pyx_InitString(__pyx_string_tab[86], &__pyx_n_s_prev_item) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  if (__Pyx_InitString(__pyx_string_tab[87], &__pyx_n_s_pyx_PickleError) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  if (__Pyx_InitString(__pyx_string_tab[88], &__pyx_n_s_pyx_checksum) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  if (__Pyx_InitString(__pyx_string_tab[89], &__pyx_n_s_pyx_result) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  if (__Pyx_InitString(__pyx_string_tab[90], &__pyx_n_s_pyx_state) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  if (__Pyx_InitString(__pyx_string_tab[91], &__pyx_n_s_pyx_type) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  if (__Pyx_InitString(__pyx_string_tab[92], &__pyx_n_s_pyx_unpickle_PrefixView) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  if (__Pyx_InitString(__pyx_string_tab[93], &__pyx_n_s_pyx_unpickle_PrefixViewItemsIt) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  if (__Pyx_InitString(__pyx_string_tab[94], &__pyx_n_s_pyx_vtable) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  if (__Pyx_InitString(__pyx_string_tab[95], &__pyx_n_s_read_only) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  if (__Pyx_InitString(__pyx_string_tab[96], &__pyx_n_s_reduce) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  if (__Pyx_InitString(__pyx_string_tab[97], &__pyx_n_s_reduce_cython) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  if (__Pyx_InitString(__pyx_string_tab[98], &__pyx_n_s_reduce_ex) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  if (__Pyx_InitString(__pyx_string_tab[99], &__pyx_n_s_return) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  if (__Pyx_InitString(__pyx_string_tab[100], &__pyx_n_s_self) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  if (__Pyx_InitString(__pyx_string_tab[101], &__pyx_n_s_send) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  if (__Pyx_InitString(__pyx_string_tab[102], &__pyx_n_s_set) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  if (__Pyx_InitString(__pyx_string_tab[103], &__pyx_n_s_setstate) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  if (__Pyx_InitString(__pyx_string_tab[104], &__pyx_n_s_setstate_cython) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  if (__Pyx_InitString(__pyx_string_tab[105], &__pyx_n_s_state) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  if (__Pyx_InitString(__pyx_string_tab[106], &__pyx_n_s_store_batch) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  if (__Pyx_InitString(__pyx_string_tab[107], &__pyx_kp_s_stringsource) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  if (__Pyx_InitString(__pyx_string_tab[108], &__pyx_n_s_test) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  if (__Pyx_InitString(__pyx_string_tab[109], &__pyx_n_s_throw) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  if (__Pyx_InitString(__pyx_string_tab[110], &__pyx_n_s_tree) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  if (__Pyx_InitString(__pyx_string_tab[111], &__pyx_n_s_typing) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  if (__Pyx_InitString(__pyx_string_tab[112], &__pyx_n_s_update) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  if (__Pyx_InitString(__pyx_string_tab[113], &__pyx_n_s_use_setstate) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  if (__Pyx_InitString(__pyx_string_tab[114], &__pyx_n_s_value) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  if (__Pyx_InitString(__pyx_string_tab[115], &__pyx_n_s_view) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  if (__Pyx_InitString(__pyx_string_tab[116], &__pyx_n_s_walk) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  if (__Pyx_InitString(__pyx_string_tab[117], &__pyx_n_s_walker) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  #endif
  #if !CYTHON_USE_MODULE_STATE
  if (__Pyx_InitStrings(__pyx_string_tab) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  #endif
  __pyx_int_14202518 = PyInt_FromLong(14202518L); if (unlikely(!__pyx_int_14202518)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_228799785 = PyInt_FromLong(228799785L); if (unlikely(!__pyx_int_228799785)) __PYX_ERR(0, 1, __pyx_L1_error)
  return 0;
  __pyx_L1_error:;
  return -1;
}
/* #### Code section: init_module ### */

static CYTHON_SMALL_CODE int __Pyx_modinit_global_init_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_variable_export_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_function_export_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_type_init_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_type_import_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_variable_import_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_function_import_code(void); /*proto*/

static int __Pyx_modinit_global_init_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_global_init_code", 0);
  /*--- Global init code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_variable_export_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_variable_export_code", 0);
  /*--- Variable export code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_function_export_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_function_export_code", 0);
  /*--- Function export code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_type_init_code(void) {
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__Pyx_modinit_type_init_code", 0);
  /*--- Type init code ---*/
  __pyx_t_1 = PyImport_ImportModule("aim.storage.container"); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_3aim_7storage_9container_Container = __Pyx_ImportType(__pyx_t_1, "aim.storage.container", "Container", sizeof(struct __pyx_obj_3aim_7storage_9container_Container), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_3aim_7storage_9container_Container) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_vtabptr_3aim_7storage_9container_Container = (struct __pyx_vtabstruct_3aim_7storage_9container_Container*)__Pyx_GetVtable(__pyx_ptype_3aim_7storage_9container_Container); if (unlikely(!__pyx_vtabptr_3aim_7storage_9container_Container)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_vtabptr_3aim_7storage_10prefixview_PrefixView = &__pyx_vtable_3aim_7storage_10prefixview_PrefixView;
  __pyx_vtable_3aim_7storage_10prefixview_PrefixView.__pyx_base = *__pyx_vtabptr_3aim_7storage_9container_Container;
  __pyx_vtable_3aim_7storage_10prefixview_PrefixView.__pyx_base.close = (void (*)(struct __pyx_obj_3aim_7storage_9container_Container *, int __pyx_skip_dispatch))__pyx_f_3aim_7storage_10prefixview_10PrefixView_close;
  __pyx_vtable_3aim_7storage_10prefixview_PrefixView.__pyx_base.preload = (void (*)(struct __pyx_obj_3aim_7storage_9container_Container *, int __pyx_skip_dispatch))__pyx_f_3aim_7storage_10prefixview_10PrefixView_preload;
  __pyx_vtable_3aim_7storage_10prefixview_PrefixView.__pyx_base.get = (PyObject *(*)(struct __pyx_obj_3aim_7storage_9container_Container *, PyObject *, int __pyx_skip_dispatch, struct __pyx_opt_args_3aim_7storage_9container_9Container_get *__pyx_optional_args))__pyx_f_3aim_7storage_10prefixview_10PrefixView_get;
  __pyx_vtable_3aim_7storage_10prefixview_PrefixView.__pyx_base.set = (void (*)(struct __pyx_obj_3aim_7storage_9container_Container *, PyObject *, PyObject *, int __pyx_skip_dispatch, struct __pyx_opt_args_3aim_7storage_9container_9Container_set *__pyx_optional_args))__pyx_f_3aim_7storage_10prefixview_10PrefixView_set;
  __pyx_vtable_3aim_7storage_10prefixview_PrefixView.__pyx_base.delete_range = (void (*)(struct __pyx_obj_3aim_7storage_9container_Container *, PyObject *, PyObject *, int __pyx_skip_dispatch, struct __pyx_opt_args_3aim_7storage_9container_9Container_delete_range *__pyx_optional_args))__pyx_f_3aim_7storage_10prefixview_10PrefixView_delete_range;
  __pyx_vtable_3aim_7storage_10prefixview_PrefixView.__pyx_base.items = (struct __pyx_obj_3aim_7storage_9container_ContainerItemsIterator *(*)(struct __pyx_obj_3aim_7storage_9container_Container *, int __pyx_skip_dispatch, struct __pyx_opt_args_3aim_7storage_9container_9Container_items *__pyx_optional_args))__pyx_f_3aim_7storage_10prefixview_10PrefixView_items;
  __pyx_vtable_3aim_7storage_10prefixview_PrefixView.__pyx_base.next_item = (PyObject *(*)(struct __pyx_obj_3aim_7storage_9container_Container *, int __pyx_skip_dispatch, struct __pyx_opt_args_3aim_7storage_9container_9Container_next_item *__pyx_optional_args))__pyx_f_3aim_7storage_10prefixview_10PrefixView_next_item;
  __pyx_vtable_3aim_7storage_10prefixview_PrefixView.__pyx_base.prev_item = (PyObject *(*)(struct __pyx_obj_3aim_7storage_9container_Container *, int __pyx_skip_dispatch, struct __pyx_opt_args_3aim_7storage_9container_9Container_prev_item *__pyx_optional_args))__pyx_f_3aim_7storage_10prefixview_10PrefixView_prev_item;
  __pyx_vtable_3aim_7storage_10prefixview_PrefixView.absolute_path = (PyObject *(*)(struct __pyx_obj_3aim_7storage_10prefixview_PrefixView *, int __pyx_skip_dispatch, struct __pyx_opt_args_3aim_7storage_10prefixview_10PrefixView_absolute_path *__pyx_optional_args))__pyx_f_3aim_7storage_10prefixview_10PrefixView_absolute_path;
  __pyx_vtable_3aim_7storage_10prefixview_PrefixView.tree = (struct __pyx_obj_3aim_7storage_17containertreeview_ContainerTreeView *(*)(struct __pyx_obj_3aim_7storage_10prefixview_PrefixView *, int __pyx_skip_dispatch))__pyx_f_3aim_7storage_10prefixview_10PrefixView_tree;
  #if CYTHON_USE_TYPE_SPECS
  __pyx_t_2 = PyTuple_Pack(1, (PyObject *)__pyx_ptype_3aim_7storage_9container_Container); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_ptype_3aim_7storage_10prefixview_PrefixView = (PyTypeObject *) __Pyx_PyType_FromModuleAndSpec(__pyx_m, &__pyx_type_3aim_7storage_10prefixview_PrefixView_spec, __pyx_t_2);
  __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (unlikely(!__pyx_ptype_3aim_7storage_10prefixview_PrefixView)) __PYX_ERR(0, 7, __pyx_L1_error)
  if (__Pyx_fix_up_extension_type_from_spec(&__pyx_type_3aim_7storage_10prefixview_PrefixView_spec, __pyx_ptype_3aim_7storage_10prefixview_PrefixView) < 0) __PYX_ERR(0, 7, __pyx_L1_error)
  #else
  __pyx_ptype_3aim_7storage_10prefixview_PrefixView = &__pyx_type_3aim_7storage_10prefixview_PrefixView;
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  __pyx_ptype_3aim_7storage_10prefixview_PrefixView->tp_base = __pyx_ptype_3aim_7storage_9container_Container;
  #endif
  #if !CYTHON_USE_TYPE_SPECS
  if (__Pyx_PyType_Ready(__pyx_ptype_3aim_7storage_10prefixview_PrefixView) < 0) __PYX_ERR(0, 7, __pyx_L1_error)
  #endif
  #if PY_MAJOR_VERSION < 3
  __pyx_ptype_3aim_7storage_10prefixview_PrefixView->tp_print = 0;
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_ptype_3aim_7storage_10prefixview_PrefixView->tp_dictoffset && __pyx_ptype_3aim_7storage_10prefixview_PrefixView->tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_ptype_3aim_7storage_10prefixview_PrefixView->tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON
  {
    PyObject *wrapper = PyObject_GetAttrString((PyObject *)__pyx_ptype_3aim_7storage_10prefixview_PrefixView, "__getitem__"); if (unlikely(!wrapper)) __PYX_ERR(0, 7, __pyx_L1_error)
    if (__Pyx_IS_TYPE(wrapper, &PyWrapperDescr_Type)) {
      __pyx_wrapperbase_3aim_7storage_10prefixview_10PrefixView_12__getitem__ = *((PyWrapperDescrObject *)wrapper)->d_base;
      __pyx_wrapperbase_3aim_7storage_10prefixview_10PrefixView_12__getitem__.doc = __pyx_doc_3aim_7storage_10prefixview_10PrefixView_12__getitem__;
      ((PyWrapperDescrObject *)wrapper)->d_base = &__pyx_wrapperbase_3aim_7storage_10prefixview_10PrefixView_12__getitem__;
    }
  }
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON
  {
    PyObject *wrapper = PyObject_GetAttrString((PyObject *)__pyx_ptype_3aim_7storage_10prefixview_PrefixView, "__setitem__"); if (unlikely(!wrapper)) __PYX_ERR(0, 7, __pyx_L1_error)
    if (__Pyx_IS_TYPE(wrapper, &PyWrapperDescr_Type)) {
      __pyx_wrapperbase_3aim_7storage_10prefixview_10PrefixView_16__setitem__ = *((PyWrapperDescrObject *)wrapper)->d_base;
      __pyx_wrapperbase_3aim_7storage_10prefixview_10PrefixView_16__setitem__.doc = __pyx_doc_3aim_7storage_10prefixview_10PrefixView_16__setitem__;
      ((PyWrapperDescrObject *)wrapper)->d_base = &__pyx_wrapperbase_3aim_7storage_10prefixview_10PrefixView_16__setitem__;
    }
  }
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON
  {
    PyObject *wrapper = PyObject_GetAttrString((PyObject *)__pyx_ptype_3aim_7storage_10prefixview_PrefixView, "__delitem__"); if (unlikely(!wrapper)) __PYX_ERR(0, 7, __pyx_L1_error)
    if (__Pyx_IS_TYPE(wrapper, &PyWrapperDescr_Type)) {
      __pyx_wrapperbase_3aim_7storage_10prefixview_10PrefixView_20__delitem__ = *((PyWrapperDescrObject *)wrapper)->d_base;
      __pyx_wrapperbase_3aim_7storage_10prefixview_10PrefixView_20__delitem__.doc = __pyx_doc_3aim_7storage_10prefixview_10PrefixView_20__delitem__;
      ((PyWrapperDescrObject *)wrapper)->d_base = &__pyx_wrapperbase_3aim_7storage_10prefixview_10PrefixView_20__delitem__;
    }
  }
  #endif
  if (__Pyx_SetVtable(__pyx_ptype_3aim_7storage_10prefixview_PrefixView, __pyx_vtabptr_3aim_7storage_10prefixview_PrefixView) < 0) __PYX_ERR(0, 7, __pyx_L1_error)
  #if !CYTHON_COMPILING_IN_LIMITED_API
  if (__Pyx_MergeVtables(__pyx_ptype_3aim_7storage_10prefixview_PrefixView) < 0) __PYX_ERR(0, 7, __pyx_L1_error)
  #endif
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_PrefixView, (PyObject *) __pyx_ptype_3aim_7storage_10prefixview_PrefixView) < 0) __PYX_ERR(0, 7, __pyx_L1_error)
  if (__pyx_ptype_3aim_7storage_10prefixview_PrefixView->tp_weaklistoffset == 0) __pyx_ptype_3aim_7storage_10prefixview_PrefixView->tp_weaklistoffset = offsetof(struct __pyx_obj_3aim_7storage_10prefixview_PrefixView, __pyx_base.__weakref__);
  #if !CYTHON_COMPILING_IN_LIMITED_API
  if (__Pyx_setup_reduce((PyObject *) __pyx_ptype_3aim_7storage_10prefixview_PrefixView) < 0) __PYX_ERR(0, 7, __pyx_L1_error)
  #endif
  __pyx_ptype_3aim_7storage_9container_ContainerItemsIterator = __Pyx_ImportType(__pyx_t_1, "aim.storage.container", "ContainerItemsIterator", sizeof(struct __pyx_obj_3aim_7storage_9container_ContainerItemsIterator), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_3aim_7storage_9container_ContainerItemsIterator) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_vtabptr_3aim_7storage_9container_ContainerItemsIterator = (struct __pyx_vtabstruct_3aim_7storage_9container_ContainerItemsIterator*)__Pyx_GetVtable(__pyx_ptype_3aim_7storage_9container_ContainerItemsIterator); if (unlikely(!__pyx_vtabptr_3aim_7storage_9container_ContainerItemsIterator)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_vtabptr_3aim_7storage_10prefixview_PrefixViewItemsIterator = &__pyx_vtable_3aim_7storage_10prefixview_PrefixViewItemsIterator;
  __pyx_vtable_3aim_7storage_10prefixview_PrefixViewItemsIterator.__pyx_base = *__pyx_vtabptr_3aim_7storage_9container_ContainerItemsIterator;
  __pyx_vtable_3aim_7storage_10prefixview_PrefixViewItemsIterator.__pyx_base.__pyx_base.next = (PyObject *(*)(struct __pyx_obj_8aimrocks_11lib_rocksdb_Iterator *, int __pyx_skip_dispatch))__pyx_f_3aim_7storage_10prefixview_23PrefixViewItemsIterator_next;
  #if CYTHON_USE_TYPE_SPECS
  __pyx_t_2 = PyTuple_Pack(1, (PyObject *)__pyx_ptype_3aim_7storage_9container_ContainerItemsIterator); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 328, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_ptype_3aim_7storage_10prefixview_PrefixViewItemsIterator = (PyTypeObject *) __Pyx_PyType_FromModuleAndSpec(__pyx_m, &__pyx_type_3aim_7storage_10prefixview_PrefixViewItemsIterator_spec, __pyx_t_2);
  __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (unlikely(!__pyx_ptype_3aim_7storage_10prefixview_PrefixViewItemsIterator)) __PYX_ERR(0, 328, __pyx_L1_error)
  if (__Pyx_fix_up_extension_type_from_spec(&__pyx_type_3aim_7storage_10prefixview_PrefixViewItemsIterator_spec, __pyx_ptype_3aim_7storage_10prefixview_PrefixViewItemsIterator) < 0) __PYX_ERR(0, 328, __pyx_L1_error)
  #else
  __pyx_ptype_3aim_7storage_10prefixview_PrefixViewItemsIterator = &__pyx_type_3aim_7storage_10prefixview_PrefixViewItemsIterator;
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  __pyx_ptype_3aim_7storage_10prefixview_PrefixViewItemsIterator->tp_base = __pyx_ptype_3aim_7storage_9container_ContainerItemsIterator;
  #endif
  #if !CYTHON_USE_TYPE_SPECS
  if (__Pyx_PyType_Ready(__pyx_ptype_3aim_7storage_10prefixview_PrefixViewItemsIterator) < 0) __PYX_ERR(0, 328, __pyx_L1_error)
  #endif
  #if PY_MAJOR_VERSION < 3
  __pyx_ptype_3aim_7storage_10prefixview_PrefixViewItemsIterator->tp_print = 0;
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_ptype_3aim_7storage_10prefixview_PrefixViewItemsIterator->tp_dictoffset && __pyx_ptype_3aim_7storage_10prefixview_PrefixViewItemsIterator->tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_ptype_3aim_7storage_10prefixview_PrefixViewItemsIterator->tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  #endif
  if (__Pyx_SetVtable(__pyx_ptype_3aim_7storage_10prefixview_PrefixViewItemsIterator, __pyx_vtabptr_3aim_7storage_10prefixview_PrefixViewItemsIterator) < 0) __PYX_ERR(0, 328, __pyx_L1_error)
  #if !CYTHON_COMPILING_IN_LIMITED_API
  if (__Pyx_MergeVtables(__pyx_ptype_3aim_7storage_10prefixview_PrefixViewItemsIterator) < 0) __PYX_ERR(0, 328, __pyx_L1_error)
  #endif
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_PrefixViewItemsIterator, (PyObject *) __pyx_ptype_3aim_7storage_10prefixview_PrefixViewItemsIterator) < 0) __PYX_ERR(0, 328, __pyx_L1_error)
  #if !CYTHON_COMPILING_IN_LIMITED_API
  if (__Pyx_setup_reduce((PyObject *) __pyx_ptype_3aim_7storage_10prefixview_PrefixViewItemsIterator) < 0) __PYX_ERR(0, 328, __pyx_L1_error)
  #endif
  #if CYTHON_USE_TYPE_SPECS
  __pyx_ptype_3aim_7storage_10prefixview___pyx_scope_struct__walk = (PyTypeObject *) __Pyx_PyType_FromModuleAndSpec(__pyx_m, &__pyx_type_3aim_7storage_10prefixview___pyx_scope_struct__walk_spec, NULL); if (unlikely(!__pyx_ptype_3aim_7storage_10prefixview___pyx_scope_struct__walk)) __PYX_ERR(0, 196, __pyx_L1_error)
  if (__Pyx_fix_up_extension_type_from_spec(&__pyx_type_3aim_7storage_10prefixview___pyx_scope_struct__walk_spec, __pyx_ptype_3aim_7storage_10prefixview___pyx_scope_struct__walk) < 0) __PYX_ERR(0, 196, __pyx_L1_error)
  #else
  __pyx_ptype_3aim_7storage_10prefixview___pyx_scope_struct__walk = &__pyx_type_3aim_7storage_10prefixview___pyx_scope_struct__walk;
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  #endif
  #if !CYTHON_USE_TYPE_SPECS
  if (__Pyx_PyType_Ready(__pyx_ptype_3aim_7storage_10prefixview___pyx_scope_struct__walk) < 0) __PYX_ERR(0, 196, __pyx_L1_error)
  #endif
  #if PY_MAJOR_VERSION < 3
  __pyx_ptype_3aim_7storage_10prefixview___pyx_scope_struct__walk->tp_print = 0;
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_ptype_3aim_7storage_10prefixview___pyx_scope_struct__walk->tp_dictoffset && __pyx_ptype_3aim_7storage_10prefixview___pyx_scope_struct__walk->tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_ptype_3aim_7storage_10prefixview___pyx_scope_struct__walk->tp_getattro = __Pyx_PyObject_GenericGetAttrNoDict;
  }
  #endif
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_modinit_type_import_code(void) {
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__Pyx_modinit_type_import_code", 0);
  /*--- Type import code ---*/
  __pyx_t_1 = PyImport_ImportModule(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_t_1)) __PYX_ERR(4, 9, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_7cpython_4type_type = __Pyx_ImportType(__pyx_t_1, __Pyx_BUILTIN_MODULE_NAME, "type", 
  #if defined(PYPY_VERSION_NUM) && PYPY_VERSION_NUM < 0x050B0000
  sizeof(PyTypeObject),
  #elif CYTHON_COMPILING_IN_LIMITED_API
  sizeof(PyTypeObject),
  #else
  sizeof(PyHeapTypeObject),
  #endif
  __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_7cpython_4type_type) __PYX_ERR(4, 9, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_t_1)) __PYX_ERR(5, 8, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_7cpython_4bool_bool = __Pyx_ImportType(__pyx_t_1, __Pyx_BUILTIN_MODULE_NAME, "bool", sizeof(PyBoolObject), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_7cpython_4bool_bool) __PYX_ERR(5, 8, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 15, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_7cpython_7complex_complex = __Pyx_ImportType(__pyx_t_1, __Pyx_BUILTIN_MODULE_NAME, "complex", sizeof(PyComplexObject), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_7cpython_7complex_complex) __PYX_ERR(6, 15, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule("aimrocks.lib_rocksdb"); if (unlikely(!__pyx_t_1)) __PYX_ERR(7, 70, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_8aimrocks_11lib_rocksdb_PyComparator = __Pyx_ImportType(__pyx_t_1, "aimrocks.lib_rocksdb", "PyComparator", sizeof(struct __pyx_obj_8aimrocks_11lib_rocksdb_PyComparator), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_8aimrocks_11lib_rocksdb_PyComparator) __PYX_ERR(7, 70, __pyx_L1_error)
  __pyx_vtabptr_8aimrocks_11lib_rocksdb_PyComparator = (struct __pyx_vtabstruct_8aimrocks_11lib_rocksdb_PyComparator*)__Pyx_GetVtable(__pyx_ptype_8aimrocks_11lib_rocksdb_PyComparator); if (unlikely(!__pyx_vtabptr_8aimrocks_11lib_rocksdb_PyComparator)) __PYX_ERR(7, 70, __pyx_L1_error)
  __pyx_ptype_8aimrocks_11lib_rocksdb_PyGenericComparator = __Pyx_ImportType(__pyx_t_1, "aimrocks.lib_rocksdb", "PyGenericComparator", sizeof(struct __pyx_obj_8aimrocks_11lib_rocksdb_PyGenericComparator), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_8aimrocks_11lib_rocksdb_PyGenericComparator) __PYX_ERR(7, 77, __pyx_L1_error)
  __pyx_vtabptr_8aimrocks_11lib_rocksdb_PyGenericComparator = (struct __pyx_vtabstruct_8aimrocks_11lib_rocksdb_PyGenericComparator*)__Pyx_GetVtable(__pyx_ptype_8aimrocks_11lib_rocksdb_PyGenericComparator); if (unlikely(!__pyx_vtabptr_8aimrocks_11lib_rocksdb_PyGenericComparator)) __PYX_ERR(7, 77, __pyx_L1_error)
  __pyx_ptype_8aimrocks_11lib_rocksdb_PyBytewiseComparator = __Pyx_ImportType(__pyx_t_1, "aimrocks.lib_rocksdb", "PyBytewiseComparator", sizeof(struct __pyx_obj_8aimrocks_11lib_rocksdb_PyBytewiseComparator), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_8aimrocks_11lib_rocksdb_PyBytewiseComparator) __PYX_ERR(7, 86, __pyx_L1_error)
  __pyx_vtabptr_8aimrocks_11lib_rocksdb_PyBytewiseComparator = (struct __pyx_vtabstruct_8aimrocks_11lib_rocksdb_PyBytewiseComparator*)__Pyx_GetVtable(__pyx_ptype_8aimrocks_11lib_rocksdb_PyBytewiseComparator); if (unlikely(!__pyx_vtabptr_8aimrocks_11lib_rocksdb_PyBytewiseComparator)) __PYX_ERR(7, 86, __pyx_L1_error)
  __pyx_ptype_8aimrocks_11lib_rocksdb_PyFilterPolicy = __Pyx_ImportType(__pyx_t_1, "aimrocks.lib_rocksdb", "PyFilterPolicy", sizeof(struct __pyx_obj_8aimrocks_11lib_rocksdb_PyFilterPolicy), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_8aimrocks_11lib_rocksdb_PyFilterPolicy) __PYX_ERR(7, 108, __pyx_L1_error)
  __pyx_vtabptr_8aimrocks_11lib_rocksdb_PyFilterPolicy = (struct __pyx_vtabstruct_8aimrocks_11lib_rocksdb_PyFilterPolicy*)__Pyx_GetVtable(__pyx_ptype_8aimrocks_11lib_rocksdb_PyFilterPolicy); if (unlikely(!__pyx_vtabptr_8aimrocks_11lib_rocksdb_PyFilterPolicy)) __PYX_ERR(7, 108, __pyx_L1_error)
  __pyx_ptype_8aimrocks_11lib_rocksdb_PyGenericFilterPolicy = __Pyx_ImportType(__pyx_t_1, "aimrocks.lib_rocksdb", "PyGenericFilterPolicy", sizeof(struct __pyx_obj_8aimrocks_11lib_rocksdb_PyGenericFilterPolicy), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_8aimrocks_11lib_rocksdb_PyGenericFilterPolicy) __PYX_ERR(7, 113, __pyx_L1_error)
  __pyx_vtabptr_8aimrocks_11lib_rocksdb_PyGenericFilterPolicy = (struct __pyx_vtabstruct_8aimrocks_11lib_rocksdb_PyGenericFilterPolicy*)__Pyx_GetVtable(__pyx_ptype_8aimrocks_11lib_rocksdb_PyGenericFilterPolicy); if (unlikely(!__pyx_vtabptr_8aimrocks_11lib_rocksdb_PyGenericFilterPolicy)) __PYX_ERR(7, 113, __pyx_L1_error)
  __pyx_ptype_8aimrocks_11lib_rocksdb_PyBloomFilterPolicy = __Pyx_ImportType(__pyx_t_1, "aimrocks.lib_rocksdb", "PyBloomFilterPolicy", sizeof(struct __pyx_obj_8aimrocks_11lib_rocksdb_PyBloomFilterPolicy), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_8aimrocks_11lib_rocksdb_PyBloomFilterPolicy) __PYX_ERR(7, 137, __pyx_L1_error)
  __pyx_vtabptr_8aimrocks_11lib_rocksdb_PyBloomFilterPolicy = (struct __pyx_vtabstruct_8aimrocks_11lib_rocksdb_PyBloomFilterPolicy*)__Pyx_GetVtable(__pyx_ptype_8aimrocks_11lib_rocksdb_PyBloomFilterPolicy); if (unlikely(!__pyx_vtabptr_8aimrocks_11lib_rocksdb_PyBloomFilterPolicy)) __PYX_ERR(7, 137, __pyx_L1_error)
  __pyx_ptype_8aimrocks_11lib_rocksdb_PyMergeOperator = __Pyx_ImportType(__pyx_t_1, "aimrocks.lib_rocksdb", "PyMergeOperator", sizeof(struct __pyx_obj_8aimrocks_11lib_rocksdb_PyMergeOperator), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_8aimrocks_11lib_rocksdb_PyMergeOperator) __PYX_ERR(7, 152, __pyx_L1_error)
  __pyx_vtabptr_8aimrocks_11lib_rocksdb_PyMergeOperator = (struct __pyx_vtabstruct_8aimrocks_11lib_rocksdb_PyMergeOperator*)__Pyx_GetVtable(__pyx_ptype_8aimrocks_11lib_rocksdb_PyMergeOperator); if (unlikely(!__pyx_vtabptr_8aimrocks_11lib_rocksdb_PyMergeOperator)) __PYX_ERR(7, 152, __pyx_L1_error)
  __pyx_ptype_8aimrocks_11lib_rocksdb_PyCache = __Pyx_ImportType(__pyx_t_1, "aimrocks.lib_rocksdb", "PyCache", sizeof(struct __pyx_obj_8aimrocks_11lib_rocksdb_PyCache), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_8aimrocks_11lib_rocksdb_PyCache) __PYX_ERR(7, 183, __pyx_L1_error)
  __pyx_vtabptr_8aimrocks_11lib_rocksdb_PyCache = (struct __pyx_vtabstruct_8aimrocks_11lib_rocksdb_PyCache*)__Pyx_GetVtable(__pyx_ptype_8aimrocks_11lib_rocksdb_PyCache); if (unlikely(!__pyx_vtabptr_8aimrocks_11lib_rocksdb_PyCache)) __PYX_ERR(7, 183, __pyx_L1_error)
  __pyx_ptype_8aimrocks_11lib_rocksdb_PyLRUCache = __Pyx_ImportType(__pyx_t_1, "aimrocks.lib_rocksdb", "PyLRUCache", sizeof(struct __pyx_obj_8aimrocks_11lib_rocksdb_PyLRUCache), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_8aimrocks_11lib_rocksdb_PyLRUCache) __PYX_ERR(7, 186, __pyx_L1_error)
  __pyx_vtabptr_8aimrocks_11lib_rocksdb_PyLRUCache = (struct __pyx_vtabstruct_8aimrocks_11lib_rocksdb_PyLRUCache*)__Pyx_GetVtable(__pyx_ptype_8aimrocks_11lib_rocksdb_PyLRUCache); if (unlikely(!__pyx_vtabptr_8aimrocks_11lib_rocksdb_PyLRUCache)) __PYX_ERR(7, 186, __pyx_L1_error)
  __pyx_ptype_8aimrocks_11lib_rocksdb_PySliceTransform = __Pyx_ImportType(__pyx_t_1, "aimrocks.lib_rocksdb", "PySliceTransform", sizeof(struct __pyx_obj_8aimrocks_11lib_rocksdb_PySliceTransform), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_8aimrocks_11lib_rocksdb_PySliceTransform) __PYX_ERR(7, 195, __pyx_L1_error)
  __pyx_vtabptr_8aimrocks_11lib_rocksdb_PySliceTransform = (struct __pyx_vtabstruct_8aimrocks_11lib_rocksdb_PySliceTransform*)__Pyx_GetVtable(__pyx_ptype_8aimrocks_11lib_rocksdb_PySliceTransform); if (unlikely(!__pyx_vtabptr_8aimrocks_11lib_rocksdb_PySliceTransform)) __PYX_ERR(7, 195, __pyx_L1_error)
  __pyx_ptype_8aimrocks_11lib_rocksdb_PyTableFactory = __Pyx_ImportType(__pyx_t_1, "aimrocks.lib_rocksdb", "PyTableFactory", sizeof(struct __pyx_obj_8aimrocks_11lib_rocksdb_PyTableFactory), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_8aimrocks_11lib_rocksdb_PyTableFactory) __PYX_ERR(7, 223, __pyx_L1_error)
  __pyx_vtabptr_8aimrocks_11lib_rocksdb_PyTableFactory = (struct __pyx_vtabstruct_8aimrocks_11lib_rocksdb_PyTableFactory*)__Pyx_GetVtable(__pyx_ptype_8aimrocks_11lib_rocksdb_PyTableFactory); if (unlikely(!__pyx_vtabptr_8aimrocks_11lib_rocksdb_PyTableFactory)) __PYX_ERR(7, 223, __pyx_L1_error)
  __pyx_ptype_8aimrocks_11lib_rocksdb_BlockBasedTableFactory = __Pyx_ImportType(__pyx_t_1, "aimrocks.lib_rocksdb", "BlockBasedTableFactory", sizeof(struct __pyx_obj_8aimrocks_11lib_rocksdb_BlockBasedTableFactory), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_8aimrocks_11lib_rocksdb_BlockBasedTableFactory) __PYX_ERR(7, 228, __pyx_L1_error)
  __pyx_vtabptr_8aimrocks_11lib_rocksdb_BlockBasedTableFactory = (struct __pyx_vtabstruct_8aimrocks_11lib_rocksdb_BlockBasedTableFactory*)__Pyx_GetVtable(__pyx_ptype_8aimrocks_11lib_rocksdb_BlockBasedTableFactory); if (unlikely(!__pyx_vtabptr_8aimrocks_11lib_rocksdb_BlockBasedTableFactory)) __PYX_ERR(7, 228, __pyx_L1_error)
  __pyx_ptype_8aimrocks_11lib_rocksdb_PlainTableFactory = __Pyx_ImportType(__pyx_t_1, "aimrocks.lib_rocksdb", "PlainTableFactory", sizeof(struct __pyx_obj_8aimrocks_11lib_rocksdb_PlainTableFactory), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_8aimrocks_11lib_rocksdb_PlainTableFactory) __PYX_ERR(7, 244, __pyx_L1_error)
  __pyx_vtabptr_8aimrocks_11lib_rocksdb_PlainTableFactory = (struct __pyx_vtabstruct_8aimrocks_11lib_rocksdb_PlainTableFactory*)__Pyx_GetVtable(__pyx_ptype_8aimrocks_11lib_rocksdb_PlainTableFactory); if (unlikely(!__pyx_vtabptr_8aimrocks_11lib_rocksdb_PlainTableFactory)) __PYX_ERR(7, 244, __pyx_L1_error)
  __pyx_ptype_8aimrocks_11lib_rocksdb_PyMemtableFactory = __Pyx_ImportType(__pyx_t_1, "aimrocks.lib_rocksdb", "PyMemtableFactory", sizeof(struct __pyx_obj_8aimrocks_11lib_rocksdb_PyMemtableFactory), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_8aimrocks_11lib_rocksdb_PyMemtableFactory) __PYX_ERR(7, 258, __pyx_L1_error)
  __pyx_vtabptr_8aimrocks_11lib_rocksdb_PyMemtableFactory = (struct __pyx_vtabstruct_8aimrocks_11lib_rocksdb_PyMemtableFactory*)__Pyx_GetVtable(__pyx_ptype_8aimrocks_11lib_rocksdb_PyMemtableFactory); if (unlikely(!__pyx_vtabptr_8aimrocks_11lib_rocksdb_PyMemtableFactory)) __PYX_ERR(7, 258, __pyx_L1_error)
  __pyx_ptype_8aimrocks_11lib_rocksdb_SkipListMemtableFactory = __Pyx_ImportType(__pyx_t_1, "aimrocks.lib_rocksdb", "SkipListMemtableFactory", sizeof(struct __pyx_obj_8aimrocks_11lib_rocksdb_SkipListMemtableFactory), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_8aimrocks_11lib_rocksdb_SkipListMemtableFactory) __PYX_ERR(7, 262, __pyx_L1_error)
  __pyx_vtabptr_8aimrocks_11lib_rocksdb_SkipListMemtableFactory = (struct __pyx_vtabstruct_8aimrocks_11lib_rocksdb_SkipListMemtableFactory*)__Pyx_GetVtable(__pyx_ptype_8aimrocks_11lib_rocksdb_SkipListMemtableFactory); if (unlikely(!__pyx_vtabptr_8aimrocks_11lib_rocksdb_SkipListMemtableFactory)) __PYX_ERR(7, 262, __pyx_L1_error)
  __pyx_ptype_8aimrocks_11lib_rocksdb_VectorMemtableFactory = __Pyx_ImportType(__pyx_t_1, "aimrocks.lib_rocksdb", "VectorMemtableFactory", sizeof(struct __pyx_obj_8aimrocks_11lib_rocksdb_VectorMemtableFactory), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_8aimrocks_11lib_rocksdb_VectorMemtableFactory) __PYX_ERR(7, 266, __pyx_L1_error)
  __pyx_vtabptr_8aimrocks_11lib_rocksdb_VectorMemtableFactory = (struct __pyx_vtabstruct_8aimrocks_11lib_rocksdb_VectorMemtableFactory*)__Pyx_GetVtable(__pyx_ptype_8aimrocks_11lib_rocksdb_VectorMemtableFactory); if (unlikely(!__pyx_vtabptr_8aimrocks_11lib_rocksdb_VectorMemtableFactory)) __PYX_ERR(7, 266, __pyx_L1_error)
  __pyx_ptype_8aimrocks_11lib_rocksdb_HashSkipListMemtableFactory = __Pyx_ImportType(__pyx_t_1, "aimrocks.lib_rocksdb", "HashSkipListMemtableFactory", sizeof(struct __pyx_obj_8aimrocks_11lib_rocksdb_HashSkipListMemtableFactory), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_8aimrocks_11lib_rocksdb_HashSkipListMemtableFactory) __PYX_ERR(7, 270, __pyx_L1_error)
  __pyx_vtabptr_8aimrocks_11lib_rocksdb_HashSkipListMemtableFactory = (struct __pyx_vtabstruct_8aimrocks_11lib_rocksdb_HashSkipListMemtableFactory*)__Pyx_GetVtable(__pyx_ptype_8aimrocks_11lib_rocksdb_HashSkipListMemtableFactory); if (unlikely(!__pyx_vtabptr_8aimrocks_11lib_rocksdb_HashSkipListMemtableFactory)) __PYX_ERR(7, 270, __pyx_L1_error)
  __pyx_ptype_8aimrocks_11lib_rocksdb_HashLinkListMemtableFactory = __Pyx_ImportType(__pyx_t_1, "aimrocks.lib_rocksdb", "HashLinkListMemtableFactory", sizeof(struct __pyx_obj_8aimrocks_11lib_rocksdb_HashLinkListMemtableFactory), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_8aimrocks_11lib_rocksdb_HashLinkListMemtableFactory) __PYX_ERR(7, 278, __pyx_L1_error)
  __pyx_vtabptr_8aimrocks_11lib_rocksdb_HashLinkListMemtableFactory = (struct __pyx_vtabstruct_8aimrocks_11lib_rocksdb_HashLinkListMemtableFactory*)__Pyx_GetVtable(__pyx_ptype_8aimrocks_11lib_rocksdb_HashLinkListMemtableFactory); if (unlikely(!__pyx_vtabptr_8aimrocks_11lib_rocksdb_HashLinkListMemtableFactory)) __PYX_ERR(7, 278, __pyx_L1_error)
  __pyx_ptype_8aimrocks_11lib_rocksdb_CompressionType = __Pyx_ImportType(__pyx_t_1, "aimrocks.lib_rocksdb", "CompressionType", sizeof(struct __pyx_obj_8aimrocks_11lib_rocksdb_CompressionType), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_8aimrocks_11lib_rocksdb_CompressionType) __PYX_ERR(7, 284, __pyx_L1_error)
  __pyx_ptype_8aimrocks_11lib_rocksdb_CompactionPri = __Pyx_ImportType(__pyx_t_1, "aimrocks.lib_rocksdb", "CompactionPri", sizeof(struct __pyx_obj_8aimrocks_11lib_rocksdb_CompactionPri), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_8aimrocks_11lib_rocksdb_CompactionPri) __PYX_ERR(7, 297, __pyx_L1_error)
  __pyx_ptype_8aimrocks_11lib_rocksdb__ColumnFamilyHandle = __Pyx_ImportType(__pyx_t_1, "aimrocks.lib_rocksdb", "_ColumnFamilyHandle", sizeof(struct __pyx_obj_8aimrocks_11lib_rocksdb__ColumnFamilyHandle), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_8aimrocks_11lib_rocksdb__ColumnFamilyHandle) __PYX_ERR(7, 304, __pyx_L1_error)
  __pyx_vtabptr_8aimrocks_11lib_rocksdb__ColumnFamilyHandle = (struct __pyx_vtabstruct_8aimrocks_11lib_rocksdb__ColumnFamilyHandle*)__Pyx_GetVtable(__pyx_ptype_8aimrocks_11lib_rocksdb__ColumnFamilyHandle); if (unlikely(!__pyx_vtabptr_8aimrocks_11lib_rocksdb__ColumnFamilyHandle)) __PYX_ERR(7, 304, __pyx_L1_error)
  __pyx_ptype_8aimrocks_11lib_rocksdb_ColumnFamilyHandle = __Pyx_ImportType(__pyx_t_1, "aimrocks.lib_rocksdb", "ColumnFamilyHandle", sizeof(struct __pyx_obj_8aimrocks_11lib_rocksdb_ColumnFamilyHandle), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_8aimrocks_11lib_rocksdb_ColumnFamilyHandle) __PYX_ERR(7, 320, __pyx_L1_error)
  __pyx_vtabptr_8aimrocks_11lib_rocksdb_ColumnFamilyHandle = (struct __pyx_vtabstruct_8aimrocks_11lib_rocksdb_ColumnFamilyHandle*)__Pyx_GetVtable(__pyx_ptype_8aimrocks_11lib_rocksdb_ColumnFamilyHandle); if (unlikely(!__pyx_vtabptr_8aimrocks_11lib_rocksdb_ColumnFamilyHandle)) __PYX_ERR(7, 320, __pyx_L1_error)
  __pyx_ptype_8aimrocks_11lib_rocksdb_ColumnFamilyOptions = __Pyx_ImportType(__pyx_t_1, "aimrocks.lib_rocksdb", "ColumnFamilyOptions", sizeof(struct __pyx_obj_8aimrocks_11lib_rocksdb_ColumnFamilyOptions), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_8aimrocks_11lib_rocksdb_ColumnFamilyOptions) __PYX_ERR(7, 342, __pyx_L1_error)
  __pyx_ptype_8aimrocks_11lib_rocksdb_Options = __Pyx_ImportType(__pyx_t_1, "aimrocks.lib_rocksdb", "Options", sizeof(struct __pyx_obj_8aimrocks_11lib_rocksdb_Options), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_8aimrocks_11lib_rocksdb_Options) __PYX_ERR(7, 455, __pyx_L1_error)
  __pyx_ptype_8aimrocks_11lib_rocksdb_WriteBatch = __Pyx_ImportType(__pyx_t_1, "aimrocks.lib_rocksdb", "WriteBatch", sizeof(struct __pyx_obj_8aimrocks_11lib_rocksdb_WriteBatch), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_8aimrocks_11lib_rocksdb_WriteBatch) __PYX_ERR(7, 572, __pyx_L1_error)
  __pyx_vtabptr_8aimrocks_11lib_rocksdb_WriteBatch = (struct __pyx_vtabstruct_8aimrocks_11lib_rocksdb_WriteBatch*)__Pyx_GetVtable(__pyx_ptype_8aimrocks_11lib_rocksdb_WriteBatch); if (unlikely(!__pyx_vtabptr_8aimrocks_11lib_rocksdb_WriteBatch)) __PYX_ERR(7, 572, __pyx_L1_error)
  __pyx_ptype_8aimrocks_11lib_rocksdb_WriteBatchIterator = __Pyx_ImportType(__pyx_t_1, "aimrocks.lib_rocksdb", "WriteBatchIterator", sizeof(struct __pyx_obj_8aimrocks_11lib_rocksdb_WriteBatchIterator), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_8aimrocks_11lib_rocksdb_WriteBatchIterator) __PYX_ERR(7, 586, __pyx_L1_error)
  __pyx_ptype_8aimrocks_11lib_rocksdb_IDB = __Pyx_ImportType(__pyx_t_1, "aimrocks.lib_rocksdb", "IDB", sizeof(struct __pyx_obj_8aimrocks_11lib_rocksdb_IDB), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_8aimrocks_11lib_rocksdb_IDB) __PYX_ERR(7, 596, __pyx_L1_error)
  __pyx_vtabptr_8aimrocks_11lib_rocksdb_IDB = (struct __pyx_vtabstruct_8aimrocks_11lib_rocksdb_IDB*)__Pyx_GetVtable(__pyx_ptype_8aimrocks_11lib_rocksdb_IDB); if (unlikely(!__pyx_vtabptr_8aimrocks_11lib_rocksdb_IDB)) __PYX_ERR(7, 596, __pyx_L1_error)
  __pyx_ptype_8aimrocks_11lib_rocksdb_DB = __Pyx_ImportType(__pyx_t_1, "aimrocks.lib_rocksdb", "DB", sizeof(struct __pyx_obj_8aimrocks_11lib_rocksdb_DB), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_8aimrocks_11lib_rocksdb_DB) __PYX_ERR(7, 612, __pyx_L1_error)
  __pyx_vtabptr_8aimrocks_11lib_rocksdb_DB = (struct __pyx_vtabstruct_8aimrocks_11lib_rocksdb_DB*)__Pyx_GetVtable(__pyx_ptype_8aimrocks_11lib_rocksdb_DB); if (unlikely(!__pyx_vtabptr_8aimrocks_11lib_rocksdb_DB)) __PYX_ERR(7, 612, __pyx_L1_error)
  __pyx_ptype_8aimrocks_11lib_rocksdb_Snapshot = __Pyx_ImportType(__pyx_t_1, "aimrocks.lib_rocksdb", "Snapshot", sizeof(struct __pyx_obj_8aimrocks_11lib_rocksdb_Snapshot), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_8aimrocks_11lib_rocksdb_Snapshot) __PYX_ERR(7, 655, __pyx_L1_error)
  __pyx_ptype_8aimrocks_11lib_rocksdb_Iterator = __Pyx_ImportType(__pyx_t_1, "aimrocks.lib_rocksdb", "Iterator", sizeof(struct __pyx_obj_8aimrocks_11lib_rocksdb_Iterator), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_8aimrocks_11lib_rocksdb_Iterator) __PYX_ERR(7, 663, __pyx_L1_error)
  __pyx_vtabptr_8aimrocks_11lib_rocksdb_Iterator = (struct __pyx_vtabstruct_8aimrocks_11lib_rocksdb_Iterator*)__Pyx_GetVtable(__pyx_ptype_8aimrocks_11lib_rocksdb_Iterator); if (unlikely(!__pyx_vtabptr_8aimrocks_11lib_rocksdb_Iterator)) __PYX_ERR(7, 663, __pyx_L1_error)
  __pyx_ptype_8aimrocks_11lib_rocksdb_BaseIterator = __Pyx_ImportType(__pyx_t_1, "aimrocks.lib_rocksdb", "BaseIterator", sizeof(struct __pyx_obj_8aimrocks_11lib_rocksdb_BaseIterator), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_8aimrocks_11lib_rocksdb_BaseIterator) __PYX_ERR(7, 671, __pyx_L1_error)
  __pyx_vtabptr_8aimrocks_11lib_rocksdb_BaseIterator = (struct __pyx_vtabstruct_8aimrocks_11lib_rocksdb_BaseIterator*)__Pyx_GetVtable(__pyx_ptype_8aimrocks_11lib_rocksdb_BaseIterator); if (unlikely(!__pyx_vtabptr_8aimrocks_11lib_rocksdb_BaseIterator)) __PYX_ERR(7, 671, __pyx_L1_error)
  __pyx_ptype_8aimrocks_11lib_rocksdb_KeysIterator = __Pyx_ImportType(__pyx_t_1, "aimrocks.lib_rocksdb", "KeysIterator", sizeof(struct __pyx_obj_8aimrocks_11lib_rocksdb_KeysIterator), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_8aimrocks_11lib_rocksdb_KeysIterator) __PYX_ERR(7, 687, __pyx_L1_error)
  __pyx_vtabptr_8aimrocks_11lib_rocksdb_KeysIterator = (struct __pyx_vtabstruct_8aimrocks_11lib_rocksdb_KeysIterator*)__Pyx_GetVtable(__pyx_ptype_8aimrocks_11lib_rocksdb_KeysIterator); if (unlikely(!__pyx_vtabptr_8aimrocks_11lib_rocksdb_KeysIterator)) __PYX_ERR(7, 687, __pyx_L1_error)
  __pyx_ptype_8aimrocks_11lib_rocksdb_ValuesIterator = __Pyx_ImportType(__pyx_t_1, "aimrocks.lib_rocksdb", "ValuesIterator", sizeof(struct __pyx_obj_8aimrocks_11lib_rocksdb_ValuesIterator), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_8aimrocks_11lib_rocksdb_ValuesIterator) __PYX_ERR(7, 690, __pyx_L1_error)
  __pyx_vtabptr_8aimrocks_11lib_rocksdb_ValuesIterator = (struct __pyx_vtabstruct_8aimrocks_11lib_rocksdb_ValuesIterator*)__Pyx_GetVtable(__pyx_ptype_8aimrocks_11lib_rocksdb_ValuesIterator); if (unlikely(!__pyx_vtabptr_8aimrocks_11lib_rocksdb_ValuesIterator)) __PYX_ERR(7, 690, __pyx_L1_error)
  __pyx_ptype_8aimrocks_11lib_rocksdb_ItemsIterator = __Pyx_ImportType(__pyx_t_1, "aimrocks.lib_rocksdb", "ItemsIterator", sizeof(struct __pyx_obj_8aimrocks_11lib_rocksdb_ItemsIterator), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_8aimrocks_11lib_rocksdb_ItemsIterator) __PYX_ERR(7, 693, __pyx_L1_error)
  __pyx_vtabptr_8aimrocks_11lib_rocksdb_ItemsIterator = (struct __pyx_vtabstruct_8aimrocks_11lib_rocksdb_ItemsIterator*)__Pyx_GetVtable(__pyx_ptype_8aimrocks_11lib_rocksdb_ItemsIterator); if (unlikely(!__pyx_vtabptr_8aimrocks_11lib_rocksdb_ItemsIterator)) __PYX_ERR(7, 693, __pyx_L1_error)
  __pyx_ptype_8aimrocks_11lib_rocksdb_ReversedIterator = __Pyx_ImportType(__pyx_t_1, "aimrocks.lib_rocksdb", "ReversedIterator", sizeof(struct __pyx_obj_8aimrocks_11lib_rocksdb_ReversedIterator), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_8aimrocks_11lib_rocksdb_ReversedIterator) __PYX_ERR(7, 696, __pyx_L1_error)
  __pyx_vtabptr_8aimrocks_11lib_rocksdb_ReversedIterator = (struct __pyx_vtabstruct_8aimrocks_11lib_rocksdb_ReversedIterator*)__Pyx_GetVtable(__pyx_ptype_8aimrocks_11lib_rocksdb_ReversedIterator); if (unlikely(!__pyx_vtabptr_8aimrocks_11lib_rocksdb_ReversedIterator)) __PYX_ERR(7, 696, __pyx_L1_error)
  __pyx_ptype_8aimrocks_11lib_rocksdb_BackupEngine = __Pyx_ImportType(__pyx_t_1, "aimrocks.lib_rocksdb", "BackupEngine", sizeof(struct __pyx_obj_8aimrocks_11lib_rocksdb_BackupEngine), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_8aimrocks_11lib_rocksdb_BackupEngine) __PYX_ERR(7, 708, __pyx_L1_error)
  __pyx_vtabptr_8aimrocks_11lib_rocksdb_BackupEngine = (struct __pyx_vtabstruct_8aimrocks_11lib_rocksdb_BackupEngine*)__Pyx_GetVtable(__pyx_ptype_8aimrocks_11lib_rocksdb_BackupEngine); if (unlikely(!__pyx_vtabptr_8aimrocks_11lib_rocksdb_BackupEngine)) __PYX_ERR(7, 708, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule("aim.storage.utils"); if (unlikely(!__pyx_t_1)) __PYX_ERR(8, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_3aim_7storage_5utils_ArrayFlagType = __Pyx_ImportType(__pyx_t_1, "aim.storage.utils", "ArrayFlagType", sizeof(struct __pyx_obj_3aim_7storage_5utils_ArrayFlagType), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_3aim_7storage_5utils_ArrayFlagType) __PYX_ERR(8, 6, __pyx_L1_error)
  __pyx_ptype_3aim_7storage_5utils_ObjectFlagType = __Pyx_ImportType(__pyx_t_1, "aim.storage.utils", "ObjectFlagType", sizeof(struct __pyx_obj_3aim_7storage_5utils_ObjectFlagType), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_3aim_7storage_5utils_ObjectFlagType) __PYX_ERR(8, 9, __pyx_L1_error)
  __pyx_ptype_3aim_7storage_5utils_CustomObjectFlagType = __Pyx_ImportType(__pyx_t_1, "aim.storage.utils", "CustomObjectFlagType", sizeof(struct __pyx_obj_3aim_7storage_5utils_CustomObjectFlagType), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_3aim_7storage_5utils_CustomObjectFlagType) __PYX_ERR(8, 12, __pyx_L1_error)
  __pyx_ptype_3aim_7storage_5utils_BLOB = __Pyx_ImportType(__pyx_t_1, "aim.storage.utils", "BLOB", sizeof(struct __pyx_obj_3aim_7storage_5utils_BLOB), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_3aim_7storage_5utils_BLOB) __PYX_ERR(8, 16, __pyx_L1_error)
  __pyx_vtabptr_3aim_7storage_5utils_BLOB = (struct __pyx_vtabstruct_3aim_7storage_5utils_BLOB*)__Pyx_GetVtable(__pyx_ptype_3aim_7storage_5utils_BLOB); if (unlikely(!__pyx_vtabptr_3aim_7storage_5utils_BLOB)) __PYX_ERR(8, 16, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule("aim.storage.treeview"); if (unlikely(!__pyx_t_1)) __PYX_ERR(9, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_3aim_7storage_8treeview_TreeView = __Pyx_ImportType(__pyx_t_1, "aim.storage.treeview", "TreeView", sizeof(struct __pyx_obj_3aim_7storage_8treeview_TreeView), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_3aim_7storage_8treeview_TreeView) __PYX_ERR(9, 4, __pyx_L1_error)
  __pyx_vtabptr_3aim_7storage_8treeview_TreeView = (struct __pyx_vtabstruct_3aim_7storage_8treeview_TreeView*)__Pyx_GetVtable(__pyx_ptype_3aim_7storage_8treeview_TreeView); if (unlikely(!__pyx_vtabptr_3aim_7storage_8treeview_TreeView)) __PYX_ERR(9, 4, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule("aim.storage.containertreeview"); if (unlikely(!__pyx_t_1)) __PYX_ERR(10, 10, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_3aim_7storage_17containertreeview_ContainerTreeView = __Pyx_ImportType(__pyx_t_1, "aim.storage.containertreeview", "ContainerTreeView", sizeof(struct __pyx_obj_3aim_7storage_17containertreeview_ContainerTreeView), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_3aim_7storage_17containertreeview_ContainerTreeView) __PYX_ERR(10, 10, __pyx_L1_error)
  __pyx_vtabptr_3aim_7storage_17containertreeview_ContainerTreeView = (struct __pyx_vtabstruct_3aim_7storage_17containertreeview_ContainerTreeView*)__Pyx_GetVtable(__pyx_ptype_3aim_7storage_17containertreeview_ContainerTreeView); if (unlikely(!__pyx_vtabptr_3aim_7storage_17containertreeview_ContainerTreeView)) __PYX_ERR(10, 10, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_modinit_variable_import_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_variable_import_code", 0);
  /*--- Variable import code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_function_import_code(void) {
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__Pyx_modinit_function_import_code", 0);
  /*--- Function import code ---*/
  __pyx_t_1 = PyImport_ImportModule("aim.storage.encoding.encoding"); if (!__pyx_t_1) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (__Pyx_ImportFunction(__pyx_t_1, "decode", (void (**)(void))&__pyx_f_3aim_7storage_8encoding_8encoding_decode, "PyObject *(PyObject *, int __pyx_skip_dispatch)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_RefNannyFinishContext();
  return -1;
}


#if PY_MAJOR_VERSION >= 3
#if CYTHON_PEP489_MULTI_PHASE_INIT
static PyObject* __pyx_pymod_create(PyObject *spec, PyModuleDef *def); /*proto*/
static int __pyx_pymod_exec_prefixview(PyObject* module); /*proto*/
static PyModuleDef_Slot __pyx_moduledef_slots[] = {
  {Py_mod_create, (void*)__pyx_pymod_create},
  {Py_mod_exec, (void*)__pyx_pymod_exec_prefixview},
  {0, NULL}
};
#endif

#ifdef __cplusplus
namespace {
  struct PyModuleDef __pyx_moduledef =
  #else
  static struct PyModuleDef __pyx_moduledef =
  #endif
  {
      PyModuleDef_HEAD_INIT,
      "prefixview",
      0, /* m_doc */
    #if CYTHON_PEP489_MULTI_PHASE_INIT
      0, /* m_size */
    #elif CYTHON_USE_MODULE_STATE
      sizeof(__pyx_mstate), /* m_size */
    #else
      -1, /* m_size */
    #endif
      __pyx_methods /* m_methods */,
    #if CYTHON_PEP489_MULTI_PHASE_INIT
      __pyx_moduledef_slots, /* m_slots */
    #else
      NULL, /* m_reload */
    #endif
    #if CYTHON_USE_MODULE_STATE
      __pyx_m_traverse, /* m_traverse */
      __pyx_m_clear, /* m_clear */
      NULL /* m_free */
    #else
      NULL, /* m_traverse */
      NULL, /* m_clear */
      NULL /* m_free */
    #endif
  };
  #ifdef __cplusplus
} /* anonymous namespace */
#endif
#endif

#ifndef CYTHON_NO_PYINIT_EXPORT
#define __Pyx_PyMODINIT_FUNC PyMODINIT_FUNC
#elif PY_MAJOR_VERSION < 3
#ifdef __cplusplus
#define __Pyx_PyMODINIT_FUNC extern "C" void
#else
#define __Pyx_PyMODINIT_FUNC void
#endif
#else
#ifdef __cplusplus
#define __Pyx_PyMODINIT_FUNC extern "C" PyObject *
#else
#define __Pyx_PyMODINIT_FUNC PyObject *
#endif
#endif


#if PY_MAJOR_VERSION < 3
__Pyx_PyMODINIT_FUNC initprefixview(void) CYTHON_SMALL_CODE; /*proto*/
__Pyx_PyMODINIT_FUNC initprefixview(void)
#else
__Pyx_PyMODINIT_FUNC PyInit_prefixview(void) CYTHON_SMALL_CODE; /*proto*/
__Pyx_PyMODINIT_FUNC PyInit_prefixview(void)
#if CYTHON_PEP489_MULTI_PHASE_INIT
{
  return PyModuleDef_Init(&__pyx_moduledef);
}
static CYTHON_SMALL_CODE int __Pyx_check_single_interpreter(void) {
    #if PY_VERSION_HEX >= 0x030700A1
    static PY_INT64_T main_interpreter_id = -1;
    PY_INT64_T current_id = PyInterpreterState_GetID(PyThreadState_Get()->interp);
    if (main_interpreter_id == -1) {
        main_interpreter_id = current_id;
        return (unlikely(current_id == -1)) ? -1 : 0;
    } else if (unlikely(main_interpreter_id != current_id))
    #else
    static PyInterpreterState *main_interpreter = NULL;
    PyInterpreterState *current_interpreter = PyThreadState_Get()->interp;
    if (!main_interpreter) {
        main_interpreter = current_interpreter;
    } else if (unlikely(main_interpreter != current_interpreter))
    #endif
    {
        PyErr_SetString(
            PyExc_ImportError,
            "Interpreter change detected - this module can only be loaded into one interpreter per process.");
        return -1;
    }
    return 0;
}
#if CYTHON_COMPILING_IN_LIMITED_API
static CYTHON_SMALL_CODE int __Pyx_copy_spec_to_module(PyObject *spec, PyObject *module, const char* from_name, const char* to_name, int allow_none)
#else
static CYTHON_SMALL_CODE int __Pyx_copy_spec_to_module(PyObject *spec, PyObject *moddict, const char* from_name, const char* to_name, int allow_none)
#endif
{
    PyObject *value = PyObject_GetAttrString(spec, from_name);
    int result = 0;
    if (likely(value)) {
        if (allow_none || value != Py_None) {
#if CYTHON_COMPILING_IN_LIMITED_API
            result = PyModule_AddObject(module, to_name, value);
#else
            result = PyDict_SetItemString(moddict, to_name, value);
#endif
        }
        Py_DECREF(value);
    } else if (PyErr_ExceptionMatches(PyExc_AttributeError)) {
        PyErr_Clear();
    } else {
        result = -1;
    }
    return result;
}
static CYTHON_SMALL_CODE PyObject* __pyx_pymod_create(PyObject *spec, PyModuleDef *def) {
    PyObject *module = NULL, *moddict, *modname;
    CYTHON_UNUSED_VAR(def);
    if (__Pyx_check_single_interpreter())
        return NULL;
    if (__pyx_m)
        return __Pyx_NewRef(__pyx_m);
    modname = PyObject_GetAttrString(spec, "name");
    if (unlikely(!modname)) goto bad;
    module = PyModule_NewObject(modname);
    Py_DECREF(modname);
    if (unlikely(!module)) goto bad;
#if CYTHON_COMPILING_IN_LIMITED_API
    moddict = module;
#else
    moddict = PyModule_GetDict(module);
    if (unlikely(!moddict)) goto bad;
#endif
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "loader", "__loader__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "origin", "__file__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "parent", "__package__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "submodule_search_locations", "__path__", 0) < 0)) goto bad;
    return module;
bad:
    Py_XDECREF(module);
    return NULL;
}


static CYTHON_SMALL_CODE int __pyx_pymod_exec_prefixview(PyObject *__pyx_pyinit_module)
#endif
#endif
{
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannyDeclarations
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  if (__pyx_m) {
    if (__pyx_m == __pyx_pyinit_module) return 0;
    PyErr_SetString(PyExc_RuntimeError, "Module 'prefixview' has already been imported. Re-initialisation is not supported.");
    return -1;
  }
  #elif PY_MAJOR_VERSION >= 3
  if (__pyx_m) return __Pyx_NewRef(__pyx_m);
  #endif
  /*--- Module creation code ---*/
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  __pyx_m = __pyx_pyinit_module;
  Py_INCREF(__pyx_m);
  #else
  #if PY_MAJOR_VERSION < 3
  __pyx_m = Py_InitModule4("prefixview", __pyx_methods, 0, 0, PYTHON_API_VERSION); Py_XINCREF(__pyx_m);
  if (unlikely(!__pyx_m)) __PYX_ERR(0, 1, __pyx_L1_error)
  #elif CYTHON_COMPILING_IN_LIMITED_API
  __pyx_t_1 = PyModule_Create(&__pyx_moduledef); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (unlikely((PyState_AddModule(__pyx_t_1, &__pyx_moduledef) < 0))) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  #else
  __pyx_m = PyModule_Create(&__pyx_moduledef);
  if (unlikely(!__pyx_m)) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #endif
  __pyx_d = PyModule_GetDict(__pyx_m); if (unlikely(!__pyx_d)) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_INCREF(__pyx_d);
  __pyx_b = PyImport_AddModule(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_b)) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_INCREF(__pyx_b);
  __pyx_cython_runtime = PyImport_AddModule((char *) "cython_runtime"); if (unlikely(!__pyx_cython_runtime)) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_INCREF(__pyx_cython_runtime);
  if (PyObject_SetAttrString(__pyx_m, "__builtins__", __pyx_b) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  #if CYTHON_REFNANNY
__Pyx_RefNanny = __Pyx_RefNannyImportAPI("refnanny");
if (!__Pyx_RefNanny) {
  PyErr_Clear();
  __Pyx_RefNanny = __Pyx_RefNannyImportAPI("Cython.Runtime.refnanny");
  if (!__Pyx_RefNanny)
      Py_FatalError("failed to import 'refnanny' module");
}
#endif
  __Pyx_RefNannySetupContext("__Pyx_PyMODINIT_FUNC PyInit_prefixview(void)", 0);
  if (__Pyx_check_binary_version() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #ifdef __Pxy_PyFrame_Initialize_Offsets
  __Pxy_PyFrame_Initialize_Offsets();
  #endif
  __pyx_empty_tuple = PyTuple_New(0); if (unlikely(!__pyx_empty_tuple)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_empty_bytes = PyBytes_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_bytes)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_empty_unicode = PyUnicode_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_unicode)) __PYX_ERR(0, 1, __pyx_L1_error)
  #ifdef __Pyx_CyFunction_USED
  if (__pyx_CyFunction_init(__pyx_m) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_FusedFunction_USED
  if (__pyx_FusedFunction_init(__pyx_m) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Coroutine_USED
  if (__pyx_Coroutine_init(__pyx_m) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Generator_USED
  if (__pyx_Generator_init(__pyx_m) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_AsyncGen_USED
  if (__pyx_AsyncGen_init(__pyx_m) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_StopAsyncIteration_USED
  if (__pyx_StopAsyncIteration_init(__pyx_m) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  /*--- Library function declarations ---*/
  /*--- Threads initialization code ---*/
  #if defined(WITH_THREAD) && PY_VERSION_HEX < 0x030700F0 && defined(__PYX_FORCE_INIT_THREADS) && __PYX_FORCE_INIT_THREADS
  PyEval_InitThreads();
  #endif
  /*--- Initialize various global constants etc. ---*/
  if (__Pyx_InitGlobals() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #if PY_MAJOR_VERSION < 3 && (__PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT)
  if (__Pyx_init_sys_getdefaultencoding_params() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  if (__pyx_module_is_main_aim__storage__prefixview) {
    if (PyObject_SetAttr(__pyx_m, __pyx_n_s_name, __pyx_n_s_main) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  }
  #if PY_MAJOR_VERSION >= 3
  {
    PyObject *modules = PyImport_GetModuleDict(); if (unlikely(!modules)) __PYX_ERR(0, 1, __pyx_L1_error)
    if (!PyDict_GetItemString(modules, "aim.storage.prefixview")) {
      if (unlikely((PyDict_SetItemString(modules, "aim.storage.prefixview", __pyx_m) < 0))) __PYX_ERR(0, 1, __pyx_L1_error)
    }
  }
  #endif
  /*--- Builtin init code ---*/
  if (__Pyx_InitCachedBuiltins() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Constants init code ---*/
  if (__Pyx_InitCachedConstants() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Global type/function init code ---*/
  (void)__Pyx_modinit_global_init_code();
  (void)__Pyx_modinit_variable_export_code();
  (void)__Pyx_modinit_function_export_code();
  if (unlikely((__Pyx_modinit_type_init_code() < 0))) __PYX_ERR(0, 1, __pyx_L1_error)
  if (unlikely((__Pyx_modinit_type_import_code() < 0))) __PYX_ERR(0, 1, __pyx_L1_error)
  (void)__Pyx_modinit_variable_import_code();
  if (unlikely((__Pyx_modinit_function_import_code() < 0))) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Execution code ---*/
  #if defined(__Pyx_Generator_USED) || defined(__Pyx_Coroutine_USED)
  if (__Pyx_patch_abc() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif

  /* "aim/storage/prefixview.py":1
 * from aim.storage.container import Container, ContainerItemsIterator, ContainerKey, ContainerValue             # <<<<<<<<<<<<<<
 * from aim.storage.containertreeview import ContainerTreeView
 * 
 */
  __pyx_t_1 = PyList_New(4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_s_Container);
  __Pyx_GIVEREF(__pyx_n_s_Container);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_s_Container);
  __Pyx_INCREF(__pyx_n_s_ContainerItemsIterator);
  __Pyx_GIVEREF(__pyx_n_s_ContainerItemsIterator);
  PyList_SET_ITEM(__pyx_t_1, 1, __pyx_n_s_ContainerItemsIterator);
  __Pyx_INCREF(__pyx_n_s_ContainerKey);
  __Pyx_GIVEREF(__pyx_n_s_ContainerKey);
  PyList_SET_ITEM(__pyx_t_1, 2, __pyx_n_s_ContainerKey);
  __Pyx_INCREF(__pyx_n_s_ContainerValue);
  __Pyx_GIVEREF(__pyx_n_s_ContainerValue);
  PyList_SET_ITEM(__pyx_t_1, 3, __pyx_n_s_ContainerValue);
  __pyx_t_2 = __Pyx_Import(__pyx_n_s_aim_storage_container, __pyx_t_1, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_ImportFrom(__pyx_t_2, __pyx_n_s_ContainerKey); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_ContainerKey, __pyx_t_1) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_ImportFrom(__pyx_t_2, __pyx_n_s_ContainerValue); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_ContainerValue, __pyx_t_1) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "aim/storage/prefixview.py":2
 * from aim.storage.container import Container, ContainerItemsIterator, ContainerKey, ContainerValue
 * from aim.storage.containertreeview import ContainerTreeView             # <<<<<<<<<<<<<<
 * 
 * from typing import Iterator, Tuple
 */
  __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_n_s_ContainerTreeView);
  __Pyx_GIVEREF(__pyx_n_s_ContainerTreeView);
  PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_ContainerTreeView);
  __pyx_t_1 = __Pyx_Import(__pyx_n_s_aim_storage_containertreeview, __pyx_t_2, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "aim/storage/prefixview.py":4
 * from aim.storage.containertreeview import ContainerTreeView
 * 
 * from typing import Iterator, Tuple             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_1 = PyList_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_s_Iterator);
  __Pyx_GIVEREF(__pyx_n_s_Iterator);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_s_Iterator);
  __Pyx_INCREF(__pyx_n_s_Tuple);
  __Pyx_GIVEREF(__pyx_n_s_Tuple);
  PyList_SET_ITEM(__pyx_t_1, 1, __pyx_n_s_Tuple);
  __pyx_t_2 = __Pyx_Import(__pyx_n_s_typing, __pyx_t_1, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_ImportFrom(__pyx_t_2, __pyx_n_s_Iterator); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_Iterator, __pyx_t_1) < 0) __PYX_ERR(0, 4, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_ImportFrom(__pyx_t_2, __pyx_n_s_Tuple); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_Tuple, __pyx_t_1) < 0) __PYX_ERR(0, 4, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "aim/storage/prefixview.py":44
 *         self.read_only = read_only
 * 
 *     def close(self):             # <<<<<<<<<<<<<<
 *         """Close all the resources."""
 *         pass
 */
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_3aim_7storage_10prefixview_10PrefixView_3close, __Pyx_CYFUNCTION_CCLASS, __pyx_n_s_PrefixView_close, NULL, __pyx_n_s_aim_storage_prefixview, __pyx_d, ((PyObject *)__pyx_codeobj__5)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 44, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem((PyObject *)__pyx_ptype_3aim_7storage_10prefixview_PrefixView->tp_dict, __pyx_n_s_close, __pyx_t_2) < 0) __PYX_ERR(0, 44, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_3aim_7storage_10prefixview_PrefixView);

  /* "aim/storage/prefixview.py":48
 *         pass
 * 
 *     def preload(self):             # <<<<<<<<<<<<<<
 *         """Preload the Container.
 * 
 */
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_3aim_7storage_10prefixview_10PrefixView_5preload, __Pyx_CYFUNCTION_CCLASS, __pyx_n_s_PrefixView_preload, NULL, __pyx_n_s_aim_storage_prefixview, __pyx_d, ((PyObject *)__pyx_codeobj__7)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 48, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem((PyObject *)__pyx_ptype_3aim_7storage_10prefixview_PrefixView->tp_dict, __pyx_n_s_preload, __pyx_t_2) < 0) __PYX_ERR(0, 48, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_3aim_7storage_10prefixview_PrefixView);

  /* "aim/storage/prefixview.py":58
 *         self.parent.preload()
 * 
 *     def finalize(self, index: Container):             # <<<<<<<<<<<<<<
 *         """Finalize the Container.
 * 
 */
  __pyx_t_2 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 58, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_index, __pyx_n_s_Container) < 0) __PYX_ERR(0, 58, __pyx_L1_error)
  __pyx_t_1 = __Pyx_CyFunction_New(&__pyx_mdef_3aim_7storage_10prefixview_10PrefixView_7finalize, __Pyx_CYFUNCTION_CCLASS, __pyx_n_s_PrefixView_finalize, NULL, __pyx_n_s_aim_storage_prefixview, __pyx_d, ((PyObject *)__pyx_codeobj__9)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 58, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_CyFunction_SetAnnotationsDict(__pyx_t_1, __pyx_t_2);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_3aim_7storage_10prefixview_PrefixView->tp_dict, __pyx_n_s_finalize, __pyx_t_1) < 0) __PYX_ERR(0, 58, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  PyType_Modified(__pyx_ptype_3aim_7storage_10prefixview_PrefixView);

  /* "aim/storage/prefixview.py":68
 *         self.parent.finalize(index=index)
 * 
 *     def absolute_path(             # <<<<<<<<<<<<<<
 *         self,
 *         path: bytes = None
 */
  __pyx_t_1 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 68, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_path, __pyx_n_s_bytes) < 0) __PYX_ERR(0, 68, __pyx_L1_error)
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_3aim_7storage_10prefixview_10PrefixView_9absolute_path, __Pyx_CYFUNCTION_CCLASS, __pyx_n_s_PrefixView_absolute_path, NULL, __pyx_n_s_aim_storage_prefixview, __pyx_d, ((PyObject *)__pyx_codeobj__11)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 68, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_2, __pyx_tuple__12);
  __Pyx_CyFunction_SetAnnotationsDict(__pyx_t_2, __pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_3aim_7storage_10prefixview_PrefixView->tp_dict, __pyx_n_s_absolute_path, __pyx_t_2) < 0) __PYX_ERR(0, 68, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_3aim_7storage_10prefixview_PrefixView);

  /* "aim/storage/prefixview.py":83
 *         return self.prefix + path
 * 
 *     def get(             # <<<<<<<<<<<<<<
 *         self,
 *         key: ContainerKey,
 */
  __pyx_t_2 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 83, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_key, __pyx_n_s_ContainerKey) < 0) __PYX_ERR(0, 83, __pyx_L1_error)
  __pyx_t_1 = __Pyx_CyFunction_New(&__pyx_mdef_3aim_7storage_10prefixview_10PrefixView_11get, __Pyx_CYFUNCTION_CCLASS, __pyx_n_s_PrefixView_get, NULL, __pyx_n_s_aim_storage_prefixview, __pyx_d, ((PyObject *)__pyx_codeobj__14)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 83, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_1, __pyx_tuple__12);
  __Pyx_CyFunction_SetAnnotationsDict(__pyx_t_1, __pyx_t_2);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_3aim_7storage_10prefixview_PrefixView->tp_dict, __pyx_n_s_get, __pyx_t_1) < 0) __PYX_ERR(0, 83, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  PyType_Modified(__pyx_ptype_3aim_7storage_10prefixview_PrefixView);

  /* "aim/storage/prefixview.py":103
 *         return self.parent[path]
 * 
 *     def set(             # <<<<<<<<<<<<<<
 *         self,
 *         key: ContainerKey,
 */
  __pyx_t_1 = __Pyx_PyDict_NewPresized(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 103, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_key, __pyx_n_s_ContainerKey) < 0) __PYX_ERR(0, 103, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_value, __pyx_n_s_ContainerValue) < 0) __PYX_ERR(0, 103, __pyx_L1_error)
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_3aim_7storage_10prefixview_10PrefixView_15set, __Pyx_CYFUNCTION_CCLASS, __pyx_n_s_PrefixView_set, NULL, __pyx_n_s_aim_storage_prefixview, __pyx_d, ((PyObject *)__pyx_codeobj__16)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 103, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_2, __pyx_tuple__12);
  __Pyx_CyFunction_SetAnnotationsDict(__pyx_t_2, __pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_3aim_7storage_10prefixview_PrefixView->tp_dict, __pyx_n_s_set, __pyx_t_2) < 0) __PYX_ERR(0, 103, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_3aim_7storage_10prefixview_PrefixView);

  /* "aim/storage/prefixview.py":129
 *         self.parent[path] = value
 * 
 *     def delete(             # <<<<<<<<<<<<<<
 *         self,
 *         key: ContainerKey,
 */
  __pyx_t_2 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 129, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_key, __pyx_n_s_ContainerKey) < 0) __PYX_ERR(0, 129, __pyx_L1_error)
  __pyx_t_1 = __Pyx_CyFunction_New(&__pyx_mdef_3aim_7storage_10prefixview_10PrefixView_19delete, __Pyx_CYFUNCTION_CCLASS, __pyx_n_s_PrefixView_delete, NULL, __pyx_n_s_aim_storage_prefixview, __pyx_d, ((PyObject *)__pyx_codeobj__18)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 129, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_1, __pyx_tuple__12);
  __Pyx_CyFunction_SetAnnotationsDict(__pyx_t_1, __pyx_t_2);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_3aim_7storage_10prefixview_PrefixView->tp_dict, __pyx_n_s_delete, __pyx_t_1) < 0) __PYX_ERR(0, 129, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  PyType_Modified(__pyx_ptype_3aim_7storage_10prefixview_PrefixView);

  /* "aim/storage/prefixview.py":154
 *         del self.parent[path]
 * 
 *     def delete_range(             # <<<<<<<<<<<<<<
 *         self,
 *         begin: ContainerKey,
 */
  __pyx_t_1 = __Pyx_PyDict_NewPresized(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 154, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_begin, __pyx_n_s_ContainerKey) < 0) __PYX_ERR(0, 154, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_end, __pyx_n_s_ContainerKey) < 0) __PYX_ERR(0, 154, __pyx_L1_error)
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_3aim_7storage_10prefixview_10PrefixView_23delete_range, __Pyx_CYFUNCTION_CCLASS, __pyx_n_s_PrefixView_delete_range, NULL, __pyx_n_s_aim_storage_prefixview, __pyx_d, ((PyObject *)__pyx_codeobj__20)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 154, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_2, __pyx_tuple__12);
  __Pyx_CyFunction_SetAnnotationsDict(__pyx_t_2, __pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_3aim_7storage_10prefixview_PrefixView->tp_dict, __pyx_n_s_delete_range, __pyx_t_2) < 0) __PYX_ERR(0, 154, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_3aim_7storage_10prefixview_PrefixView);

  /* "aim/storage/prefixview.py":166
 *                                  store_batch=store_batch)
 * 
 *     def next_item(             # <<<<<<<<<<<<<<
 *         self,
 *         key: ContainerKey = b''
 */
  __pyx_t_2 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 166, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_key, __pyx_n_s_ContainerKey) < 0) __PYX_ERR(0, 166, __pyx_L1_error)
  __pyx_t_1 = __Pyx_CyFunction_New(&__pyx_mdef_3aim_7storage_10prefixview_10PrefixView_25next_item, __Pyx_CYFUNCTION_CCLASS, __pyx_n_s_PrefixView_next_item, NULL, __pyx_n_s_aim_storage_prefixview, __pyx_d, ((PyObject *)__pyx_codeobj__22)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 166, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_1, __pyx_tuple__23);
  __Pyx_CyFunction_SetAnnotationsDict(__pyx_t_1, __pyx_t_2);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_3aim_7storage_10prefixview_PrefixView->tp_dict, __pyx_n_s_next_item, __pyx_t_1) < 0) __PYX_ERR(0, 166, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  PyType_Modified(__pyx_ptype_3aim_7storage_10prefixview_PrefixView);

  /* "aim/storage/prefixview.py":181
 *         return keys, value
 * 
 *     def prev_item(             # <<<<<<<<<<<<<<
 *         self,
 *         key: ContainerKey = b''
 */
  __pyx_t_1 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 181, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_key, __pyx_n_s_ContainerKey) < 0) __PYX_ERR(0, 181, __pyx_L1_error)
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_3aim_7storage_10prefixview_10PrefixView_27prev_item, __Pyx_CYFUNCTION_CCLASS, __pyx_n_s_PrefixView_prev_item, NULL, __pyx_n_s_aim_storage_prefixview, __pyx_d, ((PyObject *)__pyx_codeobj__25)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 181, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_2, __pyx_tuple__26);
  __Pyx_CyFunction_SetAnnotationsDict(__pyx_t_2, __pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_3aim_7storage_10prefixview_PrefixView->tp_dict, __pyx_n_s_prev_item, __pyx_t_2) < 0) __PYX_ERR(0, 181, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_3aim_7storage_10prefixview_PrefixView);

  /* "aim/storage/prefixview.py":196
 *         return keys, value
 * 
 *     def walk(             # <<<<<<<<<<<<<<
 *         self,
 *         key: ContainerKey = b''
 */
  __pyx_t_2 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 196, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_key, __pyx_n_s_ContainerKey) < 0) __PYX_ERR(0, 196, __pyx_L1_error)
  __pyx_t_1 = __Pyx_CyFunction_New(&__pyx_mdef_3aim_7storage_10prefixview_10PrefixView_29walk, __Pyx_CYFUNCTION_CCLASS, __pyx_n_s_PrefixView_walk, NULL, __pyx_n_s_aim_storage_prefixview, __pyx_d, ((PyObject *)__pyx_codeobj__3)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 196, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_1, __pyx_tuple__28);
  __Pyx_CyFunction_SetAnnotationsDict(__pyx_t_1, __pyx_t_2);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_3aim_7storage_10prefixview_PrefixView->tp_dict, __pyx_n_s_walk, __pyx_t_1) < 0) __PYX_ERR(0, 196, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  PyType_Modified(__pyx_ptype_3aim_7storage_10prefixview_PrefixView);

  /* "aim/storage/prefixview.py":232
 *             p = self.absolute_path(key)
 * 
 *     def items(             # <<<<<<<<<<<<<<
 *         self,
 *         key: ContainerKey = b''
 */
  __pyx_t_1 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 232, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_key, __pyx_n_s_ContainerKey) < 0) __PYX_ERR(0, 232, __pyx_L1_error)
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_3aim_7storage_10prefixview_10PrefixView_32items, __Pyx_CYFUNCTION_CCLASS, __pyx_n_s_PrefixView_items, NULL, __pyx_n_s_aim_storage_prefixview, __pyx_d, ((PyObject *)__pyx_codeobj__30)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 232, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_2, __pyx_tuple__31);
  __Pyx_CyFunction_SetAnnotationsDict(__pyx_t_2, __pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_3aim_7storage_10prefixview_PrefixView->tp_dict, __pyx_n_s_items, __pyx_t_2) < 0) __PYX_ERR(0, 232, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_3aim_7storage_10prefixview_PrefixView);

  /* "aim/storage/prefixview.py":255
 *         return PrefixViewItemsIterator(self, key)
 * 
 *     def view(             # <<<<<<<<<<<<<<
 *         self,
 *         prefix: bytes = b''
 */
  __pyx_t_2 = __Pyx_PyDict_NewPresized(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 255, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_prefix, __pyx_n_s_bytes) < 0) __PYX_ERR(0, 255, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_return, __pyx_n_s_Container) < 0) __PYX_ERR(0, 255, __pyx_L1_error)
  __pyx_t_1 = __Pyx_CyFunction_New(&__pyx_mdef_3aim_7storage_10prefixview_10PrefixView_34view, __Pyx_CYFUNCTION_CCLASS, __pyx_n_s_PrefixView_view, NULL, __pyx_n_s_aim_storage_prefixview, __pyx_d, ((PyObject *)__pyx_codeobj__33)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 255, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_1, __pyx_tuple__28);
  __Pyx_CyFunction_SetAnnotationsDict(__pyx_t_1, __pyx_t_2);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_3aim_7storage_10prefixview_PrefixView->tp_dict, __pyx_n_s_view, __pyx_t_1) < 0) __PYX_ERR(0, 255, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  PyType_Modified(__pyx_ptype_3aim_7storage_10prefixview_PrefixView);

  /* "aim/storage/prefixview.py":284
 *         return self.parent.view(self.prefix + prefix)
 * 
 *     def tree(             # <<<<<<<<<<<<<<
 *         self
 *     ) -> ContainerTreeView:
 */
  __pyx_t_1 = __Pyx_CyFunction_New(&__pyx_mdef_3aim_7storage_10prefixview_10PrefixView_36tree, __Pyx_CYFUNCTION_CCLASS, __pyx_n_s_PrefixView_tree, NULL, __pyx_n_s_aim_storage_prefixview, __pyx_d, ((PyObject *)__pyx_codeobj__35)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 284, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem((PyObject *)__pyx_ptype_3aim_7storage_10prefixview_PrefixView->tp_dict, __pyx_n_s_tree, __pyx_t_1) < 0) __PYX_ERR(0, 284, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  PyType_Modified(__pyx_ptype_3aim_7storage_10prefixview_PrefixView);

  /* "aim/storage/prefixview.py":305
 *         return ContainerTreeView(self)
 * 
 *     def batch(self):             # <<<<<<<<<<<<<<
 *         """Creates a new batch object to store operations in before executing
 *         using :obj:`Container.commit`.
 */
  __pyx_t_1 = __Pyx_CyFunction_New(&__pyx_mdef_3aim_7storage_10prefixview_10PrefixView_38batch, __Pyx_CYFUNCTION_CCLASS, __pyx_n_s_PrefixView_batch, NULL, __pyx_n_s_aim_storage_prefixview, __pyx_d, ((PyObject *)__pyx_codeobj__37)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 305, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem((PyObject *)__pyx_ptype_3aim_7storage_10prefixview_PrefixView->tp_dict, __pyx_n_s_batch, __pyx_t_1) < 0) __PYX_ERR(0, 305, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  PyType_Modified(__pyx_ptype_3aim_7storage_10prefixview_PrefixView);

  /* "aim/storage/prefixview.py":316
 *         return self.parent.batch()
 * 
 *     def commit(             # <<<<<<<<<<<<<<
 *         self,
 *         batch
 */
  __pyx_t_1 = __Pyx_CyFunction_New(&__pyx_mdef_3aim_7storage_10prefixview_10PrefixView_40commit, __Pyx_CYFUNCTION_CCLASS, __pyx_n_s_PrefixView_commit, NULL, __pyx_n_s_aim_storage_prefixview, __pyx_d, ((PyObject *)__pyx_codeobj__39)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 316, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem((PyObject *)__pyx_ptype_3aim_7storage_10prefixview_PrefixView->tp_dict, __pyx_n_s_commit, __pyx_t_1) < 0) __PYX_ERR(0, 316, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  PyType_Modified(__pyx_ptype_3aim_7storage_10prefixview_PrefixView);

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */
  __pyx_t_1 = __Pyx_CyFunction_New(&__pyx_mdef_3aim_7storage_10prefixview_10PrefixView_42__reduce_cython__, __Pyx_CYFUNCTION_CCLASS, __pyx_n_s_PrefixView___reduce_cython, NULL, __pyx_n_s_aim_storage_prefixview, __pyx_d, ((PyObject *)__pyx_codeobj__41)); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem((PyObject *)__pyx_ptype_3aim_7storage_10prefixview_PrefixView->tp_dict, __pyx_n_s_reduce_cython, __pyx_t_1) < 0) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  PyType_Modified(__pyx_ptype_3aim_7storage_10prefixview_PrefixView);

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_PrefixView, (type(self), 0x0d8b696, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_PrefixView__set_state(self, __pyx_state)
 */
  __pyx_t_1 = __Pyx_CyFunction_New(&__pyx_mdef_3aim_7storage_10prefixview_10PrefixView_44__setstate_cython__, __Pyx_CYFUNCTION_CCLASS, __pyx_n_s_PrefixView___setstate_cython, NULL, __pyx_n_s_aim_storage_prefixview, __pyx_d, ((PyObject *)__pyx_codeobj__43)); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 16, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem((PyObject *)__pyx_ptype_3aim_7storage_10prefixview_PrefixView->tp_dict, __pyx_n_s_setstate_cython, __pyx_t_1) < 0) __PYX_ERR(2, 16, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  PyType_Modified(__pyx_ptype_3aim_7storage_10prefixview_PrefixView);

  /* "aim/storage/prefixview.py":337
 *         self.prefix_len = len(self.path)
 * 
 *     def next(self):             # <<<<<<<<<<<<<<
 *         item = self.it.next()
 * 
 */
  __pyx_t_1 = __Pyx_CyFunction_New(&__pyx_mdef_3aim_7storage_10prefixview_23PrefixViewItemsIterator_3next, __Pyx_CYFUNCTION_CCLASS, __pyx_n_s_PrefixViewItemsIterator_next, NULL, __pyx_n_s_aim_storage_prefixview, __pyx_d, ((PyObject *)__pyx_codeobj__45)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 337, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem((PyObject *)__pyx_ptype_3aim_7storage_10prefixview_PrefixViewItemsIterator->tp_dict, __pyx_n_s_next, __pyx_t_1) < 0) __PYX_ERR(0, 337, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  PyType_Modified(__pyx_ptype_3aim_7storage_10prefixview_PrefixViewItemsIterator);

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */
  __pyx_t_1 = __Pyx_CyFunction_New(&__pyx_mdef_3aim_7storage_10prefixview_23PrefixViewItemsIterator_5__reduce_cython__, __Pyx_CYFUNCTION_CCLASS, __pyx_n_s_PrefixViewItemsIterator___reduce, NULL, __pyx_n_s_aim_storage_prefixview, __pyx_d, ((PyObject *)__pyx_codeobj__47)); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem((PyObject *)__pyx_ptype_3aim_7storage_10prefixview_PrefixViewItemsIterator->tp_dict, __pyx_n_s_reduce_cython, __pyx_t_1) < 0) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  PyType_Modified(__pyx_ptype_3aim_7storage_10prefixview_PrefixViewItemsIterator);

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_PrefixViewItemsIterator, (type(self), 0xda33529, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_PrefixViewItemsIterator__set_state(self, __pyx_state)
 */
  __pyx_t_1 = __Pyx_CyFunction_New(&__pyx_mdef_3aim_7storage_10prefixview_23PrefixViewItemsIterator_7__setstate_cython__, __Pyx_CYFUNCTION_CCLASS, __pyx_n_s_PrefixViewItemsIterator___setsta, NULL, __pyx_n_s_aim_storage_prefixview, __pyx_d, ((PyObject *)__pyx_codeobj__49)); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 16, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem((PyObject *)__pyx_ptype_3aim_7storage_10prefixview_PrefixViewItemsIterator->tp_dict, __pyx_n_s_setstate_cython, __pyx_t_1) < 0) __PYX_ERR(2, 16, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  PyType_Modified(__pyx_ptype_3aim_7storage_10prefixview_PrefixViewItemsIterator);

  /* "(tree fragment)":1
 * def __pyx_unpickle_PrefixView(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */
  __pyx_t_1 = __Pyx_CyFunction_New(&__pyx_mdef_3aim_7storage_10prefixview_1__pyx_unpickle_PrefixView, 0, __pyx_n_s_pyx_unpickle_PrefixView, NULL, __pyx_n_s_aim_storage_prefixview, __pyx_d, ((PyObject *)__pyx_codeobj__51)); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pyx_unpickle_PrefixView, __pyx_t_1) < 0) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":11
 *         __pyx_unpickle_PrefixView__set_state(<PrefixView> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_PrefixView__set_state(PrefixView __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.parent = __pyx_state[0]; __pyx_result.prefix = __pyx_state[1]; __pyx_result.read_only = __pyx_state[2]
 *     if len(__pyx_state) > 3 and hasattr(__pyx_result, '__dict__'):
 */
  __pyx_t_1 = __Pyx_CyFunction_New(&__pyx_mdef_3aim_7storage_10prefixview_3__pyx_unpickle_PrefixViewItemsIterator, 0, __pyx_n_s_pyx_unpickle_PrefixViewItemsIt, NULL, __pyx_n_s_aim_storage_prefixview, __pyx_d, ((PyObject *)__pyx_codeobj__53)); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pyx_unpickle_PrefixViewItemsIt, __pyx_t_1) < 0) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "aim/storage/prefixview.py":1
 * from aim.storage.container import Container, ContainerItemsIterator, ContainerKey, ContainerValue             # <<<<<<<<<<<<<<
 * from aim.storage.containertreeview import ContainerTreeView
 * 
 */
  __pyx_t_1 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_test, __pyx_t_1) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "aim/storage/containertreeview.pxd":1
 * # distutils: language = c++             # <<<<<<<<<<<<<<
 * # cython: language_level = 3
 * 
 */

  /*--- Wrapped vars code ---*/

  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  if (__pyx_m) {
    if (__pyx_d) {
      __Pyx_AddTraceback("init aim.storage.prefixview", __pyx_clineno, __pyx_lineno, __pyx_filename);
    }
    #if !CYTHON_COMPILING_IN_LIMITED_API
    Py_CLEAR(__pyx_m);
    #endif
  } else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_ImportError, "init aim.storage.prefixview");
  }
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  return (__pyx_m != NULL) ? 0 : -1;
  #elif PY_MAJOR_VERSION >= 3
  return __pyx_m;
  #else
  return;
  #endif
}
/* #### Code section: cleanup_globals ### */
/* #### Code section: cleanup_module ### */
/* #### Code section: main_method ### */
/* #### Code section: utility_code_def ### */

/* --- Runtime support code --- */
/* Refnanny */
#if CYTHON_REFNANNY
static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname) {
    PyObject *m = NULL, *p = NULL;
    void *r = NULL;
    m = PyImport_ImportModule(modname);
    if (!m) goto end;
    p = PyObject_GetAttrString(m, "RefNannyAPI");
    if (!p) goto end;
    r = PyLong_AsVoidPtr(p);
end:
    Py_XDECREF(p);
    Py_XDECREF(m);
    return (__Pyx_RefNannyAPIStruct *)r;
}
#endif

/* PyErrExceptionMatches */
#if CYTHON_FAST_THREAD_STATE
static int __Pyx_PyErr_ExceptionMatchesTuple(PyObject *exc_type, PyObject *tuple) {
    Py_ssize_t i, n;
    n = PyTuple_GET_SIZE(tuple);
#if PY_MAJOR_VERSION >= 3
    for (i=0; i<n; i++) {
        if (exc_type == PyTuple_GET_ITEM(tuple, i)) return 1;
    }
#endif
    for (i=0; i<n; i++) {
        if (__Pyx_PyErr_GivenExceptionMatches(exc_type, PyTuple_GET_ITEM(tuple, i))) return 1;
    }
    return 0;
}
static CYTHON_INLINE int __Pyx_PyErr_ExceptionMatchesInState(PyThreadState* tstate, PyObject* err) {
    PyObject *exc_type = tstate->curexc_type;
    if (exc_type == err) return 1;
    if (unlikely(!exc_type)) return 0;
    if (unlikely(PyTuple_Check(err)))
        return __Pyx_PyErr_ExceptionMatchesTuple(exc_type, err);
    return __Pyx_PyErr_GivenExceptionMatches(exc_type, err);
}
#endif

/* PyErrFetchRestore */
#if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    tmp_type = tstate->curexc_type;
    tmp_value = tstate->curexc_value;
    tmp_tb = tstate->curexc_traceback;
    tstate->curexc_type = type;
    tstate->curexc_value = value;
    tstate->curexc_traceback = tb;
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
}
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
    *type = tstate->curexc_type;
    *value = tstate->curexc_value;
    *tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
}
#endif

/* PyObjectGetAttrStr */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name) {
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_getattro))
        return tp->tp_getattro(obj, attr_name);
#if PY_MAJOR_VERSION < 3
    if (likely(tp->tp_getattr))
        return tp->tp_getattr(obj, PyString_AS_STRING(attr_name));
#endif
    return PyObject_GetAttr(obj, attr_name);
}
#endif

/* PyObjectGetAttrStrNoError */
static void __Pyx_PyObject_GetAttrStr_ClearAttributeError(void) {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    if (likely(__Pyx_PyErr_ExceptionMatches(PyExc_AttributeError)))
        __Pyx_PyErr_Clear();
}
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStrNoError(PyObject* obj, PyObject* attr_name) {
    PyObject *result;
#if CYTHON_COMPILING_IN_CPYTHON && CYTHON_USE_TYPE_SLOTS && PY_VERSION_HEX >= 0x030700B1
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_getattro == PyObject_GenericGetAttr)) {
        return _PyObject_GenericGetAttrWithDict(obj, attr_name, NULL, 1);
    }
#endif
    result = __Pyx_PyObject_GetAttrStr(obj, attr_name);
    if (unlikely(!result)) {
        __Pyx_PyObject_GetAttrStr_ClearAttributeError();
    }
    return result;
}

/* GetBuiltinName */
static PyObject *__Pyx_GetBuiltinName(PyObject *name) {
    PyObject* result = __Pyx_PyObject_GetAttrStrNoError(__pyx_b, name);
    if (unlikely(!result) && !PyErr_Occurred()) {
        PyErr_Format(PyExc_NameError,
#if PY_MAJOR_VERSION >= 3
            "name '%U' is not defined", name);
#else
            "name '%.200s' is not defined", PyString_AS_STRING(name));
#endif
    }
    return result;
}

/* TupleAndListFromArray */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE void __Pyx_copy_object_array(PyObject *const *CYTHON_RESTRICT src, PyObject** CYTHON_RESTRICT dest, Py_ssize_t length) {
    PyObject *v;
    Py_ssize_t i;
    for (i = 0; i < length; i++) {
        v = dest[i] = src[i];
        Py_INCREF(v);
    }
}
static CYTHON_INLINE PyObject *
__Pyx_PyTuple_FromArray(PyObject *const *src, Py_ssize_t n)
{
    PyObject *res;
    if (n <= 0) {
        Py_INCREF(__pyx_empty_tuple);
        return __pyx_empty_tuple;
    }
    res = PyTuple_New(n);
    if (unlikely(res == NULL)) return NULL;
    __Pyx_copy_object_array(src, ((PyTupleObject*)res)->ob_item, n);
    return res;
}
static CYTHON_INLINE PyObject *
__Pyx_PyList_FromArray(PyObject *const *src, Py_ssize_t n)
{
    PyObject *res;
    if (n <= 0) {
        return PyList_New(0);
    }
    res = PyList_New(n);
    if (unlikely(res == NULL)) return NULL;
    __Pyx_copy_object_array(src, ((PyListObject*)res)->ob_item, n);
    return res;
}
#endif

/* BytesEquals */
static CYTHON_INLINE int __Pyx_PyBytes_Equals(PyObject* s1, PyObject* s2, int equals) {
#if CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_LIMITED_API
    return PyObject_RichCompareBool(s1, s2, equals);
#else
    if (s1 == s2) {
        return (equals == Py_EQ);
    } else if (PyBytes_CheckExact(s1) & PyBytes_CheckExact(s2)) {
        const char *ps1, *ps2;
        Py_ssize_t length = PyBytes_GET_SIZE(s1);
        if (length != PyBytes_GET_SIZE(s2))
            return (equals == Py_NE);
        ps1 = PyBytes_AS_STRING(s1);
        ps2 = PyBytes_AS_STRING(s2);
        if (ps1[0] != ps2[0]) {
            return (equals == Py_NE);
        } else if (length == 1) {
            return (equals == Py_EQ);
        } else {
            int result;
#if CYTHON_USE_UNICODE_INTERNALS
            Py_hash_t hash1, hash2;
            hash1 = ((PyBytesObject*)s1)->ob_shash;
            hash2 = ((PyBytesObject*)s2)->ob_shash;
            if (hash1 != hash2 && hash1 != -1 && hash2 != -1) {
                return (equals == Py_NE);
            }
#endif
            result = memcmp(ps1, ps2, (size_t)length);
            return (equals == Py_EQ) ? (result == 0) : (result != 0);
        }
    } else if ((s1 == Py_None) & PyBytes_CheckExact(s2)) {
        return (equals == Py_NE);
    } else if ((s2 == Py_None) & PyBytes_CheckExact(s1)) {
        return (equals == Py_NE);
    } else {
        int result;
        PyObject* py_result = PyObject_RichCompare(s1, s2, equals);
        if (!py_result)
            return -1;
        result = __Pyx_PyObject_IsTrue(py_result);
        Py_DECREF(py_result);
        return result;
    }
#endif
}

/* UnicodeEquals */
static CYTHON_INLINE int __Pyx_PyUnicode_Equals(PyObject* s1, PyObject* s2, int equals) {
#if CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_LIMITED_API
    return PyObject_RichCompareBool(s1, s2, equals);
#else
#if PY_MAJOR_VERSION < 3
    PyObject* owned_ref = NULL;
#endif
    int s1_is_unicode, s2_is_unicode;
    if (s1 == s2) {
        goto return_eq;
    }
    s1_is_unicode = PyUnicode_CheckExact(s1);
    s2_is_unicode = PyUnicode_CheckExact(s2);
#if PY_MAJOR_VERSION < 3
    if ((s1_is_unicode & (!s2_is_unicode)) && PyString_CheckExact(s2)) {
        owned_ref = PyUnicode_FromObject(s2);
        if (unlikely(!owned_ref))
            return -1;
        s2 = owned_ref;
        s2_is_unicode = 1;
    } else if ((s2_is_unicode & (!s1_is_unicode)) && PyString_CheckExact(s1)) {
        owned_ref = PyUnicode_FromObject(s1);
        if (unlikely(!owned_ref))
            return -1;
        s1 = owned_ref;
        s1_is_unicode = 1;
    } else if (((!s2_is_unicode) & (!s1_is_unicode))) {
        return __Pyx_PyBytes_Equals(s1, s2, equals);
    }
#endif
    if (s1_is_unicode & s2_is_unicode) {
        Py_ssize_t length;
        int kind;
        void *data1, *data2;
        if (unlikely(__Pyx_PyUnicode_READY(s1) < 0) || unlikely(__Pyx_PyUnicode_READY(s2) < 0))
            return -1;
        length = __Pyx_PyUnicode_GET_LENGTH(s1);
        if (length != __Pyx_PyUnicode_GET_LENGTH(s2)) {
            goto return_ne;
        }
#if CYTHON_USE_UNICODE_INTERNALS
        {
            Py_hash_t hash1, hash2;
        #if CYTHON_PEP393_ENABLED
            hash1 = ((PyASCIIObject*)s1)->hash;
            hash2 = ((PyASCIIObject*)s2)->hash;
        #else
            hash1 = ((PyUnicodeObject*)s1)->hash;
            hash2 = ((PyUnicodeObject*)s2)->hash;
        #endif
            if (hash1 != hash2 && hash1 != -1 && hash2 != -1) {
                goto return_ne;
            }
        }
#endif
        kind = __Pyx_PyUnicode_KIND(s1);
        if (kind != __Pyx_PyUnicode_KIND(s2)) {
            goto return_ne;
        }
        data1 = __Pyx_PyUnicode_DATA(s1);
        data2 = __Pyx_PyUnicode_DATA(s2);
        if (__Pyx_PyUnicode_READ(kind, data1, 0) != __Pyx_PyUnicode_READ(kind, data2, 0)) {
            goto return_ne;
        } else if (length == 1) {
            goto return_eq;
        } else {
            int result = memcmp(data1, data2, (size_t)(length * kind));
            #if PY_MAJOR_VERSION < 3
            Py_XDECREF(owned_ref);
            #endif
            return (equals == Py_EQ) ? (result == 0) : (result != 0);
        }
    } else if ((s1 == Py_None) & s2_is_unicode) {
        goto return_ne;
    } else if ((s2 == Py_None) & s1_is_unicode) {
        goto return_ne;
    } else {
        int result;
        PyObject* py_result = PyObject_RichCompare(s1, s2, equals);
        #if PY_MAJOR_VERSION < 3
        Py_XDECREF(owned_ref);
        #endif
        if (!py_result)
            return -1;
        result = __Pyx_PyObject_IsTrue(py_result);
        Py_DECREF(py_result);
        return result;
    }
return_eq:
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(owned_ref);
    #endif
    return (equals == Py_EQ);
return_ne:
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(owned_ref);
    #endif
    return (equals == Py_NE);
#endif
}

/* fastcall */
#if CYTHON_METH_FASTCALL
static CYTHON_INLINE PyObject * __Pyx_GetKwValue_FASTCALL(PyObject *kwnames, PyObject *const *kwvalues, PyObject *s)
{
    Py_ssize_t i, n = PyTuple_GET_SIZE(kwnames);
    for (i = 0; i < n; i++)
    {
        if (s == PyTuple_GET_ITEM(kwnames, i)) return kwvalues[i];
    }
    for (i = 0; i < n; i++)
    {
        int eq = __Pyx_PyUnicode_Equals(s, PyTuple_GET_ITEM(kwnames, i), Py_EQ);
        if (unlikely(eq != 0)) {
            if (unlikely(eq < 0)) return NULL;  // error
            return kwvalues[i];
        }
    }
    return NULL;  // not found (no exception set)
}
#endif

/* RaiseKeywordRequired */
static void __Pyx_RaiseKeywordRequired(const char* func_name, PyObject* kw_name) {
    PyErr_Format(PyExc_TypeError,
        #if PY_MAJOR_VERSION >= 3
        "%s() needs keyword-only argument %U", func_name, kw_name);
        #else
        "%s() needs keyword-only argument %s", func_name,
        PyString_AS_STRING(kw_name));
        #endif
}

/* RaiseDoubleKeywords */
static void __Pyx_RaiseDoubleKeywordsError(
    const char* func_name,
    PyObject* kw_name)
{
    PyErr_Format(PyExc_TypeError,
        #if PY_MAJOR_VERSION >= 3
        "%s() got multiple values for keyword argument '%U'", func_name, kw_name);
        #else
        "%s() got multiple values for keyword argument '%s'", func_name,
        PyString_AsString(kw_name));
        #endif
}

/* ParseKeywords */
static int __Pyx_ParseOptionalKeywords(
    PyObject *kwds,
    PyObject *const *kwvalues,
    PyObject **argnames[],
    PyObject *kwds2,
    PyObject *values[],
    Py_ssize_t num_pos_args,
    const char* function_name)
{
    PyObject *key = 0, *value = 0;
    Py_ssize_t pos = 0;
    PyObject*** name;
    PyObject*** first_kw_arg = argnames + num_pos_args;
    int kwds_is_tuple = CYTHON_METH_FASTCALL && likely(PyTuple_Check(kwds));
    while (1) {
        if (kwds_is_tuple) {
            if (pos >= PyTuple_GET_SIZE(kwds)) break;
            key = PyTuple_GET_ITEM(kwds, pos);
            value = kwvalues[pos];
            pos++;
        }
        else
        {
            if (!PyDict_Next(kwds, &pos, &key, &value)) break;
        }
        name = first_kw_arg;
        while (*name && (**name != key)) name++;
        if (*name) {
            values[name-argnames] = value;
            continue;
        }
        name = first_kw_arg;
        #if PY_MAJOR_VERSION < 3
        if (likely(PyString_Check(key))) {
            while (*name) {
                if ((CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**name) == PyString_GET_SIZE(key))
                        && _PyString_Eq(**name, key)) {
                    values[name-argnames] = value;
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    if ((**argname == key) || (
                            (CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**argname) == PyString_GET_SIZE(key))
                             && _PyString_Eq(**argname, key))) {
                        goto arg_passed_twice;
                    }
                    argname++;
                }
            }
        } else
        #endif
        if (likely(PyUnicode_Check(key))) {
            while (*name) {
                int cmp = (
                #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                    (__Pyx_PyUnicode_GET_LENGTH(**name) != __Pyx_PyUnicode_GET_LENGTH(key)) ? 1 :
                #endif
                    PyUnicode_Compare(**name, key)
                );
                if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                if (cmp == 0) {
                    values[name-argnames] = value;
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    int cmp = (**argname == key) ? 0 :
                    #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                        (__Pyx_PyUnicode_GET_LENGTH(**argname) != __Pyx_PyUnicode_GET_LENGTH(key)) ? 1 :
                    #endif
                        PyUnicode_Compare(**argname, key);
                    if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                    if (cmp == 0) goto arg_passed_twice;
                    argname++;
                }
            }
        } else
            goto invalid_keyword_type;
        if (kwds2) {
            if (unlikely(PyDict_SetItem(kwds2, key, value))) goto bad;
        } else {
            goto invalid_keyword;
        }
    }
    return 0;
arg_passed_twice:
    __Pyx_RaiseDoubleKeywordsError(function_name, key);
    goto bad;
invalid_keyword_type:
    PyErr_Format(PyExc_TypeError,
        "%.200s() keywords must be strings", function_name);
    goto bad;
invalid_keyword:
    #if PY_MAJOR_VERSION < 3
    PyErr_Format(PyExc_TypeError,
        "%.200s() got an unexpected keyword argument '%.200s'",
        function_name, PyString_AsString(key));
    #else
    PyErr_Format(PyExc_TypeError,
        "%s() got an unexpected keyword argument '%U'",
        function_name, key);
    #endif
bad:
    return -1;
}

/* RaiseArgTupleInvalid */
static void __Pyx_RaiseArgtupleInvalid(
    const char* func_name,
    int exact,
    Py_ssize_t num_min,
    Py_ssize_t num_max,
    Py_ssize_t num_found)
{
    Py_ssize_t num_expected;
    const char *more_or_less;
    if (num_found < num_min) {
        num_expected = num_min;
        more_or_less = "at least";
    } else {
        num_expected = num_max;
        more_or_less = "at most";
    }
    if (exact) {
        more_or_less = "exactly";
    }
    PyErr_Format(PyExc_TypeError,
                 "%.200s() takes %.8s %" CYTHON_FORMAT_SSIZE_T "d positional argument%.1s (%" CYTHON_FORMAT_SSIZE_T "d given)",
                 func_name, more_or_less, num_expected,
                 (num_expected == 1) ? "" : "s", num_found);
}

/* ArgTypeTest */
static int __Pyx__ArgTypeTest(PyObject *obj, PyTypeObject *type, const char *name, int exact)
{
    __Pyx_TypeName type_name;
    __Pyx_TypeName obj_type_name;
    if (unlikely(!type)) {
        PyErr_SetString(PyExc_SystemError, "Missing type object");
        return 0;
    }
    else if (exact) {
        #if PY_MAJOR_VERSION == 2
        if ((type == &PyBaseString_Type) && likely(__Pyx_PyBaseString_CheckExact(obj))) return 1;
        #endif
    }
    else {
        if (likely(__Pyx_TypeCheck(obj, type))) return 1;
    }
    type_name = __Pyx_PyType_GetName(type);
    obj_type_name = __Pyx_PyType_GetName(Py_TYPE(obj));
    PyErr_Format(PyExc_TypeError,
        "Argument '%.200s' has incorrect type (expected " __Pyx_FMT_TYPENAME
        ", got " __Pyx_FMT_TYPENAME ")", name, type_name, obj_type_name);
    __Pyx_DECREF_TypeName(type_name);
    __Pyx_DECREF_TypeName(obj_type_name);
    return 0;
}

/* PyDictVersioning */
#if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PY_UINT64_T __Pyx_get_tp_dict_version(PyObject *obj) {
    PyObject *dict = Py_TYPE(obj)->tp_dict;
    return likely(dict) ? __PYX_GET_DICT_VERSION(dict) : 0;
}
static CYTHON_INLINE PY_UINT64_T __Pyx_get_object_dict_version(PyObject *obj) {
    PyObject **dictptr = NULL;
    Py_ssize_t offset = Py_TYPE(obj)->tp_dictoffset;
    if (offset) {
#if CYTHON_COMPILING_IN_CPYTHON
        dictptr = (likely(offset > 0)) ? (PyObject **) ((char *)obj + offset) : _PyObject_GetDictPtr(obj);
#else
        dictptr = _PyObject_GetDictPtr(obj);
#endif
    }
    return (dictptr && *dictptr) ? __PYX_GET_DICT_VERSION(*dictptr) : 0;
}
static CYTHON_INLINE int __Pyx_object_dict_version_matches(PyObject* obj, PY_UINT64_T tp_dict_version, PY_UINT64_T obj_dict_version) {
    PyObject *dict = Py_TYPE(obj)->tp_dict;
    if (unlikely(!dict) || unlikely(tp_dict_version != __PYX_GET_DICT_VERSION(dict)))
        return 0;
    return obj_dict_version == __Pyx_get_object_dict_version(obj);
}
#endif

/* PyFunctionFastCall */
#if CYTHON_FAST_PYCALL && !CYTHON_VECTORCALL
static PyObject* __Pyx_PyFunction_FastCallNoKw(PyCodeObject *co, PyObject **args, Py_ssize_t na,
                                               PyObject *globals) {
    PyFrameObject *f;
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    PyObject **fastlocals;
    Py_ssize_t i;
    PyObject *result;
    assert(globals != NULL);
    /* XXX Perhaps we should create a specialized
       PyFrame_New() that doesn't take locals, but does
       take builtins without sanity checking them.
       */
    assert(tstate != NULL);
    f = PyFrame_New(tstate, co, globals, NULL);
    if (f == NULL) {
        return NULL;
    }
    fastlocals = __Pyx_PyFrame_GetLocalsplus(f);
    for (i = 0; i < na; i++) {
        Py_INCREF(*args);
        fastlocals[i] = *args++;
    }
    result = PyEval_EvalFrameEx(f,0);
    ++tstate->recursion_depth;
    Py_DECREF(f);
    --tstate->recursion_depth;
    return result;
}
static PyObject *__Pyx_PyFunction_FastCallDict(PyObject *func, PyObject **args, Py_ssize_t nargs, PyObject *kwargs) {
    PyCodeObject *co = (PyCodeObject *)PyFunction_GET_CODE(func);
    PyObject *globals = PyFunction_GET_GLOBALS(func);
    PyObject *argdefs = PyFunction_GET_DEFAULTS(func);
    PyObject *closure;
#if PY_MAJOR_VERSION >= 3
    PyObject *kwdefs;
#endif
    PyObject *kwtuple, **k;
    PyObject **d;
    Py_ssize_t nd;
    Py_ssize_t nk;
    PyObject *result;
    assert(kwargs == NULL || PyDict_Check(kwargs));
    nk = kwargs ? PyDict_Size(kwargs) : 0;
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object"))) {
        return NULL;
    }
    if (
#if PY_MAJOR_VERSION >= 3
            co->co_kwonlyargcount == 0 &&
#endif
            likely(kwargs == NULL || nk == 0) &&
            co->co_flags == (CO_OPTIMIZED | CO_NEWLOCALS | CO_NOFREE)) {
        if (argdefs == NULL && co->co_argcount == nargs) {
            result = __Pyx_PyFunction_FastCallNoKw(co, args, nargs, globals);
            goto done;
        }
        else if (nargs == 0 && argdefs != NULL
                 && co->co_argcount == Py_SIZE(argdefs)) {
            /* function called with no arguments, but all parameters have
               a default value: use default values as arguments .*/
            args = &PyTuple_GET_ITEM(argdefs, 0);
            result =__Pyx_PyFunction_FastCallNoKw(co, args, Py_SIZE(argdefs), globals);
            goto done;
        }
    }
    if (kwargs != NULL) {
        Py_ssize_t pos, i;
        kwtuple = PyTuple_New(2 * nk);
        if (kwtuple == NULL) {
            result = NULL;
            goto done;
        }
        k = &PyTuple_GET_ITEM(kwtuple, 0);
        pos = i = 0;
        while (PyDict_Next(kwargs, &pos, &k[i], &k[i+1])) {
            Py_INCREF(k[i]);
            Py_INCREF(k[i+1]);
            i += 2;
        }
        nk = i / 2;
    }
    else {
        kwtuple = NULL;
        k = NULL;
    }
    closure = PyFunction_GET_CLOSURE(func);
#if PY_MAJOR_VERSION >= 3
    kwdefs = PyFunction_GET_KW_DEFAULTS(func);
#endif
    if (argdefs != NULL) {
        d = &PyTuple_GET_ITEM(argdefs, 0);
        nd = Py_SIZE(argdefs);
    }
    else {
        d = NULL;
        nd = 0;
    }
#if PY_MAJOR_VERSION >= 3
    result = PyEval_EvalCodeEx((PyObject*)co, globals, (PyObject *)NULL,
                               args, (int)nargs,
                               k, (int)nk,
                               d, (int)nd, kwdefs, closure);
#else
    result = PyEval_EvalCodeEx(co, globals, (PyObject *)NULL,
                               args, (int)nargs,
                               k, (int)nk,
                               d, (int)nd, closure);
#endif
    Py_XDECREF(kwtuple);
done:
    Py_LeaveRecursiveCall();
    return result;
}
#endif

/* PyObjectCall */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw) {
    PyObject *result;
    ternaryfunc call = Py_TYPE(func)->tp_call;
    if (unlikely(!call))
        return PyObject_Call(func, arg, kw);
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    result = (*call)(func, arg, kw);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* PyObjectCallMethO */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg) {
    PyObject *self, *result;
    PyCFunction cfunc;
    cfunc = PyCFunction_GET_FUNCTION(func);
    self = PyCFunction_GET_SELF(func);
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    result = cfunc(self, arg);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* PyObjectFastCall */
static PyObject* __Pyx_PyObject_FastCall_fallback(PyObject *func, PyObject **args, size_t nargs, PyObject *kwargs) {
    PyObject *argstuple;
    PyObject *result;
    size_t i;
    argstuple = PyTuple_New((Py_ssize_t)nargs);
    if (unlikely(!argstuple)) return NULL;
    for (i = 0; i < nargs; i++) {
        Py_INCREF(args[i]);
        PyTuple_SET_ITEM(argstuple, (Py_ssize_t)i, args[i]);
    }
    result = __Pyx_PyObject_Call(func, argstuple, kwargs);
    Py_DECREF(argstuple);
    return result;
}
static CYTHON_INLINE PyObject* __Pyx_PyObject_FastCallDict(PyObject *func, PyObject **args, size_t _nargs, PyObject *kwargs) {
    Py_ssize_t nargs = __Pyx_PyVectorcall_NARGS(_nargs);
#if CYTHON_COMPILING_IN_CPYTHON
    if (nargs == 0 && kwargs == NULL) {
#ifdef __Pyx_CyFunction_USED
        if (__Pyx_IsCyOrPyCFunction(func))
#else
        if (PyCFunction_Check(func))
#endif
        {
            if (likely(PyCFunction_GET_FLAGS(func) & METH_NOARGS)) {
                return __Pyx_PyObject_CallMethO(func, NULL);
            }
        }
    }
    else if (nargs == 1 && kwargs == NULL) {
        if (PyCFunction_Check(func))
        {
            if (likely(PyCFunction_GET_FLAGS(func) & METH_O)) {
                return __Pyx_PyObject_CallMethO(func, args[0]);
            }
        }
    }
#endif
    #if PY_VERSION_HEX < 0x030800B1
    #if CYTHON_FAST_PYCCALL
    if (PyCFunction_Check(func)) {
        if (kwargs) {
            return _PyCFunction_FastCallDict(func, args, nargs, kwargs);
        } else {
            return _PyCFunction_FastCallKeywords(func, args, nargs, NULL);
        }
    }
    #if PY_VERSION_HEX >= 0x030700A1
    if (!kwargs && __Pyx_IS_TYPE(func, &PyMethodDescr_Type)) {
        return _PyMethodDescr_FastCallKeywords(func, args, nargs, NULL);
    }
    #endif
    #endif
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(func)) {
        return __Pyx_PyFunction_FastCallDict(func, args, nargs, kwargs);
    }
    #endif
    #endif
    #if CYTHON_VECTORCALL
    vectorcallfunc f = _PyVectorcall_Function(func);
    if (f) {
        return f(func, args, (size_t)nargs, kwargs);
    }
    #elif defined(__Pyx_CyFunction_USED) && CYTHON_BACKPORT_VECTORCALL
    if (__Pyx_CyFunction_CheckExact(func)) {
        __pyx_vectorcallfunc f = __Pyx_CyFunction_func_vectorcall(func);
        if (f) return f(func, args, (size_t)nargs, kwargs);
    }
    #endif
    if (nargs == 0) {
        return __Pyx_PyObject_Call(func, __pyx_empty_tuple, kwargs);
    }
    return __Pyx_PyObject_FastCall_fallback(func, args, (size_t)nargs, kwargs);
}

/* WriteUnraisableException */
static void __Pyx_WriteUnraisable(const char *name, int clineno,
                                  int lineno, const char *filename,
                                  int full_traceback, int nogil) {
    PyObject *old_exc, *old_val, *old_tb;
    PyObject *ctx;
    __Pyx_PyThreadState_declare
#ifdef WITH_THREAD
    PyGILState_STATE state;
    if (nogil)
        state = PyGILState_Ensure();
#ifdef _MSC_VER
    else state = (PyGILState_STATE)-1;
#endif
#endif
    CYTHON_UNUSED_VAR(clineno);
    CYTHON_UNUSED_VAR(lineno);
    CYTHON_UNUSED_VAR(filename);
    CYTHON_MAYBE_UNUSED_VAR(nogil);
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&old_exc, &old_val, &old_tb);
    if (full_traceback) {
        Py_XINCREF(old_exc);
        Py_XINCREF(old_val);
        Py_XINCREF(old_tb);
        __Pyx_ErrRestore(old_exc, old_val, old_tb);
        PyErr_PrintEx(1);
    }
    #if PY_MAJOR_VERSION < 3
    ctx = PyString_FromString(name);
    #else
    ctx = PyUnicode_FromString(name);
    #endif
    __Pyx_ErrRestore(old_exc, old_val, old_tb);
    if (!ctx) {
        PyErr_WriteUnraisable(Py_None);
    } else {
        PyErr_WriteUnraisable(ctx);
        Py_DECREF(ctx);
    }
#ifdef WITH_THREAD
    if (nogil)
        PyGILState_Release(state);
#endif
}

/* KeywordStringCheck */
static int __Pyx_CheckKeywordStrings(
    PyObject *kw,
    const char* function_name,
    int kw_allowed)
{
    PyObject* key = 0;
    Py_ssize_t pos = 0;
#if CYTHON_COMPILING_IN_PYPY
    if (!kw_allowed && PyDict_Next(kw, &pos, &key, 0))
        goto invalid_keyword;
    return 1;
#else
    if (CYTHON_METH_FASTCALL && likely(PyTuple_Check(kw))) {
        if (unlikely(PyTuple_GET_SIZE(kw) == 0))
            return 1;
        if (!kw_allowed) {
            key = PyTuple_GET_ITEM(kw, 0);
            goto invalid_keyword;
        }
#if PY_VERSION_HEX < 0x03090000
        for (pos = 0; pos < PyTuple_GET_SIZE(kw); pos++) {
            key = PyTuple_GET_ITEM(kw, pos);
            if (unlikely(!PyUnicode_Check(key)))
                goto invalid_keyword_type;
        }
#endif
        return 1;
    }
    while (PyDict_Next(kw, &pos, &key, 0)) {
        #if PY_MAJOR_VERSION < 3
        if (unlikely(!PyString_Check(key)))
        #endif
            if (unlikely(!PyUnicode_Check(key)))
                goto invalid_keyword_type;
    }
    if (!kw_allowed && unlikely(key))
        goto invalid_keyword;
    return 1;
invalid_keyword_type:
    PyErr_Format(PyExc_TypeError,
        "%.200s() keywords must be strings", function_name);
    return 0;
#endif
invalid_keyword:
    #if PY_MAJOR_VERSION < 3
    PyErr_Format(PyExc_TypeError,
        "%.200s() got an unexpected keyword argument '%.200s'",
        function_name, PyString_AsString(key));
    #else
    PyErr_Format(PyExc_TypeError,
        "%s() got an unexpected keyword argument '%U'",
        function_name, key);
    #endif
    return 0;
}

/* RaiseUnexpectedTypeError */
static int
__Pyx_RaiseUnexpectedTypeError(const char *expected, PyObject *obj)
{
    __Pyx_TypeName obj_type_name = __Pyx_PyType_GetName(Py_TYPE(obj));
    PyErr_Format(PyExc_TypeError, "Expected %s, got " __Pyx_FMT_TYPENAME,
                 expected, obj_type_name);
    __Pyx_DECREF_TypeName(obj_type_name);
    return 0;
}

/* GetItemInt */
static PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j) {
    PyObject *r;
    if (unlikely(!j)) return NULL;
    r = PyObject_GetItem(o, j);
    Py_DECREF(j);
    return r;
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              CYTHON_NCP_UNUSED int wraparound,
                                                              CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    Py_ssize_t wrapped_i = i;
    if (wraparound & unlikely(i < 0)) {
        wrapped_i += PyList_GET_SIZE(o);
    }
    if ((!boundscheck) || likely(__Pyx_is_valid_index(wrapped_i, PyList_GET_SIZE(o)))) {
        PyObject *r = PyList_GET_ITEM(o, wrapped_i);
        Py_INCREF(r);
        return r;
    }
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
#else
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              CYTHON_NCP_UNUSED int wraparound,
                                                              CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    Py_ssize_t wrapped_i = i;
    if (wraparound & unlikely(i < 0)) {
        wrapped_i += PyTuple_GET_SIZE(o);
    }
    if ((!boundscheck) || likely(__Pyx_is_valid_index(wrapped_i, PyTuple_GET_SIZE(o)))) {
        PyObject *r = PyTuple_GET_ITEM(o, wrapped_i);
        Py_INCREF(r);
        return r;
    }
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
#else
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i, int is_list,
                                                     CYTHON_NCP_UNUSED int wraparound,
                                                     CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS && CYTHON_USE_TYPE_SLOTS
    if (is_list || PyList_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyList_GET_SIZE(o);
        if ((!boundscheck) || (likely(__Pyx_is_valid_index(n, PyList_GET_SIZE(o))))) {
            PyObject *r = PyList_GET_ITEM(o, n);
            Py_INCREF(r);
            return r;
        }
    }
    else if (PyTuple_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyTuple_GET_SIZE(o);
        if ((!boundscheck) || likely(__Pyx_is_valid_index(n, PyTuple_GET_SIZE(o)))) {
            PyObject *r = PyTuple_GET_ITEM(o, n);
            Py_INCREF(r);
            return r;
        }
    } else {
        PyMappingMethods *mm = Py_TYPE(o)->tp_as_mapping;
        PySequenceMethods *sm = Py_TYPE(o)->tp_as_sequence;
        if (mm && mm->mp_subscript) {
            PyObject *r, *key = PyInt_FromSsize_t(i);
            if (unlikely(!key)) return NULL;
            r = mm->mp_subscript(o, key);
            Py_DECREF(key);
            return r;
        }
        if (likely(sm && sm->sq_item)) {
            if (wraparound && unlikely(i < 0) && likely(sm->sq_length)) {
                Py_ssize_t l = sm->sq_length(o);
                if (likely(l >= 0)) {
                    i += l;
                } else {
                    if (!PyErr_ExceptionMatches(PyExc_OverflowError))
                        return NULL;
                    PyErr_Clear();
                }
            }
            return sm->sq_item(o, i);
        }
    }
#else
    if (is_list || PySequence_Check(o)) {
        return PySequence_GetItem(o, i);
    }
#endif
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
}

/* PyObjectCallOneArg */
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *args[2] = {NULL, arg};
    return __Pyx_PyObject_FastCall(func, args+1, 1 | __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET);
}

/* ObjectGetItem */
#if CYTHON_USE_TYPE_SLOTS
static PyObject *__Pyx_PyObject_GetIndex(PyObject *obj, PyObject *index) {
    PyObject *runerr;
    Py_ssize_t key_value;
    key_value = __Pyx_PyIndex_AsSsize_t(index);
    if (likely(key_value != -1 || !(runerr = PyErr_Occurred()))) {
        return __Pyx_GetItemInt_Fast(obj, key_value, 0, 1, 1);
    }
    if (PyErr_GivenExceptionMatches(runerr, PyExc_OverflowError)) {
        __Pyx_TypeName index_type_name = __Pyx_PyType_GetName(Py_TYPE(index));
        PyErr_Clear();
        PyErr_Format(PyExc_IndexError,
            "cannot fit '" __Pyx_FMT_TYPENAME "' into an index-sized integer", index_type_name);
        __Pyx_DECREF_TypeName(index_type_name);
    }
    return NULL;
}
static PyObject *__Pyx_PyObject_GetItem_Slow(PyObject *obj, PyObject *key) {
    __Pyx_TypeName obj_type_name;
    if (likely(PyType_Check(obj))) {
        PyObject *meth = __Pyx_PyObject_GetAttrStrNoError(obj, __pyx_n_s_class_getitem);
        if (meth) {
            PyObject *result = __Pyx_PyObject_CallOneArg(meth, key);
            Py_DECREF(meth);
            return result;
        }
    }
    obj_type_name = __Pyx_PyType_GetName(Py_TYPE(obj));
    PyErr_Format(PyExc_TypeError,
        "'" __Pyx_FMT_TYPENAME "' object is not subscriptable", obj_type_name);
    __Pyx_DECREF_TypeName(obj_type_name);
    return NULL;
}
static PyObject *__Pyx_PyObject_GetItem(PyObject *obj, PyObject *key) {
    PyTypeObject *tp = Py_TYPE(obj);
    PyMappingMethods *mm = tp->tp_as_mapping;
    PySequenceMethods *sm = tp->tp_as_sequence;
    if (likely(mm && mm->mp_subscript)) {
        return mm->mp_subscript(obj, key);
    }
    if (likely(sm && sm->sq_item)) {
        return __Pyx_PyObject_GetIndex(obj, key);
    }
    return __Pyx_PyObject_GetItem_Slow(obj, key);
}
#endif

/* RaiseTooManyValuesToUnpack */
static CYTHON_INLINE void __Pyx_RaiseTooManyValuesError(Py_ssize_t expected) {
    PyErr_Format(PyExc_ValueError,
                 "too many values to unpack (expected %" CYTHON_FORMAT_SSIZE_T "d)", expected);
}

/* RaiseNeedMoreValuesToUnpack */
static CYTHON_INLINE void __Pyx_RaiseNeedMoreValuesError(Py_ssize_t index) {
    PyErr_Format(PyExc_ValueError,
                 "need more than %" CYTHON_FORMAT_SSIZE_T "d value%.1s to unpack",
                 index, (index == 1) ? "" : "s");
}

/* IterFinish */
static CYTHON_INLINE int __Pyx_IterFinish(void) {
#if CYTHON_FAST_THREAD_STATE
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    PyObject* exc_type = tstate->curexc_type;
    if (unlikely(exc_type)) {
        if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) {
            PyObject *exc_value, *exc_tb;
            exc_value = tstate->curexc_value;
            exc_tb = tstate->curexc_traceback;
            tstate->curexc_type = 0;
            tstate->curexc_value = 0;
            tstate->curexc_traceback = 0;
            Py_DECREF(exc_type);
            Py_XDECREF(exc_value);
            Py_XDECREF(exc_tb);
            return 0;
        } else {
            return -1;
        }
    }
    return 0;
#else
    if (unlikely(PyErr_Occurred())) {
        if (likely(PyErr_ExceptionMatches(PyExc_StopIteration))) {
            PyErr_Clear();
            return 0;
        } else {
            return -1;
        }
    }
    return 0;
#endif
}

/* UnpackItemEndCheck */
static int __Pyx_IternextUnpackEndCheck(PyObject *retval, Py_ssize_t expected) {
    if (unlikely(retval)) {
        Py_DECREF(retval);
        __Pyx_RaiseTooManyValuesError(expected);
        return -1;
    } else {
        return __Pyx_IterFinish();
    }
    return 0;
}

/* RaiseException */
#if PY_MAJOR_VERSION < 3
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause) {
    __Pyx_PyThreadState_declare
    CYTHON_UNUSED_VAR(cause);
    Py_XINCREF(type);
    if (!value || value == Py_None)
        value = NULL;
    else
        Py_INCREF(value);
    if (!tb || tb == Py_None)
        tb = NULL;
    else {
        Py_INCREF(tb);
        if (!PyTraceBack_Check(tb)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: arg 3 must be a traceback or None");
            goto raise_error;
        }
    }
    if (PyType_Check(type)) {
#if CYTHON_COMPILING_IN_PYPY
        if (!value) {
            Py_INCREF(Py_None);
            value = Py_None;
        }
#endif
        PyErr_NormalizeException(&type, &value, &tb);
    } else {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto raise_error;
        }
        value = type;
        type = (PyObject*) Py_TYPE(type);
        Py_INCREF(type);
        if (!PyType_IsSubtype((PyTypeObject *)type, (PyTypeObject *)PyExc_BaseException)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: exception class must be a subclass of BaseException");
            goto raise_error;
        }
    }
    __Pyx_PyThreadState_assign
    __Pyx_ErrRestore(type, value, tb);
    return;
raise_error:
    Py_XDECREF(value);
    Py_XDECREF(type);
    Py_XDECREF(tb);
    return;
}
#else
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause) {
    PyObject* owned_instance = NULL;
    if (tb == Py_None) {
        tb = 0;
    } else if (tb && !PyTraceBack_Check(tb)) {
        PyErr_SetString(PyExc_TypeError,
            "raise: arg 3 must be a traceback or None");
        goto bad;
    }
    if (value == Py_None)
        value = 0;
    if (PyExceptionInstance_Check(type)) {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto bad;
        }
        value = type;
        type = (PyObject*) Py_TYPE(value);
    } else if (PyExceptionClass_Check(type)) {
        PyObject *instance_class = NULL;
        if (value && PyExceptionInstance_Check(value)) {
            instance_class = (PyObject*) Py_TYPE(value);
            if (instance_class != type) {
                int is_subclass = PyObject_IsSubclass(instance_class, type);
                if (!is_subclass) {
                    instance_class = NULL;
                } else if (unlikely(is_subclass == -1)) {
                    goto bad;
                } else {
                    type = instance_class;
                }
            }
        }
        if (!instance_class) {
            PyObject *args;
            if (!value)
                args = PyTuple_New(0);
            else if (PyTuple_Check(value)) {
                Py_INCREF(value);
                args = value;
            } else
                args = PyTuple_Pack(1, value);
            if (!args)
                goto bad;
            owned_instance = PyObject_Call(type, args, NULL);
            Py_DECREF(args);
            if (!owned_instance)
                goto bad;
            value = owned_instance;
            if (!PyExceptionInstance_Check(value)) {
                PyErr_Format(PyExc_TypeError,
                             "calling %R should have returned an instance of "
                             "BaseException, not %R",
                             type, Py_TYPE(value));
                goto bad;
            }
        }
    } else {
        PyErr_SetString(PyExc_TypeError,
            "raise: exception class must be a subclass of BaseException");
        goto bad;
    }
    if (cause) {
        PyObject *fixed_cause;
        if (cause == Py_None) {
            fixed_cause = NULL;
        } else if (PyExceptionClass_Check(cause)) {
            fixed_cause = PyObject_CallObject(cause, NULL);
            if (fixed_cause == NULL)
                goto bad;
        } else if (PyExceptionInstance_Check(cause)) {
            fixed_cause = cause;
            Py_INCREF(fixed_cause);
        } else {
            PyErr_SetString(PyExc_TypeError,
                            "exception causes must derive from "
                            "BaseException");
            goto bad;
        }
        PyException_SetCause(value, fixed_cause);
    }
    PyErr_SetObject(type, value);
    if (tb) {
#if CYTHON_COMPILING_IN_PYPY ||  CYTHON_COMPILING_IN_LIMITED_API
        PyObject *tmp_type, *tmp_value, *tmp_tb;
        PyErr_Fetch(&tmp_type, &tmp_value, &tmp_tb);
        Py_INCREF(tb);
        PyErr_Restore(tmp_type, tmp_value, tb);
        Py_XDECREF(tmp_tb);
#else
        PyThreadState *tstate = __Pyx_PyThreadState_Current;
        PyObject* tmp_tb = tstate->curexc_traceback;
        if (tb != tmp_tb) {
            Py_INCREF(tb);
            tstate->curexc_traceback = tb;
            Py_XDECREF(tmp_tb);
        }
#endif
    }
bad:
    Py_XDECREF(owned_instance);
    return;
}
#endif

/* IterNext */
static PyObject *__Pyx_PyIter_Next2Default(PyObject* defval) {
    PyObject* exc_type;
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    exc_type = __Pyx_PyErr_Occurred();
    if (unlikely(exc_type)) {
        if (!defval || unlikely(!__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration)))
            return NULL;
        __Pyx_PyErr_Clear();
        Py_INCREF(defval);
        return defval;
    }
    if (defval) {
        Py_INCREF(defval);
        return defval;
    }
    __Pyx_PyErr_SetNone(PyExc_StopIteration);
    return NULL;
}
static void __Pyx_PyIter_Next_ErrorNoIterator(PyObject *iterator) {
    __Pyx_TypeName iterator_type_name = __Pyx_PyType_GetName(Py_TYPE(iterator));
    PyErr_Format(PyExc_TypeError,
        __Pyx_FMT_TYPENAME " object is not an iterator", iterator_type_name);
    __Pyx_DECREF_TypeName(iterator_type_name);
}
static CYTHON_INLINE PyObject *__Pyx_PyIter_Next2(PyObject* iterator, PyObject* defval) {
    PyObject* next;
    iternextfunc iternext = Py_TYPE(iterator)->tp_iternext;
    if (likely(iternext)) {
#if CYTHON_USE_TYPE_SLOTS
        next = iternext(iterator);
        if (likely(next))
            return next;
        if (unlikely(iternext == &_PyObject_NextNotImplemented))
            return NULL;
#else
        next = PyIter_Next(iterator);
        if (likely(next))
            return next;
#endif
    } else if (CYTHON_USE_TYPE_SLOTS || unlikely(!PyIter_Check(iterator))) {
        __Pyx_PyIter_Next_ErrorNoIterator(iterator);
        return NULL;
    }
#if !CYTHON_USE_TYPE_SLOTS
    else {
        next = PyIter_Next(iterator);
        if (likely(next))
            return next;
    }
#endif
    return __Pyx_PyIter_Next2Default(defval);
}

/* GetException */
#if CYTHON_FAST_THREAD_STATE
static int __Pyx__GetException(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb)
#else
static int __Pyx_GetException(PyObject **type, PyObject **value, PyObject **tb)
#endif
{
    PyObject *local_type, *local_value, *local_tb;
#if CYTHON_FAST_THREAD_STATE
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    local_type = tstate->curexc_type;
    local_value = tstate->curexc_value;
    local_tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
#else
    PyErr_Fetch(&local_type, &local_value, &local_tb);
#endif
    PyErr_NormalizeException(&local_type, &local_value, &local_tb);
#if CYTHON_FAST_THREAD_STATE
    if (unlikely(tstate->curexc_type))
#else
    if (unlikely(PyErr_Occurred()))
#endif
        goto bad;
    #if PY_MAJOR_VERSION >= 3
    if (local_tb) {
        if (unlikely(PyException_SetTraceback(local_value, local_tb) < 0))
            goto bad;
    }
    #endif
    Py_XINCREF(local_tb);
    Py_XINCREF(local_type);
    Py_XINCREF(local_value);
    *type = local_type;
    *value = local_value;
    *tb = local_tb;
#if CYTHON_FAST_THREAD_STATE
    #if CYTHON_USE_EXC_INFO_STACK
    {
        _PyErr_StackItem *exc_info = tstate->exc_info;
        tmp_type = exc_info->exc_type;
        tmp_value = exc_info->exc_value;
        tmp_tb = exc_info->exc_traceback;
        exc_info->exc_type = local_type;
        exc_info->exc_value = local_value;
        exc_info->exc_traceback = local_tb;
    }
    #else
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = local_type;
    tstate->exc_value = local_value;
    tstate->exc_traceback = local_tb;
    #endif
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
#else
    PyErr_SetExcInfo(local_type, local_value, local_tb);
#endif
    return 0;
bad:
    *type = 0;
    *value = 0;
    *tb = 0;
    Py_XDECREF(local_type);
    Py_XDECREF(local_value);
    Py_XDECREF(local_tb);
    return -1;
}

/* pep479 */
static void __Pyx_Generator_Replace_StopIteration(int in_async_gen) {
    PyObject *exc, *val, *tb, *cur_exc;
    __Pyx_PyThreadState_declare
    #ifdef __Pyx_StopAsyncIteration_USED
    int is_async_stopiteration = 0;
    #endif
    CYTHON_MAYBE_UNUSED_VAR(in_async_gen);
    cur_exc = PyErr_Occurred();
    if (likely(!__Pyx_PyErr_GivenExceptionMatches(cur_exc, PyExc_StopIteration))) {
        #ifdef __Pyx_StopAsyncIteration_USED
        if (in_async_gen && unlikely(__Pyx_PyErr_GivenExceptionMatches(cur_exc, __Pyx_PyExc_StopAsyncIteration))) {
            is_async_stopiteration = 1;
        } else
        #endif
            return;
    }
    __Pyx_PyThreadState_assign
    __Pyx_GetException(&exc, &val, &tb);
    Py_XDECREF(exc);
    Py_XDECREF(val);
    Py_XDECREF(tb);
    PyErr_SetString(PyExc_RuntimeError,
        #ifdef __Pyx_StopAsyncIteration_USED
        is_async_stopiteration ? "async generator raised StopAsyncIteration" :
        in_async_gen ? "async generator raised StopIteration" :
        #endif
        "generator raised StopIteration");
}

/* ExtTypeTest */
static CYTHON_INLINE int __Pyx_TypeTest(PyObject *obj, PyTypeObject *type) {
    __Pyx_TypeName obj_type_name;
    __Pyx_TypeName type_name;
    if (unlikely(!type)) {
        PyErr_SetString(PyExc_SystemError, "Missing type object");
        return 0;
    }
    if (likely(__Pyx_TypeCheck(obj, type)))
        return 1;
    obj_type_name = __Pyx_PyType_GetName(Py_TYPE(obj));
    type_name = __Pyx_PyType_GetName(type);
    PyErr_Format(PyExc_TypeError,
                 "Cannot convert " __Pyx_FMT_TYPENAME " to " __Pyx_FMT_TYPENAME,
                 obj_type_name, type_name);
    __Pyx_DECREF_TypeName(obj_type_name);
    __Pyx_DECREF_TypeName(type_name);
    return 0;
}

/* GetAttr3 */
static PyObject *__Pyx_GetAttr3Default(PyObject *d) {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    if (unlikely(!__Pyx_PyErr_ExceptionMatches(PyExc_AttributeError)))
        return NULL;
    __Pyx_PyErr_Clear();
    Py_INCREF(d);
    return d;
}
static CYTHON_INLINE PyObject *__Pyx_GetAttr3(PyObject *o, PyObject *n, PyObject *d) {
    PyObject *r;
#if CYTHON_USE_TYPE_SLOTS
    if (likely(PyString_Check(n))) {
        r = __Pyx_PyObject_GetAttrStrNoError(o, n);
        if (unlikely(!r) && likely(!PyErr_Occurred())) {
            r = __Pyx_NewRef(d);
        }
        return r;
    }
#endif
    r = PyObject_GetAttr(o, n);
    return (likely(r)) ? r : __Pyx_GetAttr3Default(d);
}

/* GetModuleGlobalName */
#if CYTHON_USE_DICT_VERSIONS
static PyObject *__Pyx__GetModuleGlobalName(PyObject *name, PY_UINT64_T *dict_version, PyObject **dict_cached_value)
#else
static CYTHON_INLINE PyObject *__Pyx__GetModuleGlobalName(PyObject *name)
#endif
{
    PyObject *result;
#if !CYTHON_AVOID_BORROWED_REFS
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030500A1
    result = _PyDict_GetItem_KnownHash(__pyx_d, name, ((PyASCIIObject *) name)->hash);
    __PYX_UPDATE_DICT_CACHE(__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return __Pyx_NewRef(result);
    } else if (unlikely(PyErr_Occurred())) {
        return NULL;
    }
#elif CYTHON_COMPILING_IN_LIMITED_API
    if (unlikely(!__pyx_m)) {
        return NULL;
    }
    result = PyObject_GetAttr(__pyx_m, name);
    if (likely(result)) {
        return result;
    }
#else
    result = PyDict_GetItem(__pyx_d, name);
    __PYX_UPDATE_DICT_CACHE(__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return __Pyx_NewRef(result);
    }
#endif
#else
    result = PyObject_GetItem(__pyx_d, name);
    __PYX_UPDATE_DICT_CACHE(__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return __Pyx_NewRef(result);
    }
    PyErr_Clear();
#endif
    return __Pyx_GetBuiltinName(name);
}

/* Import */
static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level) {
    PyObject *module = 0;
    PyObject *empty_dict = 0;
    PyObject *empty_list = 0;
    #if PY_MAJOR_VERSION < 3
    PyObject *py_import;
    py_import = __Pyx_PyObject_GetAttrStr(__pyx_b, __pyx_n_s_import);
    if (unlikely(!py_import))
        goto bad;
    if (!from_list) {
        empty_list = PyList_New(0);
        if (unlikely(!empty_list))
            goto bad;
        from_list = empty_list;
    }
    #endif
    empty_dict = PyDict_New();
    if (unlikely(!empty_dict))
        goto bad;
    {
        #if PY_MAJOR_VERSION >= 3
        if (level == -1) {
            if ((1) && (strchr(__Pyx_MODULE_NAME, '.'))) {
                #if CYTHON_COMPILING_IN_LIMITED_API
                module = PyImport_ImportModuleLevelObject(
                    name, empty_dict, empty_dict, from_list, 1);
                #else
                module = PyImport_ImportModuleLevelObject(
                    name, __pyx_d, empty_dict, from_list, 1);
                #endif
                if (unlikely(!module)) {
                    if (unlikely(!PyErr_ExceptionMatches(PyExc_ImportError)))
                        goto bad;
                    PyErr_Clear();
                }
            }
            level = 0;
        }
        #endif
        if (!module) {
            #if PY_MAJOR_VERSION < 3
            PyObject *py_level = PyInt_FromLong(level);
            if (unlikely(!py_level))
                goto bad;
            module = PyObject_CallFunctionObjArgs(py_import,
                name, __pyx_d, empty_dict, from_list, py_level, (PyObject *)NULL);
            Py_DECREF(py_level);
            #else
            #if CYTHON_COMPILING_IN_LIMITED_API
            module = PyImport_ImportModuleLevelObject(
                name, empty_dict, empty_dict, from_list, level);
            #else
            module = PyImport_ImportModuleLevelObject(
                name, __pyx_d, empty_dict, from_list, level);
            #endif
            #endif
        }
    }
bad:
    Py_XDECREF(empty_dict);
    Py_XDECREF(empty_list);
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(py_import);
    #endif
    return module;
}

/* ImportFrom */
static PyObject* __Pyx_ImportFrom(PyObject* module, PyObject* name) {
    PyObject* value = __Pyx_PyObject_GetAttrStr(module, name);
    if (unlikely(!value) && PyErr_ExceptionMatches(PyExc_AttributeError)) {
        PyErr_Format(PyExc_ImportError,
        #if PY_MAJOR_VERSION < 3
            "cannot import name %.230s", PyString_AS_STRING(name));
        #else
            "cannot import name %S", name);
        #endif
    }
    return value;
}

/* GetAttr */
static CYTHON_INLINE PyObject *__Pyx_GetAttr(PyObject *o, PyObject *n) {
#if CYTHON_USE_TYPE_SLOTS
#if PY_MAJOR_VERSION >= 3
    if (likely(PyUnicode_Check(n)))
#else
    if (likely(PyString_Check(n)))
#endif
        return __Pyx_PyObject_GetAttrStr(o, n);
#endif
    return PyObject_GetAttr(o, n);
}

/* HasAttr */
static CYTHON_INLINE int __Pyx_HasAttr(PyObject *o, PyObject *n) {
    PyObject *r;
    if (unlikely(!__Pyx_PyBaseString_Check(n))) {
        PyErr_SetString(PyExc_TypeError,
                        "hasattr(): attribute name must be string");
        return -1;
    }
    r = __Pyx_GetAttr(o, n);
    if (!r) {
        PyErr_Clear();
        return 0;
    } else {
        Py_DECREF(r);
        return 1;
    }
}

/* CallNextTpDealloc */
static void __Pyx_call_next_tp_dealloc(PyObject* obj, destructor current_tp_dealloc) {
    PyTypeObject* type = Py_TYPE(obj);
    while (type && type->tp_dealloc != current_tp_dealloc)
        type = type->tp_base;
    while (type && type->tp_dealloc == current_tp_dealloc)
        type = type->tp_base;
    if (type)
        type->tp_dealloc(obj);
}

/* CallNextTpTraverse */
static int __Pyx_call_next_tp_traverse(PyObject* obj, visitproc v, void *a, traverseproc current_tp_traverse) {
    PyTypeObject* type = Py_TYPE(obj);
    while (type && type->tp_traverse != current_tp_traverse)
        type = type->tp_base;
    while (type && type->tp_traverse == current_tp_traverse)
        type = type->tp_base;
    if (type && type->tp_traverse)
        return type->tp_traverse(obj, v, a);
    return 0;
}

/* CallNextTpClear */
static void __Pyx_call_next_tp_clear(PyObject* obj, inquiry current_tp_clear) {
    PyTypeObject* type = Py_TYPE(obj);
    while (type && type->tp_clear != current_tp_clear)
        type = type->tp_base;
    while (type && type->tp_clear == current_tp_clear)
        type = type->tp_base;
    if (type && type->tp_clear)
        type->tp_clear(obj);
}

/* TypeImport */
#ifndef __PYX_HAVE_RT_ImportType
#define __PYX_HAVE_RT_ImportType
static PyTypeObject *__Pyx_ImportType(PyObject *module, const char *module_name, const char *class_name,
    size_t size, enum __Pyx_ImportType_CheckSize check_size)
{
    PyObject *result = 0;
    char warning[200];
    Py_ssize_t basicsize;
#if CYTHON_COMPILING_IN_LIMITED_API
    PyObject *py_basicsize;
#endif
    result = PyObject_GetAttrString(module, class_name);
    if (!result)
        goto bad;
    if (!PyType_Check(result)) {
        PyErr_Format(PyExc_TypeError,
            "%.200s.%.200s is not a type object",
            module_name, class_name);
        goto bad;
    }
#if !CYTHON_COMPILING_IN_LIMITED_API
    basicsize = ((PyTypeObject *)result)->tp_basicsize;
#else
    py_basicsize = PyObject_GetAttrString(result, "__basicsize__");
    if (!py_basicsize)
        goto bad;
    basicsize = PyLong_AsSsize_t(py_basicsize);
    Py_DECREF(py_basicsize);
    py_basicsize = 0;
    if (basicsize == (Py_ssize_t)-1 && PyErr_Occurred())
        goto bad;
#endif
    if ((size_t)basicsize < size) {
        PyErr_Format(PyExc_ValueError,
            "%.200s.%.200s size changed, may indicate binary incompatibility. "
            "Expected %zd from C header, got %zd from PyObject",
            module_name, class_name, size, basicsize);
        goto bad;
    }
    if (check_size == __Pyx_ImportType_CheckSize_Error && (size_t)basicsize != size) {
        PyErr_Format(PyExc_ValueError,
            "%.200s.%.200s size changed, may indicate binary incompatibility. "
            "Expected %zd from C header, got %zd from PyObject",
            module_name, class_name, size, basicsize);
        goto bad;
    }
    else if (check_size == __Pyx_ImportType_CheckSize_Warn && (size_t)basicsize > size) {
        PyOS_snprintf(warning, sizeof(warning),
            "%s.%s size changed, may indicate binary incompatibility. "
            "Expected %zd from C header, got %zd from PyObject",
            module_name, class_name, size, basicsize);
        if (PyErr_WarnEx(NULL, warning, 0) < 0) goto bad;
    }
    return (PyTypeObject *)result;
bad:
    Py_XDECREF(result);
    return NULL;
}
#endif

/* GetVTable */
static void* __Pyx_GetVtable(PyTypeObject *type) {
    void* ptr;
#if CYTHON_COMPILING_IN_LIMITED_API
    PyObject *ob = PyObject_GetAttr((PyObject *)type, __pyx_n_s_pyx_vtable);
#else
    PyObject *ob = PyObject_GetItem(type->tp_dict, __pyx_n_s_pyx_vtable);
#endif
    if (!ob)
        goto bad;
    ptr = PyCapsule_GetPointer(ob, 0);
    if (!ptr && !PyErr_Occurred())
        PyErr_SetString(PyExc_RuntimeError, "invalid vtable found for imported type");
    Py_DECREF(ob);
    return ptr;
bad:
    Py_XDECREF(ob);
    return NULL;
}

/* FixUpExtensionType */
#if CYTHON_USE_TYPE_SPECS
static int __Pyx_fix_up_extension_type_from_spec(PyType_Spec *spec, PyTypeObject *type) {
#if PY_VERSION_HEX > 0x030900B1 || CYTHON_COMPILING_IN_LIMITED_API
    (void) spec;
    (void) type;
#else
    const PyType_Slot *slot = spec->slots;
    while (slot && slot->slot && slot->slot != Py_tp_members)
        slot++;
    if (slot && slot->slot == Py_tp_members) {
        int changed = 0;
#if !(PY_VERSION_HEX <= 0x030900b1 && CYTHON_COMPILING_IN_CPYTHON)
        const
#endif
            PyMemberDef *memb = (PyMemberDef*) slot->pfunc;
        while (memb && memb->name) {
            if (memb->name[0] == '_' && memb->name[1] == '_') {
#if PY_VERSION_HEX < 0x030900b1
                if (strcmp(memb->name, "__weaklistoffset__") == 0) {
                    assert(memb->type == T_PYSSIZET);
                    assert(memb->flags == READONLY);
                    type->tp_weaklistoffset = memb->offset;
                    changed = 1;
                }
                else if (strcmp(memb->name, "__dictoffset__") == 0) {
                    assert(memb->type == T_PYSSIZET);
                    assert(memb->flags == READONLY);
                    type->tp_dictoffset = memb->offset;
                    changed = 1;
                }
#if CYTHON_METH_FASTCALL
                else if (strcmp(memb->name, "__vectorcalloffset__") == 0) {
                    assert(memb->type == T_PYSSIZET);
                    assert(memb->flags == READONLY);
#if PY_VERSION_HEX >= 0x030800b4
                    type->tp_vectorcall_offset = memb->offset;
#else
                    type->tp_print = (printfunc) memb->offset;
#endif
                    changed = 1;
                }
#endif
#else
                if ((0));
#endif
#if PY_VERSION_HEX <= 0x030900b1 && CYTHON_COMPILING_IN_CPYTHON
                else if (strcmp(memb->name, "__module__") == 0) {
                    PyObject *descr;
                    assert(memb->type == T_OBJECT);
                    assert(memb->flags == 0 || memb->flags == READONLY);
                    descr = PyDescr_NewMember(type, memb);
                    if (unlikely(!descr))
                        return -1;
                    if (unlikely(PyDict_SetItem(type->tp_dict, PyDescr_NAME(descr), descr) < 0)) {
                        Py_DECREF(descr);
                        return -1;
                    }
                    Py_DECREF(descr);
                    changed = 1;
                }
#endif
            }
            memb++;
        }
        if (changed)
            PyType_Modified(type);
    }
#endif
    return 0;
}
#endif

/* PyObjectCallNoArg */
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func) {
    PyObject *arg = NULL;
    return __Pyx_PyObject_FastCall(func, (&arg)+1, 0 | __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET);
}

/* PyObjectGetMethod */
static int __Pyx_PyObject_GetMethod(PyObject *obj, PyObject *name, PyObject **method) {
    PyObject *attr;
#if CYTHON_UNPACK_METHODS && CYTHON_COMPILING_IN_CPYTHON && CYTHON_USE_PYTYPE_LOOKUP
    __Pyx_TypeName type_name;
    PyTypeObject *tp = Py_TYPE(obj);
    PyObject *descr;
    descrgetfunc f = NULL;
    PyObject **dictptr, *dict;
    int meth_found = 0;
    assert (*method == NULL);
    if (unlikely(tp->tp_getattro != PyObject_GenericGetAttr)) {
        attr = __Pyx_PyObject_GetAttrStr(obj, name);
        goto try_unpack;
    }
    if (unlikely(tp->tp_dict == NULL) && unlikely(PyType_Ready(tp) < 0)) {
        return 0;
    }
    descr = _PyType_Lookup(tp, name);
    if (likely(descr != NULL)) {
        Py_INCREF(descr);
#if defined(Py_TPFLAGS_METHOD_DESCRIPTOR) && Py_TPFLAGS_METHOD_DESCRIPTOR
        if (__Pyx_PyType_HasFeature(Py_TYPE(descr), Py_TPFLAGS_METHOD_DESCRIPTOR))
#elif PY_MAJOR_VERSION >= 3
        #ifdef __Pyx_CyFunction_USED
        if (likely(PyFunction_Check(descr) || __Pyx_IS_TYPE(descr, &PyMethodDescr_Type) || __Pyx_CyFunction_Check(descr)))
        #else
        if (likely(PyFunction_Check(descr) || __Pyx_IS_TYPE(descr, &PyMethodDescr_Type)))
        #endif
#else
        #ifdef __Pyx_CyFunction_USED
        if (likely(PyFunction_Check(descr) || __Pyx_CyFunction_Check(descr)))
        #else
        if (likely(PyFunction_Check(descr)))
        #endif
#endif
        {
            meth_found = 1;
        } else {
            f = Py_TYPE(descr)->tp_descr_get;
            if (f != NULL && PyDescr_IsData(descr)) {
                attr = f(descr, obj, (PyObject *)Py_TYPE(obj));
                Py_DECREF(descr);
                goto try_unpack;
            }
        }
    }
    dictptr = _PyObject_GetDictPtr(obj);
    if (dictptr != NULL && (dict = *dictptr) != NULL) {
        Py_INCREF(dict);
        attr = __Pyx_PyDict_GetItemStr(dict, name);
        if (attr != NULL) {
            Py_INCREF(attr);
            Py_DECREF(dict);
            Py_XDECREF(descr);
            goto try_unpack;
        }
        Py_DECREF(dict);
    }
    if (meth_found) {
        *method = descr;
        return 1;
    }
    if (f != NULL) {
        attr = f(descr, obj, (PyObject *)Py_TYPE(obj));
        Py_DECREF(descr);
        goto try_unpack;
    }
    if (likely(descr != NULL)) {
        *method = descr;
        return 0;
    }
    type_name = __Pyx_PyType_GetName(tp);
    PyErr_Format(PyExc_AttributeError,
#if PY_MAJOR_VERSION >= 3
                 "'" __Pyx_FMT_TYPENAME "' object has no attribute '%U'",
                 type_name, name);
#else
                 "'" __Pyx_FMT_TYPENAME "' object has no attribute '%.400s'",
                 type_name, PyString_AS_STRING(name));
#endif
    __Pyx_DECREF_TypeName(type_name);
    return 0;
#else
    attr = __Pyx_PyObject_GetAttrStr(obj, name);
    goto try_unpack;
#endif
try_unpack:
#if CYTHON_UNPACK_METHODS
    if (likely(attr) && PyMethod_Check(attr) && likely(PyMethod_GET_SELF(attr) == obj)) {
        PyObject *function = PyMethod_GET_FUNCTION(attr);
        Py_INCREF(function);
        Py_DECREF(attr);
        *method = function;
        return 1;
    }
#endif
    *method = attr;
    return 0;
}

/* PyObjectCallMethod0 */
static PyObject* __Pyx_PyObject_CallMethod0(PyObject* obj, PyObject* method_name) {
    PyObject *method = NULL, *result = NULL;
    int is_method = __Pyx_PyObject_GetMethod(obj, method_name, &method);
    if (likely(is_method)) {
        result = __Pyx_PyObject_CallOneArg(method, obj);
        Py_DECREF(method);
        return result;
    }
    if (unlikely(!method)) goto bad;
    result = __Pyx_PyObject_CallNoArg(method);
    Py_DECREF(method);
bad:
    return result;
}

/* ValidateBasesTuple */
#if CYTHON_COMPILING_IN_CPYTHON || CYTHON_COMPILING_IN_LIMITED_API || CYTHON_USE_TYPE_SPECS
static int __Pyx_validate_bases_tuple(const char *type_name, Py_ssize_t dictoffset, PyObject *bases) {
    Py_ssize_t i, n = PyTuple_GET_SIZE(bases);
    for (i = 1; i < n; i++)
    {
        PyObject *b0 = PyTuple_GET_ITEM(bases, i);
        PyTypeObject *b;
#if PY_MAJOR_VERSION < 3
        if (PyClass_Check(b0))
        {
            PyErr_Format(PyExc_TypeError, "base class '%.200s' is an old-style class",
                         PyString_AS_STRING(((PyClassObject*)b0)->cl_name));
            return -1;
        }
#endif
        b = (PyTypeObject*) b0;
        if (!__Pyx_PyType_HasFeature(b, Py_TPFLAGS_HEAPTYPE))
        {
            __Pyx_TypeName b_name = __Pyx_PyType_GetName(b);
            PyErr_Format(PyExc_TypeError,
                "base class '" __Pyx_FMT_TYPENAME "' is not a heap type", b_name);
            __Pyx_DECREF_TypeName(b_name);
            return -1;
        }
        if (dictoffset == 0 && b->tp_dictoffset)
        {
            __Pyx_TypeName b_name = __Pyx_PyType_GetName(b);
            PyErr_Format(PyExc_TypeError,
                "extension type '%.200s' has no __dict__ slot, "
                "but base type '" __Pyx_FMT_TYPENAME "' has: "
                "either add 'cdef dict __dict__' to the extension type "
                "or add '__slots__ = [...]' to the base type",
                type_name, b_name);
            __Pyx_DECREF_TypeName(b_name);
            return -1;
        }
    }
    return 0;
}
#endif

/* PyType_Ready */
static int __Pyx_PyType_Ready(PyTypeObject *t) {
#if CYTHON_USE_TYPE_SPECS || !(CYTHON_COMPILING_IN_CPYTHON || CYTHON_COMPILING_IN_LIMITED_API) || defined(PYSTON_MAJOR_VERSION)
    (void)__Pyx_PyObject_CallMethod0;
#if CYTHON_USE_TYPE_SPECS
    (void)__Pyx_validate_bases_tuple;
#endif
    return PyType_Ready(t);
#else
    int r;
    if (t->tp_bases && unlikely(__Pyx_validate_bases_tuple(t->tp_name, t->tp_dictoffset, t->tp_bases) == -1))
        return -1;
#if PY_VERSION_HEX >= 0x03050000
    {
        int gc_was_enabled;
    #if PY_VERSION_HEX >= 0x030A00b1
        gc_was_enabled = PyGC_Disable();
        (void)__Pyx_PyObject_CallMethod0;
    #else
        PyObject *ret, *py_status;
        PyObject *gc = NULL;
        #if PY_VERSION_HEX >= 0x030700a1 && (!CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM+0 >= 0x07030400)
        gc = PyImport_GetModule(__pyx_kp_u_gc);
        #endif
        if (unlikely(!gc)) gc = PyImport_Import(__pyx_kp_u_gc);
        if (unlikely(!gc)) return -1;
        py_status = __Pyx_PyObject_CallMethod0(gc, __pyx_kp_u_isenabled);
        if (unlikely(!py_status)) {
            Py_DECREF(gc);
            return -1;
        }
        gc_was_enabled = __Pyx_PyObject_IsTrue(py_status);
        Py_DECREF(py_status);
        if (gc_was_enabled > 0) {
            ret = __Pyx_PyObject_CallMethod0(gc, __pyx_kp_u_disable);
            if (unlikely(!ret)) {
                Py_DECREF(gc);
                return -1;
            }
            Py_DECREF(ret);
        } else if (unlikely(gc_was_enabled == -1)) {
            Py_DECREF(gc);
            return -1;
        }
    #endif
        t->tp_flags |= Py_TPFLAGS_HEAPTYPE;
#else
        (void)__Pyx_PyObject_CallMethod0;
#endif
    r = PyType_Ready(t);
#if PY_VERSION_HEX >= 0x03050000
        t->tp_flags &= ~Py_TPFLAGS_HEAPTYPE;
    #if PY_VERSION_HEX >= 0x030A00b1
        if (gc_was_enabled)
            PyGC_Enable();
    #else
        if (gc_was_enabled) {
            PyObject *tp, *v, *tb;
            PyErr_Fetch(&tp, &v, &tb);
            ret = __Pyx_PyObject_CallMethod0(gc, __pyx_kp_u_enable);
            if (likely(ret || r == -1)) {
                Py_XDECREF(ret);
                PyErr_Restore(tp, v, tb);
            } else {
                Py_XDECREF(tp);
                Py_XDECREF(v);
                Py_XDECREF(tb);
                r = -1;
            }
        }
        Py_DECREF(gc);
    #endif
    }
#endif
    return r;
#endif
}

/* PyObject_GenericGetAttrNoDict */
#if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static PyObject *__Pyx_RaiseGenericGetAttributeError(PyTypeObject *tp, PyObject *attr_name) {
    __Pyx_TypeName type_name = __Pyx_PyType_GetName(tp);
    PyErr_Format(PyExc_AttributeError,
#if PY_MAJOR_VERSION >= 3
                 "'" __Pyx_FMT_TYPENAME "' object has no attribute '%U'",
                 type_name, attr_name);
#else
                 "'" __Pyx_FMT_TYPENAME "' object has no attribute '%.400s'",
                 type_name, PyString_AS_STRING(attr_name));
#endif
    __Pyx_DECREF_TypeName(type_name);
    return NULL;
}
static CYTHON_INLINE PyObject* __Pyx_PyObject_GenericGetAttrNoDict(PyObject* obj, PyObject* attr_name) {
    PyObject *descr;
    PyTypeObject *tp = Py_TYPE(obj);
    if (unlikely(!PyString_Check(attr_name))) {
        return PyObject_GenericGetAttr(obj, attr_name);
    }
    assert(!tp->tp_dictoffset);
    descr = _PyType_Lookup(tp, attr_name);
    if (unlikely(!descr)) {
        return __Pyx_RaiseGenericGetAttributeError(tp, attr_name);
    }
    Py_INCREF(descr);
    #if PY_MAJOR_VERSION < 3
    if (likely(PyType_HasFeature(Py_TYPE(descr), Py_TPFLAGS_HAVE_CLASS)))
    #endif
    {
        descrgetfunc f = Py_TYPE(descr)->tp_descr_get;
        if (unlikely(f)) {
            PyObject *res = f(descr, obj, (PyObject *)tp);
            Py_DECREF(descr);
            return res;
        }
    }
    return descr;
}
#endif

/* PyObject_GenericGetAttr */
#if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static PyObject* __Pyx_PyObject_GenericGetAttr(PyObject* obj, PyObject* attr_name) {
    if (unlikely(Py_TYPE(obj)->tp_dictoffset)) {
        return PyObject_GenericGetAttr(obj, attr_name);
    }
    return __Pyx_PyObject_GenericGetAttrNoDict(obj, attr_name);
}
#endif

/* SetVTable */
static int __Pyx_SetVtable(PyTypeObject *type, void *vtable) {
    PyObject *ob = PyCapsule_New(vtable, 0, 0);
    if (unlikely(!ob))
        goto bad;
#if CYTHON_COMPILING_IN_LIMITED_API
    if (unlikely(PyObject_SetAttr((PyObject *) type, __pyx_n_s_pyx_vtable, ob) < 0))
#else
    if (unlikely(PyDict_SetItem(type->tp_dict, __pyx_n_s_pyx_vtable, ob) < 0))
#endif
        goto bad;
    Py_DECREF(ob);
    return 0;
bad:
    Py_XDECREF(ob);
    return -1;
}

/* MergeVTables */
#if !CYTHON_COMPILING_IN_LIMITED_API
static int __Pyx_MergeVtables(PyTypeObject *type) {
    int i;
    void** base_vtables;
    __Pyx_TypeName tp_base_name;
    __Pyx_TypeName base_name;
    void* unknown = (void*)-1;
    PyObject* bases = type->tp_bases;
    int base_depth = 0;
    {
        PyTypeObject* base = type->tp_base;
        while (base) {
            base_depth += 1;
            base = base->tp_base;
        }
    }
    base_vtables = (void**) malloc(sizeof(void*) * (base_depth + 1));
    base_vtables[0] = unknown;
    for (i = 1; i < PyTuple_GET_SIZE(bases); i++) {
        void* base_vtable = __Pyx_GetVtable(((PyTypeObject*)PyTuple_GET_ITEM(bases, i)));
        if (base_vtable != NULL) {
            int j;
            PyTypeObject* base = type->tp_base;
            for (j = 0; j < base_depth; j++) {
                if (base_vtables[j] == unknown) {
                    base_vtables[j] = __Pyx_GetVtable(base);
                    base_vtables[j + 1] = unknown;
                }
                if (base_vtables[j] == base_vtable) {
                    break;
                } else if (base_vtables[j] == NULL) {
                    goto bad;
                }
                base = base->tp_base;
            }
        }
    }
    PyErr_Clear();
    free(base_vtables);
    return 0;
bad:
    tp_base_name = __Pyx_PyType_GetName(type->tp_base);
    base_name = __Pyx_PyType_GetName((PyTypeObject*)PyTuple_GET_ITEM(bases, i));
    PyErr_Format(PyExc_TypeError,
        "multiple bases have vtable conflict: '" __Pyx_FMT_TYPENAME "' and '" __Pyx_FMT_TYPENAME "'", tp_base_name, base_name);
    __Pyx_DECREF_TypeName(tp_base_name);
    __Pyx_DECREF_TypeName(base_name);
    free(base_vtables);
    return -1;
}
#endif

/* SetupReduce */
#if !CYTHON_COMPILING_IN_LIMITED_API
static int __Pyx_setup_reduce_is_named(PyObject* meth, PyObject* name) {
  int ret;
  PyObject *name_attr;
  name_attr = __Pyx_PyObject_GetAttrStrNoError(meth, __pyx_n_s_name);
  if (likely(name_attr)) {
      ret = PyObject_RichCompareBool(name_attr, name, Py_EQ);
  } else {
      ret = -1;
  }
  if (unlikely(ret < 0)) {
      PyErr_Clear();
      ret = 0;
  }
  Py_XDECREF(name_attr);
  return ret;
}
static int __Pyx_setup_reduce(PyObject* type_obj) {
    int ret = 0;
    PyObject *object_reduce = NULL;
    PyObject *object_reduce_ex = NULL;
    PyObject *reduce = NULL;
    PyObject *reduce_ex = NULL;
    PyObject *reduce_cython = NULL;
    PyObject *setstate = NULL;
    PyObject *setstate_cython = NULL;
#if CYTHON_USE_PYTYPE_LOOKUP
    if (_PyType_Lookup((PyTypeObject*)type_obj, __pyx_n_s_getstate)) goto __PYX_GOOD;
#else
    if (PyObject_HasAttr(type_obj, __pyx_n_s_getstate)) goto __PYX_GOOD;
#endif
#if CYTHON_USE_PYTYPE_LOOKUP
    object_reduce_ex = _PyType_Lookup(&PyBaseObject_Type, __pyx_n_s_reduce_ex); if (!object_reduce_ex) goto __PYX_BAD;
#else
    object_reduce_ex = __Pyx_PyObject_GetAttrStr((PyObject*)&PyBaseObject_Type, __pyx_n_s_reduce_ex); if (!object_reduce_ex) goto __PYX_BAD;
#endif
    reduce_ex = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_reduce_ex); if (unlikely(!reduce_ex)) goto __PYX_BAD;
    if (reduce_ex == object_reduce_ex) {
#if CYTHON_USE_PYTYPE_LOOKUP
        object_reduce = _PyType_Lookup(&PyBaseObject_Type, __pyx_n_s_reduce); if (!object_reduce) goto __PYX_BAD;
#else
        object_reduce = __Pyx_PyObject_GetAttrStr((PyObject*)&PyBaseObject_Type, __pyx_n_s_reduce); if (!object_reduce) goto __PYX_BAD;
#endif
        reduce = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_reduce); if (unlikely(!reduce)) goto __PYX_BAD;
        if (reduce == object_reduce || __Pyx_setup_reduce_is_named(reduce, __pyx_n_s_reduce_cython)) {
            reduce_cython = __Pyx_PyObject_GetAttrStrNoError(type_obj, __pyx_n_s_reduce_cython);
            if (likely(reduce_cython)) {
                ret = PyDict_SetItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_reduce, reduce_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
                ret = PyDict_DelItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_reduce_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
            } else if (reduce == object_reduce || PyErr_Occurred()) {
                goto __PYX_BAD;
            }
            setstate = __Pyx_PyObject_GetAttrStrNoError(type_obj, __pyx_n_s_setstate);
            if (!setstate) PyErr_Clear();
            if (!setstate || __Pyx_setup_reduce_is_named(setstate, __pyx_n_s_setstate_cython)) {
                setstate_cython = __Pyx_PyObject_GetAttrStrNoError(type_obj, __pyx_n_s_setstate_cython);
                if (likely(setstate_cython)) {
                    ret = PyDict_SetItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_setstate, setstate_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
                    ret = PyDict_DelItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_setstate_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
                } else if (!setstate || PyErr_Occurred()) {
                    goto __PYX_BAD;
                }
            }
            PyType_Modified((PyTypeObject*)type_obj);
        }
    }
    goto __PYX_GOOD;
__PYX_BAD:
    if (!PyErr_Occurred()) {
        __Pyx_TypeName type_obj_name =
            __Pyx_PyType_GetName((PyTypeObject*)type_obj);
        PyErr_Format(PyExc_RuntimeError,
            "Unable to initialize pickling for " __Pyx_FMT_TYPENAME, type_obj_name);
        __Pyx_DECREF_TypeName(type_obj_name);
    }
    ret = -1;
__PYX_GOOD:
#if !CYTHON_USE_PYTYPE_LOOKUP
    Py_XDECREF(object_reduce);
    Py_XDECREF(object_reduce_ex);
#endif
    Py_XDECREF(reduce);
    Py_XDECREF(reduce_ex);
    Py_XDECREF(reduce_cython);
    Py_XDECREF(setstate);
    Py_XDECREF(setstate_cython);
    return ret;
}
#endif

/* FetchCommonType */
static PyObject *__Pyx_FetchSharedCythonABIModule(void) {
    PyObject *abi_module = PyImport_AddModule((char*) __PYX_ABI_MODULE_NAME);
    if (!abi_module) return NULL;
    Py_INCREF(abi_module);
    return abi_module;
}
static int __Pyx_VerifyCachedType(PyObject *cached_type,
                               const char *name,
                               Py_ssize_t basicsize,
                               Py_ssize_t expected_basicsize) {
    if (!PyType_Check(cached_type)) {
        PyErr_Format(PyExc_TypeError,
            "Shared Cython type %.200s is not a type object", name);
        return -1;
    }
    if (basicsize != expected_basicsize) {
        PyErr_Format(PyExc_TypeError,
            "Shared Cython type %.200s has the wrong size, try recompiling",
            name);
        return -1;
    }
    return 0;
}
#if !CYTHON_USE_TYPE_SPECS
static PyTypeObject* __Pyx_FetchCommonType(PyTypeObject* type) {
    PyObject* abi_module;
    PyTypeObject *cached_type = NULL;
    abi_module = __Pyx_FetchSharedCythonABIModule();
    if (!abi_module) return NULL;
    cached_type = (PyTypeObject*) PyObject_GetAttrString(abi_module, type->tp_name);
    if (cached_type) {
        if (__Pyx_VerifyCachedType(
              (PyObject *)cached_type,
              type->tp_name,
              cached_type->tp_basicsize,
              type->tp_basicsize) < 0) {
            goto bad;
        }
        goto done;
    }
    if (!PyErr_ExceptionMatches(PyExc_AttributeError)) goto bad;
    PyErr_Clear();
    if (PyType_Ready(type) < 0) goto bad;
    if (PyObject_SetAttrString(abi_module, type->tp_name, (PyObject *)type) < 0)
        goto bad;
    Py_INCREF(type);
    cached_type = type;
done:
    Py_DECREF(abi_module);
    return cached_type;
bad:
    Py_XDECREF(cached_type);
    cached_type = NULL;
    goto done;
}
#else
static PyTypeObject *__Pyx_FetchCommonTypeFromSpec(PyObject *module, PyType_Spec *spec, PyObject *bases) {
    PyObject *abi_module, *cached_type = NULL;
    abi_module = __Pyx_FetchSharedCythonABIModule();
    if (!abi_module) return NULL;
    cached_type = PyObject_GetAttrString(abi_module, spec->name);
    if (cached_type) {
        Py_ssize_t basicsize;
#if CYTHON_COMPILING_IN_LIMITED_API
        PyObject *py_basicsize;
        py_basicsize = PyObject_GetAttrString(cached_type, "__basicsize__");
        if (unlikely(!py_basicsize)) goto bad;
        basicsize = PyLong_AsSsize_t(py_basicsize);
        Py_DECREF(py_basicsize);
        py_basicsize = 0;
        if (unlikely(basicsize == (Py_ssize_t)-1) && PyErr_Occurred()) goto bad;
#else
        basicsize = likely(PyType_Check(cached_type)) ? ((PyTypeObject*) cached_type)->tp_basicsize : -1;
#endif
        if (__Pyx_VerifyCachedType(
              cached_type,
              spec->name,
              basicsize,
              spec->basicsize) < 0) {
            goto bad;
        }
        goto done;
    }
    if (!PyErr_ExceptionMatches(PyExc_AttributeError)) goto bad;
    PyErr_Clear();
    (void) module;
    cached_type = __Pyx_PyType_FromModuleAndSpec(abi_module, spec, bases);
    if (unlikely(!cached_type)) goto bad;
    if (unlikely(__Pyx_fix_up_extension_type_from_spec(spec, (PyTypeObject *) cached_type) < 0)) goto bad;
    if (PyObject_SetAttrString(abi_module, spec->name, cached_type) < 0) goto bad;
done:
    Py_DECREF(abi_module);
    assert(cached_type == NULL || PyType_Check(cached_type));
    return (PyTypeObject *) cached_type;
bad:
    Py_XDECREF(cached_type);
    cached_type = NULL;
    goto done;
}
#endif

/* PyVectorcallFastCallDict */
#if CYTHON_METH_FASTCALL
static PyObject *__Pyx_PyVectorcall_FastCallDict_kw(PyObject *func, __pyx_vectorcallfunc vc, PyObject *const *args, size_t nargs, PyObject *kw)
{
    PyObject *res = NULL;
    PyObject *kwnames;
    PyObject **newargs;
    PyObject **kwvalues;
    Py_ssize_t i, pos;
    size_t j;
    PyObject *key, *value;
    unsigned long keys_are_strings;
    Py_ssize_t nkw = PyDict_GET_SIZE(kw);
    newargs = (PyObject **)PyMem_Malloc((nargs + (size_t)nkw) * sizeof(args[0]));
    if (unlikely(newargs == NULL)) {
        PyErr_NoMemory();
        return NULL;
    }
    for (j = 0; j < nargs; j++) newargs[j] = args[j];
    kwnames = PyTuple_New(nkw);
    if (unlikely(kwnames == NULL)) {
        PyMem_Free(newargs);
        return NULL;
    }
    kwvalues = newargs + nargs;
    pos = i = 0;
    keys_are_strings = Py_TPFLAGS_UNICODE_SUBCLASS;
    while (PyDict_Next(kw, &pos, &key, &value)) {
        keys_are_strings &= Py_TYPE(key)->tp_flags;
        Py_INCREF(key);
        Py_INCREF(value);
        PyTuple_SET_ITEM(kwnames, i, key);
        kwvalues[i] = value;
        i++;
    }
    if (unlikely(!keys_are_strings)) {
        PyErr_SetString(PyExc_TypeError, "keywords must be strings");
        goto cleanup;
    }
    res = vc(func, newargs, nargs, kwnames);
cleanup:
    Py_DECREF(kwnames);
    for (i = 0; i < nkw; i++)
        Py_DECREF(kwvalues[i]);
    PyMem_Free(newargs);
    return res;
}
static CYTHON_INLINE PyObject *__Pyx_PyVectorcall_FastCallDict(PyObject *func, __pyx_vectorcallfunc vc, PyObject *const *args, size_t nargs, PyObject *kw)
{
    if (likely(kw == NULL) || PyDict_GET_SIZE(kw) == 0) {
        return vc(func, args, nargs, NULL);
    }
    return __Pyx_PyVectorcall_FastCallDict_kw(func, vc, args, nargs, kw);
}
#endif

/* CythonFunctionShared */
static CYTHON_INLINE void __Pyx__CyFunction_SetClassObj(__pyx_CyFunctionObject* f, PyObject* classobj) {
#if PY_VERSION_HEX < 0x030900B1
    __Pyx_Py_XDECREF_SET(
        __Pyx_CyFunction_GetClassObj(f),
            ((classobj) ? __Pyx_NewRef(classobj) : NULL));
#else
    __Pyx_Py_XDECREF_SET(
        ((PyCMethodObject *) (f))->mm_class,
        (PyTypeObject*)((classobj) ? __Pyx_NewRef(classobj) : NULL));
#endif
}
static PyObject *
__Pyx_CyFunction_get_doc(__pyx_CyFunctionObject *op, void *closure)
{
    CYTHON_UNUSED_VAR(closure);
    if (unlikely(op->func_doc == NULL)) {
        if (((PyCFunctionObject*)op)->m_ml->ml_doc) {
#if PY_MAJOR_VERSION >= 3
            op->func_doc = PyUnicode_FromString(((PyCFunctionObject*)op)->m_ml->ml_doc);
#else
            op->func_doc = PyString_FromString(((PyCFunctionObject*)op)->m_ml->ml_doc);
#endif
            if (unlikely(op->func_doc == NULL))
                return NULL;
        } else {
            Py_INCREF(Py_None);
            return Py_None;
        }
    }
    Py_INCREF(op->func_doc);
    return op->func_doc;
}
static int
__Pyx_CyFunction_set_doc(__pyx_CyFunctionObject *op, PyObject *value, void *context)
{
    CYTHON_UNUSED_VAR(context);
    if (value == NULL) {
        value = Py_None;
    }
    Py_INCREF(value);
    __Pyx_Py_XDECREF_SET(op->func_doc, value);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_name(__pyx_CyFunctionObject *op, void *context)
{
    CYTHON_UNUSED_VAR(context);
    if (unlikely(op->func_name == NULL)) {
#if PY_MAJOR_VERSION >= 3
        op->func_name = PyUnicode_InternFromString(((PyCFunctionObject*)op)->m_ml->ml_name);
#else
        op->func_name = PyString_InternFromString(((PyCFunctionObject*)op)->m_ml->ml_name);
#endif
        if (unlikely(op->func_name == NULL))
            return NULL;
    }
    Py_INCREF(op->func_name);
    return op->func_name;
}
static int
__Pyx_CyFunction_set_name(__pyx_CyFunctionObject *op, PyObject *value, void *context)
{
    CYTHON_UNUSED_VAR(context);
#if PY_MAJOR_VERSION >= 3
    if (unlikely(value == NULL || !PyUnicode_Check(value)))
#else
    if (unlikely(value == NULL || !PyString_Check(value)))
#endif
    {
        PyErr_SetString(PyExc_TypeError,
                        "__name__ must be set to a string object");
        return -1;
    }
    Py_INCREF(value);
    __Pyx_Py_XDECREF_SET(op->func_name, value);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_qualname(__pyx_CyFunctionObject *op, void *context)
{
    CYTHON_UNUSED_VAR(context);
    Py_INCREF(op->func_qualname);
    return op->func_qualname;
}
static int
__Pyx_CyFunction_set_qualname(__pyx_CyFunctionObject *op, PyObject *value, void *context)
{
    CYTHON_UNUSED_VAR(context);
#if PY_MAJOR_VERSION >= 3
    if (unlikely(value == NULL || !PyUnicode_Check(value)))
#else
    if (unlikely(value == NULL || !PyString_Check(value)))
#endif
    {
        PyErr_SetString(PyExc_TypeError,
                        "__qualname__ must be set to a string object");
        return -1;
    }
    Py_INCREF(value);
    __Pyx_Py_XDECREF_SET(op->func_qualname, value);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_dict(__pyx_CyFunctionObject *op, void *context)
{
    CYTHON_UNUSED_VAR(context);
    if (unlikely(op->func_dict == NULL)) {
        op->func_dict = PyDict_New();
        if (unlikely(op->func_dict == NULL))
            return NULL;
    }
    Py_INCREF(op->func_dict);
    return op->func_dict;
}
static int
__Pyx_CyFunction_set_dict(__pyx_CyFunctionObject *op, PyObject *value, void *context)
{
    CYTHON_UNUSED_VAR(context);
    if (unlikely(value == NULL)) {
        PyErr_SetString(PyExc_TypeError,
               "function's dictionary may not be deleted");
        return -1;
    }
    if (unlikely(!PyDict_Check(value))) {
        PyErr_SetString(PyExc_TypeError,
               "setting function's dictionary to a non-dict");
        return -1;
    }
    Py_INCREF(value);
    __Pyx_Py_XDECREF_SET(op->func_dict, value);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_globals(__pyx_CyFunctionObject *op, void *context)
{
    CYTHON_UNUSED_VAR(context);
    Py_INCREF(op->func_globals);
    return op->func_globals;
}
static PyObject *
__Pyx_CyFunction_get_closure(__pyx_CyFunctionObject *op, void *context)
{
    CYTHON_UNUSED_VAR(op);
    CYTHON_UNUSED_VAR(context);
    Py_INCREF(Py_None);
    return Py_None;
}
static PyObject *
__Pyx_CyFunction_get_code(__pyx_CyFunctionObject *op, void *context)
{
    PyObject* result = (op->func_code) ? op->func_code : Py_None;
    CYTHON_UNUSED_VAR(context);
    Py_INCREF(result);
    return result;
}
static int
__Pyx_CyFunction_init_defaults(__pyx_CyFunctionObject *op) {
    int result = 0;
    PyObject *res = op->defaults_getter((PyObject *) op);
    if (unlikely(!res))
        return -1;
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    op->defaults_tuple = PyTuple_GET_ITEM(res, 0);
    Py_INCREF(op->defaults_tuple);
    op->defaults_kwdict = PyTuple_GET_ITEM(res, 1);
    Py_INCREF(op->defaults_kwdict);
    #else
    op->defaults_tuple = PySequence_ITEM(res, 0);
    if (unlikely(!op->defaults_tuple)) result = -1;
    else {
        op->defaults_kwdict = PySequence_ITEM(res, 1);
        if (unlikely(!op->defaults_kwdict)) result = -1;
    }
    #endif
    Py_DECREF(res);
    return result;
}
static int
__Pyx_CyFunction_set_defaults(__pyx_CyFunctionObject *op, PyObject* value, void *context) {
    CYTHON_UNUSED_VAR(context);
    if (!value) {
        value = Py_None;
    } else if (unlikely(value != Py_None && !PyTuple_Check(value))) {
        PyErr_SetString(PyExc_TypeError,
                        "__defaults__ must be set to a tuple object");
        return -1;
    }
    Py_INCREF(value);
    __Pyx_Py_XDECREF_SET(op->defaults_tuple, value);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_defaults(__pyx_CyFunctionObject *op, void *context) {
    PyObject* result = op->defaults_tuple;
    CYTHON_UNUSED_VAR(context);
    if (unlikely(!result)) {
        if (op->defaults_getter) {
            if (unlikely(__Pyx_CyFunction_init_defaults(op) < 0)) return NULL;
            result = op->defaults_tuple;
        } else {
            result = Py_None;
        }
    }
    Py_INCREF(result);
    return result;
}
static int
__Pyx_CyFunction_set_kwdefaults(__pyx_CyFunctionObject *op, PyObject* value, void *context) {
    CYTHON_UNUSED_VAR(context);
    if (!value) {
        value = Py_None;
    } else if (unlikely(value != Py_None && !PyDict_Check(value))) {
        PyErr_SetString(PyExc_TypeError,
                        "__kwdefaults__ must be set to a dict object");
        return -1;
    }
    Py_INCREF(value);
    __Pyx_Py_XDECREF_SET(op->defaults_kwdict, value);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_kwdefaults(__pyx_CyFunctionObject *op, void *context) {
    PyObject* result = op->defaults_kwdict;
    CYTHON_UNUSED_VAR(context);
    if (unlikely(!result)) {
        if (op->defaults_getter) {
            if (unlikely(__Pyx_CyFunction_init_defaults(op) < 0)) return NULL;
            result = op->defaults_kwdict;
        } else {
            result = Py_None;
        }
    }
    Py_INCREF(result);
    return result;
}
static int
__Pyx_CyFunction_set_annotations(__pyx_CyFunctionObject *op, PyObject* value, void *context) {
    CYTHON_UNUSED_VAR(context);
    if (!value || value == Py_None) {
        value = NULL;
    } else if (unlikely(!PyDict_Check(value))) {
        PyErr_SetString(PyExc_TypeError,
                        "__annotations__ must be set to a dict object");
        return -1;
    }
    Py_XINCREF(value);
    __Pyx_Py_XDECREF_SET(op->func_annotations, value);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_annotations(__pyx_CyFunctionObject *op, void *context) {
    PyObject* result = op->func_annotations;
    CYTHON_UNUSED_VAR(context);
    if (unlikely(!result)) {
        result = PyDict_New();
        if (unlikely(!result)) return NULL;
        op->func_annotations = result;
    }
    Py_INCREF(result);
    return result;
}
static PyObject *
__Pyx_CyFunction_get_is_coroutine(__pyx_CyFunctionObject *op, void *context) {
    int is_coroutine;
    CYTHON_UNUSED_VAR(context);
    if (op->func_is_coroutine) {
        return __Pyx_NewRef(op->func_is_coroutine);
    }
    is_coroutine = op->flags & __Pyx_CYFUNCTION_COROUTINE;
#if PY_VERSION_HEX >= 0x03050000
    if (is_coroutine) {
        PyObject *module, *fromlist, *marker = __pyx_n_s_is_coroutine;
        fromlist = PyList_New(1);
        if (unlikely(!fromlist)) return NULL;
        Py_INCREF(marker);
        PyList_SET_ITEM(fromlist, 0, marker);
        module = PyImport_ImportModuleLevelObject(__pyx_n_s_asyncio_coroutines, NULL, NULL, fromlist, 0);
        Py_DECREF(fromlist);
        if (unlikely(!module)) goto ignore;
        op->func_is_coroutine = __Pyx_PyObject_GetAttrStr(module, marker);
        Py_DECREF(module);
        if (likely(op->func_is_coroutine)) {
            return __Pyx_NewRef(op->func_is_coroutine);
        }
ignore:
        PyErr_Clear();
    }
#endif
    op->func_is_coroutine = __Pyx_PyBool_FromLong(is_coroutine);
    return __Pyx_NewRef(op->func_is_coroutine);
}
static PyGetSetDef __pyx_CyFunction_getsets[] = {
    {(char *) "func_doc", (getter)__Pyx_CyFunction_get_doc, (setter)__Pyx_CyFunction_set_doc, 0, 0},
    {(char *) "__doc__",  (getter)__Pyx_CyFunction_get_doc, (setter)__Pyx_CyFunction_set_doc, 0, 0},
    {(char *) "func_name", (getter)__Pyx_CyFunction_get_name, (setter)__Pyx_CyFunction_set_name, 0, 0},
    {(char *) "__name__", (getter)__Pyx_CyFunction_get_name, (setter)__Pyx_CyFunction_set_name, 0, 0},
    {(char *) "__qualname__", (getter)__Pyx_CyFunction_get_qualname, (setter)__Pyx_CyFunction_set_qualname, 0, 0},
    {(char *) "func_dict", (getter)__Pyx_CyFunction_get_dict, (setter)__Pyx_CyFunction_set_dict, 0, 0},
    {(char *) "__dict__", (getter)__Pyx_CyFunction_get_dict, (setter)__Pyx_CyFunction_set_dict, 0, 0},
    {(char *) "func_globals", (getter)__Pyx_CyFunction_get_globals, 0, 0, 0},
    {(char *) "__globals__", (getter)__Pyx_CyFunction_get_globals, 0, 0, 0},
    {(char *) "func_closure", (getter)__Pyx_CyFunction_get_closure, 0, 0, 0},
    {(char *) "__closure__", (getter)__Pyx_CyFunction_get_closure, 0, 0, 0},
    {(char *) "func_code", (getter)__Pyx_CyFunction_get_code, 0, 0, 0},
    {(char *) "__code__", (getter)__Pyx_CyFunction_get_code, 0, 0, 0},
    {(char *) "func_defaults", (getter)__Pyx_CyFunction_get_defaults, (setter)__Pyx_CyFunction_set_defaults, 0, 0},
    {(char *) "__defaults__", (getter)__Pyx_CyFunction_get_defaults, (setter)__Pyx_CyFunction_set_defaults, 0, 0},
    {(char *) "__kwdefaults__", (getter)__Pyx_CyFunction_get_kwdefaults, (setter)__Pyx_CyFunction_set_kwdefaults, 0, 0},
    {(char *) "__annotations__", (getter)__Pyx_CyFunction_get_annotations, (setter)__Pyx_CyFunction_set_annotations, 0, 0},
    {(char *) "_is_coroutine", (getter)__Pyx_CyFunction_get_is_coroutine, 0, 0, 0},
    {0, 0, 0, 0, 0}
};
static PyMemberDef __pyx_CyFunction_members[] = {
    {(char *) "__module__", T_OBJECT, offsetof(PyCFunctionObject, m_module), 0, 0},
#if CYTHON_USE_TYPE_SPECS
    {(char *) "__dictoffset__", T_PYSSIZET, offsetof(__pyx_CyFunctionObject, func_dict), READONLY, 0},
#if CYTHON_METH_FASTCALL
#if CYTHON_BACKPORT_VECTORCALL
    {(char *) "__vectorcalloffset__", T_PYSSIZET, offsetof(__pyx_CyFunctionObject, func_vectorcall), READONLY, 0},
#else
    {(char *) "__vectorcalloffset__", T_PYSSIZET, offsetof(PyCFunctionObject, vectorcall), READONLY, 0},
#endif
#endif
#if PY_VERSION_HEX < 0x030500A0
    {(char *) "__weaklistoffset__", T_PYSSIZET, offsetof(__pyx_CyFunctionObject, func_weakreflist), READONLY, 0},
#else
    {(char *) "__weaklistoffset__", T_PYSSIZET, offsetof(PyCFunctionObject, m_weakreflist), READONLY, 0},
#endif
#endif
    {0, 0, 0,  0, 0}
};
static PyObject *
__Pyx_CyFunction_reduce(__pyx_CyFunctionObject *m, PyObject *args)
{
    CYTHON_UNUSED_VAR(args);
#if PY_MAJOR_VERSION >= 3
    Py_INCREF(m->func_qualname);
    return m->func_qualname;
#else
    return PyString_FromString(((PyCFunctionObject*)m)->m_ml->ml_name);
#endif
}
static PyMethodDef __pyx_CyFunction_methods[] = {
    {"__reduce__", (PyCFunction)__Pyx_CyFunction_reduce, METH_VARARGS, 0},
    {0, 0, 0, 0}
};
#if PY_VERSION_HEX < 0x030500A0
#define __Pyx_CyFunction_weakreflist(cyfunc) ((cyfunc)->func_weakreflist)
#else
#define __Pyx_CyFunction_weakreflist(cyfunc) (((PyCFunctionObject*)cyfunc)->m_weakreflist)
#endif
static PyObject *__Pyx_CyFunction_Init(__pyx_CyFunctionObject *op, PyMethodDef *ml, int flags, PyObject* qualname,
                                       PyObject *closure, PyObject *module, PyObject* globals, PyObject* code) {
    PyCFunctionObject *cf = (PyCFunctionObject*) op;
    if (unlikely(op == NULL))
        return NULL;
    op->flags = flags;
    __Pyx_CyFunction_weakreflist(op) = NULL;
    cf->m_ml = ml;
    cf->m_self = (PyObject *) op;
    Py_XINCREF(closure);
    op->func_closure = closure;
    Py_XINCREF(module);
    cf->m_module = module;
    op->func_dict = NULL;
    op->func_name = NULL;
    Py_INCREF(qualname);
    op->func_qualname = qualname;
    op->func_doc = NULL;
#if PY_VERSION_HEX < 0x030900B1
    op->func_classobj = NULL;
#else
    ((PyCMethodObject*)op)->mm_class = NULL;
#endif
    op->func_globals = globals;
    Py_INCREF(op->func_globals);
    Py_XINCREF(code);
    op->func_code = code;
    op->defaults_pyobjects = 0;
    op->defaults_size = 0;
    op->defaults = NULL;
    op->defaults_tuple = NULL;
    op->defaults_kwdict = NULL;
    op->defaults_getter = NULL;
    op->func_annotations = NULL;
    op->func_is_coroutine = NULL;
#if CYTHON_METH_FASTCALL
    switch (ml->ml_flags & (METH_VARARGS | METH_FASTCALL | METH_NOARGS | METH_O | METH_KEYWORDS | METH_METHOD)) {
    case METH_NOARGS:
        __Pyx_CyFunction_func_vectorcall(op) = __Pyx_CyFunction_Vectorcall_NOARGS;
        break;
    case METH_O:
        __Pyx_CyFunction_func_vectorcall(op) = __Pyx_CyFunction_Vectorcall_O;
        break;
    case METH_METHOD | METH_FASTCALL | METH_KEYWORDS:
        __Pyx_CyFunction_func_vectorcall(op) = __Pyx_CyFunction_Vectorcall_FASTCALL_KEYWORDS_METHOD;
        break;
    case METH_FASTCALL | METH_KEYWORDS:
        __Pyx_CyFunction_func_vectorcall(op) = __Pyx_CyFunction_Vectorcall_FASTCALL_KEYWORDS;
        break;
    case METH_VARARGS | METH_KEYWORDS:
        __Pyx_CyFunction_func_vectorcall(op) = NULL;
        break;
    default:
        PyErr_SetString(PyExc_SystemError, "Bad call flags for CyFunction");
        Py_DECREF(op);
        return NULL;
    }
#endif
    return (PyObject *) op;
}
static int
__Pyx_CyFunction_clear(__pyx_CyFunctionObject *m)
{
    Py_CLEAR(m->func_closure);
    Py_CLEAR(((PyCFunctionObject*)m)->m_module);
    Py_CLEAR(m->func_dict);
    Py_CLEAR(m->func_name);
    Py_CLEAR(m->func_qualname);
    Py_CLEAR(m->func_doc);
    Py_CLEAR(m->func_globals);
    Py_CLEAR(m->func_code);
#if PY_VERSION_HEX < 0x030900B1
    Py_CLEAR(__Pyx_CyFunction_GetClassObj(m));
#else
    {
        PyObject *cls = (PyObject*) ((PyCMethodObject *) (m))->mm_class;
        ((PyCMethodObject *) (m))->mm_class = NULL;
        Py_XDECREF(cls);
    }
#endif
    Py_CLEAR(m->defaults_tuple);
    Py_CLEAR(m->defaults_kwdict);
    Py_CLEAR(m->func_annotations);
    Py_CLEAR(m->func_is_coroutine);
    if (m->defaults) {
        PyObject **pydefaults = __Pyx_CyFunction_Defaults(PyObject *, m);
        int i;
        for (i = 0; i < m->defaults_pyobjects; i++)
            Py_XDECREF(pydefaults[i]);
        PyObject_Free(m->defaults);
        m->defaults = NULL;
    }
    return 0;
}
static void __Pyx__CyFunction_dealloc(__pyx_CyFunctionObject *m)
{
    if (__Pyx_CyFunction_weakreflist(m) != NULL)
        PyObject_ClearWeakRefs((PyObject *) m);
    __Pyx_CyFunction_clear(m);
    __Pyx_PyHeapTypeObject_GC_Del(m);
}
static void __Pyx_CyFunction_dealloc(__pyx_CyFunctionObject *m)
{
    PyObject_GC_UnTrack(m);
    __Pyx__CyFunction_dealloc(m);
}
static int __Pyx_CyFunction_traverse(__pyx_CyFunctionObject *m, visitproc visit, void *arg)
{
    Py_VISIT(m->func_closure);
    Py_VISIT(((PyCFunctionObject*)m)->m_module);
    Py_VISIT(m->func_dict);
    Py_VISIT(m->func_name);
    Py_VISIT(m->func_qualname);
    Py_VISIT(m->func_doc);
    Py_VISIT(m->func_globals);
    Py_VISIT(m->func_code);
    Py_VISIT(__Pyx_CyFunction_GetClassObj(m));
    Py_VISIT(m->defaults_tuple);
    Py_VISIT(m->defaults_kwdict);
    Py_VISIT(m->func_is_coroutine);
    if (m->defaults) {
        PyObject **pydefaults = __Pyx_CyFunction_Defaults(PyObject *, m);
        int i;
        for (i = 0; i < m->defaults_pyobjects; i++)
            Py_VISIT(pydefaults[i]);
    }
    return 0;
}
static PyObject*
__Pyx_CyFunction_repr(__pyx_CyFunctionObject *op)
{
#if PY_MAJOR_VERSION >= 3
    return PyUnicode_FromFormat("<cyfunction %U at %p>",
                                op->func_qualname, (void *)op);
#else
    return PyString_FromFormat("<cyfunction %s at %p>",
                               PyString_AsString(op->func_qualname), (void *)op);
#endif
}
static PyObject * __Pyx_CyFunction_CallMethod(PyObject *func, PyObject *self, PyObject *arg, PyObject *kw) {
    PyCFunctionObject* f = (PyCFunctionObject*)func;
    PyCFunction meth = f->m_ml->ml_meth;
    Py_ssize_t size;
    switch (f->m_ml->ml_flags & (METH_VARARGS | METH_KEYWORDS | METH_NOARGS | METH_O)) {
    case METH_VARARGS:
        if (likely(kw == NULL || PyDict_Size(kw) == 0))
            return (*meth)(self, arg);
        break;
    case METH_VARARGS | METH_KEYWORDS:
        return (*(PyCFunctionWithKeywords)(void*)meth)(self, arg, kw);
    case METH_NOARGS:
        if (likely(kw == NULL || PyDict_Size(kw) == 0)) {
            size = PyTuple_GET_SIZE(arg);
            if (likely(size == 0))
                return (*meth)(self, NULL);
            PyErr_Format(PyExc_TypeError,
                "%.200s() takes no arguments (%" CYTHON_FORMAT_SSIZE_T "d given)",
                f->m_ml->ml_name, size);
            return NULL;
        }
        break;
    case METH_O:
        if (likely(kw == NULL || PyDict_Size(kw) == 0)) {
            size = PyTuple_GET_SIZE(arg);
            if (likely(size == 1)) {
                PyObject *result, *arg0;
                #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
                arg0 = PyTuple_GET_ITEM(arg, 0);
                #else
                arg0 = PySequence_ITEM(arg, 0); if (unlikely(!arg0)) return NULL;
                #endif
                result = (*meth)(self, arg0);
                #if !(CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS)
                Py_DECREF(arg0);
                #endif
                return result;
            }
            PyErr_Format(PyExc_TypeError,
                "%.200s() takes exactly one argument (%" CYTHON_FORMAT_SSIZE_T "d given)",
                f->m_ml->ml_name, size);
            return NULL;
        }
        break;
    default:
        PyErr_SetString(PyExc_SystemError, "Bad call flags for CyFunction");
        return NULL;
    }
    PyErr_Format(PyExc_TypeError, "%.200s() takes no keyword arguments",
                 f->m_ml->ml_name);
    return NULL;
}
static CYTHON_INLINE PyObject *__Pyx_CyFunction_Call(PyObject *func, PyObject *arg, PyObject *kw) {
    return __Pyx_CyFunction_CallMethod(func, ((PyCFunctionObject*)func)->m_self, arg, kw);
}
static PyObject *__Pyx_CyFunction_CallAsMethod(PyObject *func, PyObject *args, PyObject *kw) {
    PyObject *result;
    __pyx_CyFunctionObject *cyfunc = (__pyx_CyFunctionObject *) func;
#if CYTHON_METH_FASTCALL
     __pyx_vectorcallfunc vc = __Pyx_CyFunction_func_vectorcall(cyfunc);
    if (vc) {
#if CYTHON_ASSUME_SAFE_MACROS
        return __Pyx_PyVectorcall_FastCallDict(func, vc, &PyTuple_GET_ITEM(args, 0), (size_t)PyTuple_GET_SIZE(args), kw);
#else
        (void) &__Pyx_PyVectorcall_FastCallDict;
        return PyVectorcall_Call(func, args, kw);
#endif
    }
#endif
    if ((cyfunc->flags & __Pyx_CYFUNCTION_CCLASS) && !(cyfunc->flags & __Pyx_CYFUNCTION_STATICMETHOD)) {
        Py_ssize_t argc;
        PyObject *new_args;
        PyObject *self;
        argc = PyTuple_GET_SIZE(args);
        new_args = PyTuple_GetSlice(args, 1, argc);
        if (unlikely(!new_args))
            return NULL;
        self = PyTuple_GetItem(args, 0);
        if (unlikely(!self)) {
            Py_DECREF(new_args);
            return NULL;
        }
        result = __Pyx_CyFunction_CallMethod(func, self, new_args, kw);
        Py_DECREF(new_args);
    } else {
        result = __Pyx_CyFunction_Call(func, args, kw);
    }
    return result;
}
#if CYTHON_METH_FASTCALL
static CYTHON_INLINE int __Pyx_CyFunction_Vectorcall_CheckArgs(__pyx_CyFunctionObject *cyfunc, Py_ssize_t nargs, PyObject *kwnames)
{
    int ret = 0;
    if ((cyfunc->flags & __Pyx_CYFUNCTION_CCLASS) && !(cyfunc->flags & __Pyx_CYFUNCTION_STATICMETHOD)) {
        if (unlikely(nargs < 1)) {
            PyErr_Format(PyExc_TypeError, "%.200s() needs an argument",
                         ((PyCFunctionObject*)cyfunc)->m_ml->ml_name);
            return -1;
        }
        ret = 1;
    }
    if (unlikely(kwnames) && unlikely(PyTuple_GET_SIZE(kwnames))) {
        PyErr_Format(PyExc_TypeError,
                     "%.200s() takes no keyword arguments", ((PyCFunctionObject*)cyfunc)->m_ml->ml_name);
        return -1;
    }
    return ret;
}
static PyObject * __Pyx_CyFunction_Vectorcall_NOARGS(PyObject *func, PyObject *const *args, size_t nargsf, PyObject *kwnames)
{
    __pyx_CyFunctionObject *cyfunc = (__pyx_CyFunctionObject *)func;
    PyMethodDef* def = ((PyCFunctionObject*)cyfunc)->m_ml;
#if CYTHON_BACKPORT_VECTORCALL
    Py_ssize_t nargs = (Py_ssize_t)nargsf;
#else
    Py_ssize_t nargs = PyVectorcall_NARGS(nargsf);
#endif
    PyObject *self;
    switch (__Pyx_CyFunction_Vectorcall_CheckArgs(cyfunc, nargs, kwnames)) {
    case 1:
        self = args[0];
        args += 1;
        nargs -= 1;
        break;
    case 0:
        self = ((PyCFunctionObject*)cyfunc)->m_self;
        break;
    default:
        return NULL;
    }
    if (unlikely(nargs != 0)) {
        PyErr_Format(PyExc_TypeError,
            "%.200s() takes no arguments (%" CYTHON_FORMAT_SSIZE_T "d given)",
            def->ml_name, nargs);
        return NULL;
    }
    return def->ml_meth(self, NULL);
}
static PyObject * __Pyx_CyFunction_Vectorcall_O(PyObject *func, PyObject *const *args, size_t nargsf, PyObject *kwnames)
{
    __pyx_CyFunctionObject *cyfunc = (__pyx_CyFunctionObject *)func;
    PyMethodDef* def = ((PyCFunctionObject*)cyfunc)->m_ml;
#if CYTHON_BACKPORT_VECTORCALL
    Py_ssize_t nargs = (Py_ssize_t)nargsf;
#else
    Py_ssize_t nargs = PyVectorcall_NARGS(nargsf);
#endif
    PyObject *self;
    switch (__Pyx_CyFunction_Vectorcall_CheckArgs(cyfunc, nargs, kwnames)) {
    case 1:
        self = args[0];
        args += 1;
        nargs -= 1;
        break;
    case 0:
        self = ((PyCFunctionObject*)cyfunc)->m_self;
        break;
    default:
        return NULL;
    }
    if (unlikely(nargs != 1)) {
        PyErr_Format(PyExc_TypeError,
            "%.200s() takes exactly one argument (%" CYTHON_FORMAT_SSIZE_T "d given)",
            def->ml_name, nargs);
        return NULL;
    }
    return def->ml_meth(self, args[0]);
}
static PyObject * __Pyx_CyFunction_Vectorcall_FASTCALL_KEYWORDS(PyObject *func, PyObject *const *args, size_t nargsf, PyObject *kwnames)
{
    __pyx_CyFunctionObject *cyfunc = (__pyx_CyFunctionObject *)func;
    PyMethodDef* def = ((PyCFunctionObject*)cyfunc)->m_ml;
#if CYTHON_BACKPORT_VECTORCALL
    Py_ssize_t nargs = (Py_ssize_t)nargsf;
#else
    Py_ssize_t nargs = PyVectorcall_NARGS(nargsf);
#endif
    PyObject *self;
    switch (__Pyx_CyFunction_Vectorcall_CheckArgs(cyfunc, nargs, NULL)) {
    case 1:
        self = args[0];
        args += 1;
        nargs -= 1;
        break;
    case 0:
        self = ((PyCFunctionObject*)cyfunc)->m_self;
        break;
    default:
        return NULL;
    }
    return ((_PyCFunctionFastWithKeywords)(void(*)(void))def->ml_meth)(self, args, nargs, kwnames);
}
static PyObject * __Pyx_CyFunction_Vectorcall_FASTCALL_KEYWORDS_METHOD(PyObject *func, PyObject *const *args, size_t nargsf, PyObject *kwnames)
{
    __pyx_CyFunctionObject *cyfunc = (__pyx_CyFunctionObject *)func;
    PyMethodDef* def = ((PyCFunctionObject*)cyfunc)->m_ml;
    PyTypeObject *cls = (PyTypeObject *) __Pyx_CyFunction_GetClassObj(cyfunc);
#if CYTHON_BACKPORT_VECTORCALL
    Py_ssize_t nargs = (Py_ssize_t)nargsf;
#else
    Py_ssize_t nargs = PyVectorcall_NARGS(nargsf);
#endif
    PyObject *self;
    switch (__Pyx_CyFunction_Vectorcall_CheckArgs(cyfunc, nargs, NULL)) {
    case 1:
        self = args[0];
        args += 1;
        nargs -= 1;
        break;
    case 0:
        self = ((PyCFunctionObject*)cyfunc)->m_self;
        break;
    default:
        return NULL;
    }
    return ((__Pyx_PyCMethod)(void(*)(void))def->ml_meth)(self, cls, args, nargs, kwnames);
}
#endif
#if CYTHON_USE_TYPE_SPECS
static PyType_Slot __pyx_CyFunctionType_slots[] = {
    {Py_tp_dealloc, (void *)__Pyx_CyFunction_dealloc},
    {Py_tp_repr, (void *)__Pyx_CyFunction_repr},
    {Py_tp_call, (void *)__Pyx_CyFunction_CallAsMethod},
    {Py_tp_traverse, (void *)__Pyx_CyFunction_traverse},
    {Py_tp_clear, (void *)__Pyx_CyFunction_clear},
    {Py_tp_methods, (void *)__pyx_CyFunction_methods},
    {Py_tp_members, (void *)__pyx_CyFunction_members},
    {Py_tp_getset, (void *)__pyx_CyFunction_getsets},
    {Py_tp_descr_get, (void *)__Pyx_PyMethod_New},
    {0, 0},
};
static PyType_Spec __pyx_CyFunctionType_spec = {
    __PYX_TYPE_MODULE_PREFIX "cython_function_or_method",
    sizeof(__pyx_CyFunctionObject),
    0,
#ifdef Py_TPFLAGS_METHOD_DESCRIPTOR
    Py_TPFLAGS_METHOD_DESCRIPTOR |
#endif
#ifdef _Py_TPFLAGS_HAVE_VECTORCALL
    _Py_TPFLAGS_HAVE_VECTORCALL |
#endif
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_HAVE_GC | Py_TPFLAGS_BASETYPE,
    __pyx_CyFunctionType_slots
};
#else
static PyTypeObject __pyx_CyFunctionType_type = {
    PyVarObject_HEAD_INIT(0, 0)
    __PYX_TYPE_MODULE_PREFIX "cython_function_or_method",
    sizeof(__pyx_CyFunctionObject),
    0,
    (destructor) __Pyx_CyFunction_dealloc,
#if !CYTHON_METH_FASTCALL
    0,
#elif CYTHON_BACKPORT_VECTORCALL
    (printfunc)offsetof(__pyx_CyFunctionObject, func_vectorcall),
#else
    offsetof(PyCFunctionObject, vectorcall),
#endif
    0,
    0,
#if PY_MAJOR_VERSION < 3
    0,
#else
    0,
#endif
    (reprfunc) __Pyx_CyFunction_repr,
    0,
    0,
    0,
    0,
    __Pyx_CyFunction_CallAsMethod,
    0,
    0,
    0,
    0,
#ifdef Py_TPFLAGS_METHOD_DESCRIPTOR
    Py_TPFLAGS_METHOD_DESCRIPTOR |
#endif
#ifdef _Py_TPFLAGS_HAVE_VECTORCALL
    _Py_TPFLAGS_HAVE_VECTORCALL |
#endif
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_HAVE_GC | Py_TPFLAGS_BASETYPE,
    0,
    (traverseproc) __Pyx_CyFunction_traverse,
    (inquiry) __Pyx_CyFunction_clear,
    0,
#if PY_VERSION_HEX < 0x030500A0
    offsetof(__pyx_CyFunctionObject, func_weakreflist),
#else
    offsetof(PyCFunctionObject, m_weakreflist),
#endif
    0,
    0,
    __pyx_CyFunction_methods,
    __pyx_CyFunction_members,
    __pyx_CyFunction_getsets,
    0,
    0,
    __Pyx_PyMethod_New,
    0,
    offsetof(__pyx_CyFunctionObject, func_dict),
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
#if PY_VERSION_HEX >= 0x030400a1
    0,
#endif
#if PY_VERSION_HEX >= 0x030800b1
    0,
#endif
#if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
    0,
#endif
#if CYTHON_COMPILING_IN_PYPY && PYPY_VERSION_NUM+0 >= 0x06000000
    0,
#endif
};
#endif
static int __pyx_CyFunction_init(PyObject *module) {
#if CYTHON_USE_TYPE_SPECS
    __pyx_CyFunctionType = __Pyx_FetchCommonTypeFromSpec(module, &__pyx_CyFunctionType_spec, NULL);
#else
    (void) module;
    __pyx_CyFunctionType = __Pyx_FetchCommonType(&__pyx_CyFunctionType_type);
#endif
    if (unlikely(__pyx_CyFunctionType == NULL)) {
        return -1;
    }
    return 0;
}
static CYTHON_INLINE void *__Pyx_CyFunction_InitDefaults(PyObject *func, size_t size, int pyobjects) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->defaults = PyObject_Malloc(size);
    if (unlikely(!m->defaults))
        return PyErr_NoMemory();
    memset(m->defaults, 0, size);
    m->defaults_pyobjects = pyobjects;
    m->defaults_size = size;
    return m->defaults;
}
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsTuple(PyObject *func, PyObject *tuple) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->defaults_tuple = tuple;
    Py_INCREF(tuple);
}
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsKwDict(PyObject *func, PyObject *dict) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->defaults_kwdict = dict;
    Py_INCREF(dict);
}
static CYTHON_INLINE void __Pyx_CyFunction_SetAnnotationsDict(PyObject *func, PyObject *dict) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->func_annotations = dict;
    Py_INCREF(dict);
}

/* CythonFunction */
static PyObject *__Pyx_CyFunction_New(PyMethodDef *ml, int flags, PyObject* qualname,
                                      PyObject *closure, PyObject *module, PyObject* globals, PyObject* code) {
    PyObject *op = __Pyx_CyFunction_Init(
        PyObject_GC_New(__pyx_CyFunctionObject, __pyx_CyFunctionType),
        ml, flags, qualname, closure, module, globals, code
    );
    if (likely(op)) {
        PyObject_GC_Track(op);
    }
    return op;
}

/* CLineInTraceback */
#ifndef CYTHON_CLINE_IN_TRACEBACK
static int __Pyx_CLineForTraceback(CYTHON_NCP_UNUSED PyThreadState *tstate, int c_line) {
    PyObject *use_cline;
    PyObject *ptype, *pvalue, *ptraceback;
#if CYTHON_COMPILING_IN_CPYTHON
    PyObject **cython_runtime_dict;
#endif
    if (unlikely(!__pyx_cython_runtime)) {
        return c_line;
    }
    __Pyx_ErrFetchInState(tstate, &ptype, &pvalue, &ptraceback);
#if CYTHON_COMPILING_IN_CPYTHON
    cython_runtime_dict = _PyObject_GetDictPtr(__pyx_cython_runtime);
    if (likely(cython_runtime_dict)) {
        __PYX_PY_DICT_LOOKUP_IF_MODIFIED(
            use_cline, *cython_runtime_dict,
            __Pyx_PyDict_GetItemStr(*cython_runtime_dict, __pyx_n_s_cline_in_traceback))
    } else
#endif
    {
      PyObject *use_cline_obj = __Pyx_PyObject_GetAttrStrNoError(__pyx_cython_runtime, __pyx_n_s_cline_in_traceback);
      if (use_cline_obj) {
        use_cline = PyObject_Not(use_cline_obj) ? Py_False : Py_True;
        Py_DECREF(use_cline_obj);
      } else {
        PyErr_Clear();
        use_cline = NULL;
      }
    }
    if (!use_cline) {
        c_line = 0;
        PyObject_SetAttr(__pyx_cython_runtime, __pyx_n_s_cline_in_traceback, Py_False);
    }
    else if (use_cline == Py_False || (use_cline != Py_True && PyObject_Not(use_cline) != 0)) {
        c_line = 0;
    }
    __Pyx_ErrRestoreInState(tstate, ptype, pvalue, ptraceback);
    return c_line;
}
#endif

/* CodeObjectCache */
#if !CYTHON_COMPILING_IN_LIMITED_API
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line) {
    int start = 0, mid = 0, end = count - 1;
    if (end >= 0 && code_line > entries[end].code_line) {
        return count;
    }
    while (start < end) {
        mid = start + (end - start) / 2;
        if (code_line < entries[mid].code_line) {
            end = mid;
        } else if (code_line > entries[mid].code_line) {
             start = mid + 1;
        } else {
            return mid;
        }
    }
    if (code_line <= entries[mid].code_line) {
        return mid;
    } else {
        return mid + 1;
    }
}
static PyCodeObject *__pyx_find_code_object(int code_line) {
    PyCodeObject* code_object;
    int pos;
    if (unlikely(!code_line) || unlikely(!__pyx_code_cache.entries)) {
        return NULL;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if (unlikely(pos >= __pyx_code_cache.count) || unlikely(__pyx_code_cache.entries[pos].code_line != code_line)) {
        return NULL;
    }
    code_object = __pyx_code_cache.entries[pos].code_object;
    Py_INCREF(code_object);
    return code_object;
}
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object) {
    int pos, i;
    __Pyx_CodeObjectCacheEntry* entries = __pyx_code_cache.entries;
    if (unlikely(!code_line)) {
        return;
    }
    if (unlikely(!entries)) {
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Malloc(64*sizeof(__Pyx_CodeObjectCacheEntry));
        if (likely(entries)) {
            __pyx_code_cache.entries = entries;
            __pyx_code_cache.max_count = 64;
            __pyx_code_cache.count = 1;
            entries[0].code_line = code_line;
            entries[0].code_object = code_object;
            Py_INCREF(code_object);
        }
        return;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if ((pos < __pyx_code_cache.count) && unlikely(__pyx_code_cache.entries[pos].code_line == code_line)) {
        PyCodeObject* tmp = entries[pos].code_object;
        entries[pos].code_object = code_object;
        Py_DECREF(tmp);
        return;
    }
    if (__pyx_code_cache.count == __pyx_code_cache.max_count) {
        int new_max = __pyx_code_cache.max_count + 64;
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Realloc(
            __pyx_code_cache.entries, ((size_t)new_max) * sizeof(__Pyx_CodeObjectCacheEntry));
        if (unlikely(!entries)) {
            return;
        }
        __pyx_code_cache.entries = entries;
        __pyx_code_cache.max_count = new_max;
    }
    for (i=__pyx_code_cache.count; i>pos; i--) {
        entries[i] = entries[i-1];
    }
    entries[pos].code_line = code_line;
    entries[pos].code_object = code_object;
    __pyx_code_cache.count++;
    Py_INCREF(code_object);
}
#endif

/* AddTraceback */
#include "compile.h"
#include "frameobject.h"
#include "traceback.h"
#if CYTHON_COMPILING_IN_LIMITED_API
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename) {
    if (c_line) {
        (void) __pyx_cfilenm;
        c_line = __Pyx_CLineForTraceback(__Pyx_PyThreadState_Current, c_line);
    }
    _PyTraceback_Add(funcname, filename, c_line ? -c_line : py_line);
}
#else
static PyCodeObject* __Pyx_CreateCodeObjectForTraceback(
            const char *funcname, int c_line,
            int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyObject *py_srcfile = 0;
    PyObject *py_funcname = 0;
    #if PY_MAJOR_VERSION < 3
    py_srcfile = PyString_FromString(filename);
    #else
    py_srcfile = PyUnicode_FromString(filename);
    #endif
    if (!py_srcfile) goto bad;
    if (c_line) {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        #else
        py_funcname = PyUnicode_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        #endif
    }
    else {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromString(funcname);
        #else
        py_funcname = PyUnicode_FromString(funcname);
        #endif
    }
    if (!py_funcname) goto bad;
    py_code = __Pyx_PyCode_New(
        0,
        0,
        0,
        0,
        0,
        0,
        __pyx_empty_bytes, /*PyObject *code,*/
        __pyx_empty_tuple, /*PyObject *consts,*/
        __pyx_empty_tuple, /*PyObject *names,*/
        __pyx_empty_tuple, /*PyObject *varnames,*/
        __pyx_empty_tuple, /*PyObject *freevars,*/
        __pyx_empty_tuple, /*PyObject *cellvars,*/
        py_srcfile,   /*PyObject *filename,*/
        py_funcname,  /*PyObject *name,*/
        py_line,
        __pyx_empty_bytes  /*PyObject *lnotab*/
    );
    Py_DECREF(py_srcfile);
    Py_DECREF(py_funcname);
    return py_code;
bad:
    Py_XDECREF(py_srcfile);
    Py_XDECREF(py_funcname);
    return NULL;
}
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyFrameObject *py_frame = 0;
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    if (c_line) {
        c_line = __Pyx_CLineForTraceback(tstate, c_line);
    }
    py_code = __pyx_find_code_object(c_line ? -c_line : py_line);
    if (!py_code) {
        py_code = __Pyx_CreateCodeObjectForTraceback(
            funcname, c_line, py_line, filename);
        if (!py_code) goto bad;
        __pyx_insert_code_object(c_line ? -c_line : py_line, py_code);
    }
    py_frame = PyFrame_New(
        tstate,            /*PyThreadState *tstate,*/
        py_code,           /*PyCodeObject *code,*/
        __pyx_d,    /*PyObject *globals,*/
        0                  /*PyObject *locals*/
    );
    if (!py_frame) goto bad;
    __Pyx_PyFrame_SetLineNumber(py_frame, py_line);
    PyTraceBack_Here(py_frame);
bad:
    Py_XDECREF(py_code);
    Py_XDECREF(py_frame);
}
#endif

/* CIntFromPyVerify */
#define __PYX_VERIFY_RETURN_INT(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 0)
#define __PYX_VERIFY_RETURN_INT_EXC(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 1)
#define __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, exc)\
    {\
        func_type value = func_value;\
        if (sizeof(target_type) < sizeof(func_type)) {\
            if (unlikely(value != (func_type) (target_type) value)) {\
                func_type zero = 0;\
                if (exc && unlikely(value == (func_type)-1 && PyErr_Occurred()))\
                    return (target_type) -1;\
                if (is_unsigned && unlikely(value < zero))\
                    goto raise_neg_overflow;\
                else\
                    goto raise_overflow;\
            }\
        }\
        return (target_type) value;\
    }

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int(int value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const int neg_one = (int) -1, const_zero = (int) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(int) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(int) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(int) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(int) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(int) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(int),
                                     little, !is_unsigned);
    }
}

/* CIntFromPy */
static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const int neg_one = (int) -1, const_zero = (int) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if ((sizeof(int) < sizeof(long))) {
            __PYX_VERIFY_RETURN_INT(int, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (int) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case  1: __PYX_VERIFY_RETURN_INT(int, digit, digits[0])
                case 2:
                    if ((8 * sizeof(int) > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int) >= 2 * PyLong_SHIFT)) {
                            return (int) (((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if ((8 * sizeof(int) > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int) >= 3 * PyLong_SHIFT)) {
                            return (int) (((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if ((8 * sizeof(int) > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int) >= 4 * PyLong_SHIFT)) {
                            return (int) (((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (int) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if ((sizeof(int) <= sizeof(unsigned long))) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if ((sizeof(int) <= sizeof(unsigned PY_LONG_LONG))) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case -1: __PYX_VERIFY_RETURN_INT(int, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(int,  digit, +digits[0])
                case -2:
                    if ((8 * sizeof(int) - 1 > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int) - 1 > 2 * PyLong_SHIFT)) {
                            return (int) (((int)-1)*(((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if ((8 * sizeof(int) > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int) - 1 > 2 * PyLong_SHIFT)) {
                            return (int) ((((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if ((8 * sizeof(int) - 1 > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int) - 1 > 3 * PyLong_SHIFT)) {
                            return (int) (((int)-1)*(((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if ((8 * sizeof(int) > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int) - 1 > 3 * PyLong_SHIFT)) {
                            return (int) ((((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if ((8 * sizeof(int) - 1 > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int) - 1 > 4 * PyLong_SHIFT)) {
                            return (int) (((int)-1)*(((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if ((8 * sizeof(int) > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int) - 1 > 4 * PyLong_SHIFT)) {
                            return (int) ((((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if ((sizeof(int) <= sizeof(long))) {
                __PYX_VERIFY_RETURN_INT_EXC(int, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if ((sizeof(int) <= sizeof(PY_LONG_LONG))) {
                __PYX_VERIFY_RETURN_INT_EXC(int, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if (CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_LIMITED_API) && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available, cannot convert large numbers");
#else
            int val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (int) -1;
        }
    } else {
        int val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (int) -1;
        val = __Pyx_PyInt_As_int(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to int");
    return (int) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to int");
    return (int) -1;
}

/* CIntFromPy */
static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const long neg_one = (long) -1, const_zero = (long) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if ((sizeof(long) < sizeof(long))) {
            __PYX_VERIFY_RETURN_INT(long, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (long) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case  1: __PYX_VERIFY_RETURN_INT(long, digit, digits[0])
                case 2:
                    if ((8 * sizeof(long) > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(long) >= 2 * PyLong_SHIFT)) {
                            return (long) (((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if ((8 * sizeof(long) > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(long) >= 3 * PyLong_SHIFT)) {
                            return (long) (((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if ((8 * sizeof(long) > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(long) >= 4 * PyLong_SHIFT)) {
                            return (long) (((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (long) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if ((sizeof(long) <= sizeof(unsigned long))) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if ((sizeof(long) <= sizeof(unsigned PY_LONG_LONG))) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case -1: __PYX_VERIFY_RETURN_INT(long, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(long,  digit, +digits[0])
                case -2:
                    if ((8 * sizeof(long) - 1 > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(long) - 1 > 2 * PyLong_SHIFT)) {
                            return (long) (((long)-1)*(((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if ((8 * sizeof(long) > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(long) - 1 > 2 * PyLong_SHIFT)) {
                            return (long) ((((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if ((8 * sizeof(long) - 1 > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(long) - 1 > 3 * PyLong_SHIFT)) {
                            return (long) (((long)-1)*(((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if ((8 * sizeof(long) > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(long) - 1 > 3 * PyLong_SHIFT)) {
                            return (long) ((((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if ((8 * sizeof(long) - 1 > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(long) - 1 > 4 * PyLong_SHIFT)) {
                            return (long) (((long)-1)*(((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if ((8 * sizeof(long) > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(long) - 1 > 4 * PyLong_SHIFT)) {
                            return (long) ((((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if ((sizeof(long) <= sizeof(long))) {
                __PYX_VERIFY_RETURN_INT_EXC(long, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if ((sizeof(long) <= sizeof(PY_LONG_LONG))) {
                __PYX_VERIFY_RETURN_INT_EXC(long, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if (CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_LIMITED_API) && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available, cannot convert large numbers");
#else
            long val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (long) -1;
        }
    } else {
        long val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (long) -1;
        val = __Pyx_PyInt_As_long(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to long");
    return (long) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to long");
    return (long) -1;
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const long neg_one = (long) -1, const_zero = (long) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(long) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(long) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(long) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(long),
                                     little, !is_unsigned);
    }
}

/* FormatTypeName */
#if CYTHON_COMPILING_IN_LIMITED_API
static __Pyx_TypeName
__Pyx_PyType_GetName(PyTypeObject* tp)
{
    PyObject *name = __Pyx_PyObject_GetAttrStr((PyObject *)tp,
                                               __pyx_n_s_name);
    if (unlikely(name == NULL) || unlikely(!PyUnicode_Check(name))) {
        PyErr_Clear();
        Py_XSETREF(name, __Pyx_NewRef(__pyx_n_s__54));
    }
    return name;
}
#endif

/* FastTypeChecks */
#if CYTHON_COMPILING_IN_CPYTHON
static int __Pyx_InBases(PyTypeObject *a, PyTypeObject *b) {
    while (a) {
        a = a->tp_base;
        if (a == b)
            return 1;
    }
    return b == &PyBaseObject_Type;
}
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b) {
    PyObject *mro;
    if (a == b) return 1;
    mro = a->tp_mro;
    if (likely(mro)) {
        Py_ssize_t i, n;
        n = PyTuple_GET_SIZE(mro);
        for (i = 0; i < n; i++) {
            if (PyTuple_GET_ITEM(mro, i) == (PyObject *)b)
                return 1;
        }
        return 0;
    }
    return __Pyx_InBases(a, b);
}
static CYTHON_INLINE int __Pyx_IsAnySubtype2(PyTypeObject *cls, PyTypeObject *a, PyTypeObject *b) {
    PyObject *mro;
    if (cls == a || cls == b) return 1;
    mro = cls->tp_mro;
    if (likely(mro)) {
        Py_ssize_t i, n;
        n = PyTuple_GET_SIZE(mro);
        for (i = 0; i < n; i++) {
            PyObject *base = PyTuple_GET_ITEM(mro, i);
            if (base == (PyObject *)a || base == (PyObject *)b)
                return 1;
        }
        return 0;
    }
    return __Pyx_InBases(cls, a) || __Pyx_InBases(cls, b);
}
#if PY_MAJOR_VERSION == 2
static int __Pyx_inner_PyErr_GivenExceptionMatches2(PyObject *err, PyObject* exc_type1, PyObject* exc_type2) {
    PyObject *exception, *value, *tb;
    int res;
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&exception, &value, &tb);
    res = exc_type1 ? PyObject_IsSubclass(err, exc_type1) : 0;
    if (unlikely(res == -1)) {
        PyErr_WriteUnraisable(err);
        res = 0;
    }
    if (!res) {
        res = PyObject_IsSubclass(err, exc_type2);
        if (unlikely(res == -1)) {
            PyErr_WriteUnraisable(err);
            res = 0;
        }
    }
    __Pyx_ErrRestore(exception, value, tb);
    return res;
}
#else
static CYTHON_INLINE int __Pyx_inner_PyErr_GivenExceptionMatches2(PyObject *err, PyObject* exc_type1, PyObject *exc_type2) {
    if (exc_type1) {
        return __Pyx_IsAnySubtype2((PyTypeObject*)err, (PyTypeObject*)exc_type1, (PyTypeObject*)exc_type2);
    } else {
        return __Pyx_IsSubtype((PyTypeObject*)err, (PyTypeObject*)exc_type2);
    }
}
#endif
static int __Pyx_PyErr_GivenExceptionMatchesTuple(PyObject *exc_type, PyObject *tuple) {
    Py_ssize_t i, n;
    assert(PyExceptionClass_Check(exc_type));
    n = PyTuple_GET_SIZE(tuple);
#if PY_MAJOR_VERSION >= 3
    for (i=0; i<n; i++) {
        if (exc_type == PyTuple_GET_ITEM(tuple, i)) return 1;
    }
#endif
    for (i=0; i<n; i++) {
        PyObject *t = PyTuple_GET_ITEM(tuple, i);
        #if PY_MAJOR_VERSION < 3
        if (likely(exc_type == t)) return 1;
        #endif
        if (likely(PyExceptionClass_Check(t))) {
            if (__Pyx_inner_PyErr_GivenExceptionMatches2(exc_type, NULL, t)) return 1;
        } else {
        }
    }
    return 0;
}
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject* exc_type) {
    if (likely(err == exc_type)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        if (likely(PyExceptionClass_Check(exc_type))) {
            return __Pyx_inner_PyErr_GivenExceptionMatches2(err, NULL, exc_type);
        } else if (likely(PyTuple_Check(exc_type))) {
            return __Pyx_PyErr_GivenExceptionMatchesTuple(err, exc_type);
        } else {
        }
    }
    return PyErr_GivenExceptionMatches(err, exc_type);
}
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *exc_type1, PyObject *exc_type2) {
    assert(PyExceptionClass_Check(exc_type1));
    assert(PyExceptionClass_Check(exc_type2));
    if (likely(err == exc_type1 || err == exc_type2)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        return __Pyx_inner_PyErr_GivenExceptionMatches2(err, exc_type1, exc_type2);
    }
    return (PyErr_GivenExceptionMatches(err, exc_type1) || PyErr_GivenExceptionMatches(err, exc_type2));
}
#endif

/* GetTopmostException */
#if CYTHON_USE_EXC_INFO_STACK
static _PyErr_StackItem *
__Pyx_PyErr_GetTopmostException(PyThreadState *tstate)
{
    _PyErr_StackItem *exc_info = tstate->exc_info;
    while ((exc_info->exc_type == NULL || exc_info->exc_type == Py_None) &&
           exc_info->previous_item != NULL)
    {
        exc_info = exc_info->previous_item;
    }
    return exc_info;
}
#endif

/* SaveResetException */
#if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx__ExceptionSave(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
    #if CYTHON_USE_EXC_INFO_STACK
    _PyErr_StackItem *exc_info = __Pyx_PyErr_GetTopmostException(tstate);
    *type = exc_info->exc_type;
    *value = exc_info->exc_value;
    *tb = exc_info->exc_traceback;
    #else
    *type = tstate->exc_type;
    *value = tstate->exc_value;
    *tb = tstate->exc_traceback;
    #endif
    Py_XINCREF(*type);
    Py_XINCREF(*value);
    Py_XINCREF(*tb);
}
static CYTHON_INLINE void __Pyx__ExceptionReset(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    #if CYTHON_USE_EXC_INFO_STACK
    _PyErr_StackItem *exc_info = tstate->exc_info;
    tmp_type = exc_info->exc_type;
    tmp_value = exc_info->exc_value;
    tmp_tb = exc_info->exc_traceback;
    exc_info->exc_type = type;
    exc_info->exc_value = value;
    exc_info->exc_traceback = tb;
    #else
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = type;
    tstate->exc_value = value;
    tstate->exc_traceback = tb;
    #endif
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
}
#endif

/* SwapException */
#if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx__ExceptionSwap(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    #if CYTHON_USE_EXC_INFO_STACK
    _PyErr_StackItem *exc_info = tstate->exc_info;
    tmp_type = exc_info->exc_type;
    tmp_value = exc_info->exc_value;
    tmp_tb = exc_info->exc_traceback;
    exc_info->exc_type = *type;
    exc_info->exc_value = *value;
    exc_info->exc_traceback = *tb;
    #else
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = *type;
    tstate->exc_value = *value;
    tstate->exc_traceback = *tb;
    #endif
    *type = tmp_type;
    *value = tmp_value;
    *tb = tmp_tb;
}
#else
static CYTHON_INLINE void __Pyx_ExceptionSwap(PyObject **type, PyObject **value, PyObject **tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    PyErr_GetExcInfo(&tmp_type, &tmp_value, &tmp_tb);
    PyErr_SetExcInfo(*type, *value, *tb);
    *type = tmp_type;
    *value = tmp_value;
    *tb = tmp_tb;
}
#endif

/* PyObjectCall2Args */
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call2Args(PyObject* function, PyObject* arg1, PyObject* arg2) {
    PyObject *args[3] = {NULL, arg1, arg2};
    return __Pyx_PyObject_FastCall(function, args+1, 2 | __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET);
}

/* PyObjectCallMethod1 */
static PyObject* __Pyx__PyObject_CallMethod1(PyObject* method, PyObject* arg) {
    PyObject *result = __Pyx_PyObject_CallOneArg(method, arg);
    Py_DECREF(method);
    return result;
}
static PyObject* __Pyx_PyObject_CallMethod1(PyObject* obj, PyObject* method_name, PyObject* arg) {
    PyObject *method = NULL, *result;
    int is_method = __Pyx_PyObject_GetMethod(obj, method_name, &method);
    if (likely(is_method)) {
        result = __Pyx_PyObject_Call2Args(method, obj, arg);
        Py_DECREF(method);
        return result;
    }
    if (unlikely(!method)) return NULL;
    return __Pyx__PyObject_CallMethod1(method, arg);
}

/* CoroutineBase */
#include <frameobject.h>
#define __Pyx_Coroutine_Undelegate(gen) Py_CLEAR((gen)->yieldfrom)
static int __Pyx_PyGen__FetchStopIterationValue(PyThreadState *__pyx_tstate, PyObject **pvalue) {
    PyObject *et, *ev, *tb;
    PyObject *value = NULL;
    CYTHON_UNUSED_VAR(__pyx_tstate);
    __Pyx_ErrFetch(&et, &ev, &tb);
    if (!et) {
        Py_XDECREF(tb);
        Py_XDECREF(ev);
        Py_INCREF(Py_None);
        *pvalue = Py_None;
        return 0;
    }
    if (likely(et == PyExc_StopIteration)) {
        if (!ev) {
            Py_INCREF(Py_None);
            value = Py_None;
        }
#if PY_VERSION_HEX >= 0x030300A0
        else if (likely(__Pyx_IS_TYPE(ev, (PyTypeObject*)PyExc_StopIteration))) {
            value = ((PyStopIterationObject *)ev)->value;
            Py_INCREF(value);
            Py_DECREF(ev);
        }
#endif
        else if (unlikely(PyTuple_Check(ev))) {
            if (PyTuple_GET_SIZE(ev) >= 1) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
                value = PyTuple_GET_ITEM(ev, 0);
                Py_INCREF(value);
#else
                value = PySequence_ITEM(ev, 0);
#endif
            } else {
                Py_INCREF(Py_None);
                value = Py_None;
            }
            Py_DECREF(ev);
        }
        else if (!__Pyx_TypeCheck(ev, (PyTypeObject*)PyExc_StopIteration)) {
            value = ev;
        }
        if (likely(value)) {
            Py_XDECREF(tb);
            Py_DECREF(et);
            *pvalue = value;
            return 0;
        }
    } else if (!__Pyx_PyErr_GivenExceptionMatches(et, PyExc_StopIteration)) {
        __Pyx_ErrRestore(et, ev, tb);
        return -1;
    }
    PyErr_NormalizeException(&et, &ev, &tb);
    if (unlikely(!PyObject_TypeCheck(ev, (PyTypeObject*)PyExc_StopIteration))) {
        __Pyx_ErrRestore(et, ev, tb);
        return -1;
    }
    Py_XDECREF(tb);
    Py_DECREF(et);
#if PY_VERSION_HEX >= 0x030300A0
    value = ((PyStopIterationObject *)ev)->value;
    Py_INCREF(value);
    Py_DECREF(ev);
#else
    {
        PyObject* args = __Pyx_PyObject_GetAttrStr(ev, __pyx_n_s_args);
        Py_DECREF(ev);
        if (likely(args)) {
            value = PySequence_GetItem(args, 0);
            Py_DECREF(args);
        }
        if (unlikely(!value)) {
            __Pyx_ErrRestore(NULL, NULL, NULL);
            Py_INCREF(Py_None);
            value = Py_None;
        }
    }
#endif
    *pvalue = value;
    return 0;
}
static CYTHON_INLINE
void __Pyx_Coroutine_ExceptionClear(__Pyx_ExcInfoStruct *exc_state) {
    PyObject *t, *v, *tb;
    t = exc_state->exc_type;
    v = exc_state->exc_value;
    tb = exc_state->exc_traceback;
    exc_state->exc_type = NULL;
    exc_state->exc_value = NULL;
    exc_state->exc_traceback = NULL;
    Py_XDECREF(t);
    Py_XDECREF(v);
    Py_XDECREF(tb);
}
#define __Pyx_Coroutine_AlreadyRunningError(gen)  (__Pyx__Coroutine_AlreadyRunningError(gen), (PyObject*)NULL)
static void __Pyx__Coroutine_AlreadyRunningError(__pyx_CoroutineObject *gen) {
    const char *msg;
    CYTHON_MAYBE_UNUSED_VAR(gen);
    if ((0)) {
    #ifdef __Pyx_Coroutine_USED
    } else if (__Pyx_Coroutine_Check((PyObject*)gen)) {
        msg = "coroutine already executing";
    #endif
    #ifdef __Pyx_AsyncGen_USED
    } else if (__Pyx_AsyncGen_CheckExact((PyObject*)gen)) {
        msg = "async generator already executing";
    #endif
    } else {
        msg = "generator already executing";
    }
    PyErr_SetString(PyExc_ValueError, msg);
}
#define __Pyx_Coroutine_NotStartedError(gen)  (__Pyx__Coroutine_NotStartedError(gen), (PyObject*)NULL)
static void __Pyx__Coroutine_NotStartedError(PyObject *gen) {
    const char *msg;
    CYTHON_MAYBE_UNUSED_VAR(gen);
    if ((0)) {
    #ifdef __Pyx_Coroutine_USED
    } else if (__Pyx_Coroutine_Check(gen)) {
        msg = "can't send non-None value to a just-started coroutine";
    #endif
    #ifdef __Pyx_AsyncGen_USED
    } else if (__Pyx_AsyncGen_CheckExact(gen)) {
        msg = "can't send non-None value to a just-started async generator";
    #endif
    } else {
        msg = "can't send non-None value to a just-started generator";
    }
    PyErr_SetString(PyExc_TypeError, msg);
}
#define __Pyx_Coroutine_AlreadyTerminatedError(gen, value, closing)  (__Pyx__Coroutine_AlreadyTerminatedError(gen, value, closing), (PyObject*)NULL)
static void __Pyx__Coroutine_AlreadyTerminatedError(PyObject *gen, PyObject *value, int closing) {
    CYTHON_MAYBE_UNUSED_VAR(gen);
    CYTHON_MAYBE_UNUSED_VAR(closing);
    #ifdef __Pyx_Coroutine_USED
    if (!closing && __Pyx_Coroutine_Check(gen)) {
        PyErr_SetString(PyExc_RuntimeError, "cannot reuse already awaited coroutine");
    } else
    #endif
    if (value) {
        #ifdef __Pyx_AsyncGen_USED
        if (__Pyx_AsyncGen_CheckExact(gen))
            PyErr_SetNone(__Pyx_PyExc_StopAsyncIteration);
        else
        #endif
        PyErr_SetNone(PyExc_StopIteration);
    }
}
static
PyObject *__Pyx_Coroutine_SendEx(__pyx_CoroutineObject *self, PyObject *value, int closing) {
    __Pyx_PyThreadState_declare
    PyThreadState *tstate;
    __Pyx_ExcInfoStruct *exc_state;
    PyObject *retval;
    assert(!self->is_running);
    if (unlikely(self->resume_label == 0)) {
        if (unlikely(value && value != Py_None)) {
            return __Pyx_Coroutine_NotStartedError((PyObject*)self);
        }
    }
    if (unlikely(self->resume_label == -1)) {
        return __Pyx_Coroutine_AlreadyTerminatedError((PyObject*)self, value, closing);
    }
#if CYTHON_FAST_THREAD_STATE
    __Pyx_PyThreadState_assign
    tstate = __pyx_tstate;
#else
    tstate = __Pyx_PyThreadState_Current;
#endif
    exc_state = &self->gi_exc_state;
    if (exc_state->exc_type) {
        #if CYTHON_COMPILING_IN_PYPY
        #else
        if (exc_state->exc_traceback) {
            PyTracebackObject *tb = (PyTracebackObject *) exc_state->exc_traceback;
            PyFrameObject *f = tb->tb_frame;
            Py_XINCREF(tstate->frame);
            assert(f->f_back == NULL);
            f->f_back = tstate->frame;
        }
        #endif
    }
#if CYTHON_USE_EXC_INFO_STACK
    exc_state->previous_item = tstate->exc_info;
    tstate->exc_info = exc_state;
#else
    if (exc_state->exc_type) {
        __Pyx_ExceptionSwap(&exc_state->exc_type, &exc_state->exc_value, &exc_state->exc_traceback);
    } else {
        __Pyx_Coroutine_ExceptionClear(exc_state);
        __Pyx_ExceptionSave(&exc_state->exc_type, &exc_state->exc_value, &exc_state->exc_traceback);
    }
#endif
    self->is_running = 1;
    retval = self->body(self, tstate, value);
    self->is_running = 0;
#if CYTHON_USE_EXC_INFO_STACK
    exc_state = &self->gi_exc_state;
    tstate->exc_info = exc_state->previous_item;
    exc_state->previous_item = NULL;
    __Pyx_Coroutine_ResetFrameBackpointer(exc_state);
#endif
    return retval;
}
static CYTHON_INLINE void __Pyx_Coroutine_ResetFrameBackpointer(__Pyx_ExcInfoStruct *exc_state) {
    PyObject *exc_tb = exc_state->exc_traceback;
    if (likely(exc_tb)) {
#if CYTHON_COMPILING_IN_PYPY
#else
        PyTracebackObject *tb = (PyTracebackObject *) exc_tb;
        PyFrameObject *f = tb->tb_frame;
        Py_CLEAR(f->f_back);
#endif
    }
}
static CYTHON_INLINE
PyObject *__Pyx_Coroutine_MethodReturn(PyObject* gen, PyObject *retval) {
    CYTHON_MAYBE_UNUSED_VAR(gen);
    if (unlikely(!retval)) {
        __Pyx_PyThreadState_declare
        __Pyx_PyThreadState_assign
        if (!__Pyx_PyErr_Occurred()) {
            PyObject *exc = PyExc_StopIteration;
            #ifdef __Pyx_AsyncGen_USED
            if (__Pyx_AsyncGen_CheckExact(gen))
                exc = __Pyx_PyExc_StopAsyncIteration;
            #endif
            __Pyx_PyErr_SetNone(exc);
        }
    }
    return retval;
}
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x03030000 && (defined(__linux__) || PY_VERSION_HEX >= 0x030600B3)
static CYTHON_INLINE
PyObject *__Pyx_PyGen_Send(PyGenObject *gen, PyObject *arg) {
#if PY_VERSION_HEX <= 0x030A00A1
    return _PyGen_Send(gen, arg);
#else
    PyObject *result;
    if (PyIter_Send((PyObject*)gen, arg ? arg : Py_None, &result) == PYGEN_RETURN) {
        if (PyAsyncGen_CheckExact(gen)) {
            assert(result == Py_None);
            PyErr_SetNone(PyExc_StopAsyncIteration);
        }
        else if (result == Py_None) {
            PyErr_SetNone(PyExc_StopIteration);
        }
        else {
            _PyGen_SetStopIterationValue(result);
        }
        Py_CLEAR(result);
    }
    return result;
#endif
}
#endif
static CYTHON_INLINE
PyObject *__Pyx_Coroutine_FinishDelegation(__pyx_CoroutineObject *gen) {
    PyObject *ret;
    PyObject *val = NULL;
    __Pyx_Coroutine_Undelegate(gen);
    __Pyx_PyGen__FetchStopIterationValue(__Pyx_PyThreadState_Current, &val);
    ret = __Pyx_Coroutine_SendEx(gen, val, 0);
    Py_XDECREF(val);
    return ret;
}
static PyObject *__Pyx_Coroutine_Send(PyObject *self, PyObject *value) {
    PyObject *retval;
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject*) self;
    PyObject *yf = gen->yieldfrom;
    if (unlikely(gen->is_running))
        return __Pyx_Coroutine_AlreadyRunningError(gen);
    if (yf) {
        PyObject *ret;
        gen->is_running = 1;
        #ifdef __Pyx_Generator_USED
        if (__Pyx_Generator_CheckExact(yf)) {
            ret = __Pyx_Coroutine_Send(yf, value);
        } else
        #endif
        #ifdef __Pyx_Coroutine_USED
        if (__Pyx_Coroutine_Check(yf)) {
            ret = __Pyx_Coroutine_Send(yf, value);
        } else
        #endif
        #ifdef __Pyx_AsyncGen_USED
        if (__pyx_PyAsyncGenASend_CheckExact(yf)) {
            ret = __Pyx_async_gen_asend_send(yf, value);
        } else
        #endif
        #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x03030000 && (defined(__linux__) || PY_VERSION_HEX >= 0x030600B3)
        if (PyGen_CheckExact(yf)) {
            ret = __Pyx_PyGen_Send((PyGenObject*)yf, value == Py_None ? NULL : value);
        } else
        #endif
        #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x03050000 && defined(PyCoro_CheckExact) && (defined(__linux__) || PY_VERSION_HEX >= 0x030600B3)
        if (PyCoro_CheckExact(yf)) {
            ret = __Pyx_PyGen_Send((PyGenObject*)yf, value == Py_None ? NULL : value);
        } else
        #endif
        {
            if (value == Py_None)
                ret = __Pyx_PyObject_GetIterNextFunc(yf)(yf);
            else
                ret = __Pyx_PyObject_CallMethod1(yf, __pyx_n_s_send, value);
        }
        gen->is_running = 0;
        if (likely(ret)) {
            return ret;
        }
        retval = __Pyx_Coroutine_FinishDelegation(gen);
    } else {
        retval = __Pyx_Coroutine_SendEx(gen, value, 0);
    }
    return __Pyx_Coroutine_MethodReturn(self, retval);
}
static int __Pyx_Coroutine_CloseIter(__pyx_CoroutineObject *gen, PyObject *yf) {
    PyObject *retval = NULL;
    int err = 0;
    #ifdef __Pyx_Generator_USED
    if (__Pyx_Generator_CheckExact(yf)) {
        retval = __Pyx_Coroutine_Close(yf);
        if (!retval)
            return -1;
    } else
    #endif
    #ifdef __Pyx_Coroutine_USED
    if (__Pyx_Coroutine_Check(yf)) {
        retval = __Pyx_Coroutine_Close(yf);
        if (!retval)
            return -1;
    } else
    if (__Pyx_CoroutineAwait_CheckExact(yf)) {
        retval = __Pyx_CoroutineAwait_Close((__pyx_CoroutineAwaitObject*)yf, NULL);
        if (!retval)
            return -1;
    } else
    #endif
    #ifdef __Pyx_AsyncGen_USED
    if (__pyx_PyAsyncGenASend_CheckExact(yf)) {
        retval = __Pyx_async_gen_asend_close(yf, NULL);
    } else
    if (__pyx_PyAsyncGenAThrow_CheckExact(yf)) {
        retval = __Pyx_async_gen_athrow_close(yf, NULL);
    } else
    #endif
    {
        PyObject *meth;
        gen->is_running = 1;
        meth = __Pyx_PyObject_GetAttrStrNoError(yf, __pyx_n_s_close);
        if (unlikely(!meth)) {
            if (unlikely(PyErr_Occurred())) {
                PyErr_WriteUnraisable(yf);
            }
        } else {
            retval = __Pyx_PyObject_CallNoArg(meth);
            Py_DECREF(meth);
            if (unlikely(!retval))
                err = -1;
        }
        gen->is_running = 0;
    }
    Py_XDECREF(retval);
    return err;
}
static PyObject *__Pyx_Generator_Next(PyObject *self) {
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject*) self;
    PyObject *yf = gen->yieldfrom;
    if (unlikely(gen->is_running))
        return __Pyx_Coroutine_AlreadyRunningError(gen);
    if (yf) {
        PyObject *ret;
        gen->is_running = 1;
        #ifdef __Pyx_Generator_USED
        if (__Pyx_Generator_CheckExact(yf)) {
            ret = __Pyx_Generator_Next(yf);
        } else
        #endif
        #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x03030000 && (defined(__linux__) || PY_VERSION_HEX >= 0x030600B3)
        if (PyGen_CheckExact(yf)) {
            ret = __Pyx_PyGen_Send((PyGenObject*)yf, NULL);
        } else
        #endif
        #ifdef __Pyx_Coroutine_USED
        if (__Pyx_Coroutine_Check(yf)) {
            ret = __Pyx_Coroutine_Send(yf, Py_None);
        } else
        #endif
            ret = __Pyx_PyObject_GetIterNextFunc(yf)(yf);
        gen->is_running = 0;
        if (likely(ret)) {
            return ret;
        }
        return __Pyx_Coroutine_FinishDelegation(gen);
    }
    return __Pyx_Coroutine_SendEx(gen, Py_None, 0);
}
static PyObject *__Pyx_Coroutine_Close_Method(PyObject *self, PyObject *arg) {
    CYTHON_UNUSED_VAR(arg);
    return __Pyx_Coroutine_Close(self);
}
static PyObject *__Pyx_Coroutine_Close(PyObject *self) {
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject *) self;
    PyObject *retval, *raised_exception;
    PyObject *yf = gen->yieldfrom;
    int err = 0;
    if (unlikely(gen->is_running))
        return __Pyx_Coroutine_AlreadyRunningError(gen);
    if (yf) {
        Py_INCREF(yf);
        err = __Pyx_Coroutine_CloseIter(gen, yf);
        __Pyx_Coroutine_Undelegate(gen);
        Py_DECREF(yf);
    }
    if (err == 0)
        PyErr_SetNone(PyExc_GeneratorExit);
    retval = __Pyx_Coroutine_SendEx(gen, NULL, 1);
    if (unlikely(retval)) {
        const char *msg;
        Py_DECREF(retval);
        if ((0)) {
        #ifdef __Pyx_Coroutine_USED
        } else if (__Pyx_Coroutine_Check(self)) {
            msg = "coroutine ignored GeneratorExit";
        #endif
        #ifdef __Pyx_AsyncGen_USED
        } else if (__Pyx_AsyncGen_CheckExact(self)) {
#if PY_VERSION_HEX < 0x03060000
            msg = "async generator ignored GeneratorExit - might require Python 3.6+ finalisation (PEP 525)";
#else
            msg = "async generator ignored GeneratorExit";
#endif
        #endif
        } else {
            msg = "generator ignored GeneratorExit";
        }
        PyErr_SetString(PyExc_RuntimeError, msg);
        return NULL;
    }
    raised_exception = PyErr_Occurred();
    if (likely(!raised_exception || __Pyx_PyErr_GivenExceptionMatches2(raised_exception, PyExc_GeneratorExit, PyExc_StopIteration))) {
        if (raised_exception) PyErr_Clear();
        Py_INCREF(Py_None);
        return Py_None;
    }
    return NULL;
}
static PyObject *__Pyx__Coroutine_Throw(PyObject *self, PyObject *typ, PyObject *val, PyObject *tb,
                                        PyObject *args, int close_on_genexit) {
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject *) self;
    PyObject *yf = gen->yieldfrom;
    if (unlikely(gen->is_running))
        return __Pyx_Coroutine_AlreadyRunningError(gen);
    if (yf) {
        PyObject *ret;
        Py_INCREF(yf);
        if (__Pyx_PyErr_GivenExceptionMatches(typ, PyExc_GeneratorExit) && close_on_genexit) {
            int err = __Pyx_Coroutine_CloseIter(gen, yf);
            Py_DECREF(yf);
            __Pyx_Coroutine_Undelegate(gen);
            if (err < 0)
                return __Pyx_Coroutine_MethodReturn(self, __Pyx_Coroutine_SendEx(gen, NULL, 0));
            goto throw_here;
        }
        gen->is_running = 1;
        if (0
        #ifdef __Pyx_Generator_USED
            || __Pyx_Generator_CheckExact(yf)
        #endif
        #ifdef __Pyx_Coroutine_USED
            || __Pyx_Coroutine_Check(yf)
        #endif
            ) {
            ret = __Pyx__Coroutine_Throw(yf, typ, val, tb, args, close_on_genexit);
        #ifdef __Pyx_Coroutine_USED
        } else if (__Pyx_CoroutineAwait_CheckExact(yf)) {
            ret = __Pyx__Coroutine_Throw(((__pyx_CoroutineAwaitObject*)yf)->coroutine, typ, val, tb, args, close_on_genexit);
        #endif
        } else {
            PyObject *meth = __Pyx_PyObject_GetAttrStrNoError(yf, __pyx_n_s_throw);
            if (unlikely(!meth)) {
                Py_DECREF(yf);
                if (unlikely(PyErr_Occurred())) {
                    gen->is_running = 0;
                    return NULL;
                }
                __Pyx_Coroutine_Undelegate(gen);
                gen->is_running = 0;
                goto throw_here;
            }
            if (likely(args)) {
                ret = __Pyx_PyObject_Call(meth, args, NULL);
            } else {
                PyObject *cargs[4] = {NULL, typ, val, tb};
                ret = __Pyx_PyObject_FastCall(meth, cargs+1, 3 | __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET);
            }
            Py_DECREF(meth);
        }
        gen->is_running = 0;
        Py_DECREF(yf);
        if (!ret) {
            ret = __Pyx_Coroutine_FinishDelegation(gen);
        }
        return __Pyx_Coroutine_MethodReturn(self, ret);
    }
throw_here:
    __Pyx_Raise(typ, val, tb, NULL);
    return __Pyx_Coroutine_MethodReturn(self, __Pyx_Coroutine_SendEx(gen, NULL, 0));
}
static PyObject *__Pyx_Coroutine_Throw(PyObject *self, PyObject *args) {
    PyObject *typ;
    PyObject *val = NULL;
    PyObject *tb = NULL;
    if (unlikely(!PyArg_UnpackTuple(args, (char *)"throw", 1, 3, &typ, &val, &tb)))
        return NULL;
    return __Pyx__Coroutine_Throw(self, typ, val, tb, args, 1);
}
static CYTHON_INLINE int __Pyx_Coroutine_traverse_excstate(__Pyx_ExcInfoStruct *exc_state, visitproc visit, void *arg) {
    Py_VISIT(exc_state->exc_type);
    Py_VISIT(exc_state->exc_value);
    Py_VISIT(exc_state->exc_traceback);
    return 0;
}
static int __Pyx_Coroutine_traverse(__pyx_CoroutineObject *gen, visitproc visit, void *arg) {
    Py_VISIT(gen->closure);
    Py_VISIT(gen->classobj);
    Py_VISIT(gen->yieldfrom);
    return __Pyx_Coroutine_traverse_excstate(&gen->gi_exc_state, visit, arg);
}
static int __Pyx_Coroutine_clear(PyObject *self) {
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject *) self;
    Py_CLEAR(gen->closure);
    Py_CLEAR(gen->classobj);
    Py_CLEAR(gen->yieldfrom);
    __Pyx_Coroutine_ExceptionClear(&gen->gi_exc_state);
#ifdef __Pyx_AsyncGen_USED
    if (__Pyx_AsyncGen_CheckExact(self)) {
        Py_CLEAR(((__pyx_PyAsyncGenObject*)gen)->ag_finalizer);
    }
#endif
    Py_CLEAR(gen->gi_code);
    Py_CLEAR(gen->gi_frame);
    Py_CLEAR(gen->gi_name);
    Py_CLEAR(gen->gi_qualname);
    Py_CLEAR(gen->gi_modulename);
    return 0;
}
static void __Pyx_Coroutine_dealloc(PyObject *self) {
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject *) self;
    PyObject_GC_UnTrack(gen);
    if (gen->gi_weakreflist != NULL)
        PyObject_ClearWeakRefs(self);
    if (gen->resume_label >= 0) {
        PyObject_GC_Track(self);
#if PY_VERSION_HEX >= 0x030400a1 && CYTHON_USE_TP_FINALIZE
        if (unlikely(PyObject_CallFinalizerFromDealloc(self)))
#else
        Py_TYPE(gen)->tp_del(self);
        if (unlikely(Py_REFCNT(self) > 0))
#endif
        {
            return;
        }
        PyObject_GC_UnTrack(self);
    }
#ifdef __Pyx_AsyncGen_USED
    if (__Pyx_AsyncGen_CheckExact(self)) {
        /* We have to handle this case for asynchronous generators
           right here, because this code has to be between UNTRACK
           and GC_Del. */
        Py_CLEAR(((__pyx_PyAsyncGenObject*)self)->ag_finalizer);
    }
#endif
    __Pyx_Coroutine_clear(self);
    __Pyx_PyHeapTypeObject_GC_Del(gen);
}
static void __Pyx_Coroutine_del(PyObject *self) {
    PyObject *error_type, *error_value, *error_traceback;
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject *) self;
    __Pyx_PyThreadState_declare
    if (gen->resume_label < 0) {
        return;
    }
#if !CYTHON_USE_TP_FINALIZE
    assert(self->ob_refcnt == 0);
    __Pyx_SET_REFCNT(self, 1);
#endif
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&error_type, &error_value, &error_traceback);
#ifdef __Pyx_AsyncGen_USED
    if (__Pyx_AsyncGen_CheckExact(self)) {
        __pyx_PyAsyncGenObject *agen = (__pyx_PyAsyncGenObject*)self;
        PyObject *finalizer = agen->ag_finalizer;
        if (finalizer && !agen->ag_closed) {
            PyObject *res = __Pyx_PyObject_CallOneArg(finalizer, self);
            if (unlikely(!res)) {
                PyErr_WriteUnraisable(self);
            } else {
                Py_DECREF(res);
            }
            __Pyx_ErrRestore(error_type, error_value, error_traceback);
            return;
        }
    }
#endif
    if (unlikely(gen->resume_label == 0 && !error_value)) {
#ifdef __Pyx_Coroutine_USED
#ifdef __Pyx_Generator_USED
    if (!__Pyx_Generator_CheckExact(self))
#endif
        {
        PyObject_GC_UnTrack(self);
#if PY_MAJOR_VERSION >= 3  || defined(PyErr_WarnFormat)
        if (unlikely(PyErr_WarnFormat(PyExc_RuntimeWarning, 1, "coroutine '%.50S' was never awaited", gen->gi_qualname) < 0))
            PyErr_WriteUnraisable(self);
#else
        {PyObject *msg;
        char *cmsg;
        #if CYTHON_COMPILING_IN_PYPY
        msg = NULL;
        cmsg = (char*) "coroutine was never awaited";
        #else
        char *cname;
        PyObject *qualname;
        qualname = gen->gi_qualname;
        cname = PyString_AS_STRING(qualname);
        msg = PyString_FromFormat("coroutine '%.50s' was never awaited", cname);
        if (unlikely(!msg)) {
            PyErr_Clear();
            cmsg = (char*) "coroutine was never awaited";
        } else {
            cmsg = PyString_AS_STRING(msg);
        }
        #endif
        if (unlikely(PyErr_WarnEx(PyExc_RuntimeWarning, cmsg, 1) < 0))
            PyErr_WriteUnraisable(self);
        Py_XDECREF(msg);}
#endif
        PyObject_GC_Track(self);
        }
#endif
    } else {
        PyObject *res = __Pyx_Coroutine_Close(self);
        if (unlikely(!res)) {
            if (PyErr_Occurred())
                PyErr_WriteUnraisable(self);
        } else {
            Py_DECREF(res);
        }
    }
    __Pyx_ErrRestore(error_type, error_value, error_traceback);
#if !CYTHON_USE_TP_FINALIZE
    assert(Py_REFCNT(self) > 0);
    if (likely(--self->ob_refcnt == 0)) {
        return;
    }
    {
        Py_ssize_t refcnt = Py_REFCNT(self);
        _Py_NewReference(self);
        __Pyx_SET_REFCNT(self, refcnt);
    }
#if CYTHON_COMPILING_IN_CPYTHON
    assert(PyType_IS_GC(Py_TYPE(self)) &&
           _Py_AS_GC(self)->gc.gc_refs != _PyGC_REFS_UNTRACKED);
    _Py_DEC_REFTOTAL;
#endif
#ifdef COUNT_ALLOCS
    --Py_TYPE(self)->tp_frees;
    --Py_TYPE(self)->tp_allocs;
#endif
#endif
}
static PyObject *
__Pyx_Coroutine_get_name(__pyx_CoroutineObject *self, void *context)
{
    PyObject *name = self->gi_name;
    CYTHON_UNUSED_VAR(context);
    if (unlikely(!name)) name = Py_None;
    Py_INCREF(name);
    return name;
}
static int
__Pyx_Coroutine_set_name(__pyx_CoroutineObject *self, PyObject *value, void *context)
{
    CYTHON_UNUSED_VAR(context);
#if PY_MAJOR_VERSION >= 3
    if (unlikely(value == NULL || !PyUnicode_Check(value)))
#else
    if (unlikely(value == NULL || !PyString_Check(value)))
#endif
    {
        PyErr_SetString(PyExc_TypeError,
                        "__name__ must be set to a string object");
        return -1;
    }
    Py_INCREF(value);
    __Pyx_Py_XDECREF_SET(self->gi_name, value);
    return 0;
}
static PyObject *
__Pyx_Coroutine_get_qualname(__pyx_CoroutineObject *self, void *context)
{
    PyObject *name = self->gi_qualname;
    CYTHON_UNUSED_VAR(context);
    if (unlikely(!name)) name = Py_None;
    Py_INCREF(name);
    return name;
}
static int
__Pyx_Coroutine_set_qualname(__pyx_CoroutineObject *self, PyObject *value, void *context)
{
    CYTHON_UNUSED_VAR(context);
#if PY_MAJOR_VERSION >= 3
    if (unlikely(value == NULL || !PyUnicode_Check(value)))
#else
    if (unlikely(value == NULL || !PyString_Check(value)))
#endif
    {
        PyErr_SetString(PyExc_TypeError,
                        "__qualname__ must be set to a string object");
        return -1;
    }
    Py_INCREF(value);
    __Pyx_Py_XDECREF_SET(self->gi_qualname, value);
    return 0;
}
static PyObject *
__Pyx_Coroutine_get_frame(__pyx_CoroutineObject *self, void *context)
{
    PyObject *frame = self->gi_frame;
    CYTHON_UNUSED_VAR(context);
    if (!frame) {
        if (unlikely(!self->gi_code)) {
            Py_RETURN_NONE;
        }
        frame = (PyObject *) PyFrame_New(
            PyThreadState_Get(),            /*PyThreadState *tstate,*/
            (PyCodeObject*) self->gi_code,  /*PyCodeObject *code,*/
            __pyx_d,                 /*PyObject *globals,*/
            0                               /*PyObject *locals*/
        );
        if (unlikely(!frame))
            return NULL;
        self->gi_frame = frame;
    }
    Py_INCREF(frame);
    return frame;
}
static __pyx_CoroutineObject *__Pyx__Coroutine_New(
            PyTypeObject* type, __pyx_coroutine_body_t body, PyObject *code, PyObject *closure,
            PyObject *name, PyObject *qualname, PyObject *module_name) {
    __pyx_CoroutineObject *gen = PyObject_GC_New(__pyx_CoroutineObject, type);
    if (unlikely(!gen))
        return NULL;
    return __Pyx__Coroutine_NewInit(gen, body, code, closure, name, qualname, module_name);
}
static __pyx_CoroutineObject *__Pyx__Coroutine_NewInit(
            __pyx_CoroutineObject *gen, __pyx_coroutine_body_t body, PyObject *code, PyObject *closure,
            PyObject *name, PyObject *qualname, PyObject *module_name) {
    gen->body = body;
    gen->closure = closure;
    Py_XINCREF(closure);
    gen->is_running = 0;
    gen->resume_label = 0;
    gen->classobj = NULL;
    gen->yieldfrom = NULL;
    gen->gi_exc_state.exc_type = NULL;
    gen->gi_exc_state.exc_value = NULL;
    gen->gi_exc_state.exc_traceback = NULL;
#if CYTHON_USE_EXC_INFO_STACK
    gen->gi_exc_state.previous_item = NULL;
#endif
    gen->gi_weakreflist = NULL;
    Py_XINCREF(qualname);
    gen->gi_qualname = qualname;
    Py_XINCREF(name);
    gen->gi_name = name;
    Py_XINCREF(module_name);
    gen->gi_modulename = module_name;
    Py_XINCREF(code);
    gen->gi_code = code;
    gen->gi_frame = NULL;
    PyObject_GC_Track(gen);
    return gen;
}

/* PatchModuleWithCoroutine */
static PyObject* __Pyx_Coroutine_patch_module(PyObject* module, const char* py_code) {
#if defined(__Pyx_Generator_USED) || defined(__Pyx_Coroutine_USED)
    int result;
    PyObject *globals, *result_obj;
    globals = PyDict_New();  if (unlikely(!globals)) goto ignore;
    result = PyDict_SetItemString(globals, "_cython_coroutine_type",
    #ifdef __Pyx_Coroutine_USED
        (PyObject*)__pyx_CoroutineType);
    #else
        Py_None);
    #endif
    if (unlikely(result < 0)) goto ignore;
    result = PyDict_SetItemString(globals, "_cython_generator_type",
    #ifdef __Pyx_Generator_USED
        (PyObject*)__pyx_GeneratorType);
    #else
        Py_None);
    #endif
    if (unlikely(result < 0)) goto ignore;
    if (unlikely(PyDict_SetItemString(globals, "_module", module) < 0)) goto ignore;
    if (unlikely(PyDict_SetItemString(globals, "__builtins__", __pyx_b) < 0)) goto ignore;
    result_obj = PyRun_String(py_code, Py_file_input, globals, globals);
    if (unlikely(!result_obj)) goto ignore;
    Py_DECREF(result_obj);
    Py_DECREF(globals);
    return module;
ignore:
    Py_XDECREF(globals);
    PyErr_WriteUnraisable(module);
    if (unlikely(PyErr_WarnEx(PyExc_RuntimeWarning, "Cython module failed to patch module with custom type", 1) < 0)) {
        Py_DECREF(module);
        module = NULL;
    }
#else
    py_code++;
#endif
    return module;
}

/* PatchGeneratorABC */
#ifndef CYTHON_REGISTER_ABCS
#define CYTHON_REGISTER_ABCS 1
#endif
#if defined(__Pyx_Generator_USED) || defined(__Pyx_Coroutine_USED)
static PyObject* __Pyx_patch_abc_module(PyObject *module);
static PyObject* __Pyx_patch_abc_module(PyObject *module) {
    module = __Pyx_Coroutine_patch_module(
        module, ""
"if _cython_generator_type is not None:\n"
"    try: Generator = _module.Generator\n"
"    except AttributeError: pass\n"
"    else: Generator.register(_cython_generator_type)\n"
"if _cython_coroutine_type is not None:\n"
"    try: Coroutine = _module.Coroutine\n"
"    except AttributeError: pass\n"
"    else: Coroutine.register(_cython_coroutine_type)\n"
    );
    return module;
}
#endif
static int __Pyx_patch_abc(void) {
#if defined(__Pyx_Generator_USED) || defined(__Pyx_Coroutine_USED)
    static int abc_patched = 0;
    if (CYTHON_REGISTER_ABCS && !abc_patched) {
        PyObject *module;
        module = PyImport_ImportModule((PY_MAJOR_VERSION >= 3) ? "collections.abc" : "collections");
        if (unlikely(!module)) {
            PyErr_WriteUnraisable(NULL);
            if (unlikely(PyErr_WarnEx(PyExc_RuntimeWarning,
                    ((PY_MAJOR_VERSION >= 3) ?
                        "Cython module failed to register with collections.abc module" :
                        "Cython module failed to register with collections module"), 1) < 0)) {
                return -1;
            }
        } else {
            module = __Pyx_patch_abc_module(module);
            abc_patched = 1;
            if (unlikely(!module))
                return -1;
            Py_DECREF(module);
        }
        module = PyImport_ImportModule("backports_abc");
        if (module) {
            module = __Pyx_patch_abc_module(module);
            Py_XDECREF(module);
        }
        if (!module) {
            PyErr_Clear();
        }
    }
#else
    if ((0)) __Pyx_Coroutine_patch_module(NULL, NULL);
#endif
    return 0;
}

/* Generator */
static PyMethodDef __pyx_Generator_methods[] = {
    {"send", (PyCFunction) __Pyx_Coroutine_Send, METH_O,
     (char*) PyDoc_STR("send(arg) -> send 'arg' into generator,\nreturn next yielded value or raise StopIteration.")},
    {"throw", (PyCFunction) __Pyx_Coroutine_Throw, METH_VARARGS,
     (char*) PyDoc_STR("throw(typ[,val[,tb]]) -> raise exception in generator,\nreturn next yielded value or raise StopIteration.")},
    {"close", (PyCFunction) __Pyx_Coroutine_Close_Method, METH_NOARGS,
     (char*) PyDoc_STR("close() -> raise GeneratorExit inside generator.")},
    {0, 0, 0, 0}
};
static PyMemberDef __pyx_Generator_memberlist[] = {
    {(char *) "gi_running", T_BOOL, offsetof(__pyx_CoroutineObject, is_running), READONLY, NULL},
    {(char*) "gi_yieldfrom", T_OBJECT, offsetof(__pyx_CoroutineObject, yieldfrom), READONLY,
     (char*) PyDoc_STR("object being iterated by 'yield from', or None")},
    {(char*) "gi_code", T_OBJECT, offsetof(__pyx_CoroutineObject, gi_code), READONLY, NULL},
    {(char *) "__module__", T_OBJECT, offsetof(__pyx_CoroutineObject, gi_modulename), 0, 0},
#if CYTHON_USE_TYPE_SPECS
    {(char *) "__weaklistoffset__", T_PYSSIZET, offsetof(__pyx_CoroutineObject, gi_weakreflist), READONLY, 0},
#endif
    {0, 0, 0, 0, 0}
};
static PyGetSetDef __pyx_Generator_getsets[] = {
    {(char *) "__name__", (getter)__Pyx_Coroutine_get_name, (setter)__Pyx_Coroutine_set_name,
     (char*) PyDoc_STR("name of the generator"), 0},
    {(char *) "__qualname__", (getter)__Pyx_Coroutine_get_qualname, (setter)__Pyx_Coroutine_set_qualname,
     (char*) PyDoc_STR("qualified name of the generator"), 0},
    {(char *) "gi_frame", (getter)__Pyx_Coroutine_get_frame, NULL,
     (char*) PyDoc_STR("Frame of the generator"), 0},
    {0, 0, 0, 0, 0}
};
#if CYTHON_USE_TYPE_SPECS
static PyType_Slot __pyx_GeneratorType_slots[] = {
    {Py_tp_dealloc, (void *)__Pyx_Coroutine_dealloc},
    {Py_tp_traverse, (void *)__Pyx_Coroutine_traverse},
    {Py_tp_iter, (void *)PyObject_SelfIter},
    {Py_tp_iternext, (void *)__Pyx_Generator_Next},
    {Py_tp_methods, (void *)__pyx_Generator_methods},
    {Py_tp_members, (void *)__pyx_Generator_memberlist},
    {Py_tp_getset, (void *)__pyx_Generator_getsets},
    {Py_tp_getattro, (void *) __Pyx_PyObject_GenericGetAttrNoDict},
#if CYTHON_USE_TP_FINALIZE
    {Py_tp_finalize, (void *)__Pyx_Coroutine_del},
#endif
    {0, 0},
};
static PyType_Spec __pyx_GeneratorType_spec = {
    __PYX_TYPE_MODULE_PREFIX "generator",
    sizeof(__pyx_CoroutineObject),
    0,
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_HAVE_GC | Py_TPFLAGS_HAVE_FINALIZE,
    __pyx_GeneratorType_slots
};
#else
static PyTypeObject __pyx_GeneratorType_type = {
    PyVarObject_HEAD_INIT(0, 0)
    __PYX_TYPE_MODULE_PREFIX "generator",
    sizeof(__pyx_CoroutineObject),
    0,
    (destructor) __Pyx_Coroutine_dealloc,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_HAVE_GC | Py_TPFLAGS_HAVE_FINALIZE,
    0,
    (traverseproc) __Pyx_Coroutine_traverse,
    0,
    0,
    offsetof(__pyx_CoroutineObject, gi_weakreflist),
    0,
    (iternextfunc) __Pyx_Generator_Next,
    __pyx_Generator_methods,
    __pyx_Generator_memberlist,
    __pyx_Generator_getsets,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
#if CYTHON_USE_TP_FINALIZE
    0,
#else
    __Pyx_Coroutine_del,
#endif
    0,
#if CYTHON_USE_TP_FINALIZE
    __Pyx_Coroutine_del,
#elif PY_VERSION_HEX >= 0x030400a1
    0,
#endif
#if PY_VERSION_HEX >= 0x030800b1
    0,
#endif
#if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
    0,
#endif
#if CYTHON_COMPILING_IN_PYPY && PYPY_VERSION_NUM+0 >= 0x06000000
    0,
#endif
};
#endif
static int __pyx_Generator_init(PyObject *module) {
#if CYTHON_USE_TYPE_SPECS
    __pyx_GeneratorType = __Pyx_FetchCommonTypeFromSpec(module, &__pyx_GeneratorType_spec, NULL);
#else
    (void) module;
    __pyx_GeneratorType_type.tp_getattro = __Pyx_PyObject_GenericGetAttrNoDict;
    __pyx_GeneratorType_type.tp_iter = PyObject_SelfIter;
    __pyx_GeneratorType = __Pyx_FetchCommonType(&__pyx_GeneratorType_type);
#endif
    if (unlikely(!__pyx_GeneratorType)) {
        return -1;
    }
    return 0;
}

/* CheckBinaryVersion */
static int __Pyx_check_binary_version(void) {
    char ctversion[4], rtversion[4];
    PyOS_snprintf(ctversion, 4, "%d.%d", PY_MAJOR_VERSION, PY_MINOR_VERSION);
    PyOS_snprintf(rtversion, 4, "%s", Py_GetVersion());
    if (ctversion[0] != rtversion[0] || ctversion[2] != rtversion[2]) {
        char message[200];
        PyOS_snprintf(message, sizeof(message),
                      "compiletime version %s of module '%.100s' "
                      "does not match runtime version %s",
                      ctversion, __Pyx_MODULE_NAME, rtversion);
        return PyErr_WarnEx(NULL, message, 1);
    }
    return 0;
}

/* FunctionImport */
#ifndef __PYX_HAVE_RT_ImportFunction
#define __PYX_HAVE_RT_ImportFunction
static int __Pyx_ImportFunction(PyObject *module, const char *funcname, void (**f)(void), const char *sig) {
    PyObject *d = 0;
    PyObject *cobj = 0;
    union {
        void (*fp)(void);
        void *p;
    } tmp;
    d = PyObject_GetAttrString(module, (char *)"__pyx_capi__");
    if (!d)
        goto bad;
    cobj = PyDict_GetItemString(d, funcname);
    if (!cobj) {
        PyErr_Format(PyExc_ImportError,
            "%.200s does not export expected C function %.200s",
                PyModule_GetName(module), funcname);
        goto bad;
    }
    if (!PyCapsule_IsValid(cobj, sig)) {
        PyErr_Format(PyExc_TypeError,
            "C function %.200s.%.200s has wrong signature (expected %.500s, got %.500s)",
             PyModule_GetName(module), funcname, sig, PyCapsule_GetName(cobj));
        goto bad;
    }
    tmp.p = PyCapsule_GetPointer(cobj, sig);
    *f = tmp.fp;
    if (!(*f))
        goto bad;
    Py_DECREF(d);
    return 0;
bad:
    Py_XDECREF(d);
    return -1;
}
#endif

/* InitStrings */
#if PY_MAJOR_VERSION >= 3
static int __Pyx_InitString(__Pyx_StringTabEntry t, PyObject **str) {
    if (t.is_unicode | t.is_str) {
        if (t.intern) {
            *str = PyUnicode_InternFromString(t.s);
        } else if (t.encoding) {
            *str = PyUnicode_Decode(t.s, t.n - 1, t.encoding, NULL);
        } else {
            *str = PyUnicode_FromStringAndSize(t.s, t.n - 1);
        }
    } else {
        *str = PyBytes_FromStringAndSize(t.s, t.n - 1);
    }
    if (!*str)
        return -1;
    if (PyObject_Hash(*str) == -1)
        return -1;
    return 0;
}
#endif
#if !CYTHON_COMPILING_IN_LIMITED_API
static int __Pyx_InitStrings(__Pyx_StringTabEntry *t) {
    while (t->p) {
        #if PY_MAJOR_VERSION >= 3
        __Pyx_InitString(*t, t->p);
        #else
        if (t->is_unicode) {
            *t->p = PyUnicode_DecodeUTF8(t->s, t->n - 1, NULL);
        } else if (t->intern) {
            *t->p = PyString_InternFromString(t->s);
        } else {
            *t->p = PyString_FromStringAndSize(t->s, t->n - 1);
        }
        if (!*t->p)
            return -1;
        if (PyObject_Hash(*t->p) == -1)
            return -1;
        #endif
        ++t;
    }
    return 0;
}
#endif

static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char* c_str) {
    return __Pyx_PyUnicode_FromStringAndSize(c_str, (Py_ssize_t)strlen(c_str));
}
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject* o) {
    Py_ssize_t ignore;
    return __Pyx_PyObject_AsStringAndSize(o, &ignore);
}
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
#if !CYTHON_PEP393_ENABLED
static const char* __Pyx_PyUnicode_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
    char* defenc_c;
    PyObject* defenc = _PyUnicode_AsDefaultEncodedString(o, NULL);
    if (!defenc) return NULL;
    defenc_c = PyBytes_AS_STRING(defenc);
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
    {
        char* end = defenc_c + PyBytes_GET_SIZE(defenc);
        char* c;
        for (c = defenc_c; c < end; c++) {
            if ((unsigned char) (*c) >= 128) {
                PyUnicode_AsASCIIString(o);
                return NULL;
            }
        }
    }
#endif
    *length = PyBytes_GET_SIZE(defenc);
    return defenc_c;
}
#else
static CYTHON_INLINE const char* __Pyx_PyUnicode_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
    if (unlikely(__Pyx_PyUnicode_READY(o) == -1)) return NULL;
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
    if (likely(PyUnicode_IS_ASCII(o))) {
        *length = PyUnicode_GET_LENGTH(o);
        return PyUnicode_AsUTF8(o);
    } else {
        PyUnicode_AsASCIIString(o);
        return NULL;
    }
#else
    return PyUnicode_AsUTF8AndSize(o, length);
#endif
}
#endif
#endif
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
    if (
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
            __Pyx_sys_getdefaultencoding_not_ascii &&
#endif
            PyUnicode_Check(o)) {
        return __Pyx_PyUnicode_AsStringAndSize(o, length);
    } else
#endif
#if (!CYTHON_COMPILING_IN_PYPY && !CYTHON_COMPILING_IN_LIMITED_API) || (defined(PyByteArray_AS_STRING) && defined(PyByteArray_GET_SIZE))
    if (PyByteArray_Check(o)) {
        *length = PyByteArray_GET_SIZE(o);
        return PyByteArray_AS_STRING(o);
    } else
#endif
    {
        char* result;
        int r = PyBytes_AsStringAndSize(o, &result, length);
        if (unlikely(r < 0)) {
            return NULL;
        } else {
            return result;
        }
    }
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject* x) {
   int is_true = x == Py_True;
   if (is_true | (x == Py_False) | (x == Py_None)) return is_true;
   else return PyObject_IsTrue(x);
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrueAndDecref(PyObject* x) {
    int retval;
    if (unlikely(!x)) return -1;
    retval = __Pyx_PyObject_IsTrue(x);
    Py_DECREF(x);
    return retval;
}
static PyObject* __Pyx_PyNumber_IntOrLongWrongResultType(PyObject* result, const char* type_name) {
    __Pyx_TypeName result_type_name = __Pyx_PyType_GetName(Py_TYPE(result));
#if PY_MAJOR_VERSION >= 3
    if (PyLong_Check(result)) {
        if (PyErr_WarnFormat(PyExc_DeprecationWarning, 1,
                "__int__ returned non-int (type " __Pyx_FMT_TYPENAME ").  "
                "The ability to return an instance of a strict subclass of int is deprecated, "
                "and may be removed in a future version of Python.",
                result_type_name)) {
            __Pyx_DECREF_TypeName(result_type_name);
            Py_DECREF(result);
            return NULL;
        }
        __Pyx_DECREF_TypeName(result_type_name);
        return result;
    }
#endif
    PyErr_Format(PyExc_TypeError,
                 "__%.4s__ returned non-%.4s (type " __Pyx_FMT_TYPENAME ")",
                 type_name, type_name, result_type_name);
    __Pyx_DECREF_TypeName(result_type_name);
    Py_DECREF(result);
    return NULL;
}
static CYTHON_INLINE PyObject* __Pyx_PyNumber_IntOrLong(PyObject* x) {
#if CYTHON_USE_TYPE_SLOTS
  PyNumberMethods *m;
#endif
  const char *name = NULL;
  PyObject *res = NULL;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_Check(x) || PyLong_Check(x)))
#else
  if (likely(PyLong_Check(x)))
#endif
    return __Pyx_NewRef(x);
#if CYTHON_USE_TYPE_SLOTS
  m = Py_TYPE(x)->tp_as_number;
  #if PY_MAJOR_VERSION < 3
  if (m && m->nb_int) {
    name = "int";
    res = m->nb_int(x);
  }
  else if (m && m->nb_long) {
    name = "long";
    res = m->nb_long(x);
  }
  #else
  if (likely(m && m->nb_int)) {
    name = "int";
    res = m->nb_int(x);
  }
  #endif
#else
  if (!PyBytes_CheckExact(x) && !PyUnicode_CheckExact(x)) {
    res = PyNumber_Int(x);
  }
#endif
  if (likely(res)) {
#if PY_MAJOR_VERSION < 3
    if (unlikely(!PyInt_Check(res) && !PyLong_Check(res))) {
#else
    if (unlikely(!PyLong_CheckExact(res))) {
#endif
        return __Pyx_PyNumber_IntOrLongWrongResultType(res, name);
    }
  }
  else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_TypeError,
                    "an integer is required");
  }
  return res;
}
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject* b) {
  Py_ssize_t ival;
  PyObject *x;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_CheckExact(b))) {
    if (sizeof(Py_ssize_t) >= sizeof(long))
        return PyInt_AS_LONG(b);
    else
        return PyInt_AsSsize_t(b);
  }
#endif
  if (likely(PyLong_CheckExact(b))) {
    #if CYTHON_USE_PYLONG_INTERNALS
    const digit* digits = ((PyLongObject*)b)->ob_digit;
    const Py_ssize_t size = Py_SIZE(b);
    if (likely(__Pyx_sst_abs(size) <= 1)) {
        ival = likely(size) ? digits[0] : 0;
        if (size == -1) ival = -ival;
        return ival;
    } else {
      switch (size) {
         case 2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
      }
    }
    #endif
    return PyLong_AsSsize_t(b);
  }
  x = PyNumber_Index(b);
  if (!x) return -1;
  ival = PyInt_AsSsize_t(x);
  Py_DECREF(x);
  return ival;
}
static CYTHON_INLINE Py_hash_t __Pyx_PyIndex_AsHash_t(PyObject* o) {
  if (sizeof(Py_hash_t) == sizeof(Py_ssize_t)) {
    return (Py_hash_t) __Pyx_PyIndex_AsSsize_t(o);
#if PY_MAJOR_VERSION < 3
  } else if (likely(PyInt_CheckExact(o))) {
    return PyInt_AS_LONG(o);
#endif
  } else {
    Py_ssize_t ival;
    PyObject *x;
    x = PyNumber_Index(o);
    if (!x) return -1;
    ival = PyInt_AsLong(x);
    Py_DECREF(x);
    return ival;
  }
}
static CYTHON_INLINE PyObject * __Pyx_PyBool_FromLong(long b) {
  return b ? __Pyx_NewRef(Py_True) : __Pyx_NewRef(Py_False);
}
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t ival) {
    return PyInt_FromSize_t(ival);
}


/* #### Code section: end ### */
#endif /* Py_PYTHON_H */
